   
   set (MOD_QTLMAP ${CMAKE_CURRENT_BINARY_DIR}/mod)    
   
   #Ajout de la date pour la datation de la compilation de l executable
   set (ADD_OPTIONS "-DDATE_BUILD=\"'${NOW}'\"")
   
   #ajout de openmp 
   IF (OPENMP)
      add_definitions(-DMANAGE_OMP)
      set (ADD_OPTIONS "${ADD_OPTIONS} -DMANAGE_OMP")
   ENDIF (OPENMP)
   
   #Gestion du BENCHMARK de QTLMAP
   IF (BENCHMARK)
     set (ADD_OPTIONS "${ADD_OPTIONS} -DBENCHMARK_VIEW")
   ENDIF (BENCHMARK)
   
   #ADDITIONAL FLAGS
   IF (NOT CMAKE_ADDITIONAL_FLAGS STREQUAL "")
     set (ADD_OPTIONS "${ADD_OPTIONS} ${CMAKE_ADDITIONAL_FLAGS}")
   ENDIF (NOT CMAKE_ADDITIONAL_FLAGS STREQUAL "")
   
   
   FIND_PATH(CUBLAS_CUBLASH_INCLUDE_DIR cublas.h ${CUBLAS_INCLUDE_PATH}/include /usr/local/cuda/include)
   FIND_PATH(CUBLAS_COMMONH_INCLUDE_DIR fortran_common.h ${CUBLAS_INCLUDE_PATH}/src /usr/local/cuda/src)

   IF ( CUBLAS_CUBLASH_INCLUDE_DIR AND CUBLAS_COMMONH_INCLUDE_DIR )
    SET(CUBLAS_FOUND TRUE)
   ENDIF ( CUBLAS_CUBLASH_INCLUDE_DIR AND CUBLAS_COMMONH_INCLUDE_DIR )

   IF (CUBLAS AND CUBLAS_FOUND)
     add_definitions(-DMANAGE_CUBLAS)
   ENDIF (CUBLAS AND CUBLAS_FOUND)
   
    
   MESSAGE(STATUS  "C Compiler: ${CMAKE_C_COMPILER}")
   if (CMAKE_C_COMPILER MATCHES "gcc") #gnu
    set(NLOPT_ACTIVE "true")
    set (ADD_OPTIONS "${ADD_OPTIONS} -DHAVE_NLOPT")
    set (CMAKE_C_FLAGS "${CMAKE_C_FLAGS} ${ADD_OPTIONS}")
    set (CMAKE_CXX_FLAGS "${CMAKE_CXX_FLAGS} ${ADD_OPTIONS}")
    
    MESSAGE(STATUS  " ** Active NLOPT ** ")
   else (CMAKE_C_COMPILER MATCHES "gcc")
    MESSAGE(STATUS  " ******************************* ")
    MESSAGE(STATUS  " ***** NLOPT IS NOT ACTIVE ***** ")
    MESSAGE(STATUS  " ******************************* ")
   endif (CMAKE_C_COMPILER MATCHES "gcc")
       
   IF (WIN32)
    add_definitions(-DWINDOWS)
    set (ADD_OPTIONS "${ADD_OPTIONS} -DWINDOWS")
   ELSE (WIN32)
    add_definitions(-DLINUX)
    set (ADD_OPTIONS "${ADD_OPTIONS} -DLINUX")
   ENDIF (WIN32)
   
   get_filename_component (Fortran_COMPILER_NAME ${CMAKE_Fortran_COMPILER} NAME)

# make sure that the default is a RELEASE
	if (NOT CMAKE_BUILD_TYPE)
	  set (CMAKE_BUILD_TYPE RELEASE CACHE STRING "Choose the type of build, options are: None Debug Release." FORCE)
	endif (NOT CMAKE_BUILD_TYPE)
    
    
   #Gestion MPI
   #-----------
   IF (Fortran_COMPILER_NAME MATCHES "mpif90")
       MESSAGE(STATUS  "MPI Found")
       FIND_PATH(MPI_INCLUDE_DIR mpi.h ${MPI_INCLUDE_PATH}/include /usr/include/mpi /usr/local/include/mpi)
      # FIND_LIBRARY(MPI_LIBRARY NAMES mpi PATH  ${MPI_INCLUDE_PATH}/lib /usr/lib /usr/local/) 

       IF (MPI_INCLUDE_DIR)
          SET(MPI_FOUND TRUE)
       ENDIF (MPI_INCLUDE_DIR)


       IF (MPI_FOUND)
         IF (NOT MPI_FIND_QUIETLY)
          MESSAGE(STATUS "Found MPI: ${MPI_LIBRARY}")
         ENDIF (NOT MPI_FIND_QUIETLY)
       ELSE (MPI_FOUND)
         IF (MPI_FIND_REQUIRED)
       #   MESSAGE(FATAL_ERROR "Could not find MPI")
         ENDIF (MPI_FIND_REQUIRED)
       ENDIF (MPI_FOUND)

      # include_directories(${MPI_INCLUDE_PATH})
       set (QTLMAP_ALL_INCLUDE "${QTLMAP_ALL_INCLUDE} ${MPI_INCLUDE_DIR}" )
       #set(LIBS ${LIBS} ${MPI_LIBRARY})
       MESSAGE(STATUS "==>INCLUDE MPI:${MPI_INCLUDE_DIR}")
       #MESSAGE(STATUS "==>LIB MPI:${MPI_LIBRARY}")

       link_directories(${MPI_LIBRARY})
     #  set(CMAKE_Fortran_COMPILER "mpif90")
       set(Fortran_COMPILER_NAME $ENV{MPICH_F90})
       
       IF (DEFINED Fortran_COMPILER_NAME) 
         MESSAGE(STATUS  "*** Compilo MPI :${Fortran_COMPILER_NAME} *****")
       ELSE ( DEFINED Fortran_COMPILER_NAME)
         MESSAGE(FATAL_ERROR  "**** Set environment variable MPICH_F90 (setenv MPICH_F90 <compiler>) | export MPICH_F90=<compiler>) *** ")
       ENDIF (DEFINED Fortran_COMPILER_NAME)

     #  SET(ENV{MPICH_F90} "${Fortran_COMPILER_NAME}")
     #  SET(ENV{MPICH_F90LINKER} "${Fortran_COMPILER_NAME}")
       add_definitions(-DMPI_ACTIVE)
       set (ADD_OPTIONS "${ADD_OPTIONS} -DMPI_ACTIVE")
 #option pour debugging...a mettre en place
 #-mpe=mpicheck
 #-mpe=mpitrace
 #    ELSE (MPI_FOUND)
 #        MESSAGE(STATUS  "MPI Not Found (required to compile modules)")
 #    ENDIF (MPI_FOUND)
    ENDIF (Fortran_COMPILER_NAME MATCHES "mpif90") 
    
#----------------------- GFORTRAN ----------------------------------------	
	if (Fortran_COMPILER_NAME MATCHES "gfortran") #gnu
	   set (CMAKE_Fortran_MODULE_DIRECTORY ${MOD_QTLMAP} )
       IF (GPROF)
         set (ADD_OPTIONS "${ADD_OPTIONS} -pg")
       ENDIF (GPROF)
       
   	   set (CMAKE_Fortran_FLAGS_RELEASE "${ADD_OPTIONS} -DMANAGE_OMP -fopenmp -static-libgfortran -O4")
       set (CMAKE_Fortran_FLAGS_DEBUG   "${ADD_OPTIONS} -DMANAGE_OMP -fopenmp -static-libgfortran -fbacktrace -ggdb -fbounds-check -fcheck-array-temporaries -Warray-bounds -Wsurprising -Wunderflow  -Walign-commons -fno-f2c -O0 -g")
       set (CMAKE_Fortran_FLAGS77  "")
       set (CMAKE_Fortran_FLAGS95  "")
	  
#----------------------- IFORT ----------------------------------------
	elseif (Fortran_COMPILER_NAME MATCHES "ifort") # intel
       
       set (CMAKE_Fortran_MODULE_DIRECTORY ${MOD_QTLMAP} )
       FIND_PACKAGE(IntelQTLMapPref) 
	   set (CMAKE_Fortran_FLAGS_RELEASE "${ADD_OPTIONS} -O4 -heap-arrays -openmp -openmp-lib=compat -DMANAGE_OMP")
	   set (CMAKE_Fortran_FLAGS_DEBUG   "${ADD_OPTIONS} -heap-arrays -ftrapuv  -openmp -openmp-lib=compat -DMANAGE_OMP -check all -DDEBUG ${ADD_OPTIONS} -O0 -g -fp-stack-check -traceback -debug all")
	   
	   set (CMAKE_Fortran_FLAGSF77 "-f77rtl")
	   set (CMAKE_Fortran_FLAGSF95 "-fpp -stand f95 -free")
	   
#----------------------- G95 ----------------------------------------
	elseif (Fortran_COMPILER_NAME MATCHES "g95") 
	    
	   set (CMAKE_Fortran_FLAGS_RELEASE "${ADD_OPTIONS} -O3  -fno-second-underscore  ")
	   set (CMAKE_Fortran_FLAGS_DEBUG   "${ADD_OPTIONS} -O0  -fno-second-underscore -g -fbounds-check -ftrace=frame -fzero")
	   set (CMAKE_Fortran_FLAGSF77 "")
	   set (CMAKE_Fortran_FLAGSF95 "-cpp")
	   
	    #linker option
       set(LIBS ${LIBS} -lf95)
	   
#----------------------- F95 ----------------------------------------
    elseif (Fortran_COMPILER_NAME MATCHES "f95") # NAG
       find_library(NAG_LIBRARY NAMES nag_nag nag nag64)
       IF ( NOT NAG_LIBRARY )
              MESSAGE(FATAL_ERROR "$$$$$$$$ Can not find NAG library $$$$$$$$$ ")
       ELSE ( NOT NAG_LIBRARY )
              MESSAGE(STATUS "******************************************* ")
              MESSAGE(STATUS "************** NAG library **************** ")
              MESSAGE(STATUS "******************************************* ")
              MESSAGE(STATUS "Find NAG: ${NAG_LIBRARY}")
              #add_library(nag_nag SHARED ${NAG_LIBRARY})
              find_library(PHREAD_LIBRARY NAMES pthread)
              set(LIBS ${LIBS} ${NAG_LIBRARY} ${PHREAD_LIBRARY})
              IF (NOT PHREAD_LIBRARY)
                MESSAGE (FATAL_ERROR "Can not find library pthread")
              ENDIF (NOT PHREAD_LIBRARY)
              add_definitions(-DHAVE_LIBNAG)
       ENDIF ( NOT NAG_LIBRARY )

	   set (CMAKE_Fortran_FLAGS_RELEASE "${ADD_OPTIONS} -O4 -maxcontin=200  -gline -colour -ieee=full")
	   set (CMAKE_Fortran_FLAGS_DEBUG   "${ADD_OPTIONS} -nan -maxcontin=200 -w=obs -g -C -gline -colour -ieee=stop")
	   
       set (CMAKE_Fortran_FLAGSF77 "-dusty -dcfuns")
	   set (CMAKE_Fortran_FLAGSF95 "-f95")

       SET(CMAKE_SHARED_LIBRARY_LINK_Fortran_FLAGS "")
	   SET(CMAKE_SKIP_BUILD_RPATH  TRUE)


    else (Fortran_COMPILER_NAME MATCHES "gfortran")
	  message ("CMAKE_Fortran_COMPILER full path: " ${CMAKE_Fortran_COMPILER})
	  message ("Fortran compiler: " ${Fortran_COMPILER_NAME})
	  message (FATAL_ERROR "The compiler no matches with this configuration file.")
#	  set (CMAKE_Fortran_FLAGS_RELEASE "-O2 -Wp,-DDATE_BUILD=\"'${NOW}'\"")
#	  set (CMAKE_Fortran_FLAGS_DEBUG   "-O0 -g -Wp,-DDEBUG,-DDATE_BUILD=\"'${NOW}'\"")	 
    endif (Fortran_COMPILER_NAME MATCHES "gfortran")
    
    
       #affichage
    if (CMAKE_BUILD_TYPE STREQUAL "Debug")
	  MESSAGE(STATUS "*****************DEBUG************************** ")
	  MESSAGE(STATUS ${CMAKE_Fortran_FLAGS_DEBUG})
	else (CMAKE_BUILD_TYPE STREQUAL "Debug")
	  MESSAGE(STATUS "*****************RELEASE************************** ")
	  MESSAGE(STATUS "Compiler Options :${CMAKE_Fortran_FLAGS_RELEASE}")
	endif (CMAKE_BUILD_TYPE STREQUAL "Debug")
	  MESSAGE(STATUS "F77 : ${CMAKE_Fortran_FLAGSF77}")
	  MESSAGE(STATUS "F95 : ${CMAKE_Fortran_FLAGSF95}")
	MESSAGE(STATUS "*************************************************** ")
	
	
	
    
