/*
 * struct_qtlmap.c
 *
 *  Created on: 1 oct. 2010
 *      Author: ofilangi
 */
#include <stdio.h>
#include <stdlib.h>
#include <cuda.h>
#include <cublas.h>
#include "struct_qtlmap.h"

#define bool unsigned char
#define true 0
#define false 1


/*********************************************************************************************************************************************************************************************
*
*   DEBUG INFORMATION
*
**********************************************************************************************************************************************************************************************/

void debug_print_adresse_structure_process(QTLMapDevicePtrStructProcess * idStruct) {

   printf("****************** PROCESS == VALEURS DE L'ADRESSAGE **********************\n");
   printf("\nValeurs des champs :\n\n");
   printf("XINC   :%p,%lld\n",idStruct->devicePtrXinc,(ADD_FORTRAN)idStruct->devicePtrXinc);
   printf("XX     :%p,%lld\n",idStruct->devicePtrXX,(ADD_FORTRAN)idStruct->devicePtrXX);
   printf("VECSOL :%p,%lld\n",idStruct->devicePtrVecsol,(ADD_FORTRAN)idStruct->devicePtrVecsol);
   printf("RHS    :%p,%lld\n",idStruct->devicePtrRHS,(ADD_FORTRAN)idStruct->devicePtrRHS);
   printf("XB     :%p,%lld\n",idStruct->devicePtrXB,(ADD_FORTRAN)idStruct->devicePtrXB);
   printf("****************************************\n");
   printf("NTNIVMAX  :%i\n",idStruct->ntnivmax);
   printf("****************************************\n\n");
}
void debug_print_adresse_structure_user(QTLMapDevicePtrStructUser * idStruct) {

   printf("****************** USER == VALEURS DE L'ADRESSAGE **********************\n");
   printf("\nValeurs des champs :\n\n");
   printf("Y      :%p,%lld\n",idStruct->devicePtrY,(ADD_FORTRAN)idStruct->devicePtrY);
   printf("****************************************\n");
   printf("NCAR      :%i\n",idStruct->ncar);
   printf("NKD       :%i\n",idStruct->nkd);
   printf("****************************************\n\n");
}

void debug_print_gpu_memory_XX(QTLMapDevicePtrStructProcess * idStruct,int ntniv) {

   DT * XX = (DT *)malloc(sizeof(DT)*(idStruct->ntnivmax)*(idStruct->ntnivmax));

   cublasStatus status = cublasGetMatrix(idStruct->ntnivmax,idStruct->ntnivmax,sizeof(DT),idStruct->devicePtrXX,idStruct->ntnivmax,XX,idStruct->ntnivmax);
   EXIT_ERROR_USE_DEVICE(status,"cublasGetMatrix","devicePtrXX");
   printf("\n") ;
   printf("*************************** XX :\n") ;
   int i,j;

   for (i=0;i<ntniv;i++) {
     for (j=0;j<ntniv;j++) {
       printf(" %f ",XX[j*ntniv+i]) ;
     }
     printf("\n") ;
   }
   printf("\n") ;
   free(XX);
}

void debug_print_gpu_memory_Vecsol(QTLMapDevicePtrStructProcess * idStruct,int ntniv) {

   bool * vecsol = (bool *)malloc(sizeof(bool)*(ntniv));
   int incx=1,incy=1;

   cublasStatus status = cublasGetVector(ntniv,sizeof(bool),idStruct->devicePtrVecsol,incx,vecsol,incy);
   EXIT_ERROR_USE_DEVICE(status,"cublasGetVector","devicePtrVecsol");

   printf("\n") ;
   printf("*************************** VECSOL :\n") ;
   int i;
   for (i=0;i<ntniv;i++) {
      printf(" %i ",vecsol[i]) ;
   }
   printf("\n") ;
   free(vecsol);
}

void debug_print_gpu_memory_RHS(QTLMapDevicePtrStructProcess * idStruct,int ntniv) {

   DT * rhs = (DT *)malloc(sizeof(DT)*(ntniv));
   int incx=1,incy=1;
   printf("%p\n",idStruct);
   printf("%p\n",idStruct->devicePtrRHS);
   printf("%p , %i\n",rhs,ntniv);
   cublasStatus status = cublasGetVector(ntniv,sizeof(DT),idStruct->devicePtrRHS,incx,rhs,incy);
   EXIT_ERROR_USE_DEVICE(status,"cublasGetVector","devicePtrRHS");
   printf("\n") ;
   printf("*************************** RHS :\n") ;
   int i;
   for (i=0;i<ntniv;i++) {
      printf(" %f ",rhs[i]) ;
   }
   printf("\n") ;
   free(rhs);
}

void debug_print_gpu_memory_XB(QTLMapDevicePtrStructProcess * idStruct,int nkd,int ntniv) {

   DT * XB = (DT *)malloc(sizeof(DT)*(nkd));
   int incx=1,incy=1;

   cublasStatus status = cublasGetVector(nkd,sizeof(DT),idStruct->devicePtrXB,incx,XB,incy);
   EXIT_ERROR_USE_DEVICE(status,"cublasGetVector","devicePtrXB");

   printf("\n") ;
   printf("*************************** XB :\n") ;
   int i;
   for (i=0;i<nkd;i++) {
      printf(" %f ",XB[i]) ;
   }
   printf("\n") ;
   free(XB);
}

void debug_print_gpu_memory_Y(QTLMapDevicePtrStructUser * idStruct) {

   DT * y = (DT *)malloc(sizeof(DT)*(idStruct->nkd));
   int incx=1,incy=1;

   cublasStatus status = cublasGetVector(idStruct->nkd,sizeof(DT),idStruct->devicePtrY,incx,y,incy);
   EXIT_ERROR_USE_DEVICE(status,"cublasGetVector","devicePtrY");

   printf("\n") ;
   printf("*************************** Y :\n") ;
   int i;
   for (i=0;i<idStruct->nkd;i++) {
      printf(" %f ",y[i]) ;
   }
   printf("\n") ;
   free(y);
}

/*
* Initialisation de la memoire a 0 en mode debug
*/
void init_memory_debug_process(QTLMapDevicePtrStructProcess * idStruct,int nkd) {

  int incx=1;
  int incy=1;
  cublasStatus status;
  DT * tab1 ;
  bool * tab2;

  tab1 = (DT *)malloc(sizeof(DT)*(nkd));
  int i,j;
  for (i=0;i<nkd;i++) tab1[i]=0.0 ;
  status = cublasSetVector(nkd,sizeof(DT),tab1,incx,idStruct->devicePtrXB,incy);
  EXIT_ERROR_USE_DEVICE(status,"cublasSetVector","devicePtrXB");
  free(tab1);

  tab1 = (DT *)malloc(sizeof(DT)*(idStruct->ntnivmax));
  for (i=0;i<idStruct->ntnivmax;i++) tab1[i]=0.0 ;
  status = cublasSetVector(idStruct->ntnivmax,sizeof(DT),tab1,incx,idStruct->devicePtrRHS,incy);
  EXIT_ERROR_USE_DEVICE(status,"cublasSetVector","devicePtrRHS");
  free(tab1);

  tab2 = (bool *)malloc(sizeof(bool)*(idStruct->ntnivmax));
  for (i=0;i<idStruct->ntnivmax;i++) tab2[i]=false ;
  status = cublasSetVector(idStruct->ntnivmax,sizeof(bool),tab2,incx,idStruct->devicePtrVecsol,incy);
  EXIT_ERROR_USE_DEVICE(status,"cublasSetVector","devicePtrVecsol");
  free(tab2);

  tab1 = (DT *)malloc(sizeof(DT)*(idStruct->ntnivmax*idStruct->ntnivmax));
  for (i=0;i<idStruct->ntnivmax;i++) {
   for (j=0;j<idStruct->ntnivmax;j++) {
    tab1[j*(idStruct->ntnivmax)+i]=0.0 ;
   }
  }
  status = cublasSetMatrix(idStruct->ntnivmax,idStruct->ntnivmax,sizeof(float),tab1,idStruct->ntnivmax,idStruct->devicePtrXX,idStruct->ntnivmax);
  EXIT_ERROR_USE_DEVICE(status,"cublasSetMatrix","devicePtrXX");

}


/*********************************************************************************************************************************************************************************************
*
*   INTERFACE WITH QTLMAP ( FORTRAN )
    * cuda_set_qtlmap_user_struct_(float *y, int *ndP, int *ntnivmaxP,ADD_FORTRAN * idUrlStructUserData)
*   * cuda_init_qtlmap_process_struct_(int *nkdP, int *ntnivP,ADD_FORTRAN * idUrlStruct)                                : initialize the contexte
*   * cuda_homoscedatic_resolv_(float *xinc,int *nkdPtr,int*ntnivPtr,double * seuil_choPtr,ADD_FORTRAN * idUrlStruct )    : apply analysis
*   * cuda_free_qtlmap_struct_process_(ADD_FORTRAN * idUrlStruct)                                                                 ! release memory
*   * cuda_free_qtlmap_struct_user_(ADD_FORTRAN * idUrlStruct)
*
**********************************************************************************************************************************************************************************************/

void cuda_set_qtlmap_user_struct_(double *y, int *np, int * nbL,int * nbC, ADD_FORTRAN * idUrlStructUserData) {

   int size ;
   int incx=1;
   int incy=1;

   cublasStatus status;
   QTLMapDevicePtrStructUser *idStruct;

   idStruct = (QTLMapDevicePtrStructUser *)malloc(sizeof(QTLMapDevicePtrStructUser));
   *idUrlStructUserData = (ADD_FORTRAN) idStruct ;

   idStruct->np   = * np;
   idStruct->nkd  = * nbL;
   idStruct->ncar = * nbC ;

#if(__PRINT_DEBUG_ANALYSE)
        printf("\n=====================================================\n") ;
        printf("\n==                   CUBLASALLOC DATA USER         ==\n") ;
        printf("\n=====================================================\n") ;
#endif

   // Initialisation CUBLAS
   // ---------------------
   cublasInit();

   /*--------------------------------------------------------------------------
   *            DEVICE ALLOC VIA CUBLAS
   *-----------------------------------------------------------------------------*/

   // Y
   // --
   size=(idStruct->nkd )*(idStruct->ncar);
   status = cublasAlloc(size,sizeof(DT),&idStruct->devicePtrY);
   EXIT_ERROR_ALLOC_DEVICE(status,"devicePtrY");

   /* Cast en DT */
   DT * yt = (DT *) malloc(size*sizeof(DT)) ;
   int i,j;
   for (i=0;i<idStruct->nkd ;i ++) {
     for (j=0 ; j<idStruct->ncar ; j++) {
       yt[j*(idStruct->nkd)+i] = (DT)(y[j*(idStruct->nkd)+i]);
     }
   }

   status = cublasSetVector(size,sizeof(DT),yt,incx,idStruct->devicePtrY,incy);
   EXIT_ERROR_USE_DEVICE(status,"cublasSetVector","devicePtrY");

#if(__PRINT_DEBUG_ANALYSE)
   printf("Adressage passe en argument      :%p\n",idStruct);
   printf("Valeur du cast en entier idStruct:%lld\n",(ADD_FORTRAN)idStruct);
   debug_print_adresse_structure_user(idStruct);
#endif
}


void cuda_init_qtlmap_process_struct_(int *ntnivmaxP,int *nkdP,ADD_FORTRAN * idUrlStruct) {

   int size ;
   int nkd = *nkdP;
   int ntnivmax = *ntnivmaxP;
   cublasStatus status;
   QTLMapDevicePtrStructProcess *idStruct;

   idStruct = (QTLMapDevicePtrStructProcess *)malloc(sizeof(QTLMapDevicePtrStructProcess));
   *idUrlStruct = (ADD_FORTRAN) idStruct ;
   idStruct->ntnivmax = ntnivmax ;

#if(__PRINT_DEBUG_ANALYSE)
        printf("\n=====================================================\n") ;
        printf("\n==                   CUBLASALLOC                    ==\n") ;
        printf("\n=====================================================\n") ;
#endif

   // Initialisation CUBLAS
   // ---------------------
   // cublasInit();

   /*--------------------------------------------------------------------------
   *            DEVICE ALLOC VIA CUBLAS
   *-----------------------------------------------------------------------------*/

    // XINC
   size=ntnivmax*nkd;
   status = cublasAlloc(size,sizeof(DT),&idStruct->devicePtrXinc);
   EXIT_ERROR_ALLOC_DEVICE(status,"devicePtrXinc");

   // XX
   size=ntnivmax*ntnivmax;
   status = cublasAlloc(size,sizeof(DT),&idStruct->devicePtrXX);
   EXIT_ERROR_ALLOC_DEVICE(status,"devicePtrXX");

   // VECSOL
   size=ntnivmax;
   status = cublasAlloc(size,sizeof(bool),&idStruct->devicePtrVecsol);
   EXIT_ERROR_ALLOC_DEVICE(status,"devicePtrVecsol");

   // RHS (solution)
   size=ntnivmax;
   status = cublasAlloc(size,sizeof(DT),&idStruct->devicePtrRHS);
   EXIT_ERROR_ALLOC_DEVICE(status,"devicePtrRHS");

   //XB
   size=nkd;
   status = cublasAlloc(size,sizeof(DT),&idStruct->devicePtrXB);
   EXIT_ERROR_ALLOC_DEVICE(status,"devicePtrXB");

#if(__PRINT_DEBUG_ANALYSE)
   init_memory_debug_process(idStruct,nkd);
   printf("Adressage passe en argument      :%p\n",idStruct);
   printf("Valeur du cast en entier idStruct:%lld\n",(ADD_FORTRAN)idStruct);
   debug_print_adresse_structure_process(idStruct);
#endif
}


void cuda_free_qtlmap_struct_process_(ADD_FORTRAN * idUrlStruct) {

   QTLMapDevicePtrStructProcess * idStructProcess ;
   idStructProcess = (QTLMapDevicePtrStructProcess *) (*idUrlStruct) ;

#if(__PRINT_DEBUG_ANALYSE)
        printf("\n=====================================================\n") ;
        printf("\n==                   CUBLASFREE PROCESS            ==\n") ;
        printf("\n=====================================================\n") ;
#endif

   cublasFree(idStructProcess->devicePtrXinc);
   cublasFree(idStructProcess->devicePtrXX);
   cublasFree(idStructProcess->devicePtrVecsol);
   cublasFree(idStructProcess->devicePtrRHS);
   cublasFree(idStructProcess->devicePtrXB);

}


void cuda_free_qtlmap_struct_user_(ADD_FORTRAN * idUrlStruct) {

   QTLMapDevicePtrStructUser * idStructUser ;
   idStructUser = (QTLMapDevicePtrStructUser *) (*idUrlStruct) ;

#if(__PRINT_DEBUG_ANALYSE)
        printf("\n=====================================================\n") ;
        printf("\n==                   CUBLASFREE  USER              ==\n") ;
        printf("\n=====================================================\n") ;
#endif

   cublasFree(idStructUser->devicePtrY);
}

