/*
DECLARATION
*/
%{ /*** C/C++ Declarations ***/

//To disable the asserts
//#define NDEBUG 

#include <stdio.h>
#include <string>
#include <vector>
#include <iostream>
#include <assert.h>
#include "expression.h"
	
using namespace std;
%}
/*** yacc/bison Declarations ***/

/* Require bison 2.3 or later */
%require "2.3"

/* add debug output code to generated parser. disable this for release
 * versions. */
%debug

/* start symbol */
%start QTLMAP_PANALYSE_GRAM

/* write out a header file containing the token defines */
%defines

/* use newer C++ skeleton file */
%skeleton "lalr1.cc"

/* namespace to enclose parser in */
%name-prefix="example"

/* set the parser's class identifier */
%define "parser_class_name" "Parser"

/* keep track of the current position within the input */
%locations
%initial-action
{
    // initialize the initial location object
    @$.begin.filename = @$.end.filename = &driver.streamname;
};

/* The driver is passed by reference to the parser and to the scanner. This
 * provides a simple but effective pure interface, not relying on global
 * variables. */
%parse-param { class Driver& driver }

/* verbose error messages */
%error-verbose

 /*** BEGIN EXAMPLE - Change the example grammar's tokens below ***/

%union {
    int  			                      integerVal;
    double 			                      doubleVal;
    string*		                          stringVal;
    
    map<string,int>*                      mapStringInt;
    map<string,double>*                   mapStringDouble;
    map<string,string>*                   mapStringString;
    map<string,vector<string> >*          mapStringVectorString;
    map<string,vector<class ObjectPhenoNode*>*>* mapStringVectorObjectPhenoNode;

    class QtlMapGenericObject*            qtlmapGenericObjectNode;
    class MapNode*		                  mapNode;
    class PhenotypeGenNode*               phenoGenNode;
    class ObjectPhenoNode*                objectPhenoNode;
    class GenotypeGenNode*                genotypeGenNode;
    class GenealogyGenNode*               genealogyGenNode;
    class TraitsAndFileInformation*       traitsAndFileInformationNode;
    class FileAndVectorMapNode*           fileAndVectorMapNode;
    class ModelNode*                      modelNode;
    class EffectNode*                     effectnode;
    class QTLNode*                        qtlnode;
    
    vector<string> *                      vectorstringVal;
    vector<double>*                       vectorDoubleVal;
    vector<vector<double>*>*              vectorDoubleDoubleVal;
    vector<ObjectPhenoNode*>*             vectorObjectPhenoNode;
    vector<MapNode*>*                     vectorMapNode;
    vector<EffectNode*> *                 listeffectnode;
    vector<QTLNode*>*                     listqtlnode;
    
}

%token			    END	           0	"end of file"
%token			    EOL		            "end of line"
%token <integerVal> INTEGER		        "int"
%token <doubleVal> 	DOUBLE		        "double"
%token <stringVal> 	STRING		        "string"
%token ANALYSIS_LX		    "* Analysis block *"

%token  SIMULATION_LX		"* Simulation block *"
%token  BLOCK_START		    "* { *"
%token  BLOCK_END		    "* } *"
%token  START_MATRIX		"* [ *"
%token  END_MATRIX		    "* ] *"
%token  AFF         	    "* : *"

%token  OUTPUT_FILE_LX		"outputFile"
%token  OUTPUT_SUMMARY_LX   "outputSummary"
%token  OUTPUT_XML_LX       "outputXml"
%token  OUTPUT_LRT_LX       "outputLrt"
%token  OUTPUT_QTLEFFECT_LX "outputQtlEffect"
%token  OUTPUT_PDED_LX      "outputPded"
%token  OUTPUT_PDED_JOIN_LX "outputPdedJoin"
%token  OUTPUT_HAPLOTYPE_LX "outputHaplotype"
%token  SIMULATION_FILE_LX  "outputSimul"

%token 	GENOTYPE_LX         "genotype"
%token 	GENOTYPE_SIMUL_LX   "genotypeSimul"
%token 	MAP_LX              "map"
%token 	UNKNOWNCHAR_LX      "unknownChar"
%token 	GENEALOGY_LX        "genealogy"
%token 	GENEA_SIMUL_LX      "genealogySimul"
%token 	PHENOTYPE_LX        "phenotype"
%token 	PHENOTYPE_SIMUL_LX  "phenotypeSimul"

%token 	FE_LX               "fixedEffect"
%token 	COV_LX              "covariate"
%token 	TRAIT_LX            "trait"
%token 	SIMUL_TRAIT_LX      "simulTrait"
%token 	GENEPROB_LX         "geneProb"

%token 	CHROMO_LX           "chromosome"
%token 	CHROMOS_LX          "chromosomes"
%token 	DENSITY_LX          "density"
%token 	NB_ALLELE_BY_M_LX   "nbAlleleByMarker"
%token 	SIZE_LX             "size"

%token 	NB_SIRES_LX            "nbSires"
%token 	NB_DAMS_BY_SIRE_LX     "nbDamsBySire"
%token 	NB_PROGENIES_BY_DAM_LX "nbProgeniesByDam"

%token 	HERITABILITY_LX     "heritability"
%token 	CENSURE_POSITION_LX "censuredPosition"
%token 	PRESENT_POSITION_LX "presentPosition"
%token 	UNDEFINED_VALUE_LX  "undefinedValue"
%token 	POSITION_START_LX   "positionStart"
%token 	POSITION_END_LX     "positionEnd"
%token 	MODEL_LX            "model"

%token 	QTL_LX              "QTL or qtl"
%token<stringVal> 	QTL_MULT_LX             "{nameEffect or name covariate}*QTL or {nameEffect or name covariate}xQTL"
%token 	PLUS_LX             "+"

%token 	CORRELATION_MATRIX_LX "correlationMatrix"
%token 	FREQ_ALLELE_GD_PARENT_LX "allFreqOnGrdParent"
%token 	EFFECT_LX            "effect"
%token 	NAME_TRAIT_LX        "nameTrait"
%token 	VALUE_LX             "value"
%token 	UNIT_LX             "unit"
%token<stringVal> 	MORGAN_LX           "morgan"
%token<stringVal> 	CENTIMORGAN_LX      "centimorgan"
%token 	NAME_LX             "name"
%token 	POSITION_LX         "position"
%token 	TYPE_LX             "type"
%token 	FILE_LX             "file"
%token 	YES_LX              "yes"
%token 	NO_LX               "no"
%token 	ON_LX               "on"
%token 	OFF_LX              "off"
%token 	TRUE_LX             "true"
%token 	FALSE_LX            "false"
%token<stringVal> 	REAL_LX             "real"
%token<stringVal> 	INTEGER_LX          "integer"
%token<stringVal> 	CATEGORIAL_LX       "categorial"
%token<stringVal> 	AVERAGE_LX          "average"
%token<stringVal> 	F2_LX               "F2"
%token<stringVal> 	BC_LX               "BC"
%token<stringVal> 	OUTBRED_LX          "OUTBRED"


 /* file node */
%type <stringVal> FILE_INPUT MAP_ATT TYPE_UNIT_MAP GENOTYPE_ATT NAME_ATT TYPE_PHENO_ATT_SIM TYPE_GENEA ATT_SIMULATION_FILE
%type <stringVal> ATT_OUTPUT ATT_SUMMARY OUTPUT_XML_NODE OUTPUT_LRT OUTPUT_QTLEFFECT OUTPUT_PDED OUTPUT_PDED_JOIN OUTPUT_HAPLOTYPE

%type <integerVal> POSITION_ATT ATT_POS_START ATT_POS_END
%type <doubleVal> HERITABILITY_ATT
%type <vectorstringVal> CHROMOSOME_ATT LIST_STRING_INTERNAL LISTSTRING

%type <mapStringInt> ATT_POS_GEN

%type <mapStringVectorString> DESC_MODEL
%type <mapStringDouble> PHENO_PRESENT_ATT TYPE_PHENO_ATT
%type <mapStringString> ATT_OPTIONAL_ANALYSIS_UNIT ATT_FILES_NEEDED ATT_SIMUL_FILES_NEEDED

%type <vectorDoubleVal> LIST_DOUBLE VECTOR
%type <vectorDoubleDoubleVal> LIST_VECTOR 
%type <vectorDoubleDoubleVal> CORR_MATRIX 
%type <vectorDoubleDoubleVal> CORRELATION_MATRIX_DEF
%type <effectnode> QTL_EFFECT_DEF
%type <listeffectnode> QTL_EFFECT_DEF_ALL
%type <qtlnode> QTL_DEF
%type <listqtlnode> ALL_QTL_DEF
%type <modelNode> MODEL_ATT PHENO_MODEL_ATT
%type <objectPhenoNode> ATT_TRAIT ATT_GENEPROB SIMUL_TRAITS_DEF_UNIT

%type <traitsAndFileInformationNode> SWITCH_TRAIT_DEF
%type <fileAndVectorMapNode> GENOTYPE_SIMUL_ATT

%type <vectorObjectPhenoNode> ATT_FE ATT_COV ATT_TRAIT_GEN ATT_TRAIT_NEXT ATT_TRAITS SIMUL_TRAITS_DEF
%type <mapStringVectorObjectPhenoNode> ATT_PHENO
%type <phenoGenNode> TRAITS_DEF SWITCH_TRAITS_DEF
%type <mapNode> MAP_DEF  MAP_SIMUL_DEF
%type <vectorMapNode> MAP_SIMUL
%type <genotypeGenNode> GENOTYPE_DEF GENOTYPE_SIMUL_DEF SWITCH_GENOTYPE_DEF
%type <genealogyGenNode> GENEALOGY_DEF GENEALOGY_SIMUL_DEF SWITCH_GENEALOGY_DEF

%type <qtlmapGenericObjectNode> ANALYSE SIMULATION

%destructor { delete $$; } STRING  FILE_INPUT MAP_ATT TYPE_UNIT_MAP GENOTYPE_ATT TYPE_GENEA
%destructor { delete $$; } MAP_DEF QTL_EFFECT_DEF CORRELATION_MATRIX_DEF LIST_VECTOR VECTOR LIST_DOUBLE QTL_EFFECT_DEF_ALL
%destructor { delete $$; } QTL_DEF ALL_QTL_DEF CHROMOSOME_ATT LIST_STRING_INTERNAL LISTSTRING ATT_POS_GEN DESC_MODEL MODEL_ATT
/* on nettoie si on instancie un noeud....*/
 /* %destructor { delete $$; } GRAMMAIRE ANALYSE */

 /*** END EXAMPLE - Change the example grammar's tokens above ***/

%{

#include "driver.h"
#include "scanner.h"

/* this "connects" the bison parser in the driver to the flex scanner class
 * object. it defines the yylex() function call to pull the next token from the
 * current lexer object of the driver context. */
#undef yylex
#define yylex driver.lexer->lex

%}

/*
 RULES
*/

%%
QTLMAP_PANALYSE_GRAM  :  /* empty */
                           ANALYSE  {  driver.context.add("rootNode",$1);}            /* Analysis case */ 
                         |  
                          SIMULATION {  driver.context.add("rootNode",$1);}  ;       /* Simulation case */

  /*************************************************************/
  /*             ANALALYSE PART                                */
  /*************************************************************/                         
ANALYSE    : ANALYSIS_LX BLOCK_START       
                         ATT_FILES_NEEDED
                         ATT_OPTIONAL_ANALYSIS_UNIT  /* all optional output analysis */
                         GENOTYPE_DEF  /* Definition/description of the genotype data*/
                         GENEALOGY_DEF /* Definition/description of the genealogy data*/
                         TRAITS_DEF
                         BLOCK_END 
                         {
	                       AnalyseNode *an= new AnalyseNode();
	                       map<string,string>::iterator it = $3->find("outputFile");
	                       if ( it != $3->end() ) an->outputFile = (*it).second;
	                       it = $3->find("outputSummary");
	                       if ( it != $3->end() ) an->outputSummary = (*it).second;
	                       it = $4->find("xmlFile");
                           if ( it != $4->end() ) an->xmlFile = (*it).second;
                           it = $4->find("lrtFile");
                           if ( it != $4->end() ) an->lrtFile = (*it).second;
                           it = $4->find("qtlEffectFile");
                           if ( it != $4->end() ) an->qtlEffectFile = (*it).second;
                           it = $4->find("pdedFile");
                           if ( it != $4->end() ) an->pdedFile = (*it).second;
                           it = $4->find("pdedJoinFile");
                           if ( it != $4->end() ) an->pdedJoinFile = (*it).second;
                           it = $4->find("haplotypeFile");
                           if ( it != $4->end() ) an->haplotypeFile = (*it).second;
                           an->genotypeGen=$5;
                           an->genealogyGen =$6;
                           an->phenotypeGen=$7;
                           $$=an;
                           string message;
                           if ( ! $$->check(message) ) {
                        	   error(yyloc, message);
                        	   YYERROR;
                           }
                           
                         } ;
/**
 * Type : mapStringString
 * return necessery output file for a analyse or a simulation
 */
ATT_FILES_NEEDED   :  ATT_OUTPUT     /* output file have to writed */
                      ATT_SUMMARY   /* output summary have to writed*/
                      {
	                    $$ = new map<string,string>();
	                    $$->insert(pair<string,string>("outputFile",*$1));delete $1;
	                    $$->insert(pair<string,string>("outputSummary",*$2));delete $2;
                      }
                      | ATT_SUMMARY
                      ATT_OUTPUT 
                      {
                    	  $$ = new map<string,string>();
                    	  $$->insert(pair<string,string>("outputSummary",*$1));delete $1;
                    	  $$->insert(pair<string,string>("outputFile",*$2));delete $2;
                      }
                    ;
/**
 * Type : mapStringString
 */
ATT_OPTIONAL_ANALYSIS_UNIT :  {$$ = new map<string,string>();} /* nothing */
                        | OUTPUT_XML_NODE ATT_OPTIONAL_ANALYSIS_UNIT {$$ = $2; $$->insert(pair<string,string>("xmlFile",*$1));delete $1;}
                        | OUTPUT_LRT ATT_OPTIONAL_ANALYSIS_UNIT      {$$ = $2; $$->insert(pair<string,string>("lrtFile",*$1));delete $1;}
                        | OUTPUT_QTLEFFECT ATT_OPTIONAL_ANALYSIS_UNIT {$$ = $2; $$->insert(pair<string,string>("qtlEffectFile",*$1));delete $1;}
                        | OUTPUT_PDED ATT_OPTIONAL_ANALYSIS_UNIT      {$$ = $2; $$->insert(pair<string,string>("pdedFile",*$1));delete $1;}
                        | OUTPUT_PDED_JOIN ATT_OPTIONAL_ANALYSIS_UNIT {$$ = $2; $$->insert(pair<string,string>("pdedJoinFile",*$1));delete $1;}
                        | OUTPUT_HAPLOTYPE ATT_OPTIONAL_ANALYSIS_UNIT {$$ = $2; $$->insert(pair<string,string>("haplotypeFile",*$1));delete $1;};


    /* Setting files rules */
    /* ------------------- */
ATT_OUTPUT       : OUTPUT_FILE_LX AFF STRING { $$ = $3; };

ATT_SUMMARY      : OUTPUT_SUMMARY_LX AFF STRING {$$ = $3} ;

OUTPUT_XML_NODE  : OUTPUT_XML_LX AFF STRING  {$$ = $3} ;

OUTPUT_LRT       : OUTPUT_LRT_LX AFF STRING  {$$ = $3} ;

OUTPUT_QTLEFFECT : OUTPUT_QTLEFFECT_LX AFF STRING {$$ = $3} ;

OUTPUT_PDED      : OUTPUT_PDED_LX AFF STRING  {$$ = $3} ;

OUTPUT_PDED_JOIN : OUTPUT_PDED_JOIN_LX AFF STRING  {$$ = $3} ;

OUTPUT_HAPLOTYPE : OUTPUT_HAPLOTYPE_LX AFF STRING  {$$ = $3} ;

   /* Genotype and Map DEFINITION */
   /* --------------------------- */
/**
 * Type : GenotypeGenNode
 */
GENOTYPE_DEF     : GENOTYPE_LX BLOCK_START
                               FILE_INPUT
                               GENOTYPE_ATT
	                           MAP_DEF 
	                           BLOCK_END 
	                           {
	                            GenotypeNode *gen = new GenotypeNode();
	                            gen->file=*$3;
	                            delete $3;
	                            gen->unknownChar=*$4;
	                            delete $4;
	                            gen->lMap.push_back($5);
	                            $$=gen;
	                           }
	                           ;

GENOTYPE_ATT : UNKNOWNCHAR_LX AFF STRING { $$ = $3; };

MAP_DEF       : MAP_LX BLOCK_START
                       FILE_INPUT
                       MAP_ATT
                       CHROMOSOME_ATT
                       BLOCK_END 
                       { 
	                      MapNode * map = new MapNode();
	                      map->file=*$3;delete $3;
	                      map->unit=*$4;delete $4;
	                      map->chromosomes=*$5;delete $5;
	                      $$ = map;
                       };

MAP_ATT      : UNIT_LX AFF TYPE_UNIT_MAP { $$ = $3; } ;
                

TYPE_UNIT_MAP :  MORGAN_LX | CENTIMORGAN_LX ;

CHROMOSOME_ATT : CHROMOS_LX AFF STRING      { $$ = new vector<string>(); $$->push_back(*$3);delete $3;}
                |CHROMOS_LX AFF LISTSTRING  { $$ = $3 ;}
               
               
   /* Genealogy DEFINITION */
   /* -------------------- */
/**
 * Type : GenealogyGenNode
 */
GENEALOGY_DEF  : GENEALOGY_LX BLOCK_START
                              FILE_INPUT
                              BLOCK_END 
                              {
	                          GenealogyNode * gen = new GenealogyNode();
                              gen->file=*$3;delete $3;
                              $$ = gen;
                              };

   /* Trait DEFINITION */
   /* ---------------- */
/**
 * Type : PhenotypeGenNode
 */
TRAITS_DEF  : PHENOTYPE_LX BLOCK_START
                           FILE_INPUT
                           ATT_PHENO
                           BLOCK_END 
                           {
                        	   PhenotypeNode * pheno = new PhenotypeNode();
                        	   pheno->file=*$3;delete $3;
                        	   map<string,vector<ObjectPhenoNode*> *>::iterator it = $4->find("lfe");
                        	   if ( it != $4->end() ) {
                        		   vector<ObjectPhenoNode*> * v = (*it).second;
                        		   for ( int i=0 ; i<v->size();i++) {
                        		      pheno->lfe.push_back(dynamic_cast<FixedEffectNode*>((*v)[i]));
                        		   }
                        		   v->clear();
                        		   delete v;
                        	   }
                        	   it = $4->find("lcov");
                        	   if ( it != $4->end() ) {
                        		   vector<ObjectPhenoNode*> * v = (*it).second;
                        		   for ( int i=0 ; i<v->size();i++) {
                        			   pheno->lcov.push_back(dynamic_cast<CovariateNode*>((*v)[i]));
                        		   }
                        		   v->clear();
                        		   delete v;
                        	   }
                        	   it = $4->find("ltraits");
                        	   if ( it != $4->end() ) {
                        		   vector<ObjectPhenoNode*> * v = (*it).second;
                        		   for ( int i=0 ; i<v->size();i++) {
                        			   pheno->ltraits.push_back((*v)[i]);
                        		   }
                        		   v->clear();
                        		   delete v;
                        	   }
                        	   $4->clear();
                        	   $$=pheno;
                        	   string message;
                        	   if ( ! $$->check(message) ) {
                        		   error(yyloc, message);
                        		   YYERROR;
                        	   }
                           } ;
/**
 * Type mapStringVectorObjectPhenoNode
 */
ATT_PHENO   :  ATT_FE 
               ATT_COV 
               ATT_TRAIT_GEN 
               {
	             $$ = new map<string,vector<ObjectPhenoNode*> *>();
	             $$->insert(pair<string,vector<ObjectPhenoNode*> *>("lfe",$1));
	             $$->insert(pair<string,vector<ObjectPhenoNode*> *>("lcov",$2));
	             $$->insert(pair<string,vector<ObjectPhenoNode*> *>("ltraits",$3));
               } ;
/**
 * Type vectorObjectPhenoNode
 */
ATT_FE      : { $$ = new vector<ObjectPhenoNode*>();}
             | FE_LX BLOCK_START
                    NAME_ATT         /* name of the fixed effect */
                    POSITION_ATT     /* position on the file */
                    BLOCK_END 
                    ATT_FE 
                    { 
                    	$$ = $6;
                    	FixedEffectNode * fe = new FixedEffectNode() ;
                        fe->name = *$3;delete $3;
                    	fe->position = $4;
                    	vector<ObjectPhenoNode*>::iterator it = $$->begin(); 
                    	$$->insert(it,fe);
                    };
/**
 * Type vectorObjectPhenoNode
 */
ATT_COV     : { $$ = new vector<ObjectPhenoNode*>();}
             | COV_LX BLOCK_START
                    NAME_ATT         /* name of the covariate */
                    POSITION_ATT     /* position on the file */
                    BLOCK_END 
                    ATT_COV 
                    {
                    	$$ = $6;
                    	CovariateNode * cov = new CovariateNode() ;
                    	cov->name = *$3;delete $3;
                    	cov->position = $4;
                    	vector<ObjectPhenoNode*>::iterator it = $$->begin(); 
                    	$$->insert(it,cov);
                    };
/**
* Type vectorObjectPhenoNode
*/
ATT_TRAIT_GEN  :   ATT_TRAITS { $$ = $1;}          /* definition of classic trait or gene for transcriptomic data */
                 | ATT_GENEPROB { $$ = new vector<ObjectPhenoNode*>(); $$->push_back($1); };

/**
 * Type ObjectPhenoNode
 */
     /* TRANSCRIPTOMIC DATA SPECIFICATION */ 
ATT_GENEPROB   : GENEPROB_LX BLOCK_START 
                             ATT_POS_GEN
                             HERITABILITY_ATT
                             UNDEFINED_VALUE_LX AFF DOUBLE 
                             PHENO_MODEL_ATT
                             BLOCK_END 
                             {
                            	 GeneProbNode * gp = new GeneProbNode() ;
                            	 map<string,int>::iterator it = $3->find("positionStart");
                            	 if ( it != $3->end() ) gp->positionStart = (*it).second;
                            	 it = $3->find("positionEnd");
                            	 if ( it != $3->end() ) gp->positionEnd = (*it).second;
                            	 gp->heritability=$4;
                            	 gp->undefinedValue=$7;
                            	 gp->model = $8;
                                 $$ = gp;
                                 string message;
                                 if ( ! $$->check(message) ) {
                                	 error(yyloc, message);
                                	 YYERROR;
                                 }
                             };
/*
 * Type map<string,int>
 * key : positionStart,positionEnd
 */
ATT_POS_GEN    : /* nothing */{ $$ = new map<string,int>(); }
                |  ATT_POS_START { $$ = new map<string,int>();$$->insert(pair<string,int>("positionStart",$1));}
                |  ATT_POS_END  { $$ = new map<string,int>();$$->insert(pair<string,int>("positionEnd",$1));}
                | ATT_POS_START ATT_POS_END{ $$ = new map<string,int>();
                                             $$->insert(pair<string,int>("positionStart",$1));
                                             $$->insert(pair<string,int>("positionEnd",$2));};
             
ATT_POS_START  : POSITION_START_LX AFF INTEGER {$$ = $3;};

ATT_POS_END    : POSITION_END_LX AFF INTEGER { $$ = $3 ;} ;


     /* CLASSICAL DATA SPECIFICATION */ 
ATT_TRAITS     :  ATT_TRAIT ATT_TRAIT_NEXT { $$ = $2; $$->push_back($1);} ;  /* at least one trait */

/**
 * Type vectorObjectPhenoNode
 */
ATT_TRAIT_NEXT :  { $$ = new vector<ObjectPhenoNode*>();}
                 | ATT_TRAIT ATT_TRAIT_NEXT { $$ = $2; $$->push_back($1); } ;

/**
 * ObjectPhenoNode
 */
ATT_TRAIT      :  TRAIT_LX BLOCK_START
                           NAME_ATT
                           POSITION_ATT
                           TYPE_PHENO_ATT
                           HERITABILITY_ATT
                           PHENO_PRESENT_ATT
                           PHENO_MODEL_ATT
                           BLOCK_END 
                           {
                        	   TraitNode * trait = new TraitNode() ;
                               trait->name=*$3;delete $3;
                               trait->position=$4;
                               map<string,double>::iterator it ;
                               if ( $5->find("real") != $5->end() ) {
                            	   trait->type="real";
                               } else if ( $5->find("integer") != $5->end() ) {
                            	   trait->type="integer";
                               } else if ( $5->find("categorial") != $5->end()  ) {
                            	   trait->type="categorial";
                               } else if ( $5->find("average") != $5->end() ) {
                            	   trait->type="average";
                            	   it = $5->find("censoredPosition");
                            	   if ( it == $5->end() ) {
                            		   error(yyloc, "censoredPosition have to be set for trait ["+trait->name+"]");
                                       YYERROR;
                            	   }
                            	   trait->isCensoredPosition = true;
                            	   trait->censoredPosition = (int)(*it).second;
                               }
                               
                               trait->heritability=$6;
                               it = $7->find("presentPosition");
                               if ( it != $7->end() ) {
                            	   trait->isPresentPosition=true;
                            	   trait->presentPosition=(int)(*it).second;
                               } else {
                            	   it = $7->find("undefinedValue");
                            	   if ( it != $7->end() ) {
                            		   trait->isUndefinedValue=true;
                            		   trait->undefinedValue=(*it).second;
                            	   } else {
                            		   error(yyloc, "presentPosition or UndefinedValue have to be set for trait ["+trait->name+"]");
                            		   YYERROR;
                            	   }
                               }
                               trait->model=$8;
                               $$ = trait;
                           } ;

/**
 * type map<string,double>
 * key real         --> no value associated
 * key integer      --> no value associated
 * key categorial   --> no value associated
 * key average      --> no value associated
 * key censoredPosition -> index position
 */
TYPE_PHENO_ATT     :  TYPE_LX AFF REAL_LX
                     {
	                    $$ = new map<string,double>();
	                    pair<string,double> elt;
	                    elt.first="real";
	                    elt.second=0.0;
	                    $$->insert(elt);
                     }
                    | TYPE_LX AFF INTEGER_LX
                    { 
                    	$$ = new map<string,double>();
                    	pair<string,double> elt;
                    	elt.first="integer";
                    	elt.second=0.0;
                    	$$->insert(elt);

                    }
                    | TYPE_LX AFF CATEGORIAL_LX
                    {
	                    $$ = new map<string,double>();
	                    pair<string,double> elt;
	                    elt.first="categorial";
	                    elt.second=0.0;
	                    $$->insert(elt);
                    }
                    | TYPE_LX AFF AVERAGE_LX        /* if average data , censurePosition have to defined */
                      CENSURE_POSITION_LX AFF INTEGER 
                      {
                    	$$ = new map<string,double>();
                    	pair<string,double> elt;
                    	elt.first="average";
                    	elt.second=0.0;
                    	$$->insert(elt);
                    	elt.first="censoredPosition";
                        elt.second=(double)$6;                   
                      };
/**
 * Type : double
 */
HERITABILITY_ATT   : HERITABILITY_LX AFF DOUBLE {$$ = $3;};

/*
 * map<string,double>
 * key presentPosition (la position (int) est caste en double...)
 * key undefinedValue
 */
PHENO_PRESENT_ATT  :  PRESENT_POSITION_LX AFF INTEGER   /* definition of a colomne or the self value */
                     {
	                     $$ = new map<string,double>();
	                     pair<string,double> elt;
	                     elt.first="presentPosition";
                         elt.second=(double)$3;
                         $$->insert(elt);
                     	 //trait->isPresentPosition=true;
                      };
                     | UNDEFINED_VALUE_LX AFF DOUBLE 
                     {
                    	 $$ = new map<string,double>();
                    	 pair<string,double> elt;
                    	 elt.first="undefinedValue";
                    	 elt.second=$3;
                    	 $$->insert(elt);
                    	 //trait->isUndefinedValue=true;
                     };

PHENO_MODEL_ATT : /* nothing */ { $$ = NULL;}    
                    | MODEL_ATT {$$ = $1} ;

/*
 * Type ModelNode *
 */
MODEL_ATT          : MODEL_LX AFF DESC_MODEL 
                     {
                       $$ = new ModelNode() ;
                       map<string,vector<string> >::iterator it = $3->find("lfeOrCov");
                       if ( it != $3->end() ) {
                          $$->lfeOrCov = (*it).second;
                       }
                       it = $3->find("lfeQtl");
                       if ( it != $3->end() ) {
                    	   $$->lfeOrCov = (*it).second;
                       }
                     }
                     ;

/*
 * Type map<string,vector<string>>*
 * key lfeOrCov
 * key lfeQtl
 */
DESC_MODEL         : STRING 
                     { 
	                   $$ = new  map<string,vector<string> >();
	                   pair<string,vector<string> > elt;
	                   elt.first = "lfeOrCov";
	                   elt.second.push_back(*$1);delete $1;
	                   $$->insert(elt);
                     } ;

DESC_MODEL         : QTL_MULT_LX
                     {
	                    $$ = new  map<string,vector<string> >();
		                pair<string,vector<string> > elt;
		                elt.first = "lfeQtl";
		                elt.second.push_back(*$1);delete $1;
		                $$->insert(elt);
                     } ;

DESC_MODEL         : STRING PLUS_LX DESC_MODEL 
                     {
                    	 $$ = $3;
                    	 map<string,vector<string> >::iterator it = $$->find("lfeOrCov");
                    	 if ( it == $$->end() ) {
                    	    pair<string,vector<string> > elt;
                    	    elt.first = "lfeOrCov";
                            elt.second.push_back(*$1);delete $1;
                    	    $$->insert(elt);
                    	 } else {
                    		 (*it).second.insert((*it).second.begin(),*$1);delete $1;
                    	 }
                     } ;

DESC_MODEL         : QTL_MULT_LX  
                     PLUS_LX DESC_MODEL
                     {
                    	 $$ = $3;
                         map<string,vector<string> >::iterator it = $$->find("lfeQtl");
                    	 if ( it == $$->end() ) {
                    	    pair<string,vector<string> > elt;
                    	    elt.first = "lfeQtl";
                    	    elt.second.push_back(*$1);delete $1;
                    	    $$->insert(elt);
                    	 } else {
                    	    (*it).second.insert((*it).second.begin(),*$1);delete $1;
                    	 } 
                     }
                     ;


    /*************************************************************/
    /*             SIMULATION PART                                */
    /*************************************************************/    
/**
 * Type QtlMapGenericObject
 */
SIMULATION : SIMULATION_LX BLOCK_START 
                           ATT_SIMUL_FILES_NEEDED
                           SWITCH_GENOTYPE_DEF
                           SWITCH_GENEALOGY_DEF
                           SWITCH_TRAITS_DEF
                           BLOCK_END 
                           {
	                           SimulationNode *sim = new SimulationNode();
		                       map<string,string>::iterator it = $3->find("outputFile");
		                       if ( it != $3->end() ) sim->outputFile = (*it).second;
		                       it = $3->find("outputSummary");
		                       if ( it != $3->end() ) sim->outputSummary = (*it).second;
		                       it = $3->find("outputSimul");
	                           if ( it != $3->end() ) sim->outputSimul = (*it).second;
	                           sim->genotypeGen=$4;
	                           sim->genealogyGen =$5;
	                           sim->phenotypeGen=$6;
	                           $$ = sim;
	                           string message;
	                           if ( ! $$->check(message) ) {
	                        	   error(yyloc, message);
	                        	   YYERROR;
	                           }
	                           
                           }
                           ; 

/**
 * Type : mapStringString
 */
ATT_SIMUL_FILES_NEEDED   :  ATT_FILES_NEEDED
                            ATT_SIMULATION_FILE { $$ = $1; $$->insert(pair<string,string>("outputSimul",*$2));delete $2;}
                          | ATT_SIMULATION_FILE
                            ATT_FILES_NEEDED { $$ = $2; $$->insert(pair<string,string>("outputSimul",*$1));delete $1;} ;

ATT_SIMULATION_FILE      :  SIMULATION_FILE_LX AFF STRING { $$ = $3;};

/**
 * Type : GenotypeGenNode
 */
SWITCH_GENOTYPE_DEF      :  GENOTYPE_DEF    { $$ = $1; } /* The genotype is not simulated */
                          | GENOTYPE_SIMUL_DEF { $$ = $1; } ;

/**
 * Type : GenealogyGenNode
 */
SWITCH_GENEALOGY_DEF     :  GENEALOGY_DEF  { $$ = $1; }  /* Genealogy is not simulated*/
                          | GENEALOGY_SIMUL_DEF { $$ = $1;} ;   


   /* Simulation of the genotype */
/**
 * Type : GenotypeGenNode
 */
GENOTYPE_SIMUL_DEF    :  GENOTYPE_SIMUL_LX BLOCK_START
                                           GENOTYPE_SIMUL_ATT
                                           BLOCK_END
                                           {
	                                           GenotypeSimulNode *gen= new GenotypeSimulNode();
                                        	   gen->isFile=$3->isFile;
                                        	   gen->file=$3->file;
                                        	   gen->lMap=(*($3->lmaps));
                                               $$=gen;
                                               string message;
                                               if ( ! $$->check(message) ) {
                                            	   error(yyloc, message);
                                            	   YYERROR;
                                               }
                                           }
                                           ; 
                                           
/**
 * Type : FileAndVectorMapNode
 */
GENOTYPE_SIMUL_ATT    :  FILE_INPUT MAP_SIMUL
                         {
	                        $$ = new FileAndVectorMapNode();
	                        $$->isFile = true;
	                        $$->file=*$1;delete $1;
	                        $$->lmaps=$2;
                         }
                       | MAP_SIMUL FILE_INPUT
                       {
                       	    $$ = new FileAndVectorMapNode();
                       	    $$->isFile = true;
                       	    $$->file=*$2;delete $2;
                       	    $$->lmaps=$1;
                       }
                       | MAP_SIMUL 
                       {
                    	   $$ = new FileAndVectorMapNode();
                    	   $$->isFile = false;
                    	   $$->lmaps=$1;
                       };

/**
 * Type : vectorMapNode
 */
MAP_SIMUL             :  MAP_SIMUL_DEF { $$ = new vector<MapNode*>();$$->push_back($1);}
                       | MAP_SIMUL_DEF MAP_SIMUL { $$ = $2; $$->insert($$->begin(),$1);} ;
/**
 * Type : MapNode
 */
MAP_SIMUL_DEF         :  CHROMO_LX BLOCK_START
                                   NAME_ATT
                                   DENSITY_LX AFF DOUBLE
                                   NB_ALLELE_BY_M_LX AFF INTEGER
                                   SIZE_LX AFF DOUBLE
                                   MAP_ATT 
                                   BLOCK_END
                                   {
                                   	 ChromosomeNode * chr = new ChromosomeNode();
                                   	 chr->name=*$3;delete $3;
                                   	 chr->density = $6;
                                   	 chr->nbAlleleByMarker = $9;
                                   	 chr->size=$12;
                                   	 chr->unit=*$13;delete $13;
                                   	 $$=chr;
                                     string message;
                                   	 if ( ! $$->check(message) ) {
                                    	 error(yyloc, message );
                                    	 YYERROR;
                                     }
                                   }
                                   ;
                                   

/**
 * Type : GenealogyGenNode
 */
GENEALOGY_SIMUL_DEF    : GENEA_SIMUL_LX BLOCK_START
                                        TYPE_LX AFF TYPE_GENEA
                                        NB_SIRES_LX AFF INTEGER
                                        NB_DAMS_BY_SIRE_LX AFF INTEGER
                                        NB_PROGENIES_BY_DAM_LX AFF INTEGER
                                        BLOCK_END 
                                        {
                                       	  GenalogySimulNode *gen = new GenalogySimulNode();
                                          gen->type=*$5;delete $5;
                                          gen->nbSires= $8;
                                       	  gen->nbDamsBySire = $11;
                                       	  gen->nbProgeniesByDam=$14;
                                       	  $$=gen;
                                       	  string message;
                                       	  if ( ! $$->check(message) ) {
                                       		  error(yyloc, message );
                                       		  YYERROR;
                                       	  }
                                       	  
                                        };

TYPE_GENEA            : F2_LX | BC_LX | OUTBRED_LX ;

/**
 * Type : PhenotypeGenNode
 */
SWITCH_TRAITS_DEF      : PHENOTYPE_SIMUL_LX BLOCK_START 
                                           SWITCH_TRAIT_DEF 
                                           CORRELATION_MATRIX_DEF 
                                           ALL_QTL_DEF
                                           BLOCK_END 
                                           {
                                        	  PhenotypeSimulNode * phen = new PhenotypeSimulNode();
                                        	  phen->isFile = $3->isFile;
                                        	  phen->file = $3->file;
                                        	  vector<ObjectPhenoNode*> v = (*($3->lfep));
                                        	  for (int i=0;i<v.size();i++) {
                                        	      phen->lfe.push_back(dynamic_cast<FixedEffectNode*>(v[i]));
                                        	  }
                                        	  v = (*($3->lcovp));
                                        	  for (int i=0;i<v.size();i++) {
                                        		  phen->lcov.push_back(dynamic_cast<CovariateNode*>(v[i]));
                                        	  }
                                        	  v = (*($3->ltraitsp));
                                        	  for (int i=0;i<v.size();i++) {
                                        		  phen->ltraits.push_back(v[i]);
                                        	  }

                                        	  phen->correlationMatrix=$4;
                                        	  phen->lQtls=*$5;delete $5;
                                        	  $$ = phen;
                                              string message;
                                              if ( ! $$->check(message) ) {
                                                    error(yyloc, message );
                                                    YYERROR;
                                              }
                                            } ;

/**
 * Type : traitsAndFileInformationNode
 */
SWITCH_TRAIT_DEF      :   FILE_INPUT 
                          ATT_PHENO /* Traits are used for estimation of a threshold . take unknown data*/
                          {
	                        $$ = new TraitsAndFileInformation();
	                        $$->file=*$1;delete $1;
	                        $$->isFile=true;
	                        map<string,vector<ObjectPhenoNode*> *>::iterator it ;
	                        it = $2->find("lfe");
	                        if ( it != $2->end()) $$->lfep=(*it).second;
	                        it = $2->find("lcov");
	                        if ( it != $2->end()) $$->lcovp=(*it).second;
	                        it = $2->find("ltraits");
	                        if ( it != $2->end()) $$->ltraitsp=(*it).second;
	                        $2->clear();
                          }  
                        | SIMUL_TRAITS_DEF 
                         {
                        	$$ = new TraitsAndFileInformation();
                        	$$->file="";
                        	$$->isFile=false;
                        	$$->lfep=new vector<ObjectPhenoNode *>();
                        	$$->lcovp=new vector<ObjectPhenoNode *>();
                        	$$->ltraitsp=$1;
                         } ;
/**
 * Type vectorObjectPhenoNode
 */
SIMUL_TRAITS_DEF      : SIMUL_TRAITS_DEF_UNIT { $$ = new vector<ObjectPhenoNode *>(); $$->push_back($1);}
                       | SIMUL_TRAITS_DEF_UNIT SIMUL_TRAITS_DEF { $$ = $2 ; $$->insert($$->begin(),$1);} ;

SIMUL_TRAITS_DEF_UNIT      :  SIMUL_TRAIT_LX BLOCK_START
                                        NAME_ATT
                                        TYPE_PHENO_ATT_SIM
                                        HERITABILITY_ATT
                                        BLOCK_END 
                                        {
                                       	   SimulTraitNode *s= new SimulTraitNode();
                                       	   s->name=*$3;delete $3;
                                       	   s->type=*$4;delete $4;
                                       	   s->heritability=$5;
                                       	   $$=s;
                                        }
                                        ;

TYPE_PHENO_ATT_SIM     :  TYPE_LX AFF REAL_LX   { $$ = $3;}       
                        | TYPE_LX AFF INTEGER_LX{ $$ = $3;}       
                        | TYPE_LX AFF CATEGORIAL_LX{ $$ = $3;}       
                        | TYPE_LX AFF AVERAGE_LX { $$ = $3;} ;
                    
CORRELATION_MATRIX_DEF  : /* nothing */{ $$ = new vector<vector<double>*>(); }
                         | CORRELATION_MATRIX_LX AFF CORR_MATRIX { $$ = $3; } ;

CORR_MATRIX             : START_MATRIX LIST_VECTOR  END_MATRIX { $$ = $2; };

LIST_VECTOR             :  VECTOR  { $$ = new vector<vector<double>*>(); $$->push_back($1);}
                         | VECTOR LIST_VECTOR { $$ = $2; vector<vector<double>*>::iterator it = $$->begin(); $$->insert(it,$1);};;

VECTOR                  : START_MATRIX LIST_DOUBLE END_MATRIX { $$ = $2; } ;

LIST_DOUBLE             : DOUBLE { $$ = new vector<double>(); $$->push_back($1);}
                         | DOUBLE LIST_DOUBLE  { $$ = $2; vector<double>::iterator it = $$->begin(); $$->insert(it,$1);};

ALL_QTL_DEF             : { $$ = new vector<QTLNode *>();}
                         | QTL_DEF ALL_QTL_DEF { $$ = $2;vector<QTLNode*>::iterator it = $$->begin(); $$->insert(it,$1); }

QTL_DEF                 : QTL_LX BLOCK_START
                                 CHROMO_LX AFF STRING
                                 POSITION_LX AFF DOUBLE
                                 FREQ_ALLELE_GD_PARENT_LX AFF DOUBLE
                                 QTL_EFFECT_DEF_ALL
                                 BLOCK_END 
                                 {
                                    $$ = new QTLNode();
                                    $$->chromosome=*$5;delete $5;
                                    $$->position=$8;
                                    $$->allFreqOnGrdParent=$11;
                                    $$->lEffects=*$12;delete $12;
                                    $12->clear();
                                 }
                                 ;

QTL_EFFECT_DEF_ALL      : QTL_EFFECT_DEF { $$ = new vector<EffectNode*>() ;$$->push_back($1);}
                         | QTL_EFFECT_DEF QTL_EFFECT_DEF_ALL { $$ = $2;vector<EffectNode*>::iterator it = $$->begin(); $$->insert(it,$1); };

QTL_EFFECT_DEF          : EFFECT_LX BLOCK_START
                                    NAME_TRAIT_LX AFF STRING
                                    VALUE_LX  AFF DOUBLE
                                    BLOCK_END 
                                    {
	                                   $$ = new  EffectNode();
	                                   $$->nameTrait=*$5;delete $5;
	                                   $$->value=$8;
                                    } ;

   /* Type base DEFINITION */
   /* --------------------------- */
               
LISTSTRING    :   START_MATRIX LIST_STRING_INTERNAL  END_MATRIX { $$ = $2; };
LIST_STRING_INTERNAL   :   STRING { $$ = new vector<string>(); $$->push_back(*$1);delete $1;}
                | STRING LIST_STRING_INTERNAL  { $$ = $2; vector<string>::iterator it = $$->begin(); $$->insert(it,*$1);delete $1;};
                
FILE_INPUT    : FILE_LX AFF STRING {$$ = $3;} ;

NAME_ATT      : NAME_LX AFF STRING {$$ = $3;} ;

POSITION_ATT  : POSITION_LX AFF INTEGER {$$ = $3;};
 /*
BOOLEAN       :  YES_LX 
               | NO_LX 
               | OFF_LX 
               | ON_LX 
               | TRUE_LX 
               | FALSE_LX
  */             
/*REAL_INT   : EXP_DIGIT'.'EXP_DIGIT  */ /* flottant */
/*         | EXP_DIGIT    ;   *//* entier */
/*
EXP_DIGIT  : EXP_DIGIT DIGIT
            | DIGIT { $$ = $1 - '0';}
  ; */
/*
STRING     : STRING CHAR ;
*/
%%
void example::Parser::error(const Parser::location_type& l,
			    const std::string& m)
{
    driver.error(l, m);
}


