

%{
#include <string>
#include "scanner.h"

	/* import the parser's token type into a local typedef */
typedef example::Parser::token token;
typedef example::Parser::token_type token_type;
	
	/* By default yylex returns int, we use token_type. Unfortunately yyterminate
	 * by default returns 0, which is not of token_type. */
#define yyterminate() return token::END

	/* This disables inclusion of unistd.h, which is not available under Visual C++
	 * on Win32. The C++ scanner uses STL streams instead. */
#define YY_NO_UNISTD_H
	
%}
/*** Flex Declarations and Options ***/

/* enable c++ scanner class generation */
%option c++

/* change the name of the scanner class. results in "QTLMapFlexLexer" */
%option prefix="QTLMap"

/* the manual says "somewhat more optimized" */
%option batch

/* enable scanner to generate debug output. disable this for release
 * versions. */
%option debug

/* no support for include files is planned */
%option yywrap nounput 

/* enables the use of start condition stacks */
%option stack

/* The following paragraph suffices to track locations accurately. Each time
 * yylex is invoked, the begin position is moved onto the end position. */
%{
#define YY_USER_ACTION  yylloc->columns(yyleng);
%}

   /* digit    [0-9] 
   //id       [a-z][a-z0-9]*    
   //blanks   [ \t]+             
   //integer    {digit}+          
   //exp  [eE][+-]?{integer}      
   //real    {integer}("."{integer})?{exp}?   */

%% 
     /* Regular Expressions Part */

     /* code to place at the beginning of yylex() */
%{
   // reset location
   yylloc->step();
%}

     /*** BEGIN EXAMPLE - Change the example lexer rules below ***/
\/\*.*\*\/    { yylloc->step();} 

[+-]?[0-9]+ {
	//std::ostringstream out;
    yylval->integerVal = atoi(yytext);
    return token::INTEGER;
}

[+-]?[0-9]+"."[0-9]* {
    yylval->doubleVal = atof(yytext);
    return token::DOUBLE;
}

    /*** KEYWORD ***/
    /* NOTE OFI : FAIRE DES DEFINE OU CONST TOUT EN HAUT DU FICHIER *******************/ 
    /* manage comments */

"analysis"      { return token::ANALYSIS_LX; }

"simulation"    { return(token::SIMULATION_LX); }

"outputFile"     { return(token::OUTPUT_FILE_LX); }

"outputSummary"   { return(token::OUTPUT_SUMMARY_LX); }

"{"             { return(token::BLOCK_START); }

"}"             { return (token::BLOCK_END); }

":"             { return(token::AFF); }

"outputXml"             { return token::OUTPUT_XML_LX ; }
"outputLrt"             { return token::OUTPUT_LRT_LX ; }
"outputQtlEffect"       { return token::OUTPUT_QTLEFFECT_LX ;}
"outputPded"            { return token::OUTPUT_PDED_LX ;}
"outputPdedJoin"        { return token::OUTPUT_PDED_JOIN_LX ;}
"outputHaplotype"       { return token::OUTPUT_HAPLOTYPE_LX ;}
"outputSimul"           { return token::SIMULATION_FILE_LX ;}

"genotype"               { return token::GENOTYPE_LX ;}
"genotypeSimul"          { return token::GENOTYPE_SIMUL_LX ;}
"map"                    { return token::MAP_LX ;}        
"unknownChar"            { return token::UNKNOWNCHAR_LX ;} 
"genealogy"              { return token::GENEALOGY_LX ;}  
"genealogySimul"         { return token::GENEA_SIMUL_LX ;}  
"phenotype"              { return token::PHENOTYPE_LX ;} 
"phenotypeSimul"         { return token::PHENOTYPE_SIMUL_LX ;}

"traitSimul"             { return token::SIMUL_TRAIT_LX ;}

"fixedEffect"            { return token::FE_LX ;}        
"covariate"              { return token::COV_LX ;} 
"trait"                  { return token::TRAIT_LX ;}  
"geneProb"               { return token::GENEPROB_LX ;}  
"heritability"           { return token::HERITABILITY_LX ;}     
"position"               { return token::POSITION_LX ;} 
"censuredPosition"       { return token::CENSURE_POSITION_LX ;} 
"presentPosition"        { return token::PRESENT_POSITION_LX ;}  
"undefinedValue"         { return token::UNDEFINED_VALUE_LX ;}  
"model"                  { return token::MODEL_LX ;}  
"positionStart"          { return token::POSITION_START_LX ;}  
"positionEnd"            { return token::POSITION_END_LX ;}  

"density"                { return token::DENSITY_LX ;}  
"nbAlleleByMarker"       { return token::NB_ALLELE_BY_M_LX ;}  
"size"                   { return token::SIZE_LX ;}
"chromosome"             { return token::CHROMO_LX ;}
"chromosomes"            { return token::CHROMOS_LX ;}

"nbSires"                { return token::NB_SIRES_LX ;}
"nbDamsBySire"           { return token::NB_DAMS_BY_SIRE_LX ;}
"nbProgeniesByDam"       { return token::NB_PROGENIES_BY_DAM_LX ;}

"correlationMatrix"      { return token::CORRELATION_MATRIX_LX ;}
"allFreqOnGrdParent"     { return token::FREQ_ALLELE_GD_PARENT_LX ;}
"effect"                 { return token::EFFECT_LX ;}
"nameTrait"              { return token::NAME_TRAIT_LX ;}
"value"                  { return token::VALUE_LX ;}

"QTL"                    { return token::QTL_LX  ;}
 
  /* "qtl"                    { return token::QTL_LX  ;} */
[A-Za-z][A-Za-z0-9_,.-]*"*"[ \t]*"qtl"                 	 {  
	                               yylval->stringVal = new std::string(yytext, yyleng);
	                               size_t found=yylval->stringVal->find("*");
	                               if (found != string::npos) {
	                            	   string ch="";
	                            	   yylval->stringVal->replace(found,yylval->stringVal->size(),ch);
	                               }
	                               return token::QTL_MULT_LX ;
                          }           
[A-Za-z][A-Za-z0-9_,.-]*"x"[ \t]*"qtl"       {  
	                               yylval->stringVal = new std::string(yytext, yyleng);
	                               size_t found=yylval->stringVal->find("x");
                                   if (found != string::npos) {
	                                   string ch="";
	                               	   yylval->stringVal->replace(found,yylval->stringVal->size(),ch);
	                               }
	                               return token::QTL_MULT_LX ;
                         }     
"+"                      { return token::PLUS_LX ;}

"name"                   { return token::NAME_LX ;}
"type"                   { return token::TYPE_LX ;} 
"unit"                   { return token::UNIT_LX ;} 
"morgan"                 { 
	                       yylval->stringVal = new std::string("morgan");
	                       return token::MORGAN_LX ;
                         } 
"centimorgan"            { 
	                       yylval->stringVal = new std::string("centimorgan");
	                       return token::CENTIMORGAN_LX ;
                         }  
"file"                   { return token::FILE_LX;}  
"yes"                    { return token::YES_LX ;}  
"no"                     { return token::NO_LX ;}     
"on"                     { return token::ON_LX ;} 
"off"                    { return token::OFF_LX ;}  
"true"                   { return token::TRUE_LX ;} 
"false"                  { return token::FALSE_LX ;}  

"F2"                     { yylval->stringVal = new std::string("F2");return token::F2_LX ;}  
"BC"                     { yylval->stringVal = new std::string("BC");return token::BC_LX ;}  
"OUTBRED"                { yylval->stringVal = new std::string("OUTBRED");return token::OUTBRED_LX ;}  

"real"                   { 
 	                       yylval->stringVal = new std::string("real");
	                       return token::REAL_LX ;
                         } 
"integer"                { 
	                       yylval->stringVal = new std::string("integer");
	                       return token::INTEGER_LX ;
                         }  
"categorial"             { 
	                       yylval->stringVal = new std::string("categorial");
	                       return token::CATEGORIAL_LX ;
                         } 
"average"                { 
                           yylval->stringVal = new std::string("average");
                           return token::AVERAGE_LX ;
                         }  
"["                      { return token::START_MATRIX ;}
"]"                      { return token::END_MATRIX ;}



 /* \[.*\[token::DOUBLE[,token::DOUBLE]* \].*\] */

\".*\"  {
                    //remove two "
                    yylval->stringVal = new std::string(yytext, yyleng);
                    string ch="";
                    size_t found=yylval->stringVal->find("\"");
	                yylval->stringVal->replace(found,found+1,ch);
	                found=yylval->stringVal->rfind("\"");
	                yylval->stringVal->replace(found,found+1,ch);
                    return token::STRING;
}

[A-Za-z][A-Za-z0-9_,.-]* {
    yylval->stringVal = new std::string(yytext, yyleng);
    return token::STRING;
}

 /* gobble up white-spaces */
[ \t\r]+ {
    yylloc->step();
}

 /* gobble up end-of-lines */
\n {
    yylloc->lines(yyleng); yylloc->step();
   // return token::EOL;
 }

 /* pass all other characters up to bison */
. {
    return static_cast<token_type>(*yytext);
}

   /*** END EXAMPLE - Change the example lexer rules above ***/

%%
namespace example {

Scanner::Scanner(std::istream* in,
		 std::ostream* out)
    : QTLMapFlexLexer(in, out)
{
}

Scanner::~Scanner()
{
}

void Scanner::set_debug(bool b)
{
    yy_flex_debug = b;
}

}

/* This implementation of ExampleFlexLexer::yylex() is required to fill the
 * vtable of the class ExampleFlexLexer. We define the scanner's main yylex
 * function via YY_DECL to reside in the Scanner class instead. */

#ifdef yylex
#undef yylex
#endif

int QTLMapFlexLexer::yylex()
{
    std::cerr << "in ExampleFlexLexer::yylex() !" << std::endl;
    return 0;
}

/* When the scanner receives an end-of-file indication from YY_INPUT, it then
 * checks the yywrap() function. If yywrap() returns false (zero), then it is
 * assumed that the function has gone ahead and set up `yyin' to point to
 * another input file, and scanning continues. If it returns true (non-zero),
 * then the scanner terminates, returning 0 to its caller. */

int QTLMapFlexLexer::yywrap()
{
    return 1;
}
