#!/bin/bash 

if [ "$#" -lt 9 ] ;then
  echo  'give argument to $0 : <QTLMAPEXEC> <MAPFILE> <GENEALOGYEFILE> <GENOTYPEFILE> <TRAITFILE> <MODELFILE> <CHROMOS> <STEP> <NDMIN> '
  exit 1;
fi

QTLMAP=$1
CARTE=$2
GENEA=$3
TYP=$4
TRAIT=$5
MODEL=$6
CHROMOS=$7
STEP=$8
NDMIN=$9

BASE=`date +%H:%M:%S-%d-%m-%Y`
#BASE=../../..
#QTLMAP=${BASE}/build/Release/gfortran/src/qtlmap
REPORTDIR=`pwd`/$BASE/report
OUTDIR=`pwd`/$BASE/out
PARAMDIR=`pwd`/$BASE/param
TIMEFILE=`pwd`/$BASE/time.txt


rm -rf `pwd`/$BASE/

mkdir -p $REPORTDIR
mkdir -p $OUTDIR
mkdir -p $PARAMDIR
touch $TIMEFILE

# 1 - nom du fichier
# 2 - step
# 3 - ndmin
# 4 - output dir

function create_panalyse {
FILE=$1
STEP=$2
NDMIN=$3
OUT=$4

mkdir -p $OUT

echo "in_map=$CARTE" >> ${FILE}
echo "in_genealogy=$GENEA" >> ${FILE}
echo "in_genotype=$TYP" >>  ${FILE}
echo "in_traits=$TRAIT" >>  ${FILE}
echo "in_model=$MODEL" >>  ${FILE}
echo "opt_step = $STEP" >>  ${FILE}
echo "opt_ndmin=$NDMIN" >>  ${FILE} 
echo "opt_minsirephaseproba=0.80">>  ${FILE}
echo "opt_minsirephaseproba=0.90">>  ${FILE}
echo "opt_mindamphaseproba=0.10">>  ${FILE}
echo "opt_chromosome=$CHROMOS">>  ${FILE}
echo "opt_unknown_char=0">>  ${FILE}
echo "out_output=$OUT/result">>  ${FILE}
echo "out_summary=$OUT/summary">>  ${FILE}
echo "out_phases=$OUT/phases">>  ${FILE}
echo "out_lrtsires=$OUT/sires">>  ${FILE}
echo "out_lrtdams=$OUT/dams">>  ${FILE}
echo "out_pded=$OUT/pded">>  ${FILE}
echo "out_pdedjoin=$OUT/pdedjoin">>  ${FILE}
echo "out_pateff=$OUT/pateff">>  ${FILE}
echo "out_mateff=$OUT/mateff">>  ${FILE}
echo "out_grid2qtl=$OUT/grid2qtl">>  ${FILE}
echo "in_paramsimul=param_sim_real">>  ${FILE}
echo "out_maxlrt=$OUT/simul">>  ${FILE}

}

function exec_qtlmap {
  CALCUL=$1
  HAPLO=$2
  QTL=$3
  STEP=$4
  NDMIN=$5
  echo
  echo " ========> calcul=$CALCUL qtl=$QTL haplo=$HAPLO step=$STEP ndmin=$NDMIN"
  
  PARAM_ANALYSE=$PARAMDIR/p_an#_step${STEP}_ndmin${NDMIN}_calc${CALCUL}_hap${HAPLO}
  OUT=$OUTDIR/$CALCUL/$QTL/$HAPLO
  create_panalyse $PARAM_ANALYSE $STEP $NDMIN $OUT
  
  before="$(date +%s)"
  ${QTLMAP} $PARAM_ANALYSE --calcul=$CALCUL --qtl=$QTL --haplotype=$HAPLO &> $REPORTDIR/report_cal${CALCUL}_haplo${HAPLO}_qtl${QTL}_step${STEP}_ndmin${NDMIN}
  after="$(date +%s)"
  elapsed_seconds="$(expr $after - $before)"
  echo "calcul=$CALCUL qtl=$QTL haplo=$HAPLO step=$STEP ndmin=$NDMIN : $elapsed_seconds sec" >> $TIMEFILE

  if [ "$?" -ne 0 ]; then
    echo "$mycom *** FAILED ***"
  fi

  tail -n10 $REPORTDIR/report_cal${CALCUL}_haplo${HAPLO}_qtl${QTL}_step${STEP}_ndmin${NDMIN}

}

#--------------------------BASE-------------------------------------------------------------
CALCUL=1
HAPLO=3
QTL=2
exec_qtlmap $CALCUL $HAPLO $QTL $STEP $NDMIN
HAPLO=4
exec_qtlmap $CALCUL $HAPLO $QTL $STEP $NDMIN

#--------------------------MODLIN---------------------------------------------------------
CALCUL=2
HAPLO=3
QTL=1
exec_qtlmap $CALCUL $HAPLO $QTL $STEP $NDMIN
HAPLO=4
exec_qtlmap $CALCUL $HAPLO $QTL $STEP $NDMIN

#-------------------------LINEAR HOMO---------------------------------------------------------
CALCUL=3
HAPLO=4
QTL=2
exec_qtlmap $CALCUL $HAPLO $QTL $STEP $NDMIN

#-------------------------LINEAR HETERO-----------------------------------------------------
CALCUL=4
HAPLO=4
QTL=2
exec_qtlmap $CALCUL $HAPLO $QTL $STEP $NDMIN

#--------------------------MULTI BASE---------------------------------------------------
CALCUL=5
HAPLO=3
QTL=1
exec_qtlmap $CALCUL $HAPLO $QTL $STEP $NDMIN
HAPLO=4
exec_qtlmap $CALCUL $HAPLO $QTL $STEP $NDMIN

#--------------------------DISCRIMINANTE-----------------------------------------------------
CALCUL=6
HAPLO=3
QTL=1
exec_qtlmap $CALCUL $HAPLO $QTL $STEP $NDMIN
HAPLO=4
exec_qtlmap $CALCUL $HAPLO $QTL $STEP $NDMIN

#---------------------------COX---------------------------------------------------
CALCUL=7
HAPLO=3
QTL=1
exec_qtlmap $CALCUL $HAPLO $QTL $STEP $NDMIN
HAPLO=4
exec_qtlmap $CALCUL $HAPLO $QTL $STEP $NDMIN

#---------------------------LD---------------------------------------------------
CALCUL=8
HAPLO=3
QTL=1
exec_qtlmap $CALCUL $HAPLO $QTL $STEP $NDMIN
HAPLO=4
exec_qtlmap $CALCUL $HAPLO $QTL $STEP $NDMIN

#---------------------------LDLA---------------------------------------------------
CALCUL=9
HAPLO=3
QTL=1
exec_qtlmap $CALCUL $HAPLO $QTL $STEP $NDMIN
HAPLO=4
exec_qtlmap $CALCUL $HAPLO $QTL $STEP $NDMIN

#---------------------------LA---------------------------------------------------
CALCUL=10
HAPLO=3
QTL=1
exec_qtlmap $CALCUL $HAPLO $QTL $STEP $NDMIN
HAPLO=4
exec_qtlmap $CALCUL $HAPLO $QTL $STEP $NDMIN

#---------------------------LD HOMO LINE---------------------------------------------------
CALCUL=25
HAPLO=4
QTL=1
exec_qtlmap $CALCUL $HAPLO $QTL $STEP $NDMIN

#---------------------------LD HETERO LINEAR---------------------------------------------------
CALCUL=26
HAPLO=4
QTL=1
exec_qtlmap $CALCUL $HAPLO $QTL $STEP $NDMIN

#---------------------------LDLA HOMO LINE---------------------------------------------------
CALCUL=27
HAPLO=4
QTL=1
exec_qtlmap $CALCUL $HAPLO $QTL $STEP $NDMIN

#---------------------------LDLA HETERO LINEAR---------------------------------------------------
CALCUL=28
HAPLO=4
QTL=1
exec_qtlmap $CALCUL $HAPLO $QTL $STEP $NDMIN

#-------------------------MULTIMODLIN-----------------------
CALCUL=23
HAPLO=3
QTL=1
exec_qtlmap $CALCUL $HAPLO $QTL $STEP $NDMIN
HAPLO=4
exec_qtlmap $CALCUL $HAPLO $QTL $STEP $NDMIN

#comparaison entre les differente version d haplotype
xxdiff $OUTDIR/1/2/3/result $OUTDIR/1/2/4/result 
xxdiff $OUTDIR/2/1/3/result $OUTDIR/2/1/4/result

#comparaison entre les LA
xxdiff $OUTDIR/1/2/3/result $OUTDIR/2/1/3/result $OUTDIR/4/2/4/result

#comparaison multi
xxdiff $OUTDIR/5/1/3/result $OUTDIR/23/1/3/result

#comparaison LD
xxdiff $OUTDIR/8/1/3/result $OUTDIR/26/1/4/result

#comparaison LDLA
xxdiff $OUTDIR/9/1/3/result $OUTDIR/28/1/4/result


