!!****p* QTLMap/ANALYSE
!!  NAME
!!    ANALYSE
!!  DESCRIPTION
!!  Package data :
!!     This package implements all analysis availabale on QTLMap.
!!        +--------+
!!        |        |
!!        +--------+
!!
!!  CREATION DATE
!!  01/01/2009
!!  COPYRIGHT
!!***

!!****m* ANALYSE/m_qtlmap_analyse
!!  NAME
!!    m_qtlmap_analyse -- Interface analysis module using by the main program.
!!  SYNOPSIS
!!
!!  DESCRIPTION
!!
!!  NOTES
!!
!!  BUGS
!!
!!  HISTORY
!!
!!  SEE ALSO
!!
!!  COPYRIGHT
!!***
!! You can use this space for remarks that should not be included
!! in the documentation.
!!/
module m_qtlmap_analyse
   use m_qtlmap_tools
   use m_qtlmap_log
   use m_qtlmap_data
   use m_qtlmap_haplotype_data
   use m_qtlmap_math
   use m_qtlmap_output_handler
   use m_qtlmap_analyse_gen
   use m_qtlmap_output_xml
   use m_qtlmap_cd

   implicit none

   !USER INTERFACE
   !--------------
   public :: get_name_analyse
   public :: get_type_data_analyse
   public :: is_multitrait_analysis
   public :: need_normalize_data
   public :: analyse
   public :: quantile

   contains

!!****f* m_qtlmap_analyse/get_name_analyse
!!  NAME
!!    get_name_analyse
!!  DESCRIPTION
!!   Returns the string name of a analysis identified by the calcul id.
!!  INPUTS
!!    opt_calcul : analysis id
!!  RETURN
!!    name of the analysis
!!  SOURCE
   function get_name_analyse(opt_calcul) result (name)
       integer        , intent(in)                    :: opt_calcul
       character(len=300)                             :: name
        SELECT CASE (opt_calcul)
         CASE (ANALYSE_UNITRAIT)
              name ="UNITRAIT ANALYSIS"
         CASE (ANALYSE_UNITRAIT_MODLIN)
              name = "MODLIN ANALYSIS"
         CASE (ANALYSE_MULTIRAIT)
              name = "MULTI ANALYSIS"
         CASE (ANALYSE_UNITRAIT_LINEAR_LA_HOMO)
              name = "LINEAR LA HOMOSCEDASTIC"
         CASE (ANALYSE_UNITRAIT_LINEAR_LA_HETERO)
              name = "LINEAR LA HETEROSCEDASTIC"
         CASE (ANALYSE_UNITRAIT_LINEAR_LD_HOMO)
              name = "LINEAR LD HOMOSCEDASTIC"
         CASE (ANALYSE_UNITRAIT_LINEAR_LD_HETERO)
              name = "LINEAR LD HETEROSCEDASTIC"
         CASE (ANALYSE_UNITRAIT_LINEAR_LDLA_HOMO)
              name = "LINEAR LDLA HOMOSCEDASTIC"
         CASE (ANALYSE_UNITRAIT_LINEAR_LDLA_HETERO)
              name = "LINEAR LDLA HETEROSCEDASTIC"
         CASE( ANALYSE_UNITRAIT_LDLA)
              name = "LDLA ANALYSIS"
         CASE( ANALYSE_UNITRAIT_LDJH)
              name = "LDJH ANALYSIS"
         CASE( ANALYSE_UNITRAIT_LD)
              name = "LD ANALYSIS"
         CASE( ANALYSE_UNITRAIT_LA)
              name = "LA ANALYSIS (MODLIN)"
         CASE (ANALYSE_UNITRAIT_MODLIN_COX)
              name = "COX ANALYSIS"
         CASE (ANALYSE_MULTITRAIT_DA)
              name = "DISCRIMINANT ANALYSIS"
         CASE (ANALYSE_2QTL_INTERACTION)
              name = "2QTL INTERACTION ANALYSIS"
         CASE (ANALYSE_UNITRAIT_CONTINGENCE)
              name = "NEW MODLIN"
         CASE (ANALYSE_TRAIT_COV_CONTINGENCE)
              name = "TRAITS AS COVARIATES"
          CASE (ANALYSE_MULTITRAIT_INCIDENCE)
              name = "MULTI-TRAITS ANALYSIS WITH COVARIATE/FIXED EFFECT"
         CASE DEFAULT
            call stop_application("Unknown name description for analysis [OPT_CALCUL="//trim(str(opt_calcul))//"]")
    END SELECT
   end function get_name_analyse
!!***

!!****f* m_qtlmap_analyse/get_type_data_analyse
!!  NAME
!!    get_type_data_analyse
!!  DESCRIPTION
!!   Returns the data type which is manageg by the analysis identified by the calcul id
!!  INPUTS
!!    opt_calcul : analysis id
!!  RETURN
!!    Type of data : TYPE_DATA_CONTINUE,TYPE_DATA_COX
!! SOURCE
    function get_type_data_analyse(opt_calcul) result (typeData)
       integer        , intent(in)                    :: opt_calcul
       integer                                        :: typeData
        SELECT CASE (opt_calcul)
         CASE (ANALYSE_UNITRAIT,ANALYSE_UNITRAIT_MODLIN,ANALYSE_MULTIRAIT,ANALYSE_UNITRAIT_LINEAR_LA_HOMO,&
               ANALYSE_UNITRAIT_LINEAR_LA_HETERO,ANALYSE_UNITRAIT_LDLA,ANALYSE_UNITRAIT_LA,&
               ANALYSE_UNITRAIT_LD,ANALYSE_UNITRAIT_LDJH, ANALYSE_UNITRAIT_LINEAR_LD_HOMO,ANALYSE_UNITRAIT_LINEAR_LD_HETERO, &
               ANALYSE_UNITRAIT_LINEAR_LDLA_HETERO,ANALYSE_UNITRAIT_LINEAR_LDLA_HOMO,&
               ANALYSE_MULTITRAIT_DA,ANALYSE_2QTL_INTERACTION,ANALYSE_UNITRAIT_CONTINGENCE,&
               ANALYSE_TRAIT_COV_CONTINGENCE,ANALYSE_MULTITRAIT_INCIDENCE)
              typeData = TYPE_DATA_CONTINUE
         CASE (ANALYSE_UNITRAIT_MODLIN_COX)
            typeData = TYPE_DATA_COX
         CASE DEFAULT
            call stop_application("Unknown data type description for analysis [OPT_CALCUL="//trim(str(opt_calcul))//"]")
        END SELECT
    end function get_type_data_analyse
!!***

!!****f* m_qtlmap_analyse/is_multitrait_analysis
!!  NAME
!!    is_multitrait_analysis
!!  DESCRIPTION
!!   Returns true if the analysis is a multitrait method, false otherwise
!!  INPUTS
!!    opt_calcul : analysis id
!!  RETURN
!!    boolean : true if the analysis given in parameter is for a set of traits (false otherwise)
!! SOURCE
    function is_multitrait_analysis(opt_calcul) result (is_multi)
       integer        , intent(in)                    :: opt_calcul
       logical                                        :: is_multi
        SELECT CASE (opt_calcul)
         CASE (ANALYSE_UNITRAIT,ANALYSE_UNITRAIT_MODLIN,ANALYSE_UNITRAIT_MODLIN_COX,ANALYSE_UNITRAIT_LINEAR_LA_HOMO,&
               ANALYSE_UNITRAIT_LINEAR_LA_HETERO,ANALYSE_UNITRAIT_LDLA,ANALYSE_UNITRAIT_LA,ANALYSE_UNITRAIT_LD,&
               ANALYSE_UNITRAIT_LDJH,ANALYSE_UNITRAIT_LINEAR_LD_HOMO,ANALYSE_UNITRAIT_LINEAR_LD_HETERO, &
               ANALYSE_UNITRAIT_LINEAR_LDLA_HETERO,ANALYSE_UNITRAIT_LINEAR_LDLA_HOMO,&
               ANALYSE_2QTL_INTERACTION,ANALYSE_UNITRAIT_CONTINGENCE,&
               ANALYSE_TRAIT_COV_CONTINGENCE)
              is_multi=.false.
          return
         CASE (ANALYSE_MULTIRAIT,ANALYSE_MULTITRAIT_INCIDENCE,ANALYSE_MULTITRAIT_DA)
            is_multi=.true.
        return
         CASE DEFAULT
            call stop_application("Unknown data type description for analysis [OPT_CALCUL="//trim(str(opt_calcul))//"]")
        END SELECT

    end function is_multitrait_analysis
!!***

!!****f* m_qtlmap_analyse/need_normalize_data
!!  NAME
!!    need_normalize_data
!!  DESCRIPTION
!!   Returns true if the analysis need to normalize data
!!  INPUTS
!!    opt_calcul : analysis id
!!  RETURN
!!    boolean : true if the analysis need a data normalization.
!!
!! SOURCE
    function need_normalize_data(opt_calcul) result (normalize)
       integer        , intent(in)                    :: opt_calcul
       logical                                        :: normalize
        SELECT CASE (opt_calcul)
         CASE (ANALYSE_UNITRAIT,ANALYSE_UNITRAIT_MODLIN,ANALYSE_MULTIRAIT,ANALYSE_MULTITRAIT_DA,ANALYSE_UNITRAIT_LINEAR_LA_HOMO,&
               ANALYSE_UNITRAIT_LINEAR_LA_HETERO,ANALYSE_UNITRAIT_LDLA,ANALYSE_UNITRAIT_LA,ANALYSE_UNITRAIT_LD,&
               ANALYSE_UNITRAIT_LINEAR_LD_HOMO,ANALYSE_UNITRAIT_LINEAR_LD_HETERO, &
               ANALYSE_UNITRAIT_LINEAR_LDLA_HETERO,ANALYSE_UNITRAIT_LINEAR_LDLA_HOMO,ANALYSE_UNITRAIT_LDJH,&
               ANALYSE_2QTL_INTERACTION,ANALYSE_UNITRAIT_CONTINGENCE,&
               ANALYSE_MULTITRAIT_INCIDENCE,ANALYSE_TRAIT_COV_CONTINGENCE)
              normalize=.true.
              return
         CASE (ANALYSE_UNITRAIT_MODLIN_COX)
              normalize=.false.
              return
         CASE DEFAULT
            call stop_application("need_normalize_data : Unknown data type description for analysis [OPT_CALCUL="//&
            trim(str(opt_calcul))//"]")
        END SELECT

    end function need_normalize_data
!!***

!!****f* m_qtlmap_analyse/analyse
!!  NAME
!!    analyse
!!  DESCRIPTION
!!   Main function to analyse a user dataset with a specific analysis.
!!
!!  INPUTS
!!    opt_calcul : analysis id
!!    opt_qtl    : number of qtl hypothesis (analysis can be not compatible with this option)
!!    hdam       : LD option, take dam haplotypes in considration
!!    opt_sim    : contexte of the analysis ( COMMON_ANALYSE | SIMULATION | TRANSCIPTOME_ANALYSE )
!!  OUTPUTS
!!    lrtsol     : likelihood ratio test information (see TYPE_LRT_SOLUTION)
!!    listincsol : solution for each hypothesis finded by the analysis (see TYPE_INCIDENCE_SOLUTION)
!!    rhoi       : residual correlation matrix finded for each hypothesis
!! SOURCE
    subroutine analyse(opt_calcul,opt_qtl,lrtsol,listincsol,rhoi,hdam,opt_sim)
          use m_qtlmap_incidence
          integer        , intent(in)                    :: opt_calcul,opt_qtl
          integer ,intent(in)                            :: opt_sim
          type(TYPE_LRT_SOLUTION)  , intent(inout) ,dimension(ncar,opt_qtl)    :: lrtsol
          type(TYPE_INCIDENCE_SOLUTION)  ,intent(inout)  ,dimension(ncar,opt_qtl+1)   :: listincsol
          real (kind=dp) ,dimension(size(carac),size(carac)), intent(out)  :: rhoi
          logical                       ,intent(in)  :: hdam

          integer                        :: ic
          logical                        :: car_real

          rhoi=0.d0

          if ( opt_qtl <= 0) then
             call stop_application("bad definition of opt_qtl ["//trim(str(opt_qtl))//"]")
          end if

! arret si l'utilisateur demande d'analyser que des variables enti�res ou cat�gorielles selon
! une option non disponible (opt_qtl=2, opt_calcul=ANALYSE_MULTIRAIT  ou ANALYSE_UNITRAIT_MODLIN_cox

          car_real=.false.
          do ic=1,size(carac)
            if((natureY(ic) == 'r') .or.( natureY(ic) == 'a')) car_real=.true.
          end do
          if(.not.car_real) then
           if (opt_calcul == ANALYSE_UNITRAIT_MODLIN_cox .or.&
             opt_calcul == ANALYSE_MULTIRAIT) then
             call stop_application("Analysis option not available [OPT_CALCUL="//trim(str(opt_calcul))//"]"&
            //"[OPT_QTL="//trim(str(opt_qtl))//"] ")
           end if
          end if

            if ( opt_calcul == ANALYSE_UNITRAIT .and. opt_qtl == 1) then
               call opti_unitrait(lrtsol,listincsol,opt_sim,1)
            else if ( opt_calcul == ANALYSE_UNITRAIT .and. opt_qtl == 2 ) then
               call opti_unitrait(lrtsol,listincsol,opt_sim,2)
            else if ( opt_calcul == ANALYSE_UNITRAIT_MODLIN .and. opt_qtl == 1) then
               call opti_unitrait_modlin(lrtsol,listincsol,opt_sim)
               if (opt_sim /= TRANSCIPTOME_ANALYSE) call opti_unitrait_discret(lrtsol,listincsol,opt_sim)
            else if ( opt_calcul == ANALYSE_UNITRAIT_LDLA .and. opt_qtl == 1) then
               call opti_unitrait_modlin_new(lrtsol(:,1),listincsol,"hete","LDLA",hdam,opt_sim)
            else if ( opt_calcul == ANALYSE_UNITRAIT_LD .and. opt_qtl == 1) then
               call opti_unitrait_modlin_new(lrtsol(:,1),listincsol,"hete","LD  ",hdam,opt_sim)
            else if ( opt_calcul == ANALYSE_UNITRAIT_LA .and. opt_qtl == 1) then
               call opti_unitrait_modlin_new(lrtsol(:,1),listincsol,"hete","LA  ",hdam,opt_sim)
            else if ( opt_calcul == ANALYSE_UNITRAIT_LDJH .and. opt_qtl == 1) then
               call opti_unitrait_modlin_new(lrtsol(:,1),listincsol,"hete","LDJH",hdam,opt_sim)
            else if ( opt_calcul == ANALYSE_UNITRAIT_LINEAR_LA_HOMO .and. opt_qtl >= 1) then
               call opti_unitrait_incidence(lrtsol,listincsol,opt_sim,INCIDENCE_TYPE_CLASSIC,LINEAR_HOMOSCEDASTIC,opt_qtl,hdam)
            else if ( opt_calcul == ANALYSE_UNITRAIT_LINEAR_LA_HETERO .and. opt_qtl >= 1) then
               call opti_unitrait_incidence(lrtsol,listincsol,opt_sim,INCIDENCE_TYPE_CLASSIC,LINEAR_HETEROSCEDASTIC,opt_qtl,hdam)
             else if ( opt_calcul == ANALYSE_UNITRAIT_LINEAR_LD_HOMO .and. opt_qtl >= 1) then
               call opti_unitrait_incidence(lrtsol,listincsol,opt_sim,INCIDENCE_TYPE_LD,LINEAR_HOMOSCEDASTIC,opt_qtl,hdam)
            else if ( opt_calcul == ANALYSE_UNITRAIT_LINEAR_LD_HETERO .and. opt_qtl >= 1) then
               call opti_unitrait_incidence(lrtsol,listincsol,opt_sim,INCIDENCE_TYPE_LD,LINEAR_HETEROSCEDASTIC,opt_qtl,hdam)
                else if ( opt_calcul == ANALYSE_UNITRAIT_LINEAR_LDLA_HOMO .and. opt_qtl >= 1) then
               call opti_unitrait_incidence(lrtsol,listincsol,opt_sim,INCIDENCE_TYPE_LDLA,LINEAR_HOMOSCEDASTIC,opt_qtl,hdam)
            else if ( opt_calcul == ANALYSE_UNITRAIT_LINEAR_LDLA_HETERO .and. opt_qtl >= 1) then
               call opti_unitrait_incidence(lrtsol,listincsol,opt_sim,INCIDENCE_TYPE_LDLA,LINEAR_HETEROSCEDASTIC,opt_qtl,hdam)
            else if ( opt_calcul == ANALYSE_UNITRAIT_MODLIN_cox .and. opt_qtl == 1) then
               call opti_unitrait_modlin_cox(lrtsol,listincsol,opt_sim)
            else if ( opt_calcul == ANALYSE_MULTIRAIT .and. opt_qtl == 1) then
               call opti_multitraits(lrtsol(1,1),listincsol,rhoi,opt_sim)
            else if ( opt_calcul == ANALYSE_MULTITRAIT_DA .and. opt_qtl == 1) then
               call opti_multitrait_DA(lrtsol(1,1),opt_sim)
            else if ( opt_calcul == ANALYSE_2QTL_INTERACTION .and. opt_qtl == 2) then
               call opti_unitrait_interaction(lrtsol,opt_sim)
            else if ( opt_calcul == ANALYSE_UNITRAIT_CONTINGENCE .and. opt_qtl >= 1) then
               call opti_unitrait_incidence(lrtsol,listincsol,opt_sim,INCIDENCE_TYPE_CLASSIC,OPTIM_HETEROSCEDASTIC,opt_qtl,hdam)
            else if ( opt_calcul == ANALYSE_TRAIT_COV_CONTINGENCE .and. opt_qtl >= 1) then
               call opti_unitrait_incidence(lrtsol,listincsol,opt_sim,INCIDENCE_TRAIT_COV,OPTIM_HETEROSCEDASTIC,opt_qtl,hdam)
            else if ( opt_calcul == ANALYSE_MULTITRAIT_INCIDENCE .and. opt_qtl >= 1) then
               call opti_multitrait_modlin(lrtsol(1,:),listincsol,opt_sim,opt_qtl)
            else
               call stop_application("Analysis option not available [OPT_CALCUL="//trim(str(opt_calcul))//"]"&
               //"[OPT_QTL="//trim(str(opt_qtl))//"]")
        end if

        if (opt_sim /= SIMULATION) call print_allelic_origin


    end subroutine analyse
!!***

!!****f* m_qtlmap_analyse/opti_unitrait
!!  NAME
!!    opti_unitrait
!!  DESCRIPTION
!!   LA single trait with pre-corrected data (1 or 2 qtl to test)
!!  INPUTS
!!    nbqtl      : number of qtl to test. 1 <= nbqtl <= 2
!!    opt_sim    : contexte of the analysis ( COMMON_ANALYSE | SIMULATION | TRANSCIPTOME_ANALYSE )
!!  OUTPUTS
!!    lrtsol     : likelihood ratio test information (see TYPE_LRT_SOLUTION)
!!    listincsol : solution for each hypothesis finded by the analysis (see TYPE_INCIDENCE_SOLUTION)
!! SOURCE
      subroutine opti_unitrait(lrtsol,listincsol,opt_sim,nbqtl)
       use m_qtlmap_analyse_unitrait
       integer                , intent(in)                                       :: nbqtl
       type(TYPE_LRT_SOLUTION)         , intent(inout) ,dimension(ncar,nbqtl)    :: lrtsol
       type(TYPE_INCIDENCE_SOLUTION)  ,  intent(inout) ,dimension(ncar,nbqtl+1)  :: listincsol
       integer                        ,  intent(in)                              :: opt_sim

       integer                                           :: i,ic



       if ( nbqtl > 2 .or. nbqtl < 1 ) then
         call stop_application("Devel error : Module UNITRAIT can not perform analysis with nqtl:"//trim(str(nbqtl)))
       end if

       !$OMP PARALLEL DEFAULT(SHARED)
       call init_analyse_gen
       call init_analyse_unitrait_1QTL
       if ( nbqtl == 2) call init_analyse_unitrait_2QTL

       !$OMP DO
       do ic=1,size(carac)
           if (.not.(natureY(ic) == 'r'.or.natureY(ic) == 'a')) cycle
           call log_mess("TRAIT ["//trim(carac(ic))//"]",INFO_DEF)
           call optinit(ic)
           call opti_0qtl(ic)
           call opti_1qtl(ic,lrtsol(ic,1))

           if ( opt_sim /= SIMULATION) then
              call set_solution_hypothesis0(ic,listincsol(ic,1))
              call set_solution_hypothesis1(ic,listincsol(ic,2))
           end if

           if ( nbqtl == 2 ) then
             call opti_1car_2qtl(ic,lrtsol(ic,2))
             if ( opt_sim /= SIMULATION) then
                 call set_solution_hypothesis2(ic,listincsol(ic,3))
             end if
           end if
       end do
       !$OMP END DO
       call end_analyse_unitrait_1QTL
       if ( nbqtl == 2) call end_analyse_unitrait_2QTL
       call end_analyse_gen
       !$OMP END PARALLEL

       if (opt_sim == COMMON_ANALYSE) then ! Representation graphique
        do ic=1,ncar
           if (.not.(natureY(ic) == 'r'.or.natureY(ic) == 'a')) cycle
           call print_start_unitrait(carac(ic))
           call print_incidence_solution(listincsol(ic,1))
           call print_lrt_solution(lrtsol(ic,1))
           call print_incidence_solution(listincsol(ic,2))
           call print_paternal_maternal_effect(ic,lrtsol(ic,1))
           call print_LRT(lrtsol(ic,1))
           if ( nbqtl == 2 ) then
              call print_lrt_solution(lrtsol(ic,2))
              call print_incidence_solution(listincsol(ic,3))
              call create_grid_file_2QTL(lrtsol(ic,2))
              call print_pat_mat_effect_2QTL(ic,lrtsol(ic,2))
              call print_LRT_2QTL(lrtsol(ic,2))
           end if
        end do
       end if


       ! print txt transcriptome format
       if (opt_sim == TRANSCIPTOME_ANALYSE) then
          call  print_transcriptome_H0(listincsol(:,1))
          call  print_transcriptome(lrtsol(:,1),listincsol(:,2))
          if ( nbqtl == 2) call  print_transcriptome(lrtsol(:,2),listincsol(:,3))
       end if


       if ( opt_sim /= SIMULATION) then
        call print_summary_analyse(lrtsol(:,1),listincsol(:,2),1)
        if ( nbqtl == 2) call print_summary_analyse(lrtsol(:,2),listincsol(:,3),2)
       end if
    end subroutine opti_unitrait
!!***

!!****f* m_qtlmap_analyse/opti_unitrait_incidence
!!  NAME
!!    opti_unitrait_incidence
!!  DESCRIPTION
!!   LA / LD / LDLA single trait with model description with optimised or linearised likelihood.
!!  INPUTS
!!    type_incidence : type of incidence matrix construction (INCIDENCE_TYPE_CLASSIC, INCIDENCE_TYPE_LD, INCIDENCE_TYPE_LDLA, INCIDENCE_TRAIT_COV)
!!    type_resolution: type of resolution (LINEAR_HOMOSCEDASTIC, LINEAR_HETEROSCEDASTIC, OPTIM_HETEROSCEDASTIC )
!!    nqtl           : number of qtl to test. nqtl >= 1
!!    hdam           : LD option, take dam haplotypes in considration
!!    opt_sim        : contexte of the analysis ( COMMON_ANALYSE | SIMULATION | TRANSCIPTOME_ANALYSE )
!!  OUTPUTS
!!    lrtsol     : likelihood ratio test information (see TYPE_LRT_SOLUTION)
!!    listincsol : solution for each hypothesis finded by the analysis (see TYPE_INCIDENCE_SOLUTION)
!! SOURCE
    subroutine opti_unitrait_incidence(lrtsol,listincsol,opt_sim,type_incidence,type_resolution,nqtl,hdam)
       use m_qtlmap_incidence_linear
       use m_qtlmap_incidence_optim
       use m_qtlmap_analyse_lin_gen
       integer                  ,intent(in)                                   :: nqtl
       type(TYPE_LRT_SOLUTION)  , intent(inout) ,dimension(ncar,nqtl)         :: lrtsol
       type(TYPE_INCIDENCE_SOLUTION)  ,intent(inout)  ,dimension(ncar,nqtl+1) :: listincsol
       logical                        ,intent(in)                             :: hdam

       integer                  ,intent(in)                                   :: opt_sim,type_incidence,type_resolution

       integer                               :: ic,i,j,ip,jm

       type(INCIDENCE_GEN_STRUCT)  ,dimension(:) ,allocatable :: workstruct
       logical                                                :: islinear

       allocate (workstruct(ncar))
       select case (type_resolution)
           case (LINEAR_HOMOSCEDASTIC)
              islinear = .true.
              call init_analyse_linear()
           case (LINEAR_HETEROSCEDASTIC)
              islinear = .true.
              call init_analyse_linear()
           case (OPTIM_HETEROSCEDASTIC)
              islinear = .false.
           case default
             call stop_application("Devel Error : Unknown incidence analyse type : "//trim(str(type_resolution)))
       end select

       !$OMP PARALLEL DEFAULT(SHARED)
       call init_analyse_gen
       !$OMP DO
       do ic=1,size(carac)
       !    if (.not.(natureY(ic) == 'r'.or. natureY(ic) == 'a')) cycle
           workstruct(ic)%hdam = hdam
           call log_mess("TRAIT ["//trim(carac(ic))//"]",INFO_DEF)
           call init_workstruct(workstruct(ic),type_incidence,type_resolution,nqtl,&
           (opt_sim == COMMON_ANALYSE).and.(type_incidence /= INCIDENCE_TYPE_LD) ,(opt_sim == COMMON_ANALYSE),.false.)

           if (islinear) then
            call opti_0qtl( ic, listincsol(ic,1),workstruct(ic),model_lin_h0)
           else
            call opti_0qtl( ic, listincsol(ic,1),workstruct(ic),model_optim_h0)
           end if

           do i=1,nqtl
             workstruct(ic)%nqtl=i
             if (islinear) then
               call opti_nqtl(i,ic,workstruct(ic),listincsol(ic,1+i),lrtsol(ic,i),model_lin_hn)
             else
               call opti_nqtl(i,ic,workstruct(ic),listincsol(ic,1+i),lrtsol(ic,i),model_optim_hn)
             end if
           end do ! iqtl
       end do ! ic
       !$OMP END DO
       call end_analyse_gen
       !$OMP END PARALLEL

       if (opt_sim == COMMON_ANALYSE) then ! Representation graphique
         do ic=1,ncar
            call print_start_unitrait(carac(ic))
            call print_incidence_solution(listincsol(ic,1))
            do i=1,nqtl
              call print_lrt_solution(lrtsol(ic,i))
              call print_incidence_solution(listincsol(ic,1+i))
              call print_confusion(workstruct(ic)%nalert,workstruct(ic)%alertQtl,workstruct(ic)%corrmax)
              if ( workstruct(ic)%ntest /= 0 )  then
               call print_test_nuisances(workstruct(ic)%ntest,workstruct(ic)%listtestnuis)
              end if
              if ( i==1 ) then
                call print_paternal_maternal_effect(ic,lrtsol(ic,1))
                call print_LRT(lrtsol(ic,1))
              end if
              if ( i==2 ) then
                call create_grid_file_2QTL(lrtsol(ic,2))
                call print_pat_mat_effect_2QTL(ic,lrtsol(ic,2))
                call print_LRT_2QTL(lrtsol(ic,2))
              end if
           end do ! iqtl
         end do
       end if

       if ( opt_sim /= SIMULATION .and. (type_incidence /= INCIDENCE_TYPE_LD) ) then
        do i=1,nqtl
         call print_summary_analyse(lrtsol(:,i),listincsol(:,i+1),i)
        end do
       end if

        ! print txt transcriptome format
       if (opt_sim == TRANSCIPTOME_ANALYSE) then
          call  print_transcriptome_H0(listincsol(:,1))
          do i=1,nqtl
            call  print_transcriptome(lrtsol(:,i),listincsol(:,i+1))
          end do
       end if

       select case (type_resolution)
           case (LINEAR_HOMOSCEDASTIC)
               call end_analyse_linear
           case (LINEAR_HETEROSCEDASTIC)
               call end_analyse_linear
           case (OPTIM_HETEROSCEDASTIC)
              islinear = .false.
           case default
             call stop_application("Devel Error : Unknown incidence analyse type : "//trim(str(type_resolution)))
       end select

       do ic=1,size(carac)
        if (.not.(natureY(ic) == 'r'.or.natureY(ic) == 'a')) cycle
        call release_ws(workstruct(ic))
       end do

       deallocate (workstruct)

    end subroutine opti_unitrait_incidence
!!***

!!****f* m_qtlmap_analyse/opti_multitrait_modlin
!!  NAME
!!    opti_multitrait_modlin
!!  DESCRIPTION
!!   LA multi trait with model description (multi qtl version, interaction with fixed effect and qtl are not managed)
!!  INPUTS
!!    nqtl       : number of qtl to test. nqtl >= 1
!!    opt_sim    : contexte of the analysis ( COMMON_ANALYSE | SIMULATION | TRANSCIPTOME_ANALYSE )
!!  OUTPUTS
!!    lrtsol     : likelihood ratio test information (see TYPE_LRT_SOLUTION)
!!    listincsol : solution for each hypothesis finded by the analysis (see TYPE_INCIDENCE_SOLUTION)
!! SOURCE
    subroutine opti_multitrait_modlin(lrtsol,listincsol,opt_sim,nqtl)
       use m_qtlmap_incidence_multi
  !     use m_qtlmap_analyse_lin_gen
       integer                  ,intent(in)                                   :: nqtl
       type(TYPE_LRT_SOLUTION)  , intent(inout) ,dimension(nqtl)              :: lrtsol
       type(TYPE_INCIDENCE_SOLUTION)  ,intent(inout)  ,dimension(ncar,nqtl+1) :: listincsol
       integer                  ,intent(in)                                   :: opt_sim

       integer :: i,j,iq
       type(INCIDENCE_GEN_STRUCT)   :: workstruct

        call init_analyse_gen
        call init_workstruct(workstruct,INCIDENCE_TYPE_CLASSIC,OPTIM_HETEROSCEDASTIC,nqtl,&
           (opt_sim == COMMON_ANALYSE),(opt_sim == COMMON_ANALYSE),.true.)
        call opti_0qtl_multi( listincsol(1,1) ,workstruct, nqtl)
        !workstruct%sigsquare(1,:)=workstruct%sigsquareEstime

        if (opt_sim == COMMON_ANALYSE) then
              call print_start_multitraits()
              call print_incidence_solution(listincsol(1,1))
        end if

        do i=1,nqtl
            workstruct%nqtl=i
            call opti_nqtl_multi(i,workstruct,listincsol(1,1+i),lrtsol(i),model_optim_multi_hn,nqtl)

             if (opt_sim == COMMON_ANALYSE) then ! Representation graphique
              call print_lrt_solution(lrtsol(i))
              call print_incidence_solution(listincsol(1,1+i))

              !call print_confusion(workstruct%nalert,workstruct%alertQtl,workstruct%corrmax)
!              if ( workstruct(ic)%ntest /= 0 )  then
!               call print_test_nuisances(workstruct(ic)%ntest,workstruct(ic)%listtestnuis)
!              end if
              if ( i==1 ) then
                !call print_paternal_maternal_effect(ic,lrtsol(ic,1))
                call print_LRT(lrtsol(1))
              end if

              if ( i==2 ) then
                call create_grid_file_2QTL(lrtsol(2))
                !call print_pat_mat_effect_2QTL(ic,lrtsol(ic,2))
                call print_LRT_2QTL(lrtsol(2))
              end if

             end if
        end do ! iqtl

        call end_analyse_gen


    end subroutine opti_multitrait_modlin
!!***

!!****f* m_qtlmap_analyse/opti_unitrait_interaction
!!  NAME
!!    opti_unitrait_interaction
!!  DESCRIPTION
!!    *** DEV interaction qtl analysis ****
!! SOURCE
     subroutine opti_unitrait_interaction(lrtsol,opt_sim)
       use m_qtlmap_incidence_linear
       use m_qtlmap_incidence_optim
       use m_qtlmap_analyse_lin_gen
       type(TYPE_LRT_SOLUTION)  , intent(inout) ,dimension(size(carac),2)  :: lrtsol
       integer                  ,intent(in)                                :: opt_sim

       integer                                                             :: type_resolution = LINEAR_HOMOSCEDASTIC
       integer                                                             :: type_incidence  = INCIDENCE_INTERACTION
       !type(TYPE_INCIDENCE_SOLUTION)  ,intent(inout)  ,dimension(ncar,3)   :: listincsol

       integer                               :: ic,supnbnivest,typeModel,i,j,ip

       real (kind=dp)   ,dimension(0:2,np)                                 :: sigsquare
       real (kind=dp)          ,dimension(np)                                 :: sigsquareEstime

       !1 ->H0, 2->H1, 3->H2, 4->H3 (avec interaction)
       type(TYPE_INCIDENCE_SOLUTION)    ,dimension(size(carac),4)        :: listincsol
       type(TYPE_LRT_SOLUTION)                  ,dimension(size(carac))  :: lrtsolInteraction

       type(INCIDENCE_GEN_STRUCT)  ,dimension(:) ,allocatable :: workstruct
       logical                                                :: islinear


       call init_analyse_gen
       allocate (workstruct(ncar))

       select case (type_resolution)
           case (LINEAR_HOMOSCEDASTIC)
              islinear = .true.
              call init_analyse_linear()
           case (LINEAR_HETEROSCEDASTIC)
              islinear = .true.
              call init_analyse_linear()
           case (OPTIM_HETEROSCEDASTIC)
              islinear = .false.
           case default
             call stop_application("Devel Error : Unknown incidence analyse type : "//trim(str(type_resolution)))
       end select

       do ic=1,size(carac)
           if (.not.(natureY(ic) == 'r'.or. natureY(ic) == 'a')) cycle
           sigsquare=0.d0

           call init_workstruct(workstruct(ic),type_incidence,type_resolution,2,&
           (opt_sim == COMMON_ANALYSE),(opt_sim == COMMON_ANALYSE),.false.)
           if (islinear) then
            call opti_0qtl( ic, listincsol(ic,1),workstruct(ic),model_lin_h0)
           else
            call opti_0qtl( ic, listincsol(ic,1),workstruct(ic),model_optim_h0)
           end if

           if (opt_sim == COMMON_ANALYSE) then
              call print_start_unitrait(carac(ic))
              call print_incidence_solution(listincsol(ic,1))
           end if

           do i=1,2
             workstruct(ic)%nqtl=i
             if (islinear) then
               call opti_nqtl(i,ic,workstruct(ic),listincsol(ic,1+i),lrtsol(ic,i),model_lin_hn)
             else
               call opti_nqtl(i,ic,workstruct(ic),listincsol(ic,1+i),lrtsol(ic,i),model_optim_hn)
             end if

             if (opt_sim == COMMON_ANALYSE) then ! Representation graphique
              call print_lrt_solution(lrtsol(ic,i))
              call print_incidence_solution(listincsol(ic,1+i))
              call print_confusion(workstruct(ic)%nalert,workstruct(ic)%alertQtl,workstruct(ic)%corrmax)
              if ( workstruct(ic)%ntest /= 0 )  then
               call print_test_nuisances(workstruct(ic)%ntest,workstruct(ic)%listtestnuis)
              end if
              if ( i==1 ) then
                call print_paternal_maternal_effect(ic,lrtsol(ic,1))
                call print_LRT(lrtsol(ic,1))
              end if

              if ( i==2 ) then
                call create_grid_file_2QTL(lrtsol(ic,2))
                call print_pat_mat_effect_2QTL(ic,lrtsol(ic,2))
                call print_LRT_2QTL(lrtsol(ic,2))
              end if

             end if
           end do ! i

           !Test avec interaction seulement lineaire la resolution.....a voir par la suite...

           call opti_2qtl_linear_interaction(ic,workstruct(ic),listincsol(ic,4),lrtsolInteraction(ic),typeModel)

           if (opt_sim == COMMON_ANALYSE) then ! Representation graphique
              call print_lrt_solution(lrtsol(ic,2))
              call print_incidence_solution(listincsol(ic,4))
              call create_grid_file_2QTL(lrtsol(ic,2))
              call print_pat_mat_effect_2QTL(ic,lrtsol(ic,2))
              call print_LRT_2QTL(lrtsol(ic,2))
           end if

            ! end if
       end do ! ic
       call end_analyse_linear

       do ic=1,size(carac)
        if (.not.(natureY(ic) == 'r'.or.natureY(ic) == 'a')) cycle
        do i=1,3
         call release(listincsol(ic,i))
        end do
       end do

       do ic=1,size(carac)
        if (.not.(natureY(ic) == 'r'.or.natureY(ic) == 'a')) cycle
        call release_ws(workstruct(ic))
       end do

       deallocate (workstruct)
       call end_analyse_gen


    end subroutine opti_unitrait_interaction
!!***

!!****f* m_qtlmap_analyse/opti_unitrait_modlin
!!  NAME
!!    opti_unitrait_modlin
!!  DESCRIPTION
!!   LA single trait with model description (1 qtl)
!!  INPUTS
!!    opt_sim    : contexte of the analysis ( COMMON_ANALYSE | SIMULATION | TRANSCIPTOME_ANALYSE )
!!  OUTPUTS
!!    lrtsol     : likelihood ratio test information (see TYPE_LRT_SOLUTION)
!!    listincsol : solution for each hypothesis finded by the analysis (see TYPE_INCIDENCE_SOLUTION)
!! SOURCE
    subroutine opti_unitrait_modlin(lrtsol,listincsol,opt_sim)
       use m_qtlmap_analyse_modlin
!#ifdef MANAGE_OMP
!       use m_qtlmap_analyse_modlin_omp
!#endif
       use m_qtlmap_analyse_lin_gen

       type(TYPE_LRT_SOLUTION)  , intent(inout) ,dimension(size(carac),1)    :: lrtsol
       integer          ,intent(in)                                          :: opt_sim
       type(TYPE_INCIDENCE_SOLUTION)  ,intent(inout)  ,dimension(ncar,2)     :: listincsol
       real (kind=dp)                        :: fmax

       integer                               :: ic,supnbnivest
       !modif carole : l'estimationd e la moyenne est parametrable
       logical                               :: est_moy = .true.
       logical                               :: est_var = .true.
       integer                                           :: i

       !$OMP PARALLEL DEFAULT(SHARED) PRIVATE(fmax,supnbnivest) IF ( opt_sim /= COMMON_ANALYSE )
       call init_analyse_gen
       call init_analyse_modlin

       !$OMP DO
       do ic=1,size(carac)
           if (.not.(natureY(ic) == 'r'.or.natureY(ic) == 'a')) cycle
           call log_mess("TRAIT ["//trim(carac(ic))//"]",INFO_DEF)
           call init_analyse_lin_gen(ic,1)
           call optinit(ic)
           call opti_0qtl_modlin(ic)
           call opti_1qtl_modlin(ic,lrtsol(ic,1),fmax,supnbnivest)


           if ( opt_sim /= SIMULATION) then
             ! Formattage des solutions sous H0 et H1
              call set_solution_hypothesis0(ic,listincsol(ic,1))
 
              call set_solution_hypothesis1(ic,listincsol(ic,2))
           end if

           !a cause de test_lin ,on ne peut pas sortir l affichage de la boucle de traitement
           !cette region devient donc critique pour afficher correctement et succesivement par chaque thread les resultats....
           if (opt_sim == COMMON_ANALYSE) then ! Representation graphique
              !$OMP CRITICAL
              call print_start_unitrait(carac(ic))
              !  colinearite donne des indications sur la colinearite entre chacun
              !  des effets QTL a estimer et tous les autres effets (avant elimination
              !  des effets non estimables)
              !
              call confusion('avant',lrtsol(ic,1)%chrmax(0),ic,est_moy,'LA  ')
              ! Impression du resultat sous H0
              call print_incidence_solution(listincsol(ic,1))
              ! Impression de la courbe LRT sous H1
              call print_lrt_solution(lrtsol(ic,1))
              ! Impression des resultats sous H1
              call print_incidence_solution(listincsol(ic,2))
              ! Effet paternel et maternel sous H1 dans un fichier dedie
              call print_paternal_maternal_effect(ic,lrtsol(ic,1))
              ! Affichage de la valeur LRT a chaque position dans un fichier dedie
              call print_LRT(lrtsol(ic,1))
              !
              ! test_modlin donne la signification des effets du modele
              !
              call test_lin(lrtsol(ic,1)%chrmax(0),ic,est_moy,supnbnivest,fmax,lrtsol(ic,1)%nxmax(0))
              !
              !  confusion indique si'l y a une confusion entre les effets finalement
              !  retenus dans le modele
              call confusion('apres',lrtsol(ic,1)%chrmax(0),ic,est_moy,'LA  ')
              !$OMP END CRITICAL
          end if
          call end_analyse_lin_gen
       end do
       !$OMP END DO
       call end_analyse_modlin
       call end_analyse_gen
       !$OMP END PARALLEL

        if ( opt_sim /= SIMULATION) then
          call print_summary_analyse(lrtsol(:,1),listincsol(:,2),1)
       end if

        ! print txt transcriptome format
       if (opt_sim == TRANSCIPTOME_ANALYSE) then
          call  print_transcriptome_H0(listincsol(:,1))
          call  print_transcriptome(lrtsol(:,1),listincsol(:,2))
       end if

    end subroutine opti_unitrait_modlin
!!***

!!****f* m_qtlmap_analyse/opti_unitrait_modlin_new
!!  NAME
!!    opti_unitrait_modlin_new
!!  DESCRIPTION
!!   LA / LD / LDLA /LDJH single trait with model description (1 qtl) optimisation method
!!  INPUTS
!!    option_var  : "hete" : heteroscedastic, "homo" : homoscedastic
!!    option_anal : "LDLA", "LD  ", "LA  ","LDJH"
!!    hdam        : LD option, take dam haplotypes in considration
!!    opt_sim     : contexte of the analysis ( COMMON_ANALYSE | SIMULATION | TRANSCIPTOME_ANALYSE )
!!  OUTPUTS
!!    lrtsol     : likelihood ratio test information (see TYPE_LRT_SOLUTION)
!!    listincsol : solution for each hypothesis finded by the analysis (see TYPE_INCIDENCE_SOLUTION)
!! SOURCE
    subroutine opti_unitrait_modlin_new(lrtsol,listincsol,option_var,option_anal,hdam,opt_sim)
       use m_qtlmap_analyse_modlin_ldla
       use m_qtlmap_analyse_lin_gen

       type(TYPE_LRT_SOLUTION)  , intent(inout) ,dimension(size(carac),1)    :: lrtsol
       type(TYPE_INCIDENCE_SOLUTION)  ,intent(inout)  ,dimension(ncar,2)     :: listincsol
       character(len=4)         , intent(in)                                 :: option_var,option_anal
       integer          ,intent(in)                                          :: opt_sim
       logical          ,intent(in)                                          :: hdam

       real (kind=dp)       , dimension(size(carac),np)  :: std_all,ap_all
       logical              , dimension(nm)              :: subphasm
       real(kind=dp)                                     :: fmax
       integer                               :: ic,supnbnivest,nposx,CHR
       !modif carole : l'estimationd e la moyenne est parametrable
       logical                               :: est_moy = .true.
       logical                               :: est_var = .true.
       logical                               :: qtl,hsire

       hsire=( (option_anal == 'LD  ') .or. (option_anal == 'LDLA') .or. (option_anal == 'LDJH') )
       qtl  = ( (option_anal == 'LA  ') .or. (option_anal == 'LDLA') .or. (option_anal == 'LDJH') )

       call init_analyse_gen
       call init_analyse_modlin_ldla

       do ic=1,size(carac)
           if (.not.(natureY(ic) == 'r'.or.natureY(ic) == 'a')) cycle
           call log_mess("TRAIT ["//trim(carac(ic))//"]",INFO_DEF)
           call init_analyse_lin_gen(ic,1)
           call optinit(ic)
           call opti_0qtl_modlin_ldla(ic,.true.,option_var)
           call opti_1qtl_modlin_ldla(ic,lrtsol(ic,1),fmax,supnbnivest,.true.,option_var,option_anal,hdam)

           if ( opt_sim /= SIMULATION) then
             ! Formattage des solutions sous H0 et H1
              call set_solution_hypothesis0(ic,(option_var=="hete"),listincsol(ic,1))
              call set_solution_hypothesis1(ic,(option_var=="hete"),qtl,hsire,hdam,listincsol(ic,2))
           end if

            if (opt_sim == COMMON_ANALYSE) then ! Representation graphique
              call print_start_unitrait(carac(ic))
              !  colinearite donne des indications sur la colinearite entre chacun
              !  des effets QTL a estimer et tous les autres effets (avant elimination
              !  des effets non estimables)
              !
              call confusion('avant',lrtsol(ic,1)%chrmax(0),ic,est_moy,option_anal)
              ! Impression du resultat sous H0
              call print_incidence_solution(listincsol(ic,1))
              ! Impression de la courbe LRT sous H1
              call print_lrt_solution(lrtsol(ic,1))
              ! Impression des resultats sous H1
              call print_incidence_solution(listincsol(ic,2))
              ! Effet paternel et maternel sous H1 dans un fichier dedie
              call print_paternal_maternal_effect(ic,lrtsol(ic,1))
              ! Affichage de la valeur LRT a chaque position dans un fichier dedie
              call print_LRT(lrtsol(ic,1))
              !
              ! test_modlin donne la signification des effets du modele
              !
             ! call test_lin(lrtsol(ic,1)%chrmax(0),ic,est_moy,supnbnivest,fmax,lrtsol(ic,1)%nxmax(0))
              !
              !  confusion indique si'l y a une confusion entre les effets finalement
              !  retenus dans le modele
              call confusion('apres',lrtsol(ic,1)%chrmax(0),ic,est_moy,'LA  ')
          end if

        call end_analyse_lin_gen
       end do

!       if ( opt_sim /= OPT_SIMULATION) then
!           call get_eff_paternal_and_total(effp,efft)
!           call print_summary_0versus1(size(carac),carac,sigt,size(pere),pere,efft,effp,std_all,ap_all,xlrmax0_1,dxma0_1)
!       end if

       call end_analyse_modlin_ldla
       call end_analyse_gen

    end subroutine opti_unitrait_modlin_new
!!***

!!****f* m_qtlmap_analyse/opti_unitrait_modlin_cox
!!  NAME
!!    opti_unitrait_modlin_cox
!!  DESCRIPTION
!!   LA analysis with cox model
!!  INPUTS
!!    opt_sim     : contexte of the analysis ( COMMON_ANALYSE | SIMULATION | TRANSCIPTOME_ANALYSE )
!!  OUTPUTS
!!    lrtsol     : likelihood ratio test information (see TYPE_LRT_SOLUTION)
!!    listincsol : solution for each hypothesis finded by the analysis (see TYPE_INCIDENCE_SOLUTION)
!! SOURCE
    subroutine opti_unitrait_modlin_cox(lrtsol,listincsol,opt_sim)
        use m_qtlmap_analyse_modlin_cox
        use m_qtlmap_analyse_lin_gen

       type(TYPE_LRT_SOLUTION)  , intent(inout) ,dimension(size(carac),1)    :: lrtsol
       type(TYPE_INCIDENCE_SOLUTION)  ,intent(inout)  ,dimension(ncar,2)     :: listincsol
       integer          ,intent(in)                        :: opt_sim

       real (kind=dp)                        :: fmax
       integer                               :: ic,supnbnivest,npar,i
       logical                               :: est_moy = .false.
       logical                               :: est_var = .false.

       call init_analyse_gen
       do ic=1,size(carac)
           if (natureY(ic) /= 'r') cycle
           call log_mess("TRAIT ["//trim(carac(ic))//"]",INFO_DEF)
           call init_analyse_lin_gen(ic,1)
             !npar is modified because it is equal to 3*np + 2nm in m_qtlmap_analyse_lin_gen.f95
             npar=np+nm
           call init_analyse_modlin_cox
           call optinit(ic)
           call calcul_rang(ic)
           call opti_0qtl_modlin_cox(ic,est_moy)
           call opti_1qtl_modlin_cox(ic,lrtsol(ic,1),fmax,supnbnivest)
       !call stop_application("Survival Analysis with a Cox model (option=5) is in development ")

            if (opt_sim == COMMON_ANALYSE) then ! Representation graphique
              call print_start_unitrait(carac(ic))
               !  colinearite donne des indications sur la colinearite entre chacun
               !  des effets QTL a estimer et tous les autres effets (avant elimination
               !  des effets non estimables)
              call confusion('avant',lrtsol(ic,1)%chrmax(0),ic,.false.,'LA  ')
               ! Impression de la courbe LRT sous H1
              call print_lrt_solution(lrtsol(ic,1))

              ! Formattage des solutions sous H0 et H1
              call set_solution_hypothesis0(ic,listincsol(ic,1))
              call set_solution_hypothesis1(ic,listincsol(ic,2))
              ! Impression du resultat sous H0
              call print_incidence_solution_risk_factor(listincsol(ic,1))
              ! Impression des resultats sous H1
              call print_incidence_solution_risk_factor(listincsol(ic,2))
               ! Effet paternel et maternel sous H1 dans un fichier dedie
              call print_paternal_maternal_effect(ic,lrtsol(ic,1))
              ! Affichage de la valeur LRT a chaque position dans un fichier dedie
              call print_LRT(lrtsol(ic,1))
              !
              ! test_modlin donne la signification des effets du modele
              !
              call test_lin_cox(lrtsol(ic,1)%chrmax(0),ic,est_moy,supnbnivest,fmax,lrtsol(ic,1)%nxmax(0))
              !
              !  confusion indique si'l y a une confusion entre les effets finalement
              !  retenus dans le modele
              call confusion('apres',lrtsol(ic,1)%chrmax(0),ic,est_moy,'LA  ')
           end if
            call end_analyse_lin_gen
           call end_analyse_modlin_cox
       enddo

       do ic=1,size(carac)
        if (natureY(ic) /= 'r') cycle
        do i=1,2
         call release(listincsol(ic,i))
        end do
       end do

!TODO: Olivier il faut mettre le call print_summary_analyse(efft,effpp,lrtsol(:,1),listincsol(:,2),1) je n'y arrive pas!!!!! (carole)

       call end_analyse_gen
    end subroutine opti_unitrait_modlin_cox
!!***


!!****f* m_qtlmap_analyse/opti_unitrait_discret
!!  NAME
!!    opti_unitrait_discret
!!  DESCRIPTION
!!   LA analysis for discrete data with model description (1 qtl)
!!  INPUTS
!!    opt_sim     : contexte of the analysis ( COMMON_ANALYSE | SIMULATION | TRANSCIPTOME_ANALYSE )
!!  OUTPUTS
!!    lrtsol     : likelihood ratio test information (see TYPE_LRT_SOLUTION)
!!    listincsol : solution for each hypothesis finded by the analysis (see TYPE_INCIDENCE_SOLUTION)
!! SOURCE
    subroutine opti_unitrait_discret(lrtsol,listincsol,opt_sim)
       use m_qtlmap_analyse_discret_unitrait
       use m_qtlmap_analyse_lin_gen

       type(TYPE_LRT_SOLUTION)  , intent(inout) ,dimension(size(carac),1)    :: lrtsol
       type(TYPE_INCIDENCE_SOLUTION)  ,intent(inout)  ,dimension(ncar,2)     :: listincsol

       integer          ,intent(in)                      :: opt_sim

       real (kind=dp)                        :: fmax

       logical              , dimension(nm)              :: subphasm

       integer                               :: ic,supnbnivest,i
       logical                               :: est_moy = .false.
       logical                               :: est_var = .true.

       call init_analyse_gen
       do ic=1,size(carac)
           if (natureY(ic) /= 'i') cycle
           call log_mess("TRAIT ["//trim(carac(ic))//"]",INFO_DEF)
           call init_analyse_lin_gen(ic,1)
           call init_analyse_discret_unitrait
           call optinit(ic)
           call opti_0qtl_discret_unitrait(ic)
           call opti_1qtl_discret_unitrait(ic,lrtsol(ic,1),fmax,supnbnivest)

            if (opt_sim == COMMON_ANALYSE) then ! Representation graphique
              call print_start_unitrait(carac(ic))
              !  colinearite donne des indications sur la colinearite entre chacun
              !  des effets QTL a estimer et tous les autres effets (avant elimination
              !  des effets non estimables)
              !
              call confusion('avant',lrtsol(ic,1)%chrmax(0),ic,est_moy,'LA  ')

              ! Formattage des solutions sous H0 et H1
              call set_solution_hypothesis0(ic,listincsol(ic,1))
              call set_solution_hypothesis1(ic,listincsol(ic,2))
              ! Impression du resultat sous H0
              call print_incidence_solution(listincsol(ic,1))
              ! Impression de la courbe LRT sous H1
              call print_lrt_solution(lrtsol(ic,1))
              ! Impression des resultats sous H1
              call print_incidence_solution(listincsol(ic,2))
              ! Effet paternel et maternel sous H1 dans un fichier dedie
              call print_paternal_maternal_effect(ic,lrtsol(ic,1))
              ! Affichage de la valeur LRT a chaque position dans un fichier dedie
              call print_LRT(lrtsol(ic,1))
              !
              ! test_modlin donne la signification des effets du modele
              !
              call test_lin(lrtsol(ic,1)%chrmax(0),ic,est_moy,supnbnivest,fmax,lrtsol(ic,1)%nxmax(0))
              !
              !  confusion indique si'l y a une confusion entre les effets finalement
              !  retenus dans le modele
              call confusion('apres',lrtsol(ic,1)%chrmax(0),ic,est_moy,'LA  ')
           end if

           call end_analyse_lin_gen
           call end_analyse_discret_unitrait
       end do

       call end_analyse_gen

    end subroutine opti_unitrait_discret
!!***

!!****f* m_qtlmap_analyse/opti_multitraits
!!  NAME
!!    opti_multitraits
!!  DESCRIPTION
!!    LA analysis for a set of traits with a multivariate analysis
!!  INPUTS
!!    opt_sim     : contexte of the analysis ( COMMON_ANALYSE | SIMULATION | TRANSCIPTOME_ANALYSE )
!!  OUTPUTS
!!    lrtsol     : likelihood ratio test information (see TYPE_LRT_SOLUTION)
!!    listincsol : solution for each hypothesis finded by the analysis (see TYPE_INCIDENCE_SOLUTION)
!!    rhoi       : residual correlation matrix finded for each hypothesis
!! SOURCE
    subroutine opti_multitraits(lrtsol,listincsol,rhoi,opt_sim)
       use m_qtlmap_analyse_multitrait
       type(TYPE_LRT_SOLUTION)  , intent(inout)                        :: lrtsol
       type(TYPE_INCIDENCE_SOLUTION)  ,intent(inout)  ,dimension(2)    :: listincsol
       real (kind=dp) ,dimension(size(carac),size(carac)), intent(out) :: rhoi

       integer , intent(in)                                            :: opt_sim

       logical              , dimension(nm)              :: subphasm
       character(len=LEN_DEF) , dimension(nm)            :: submere
       logical                                           :: impfem
       real (kind=dp)       , dimension(nm)              :: xmoym1,am1
       integer                                           :: nmsub,efft
       integer            , dimension(np)                :: effpp

       integer    :: ic,i

       call init_analyse_gen

       if (ncar <= 1) then
         call stop_application("can not perform a multitrait analysis on "//trim(str(ncar))//" traits")
       end if

       do ic=1,ncar
           if( natureY(ic) == 'r'.or.natureY(ic) == 'a') cycle
           call stop_application("Multitrait analysis is only available for continuous traits")
       end do

       call init_analyse_multitrait
       call optinit_mcar
       call opti_mcar_0qtl
       call opti_mcar_1qtl(lrtsol)

       if (opt_sim == COMMON_ANALYSE .or. opt_sim == TRANSCIPTOME_ANALYSE) then ! Representation graphique

          call print_start_multitraits()
          call set_solution_hypothesis0(listincsol(1))
          call print_incidence_solution(listincsol(1))
          call print_lrt_solution(lrtsol)
          call set_solution_hypothesis1(listincsol(2))
          call print_incidence_solution(listincsol(2))

          call print_paternal_maternal_effect(1,lrtsol)
          call print_LRT(lrtsol)

      end if

      if ( opt_sim /= SIMULATION) then
        ! call get_eff_paternal_and_total(effpp,efft)
         ! TODO : summary pour multi
      end if

      call end_analyse_multitrait
      call end_analyse_gen

    end subroutine opti_multitraits
!!***

!!****f* m_qtlmap_analyse/opti_multitrait_DA
!!  NAME
!!    opti_multitrait_DA
!!  DESCRIPTION
!!    LA for a set of traits (without missing data) with a discriminante analysis
!!  INPUTS
!!    opt_sim     : contexte of the analysis ( COMMON_ANALYSE | SIMULATION | TRANSCIPTOME_ANALYSE )
!!  OUTPUTS
!!    lrtsol     : likelihood ratio test information (see TYPE_LRT_SOLUTION)
!!    rhoi       : residual correlation matrix finded for each hypothesis
!! SOURCE
 subroutine opti_multitrait_DA(lrtsol,opt_sim)
       use m_qtlmap_analyse_gen
       use m_qtlmap_analyse_multitrait_DA
       type(TYPE_LRT_SOLUTION)  , intent(out)          :: lrtsol
       integer          ,intent(in)                    :: opt_sim

       real (kind=dp) , dimension(:,:) ,allocatable    :: coeff
       real (kind=dp) , dimension(size(carac))     :: coeff_max
       real (kind=dp)       , dimension(np)  :: std_all,ap_all
       real (kind=dp)       , dimension(nm)              :: xmoym1,am1
       logical              , dimension(nm)              :: subphasmm
       character(len=LEN_DEF) , dimension(nm)              :: submere
       logical                                           :: impfem

       integer            , dimension(np)                :: effpp
       integer                                           :: i,ic,nmsub,efft,n,ilong,ix,kd,ip,chr
       real (kind=dp), dimension (nd)                    :: yda
       integer                ,dimension(size(carac))    :: lic
       real (kind=dp)                                    :: dxmax
       type(TYPE_LRT_SOLUTION) ,dimension(1) :: listLrtSol
       type(TYPE_INCIDENCE_SOLUTION)   :: incsol

       call init_analyse_gen
       allocate (coeff(ncar,get_maxnpo()))

       if (ncar <= 1) then
         call stop_application("can not perform a multitrait analysis on "//trim(str(ncar))//" traits")
       end if

       do ic=1,ncar
           if( natureY(ic) == 'r'.or.natureY(ic) == 'a') cycle
           call stop_application("Discriminant multitrait analysis is only available for continuous traits")
       end do

       call init_analyse_DA_1QTL

       coeff=0.d0

       call new(1,lrtsol)
       ncar=1
       lrtsol%lrtmax(0)=-1.d75
       dxmax=-1.d6
       lrtsol%nxmax=0
       lrtsol%chrmax=1
       do chr=1,nchr
         n=0
         ilong=get_ilong(chr)
        do ix=0,ilong,pas
         n=n+1
         call perform_DA_1QTL(chr,n,yda,coeff,get_maxnpo())
         call optinit_da(yda)
         call opti_DA_0qtl
         call opti_DA_1qtl(chr,ix,n,yda,lrtsol)
         if(absi(lrtsol%chrmax(0),lrtsol%nxmax(0)) >  dxmax) then
           dxmax=absi(lrtsol%chrmax(0),lrtsol%nxmax(0))
           coeff_max=coeff(:,n)
         end if
         call log_mess('at position ix:'//str(ix)//' LRT max:'//str(lrtsol%lrtmax(0))//&
         " POS:"//str(absi(lrtsol%chrmax(0),lrtsol%nxmax(0))),DEBUG_DEF)
       end do
      end do

       if (opt_sim == COMMON_ANALYSE) then ! Representation graphique
         std_all(:)=std
         ap_all(:)=ap
         call print_start_multitrait_DA
         call print_lrt_solution(lrtsol)
         call set_solution_hypothesis1(incsol)
         call print_incidence_solution(incsol)
         call release(incsol)

         call print_coeff_linear_combination_max(coeff_max)
         call print_paternal_maternal_effect(1,lrtsol)
         call print_LRT(lrtsol)
         call print_coeff_linear_combination(lrtsol%chrmax(0),get_ilong(lrtsol%chrmax(0)),get_maxnpo(),coeff)
       end if

       if ( opt_sim /= SIMULATION) then
       ! call get_eff_paternal_and_total(effpp,efft)
        ! TODO : summary pour multi
       end if

       ncar=size(carac)
       deallocate (coeff)

       call end_analyse_DA_1QTL
       call end_analyse_gen

    end subroutine opti_multitrait_DA
!!***

!!****f* m_qtlmap_analyse/quantile
!!  NAME
!!    quantile
!!  DESCRIPTION
!!   computes quantiles (Harrell & Davis 1982) and write a result file.
!!  INPUTS
!!    simula_file : name of file to print the output
!!    nsim        : number of simulation
!!    opt_calcul  : analysis id
!!    opt_qtl     : number of qtl hypothesis (analysis can be not compatible with this option)
!!    nqtl        : number of qtl simulated
!!    lrtsol      : likelihood ratio test information (see TYPE_LRT_SOLUTION)
!!  NOTES
!!    use : MATH_QTLMAP_M01CAF, MATH_QTLMAP_G01AAF, QUANTILE1, print_resume_simulation, print_resume_simulation_2
!! SOURCE
      subroutine quantile(simula_file,nsim,opt_calcul,opt_qtl,nqtl,lrtsol)
      character(len=LENGTH_MAX_FILE) , intent(in)   :: simula_file
      integer                        , intent(in)   :: nsim,opt_calcul,opt_qtl,nqtl ! nqtl => le nombre de qtl definit dans la simulation
                                                                                    ! opt_qtl => nombre de qtl a detecter pour le calcul
      type(TYPE_LRT_SOLUTION)  , dimension(nsim,ncar,opt_qtl),intent(in)      :: lrtsol

      real (kind=dp) :: prob(16)
      real (kind=dp) :: x(nsim),wt(nsim)
      real (kind=dp)  ,dimension (ncar,16)     :: z
      real (kind=dp)  ,dimension (1,16)     :: z2

      CHARACTER(len=LEN_BUFFER_WORD) :: temp,nameT,carac_t(1)
      integer :: nc,ifail,i,im,j,ii,iqtl,iwt
      real (kind=dp) :: xmin1,xmax1,ymu1,sig1,s21,s31,pg,wtsum,p_t,d

      data prob /.1d0,.2d0,.3d0,.4d0,.5d0,.6d0,.7d0,.8d0,.9d0,.95d0,&
      .99d0,.995d0,.9973d0,.999d0,.99947d0,.999947d0/

     nc=size(carac)
     nameT=""
     if ( is_multitrait_analysis(opt_calcul) ) then
       nc = 1
       nameT="* All traits *"
     end if

     ifail=0
     iwt = 0
     ! pour tous les lrts trouve sous avec qtl==opt_qtl
     do iqtl=1,opt_qtl   ! ? / opt_qtl
      do im=0,iqtl-1 ! on ecrit tous les test  H: im / opt_qtl
        temp=trim(str(im))//'vs'//trim(str(iqtl))//'Q'
        do i=1,nc
          do j=1,nsim
           x(j)=lrtsol(j,i,iqtl)%lrtmax(im)
          end do
          if (nameT=="") nameT=trim(carac(i))

          !Sort X from index 1 to ns ('a' ascendant)....
          call MATH_QTLMAP_M01CAF(X,1,nsim,'A',IFAIL)
          !Computing  Mean, variance, skewness, kurtosis Minimum Maximum
          call MATH_QTLMAP_G01AAF(nsim,x,IWT,WT,YMU1,SIG1,S21,S31,XMIN1,XMAX1,WTSUM,IFAIL)
          do ii=9,16
             z(i,ii)=QUANTILE1(nsim,prob(ii),X)
          end do

          call print_resume_simulation(nameT,temp,nsim,&
                      YMU1,SIG1,S21,S31,XMIN1,XMAX1,prob,z(i,:),9,16)
          nameT=""
       end do
       if ( .not. is_multitrait_analysis(opt_calcul)  ) then
         call print_resume_simulation_2(temp,ncar,carac,z)
       else
         carac_t(1)=trim(nameT)
         z2(1,:)=z(1,:)
         call print_resume_simulation_2(temp,1,carac_t,z2)
       end if
      end do
     end do

    end subroutine quantile
!!***

!!****f* m_qtlmap_analyse/QUANTILE1
!!  NAME
!!    QUANTILE1
!!  DESCRIPTION
!!    CALCUL DES QUANTILES D'UNE DISTRIBUTION ORDONNEE
!!  INPUTS
!!    n :
!!    p :
!!    x :
!!  RETURN
!!
!!  NOTES
!!    use : W
!! SOURCE
      FUNCTION QUANTILE1(n,p,X) result (Q)
      integer , intent(in)                     :: n
      real (kind=dp) ,intent(in)               :: p
      real (kind=dp) , dimension(:),intent(in) :: X

      real (kind=dp)              :: Q
      integer                     :: i

      Q=0.D0
      IF (N>SIZE(X)) THEN
         call stop_application("QUANTILE1 [m_qtlmap_analyse] N ["//trim(str(N))//&
         '] to high [max='//trim(str(SIZE(X)))//']')
      END IF
 6010 DO 6011 I=1,N
      Q=Q+W(n,I,p)*X(I)
 6011 Continue
      RETURN
      END FUNCTION QUANTILE1
!!***

!!****f* m_qtlmap_analyse/W
!!  NAME
!!     W
!!  DESCRIPTION
!!   CALCUL DE W
!!  INPUTS
!!    n   :
!!    i   :
!!    p   :
!!  NOTES
!!    use : MATH_QTLMAP_LOWERTAIL_BETA, MATH_QTLMAP_LOWERTAIL_BETA
!! SOURCE
      FUNCTION W(n,i,p) result(Wr)
      integer , intent(in)        :: n
      integer , intent(in)        :: i
      real (kind=dp) , intent(in) :: p
      real (kind=dp)              :: Wr,TOL,X1,X2,a,b
      real (kind=dp)              :: p1,p2
      integer                     :: ifail

      TOL=1.D-5
      X1=dble(i)/dble(n)
      X2=dble(i-1)/dble(n)
      a=p*dble(n+1)
      b=(1.D0-p)*dble(n+1)
      ifail=1
      !CALL MATH_QTLMAP_G01EEF(X1,a,b,tol,p1,q1,pdf1,ifail)
      call MATH_QTLMAP_LOWERTAIL_BETA(X1, a, b, TOL, p1, IFAIL)
      ifail=1
      !CALL MATH_QTLMAP_G01EEF(X2,a,b,tol,p2,q2,pdf2,ifail)
      call MATH_QTLMAP_LOWERTAIL_BETA(X2,a,b,tol,p2,ifail)
      Wr=p1-p2
      RETURN
      END FUNCTION W
!!***


end module m_qtlmap_analyse
