!!****m* ANALYSE/m_qtlmap_incidence
!!  NAME
!!    m_qtlmap_incidence
!!
!!  DESCRIPTION
!!
!!  NOTES
!!
!!  BUGS
!!
!!  HISTORY
!!
!!  SEE ALSO
!!
!!  COPYRIGHT
!!***
!! You can use this space for remarks that should not be included
!! in the documentation.
!!/
module m_qtlmap_incidence
    use m_qtlmap_data
    use qtlmap_types
    use m_qtlmap_haplotype_data
    use m_qtlmap_log
    use m_qtlmap_tools
    use m_qtlmap_solve_linear
    use m_qtlmap_optimization
    use m_qtlmap_output_handler
    use m_qtlmap_math
    use m_qtlmap_analyse_gen , only : set_ntnivmax
    use m_qtlmap_haplotype_ldla , only : free_internal_struct_ldla,set_haplo_for_ldla

    implicit none

!!****d* m_qtlmap_incidence/LINEAR_HOMOSCEDASTIC
!!  NAME
!!   LINEAR_HOMOSCEDASTIC
!!  DESCRIPTION
!!   Constant id for linear homoscedastic analysis
!!  NOTES
!!
!!***
    integer, parameter                            , public   :: LINEAR_HOMOSCEDASTIC      = 1
!!****d* m_qtlmap_incidence/LINEAR_HETEROSCEDASTIC
!!  NAME
!!   LINEAR_HETEROSCEDASTIC
!!  DESCRIPTION
!!   Constant id for linear heteroscedastic analysis
!!  NOTES
!!
!!***
    integer, parameter                            , public   :: LINEAR_HETEROSCEDASTIC    = 2
!!****d* m_qtlmap_incidence/OPTIM_HETEROSCEDASTIC
!!  NAME
!!   OPTIM_HETEROSCEDASTIC
!!  DESCRIPTION
!!   Constant id for optimized heteroscedastic analysis
!!  NOTES
!!
!!***
    integer, parameter                            , public   :: OPTIM_HETEROSCEDASTIC     = 3
!    integer, parameter                            , public   :: OPTIM_HETEROSCEDASTIC_COX = 99

    ! **************** Type de constructiond e la matrice d contingence ************************************************
!!****d* m_qtlmap_incidence/INCIDENCE_TYPE_CLASSIC
!!  NAME
!!   INCIDENCE_TYPE_CLASSIC
!!  DESCRIPTION
!!   Constant id for construction type of the contingence matrix:
!!    * General mean
!!    * Polygenic effect
!!    * QTL effect
!!    * Covariate
!!    * Fixed Effect
!!  NOTES
!!   build_incidence_matrix
!!
!!***
    integer, parameter                            , public   :: INCIDENCE_TYPE_CLASSIC        = 1
!!****d* m_qtlmap_incidence/INCIDENCE_TRAIT_COV
!!  NAME
!!   INCIDENCE_TRAIT_COV
!!  DESCRIPTION
!!   Constant id for construction type of the contingence matrix:
!!    * General mean
!!    * QTL effect
!!    * Polygenic effect
!!    * Others continue trait as a covariate
!!    * Covariate
!!    * Fixed Effect
!!  NOTES
!!   build_incidence_matrix
!!
!!***
    integer, parameter                            , public   :: INCIDENCE_TRAIT_COV           = 2
!!****d* m_qtlmap_incidence/INCIDENCE_INTERACTION
!!  NAME
!!   INCIDENCE_INTERACTION
!!  DESCRIPTION
!!   Constant id for construction type of the contingence matrix:
!!    <DEV>
!!  NOTES
!!   build_incidence_matrix
!!
!!***
    integer, parameter                            , public   :: INCIDENCE_INTERACTION         = 3

!!****d* m_qtlmap_incidence/INCIDENCE_TYPE_LD
!!  NAME
!!   INCIDENCE_TYPE_LD
!!  DESCRIPTION
!!   Constant id for construction type of the contingence matrix:
!!    * General mean
!!    * Haplotype effect
!!    * Polygenic effect
!!    * Others continue trait as a covariate
!!    * Covariate
!!    * Fixed Effect
!!  NOTES
!!   build_incidence_matrix
!!
!!***
    integer, parameter                            , public   :: INCIDENCE_TYPE_LD             = 5
!!****d* m_qtlmap_incidence/INCIDENCE_TYPE_LDLA
!!  NAME
!!   INCIDENCE_TYPE_LDLA
!!  DESCRIPTION
!!   Constant id for construction type of the contingence matrix:
!!    * General mean
!!    * Haplotype effect
!!    * QTL effect
!!    * Polygenic effect
!!    * Others continue trait as a covariate
!!    * Covariate
!!    * Fixed Effect
!!  NOTES
!!   build_incidence_matrix
!!
!!***
    integer, parameter                            , public   :: INCIDENCE_TYPE_LDLA           = 6

!!****t* m_qtlmap_incidence/family_type
!!  NAME
!!    family_type
!!
!!  DESCRIPTION
!!   Basic type with the proba of transmission of a family
!!  SOURCE
    type family_type
      integer :: firstKd = -1 ! correspond a l index du premier descendant (equivalent ligne mat incidence)
      integer :: lastKd  = -2 ! correspond a l index du dernier descedant (equivalent ligne mat incidence)
      real(kind=dp) , dimension(:,:,:),pointer :: ppt => null()!proba de trsanmition recu par le pere
      real(kind=dp) , dimension(:,:,:),pointer :: pmt => null()! proba de transmision recu par la mere si estim
     ! integer   ,dimension(:) ,pointer         :: ntniv_dam !indice dans la matrice d incidence correspondant a l effet pere et le qtl
    end type family_type
!!***

!!****t* m_qtlmap_incidence/SireFamily_type
!!  NAME
!!    SireFamily_type
!!
!!  DESCRIPTION
!!   Complex type with the proba of transmission of a sire family
!!  SOURCE
    type SireFamily_type
      type(family_type)                        :: half_sib ! famille complete de demi-frere (ligne ref incidence)
      type(family_type) , dimension(:),pointer :: full_sib => null()! famille de plein frere (ligne ref incidence)
      real(kind=dp)     , dimension(:),pointer :: sig0 => null()!variance du caractere dans la famille de pere
      real(kind=dp)     , dimension(:),pointer :: xmu0 => null()!moyenne du caractere dans la famille de pere
      real(kind=dp) , dimension(:,:),pointer   :: ppt => null()!proba de transmition recu par le pere
      !integer       ,dimension(:) ,pointer     :: ntniv_sire !indice dans la matrice d incidence correspondant a l effet pere et le qtl
    end type SireFamily_type
!!***


!!****t* m_qtlmap_incidence/DATASET_TYPE
!!  NAME
!!    DATASET_TYPE
!!
!!  DESCRIPTION
!!   description of the user dataset
!!  SOURCE
    type DATASET_TYPE
        !Index trait
        integer                                                  :: ncar             = 0
        !Struct of Families (missing data are removes) with pdd for full and half sib families
        type(SireFamily_type) , dimension(:) , pointer           :: lSires           => NULL()
        !Number of animals referenced in the contingence matrix (missing data removed)
        integer                                                  :: nkd              = 0
        !Maximum number recorded in a family full sib
        integer                                                  :: nkd_max_by_fam   = 0
        !Performance without missing data : size : nkd
        real(kind=dp)      , dimension(:,:),   pointer           :: Y                => NULL()
        integer            , dimension(:,:),   pointer           :: YDISCRETORD      => NULL()
        !Censored data without missing data : size nkd
        real(kind=dp)      , dimension(:,:),   pointer           :: CD               => NULL()
        !Covariance of traits (H0) dim : ncar,ncar
        real(kind=dp)      , dimension(:,:),   pointer           :: cov              => NULL()
        !Residual correlation (H0) dim : ncar,ncar
        real(kind=dp)      , dimension(:,:),   pointer           :: rhoi             => NULL()
        !Standart deviation (H0), dim:ncar
        real(kind=dp)      , dimension(:)  ,   pointer           :: sig              => NULL()
        !Mean (H0), dim:ncar
        real(kind=dp)      , dimension(:)  ,   pointer           :: xmu              => NULL()

    end type DATASET_TYPE
!!***

!!****t* m_qtlmap_incidence/INCIDENCE_TYPE
!!  NAME
!!    INCIDENCE_TYPE
!!
!!  DESCRIPTION
!!   description of an effect enumered in the INCIDENCE_TYPE
!!  SOURCE
    type INCIDENCE_TYPE
        !Index trait
        integer                                                :: ic       = 0
        !The associated dataset
        type(DATASET_TYPE)                      ,pointer       :: dataset  => NULL()
        !number of qtl to estim
        integer                                                :: nqtl     = 0
        ! Number of level
        integer                                                :: ntniv    = 0
        ! Number of effect
        integer                                                :: nteff    = 0
        ! tab of effect to print in the eqtl format
        logical            ,dimension(20)                      :: eqtl_print    = .true.
        integer                                                :: ntnivmax=0
        !Maximum of effect defined in the model : General mean, polygenic sire,polygenic dam, qtl sire,....
        integer                                                :: nteffmax=0
        ! Number of level which can be estimate
        integer                                                :: nbnivest = 0
        ! Number of qtl*interaction level by sire or dam
        integer                                                :: ntlev    = 0
        ! Number of level for fixed effect
        integer                                                :: nbniv    = 0
        !description of each effect
        type(DESC_EFFECT_TYPE),dimension(:),pointer            :: desc     => NULL()
        !estimability of each level ntniv
        logical        , dimension(:) , pointer                :: vecsol   => NULL()
        !precision of each level estim
        real (kind=dp)       ,dimension(:),pointer             :: precis   => NULL()
        !dim:nqtl : referenced qtl sire ntniv in the contingence matrix
        integer              ,dimension(:),pointer             :: ntniv_qtlsires => NULL()
        !dim:nqtl : referenced qtl dams ntniv in the contingence matrix
        integer              ,dimension(:),pointer             :: ntniv_qtldams  => NULL()
        !dim:ntnivmax correspondance si estimable de la colonne dans la matrice xinc reduite
        integer              ,dimension(:),pointer             :: corr_niv_nivb  => NULL()
        !optim struc
        real (kind=dp) , dimension(:) ,pointer                 :: borni            => null()
        real (kind=dp) , dimension(:) ,pointer                 :: borns            => null()! les bornes de l optimisation
        real (kind=dp) , dimension(:) ,pointer                 :: par              => null()
        real(kind=dp)                                          :: fmax             = 0.d0

        !optimisation qui fait office de corniv dans le modlin originale...
        ! on stocke pour chque kd, les indices des elements des elements non nul....
        integer        , dimension(:,:) ,pointer               :: nonull           => null()
        integer        , dimension(:)   ,pointer               :: nbnonull         => null()! nombre d element non nul

    end type INCIDENCE_TYPE
!!***

!!****t* m_qtlmap_incidence/INCIDENCE_GEN_STRUCT
!!  NAME
!!    INCIDENCE_GEN_STRUCT
!!
!!  DESCRIPTION
!!    structure pour developper des methodes generiques
!!      ce type est un agrega des structures (non commune au mode lineaire et optim) necessaires pour la solution de la vraissemblance a un point n
!!  SOURCE
    type INCIDENCE_GEN_STRUCT
       !!!!!  initializing by the caller !!!!!
       !**************************************
       integer                                      :: type_model       = -1 ! linear homo,hetero,optim,...
       integer                                      :: type_incidence   = -1 ! classic, traits as co, cox,ld
       logical                                      :: performConfusion = .false.
       logical                                      :: performTestNuis  = .false.
       logical                                      :: linear           = .false.
       logical                                      :: hdam             = .false.
       integer                                      :: nqtl

       integer                                      :: eff_general_mean = -1
       !id nteff inside xinc
       integer          ,dimension(:), pointer      :: listnteff        => null()
       !id nteff inside xinc
       integer          ,dimension(:), pointer      :: listnteffhap     => null()
       !linear information
       !------------------
       !0-->maxiqtl,np
       real (kind=dp) ,dimension(:,:,:) ,pointer      :: sigsquare        => null()
       !maxqtl,ncar,ncar
       real (kind=dp)   ,dimension(:,:,:) ,pointer    :: rhoi  => null()

       !F0,F1,etc... dim:nqtl
       real (kind=dp)   ,dimension(:) ,pointer        :: fnqtl            => null()
       !ADDITIONAL OUTPUTS INFORMATIONS
       !-------------------------------
       ! alert confusion (fill if performConfusion is true )
       type(CORR_ALERT_TYPE) ,dimension(:),pointer  :: alertQtl         => null()
       ! size of alertQtl
       integer                                      :: nalert           = 0
       ! Maximum correlation finding between qtl and other effect
       real (kind=dp)                               :: corrmax

       ! list of nuisance effect and test (fill if performTestNuis is true)
       type(TEST_NUISANCES_TYPE) ,dimension(:),pointer  :: listtestnuis => null()
       ! size of listtestnuis
       integer                                          :: ntest        = 0

    end type INCIDENCE_GEN_STRUCT
!!***

!!****t* m_qtlmap_incidence/POSITION_LRT_INCIDENCE
!!  NAME
!!    POSITION_LRT_INCIDENCE
!!
!!  DESCRIPTION
!!    structure pour developper des methodes generiques
!!      ce type est un agrega des structures (non commune au mode lineaire et optim) necessaires pour la solution de la vraissemblance a un point n
!!  SOURCE
    type POSITION_LRT_INCIDENCE
      !Information bind to the current position
      !----------------------------------------
      !nqtl
      integer        ,dimension(:) ,pointer        :: listChr      => null()
      integer        ,dimension(:) ,pointer        :: listN        => null()
      !output info fill by the model
      real(kind=dp) ,dimension(:)  ,pointer        :: lrt          => null()

    end type POSITION_LRT_INCIDENCE
!!***

!!****d* m_qtlmap_incidence/MAX_KD_PRINT
!!  NAME
!!   MAX_KD_PRINT
!!  DESCRIPTION
!!   Constant : maximum of progeny to print for the debug mode
!!  NOTES
!!   debug_write_incidence
!!***
    !Constante : maximum of progeny to print
    integer , parameter  , private                            :: MAX_KD_PRINT=20

    save

!!****v* m_qtlmap_incidence/likelyprob
!!  NAME
!!   likelyprob
!!  DESCRIPTION
!!   give the index the likely genotype dam
!!  DIMENSION
!!   * nchr
!!   * np
!!   * nm
!!  NOTES
!!   init_analyse_linear
!!***
    integer          ,dimension(:,:,:),allocatable,private   :: likelyprob

    public :: init_analyse_linear
    public :: end_analyse_linear

    public :: init_incidence
    public :: end_incidence

    public :: init_dataset_1car
    public :: end_dataset

    public :: init_position
    public :: end_position

    public :: fill_nonull_elements
    public :: matmul_incidence

    !routines to build contingence matrix
    public :: add_general_mean
    public :: add_polygenic_effect
    public :: add_qtleffect
    public :: add_qtlinteraction
    public :: add_effcov

    !changing parameter at the position
    public :: change_qtleffect
    public :: change_interaction_effect

    public :: copy_incidence_desc
    public :: release_copy_incidence_desc



    contains

!!****f* m_qtlmap_incidence/init_analyse_linear
!!  NAME
!!    init_analyse_linear
!!  DESCRIPTION
!!    initialize the linear mode
!!    * likelyprob is building from probg array
!!  NOTES
!!  SOURCE
    subroutine init_analyse_linear()
         integer :: chr,ip,jm,ngeno1,ngeno2,igprob,ig,ic

         allocate (likelyprob(nchr,np,nm))

         likelyprob = 0
         do chr=1,nchr
           do ip=1,np
             do jm=nmp(ip)+1,nmp(ip+1)
               ngeno1=ngenom(chr,jm)+1
               ngeno2=ngenom(chr,jm+1)
               igprob=ngeno1

               do ig=ngeno1+1,ngeno2
                 if (probg(chr,igprob)<probg(chr,ig))then
                   igprob = ig
                 end if
               end do
               likelyprob(chr,ip,jm)=igprob
               if ( probg(chr,igprob) <1.d0 ) then
                call log_mess("*************************************",WARNING_DEF)
                call log_mess("CAREFULLY : Linear model use the most likely genotype",WARNING_DEF)
                call log_mess("Chromosome: "//trim(ch(chr)),WARNING_DEF)
                call log_mess("Sire Family ["//trim(pere(ip))//"]",WARNING_DEF)
                call log_mess("Genotype prob for dam ["//trim(mere(jm))//"] :"//str(probg(chr,igprob)),WARNING_DEF)
                call log_mess("*************************************",WARNING_DEF)
               end if
           end do !jm
       end do ! ip
      end do ! chr
     end subroutine init_analyse_linear
!!***


!!****f* m_qtlmap_incidence/end_analyse_linear
!!  NAME
!!    end_analyse_linear
!!  DESCRIPTION
!!    close the linear mode
!!  NOTES
!!  SOURCE
     subroutine end_analyse_linear
         deallocate (likelyprob)
     end subroutine end_analyse_linear
!!***


!!****f* m_qtlmap_incidence/init_incidence
!!  NAME
!!    init_incidence
!!  DESCRIPTION
!!    initilise the incidence structure (INCIDENCE_TYPE)
!!  INPUTS
!!    ic             : index of the trait
!!    nqtl           : number of hypothesis to test
!!    type_incidence : type of contingence matrix construction (INCIDENCE_TYPE_CLASSIC, INCIDENCE_TYPE_LD, INCIDENCE_TYPE_LDLA, INCIDENCE_TRAIT_COV)
!!    dataset        : use dataset (type DATASET_TYPE)
!!  OUTPUTS
!!    incidenceDesc  : the description of the contingence matrix
!!  NOTES
!!    see copy_dataset,init_dataset_1car
!!    if dataset are not created init_dataset_1car is called otherwise copy_dataset
!!  SOURCE
   subroutine init_incidence(ic,nqtl,incidenceDesc,type_incidence,dataset)
      integer                             , intent(in)        :: ic,type_incidence
      integer                             , intent(in)        :: nqtl
      type(INCIDENCE_TYPE)                , intent(inout)     :: incidenceDesc
      type(DATASET_TYPE)                , intent(in),optional :: dataset
      integer :: npar
      incidenceDesc%nqtl=nqtl
      call set_ntnivmax(ic,nqtl,incidenceDesc%ntnivmax,incidenceDesc%nteffmax,incidenceDesc%ntlev,incidenceDesc%nbniv)

      if ( type_incidence==INCIDENCE_INTERACTION ) then
         !! pour l instant que 2 qtl...
         incidenceDesc%ntnivmax=incidenceDesc%ntnivmax+np*4+count(estime(ic,:))*16
         incidenceDesc%nteffmax=incidenceDesc%nteffmax+1
      end if

      if ( type_incidence==INCIDENCE_TRAIT_COV ) then
         incidenceDesc%ntnivmax=incidenceDesc%ntnivmax+ncar
         incidenceDesc%nteffmax=incidenceDesc%nteffmax+1
      end if

      incidenceDesc%ic       = ic
      incidenceDesc%ntniv    = 0
      incidenceDesc%nbnivest = 0
      incidenceDesc%nteff    = 0

      call log_mess("********************* INIT *****************************",DEBUG_DEF)
      call log_mess("NTNIVMAX:"//str(incidenceDesc%ntnivmax),DEBUG_DEF)
      call log_mess("NTEFFMAX:"//str(incidenceDesc%nteffmax),DEBUG_DEF)
      call log_mess("NTLEVQTL / Reproductor:"//str(incidenceDesc%ntlev),DEBUG_DEF)
      call log_mess("NBLEV FIXED EFFECT:"//str(incidenceDesc%nbniv),DEBUG_DEF)
      call log_mess("*********************************************************",DEBUG_DEF)

      allocate(incidenceDesc%desc(incidenceDesc%nteffmax))
      allocate(incidenceDesc%vecsol(incidenceDesc%ntnivmax))
      incidenceDesc%vecsol=.false.
    !  allocate(incidenceDesc%corniv(ntnivmax))
      allocate(incidenceDesc%precis(incidenceDesc%ntnivmax))
      incidenceDesc%precis=0.d0
      if ( nqtl > 0) then
       allocate (incidenceDesc%ntniv_qtlsires(nqtl))
       allocate (incidenceDesc%ntniv_qtldams(nqtl))
       incidenceDesc%ntniv_qtlsires=0
       incidenceDesc%ntniv_qtldams=0
      end if
      allocate (incidenceDesc%corr_niv_nivb(incidenceDesc%ntnivmax))

      allocate (incidenceDesc%dataset)
      if ( present(dataset) ) then
        call copy_dataset(dataset,incidenceDesc%dataset)
      else
        call init_dataset_1car(ic,nqtl,incidenceDesc%dataset)
      end if

      npar = np+incidenceDesc%ntnivmax

      if ( nmod(ic) > 1 ) npar = npar + nmod(ic)-1

      allocate (incidenceDesc%borni(npar))
      allocate (incidenceDesc%borns(npar))
      allocate (incidenceDesc%par(npar))

      incidenceDesc%borni(1:np)=SIG_MIN
      incidenceDesc%borns(1:np)=SIG_MAX
      incidenceDesc%borni(np+1:)=DEFAULT_PARAM_MIN
      incidenceDesc%borns(np+1:)=DEFAULT_PARAM_MAX

      if (nmod(ic)>1) then
          incidenceDesc%borni(np+incidenceDesc%ntnivmax+1:)=SIG_MIN
          incidenceDesc%borns(np+incidenceDesc%ntnivmax+1:)=DEFAULT_PARAM_MAX
      end if

      incidenceDesc%par=0.d0

      allocate (incidenceDesc%nonull(incidenceDesc%dataset%nkd,incidenceDesc%ntnivmax))
      allocate (incidenceDesc%nbnonull(incidenceDesc%dataset%nkd))
    end subroutine init_incidence
!!***

!!****f* m_qtlmap_incidence/end_incidence
!!  NAME
!!    end_incidence
!!  DESCRIPTION
!!    release the incidence structure (INCIDENCE_TYPE)
!!  INPUTS
!!    incidenceDesc  : the description of the contingence matrix
!!  NOTES
!!  SOURCE
    subroutine end_incidence(incidenceDesc)
      type(INCIDENCE_TYPE)                     , intent(inout) :: incidenceDesc
      integer :: i,ip,jm,ifem

      call log_mess("end_incidence for incidence...",DEBUG_DEF)
      !release structure recursive of description of each effect
      do i=1,incidenceDesc%nteff
        if ( incidenceDesc%desc(i)%haveSubDesc) then
          deallocate (incidenceDesc%desc(i)%listSubDesc)
        end if
      end do
      !release all alocated array
      deallocate(incidenceDesc%desc)
      deallocate(incidenceDesc%vecsol)
      deallocate(incidenceDesc%precis)
      deallocate (incidenceDesc%corr_niv_nivb)

      if ( incidenceDesc%nqtl > 0) then
       deallocate (incidenceDesc%ntniv_qtlsires)
       deallocate (incidenceDesc%ntniv_qtldams)
      end if

     if (associated(incidenceDesc%dataset)) then
      call end_dataset(incidenceDesc%dataset)
      deallocate (incidenceDesc%dataset)
     end if

     if (associated(incidenceDesc%par) ) then
      deallocate (incidenceDesc%borni)
      deallocate (incidenceDesc%borns)
      deallocate (incidenceDesc%par)
     end if

     if ( associated (incidenceDesc%nonull) ) deallocate(incidenceDesc%nonull)
     if ( associated (incidenceDesc%nbnonull) ) deallocate(incidenceDesc%nbnonull)

    end subroutine end_incidence
!!***

!!****f* m_qtlmap_incidence/init_workstruct
!!  NAME
!!    init_workstruct
!!  DESCRIPTION
!!    initialize the workstruct structure (INCIDENCE_GEN_STRUCT)
!!    The following information is given for each position on the chromosome :
!!    * variance family
!!    * residual correlation matrix (multi trait analysis case)
!!    * value of the likelihood
!!  INPUTS
!!   type_incidence  : type of contingence matrix construction (INCIDENCE_TYPE_CLASSIC, INCIDENCE_TYPE_LD, INCIDENCE_TYPE_LDLA, INCIDENCE_TRAIT_COV)
!!   type_resolution : type of resolution (LINEAR_HOMOSCEDASTIC, LINEAR_HETEROSCEDASTIC, OPTIM_HETEROSCEDASTIC )
!!   nqtl            : number of qtl (hypothesis)
!!   pc              : need to compute the confusion step
!!   pt              : need to compute the precision step
!!   multi           : boolean for multi trait analysis
!!  OUTPUTS
!!    workstruct  : buffer structure to save information about result among the chromosome
!!  NOTES
!!  SOURCE
   subroutine init_workstruct(workstruct,type_incidence,type_resolution,nqtl,pc,pt,multi)
          type(INCIDENCE_GEN_STRUCT)              ,intent(inout)    :: workstruct
          integer                                 ,intent(in)       :: type_incidence,type_resolution,nqtl
          logical                                 ,intent(in)       :: pc,pt,multi

          workstruct%type_model=type_resolution
          workstruct%type_incidence=type_incidence

          workstruct%linear=.false.
          if (workstruct%type_model==LINEAR_HOMOSCEDASTIC .or. workstruct%type_model==LINEAR_HETEROSCEDASTIC) then
            workstruct%linear=.true.
          end if
          workstruct%nqtl=nqtl
          workstruct%performConfusion=pc
          workstruct%performTestNuis=pt

          if (multi) then
           allocate (workstruct%sigsquare(nqtl,np,ncar))
           allocate (workstruct%rhoi(nqtl,ncar,ncar))
          else
           allocate (workstruct%sigsquare(nqtl,np,1))
          end if

          allocate (workstruct%listnteff(nqtl))
          if (workstruct%type_incidence==INCIDENCE_TYPE_LD .or. workstruct%type_incidence==INCIDENCE_TYPE_LDLA) then
             allocate (workstruct%listnteffhap(nqtl))
          end if
          allocate (workstruct%fnqtl(nqtl))

      end subroutine init_workstruct
!!***

!!****f* m_qtlmap_incidence/release_ws
!!  NAME
!!    release_ws
!!  DESCRIPTION
!!    release the workstruct structure (INCIDENCE_GEN_STRUCT)
!!  INPUTS
!!    workstruct  : buffer structure to save information about result among the chromosome
!!  NOTES
!!  SOURCE
     subroutine release_ws(workstruct)
        type(INCIDENCE_GEN_STRUCT)              ,intent(inout)  :: workstruct

         deallocate(workstruct%sigsquare)
         deallocate (workstruct%listnteff)
         deallocate (workstruct%fnqtl)
         if (associated(workstruct%rhoi)) deallocate(workstruct%rhoi)
         if (associated(workstruct%alertQtl)) deallocate(workstruct%alertQtl)
         if (associated(workstruct%listtestnuis)) deallocate(workstruct%listtestnuis)
         if (associated(workstruct%listnteffhap)) deallocate(workstruct%listnteffhap)

     end subroutine release_ws
!!***


  !constition des famille de plein et demi frere via des index
  !pas de notion d estimable...on comptabilise seulement les effectifs et
  ! on repere les lignes de la matrice d incidence
!!****f* m_qtlmap_incidence/init_dataset_1car
!!  NAME
!!    init_dataset_1car
!!  DESCRIPTION
!!    * build information family without unkwnown trait (trait and data censured/ discrete trait)
!!      (algorithm dot not use presentc and estime arrays)
!!    build an subtype array : lSire of type SireFamily_type
!!      * variance and mean for each family (sig0, xmu0)
!!      * allocation of probabilities of transmission arrays (by sire family and full family)
!!  INPUTS
!!    ic      : index of the trait
!!    nqtl    : number of qtl (hypothesis)
!!  OUTPUTS
!!    dataset : the user datatset (DATASET_TYPE)
!!
!!  NOTES
!!  SOURCE
  subroutine init_dataset_1car(ic,nqtl,dataset)
    type(DATASET_TYPE)                      , intent(inout) :: dataset
    integer                               , intent(in)      :: ic,nqtl
    logical ,dimension(nd) :: pres
    integer :: ip,jm,kd,ifem,kdd,s1,s2,chr,iq,kkd,effp,somyp
    logical :: isContinue


    call log_mess("build_family for incidence...",DEBUG_DEF)

    allocate(dataset%lSires(np))
    kdd=0

    do ip=1,np
      allocate(dataset%lSires(ip)%full_sib(nmp(ip+1)-nmp(ip)))
      ifem=0
      do jm=nmp(ip)+1,nmp(ip+1)
        ifem=ifem+1
        kd=ndm(jm)+1
        isContinue = presentc(ic,kd)
        do while ( kd<=ndm(jm+1) .and. .not. isContinue )
          kd=kd+1
          if ( kd <= nd ) then
           isContinue = presentc(ic,kd)
          end if
        end do

        if (kd>ndm(jm+1)) cycle
        kdd=kdd+1
        ! on a trouve le premier kd valide du pere ip dans la matrice d incidence
        dataset%lSires(ip)%full_sib(ifem)%firstKd=kdd
       ! print *,trim(pere(ip)),"-",trim(mere(jm)),ndm(jm)+1,ndm(jm+1),count(presentc(ic,kd+1:ndm(jm+1)))
       ! print *,kd+1,ndm(jm+1)

        kdd=kdd + count(presentc(ic,kd+1:ndm(jm+1)))
        dataset%lSires(ip)%full_sib(ifem)%lastKd=kdd
        call log_mess("Full Sib ["//trim(pere(ip))//"-"//trim(mere(jm))//"] :"//&
        trim(str(dataset%lSires(ip)%full_sib(ifem)%firstKd))//"->"&
        //trim(str(dataset%lSires(ip)%full_sib(ifem)%lastKd)),VERBOSE_DEF)
        s1=0
        do chr=1,nchr
          s1=max(s1,ngenom(chr,jm+1)-ngenom(chr,jm))
        end do
        s2=dataset%lSires(ip)%full_sib(ifem)%lastKd-dataset%lSires(ip)%full_sib(ifem)%firstKd+1
        if ( nqtl >= 1) then
          allocate(dataset%lSires(ip)%full_sib(ifem)%ppt(nqtl,s1,s2))
          allocate(dataset%lSires(ip)%full_sib(ifem)%pmt(nqtl,s1,s2))
        end if
      end do
      ifem=0
      do jm=nmp(ip)+1,nmp(ip+1)
        ifem=ifem+1
        if (dataset%lSires(ip)%full_sib(ifem)%firstKd<0) cycle
        dataset%lSires(ip)%half_sib%firstKd= dataset%lSires(ip)%full_sib(ifem)%firstKd
        exit
      end do
      ifem=nmp(ip+1)-nmp(ip)+1
      do jm=nmp(ip+1),nmp(ip)+1,-1
        ifem=ifem-1
        if (dataset%lSires(ip)%full_sib(ifem)%lastKd<0) cycle
        dataset%lSires(ip)%half_sib%lastKd=dataset%lSires(ip)%full_sib(ifem)%lastKd
        exit
      end do

      call log_mess("Half Sib ["//trim(pere(ip))//"] :"//&
        trim(str(dataset%lSires(ip)%half_sib%firstKd))//"->"//trim(str(dataset%lSires(ip)%half_sib%lastKd)),VERBOSE_DEF)
      !s1=lSires(ip)%half_sib%lastKd-lSires(ip)%half_sib%firstKd+1
      if ( nqtl >= 1) then
        allocate(dataset%lSires(ip)%ppt(nqtl,dataset%lSires(ip)%half_sib%firstKd:dataset%lSires(ip)%half_sib%lastKd))
      end if
    end do ! ip

    isContinue = natureY(ic) /= 'i'
    !pour ne pas a tester la presence des caracteres on garde ce qui est seulement present
    if ( isContinue ) then
        allocate (dataset%Y(1,count(presentc(ic,:))))
        allocate (dataset%CD(1,count(presentc(ic,:))))
      else
        allocate (dataset%YDISCRETORD(1,count(presentc(ic,:))))
    end if

    dataset%nkd=0
    dataset%nkd_max_by_fam=0

    do ip=1,np
        do jm=nmp(ip)+1,nmp(ip+1)
         dataset%nkd_max_by_fam=max(dataset%nkd_max_by_fam,count(presentc(ic,ndm(jm)+1:ndm(jm+1))))
         do kd=ndm(jm)+1,ndm(jm+1)
          if (presentc(ic,kd)) then
            dataset%nkd=dataset%nkd+1
            if ( isContinue ) then
              dataset%Y(1,dataset%nkd)=y(ic,kd)
              dataset%CD(1,dataset%nkd)=cd(ic,kd)
            else
              dataset%YDISCRETORD(1,dataset%nkd)=ydiscretord(ic,kd)
            end if
          end if
         end do !kd
        end do ! jm
    end do ! ip

    do ip=1,np
        allocate(dataset%lSires(ip)%sig0(1))
        allocate(dataset%lSires(ip)%xmu0(1))
        effp=0
        somyp=0
        do jm=nmp(ip)+1,nmp(ip+1)
          do kd=ndm(jm)+1,ndm(jm+1)
            if (presentc(ic,kd)) then
               effp=effp+1
               somyp=somyp+y(ic,kd)*cd(ic,kd)
            end if
          end do
        end do

        if (effp == 0.d0) then
           call stop_application('Father ['//trim(pere(ip))//'] has got no child with trait value ['//trim(carac(ic))//"]")
        end if
        !moyenne du caractere dans la famille du pere ip
        dataset%lSires(ip)%xmu0(1)=somyp/dble(effp)

        !ecart type du caractere dans la famille du pere ip
        dataset%lSires(ip)%sig0(1)=0.d0
        do jm=nmp(ip)+1,nmp(ip+1)
          do kd=ndm(jm)+1,ndm(jm+1)
            if (presentc(ic,kd)) then
               dataset%lSires(ip)%sig0(1)=dataset%lSires(ip)%sig0(1)+(y(ic,kd)-&
                dataset%lSires(ip)%xmu0(1))*(y(ic,kd)-dataset%lSires(ip)%xmu0(1))
            end if
          end do !kd
        end do !jm
        dataset%lSires(ip)%sig0(1)=dsqrt(dataset%lSires(ip)%sig0(1)/dble(effp-1))
      end do !ip
  end subroutine init_dataset_1car
!!***

!!****f* m_qtlmap_incidence/end_dataset
!!  NAME
!!    end_dataset
!!  DESCRIPTION
!!    - release the variable dataset of type DATASET_TYPE
!!  OUTPUTS
!!    dataset : the user datatset (DATASET_TYPE)
!!
!!  NOTES
!!  SOURCE
   subroutine end_dataset(dataset)
      type(DATASET_TYPE)                      , intent(inout) :: dataset
      integer :: ip,jm,ifem

     do ip=1,np
      if ( associated(dataset%lSires(ip)%ppt) ) deallocate (dataset%lSires(ip)%ppt)
      if ( associated(dataset%lSires(ip)%sig0) ) deallocate (dataset%lSires(ip)%sig0)
      if ( associated(dataset%lSires(ip)%xmu0) ) deallocate (dataset%lSires(ip)%xmu0)
      ifem=0
      do jm=nmp(ip)+1,nmp(ip+1)
       ifem=ifem+1
       if (dataset%lSires(ip)%full_sib(ifem)%firstkd<0) cycle
       if ( associated(dataset%lSires(ip)%full_sib(ifem)%ppt) ) &
          deallocate(dataset%lSires(ip)%full_sib(ifem)%ppt)
       if ( associated(dataset%lSires(ip)%full_sib(ifem)%pmt) ) &
          deallocate(dataset%lSires(ip)%full_sib(ifem)%pmt)
      end do
      deallocate(dataset%lSires(ip)%full_sib)
    end do
    deallocate (dataset%lSires)

    if (associated (dataset%Y)) then
      deallocate(dataset%Y)
      deallocate(dataset%CD)
    else
      deallocate(dataset%YDISCRETORD)
    end if

    if (associated (dataset%rhoi) ) deallocate(dataset%rhoi)
    if (associated (dataset%cov) ) deallocate(dataset%cov)
    if (associated (dataset%sig) ) deallocate(dataset%sig)
    if (associated (dataset%xmu) ) deallocate(dataset%xmu)


   end subroutine end_dataset
!!***

!!****f* m_qtlmap_incidence/copy_dataset
!!  NAME
!!    copy_dataset
!!  DESCRIPTION
!!    Allocate new arrays and copy a variable of type DATASET_TYPE.
!!  INPUTS
!!    dataset : the original variable
!!  OUTPUTS
!!    out_dataset : the copy of dataset
!!
!!  NOTES
!!  SOURCE
    subroutine copy_dataset(dataset,out_dataset)
      type(DATASET_TYPE)                      , intent(in) :: dataset
      type(DATASET_TYPE)                      , intent(out) :: out_dataset
      integer :: ip,jm,ifem,s1,s2,s3

     out_dataset%nkd = dataset%nkd
     out_dataset%nkd_max_by_fam = dataset%nkd_max_by_fam

     allocate (out_dataset%lSires(np))

     do ip=1,np
      out_dataset%lSires(ip)%half_sib%firstKd = dataset%lSires(ip)%half_sib%firstKd
      out_dataset%lSires(ip)%half_sib%lastKd = dataset%lSires(ip)%half_sib%lastKd

      if ( associated(dataset%lSires(ip)%ppt) ) then
        s1=size(dataset%lSires(ip)%ppt,1)
        s2=size(dataset%lSires(ip)%ppt,2)
        allocate (out_dataset%lSires(ip)%ppt(s1,s2))
        out_dataset%lSires(ip)%ppt = dataset%lSires(ip)%ppt
      end if

      if ( associated(dataset%lSires(ip)%full_sib) ) then

        nfem = size(dataset%lSires(ip)%full_sib)
        allocate ( out_dataset%lSires(ip)%full_sib(nfem) )
        do ifem=1,nfem
           out_dataset%lSires(ip)%full_sib(ifem)%firstKd = dataset%lSires(ip)%full_sib(ifem)%firstKd
           out_dataset%lSires(ip)%full_sib(ifem)%lastKd = dataset%lSires(ip)%full_sib(ifem)%lastKd
           if ( associated(dataset%lSires(ip)%full_sib(ifem)%ppt) ) then
             s1=size(dataset%lSires(ip)%full_sib(ifem)%ppt,1)
             s2=size(dataset%lSires(ip)%full_sib(ifem)%ppt,2)
             s3=size(dataset%lSires(ip)%full_sib(ifem)%ppt,3)
             allocate ( out_dataset%lSires(ip)%full_sib(ifem)%ppt(s1,s2,s3) )
             out_dataset%lSires(ip)%full_sib(ifem)%ppt = dataset%lSires(ip)%full_sib(ifem)%ppt
           end if
            if ( associated(dataset%lSires(ip)%full_sib(ifem)%pmt) ) then
             s1=size(dataset%lSires(ip)%full_sib(ifem)%pmt,1)
             s2=size(dataset%lSires(ip)%full_sib(ifem)%pmt,2)
             s3=size(dataset%lSires(ip)%full_sib(ifem)%pmt,3)
             allocate ( out_dataset%lSires(ip)%full_sib(ifem)%pmt(s1,s2,s3) )
             out_dataset%lSires(ip)%full_sib(ifem)%pmt = dataset%lSires(ip)%full_sib(ifem)%pmt
           end if
        end do

      end if

      if ( associated(dataset%lSires(ip)%sig0) ) then
        allocate (out_dataset%lSires(ip)%sig0(size(dataset%lSires(ip)%sig0)))
        out_dataset%lSires(ip)%sig0 = dataset%lSires(ip)%sig0
      end if

       if ( associated(dataset%lSires(ip)%xmu0) ) then
        allocate (out_dataset%lSires(ip)%xmu0(size(dataset%lSires(ip)%xmu0)))
        out_dataset%lSires(ip)%xmu0 = dataset%lSires(ip)%xmu0
      end if

     end do


    if (associated (dataset%Y)) then
      s1=size(dataset%Y,1)
      s2=size(dataset%Y,2)
      allocate(out_dataset%Y(s1,s2))
      allocate(out_dataset%CD(s1,s2))
      out_dataset%Y = dataset%Y
      out_dataset%CD = dataset%CD
    else
      s1=size(dataset%YDISCRETORD,1)
      s2=size(dataset%YDISCRETORD,2)
      allocate(out_dataset%YDISCRETORD(s1,s2))
      out_dataset%YDISCRETORD = dataset%YDISCRETORD
    end if

    if (associated (dataset%rhoi) ) then
      s1=size(dataset%rhoi,1)
      s2=size(dataset%rhoi,2)
      allocate(out_dataset%rhoi(s1,s2))
      out_dataset%rhoi = dataset%rhoi
    end if

    if (associated (dataset%cov) ) then
      s1=size(dataset%cov,1)
      s2=size(dataset%cov,2)
      allocate(out_dataset%cov(s1,s2))
      out_dataset%cov = dataset%cov
    end if

    if (associated (dataset%sig) ) then
      s1=size(dataset%sig,1)
      allocate(out_dataset%sig(s1))
      out_dataset%sig = dataset%sig
    end if

    if (associated (dataset%xmu) ) then
      s1=size(dataset%xmu,1)
      allocate(out_dataset%xmu(s1))
      out_dataset%xmu = dataset%xmu
    end if

   end subroutine copy_dataset
!!***

!!****f* m_qtlmap_incidence/copy_incidence_desc
!!  NAME
!!    copy_incidence_desc
!!  DESCRIPTION
!!    Allocate partial arrays and copy a variable of type DATASET_TYPE.
!!    Static informations are referenced from the original variable in_inc.
!!
!!  INPUTS
!!    in_inc : the original variable
!!  OUTPUTS
!!    out_inc : the copy of incidence description
!!
!!  NOTES
!!  SOURCE
   subroutine copy_incidence_desc (in_inc,out_inc)
       type(INCIDENCE_TYPE)  , intent(in) :: in_inc
       type(INCIDENCE_TYPE)  , intent(out) :: out_inc
       integer :: ip,ifem,jm,s1,s2,n,i

       out_inc%ic  = in_inc%ic
       out_inc%dataset  => in_inc%dataset ! attention on ne recopie pas le jeu de donnees qui est considerer comme statique et non modifiable
       out_inc%nqtl     = in_inc%nqtl
       out_inc%ntniv    = in_inc%ntniv
       out_inc%nteff    = in_inc%nteff
       out_inc%eqtl_print = in_inc%eqtl_print
       out_inc%nbnivest = in_inc%nbnivest
       out_inc%ntlev    = in_inc%ntlev
       out_inc%nbniv    = in_inc%nbniv

       if ( associated (in_inc%desc) ) then
         allocate (out_inc%desc(size(in_inc%desc)))
         do i=1,size(in_inc%desc)
          call copy(in_inc%desc(i),out_inc%desc(i))
         end do
       else
         out_inc%desc => null()
       end if

       allocate(out_inc%vecsol(size(in_inc%vecsol)))
       allocate(out_inc%precis(size(in_inc%precis)))
       if ( out_inc%nqtl > 0) then
        allocate (out_inc%ntniv_qtlsires(out_inc%nqtl))
        allocate (out_inc%ntniv_qtldams(out_inc%nqtl))
        out_inc%ntniv_qtlsires=0
        out_inc%ntniv_qtldams=0
       end if
       allocate (out_inc%corr_niv_nivb(size(in_inc%corr_niv_nivb)))

       out_inc%vecsol   = in_inc%vecsol
       out_inc%precis   = in_inc%precis
       out_inc%ntniv_qtlsires = in_inc%ntniv_qtlsires
       out_inc%ntniv_qtldams  = in_inc%ntniv_qtldams
       out_inc%corr_niv_nivb  = in_inc%corr_niv_nivb

       allocate (out_inc%dataset)
       !call init_dataset_1car(out_inc%ic,out_inc%nqtl,out_inc%dataset)
       allocate(out_inc%dataset%lSires(np))
       do ip=1,np
         allocate(out_inc%dataset%lSires(ip)%full_sib(nmp(ip+1)-nmp(ip)))
         ifem=0
         do jm=nmp(ip)+1,nmp(ip+1)
            ifem=ifem+1
            out_inc%dataset%lSires(ip)%full_sib(ifem)%firstKd=in_inc%dataset%lSires(ip)%full_sib(ifem)%firstKd
            out_inc%dataset%lSires(ip)%full_sib(ifem)%lastKd=in_inc%dataset%lSires(ip)%full_sib(ifem)%lastKd

            if ( associated(in_inc%dataset%lSires(ip)%full_sib(ifem)%ppt) ) then
             n=size(in_inc%dataset%lSires(ip)%full_sib(ifem)%ppt,1)
             s1=size(in_inc%dataset%lSires(ip)%full_sib(ifem)%ppt,2)
             s2=size(in_inc%dataset%lSires(ip)%full_sib(ifem)%ppt,3)
             allocate(out_inc%dataset%lSires(ip)%full_sib(ifem)%ppt(n,s1,s2))
             allocate(out_inc%dataset%lSires(ip)%full_sib(ifem)%pmt(n,s1,s2))
             out_inc%dataset%lSires(ip)%full_sib(ifem)%ppt=in_inc%dataset%lSires(ip)%full_sib(ifem)%ppt
             out_inc%dataset%lSires(ip)%full_sib(ifem)%pmt=in_inc%dataset%lSires(ip)%full_sib(ifem)%pmt
            end if

         end do

         out_inc%dataset%lSires(ip)%half_sib%firstKd=in_inc%dataset%lSires(ip)%half_sib%firstKd
         out_inc%dataset%lSires(ip)%half_sib%lastKd=in_inc%dataset%lSires(ip)%half_sib%lastKd
         if ( associated(in_inc%dataset%lSires(ip)%ppt) ) then
          n=size(in_inc%dataset%lSires(ip)%ppt,1)
          s1=in_inc%dataset%lSires(ip)%half_sib%firstKd
          s2=in_inc%dataset%lSires(ip)%half_sib%lastKd
          allocate(out_inc%dataset%lSires(ip)%ppt(n,s1:s2))
          out_inc%dataset%lSires(ip)%ppt=in_inc%dataset%lSires(ip)%ppt
         end if

         out_inc%dataset%lSires(ip)%sig0=>in_inc%dataset%lSires(ip)%sig0
         out_inc%dataset%lSires(ip)%xmu0=>in_inc%dataset%lSires(ip)%xmu0

       end do

       out_inc%dataset%nkd = in_inc%dataset%nkd
       out_inc%dataset%nkd_max_by_fam = in_inc%dataset%nkd_max_by_fam
       out_inc%ntnivmax =in_inc%ntnivmax
       out_inc%nteffmax =in_inc%nteffmax

       out_inc%dataset%YDISCRETORD => in_inc%dataset%YDISCRETORD
       out_inc%dataset%Y           => in_inc%dataset%Y
       out_inc%dataset%CD          => in_inc%dataset%CD
       out_inc%dataset%cov         => in_inc%dataset%Y
       out_inc%dataset%rhoi        => in_inc%dataset%CD

       ! --- Fin copy dataset

       if (associated(in_inc%nonull) ) then
        allocate (out_inc%nonull(size(in_inc%nonull,1),size(in_inc%nonull,2)))
        allocate (out_inc%nbnonull(size(in_inc%nbnonull)))
        out_inc%nonull = in_inc%nonull
        out_inc%nbnonull = in_inc%nbnonull
       end if

       !les bornes sont sense ne pas change...
       out_inc%borni=>in_inc%borni
       out_inc%borns=>in_inc%borns

       allocate (out_inc%par(size(in_inc%par)))
       out_inc%par = in_inc%par

   end subroutine copy_incidence_desc
!!***

!!****f* m_qtlmap_incidence/release_copy_incidence_desc
!!  NAME
!!    release_copy_incidence_desc
!!  DESCRIPTION
!!    release only arrays created during a copy (see copy_incidence_desc)
!!
!!  OUTPUTS
!!    copy : the copy of the incidence description
!!
!!  NOTES
!!  SOURCE
   subroutine release_copy_incidence_desc (copy)
     type(INCIDENCE_TYPE)  , intent(inout) :: copy
     integer :: i,ip,jm

     if ( associated (copy%desc) ) then
         do i=1,size(copy%desc)
          call release(copy%desc(i))
         end do
         deallocate (copy%desc)
     end if

     deallocate (copy%vecsol)
     deallocate (copy%precis)

     if ( copy%nqtl > 0 ) then
      deallocate (copy%ntniv_qtlsires)
      deallocate (copy%ntniv_qtldams)
     end if

     deallocate (copy%corr_niv_nivb)
!     call end_dataset(copy%dataset)
     do ip=1,np
       do jm=1,size(copy%dataset%lSires(ip)%full_sib)
         if ( associated(copy%dataset%lSires(ip)%full_sib(jm)%ppt) ) then
           deallocate ( copy%dataset%lSires(ip)%full_sib(jm)%ppt )
           deallocate ( copy%dataset%lSires(ip)%full_sib(jm)%pmt )
         end if
       end do
       deallocate ( copy%dataset%lSires(ip)%full_sib)
       if ( associated(copy%dataset%lSires(ip)%ppt) ) deallocate(copy%dataset%lSires(ip)%ppt )
     end do
     deallocate ( copy%dataset%lSires )
     deallocate ( copy%dataset )

     deallocate (copy%par)
     if (associated(copy%nonull) ) deallocate (copy%nonull)
     if (associated(copy%nbnonull) ) deallocate (copy%nbnonull)

   end subroutine release_copy_incidence_desc
!!***

!!****f* m_qtlmap_incidence/init_position
!!  NAME
!!    init_position
!!  DESCRIPTION
!!    Initialize a position for a multi qtl hypothesis test (POSITION_LRT_INCIDENCE), e.g :
!!     * allocation of a LRT array value (comparaison with Hi-1, Hi-2,...)
!!     * allocation of the chromosome list (genome position of all qtl)
!!     * allocation of a discrete position list (index of absi array for each qtl position)
!!  INPUTS
!!    nqtl        : the number of qtl to test
!!  OUTPUTS
!!    position    : the variable poisiton
!!
!!  NOTES
!!  SOURCE
   subroutine init_position (nqtl,position)
     integer                       , intent(in)    :: nqtl
     type(POSITION_LRT_INCIDENCE)  , intent(inout) :: position

     allocate (position%lrt(nqtl))
     allocate (position%listChr(nqtl))
     allocate (position%listN(nqtl))

   end subroutine init_position
!!***

!!****f* m_qtlmap_incidence/end_position
!!  NAME
!!    end_position
!!  DESCRIPTION
!!    release position of type POSITION_LRT_INCIDENCE
!!  OUTPUTS
!!    position    : the variable poisiton
!!
!!  NOTES
!!  SOURCE
   subroutine end_position (position)
     type(POSITION_LRT_INCIDENCE)  , intent(inout) :: position

     deallocate (position%lrt)
     deallocate (position%listChr)
     deallocate (position%listN)

   end subroutine end_position
!!***

!!****f* m_qtlmap_incidence/add_general_mean
!!  NAME
!!    add_general_mean
!!  DESCRIPTION
!!    Add a general mean effect inside a contingence matrix with a description
!!
!!  OUTPUTS
!!    xinc        : The contingence matrix
!!  incidenceDesc : The description of contingence matrix
!!
!!  NOTES
!!  SOURCE
    subroutine add_general_mean(xinc,incidenceDesc)
       type(INCIDENCE_TYPE)                                  , intent(inout) :: incidenceDesc
       real (kind=dp) , dimension(nd,incidenceDesc%ntnivmax) , intent(inout) :: xinc
       integer :: ntniv
       call log_mess("incidence : ** Add General mean ** ",DEBUG_DEF)
       incidenceDesc%ntniv = incidenceDesc%ntniv+1

       ntniv = incidenceDesc%ntniv
       incidenceDesc%nteff = incidenceDesc%nteff+1
       incidenceDesc%desc(incidenceDesc%nteff)%name="General Mean"
       incidenceDesc%desc(incidenceDesc%nteff)%start=incidenceDesc%nteff
       incidenceDesc%desc(incidenceDesc%nteff)%end=incidenceDesc%nteff
       xinc(:,ntniv) = 1
       !incidenceDesc%nivdir(:,incidenceDesc%nteff)   = 1
    end subroutine add_general_mean
!!***

!!****f* m_qtlmap_incidence/add_polygenic_effect
!!  NAME
!!    add_polygenic_effect
!!  DESCRIPTION
!!    Add a polygenic effect inside a contingence matrix with a description
!!    * An effect is adding for each Sire family
!!    * An effect is adding for each full sib family (depends ndmin variable)
!!
!!  OUTPUTS
!!    xinc        : The contingence matrix
!!  incidenceDesc : The description of contingence matrix
!!
!!  NOTES
!!  SOURCE
    subroutine add_polygenic_effect(xinc,incidenceDesc)
       type(INCIDENCE_TYPE)                    , intent(inout) :: incidenceDesc
       real (kind=dp) , dimension(nd,incidenceDesc%ntnivmax) , intent(inout) :: xinc
       integer :: ip,jm,kd,nt,j,fem,nteff,nkd
       call log_mess("incidence : ** Add Polygenic effect to estimate **",DEBUG_DEF)
       nt=incidenceDesc%ntniv
       nteff = incidenceDesc%nteff

       incidenceDesc%desc(incidenceDesc%nteff+1)%name="Sire polygenic effects"
       incidenceDesc%desc(incidenceDesc%nteff+1)%start=nt+1
       incidenceDesc%desc(incidenceDesc%nteff+1)%end=nt+np
       incidenceDesc%desc(incidenceDesc%nteff+1)%haveSubDesc=.true.
       allocate (incidenceDesc%desc(incidenceDesc%nteff+1)%listSubDesc(np))
       do ip=1,np
         incidenceDesc%desc(incidenceDesc%nteff+1)%listSubDesc(ip)%name="Sire "//trim(pere(ip))
         incidenceDesc%desc(incidenceDesc%nteff+1)%listSubDesc(ip)%start=nt+ip
         incidenceDesc%desc(incidenceDesc%nteff+1)%listSubDesc(ip)%end=nt+ip
       end do
       incidenceDesc%nteff = incidenceDesc%nteff+1
       incidenceDesc%ntniv = incidenceDesc%ntniv+np

       if ( count(estime(incidenceDesc%ic,:))>0) then
         incidenceDesc%desc(incidenceDesc%nteff+1)%name="Dam polygenic effects"
         incidenceDesc%desc(incidenceDesc%nteff+1)%start=incidenceDesc%desc(incidenceDesc%nteff)%end+1
         incidenceDesc%desc(incidenceDesc%nteff+1)%end=nt+np+count(estime(incidenceDesc%ic,:))!nbfemLoc(ic)
         incidenceDesc%desc(incidenceDesc%nteff+1)%haveSubDesc=.true.
         allocate (incidenceDesc%desc(incidenceDesc%nteff+1)%listSubDesc(count(estime(incidenceDesc%ic,:))))
         fem=0
         do ip=1,np
          do jm=nmp(ip)+1,nmp(ip+1)
           if (estime(incidenceDesc%ic,jm) ) then
             fem=fem+1
             incidenceDesc%desc(incidenceDesc%nteff+1)%listSubDesc(fem)%name="Dam "//trim(mere(jm))//" [Sire "//trim(pere(ip))//"]"
             incidenceDesc%desc(incidenceDesc%nteff+1)%listSubDesc(fem)%start=nt+np+fem
             incidenceDesc%desc(incidenceDesc%nteff+1)%listSubDesc(fem)%end=nt+np+fem
           end if
         end do
        end do
         incidenceDesc%nteff = incidenceDesc%nteff+1
         incidenceDesc%ntniv = incidenceDesc%ntniv+count(estime(incidenceDesc%ic,:))!nbfemLoc(ic)
       else

       end if
       j=0

       !initialisation
       xinc(:,nt+1:nt+np+count(estime(incidenceDesc%ic,:nm))) = 0

       nkd=0
       do ip=1,np
         xinc(:,nt+ip) = 0
         do jm=nmp(ip)+1,nmp(ip+1)
           do kd=ndm(jm)+1,ndm(jm+1)
             if ( presentc(incidenceDesc%ic,kd) ) then
              nkd =nkd +1
              xinc(nkd,nt+ip) = 1
         !    incidenceDesc%nivdir(kd,nteff+1)= nt+ip
              if (estime(incidenceDesc%ic,jm)) then
               fem=count(estime(incidenceDesc%ic,:jm)) ! on compte le nombre estimable dans la famille de pere ip
               xinc(nkd,nt+np+fem) = 1
          !    incidenceDesc%nivdir(kd,nteff+2)= nt+np+fem
             end if
             end if
           end do
         end do
       end do
    end subroutine add_polygenic_effect
!!***


!!****f* m_qtlmap_incidence/add_qtleffect
!!  NAME
!!    add_qtleffect
!!  DESCRIPTION
!!    Add a qtl effect inside a contingence matrix with a description
!!    * An effect is adding for each Sire family
!!    * An effect is adding for each dam with enough progenies (ndmin)
!!
!!  INPUTS
!!    nteff       : index of the effect
!!    numqtl      : index of the qtl effect
!!    chr         : chromosome localisation
!!    n           : the position
!!    mint        : index of the interaction to remove (otherwise mint <= 0)
!!    linear      : use the likely probability genotype dam or not
!!  OUTPUTS
!!    xinc        : The contingence matrix
!!  incidenceDesc : The description of contingence matrix
!!
!!  NOTES
!!  SOURCE
    subroutine add_qtleffect(nteff,numqtl,chr,n,xinc,incidenceDesc,mint,linear)
       integer                                 ,intent(in)     :: nteff,numqtl,chr,n
       type(INCIDENCE_TYPE)                    ,intent(inout)  :: incidenceDesc
       real (kind=dp) , dimension(nd,incidenceDesc%ntnivmax) ,intent(inout)  :: xinc
       integer                                 ,intent(in)     :: mint ! index of interaction to remove from incidence
       logical                                 ,intent(in)     :: linear

       integer :: ntniv,nteffstart,ip,fem,jm,l,s,u,nbint,jef
       logical ,dimension(nm) :: femInit

       call log_mess("incidence : ** Add Qtl effect to estimate **",DEBUG_DEF)

       nteffstart = incidenceDesc%nteff
       incidenceDesc%nteff = incidenceDesc%nteff+1
       if ( count(estime(incidenceDesc%ic,:)) > 0 ) incidenceDesc%nteff = incidenceDesc%nteff+1

       nbint=listModelTrait(incidenceDesc%ic)%nbint(numqtl) !modele(incidenceDesc%ic,3)
       incidenceDesc%ntlev=1

       do jef=1,nbint
          if ( mint == jef) cycle ! this interaction is not adding
          incidenceDesc%ntlev=incidenceDesc%ntlev*&
           nlev(listModelTrait(incidenceDesc%ic)%indexFixedEffectWithInteraction(numqtl,jef))
       end do

       incidenceDesc%desc(nteffstart+1)%name="Sire QTL effects ["//trim(str(numqtl))//"]"
       incidenceDesc%desc(nteffstart+1)%start=incidenceDesc%ntniv+1
       incidenceDesc%ntniv_qtlsires(numqtl)=incidenceDesc%desc(nteffstart+1)%start
       incidenceDesc%desc(nteffstart+1)%end=incidenceDesc%ntniv+incidenceDesc%ntlev*np

       incidenceDesc%desc(nteffstart+1)%haveSubDesc=.true.
       allocate (incidenceDesc%desc(nteffstart+1)%listSubDesc(np*incidenceDesc%ntlev))
       u=0
       do ip=1,np
        l=0
        do s=incidenceDesc%ntniv+(ip-1)*incidenceDesc%ntlev+1,incidenceDesc%ntniv+ip*incidenceDesc%ntlev
         l=l+1
         u=u+1
         incidenceDesc%desc(nteffstart+1)%listSubDesc(u)%name="Sire "//trim(pere(ip))//" Level "//trim(str(l))
         incidenceDesc%desc(nteffstart+1)%listSubDesc(u)%start=s
         incidenceDesc%desc(nteffstart+1)%listSubDesc(u)%end=s
        end do
       end do

       if ( count(estime(incidenceDesc%ic,:)) > 0 ) then
         incidenceDesc%desc(nteffstart+2)%name="Dam QTL effects ["//trim(str(numqtl))//"]"
         incidenceDesc%desc(nteffstart+2)%start=incidenceDesc%desc(nteffstart+1)%end+1
         incidenceDesc%ntniv_qtldams(numqtl)=incidenceDesc%desc(nteffstart+2)%start
         femInit=.false.
         l=0
         do jm=1,nm
           if (estime(incidenceDesc%ic,jm) ) then
             fem=iam(incidenceDesc%ic,repfem(jm))
             if (femInit(fem)) then
                cycle
             end if
             femInit(fem)=.true.
             l=l+1
           end if
         end do

         allocate (incidenceDesc%desc(nteffstart+2)%listSubDesc(l*incidenceDesc%ntlev))
         incidenceDesc%desc(nteffstart+2)%end=incidenceDesc%ntniv+incidenceDesc%ntlev*(np+l)
         incidenceDesc%desc(nteffstart+2)%haveSubDesc=.true.
         !fem=0
         femInit=.false.
         u=0
         do jm=1,nm
           if (estime(incidenceDesc%ic,jm) ) then
             fem=iam(incidenceDesc%ic,repfem(jm))
             if (femInit(fem)) then
                cycle
             end if
             femInit(fem)=.true.
             l=0
             do s=incidenceDesc%desc(nteffstart+2)%start+(fem-1)*incidenceDesc%ntlev,&
             incidenceDesc%desc(nteffstart+2)%start-1+fem*incidenceDesc%ntlev
              l=l+1
              u=u+1
              !fem=fem+1
              incidenceDesc%desc(nteffstart+2)%listSubDesc(u)%name="Dam "//trim(mere(jm))//" Level "//trim(str(l))
              incidenceDesc%desc(nteffstart+2)%listSubDesc(u)%start=s
              incidenceDesc%desc(nteffstart+2)%listSubDesc(u)%end=s
             end do
           end if
         end do

         incidenceDesc%ntniv = incidenceDesc%desc(nteffstart+2)%end
       else
         incidenceDesc%ntniv = incidenceDesc%desc(nteffstart+1)%end
       end if

       call change_qtleffect(nteff,numqtl,chr,n,xinc,incidenceDesc,mint,linear)

    end subroutine add_qtleffect
!!***

!!****f* m_qtlmap_incidence/add_haplotype_effect
!!  NAME
!!    add_haplotype_effect
!!  DESCRIPTION
!!    Add a haplotype effect inside a contingence matrix with a description
!!    An effect is adding by haplotype finded in the population at the specific position
!!    The number of haplotype is growing depending hdam optional variable.
!!
!!  INPUTS
!!    nteff       : index of the effect
!!    numhap      : index of the haplotype effect
!!    chr         : chromosome localisation
!!    n           : the position
!!    hdam        :
!!    start       : using in a inialize context
!!  OUTPUTS
!!    xinc        : The contingence matrix
!!  incidenceDesc : The description of contingence matrix
!!
!!  NOTES
!!  SOURCE
    subroutine add_haplotype_effect(nteff,numhap,chr,n,xinc,incidenceDesc,hdam,start)
       integer                                 ,intent(in)     :: nteff,numhap,chr,n
       type(INCIDENCE_TYPE)                    ,intent(inout)  :: incidenceDesc
       real (kind=dp) , dimension(nd,incidenceDesc%ntnivmax) ,intent(inout)  :: xinc
       logical                                 ,intent(in)     :: hdam,start

       !local
       integer :: ip,jm,kd,ntniv,nkd,j_gam,i_haplo,j,i_gam,geno,nteffstart
       real (kind=dp) ,dimension(:,:),allocatable        :: pp_ldla,pm_ldla
       real (kind=dp)  ,dimension(nd,NB_HAPLO_PRIOR)   :: pb_haplo
       real (kind=dp)  :: su

       allocate (pp_ldla(nd,2))
       allocate (pm_ldla(nd,2))

       ntniv = incidenceDesc%desc(nteff-1)%end
       nkd=0

       call set_haplo_for_ldla(chr,absi(chr,n),n,.true.,hdam)

       if (start) then
         nteffstart = incidenceDesc%nteff
         incidenceDesc%nteff = incidenceDesc%nteff+1
         incidenceDesc%ntniv = ntniv + 2**longhap
       else
         nteffstart = nteff - 1
       end if

       incidenceDesc%desc(nteffstart+1)%name="Haplotypes effects["//trim(str(numhap))//"]"
       incidenceDesc%desc(nteffstart+1)%start=ntniv+1
       incidenceDesc%desc(nteffstart+1)%end=ntniv+2**longhap ! nb_haplo_reduit

       incidenceDesc%desc(nteffstart+1)%haveSubDesc=.true.
       if ( associated(incidenceDesc%desc(nteffstart+1)%listSubDesc) ) &
         deallocate (incidenceDesc%desc(nteffstart+1)%listSubDesc)

       allocate (incidenceDesc%desc(nteffstart+1)%listSubDesc(nb_haplo_reduit))

       do j=1,nb_haplo_reduit
         incidenceDesc%desc(nteffstart+1)%listSubDesc(j)%name=trim(name_haplo_reduit(j))//&
           "(freq="//trim(str(pb_haplo_reduit(j)))//")"
         incidenceDesc%desc(nteffstart+1)%listSubDesc(j)%start=incidenceDesc%desc(nteffstart+1)%start+j-1
         incidenceDesc%desc(nteffstart+1)%listSubDesc(j)%end=incidenceDesc%desc(nteffstart+1)%start+j-1
       end do

       xinc(:,ntniv+1:ntniv+2**longhap) = 0.d0

       do ip=1,np
         do jm=nmp(ip)+1,nmp(ip+1)
          !if ( linear ) then
            call getprobld_lin(chr,n,incidenceDesc%ic,ip,jm,ndm(jm)+1,ndm(jm+1),pp_ldla,pm_ldla)
!           else
!            call getglobalprob(chr,n,incidenceDesc%ic,ip,jm,ndm(jm)+1,ndm(jm+1),pp_ldla,pm_ldla)
           !end if
           do kd=ndm(jm)+1,ndm(jm+1)
            if ( presentc(incidenceDesc%ic,kd) ) then
            nkd=nkd+1

            do i_haplo=1,nb_haplo_reduit
             pb_haplo(kd,i_haplo)=0.d0
           end do

! haplotype du pere
           !if(hsire)then
             do j=1,2
               su = dble(sum(pb_haplo_reduit(num_haplo_pere(ip,j,1:nb_gam_pere(ip,j)))))
               if ( su == 0 ) cycle
               do i_gam=1,nb_gam_pere(ip,j)
                 pb_haplo(kd,num_haplo_pere(ip,j,i_gam))= pb_haplo(kd,num_haplo_pere(ip,j,i_gam)) &
                                                       +pp_ldla(kd,j) * ( pb_haplo_reduit(num_haplo_pere(ip,j,i_gam)) / su )
                end do !i_gam
              end do !j
           !end if ! opt_model
! haplotype de la mere si phase connue
           if(hdam)then
            if(estime(incidenceDesc%ic,jm)) then
               geno = likelyprob(chr,ip,jm)
               do j=1,2
                 su = dble(sum(pb_haplo_reduit(num_haplo_mere(geno,j,1:nb_gam_mere(geno,j)))))
                 if ( su == 0 ) cycle
                 do i_gam=1,nb_gam_mere(geno,j)
                   pb_haplo(kd,num_haplo_mere(geno,j,i_gam))  = pb_haplo(kd,num_haplo_mere(geno,j,i_gam)) &
                                                 +pm_ldla(kd,j) * ( pb_haplo_reduit(num_haplo_mere(geno,j,i_gam)) / su )
                 end do !i_gam
               end do !j
! haplotype de la mere si phase inconnue
           else
             do j_gam=1,nb_gam(kd)
               su = dble(sum(pb_haplo_reduit(num_haplo_desc(kd,j_gam,1:nb_gam_desc(kd,j_gam)))))
               if ( su == 0 ) cycle
               do i_gam=1,nb_gam_desc(kd,j_gam)
                   pb_haplo(kd,num_haplo_desc(kd,j_gam,i_gam))  = pb_haplo(kd,num_haplo_desc(kd,j_gam,i_gam)) &
                                                 +prob_gam(kd,j_gam) * (pb_haplo_reduit(num_haplo_desc(kd,j_gam,i_gam)) /su)
               end do !i_gam
             end do !j_gam

           end if !estime
           end if !opt_model (hdam)
!
! ligne d'incidence
!
           do i_haplo=1,nb_haplo_reduit
             xinc(nkd,ntniv+i_haplo)=pb_haplo(kd,i_haplo)
           end do
          end if !presentc
         end do
        end do
       end do

       !ntniv=ntniv+nb_haplo_reduit
      ! nteff=nteff+nb_haplo_reduit

       deallocate (pp_ldla)
       deallocate (pm_ldla)
       !ajout ofi
       call free_internal_struct_ldla

    end subroutine add_haplotype_effect
!!***

!!****f* m_qtlmap_incidence/change_qtleffect
!!  NAME
!!    change_qtleffect
!!  DESCRIPTION
!!    Change the contingence matrix (probabilities of transmission information) during the chromosome scan .
!!
!!  INPUTS
!!    nteff       : index of the effect
!!    iq          : index of the qtl effect
!!    chr         : chromosome localisation
!!    n           : the position
!!    mint        : index of the interaction to remove (otherwise mint <= 0)
!!    linear      : use the likely probability genotype dam or not
!!  OUTPUTS
!!    xinc        : The contingence matrix
!!  incidenceDesc : The description of contingence matrix
!!
!!  NOTES
!!  SOURCE
    subroutine change_qtleffect(nteff,iq,chr,n,xinc,incidenceDesc,mint,linear)
       integer                                 ,intent(in)     :: nteff,iq,chr,n
       type(INCIDENCE_TYPE)                    ,intent(inout)  :: incidenceDesc
       real (kind=dp) , dimension(nd,incidenceDesc%ntnivmax) ,intent(inout)  :: xinc
       integer                                 ,intent(in)     :: mint ! index of interaction to remove from incidence
       logical                                 ,intent(in)     :: linear

       !local
       integer :: nbint,jef,nivint,ntlev,ip,jm,kd,ntt,km,ntniv,nkd
       real (kind=dp) ,dimension(:),allocatable:: pp,pm

       allocate (pp(nd))
       allocate (pm(nd))

       nbint=listModelTrait(incidenceDesc%ic)%nbint(iq)
       ntniv = incidenceDesc%desc(nteff)%start -1
       nkd=0

         do ip=1,np
          do jm=nmp(ip)+1,nmp(ip+1)
           if ( linear ) then
            call getprob_lin(chr,n,incidenceDesc%ic,ip,jm,ndm(jm)+1,ndm(jm+1),pp,pm)
           else
            call getglobalprob(chr,n,incidenceDesc%ic,ip,jm,ndm(jm)+1,ndm(jm+1),pp,pm)
           end if
           do kd=ndm(jm)+1,ndm(jm+1)
            if ( presentc(incidenceDesc%ic,kd) ) then
            nkd=nkd+1

           !  si des effets fixes sont en interaction avec l'allele paternel au qtl
           !  il faut estimer les effets qtl pour chacun des niveaux concernes
           !
           !  on commence donc par creer un effet composites regroupant l'ensemble de
           !  ces effets
           !
           nivint=1
           ntlev=1
           !nt=ntniv
           do jef=1,nbint
             if ( mint == jef) cycle ! this interaction is not adding
             nivint=nivint+ntlev*(nivx(kd,listModelTrait(incidenceDesc%ic)%indexFixedEffectWithInteraction(iq,jef))-1)
             ntlev=ntlev*nlev(listModelTrait(incidenceDesc%ic)%indexFixedEffectWithInteraction(iq,jef))
           end do
           ntt=ntniv+nivint+ntlev*(ip-1)
           xinc(nkd,ntt)=pp(kd)

          ! incidenceDesc%nivdir(kd,nteff+1)=ntt
          ! print *,'nt:',nivdir(kd,nteff+1),' vp:',pp(kd),' nteff:',nteff,' kd:',kd

           if(estime(incidenceDesc%ic,jm)) then
              km=iam(incidenceDesc%ic,repfem(jm))
              ntt= ntniv+ntlev*np+nivint+ntlev*(km-1)
              xinc(nkd,ntt)=pm(kd)
           !   incidenceDesc%nivdir(kd,nteff+2)=ntt
              ! print *,'nt:',nivdir(kd,nteff+2),' vp:',pm(kd)
           end if
          end if
         end do
        end do
       end do

       deallocate (pp)
       deallocate (pm)
       ! a modifier
       ! --> on devrait integerer l intialisation des pdds dans la boucle precedente
       ! attention les interaction qtl ne sont pas encore pris en compte
       if (.not. linear) then
         call pdd_at_position(incidenceDesc%ic,incidenceDesc%dataset,iq,chr,n)
       end if
    end subroutine change_qtleffect
!!***

!!****f* m_qtlmap_incidence/pdd_at_position
!!  NAME
!!    pdd_at_position
!!  DESCRIPTION
!!    Compute the probalities to receive the qtl from the sire
!!
!!  INPUTS
!!    ic          : index of the trait
!!    iq          : index of the qtl
!!    chr         : chromosome localisation
!!    n           : the position
!!  OUTPUTS
!!    dataset     : The user dataset
!!
!!  NOTES
!!  SOURCE
 subroutine pdd_at_position(ic,dataset,iq,chr,n)
          type (DATASET_TYPE)  , intent(inout)    :: dataset
          integer   ,intent(in)   :: ic,iq,chr,n

          integer :: ip,jm,kd,ig,kkd,igg,ifem,kkkd,kds
          real (kind=dp) :: ppt,pmt

          do ip=1,np
           dataset%lSires(ip)%ppt(iq,:)=0.d0
           ifem=0
           kkkd=0
           do jm=nmp(ip)+1,nmp(ip+1)
             ifem=ifem+1
             if (dataset%lSires(ip)%full_sib(ifem)%firstkd<0) cycle
             dataset%lSires(ip)%full_sib(ifem)%ppt(iq,:,:)=0.d0
             dataset%lSires(ip)%full_sib(ifem)%pmt(iq,:,:)=0.d0
             igg=0
             kds=kkkd
            ! print *,"ip,jm,firstkd:",ip,jm,lSires(ip)%half_sib%firstKd
             do ig=ngenom(chr,jm)+1,ngenom(chr,jm+1)
              igg=igg+1
              kkd=0
              kkkd=kds
              do kd=ngend(chr,ig)+1,ngend(chr,ig+1)
                if(presentc(ic,ndesc(chr,kd))) then
	             kkd=kkd+1
	             kkkd=kkkd+1
	             if( estime(ic,jm) )then
                  dataset%lSires(ip)%full_sib(ifem)%ppt(iq,igg,kkd)=&
                  -pdd(chr,kd,1,n)-pdd(chr,kd,2,n)+pdd(chr,kd,3,n)+pdd(chr,kd,4,n)
                  dataset%lSires(ip)%full_sib(ifem)%pmt(iq,igg,kkd)=&
                  -pdd(chr,kd,1,n)+pdd(chr,kd,2,n)-pdd(chr,kd,3,n)+pdd(chr,kd,4,n)
                 else
                  dataset%lSires(ip)%ppt(iq,dataset%lSires(ip)%half_sib%firstKd+kkkd-1)=&
                  -pdd(chr,kd,1,n)+pdd(chr,kd,3,n)
                 end if
               end if
              end do !! kd
             end do !! ig
           end do ! jm
          end do ! ip
      end subroutine pdd_at_position
!!***

!!****f* m_qtlmap_incidence/add_qtlinteraction
!!  NAME
!!    add_qtlinteraction
!!  DESCRIPTION
!!    <DEV> modele interaction
!!  NOTES
!!  SOURCE
    subroutine add_qtlinteraction(nteff,chr1,chr2,n1,n2,xinc,incidenceDesc)
       integer                                 ,intent(in)     :: nteff,chr1,chr2,n1,n2
       type(INCIDENCE_TYPE)                    ,intent(inout)  :: incidenceDesc
       real (kind=dp) , dimension(nd,incidenceDesc%ntnivmax) ,intent(inout)  :: xinc

       integer :: ntniv,nteffstart,ip,fem,jm,start,nkd
       logical ,dimension(nfem) :: femInit

       call log_mess("incidence : ** Add Qtl Interaction effect to estimate **",DEBUG_DEF)

       nteffstart = incidenceDesc%nteff
       incidenceDesc%nteff = incidenceDesc%nteff+1

       incidenceDesc%desc(nteffstart+1)%name="QTL Interaction effects"
       incidenceDesc%desc(nteffstart+1)%start=incidenceDesc%ntniv+1
       incidenceDesc%desc(nteffstart+1)%end=incidenceDesc%ntniv

       incidenceDesc%desc(nteffstart+1)%haveSubDesc=.true.

       allocate ( incidenceDesc%desc(nteffstart+1)%listSubDesc(np+count(estime(incidenceDesc%ic,:))) )

       start=incidenceDesc%desc(nteffstart+1)%start

       do ip=1,np
         ! on estime forcement les effets des haplotype pere QQ Qq qQ qq
          incidenceDesc%desc(nteffstart+1)%listSubDesc(ip)%name="Sire "//trim(pere(ip))
          incidenceDesc%desc(nteffstart+1)%listSubDesc(ip)%start=start
          incidenceDesc%desc(nteffstart+1)%listSubDesc(ip)%end=start+4-1
          start=start+4
          incidenceDesc%desc(nteffstart+1)%end=incidenceDesc%desc(nteffstart+1)%end+4
       end do

       fem=0
       do ip=1,np
         do jm=nmp(ip)+1,nmp(ip+1)
          ! on estime les effets combines pere et mere QQWW QQWw QQww QQwW.....
           if ( estime(incidenceDesc%ic,jm)) then
             fem=fem+1
             incidenceDesc%desc(nteffstart+1)%listSubDesc(np+fem)%name="Sire "//trim(pere(ip))//'- Dam '//trim(mere(jm))
             incidenceDesc%desc(nteffstart+1)%listSubDesc(np+fem)%start=start
             incidenceDesc%desc(nteffstart+1)%listSubDesc(np+fem)%end=start+16-1
             start=start+16
           end if
         end do
        end do
       incidenceDesc%desc(nteffstart+1)%end=incidenceDesc%desc(nteffstart+1)%start+start

       incidenceDesc%ntniv = start-1

       call change_interaction_effect(nteff,chr1,chr2,n1,n2,xinc,incidenceDesc)

    end subroutine add_qtlinteraction
!!***

!!****f* m_qtlmap_incidence/change_interaction_effect
!!  NAME
!!    change_interaction_effect
!!  DESCRIPTION
!!    <DEV> modele interaction
!!  NOTES
!!  SOURCE
    subroutine change_interaction_effect(nteff,chr1,chr2,n1,n2,xinc,incidenceDesc)
       integer                                 ,intent(in)     :: nteff,chr1,chr2,n1,n2
       type(INCIDENCE_TYPE)                    ,intent(inout)  :: incidenceDesc
       real (kind=dp) , dimension(nd,incidenceDesc%ntnivmax) ,intent(inout)  :: xinc

       !local
       integer :: ip,jm,kd,ntniv,kkd,ig1,ig2,kd2,fem,nkd


       ntniv = incidenceDesc%desc(nteff)%start -1
      ! print *,"NTNIV:",ntniv,' NTEFF:',nteff
       nkd=0

         do ip=1,np
          do jm=nmp(ip)+1,nmp(ip+1)
           ig1=likelyprob(chr1,ip,jm)
            do kd=ngend(chr1,ig1)+1,ngend(chr1,ig1+1)
              kkd = ndesc(chr1,kd)
             if(presentc(incidenceDesc%ic,kkd)) then
              !il faut rechercher le kd associe au chr2....
                do kd2=ngend(chr1,ig1)+1,ngend(chr1,ig1+1)
                   if ( kkd == ndesc(chr2,kd2)) exit
                end do
                nkd=nkd+1
                !naif pour l instant....
                ! effet haplotype pere
                ! QW, Qw qW qw
                xinc(nkd,ntniv+(ip-1)*4+1) = ( pdd(chr1,kd,1,n1)+ pdd(chr1,kd,2,n1)) * (pdd(chr2,kd2,1,n2)+pdd(chr2,kd2,2,n2))
                xinc(nkd,ntniv+(ip-1)*4+2) = ( pdd(chr1,kd,1,n1)+ pdd(chr1,kd,2,n1)) * (pdd(chr2,kd2,3,n2)+pdd(chr2,kd2,4,n2))
                xinc(nkd,ntniv+(ip-1)*4+3) = ( pdd(chr1,kd,3,n1)+ pdd(chr1,kd,4,n1)) * (pdd(chr2,kd2,1,n2)+pdd(chr2,kd2,2,n2))
                xinc(nkd,ntniv+(ip-1)*4+4) = ( pdd(chr1,kd,3,n1)+ pdd(chr1,kd,4,n1)) * (pdd(chr2,kd2,3,n2)+pdd(chr2,kd2,4,n2))

                if ( estime (incidenceDesc%ic,jm) ) then
                 ! effet haplotype combine QQWW QqWW qQWW qqWW QQwW QqwW qQwW qqwW QQWw QqWw qQWw qqWw QQww Qqww qQww qqww
                 fem=count(estime(incidenceDesc%ic,:jm))
                 xinc(nkd,ntniv+np*4+(fem-1)*16+1) = pdd(chr1,kd,1,n1) * pdd(chr2,kd2,1,n2) ! QQWW
                 xinc(nkd,ntniv+np*4+(fem-1)*16+2) = pdd(chr1,kd,3,n1) * pdd(chr2,kd2,1,n2) ! QqWW
                 xinc(nkd,ntniv+np*4+(fem-1)*16+3) = pdd(chr1,kd,2,n1) * pdd(chr2,kd2,1,n2) ! qQWW
                 xinc(nkd,ntniv+np*4+(fem-1)*16+4) = pdd(chr1,kd,4,n1) * pdd(chr2,kd2,1,n2) ! qqWW

                 xinc(nkd,ntniv+np*4+(fem-1)*16+5) = pdd(chr1,kd,1,n1) * pdd(chr2,kd2,2,n2) ! QQwW
                 xinc(nkd,ntniv+np*4+(fem-1)*16+6) = pdd(chr1,kd,3,n1) * pdd(chr2,kd2,2,n2) ! QqwW
                 xinc(nkd,ntniv+np*4+(fem-1)*16+7) = pdd(chr1,kd,2,n1) * pdd(chr2,kd2,2,n2) ! qQwW
                 xinc(nkd,ntniv+np*4+(fem-1)*16+8) = pdd(chr1,kd,4,n1) * pdd(chr2,kd2,2,n2) ! qqwW

                 xinc(nkd,ntniv+np*4+(fem-1)*16+9) = pdd(chr1,kd,1,n1) * pdd(chr2,kd2,3,n2) ! QQWw
                 xinc(nkd,ntniv+np*4+(fem-1)*16+10) = pdd(chr1,kd,3,n1) * pdd(chr2,kd2,3,n2) ! QqWw
                 xinc(nkd,ntniv+np*4+(fem-1)*16+11) = pdd(chr1,kd,2,n1) * pdd(chr2,kd2,3,n2) ! qQWw
                 xinc(nkd,ntniv+np*4+(fem-1)*16+12) = pdd(chr1,kd,4,n1) * pdd(chr2,kd2,3,n2) ! qqWw

                 xinc(nkd,ntniv+np*4+(fem-1)*16+13) = pdd(chr1,kd,1,n1) * pdd(chr2,kd2,4,n2) ! QQww
                 xinc(nkd,ntniv+np*4+(fem-1)*16+14) = pdd(chr1,kd,3,n1) * pdd(chr2,kd2,4,n2) ! Qqww
                 xinc(nkd,ntniv+np*4+(fem-1)*16+15) = pdd(chr1,kd,2,n1) * pdd(chr2,kd2,4,n2) ! qQww
                 xinc(nkd,ntniv+np*4+(fem-1)*16+16) = pdd(chr1,kd,4,n1) * pdd(chr2,kd2,4,n2) ! qqww
               end if
          end if
         end do ! kd
        end do ! jm
       end do ! ip


    end subroutine change_interaction_effect
!!***


!!****f* m_qtlmap_incidence/add_effcov
!!  NAME
!!    add_effcov
!!  DESCRIPTION
!!    Add the nuisances effects inside a contingence matrix with a description
!!    * An effect is adding for each covariate and fixed effect
!!  INPUTS
!!   meff         : fixed effect to not added in the contingence matrix (if meff <= 0, all fixed effects are adding)
!!   mcov         : covariate effect to not added in the contingence matrix (if mcov <= 0, all covariates effects are adding)
!!  OUTPUTS
!!    xinc        : The contingence matrix
!!  incidenceDesc : The description of contingence matrix
!!
!!  NOTES
!!  SOURCE
    subroutine add_effcov(xinc,incidenceDesc,meff,mcov)
       type(INCIDENCE_TYPE)                    , intent(inout) :: incidenceDesc
       real (kind=dp) , dimension(nd,incidenceDesc%ntnivmax) , intent(inout) :: xinc
       integer                                 , intent(in)    :: meff   ! indice on effet fixed list to remove (<=0 otherwise)
       integer                                 , intent(in)    :: mcov   ! indice on covariate list to remove (<=0 otherwise)
       integer :: nkd,ntniv,ip,jm,kd,ntnifix
       integer :: nbef,nbco,nteff,jef,jcov,ieff
       integer :: nbniv,nivd,i,j,nbef2,nbco2

       call log_mess("incidence : ** Add eff and cov ** ",DEBUG_DEF)

       nbef=listModelTrait(incidenceDesc%ic)%nbfe
       nbco=listModelTrait(incidenceDesc%ic)%nbco

       nbef2=nbef
       if ( meff /= 0 ) nbef2=nbef-1
       nbco2=nbco
       if ( mcov /=0 ) nbco2=nbco-1


       if (nbef2+nbco2 <=0)return

       nbniv=0
       ieff=incidenceDesc%nteff

       if ( nbef2 > 0 ) then
        ieff=ieff+1
        incidenceDesc%eqtl_print(ieff)=.false.
        do i=1,nbef
         if ( i == meff ) cycle
         nbniv=nlev(listModelTrait(incidenceDesc%ic)%indexFixedEffect(i))+nbniv
        end do
        incidenceDesc%desc(ieff)%name="Fixed effects"
        incidenceDesc%desc(ieff)%start=incidenceDesc%ntniv+1
        incidenceDesc%desc(ieff)%end=incidenceDesc%ntniv+nbniv
        incidenceDesc%desc(ieff)%haveSubDesc=.true.

        !init of xinc
        xinc(:,incidenceDesc%desc(ieff)%start:incidenceDesc%desc(ieff)%end)=0.d0

        allocate ( incidenceDesc%desc(ieff)%listSubDesc(nbniv) )

        j=0
        do jef=1,nbef
         if ( jef == meff ) cycle
         do i=1,nlev(listModelTrait(incidenceDesc%ic)%indexFixedEffect(jef))
         j=j+1
         incidenceDesc%desc(ieff)%listSubDesc(j)%name=trim(namefix(listModelTrait(incidenceDesc%ic)%indexFixedEffect(jef)))&
         //' level '//trim(str(i))
         incidenceDesc%desc(ieff)%listSubDesc(j)%start=incidenceDesc%ntniv+j
         incidenceDesc%desc(ieff)%listSubDesc(j)%end=incidenceDesc%ntniv+j
         end do
        end do
        incidenceDesc%nteff = incidenceDesc%nteff+1
       end if

       if ( nbco2 > 0 ) then
        ieff=ieff+1
        incidenceDesc%eqtl_print(ieff)=.false.
        incidenceDesc%desc(ieff)%name="Covariates"
        incidenceDesc%desc(ieff)%start=incidenceDesc%ntniv+nbniv+1
        incidenceDesc%desc(ieff)%end=incidenceDesc%ntniv+nbniv+nbco2
        incidenceDesc%desc(ieff)%haveSubDesc=.true.

        !init of xinc
        xinc(:,incidenceDesc%desc(ieff)%start:incidenceDesc%desc(ieff)%end)=0.d0

        allocate ( incidenceDesc%desc(ieff)%listSubDesc(nbco2) )

        j=0
        do jcov=1,nbco
         if ( jcov == mcov ) cycle
         j=j+1
         incidenceDesc%desc(ieff)%listSubDesc(j)%name=trim(namecov(listModelTrait(incidenceDesc%ic)%indexCovariate(jcov)))
         incidenceDesc%desc(ieff)%listSubDesc(j)%start=incidenceDesc%ntniv+nbniv+j
         incidenceDesc%desc(ieff)%listSubDesc(j)%end=incidenceDesc%ntniv+nbniv+j
        end do
        incidenceDesc%nteff = incidenceDesc%nteff+1
       end if


       nkd = 0
       nbniv=0
       do ip=1,np
         do jm=nmp(ip)+1,nmp(ip+1)
           do kd=ndm(jm)+1,ndm(jm+1)
            if ( .not. presentc(incidenceDesc%ic,kd) ) cycle
            ntniv=incidenceDesc%ntniv
            nkd=nkd+1
            !
            !  autres effets fixes
            !
            nbniv=0
            do jef=1,nbef
             if ( jef == meff ) cycle ! this effet fixed is not adding
            ! incidenceDesc%nivdir(kd,nteff+jef)=ntniv+nbniv+nivx(kd,modele(ic,3+jef))
             nivd=ntniv+nbniv+nivx(kd,listModelTrait(incidenceDesc%ic)%indexFixedEffect(jef))
             xinc(nkd,nivd)=1
             nbniv=nbniv+nlev(listModelTrait(incidenceDesc%ic)%indexFixedEffect(jef))
            end do

            ntnifix=ntniv+nbniv
            !covariate
            j=0
            do jcov=1,nbco
             if ( jcov == mcov ) cycle ! this covariate is not adding
             j=j+1
             ! incidenceDesc%covdir(kd,jcov)=covar(kd,modele(ic,nbt+jcov))
             xinc(nkd,ntnifix+j)=covar(kd,listModelTrait(incidenceDesc%ic)%indexCovariate(jcov))
            end do
            !ntniv=ntnifix+nbco
            !nbt=nbt+nbco
       end do
      end do
     end do

     incidenceDesc%ntniv=incidenceDesc%ntniv+nbniv+nbco2
    end subroutine add_effcov
!!***


!!****f* m_qtlmap_incidence/add_other_continue_traits_as_cov
!!  NAME
!!    add_other_continue_traits_as_cov
!!  DESCRIPTION
!!    Add traits (that are not analysing in the current session) as covariate effects inside a contingence matrix with a description
!!    * An effect is adding for each covariate and fixed effect
!!  INPUTS
!!   mcov         : covariate effect to not added in the contingence matrix (if mcov <= 0, all covariates effects are adding)
!!  OUTPUTS
!!    xinc        : The contingence matrix
!!  incidenceDesc : The description of contingence matrix
!!
!!  NOTES
!!  SOURCE
    subroutine add_other_continue_traits_as_cov(xinc,incidenceDesc,mcov)
       type(INCIDENCE_TYPE)                    , intent(inout) :: incidenceDesc
       real (kind=dp) , dimension(nd,incidenceDesc%ntnivmax) , intent(inout) :: xinc
       integer                                 , intent(in)    :: mcov   ! indice on covariate list to remove (<=0 otherwise)

       integer :: nbntniv,i,j,jj,nkd,ieff,ip,jm,kd,jcov

       ieff=incidenceDesc%nteff
       ieff=ieff+1

       nbntniv=0
       do i=1,ncar
         if ( i == incidenceDesc%ic )cycle
         if ( natureY(i) /= 'r') cycle
         nbntniv=nbntniv+1
       end do
       if ( nbntniv <= 0 ) return
       incidenceDesc%eqtl_print(ieff)=.false.
       incidenceDesc%desc(ieff)%name="Traits as covariates"
       incidenceDesc%desc(ieff)%start=incidenceDesc%ntniv+1
       incidenceDesc%desc(ieff)%end=incidenceDesc%ntniv+nbntniv
       incidenceDesc%desc(ieff)%haveSubDesc=.true.

       !init of xinc
       xinc(:,incidenceDesc%desc(ieff)%start:incidenceDesc%desc(ieff)%end)=0.d0

       allocate ( incidenceDesc%desc(ieff)%listSubDesc(nbntniv) )

       jj=0;j=0
       do jcov=1,ncar
         if ( jcov == incidenceDesc%ic )cycle ! the current trait...
         if ( natureY(jcov) /= 'r') cycle ! can not be treat...
         jj=jj+1
         if ( jj == mcov ) cycle
         j=j+1
         incidenceDesc%desc(ieff)%listSubDesc(j)%name=trim(carac(jcov))
         incidenceDesc%desc(ieff)%listSubDesc(j)%start=incidenceDesc%ntniv+j
         incidenceDesc%desc(ieff)%listSubDesc(j)%end=incidenceDesc%ntniv+j

         nkd=0
         do ip=1,np
          do jm=nmp(ip)+1,nmp(ip+1)
           do kd=ndm(jm)+1,ndm(jm+1)
            if ( .not. presentc(incidenceDesc%ic,kd) ) cycle
             nkd=nkd+1
             if ( presentc(jcov,kd) ) then
               xinc(nkd,incidenceDesc%desc(ieff)%listSubDesc(j)%start)= y(jcov,kd)
             else
               xinc(nkd,incidenceDesc%desc(ieff)%listSubDesc(j)%start)= 0.d0
             end if
           end do !kd
          end do !jm
         end do !ip
       end do ! jcov
       incidenceDesc%nteff = incidenceDesc%nteff+1
       incidenceDesc%ntniv=incidenceDesc%ntniv+nbntniv

    end subroutine add_other_continue_traits_as_cov

!remove the last effect describe by the INCIDENCE_TYPE
    subroutine remove_last_effect(incidenceDesc,xinc)
       type(INCIDENCE_TYPE)                    , intent(inout) :: incidenceDesc
       real (kind=dp) , dimension(nd,incidenceDesc%ntnivmax) , intent(inout) :: xinc

       xinc(:,incidenceDesc%desc(incidenceDesc%nteff)%start:incidenceDesc%desc(incidenceDesc%nteff)%end)=0.d0
       incidenceDesc%ntniv=incidenceDesc%desc(incidenceDesc%nteff)%start-1
       ! deallocation of sub effect
       if (associated(incidenceDesc%desc(incidenceDesc%nteff)%listSubDesc)) then
         deallocate ( incidenceDesc%desc(incidenceDesc%nteff)%listSubDesc )
       end if

       incidenceDesc%nteff=incidenceDesc%nteff-1
    end subroutine remove_last_effect
!!***

!!****f* m_qtlmap_incidence/getprob_lin
!!  NAME
!!    getprob_lin
!!  DESCRIPTION
!!    Get the probabilities to receive qtl in a linear context depending the probabilities of transmission
!!  INPUTS
!!    ic          : index of the trait
!!    ip          : index of sire
!!    jm          : index of dam
!!    nd1         : first index of progenies of family (ip,jm)
!!    nd2         : last index of progenies of family (ip,jm)
!!    chr         : chromosome localisation
!!    n           : the position
!!  OUTPUTS
!!    probp       : The probabilities to receive the qtl from the sire
!!    probm       : The probabilities to receive the qtl from the dam
!!
!!  NOTES
!!  SOURCE
      subroutine getprob_lin(chr,n,ic,ip,jm,nd1,nd2,probp,probm)
          integer                 ,intent(in)                 :: chr,n,ic,ip,jm,nd1,nd2
          real (kind=dp)   ,dimension(nd)   ,intent(out) :: probp,probm

          integer :: kd,ig,kkd
          real (kind=dp) :: p

          probp=0.d0
          probm=0.d0
          ig=likelyprob(chr,ip,jm)
          do kkd=nd1,nd2
           do kd=ngend(chr,ig)+1,ngend(chr,ig+1)
            if ( kkd == ndesc(chr,kd)) then
             if(presentc(ic,ndesc(chr,kkd))) then
	            if( estime(ic,jm) )then
                 probp(kkd)=-pdd(chr,kd,1,n)-pdd(chr,kd,2,n)+pdd(chr,kd,3,n)+pdd(chr,kd,4,n)
                 probm(kkd)=-pdd(chr,kd,1,n)+pdd(chr,kd,2,n)-pdd(chr,kd,3,n)+pdd(chr,kd,4,n)
                 exit
                else
                 probp(kkd)=-pdd(chr,kd,1,n)+pdd(chr,kd,3,n)
                 exit
                end if
              end if
             end if
           end do
          end do

      end subroutine getprob_lin
!!***


!!****f* m_qtlmap_incidence/getprobld_lin
!!  NAME
!!    getprobld_lin
!!  DESCRIPTION
!!    Get the probabilities to receive an haplotype
!!  INPUTS
!!    ic          : index of the trait
!!    ip          : index of sire
!!    jm          : index of dam
!!    nd1         : first index of progenies of family (ip,jm)
!!    nd2         : last index of progenies of family (ip,jm)
!!    chr         : chromosome localisation
!!    n           : the position
!!  OUTPUTS
!!    pp_ldla     :
!!    pm_ldla     :
!!
!!  NOTES
!!  SOURCE
 !attention pp_ldla,pm_ldla de taille nd car lineaire (1 seul genotype pour toute les meres)
      subroutine getprobld_lin(chr,n,ic,ip,jm,nd1,nd2,pp_ldla,pm_ldla)
          integer                 ,intent(in)                 :: chr,n,ic,ip,jm,nd1,nd2
          real (kind=dp)   ,dimension(nd,2)   ,intent(out) :: pp_ldla,pm_ldla

          integer :: kd,ig,kkd
          real (kind=dp) :: p

          pp_ldla=0.d0
          pm_ldla=0.d0
          ig=likelyprob(chr,ip,jm)
          do kkd=nd1,nd2
           do kd=ngend(chr,ig)+1,ngend(chr,ig+1)
            if ( kkd == ndesc(chr,kd)) then
             if(presentc(ic,kkd)) then
	            if( estime(ic,jm) )then
                  pp_ldla(kkd,1)=pp_ldla(kkd,1)+probg(chr,ig)*(pdd(chr,kd,1,n)+pdd(chr,kd,2,n))
                  pp_ldla(kkd,2)=pp_ldla(kkd,2)+probg(chr,ig)*(pdd(chr,kd,3,n)+pdd(chr,kd,4,n))
                  pm_ldla(kkd,1)=probg(chr,ig)*(pdd(chr,kd,1,n)+pdd(chr,kd,3,n))
                  pm_ldla(kkd,2)=probg(chr,ig)*(pdd(chr,kd,2,n)+pdd(chr,kd,4,n))
                 exit
                else
                  pp_ldla(kkd,1)=pdd(chr,kd,1,n)
                  pp_ldla(kkd,2)=pdd(chr,kd,3,n)
                 exit
                end if
              end if
             end if
           end do
          end do

      end subroutine getprobld_lin
!!***


!!****f* m_qtlmap_incidence/getglobalprob
!!  NAME
!!    getglobalprob
!!  DESCRIPTION
!!    Get the probabilities to receive a qtleffect in a general context
!!  INPUTS
!!    ic          : index of the trait
!!    ip          : index of sire
!!    jm          : index of dam
!!    nd1         : first index of progenies of family (ip,jm)
!!    nd2         : last index of progenies of family (ip,jm)
!!    chr         : chromosome localisation
!!    n           : the position
!!  OUTPUTS
!!    probp       :
!!    probm       :
!!
!!  NOTES
!!  SOURCE
     subroutine getglobalprob(chr,n,ic,ip,jm,nd1,nd2,probp,probm)
          integer                 ,intent(in)                 :: chr,n,ic,ip,jm,nd1,nd2
          real (kind=dp)    ,dimension(nd)   ,intent(out) :: probp,probm

          integer :: kd,ig,kkd
          real (kind=dp) :: ppt,pmt
          probp=0.d0
          probm=0.d0
          do kkd=nd1,nd2
            do ig=ngenom(chr,jm)+1,ngenom(chr,jm+1)
             do kd=ngend(chr,ig)+1,ngend(chr,ig+1)
              if ( kkd == ndesc(chr,kd)) then
               if(presentc(ic,ndesc(chr,kkd))) then
	             if( estime(ic,jm) )then
                  ppt=-pdd(chr,kd,1,n)-pdd(chr,kd,2,n)+pdd(chr,kd,3,n)+pdd(chr,kd,4,n)
                  probp(kkd)=probp(kkd)+probg(chr,ig)*ppt
                  pmt=-pdd(chr,kd,1,n)+pdd(chr,kd,2,n)-pdd(chr,kd,3,n)+pdd(chr,kd,4,n)
                  probm(kkd)=probm(kkd)+probg(chr,ig)*pmt
                else
                  ppt=-pdd(chr,kd,1,n)+pdd(chr,kd,3,n)
                  probp(kkd)=ppt
                end if
                exit
              end if
             end if
           end do !! kd
          end do !! ig
        end do !! kkd

      end subroutine getglobalprob
!!***


!!****f* m_qtlmap_incidence/get_precision
!!  NAME
!!    get_precision
!!  DESCRIPTION
!!    Compute the precision for each level of the incidence matrix.
!!  INPUTS
!!    xx          : X'.X ,X contingence matrix
!!    xxx         : buffer array
!!    jm          : index of dam
!!    nd1         : first index of progenies of family (ip,jm)
!!    nd2         : last index of progenies of family (ip,jm)
!!    chr         : chromosome localisation
!!    n           : the position
!!  OUTPUTS
!!  incidenceDesc%precis : Precision of each level of the incidence matrix.
!!
!!  NOTES
!!    ginv1
!!  SOURCE
  subroutine get_precision(xx,xxx,incidenceDesc)
      type(INCIDENCE_TYPE)                                ,intent(inout) :: incidenceDesc
      real (kind=dp)  ,dimension(incidenceDesc%ntnivmax,incidenceDesc%ntnivmax)       , intent(in)   :: xx
      real (kind=dp)  ,dimension(incidenceDesc%ntnivmax,incidenceDesc%ntnivmax)       , intent(out)  :: xxx

      integer                                             :: iniv,kniv,jniv,lniv,irank
      real (kind=dp)                                      :: ups

!
!  Reduction de la matrice d'incidence
!
       xxx(:incidenceDesc%ntniv,:incidenceDesc%ntniv)=0.d0

       kniv=0
       do iniv =1,incidenceDesc%ntniv
         if(incidenceDesc%vecsol(iniv)) then
           kniv=kniv+1
           lniv=0
           do jniv=1,incidenceDesc%ntniv
             if(incidenceDesc%vecsol(jniv)) then
               lniv=lniv+1
               xxx(kniv,lniv)=xx(iniv,jniv)
             end if
           end do
          end if
       end do
!
!  inversion de la matrice d'incidence reduite pour calcul des precisions
!

      ups=1.d-15
      call ginv1(xxx,lniv,size(xxx,1),ups,irank)

      jniv=0
      do iniv=1,incidenceDesc%ntniv
        incidenceDesc%precis(iniv)=9999.d0
        if(incidenceDesc%vecsol(iniv)) then
          jniv=jniv+1
         incidenceDesc%precis(iniv)=xxx(jniv,jniv)
        end if
      end do

      end subroutine get_precision
!!***

!!****f* m_qtlmap_incidence/get_correlations
!!  NAME
!!    get_correlations
!!  DESCRIPTION
!!    Compute the correlation between level of the incidence matrix.
!!  INPUTS
!!  incidenceDesc : the description of the incidence matrix.
!!    xxx         : buffer array provide by get_precision function
!!  OUTPUTS
!!    xcor        : the correlation
!!
!!  NOTES
!!    xxx come from the get_precision function.
!!  SOURCE
     subroutine get_correlations(incidenceDesc,xxx,xcor)
       type(INCIDENCE_TYPE)                          ,intent(in)  :: incidenceDesc
       real (kind=dp)  ,dimension(incidenceDesc%ntnivmax,incidenceDesc%ntnivmax) ,intent(in)  :: xxx
       real (kind=dp)  ,dimension(incidenceDesc%ntnivmax,incidenceDesc%ntnivmax) ,intent(out) :: xcor
       integer ::ief,jef,i,j

       xcor=0.d0
       ief=0
       do i=1,incidenceDesc%ntniv
         if(incidenceDesc%vecsol(i))then
            ief=ief+1
            jef=0
            do j=1,incidenceDesc%ntniv
              if(incidenceDesc%vecsol(j))then
                jef=jef+1
                if (xxx(ief,ief) == 0 .or. xxx(jef,jef) == 0) then
                  xcor(i,j)=0.d0
                else
                  xcor(i,j)=xxx(ief,jef)/dsqrt(xxx(ief,ief)*xxx(jef,jef))
                end if
               end if
            end do
         end if
       end do
      end subroutine get_correlations
!!***

!!****f* m_qtlmap_incidence/confusion_between_qtl_and_effect
!!  NAME
!!    confusion_between_qtl_and_effect
!!  DESCRIPTION
!!   Detect a confusion betwwen a qtl effect and the other effect described by the model.
!!  INPUTS
!!  incidenceDesc : the description of the incidence matrix.
!!    xcor        : the correlation matrix
!!  index_effect  : the index qtl effect to test with the other effect
!!  OUTPUTS
!!    sizealert     : number of alert detected
!!    alertCorrQtl  : the correlation
!!    corrmax       : the maximum correlation finded succesively
!!
!!  NOTES
!!    xxx come from the get_precision function.
!!  HISTORY
!!    14/09/2010 : correction bug dimension de femInit
!!  SOURCE
      subroutine confusion_between_qtl_and_effect(incidenceDesc,xcor,index_effect,&
      sizealert,alertCorrQtl,nalert,corrmax)
        type(INCIDENCE_TYPE)                          ,intent(in)  :: incidenceDesc
        real (kind=dp)  ,dimension(incidenceDesc%ntnivmax,incidenceDesc%ntnivmax) ,intent(in)  :: xcor
        integer                                       ,intent(in)  :: index_effect
        integer                                       ,intent(in)  :: sizealert
        type(CORR_ALERT_TYPE) ,dimension(sizealert)   ,intent(out) :: alertCorrQtl
        integer                                       ,intent(out) :: nalert
        real (kind=dp)                              ,intent(inout) :: corrmax

        integer :: numqtl,ip,jm,iqtl,jniv,ntlev,id,s,e,i,fem
        logical ,dimension(:),allocatable :: femInit

        nalert=0
        allocate (femInit(maxval(iam)))
        !
        !  on caclule les correlations entre les colonnes de X'X des effets qtl pere et de
        !  tous les autres effets
        !
        do numqtl=1,incidenceDesc%nqtl
         do ip=1,np
           ntlev=0
           !print *,"ip:",ip
           s=incidenceDesc%ntniv_qtlsires(numqtl)+(ip-1)*incidenceDesc%ntlev
           e=s+incidenceDesc%ntlev-1
           do iqtl=s,e
           !  print *,"iqtl:",iqtl
             ntlev=ntlev+1

             if (incidenceDesc%vecsol(iqtl))then
              !
              !  test d'une confusion avec l'effect concerné
              !

              do jniv=incidenceDesc%desc(index_effect)%start,incidenceDesc%desc(index_effect)%end
               if (abs(xcor(iqtl,jniv))>=corrmax) corrmax=xcor(iqtl,jniv)
               !  si la correlation est superieure a corMax, on imprime une alerte

               if(dabs(xcor(iqtl,jniv)).ge.THRES_CONFUSION)then
                nalert=nalert+1
                alertCorrQtl(nalert)%qtl=numqtl
                alertCorrQtl(nalert)%ip=ip
                alertCorrQtl(nalert)%corr=xcor(iqtl,jniv)
                alertCorrQtl(nalert)%ntlev=ntlev
                id=jniv-incidenceDesc%desc(index_effect)%start+1
                alertCorrQtl(nalert)%name_effect=incidenceDesc%desc(index_effect)%name
                alertCorrQtl(nalert)%name_level=incidenceDesc%desc(index_effect)%listSubDesc(id)%name
               end if
              end do ! jniv
            end if
           end do ! iqtl
!
!  fin pour le niveau du qtl du pere
!
        end do ! ip

        femInit=.false.
         do jm=1,nm
          if (estime(incidenceDesc%ic,jm) ) then
             fem=iam(incidenceDesc%ic,repfem(jm))
             if (femInit(fem)) then
                cycle
             end if
             femInit(fem)=.true.
          !  print *,'jm:',jm
           ntlev=0
           s=incidenceDesc%ntniv_qtldams(numqtl)+(fem-1)*incidenceDesc%ntlev
           e=s+incidenceDesc%ntlev-1
           do iqtl=s,e
        !     print *,"iqtl:",iqtl
             ntlev=ntlev+1

             if (incidenceDesc%vecsol(iqtl))then
              !
              !  test d'une confusion avec l'effect concerné
              !
              do jniv=incidenceDesc%desc(index_effect)%start,incidenceDesc%desc(index_effect)%end
          !     print *,'jniv:',jniv
           !    print *,abs(xcor(iqtl,jniv))
               if (abs(xcor(iqtl,jniv))>=corrmax) corrmax=xcor(iqtl,jniv)
               !  si la correlation est superieure a corMax, on imprime une alerte
               !
               if(dabs(xcor(iqtl,jniv)).ge.THRES_CONFUSION)then
                nalert=nalert+1
                alertCorrQtl(nalert)%qtl=numqtl
                alertCorrQtl(nalert)%jm=jm
                alertCorrQtl(nalert)%corr=xcor(iqtl,jniv)
                alertCorrQtl(nalert)%ntlev=ntlev
                id=jniv-incidenceDesc%desc(index_effect)%start+1
                alertCorrQtl(nalert)%name_effect=incidenceDesc%desc(index_effect)%name
                alertCorrQtl(nalert)%name_level=incidenceDesc%desc(index_effect)%listSubDesc(id)%name
               end if
              end do ! jniv
             end if
            end do ! iqtl

!
!  fin pour le niveau du qtl du pere
!
         end if
        end do ! jm
      end do ! numqtl
      deallocate (femInit)

      end subroutine confusion_between_qtl_and_effect
!!***

!!****f* m_qtlmap_incidence/confusion_type1
!!  NAME
!!    confusion_type1
!!  DESCRIPTION
!!
!!  INPUTS
!!  incidenceDesc : the description of the incidence matrix.
!!    xxx         :
!!  OUTPUTS
!!    workstruct  :
!!
!!  NOTES
!!    xxx come from the get_precision function.
!!  SOURCE
      subroutine confusion_type1(incidenceDesc,xxx,workstruct)
        type(INCIDENCE_TYPE)                          ,intent(in)  :: incidenceDesc
        real (kind=dp)  ,dimension(incidenceDesc%ntnivmax,incidenceDesc%ntnivmax) ,intent(in)  :: xxx
        type(INCIDENCE_GEN_STRUCT)            ,intent(inout)       :: workstruct

        real (kind=dp)  ,dimension(incidenceDesc%ntnivmax,incidenceDesc%ntnivmax)                     :: xcor
        type(CORR_ALERT_TYPE) ,dimension(:),allocatable  :: alerts
        type(CORR_ALERT_TYPE) ,dimension(:),allocatable  :: as
        integer :: ialert,sizealert,eff,i,j,starteff

        call get_correlations(incidenceDesc,xxx,xcor)
        workstruct%corrmax=0.d0

        sizealert=( np+count(estime(incidenceDesc%ic,:)) )*incidenceDesc%ntlev*incidenceDesc%nbnivest

        allocate (alerts(sizealert))
        allocate (as(sizealert*incidenceDesc%nqtl))

        workstruct%nalert=0
        starteff = workstruct%listnteff(workstruct%nqtl)+1
        if (count(estime(incidenceDesc%ic,:))>0) starteff = starteff+1
        do eff=starteff,incidenceDesc%nteff
           call log_mess("computing confusion beetween qtls and:"//incidenceDesc%desc(eff)%name,VERBOSE_DEF)
           call confusion_between_qtl_and_effect(incidenceDesc,xcor,&
           eff,sizealert,alerts,ialert,workstruct%corrmax)
           if (ialert>0) then
      !       print *,"ialert:",alerts(1)%corr,alerts(2)%corr,alerts(3)%corr
             as(workstruct%nalert+1:workstruct%nalert+ialert)=alerts(:ialert)
             workstruct%nalert=workstruct%nalert+ialert
           end if
        end do

        allocate (workstruct%alertQtl(workstruct%nalert))
        workstruct%alertQtl=as(:workstruct%nalert)

        deallocate (alerts)
        deallocate (as)

      end subroutine confusion_type1
!!***

!!****f* m_qtlmap_incidence/get_incidence_reduit
!!  NAME
!!    get_incidence_reduit
!!  DESCRIPTION
!!
!!  INPUTS
!!  incidenceDesc : the description of the incidence matrix.
!!    xinc        : the contingence matrix
!!  OUTPUTS
!!    xincred     : the incontingence matrixithout level which are not estimable (the reduction contingence matrix)
!!
!!  NOTES
!!  SOURCE
    subroutine get_incidence_reduit(incidenceDesc,xinc,xincred)
      type(INCIDENCE_TYPE)                               ,intent(in)          :: incidenceDesc
      real (kind=dp)   ,dimension(nd,incidenceDesc%ntnivmax)           ,intent(in)          :: xinc
      real (kind=dp)   ,dimension(incidenceDesc%dataset%nkd,incidenceDesc%nbnivest) ,intent(inout)  :: xincred

      integer :: i,j
      j=0
      do i=1,incidenceDesc%ntniv
         if ( incidenceDesc%vecsol(i) ) then
           j=j+1
           xincred(:,j)=xinc(:incidenceDesc%dataset%nkd,i)
         end if
      end do

    end subroutine get_incidence_reduit
!!***

!!****f* m_qtlmap_incidence/fill_nonull_elements
!!  NAME
!!    fill_nonull_elements
!!  DESCRIPTION
!!    Compacte a sparse matrice.
!!  INPUTS
!!    xincred              : the reduction contingence matrix
!!  OUTPUTS
!!    incidenceDesc%nonull : the compacted array with elements not null.
!!  incidenceDesc%nbnonull : index in the xincred of no null element by line
!!
!!  NOTES
!!  SOURCE
    subroutine fill_nonull_elements(incidenceDesc,s1,s2,xincred)
      integer                     ,intent(in)         :: s1,s2
      type(INCIDENCE_TYPE)       ,intent(inout)       :: incidenceDesc
      real (kind=dp)   ,dimension(s1,s2) ,intent(in)  :: xincred
      integer :: i,j,k

      do i=1,incidenceDesc%dataset%nkd
       k=0
       do j=1,incidenceDesc%nbnivest
        if ( xincred(i,j) /= 0.d0 ) then
         k=k+1
         incidenceDesc%nonull(i,k)=j
        end if
       end do
       incidenceDesc%nbnonull(i)=k
      end do

    end subroutine fill_nonull_elements
!!***

!!****f* m_qtlmap_incidence/matmul_incidence
!!  NAME
!!    matmul_incidence
!!  DESCRIPTION
!!    multiplication matrix routine using the compacted sparse matrix structure (incidenceDesc%nonull,incidenceDesc%nbnonull)
!!  INPUTS
!!    kd1                  : first line index
!!    kd2                  : last line index
!!    incidenceDesc%nonull : the compacted array with elements not null.
!!  incidenceDesc%nbnonull : index in the xincred of no null element by line
!!   sizeInXinc1           : size of the first dimension of xinc
!!   sizeInXinc2           : size of the second dimension of xinc
!!   in_X                  : the vector to multiplied
!!   SizeOutVec            : size of the result vector
!!   startRes              : first index in the result vector to set result
!!
!!  OUTPUTS
!!   out_vec               : the result vector
!!
!!  NOTES
!!  SOURCE
    subroutine matmul_incidence(kd1,kd2,incidenceDesc,sizeInXinc1,sizeInXinc2,in_xinc,in_X,SizeOutVec,startRes,out_vec)
      integer                                      ,intent(in)          :: kd1,kd2,SizeOutVec,startRes,sizeInXinc1,sizeInXinc2
      type(INCIDENCE_TYPE)                         ,intent(in)          :: incidenceDesc
      real (kind=dp)   ,dimension(sizeInXinc1,sizeInXinc2) ,intent(in)  :: in_xinc
      real (kind=dp)   ,dimension(incidenceDesc%nbnivest)  ,intent(in)  :: in_X
      real (kind=dp)   ,dimension(SizeOutVec)              ,intent(out) :: out_vec

      integer :: i,j,k
      out_vec(:) = 0.d0
      k=startRes
      do i=kd1,kd2
        do j=1,incidenceDesc%nbnonull(i)
           out_vec(k) = out_vec(k) + in_X(incidenceDesc%nonull(i,j))*in_xinc(i,incidenceDesc%nonull(i,j))
        end do
        k=k+1
      end do

    end subroutine matmul_incidence
!!***

!!****f* m_qtlmap_incidence/model_XT_X
!!  NAME
!!    model_XT_X
!!  DESCRIPTION
!! X contingence matrix
!! Build X' X : Model linear homoscedastic without cd
!!              or
!!              Non linear model (modlin)
!!
!! Dimension of X : ND line NTNIV Column
!!
!!                Mu sire1 ...sireN ..MatEff1 Mateff
!!     Animal1    1
!!     ....       1
!!     AnimalN    1
!!  INPUTS
!!    xinc                 : the contingence matrix
!!  OUTPUTS
!!    XX                   : X' X
!!
!!  NOTES
!!    Context homoscedastic
!!  SOURCE
      subroutine model_XT_X(xinc,incidenceDesc,XX)
         type(INCIDENCE_TYPE)                         ,intent(in)   :: incidenceDesc
         real (kind=dp)       , dimension(:,:), intent(in)  :: xinc
         real (kind=dp) , dimension(incidenceDesc%ntnivmax,incidenceDesc%ntnivmax), intent(out) :: XX

         real (kind=dp)       , dimension(incidenceDesc%dataset%nkd,incidenceDesc%ntniv) :: temp
         
       !  print *,'-->',ntniv
         temp = xinc(:incidenceDesc%dataset%nkd,:incidenceDesc%ntniv)
         !
         !  construction de X'X
         !
         XX=0.d0
         XX(:incidenceDesc%ntniv,:incidenceDesc%ntniv)=matmul(transpose(temp),temp)

      end subroutine model_XT_X
!!***

!!****f* m_qtlmap_incidence/set_VInv
!!  NAME
!!    set_VInv
!!  DESCRIPTION
!!  Build V and VInv diagonal matrix with :
!!    V(k,k)    = CD(i,j,k) / var(i)^2
!!    VInv(k,k) = var(i)^2 / CD(i,j,k)
!!                                        i sire,j dam ,k progeny
!!
!!  INPUTS
!!    incidenceDesc       : description of the incidence matrix
!!  OUTPUTS
!!    V                   : CD(i,j,k) / var(i)^2 , i sire,j dam ,k progeny.
!!    VInv                : var(i)^2 / CD(i,j,k) , i sire,j dam ,k progeny.
!!  NOTES
!!    Context heteroscedastic
!!  SOURCE
      subroutine set_VInv( incidenceDesc,sig, V,VInv )
         type(INCIDENCE_TYPE)          , intent(in)      :: incidenceDesc
         real (kind=dp) , dimension(np), intent(in)      :: sig
         real (kind=dp) , dimension(incidenceDesc%dataset%nkd,incidenceDesc%dataset%nkd), intent(out)  :: V,VInv
         integer :: l,ip,jm,kd
         real (kind=dp) :: sigsquare
         VInv=0.d0

         do ip=1,np
          sigsquare = sig(ip)*sig(ip)

          do kd=incidenceDesc%dataset%lSires(ip)%half_sib%firstKd,incidenceDesc%dataset%lSires(ip)%half_sib%lastKd
           VInv(kd,kd)=incidenceDesc%dataset%cd(1,kd)/sigsquare
           V(kd,kd)=sigsquare/incidenceDesc%dataset%cd(1,kd)
          end do

         end do

      end subroutine set_VInv
!!***

!!****f* m_qtlmap_incidence/model_XT_V_X
!!  NAME
!!    model_XT_V_X
!!  DESCRIPTION
!!  Build X' V-1 X :
!!
!!   X : the contingence matrix
!!   V-1 : diagonal matrix / VInv(k,k) = var(i)^2 / CD(i,j,k) , i sire,j dam ,k progeny
!!  INPUTS
!!    incidenceDesc       : the description of the contingence matrix
!!    xinc                : the contingence matrix
!!    VInv                : diagonal matrix / VInv(k,k) = var(i)^2 / CD(i,j,k) , i sire,j dam ,k progeny
!!  OUTPUTS
!!    XVX                 : X' V-1 X
!!
!!  NOTES
!!    Context heteroscedastic
!!  SOURCE
      subroutine model_XT_V_X(xinc,incidenceDesc,VInv,XVX)
         type(INCIDENCE_TYPE)                         ,intent(in)  :: incidenceDesc
         real (kind=dp)       , dimension(nd,incidenceDesc%ntnivmax), intent(in) :: xinc
         real (kind=dp)       , dimension(incidenceDesc%dataset%nkd,incidenceDesc%dataset%nkd) , intent(in):: VInv
         real (kind=dp) , dimension(incidenceDesc%ntnivmax,incidenceDesc%ntnivmax), intent(out)    :: XVX


         real (kind=dp) , dimension(incidenceDesc%ntniv,incidenceDesc%dataset%nkd)    :: XVXTemp
         real (kind=dp) , dimension(incidenceDesc%dataset%nkd,incidenceDesc%ntniv)    :: temp

         integer :: i,j,k
         !
         !  construction de X' V-1 X
         !
         XVX=0.d0
         !stop
         temp = xinc(:incidenceDesc%dataset%nkd,:incidenceDesc%ntniv)

         XVXTemp=matmul(transpose(temp),VInv)
         XVX(:incidenceDesc%ntniv,:incidenceDesc%ntniv)=matmul(XVXTemp,temp)

      end subroutine model_XT_V_X
!!***


!!****f* m_qtlmap_incidence/estim_cholesky
!!  NAME
!!    estim_cholesky
!!  DESCRIPTION
!!      Estimability with a decomposition of cholesky method
!!
!!      X'.X = L . L*
!!      where L is a lower triangular matrix with strictly positive diagonal entries, and L* denotes the conjugate transpose of L
!!
!!      output : incidenceDesc%vecsol  : array of boolean : true if estim , false otherwise
!!              incidenceDesc%nbnivest : number of effect to estimate according the decompostion
!!  INPUTS
!!    XX                         : X'.X
!!    sizeTriangComp             : size of the vector result
!!  OUTPUTS
!!    incidenceDesc%vecsol       : a logical array with the estimability of each level incidence matrix
!!    incidenceDesc%nbnivest     : the number of estimable level
!!    triangComp                 : X' V-1 X
!!
!!  NOTES
!!    triangComp is using in the resolution for the linear context
!!
!!  SOURCE
       subroutine estim_cholesky(XX,incidenceDesc,sizeTriangComp,triangComp)
         type(INCIDENCE_TYPE)                                                     , intent(inout) :: incidenceDesc
         real (kind=dp) , dimension(incidenceDesc%ntnivmax,incidenceDesc%ntnivmax), intent(in)    :: XX
         integer                                                                   ,intent(in)    :: sizeTriangComp
         real (kind=dp) , dimension(sizeTriangComp,sizeTriangComp),intent(out) :: triangComp
         integer                                 :: i,j,k,ief,jef
         real (kind=dp) , dimension(incidenceDesc%ntniv,incidenceDesc%ntniv) :: triang

         !  application de la d�composition de choleski pour d�terminer les contraintes
         !
         !  Dans le vecteur vecsol les effets � estimer sont indiqu�s � TRUE et ceux
         !  qui ne sont pas estimables � FALSE
         !
         !  La matrice triang est la matrice triangulaire sup�rieure M telle que
         !  M'M = X'X
         !
         if ( sizeTriangComp < incidenceDesc%ntniv ) then
           call stop_application("Devel error in m_incidence:estim_cholesky")
         end if

         incidenceDesc%vecsol=.true.
         triang=0.d0
         triangComp=0.d0

         do j=1,incidenceDesc%ntniv
           triang(j,j)=xx(j,j)
           do k=1,j-1
            triang(j,j)=triang(j,j)- triang(j,k)*triang(j,k)
           end do
           if(triang(j,j).gt. SEUIL_CHO) then
             incidenceDesc%nbnivest=incidenceDesc%nbnivest+1
             triang(j,j)=dsqrt(triang(j,j))
             do i=j+1,incidenceDesc%ntniv
               triang(i,j)=xx(i,j)
               do k=1,j-1
                triang(i,j)=triang(i,j)-triang(i,k)*triang(j,k)
               end do
               triang(i,j)=triang(i,j)/triang(j,j)
               triang(j,i)=triang(i,j)
             end do
           else
            incidenceDesc%vecsol(j)=.false.
           end if
         end do

         i=0
         incidenceDesc%nbnivest=0
         do ief=1,incidenceDesc%ntniv
          if(incidenceDesc%vecsol(ief))then
           incidenceDesc%nbnivest=incidenceDesc%nbnivest+1
           i=i+1
           j=0
           do jef=1,incidenceDesc%ntniv
             if (incidenceDesc%vecsol(jef)) then
               j=j+1
               triangComp(i,j) = triang(ief,jef)
             end if
           end do
       !    corniv(ief)=nbnivest
          end if
         end do

       end subroutine estim_cholesky
!!***


!!****f* m_qtlmap_incidence/set_MR
!!  NAME
!!    set_MR
!!  DESCRIPTION
!!     reduction Matrix :
!!       Mr = Xr' Vestim -1 Xr
!!
!!  INPUTS
!!    XX                         : X'.X
!!    incidenceDesc              : the description of contingence matrix
!!    sizeTriangComp             : size of the vector result
!!  OUTPUTS
!!    Mr                         : the number of estimable level
!!
!!  NOTES
!!    This function are not using....Mr was using for the resolution of Bestim. We are
!!    using the LU resolution (with L  lower triangular matrix from cholesky decomposition)
!!
!!  SOURCE
        subroutine set_MR(XX,incidenceDesc,Mr)
         type(INCIDENCE_TYPE)                         , intent(in)    :: incidenceDesc
         real (kind=dp) , dimension(incidenceDesc%ntnivmax,incidenceDesc%ntnivmax), intent(in)    :: XX
         real (kind=dp) , dimension(incidenceDesc%ntnivmax,incidenceDesc%ntnivmax) , intent(out)  :: Mr

         integer :: ki,kj,i,j,ix
         !
         !  compactage de triang
         !
         Mr=0.d0
         ki=0

         do i=1,incidenceDesc%ntniv
           if(incidenceDesc%vecsol(i)) then
             ki=ki+1
             kj=0
             do j=1,incidenceDesc%ntniv
              if(incidenceDesc%vecsol(j))then
               kj=kj+1
               Mr(ki,kj)=XX(i,j)
              end if
             end do
           end if
         end do

       end subroutine set_MR
!!***

!!****f* m_qtlmap_incidence/set_RHS
!!  NAME
!!    set_RHS
!!  DESCRIPTION
!!   Build reduction of matrix : RHS = Xr'  Xr
!!  INPUTS
!!   incidenceDesc              : the description of contingence matrix
!!   xincreduit                 : the reduction of the contingence matrix
!!  OUTPUTS
!!   RHS                        : Xr'  Xr
!!  NOTES
!!
!!  SOURCE
       subroutine set_RHS(xincreduit,incidenceDesc,RHS)
         type(INCIDENCE_TYPE)                         , intent(in) :: incidenceDesc
         real (kind=dp) , dimension(nd,incidenceDesc%ntnivmax), intent(in)       :: xincreduit
         real (kind=dp) , dimension(incidenceDesc%ntnivmax) , intent(out)        :: RHS

         real (kind=dp) , dimension(incidenceDesc%dataset%nkd,incidenceDesc%nbnivest) :: temp

         RHS=0.d0
         temp = xincreduit(:incidenceDesc%dataset%nkd,:incidenceDesc%nbnivest)
         RHS(:incidenceDesc%nbnivest)=matmul(transpose(temp),incidenceDesc%dataset%Y(1,:))
       end subroutine set_RHS
!!***

!!****f* m_qtlmap_incidence/set_RHS_V
!!  NAME
!!    set_RHS
!!  DESCRIPTION
!!   Let the diagonal matrix V -1 = var(i)^2 / CD(i,j,k) , i sire,j dam ,k progeny.
!!
!!   Build reduction of matrix : RHS = Xr' V -1  Xr
!!
!!  INPUTS
!!   incidenceDesc              : the description of contingence matrix
!!   xincreduit                 : the reduction of the contingence matrix
!!   VInv                       : var(i)^2 / CD(i,j,k) , i sire,j dam ,k progeny.
!!  OUTPUTS
!!   RHS                        : Xr' V -1  Xr
!!  NOTES
!!
!!  SOURCE
       subroutine set_RHS_V(xincreduit,VInv,incidenceDesc,RHS)
         type(INCIDENCE_TYPE)                         , intent(in) :: incidenceDesc
         real (kind=dp) , dimension(nd,incidenceDesc%ntnivmax), intent(in)       :: xincreduit
         real (kind=dp) , dimension(incidenceDesc%dataset%nkd,incidenceDesc%dataset%nkd) , intent(in):: VInv
         real (kind=dp) , dimension(incidenceDesc%ntnivmax) , intent(out)        :: RHS

         real (kind=dp) , dimension(incidenceDesc%dataset%nkd,incidenceDesc%nbnivest) :: temp
         real (kind=dp)       , dimension(incidenceDesc%nbnivest,incidenceDesc%dataset%nkd) :: RHSTemp
         integer :: i

         RHS=0.d0
         temp = xincreduit(:incidenceDesc%dataset%nkd,:incidenceDesc%nbnivest)

         RHSTemp=matmul(transpose(temp),VInv)
         RHS(:incidenceDesc%nbnivest)=matmul(RHSTemp,incidenceDesc%dataset%Y(1,:))
       end subroutine set_RHS_V
!!***

!!****f* m_qtlmap_incidence/set_corrxinc
!!  NAME
!!    set_corrxinc
!!  DESCRIPTION
!!  remove parameter which are not estimable from the original contingence matrix
!!
!!  Sample
!!
!!
!!         ( 1 1 0 )         ( T )          ( 1 1 )
!!   X'X = ( 1 1 0 )  Vecsol ( T )  ==>     ( 1 1 )
!!         ( 1 0 1 )         ( F )          ( 1 0 )
!!
!!  INPUTS
!!   incidenceDesc              : the description of contingence matrix
!!   xinc                       : the contingence matrix
!!  OUTPUTS
!!   XXOUT                      : xinc with column parameter not estimable removed
!!
!!  NOTES
!!
!!  SOURCE
       subroutine set_corrxinc(Xinc,incidenceDesc,XXOUT)
         type(INCIDENCE_TYPE)                         , intent(inout) :: incidenceDesc
         real (kind=dp) , dimension(:,:), intent(in)          :: Xinc
         real (kind=dp) , dimension(nd,incidenceDesc%ntnivmax) , intent(out)        :: XXOUT
         integer :: i,inb

         inb=0
         incidenceDesc%corr_niv_nivb=-1
         do i=1,incidenceDesc%ntniv
           if(incidenceDesc%vecsol(i)) then
            inb=inb+1
            incidenceDesc%corr_niv_nivb(i)=inb
            XXOUT(:,inb)=Xinc(:,i)
           end if
         end do

       end subroutine set_corrxinc
!!***

!!****f* m_qtlmap_incidence/debug_write_incidence
!!  NAME
!!    debug_write_incidence
!!  DESCRIPTION
!!    Print the contingence matrix and additionnal information.
!!    The user have the possibility to print a part of the contingence matrix
!!
!!  INPUTS
!!   incidenceDesc              : the description of contingence matrix
!!   xinc                       : the contingence matrix
!!  NOTES
!!
!!***
     subroutine debug_write_incidence(xinc,incidenceDesc)
       type(INCIDENCE_TYPE) , intent(in)                    :: incidenceDesc
       real (kind=dp) , dimension(nd,incidenceDesc%ntnivmax) , intent(in) :: xinc

       integer  :: i,maxPrint,kd,j,start,end,ip,jm,jjm
       character(len=500)                        :: mf1,myfmt,myfmt2
       character(len=10) :: c,c2
       logical ::is_ok
    !   print *,trim(additional_info)
       print *,""
       print *," **** SUMMARY **** "
       print *,' NAME             ', ' | START INDEX', ' | END INDEX'

       print *,"nombre de niveau         :",incidenceDesc%ntniv
       print *,"nombre de niveau estime  :",incidenceDesc%nbnivest
       print *,"interaction effet fix-qtl:",incidenceDesc%ntlev
       if ( associated (incidenceDesc%ntniv_qtlsires) ) then
         print *,"indice des qtls pere     :",incidenceDesc%ntniv_qtlsires
       end if
       if ( associated(incidenceDesc%ntniv_qtldams) ) then
          print *,"indice des qtls mere     :",incidenceDesc%ntniv_qtldams
       end if
       print *,"famille de pere avec indice de descendant"
       do ip=1,np
          jjm=0
          do jm=nmp(ip)+1,nmp(ip+1)
            jjm=jjm+1
            print *,"ip:",ip," jm:",jm," kd1:",incidenceDesc%dataset%lSires(ip)%full_sib(jjm)%firstKd,&
            " kd2:",incidenceDesc%dataset%lSires(ip)%full_sib(jjm)%lastKd,'( ndm:',ndm(jm)+1,ndm(jm+1),')'
          end do
       end do

       !for each effect in the incidence
       do i=1,incidenceDesc%nteff
          print *,"--",i,'---'
          write (*,FMT="("//trim(str(incidenceDesc%nteff))//"(a22,i4,i4))") &
          trim(incidenceDesc%desc(i)%name), incidenceDesc%desc(i)%start,incidenceDesc%desc(i)%end
          if ( incidenceDesc%desc(i)%haveSubDesc) then
             do j=1,size(incidenceDesc%desc(i)%listSubDesc)
               write (*,*) "   **sub**   ",incidenceDesc%desc(i)%listSubDesc(j)%name,&
                                           incidenceDesc%desc(i)%listSubDesc(j)%start,&
                                           incidenceDesc%desc(i)%listSubDesc(j)%end, &
                                           incidenceDesc%vecsol(incidenceDesc%desc(i)%listSubDesc(j)%start)
             end do
          end if
       end do

       !header
       myfmt="(i5,':',1x"
       do i=1,incidenceDesc%nteff
          myfmt=trim(myfmt)//",'|',f5.1"
          if ( incidenceDesc%desc(i)%end-incidenceDesc%desc(i)%start > 0) then
            myfmt=trim(myfmt)//','//trim(str(incidenceDesc%desc(i)%end-incidenceDesc%desc(i)%start))//'(f5.1)'
          else
            if ( incidenceDesc%desc(i)%end-incidenceDesc%desc(i)%start < 0 ) then
              print *,"**ERREUR** END PLUS PETIT QUE START"
              print *,incidenceDesc%desc(i)%name,incidenceDesc%desc(i)%start,incidenceDesc%desc(i)%end
              stop
            end if
          end if
       end do
       myfmt=trim(myfmt)//")"

       do
         write (*,*) "size xinc  (nkd):",incidenceDesc%dataset%nkd
         write (*,*) "[0-9999] [1-9999] : indice start et indice end de la matrice d incidence "
         write (*,*) "c: sortir "
         read (*,fmt="(a12)") c
         if (trim(c) == "c") exit
         start=1
         end=size(xinc,1)
         c2=trim(next_word(c,is_ok))
         if (is_ok) start=get_int(c2)
         c2=trim(next_word(c,is_ok))
         if (is_ok) end=get_int(c2)
         start=max(1,start)
         end=min(end,size(xinc,1))
         write (*,*) "INDICE",('|'//trim(incidenceDesc%desc(i)%name),i=1,incidenceDesc%nteff)
         write (*,*) "------",(('-',j=1,len(trim(incidenceDesc%desc(i)%name))+1),i=1,incidenceDesc%nteff)

         do kd=start,end
           write (*,FMT=myfmt) kd,(xinc(kd,i),i=1,incidenceDesc%ntniv)
         end do
         write (*,*) "..."
       end do

     end subroutine debug_write_incidence

!!****f* m_qtlmap_incidence/set_solution
!!  NAME
!!    set_solution
!!  DESCRIPTION
!!    Fill incsol the solution from the estimimation give in parameters
!!  INPUTS
!!    hypothesis      : the hypothesis
!!    workstruct      :
!!    sigsquareEstime : residual variance
!!    Bestim          : solution
!!    incidenceDesc   : description of the incidence
!!    lennteffQtl     : length of listnteffQtl
!!    listnteffQtl    : index of each qtl effect inside the contingence matrix
!!  OUTPUTS
!!    incsol          : description of the solution
!!  NOTES
!!
!!  SOURCE
    subroutine set_solution(hypothesis,workstruct,sigsquareEstime,Bestim,incidenceDesc,incsol,lennteffQtl,listnteffQtl)
       integer                        , intent(in)          :: hypothesis
       type(INCIDENCE_GEN_STRUCT)            ,intent(in)    :: workstruct
       real (kind=dp) , dimension(np)       , intent(in)    :: sigsquareEstime
       type(INCIDENCE_TYPE) , intent(in)                    :: incidenceDesc
       real (kind=dp) , dimension(incidenceDesc%ntnivmax) , intent(in)    :: Bestim      ! estimation of parameter on the maximum finded
       type(TYPE_INCIDENCE_SOLUTION)      ,intent(inout)    :: incsol
       integer                               , intent(in)   ::lennteffQtl
       integer     , dimension(lennteffQtl)  , intent(in)   ::listnteffQtl
       integer :: i,j,k,ip,maxNbPar,g,nb,INDEX,ntniv,nt

       incsol%hypothesis=hypothesis
       call log_mess( "" , VERBOSE_DEF)
       call log_mess( "-------------------------------", VERBOSE_DEF)
       call log_mess( "Estimation of parameters under "//trim(str(incsol%hypothesis)), VERBOSE_DEF)
       call log_mess( "-------------------------------", VERBOSE_DEF)
       call log_mess( "rank of incidence:"//trim(str(incidenceDesc%ntniv)), VERBOSE_DEF)
       call log_mess( "rank max to resolve :"//trim(str(incidenceDesc%ntnivmax)), VERBOSE_DEF)

       allocate (incsol%sig(1,np))
       allocate (incsol%groupeName(incidenceDesc%nteff))
       allocate (incsol%eqtl_print(incidenceDesc%nteff))
       allocate (incsol%nbParameterGroup(incidenceDesc%nteff))
       if (hypothesis > 0) then
         allocate (incsol%qtl_groupeName(1,hypothesis))

         if ( lennteffQtl < hypothesis ) then
           call stop_application("Devel error : bad init of array ** listnteff **")
         end if

         do i=1,hypothesis
           incsol%qtl_groupeName(1,i)=listnteffQtl(i)
         end do

       end if

       maxNbPar=1
       do i=1,incidenceDesc%nteff
         if ( incidenceDesc%desc(i)%haveSubDesc) then
           nb=0
           do j=1,size(incidenceDesc%desc(i)%listSubDesc)
              nb = nb + incidenceDesc%desc(i)%listSubDesc(j)%end - incidenceDesc%desc(i)%listSubDesc(j)%start + 1
           end do
           maxNbPar = max(maxNbPar,nb)
         end if
       end do

       allocate (incsol%parameterName(incidenceDesc%nteff,maxNbPar))
       allocate (incsol%paramaterValue(incidenceDesc%nteff,maxNbPar))
       allocate (incsol%parameterVecsol(incidenceDesc%nteff,maxNbPar))
       allocate (incsol%parameterPrecis(incidenceDesc%nteff,maxNbPar))

       do ip=1,np
        incsol%sig(1,ip)=sqrt(sigsquareEstime(ip))*sigt(incidenceDesc%ic)
       end do

       nt=0
       do i=1,incidenceDesc%nteff
          incsol%eqtl_print(i) = incidenceDesc%eqtl_print(i)
          incsol%groupeName(i) = incidenceDesc%desc(i)%name
          if ( incidenceDesc%desc(i)%haveSubDesc) then
            incsol%nbParameterGroup(i)=0
            do j=1,size(incidenceDesc%desc(i)%listSubDesc)
              INDEX=0
              do k=incidenceDesc%desc(i)%listSubDesc(j)%start,incidenceDesc%desc(i)%listSubDesc(j)%end
                 INDEX=INDEX+1
                 incsol%nbParameterGroup(i) = incsol%nbParameterGroup(i) +1
                 g=incsol%nbParameterGroup(i)
                 incsol%parameterName(i,g)=incidenceDesc%desc(i)%listSubDesc(j)%name//" "//adjustl(STR(INDEX))
                 ntniv=incidenceDesc%desc(i)%listSubDesc(j)%start+INDEX -1
                 incsol%parameterVecsol(i,g)=incidenceDesc%vecsol(ntniv)
                 if ( incsol%parameterVecsol(i,g) ) then
                    nt = nt + 1
                    incsol%paramaterValue(i,g)= Bestim(nt)*sigt(incidenceDesc%ic)
                    incsol%parameterPrecis(i,g) = incidenceDesc%precis(ntniv)
                 end if
              end do
            end do
          else
            incsol%nbParameterGroup(i)=1
            incsol%parameterName(i,1)=incidenceDesc%desc(i)%name
            incsol%parameterVecsol(i,1)=incidenceDesc%vecsol(incidenceDesc%desc(i)%start)
            if ( incsol%parameterVecsol(i,1) ) then
              nt = nt + 1
              incsol%paramaterValue(i,1)  = Bestim(nt)*sigt(incidenceDesc%ic)
              incsol%parameterPrecis(i,1) = incidenceDesc%precis(incidenceDesc%desc(i)%start)
            end if
          end if
       end do

       !specific treatment for general mean
       if ( workstruct%eff_general_mean > 0 ) then
        if ( incsol%parameterVecsol(workstruct%eff_general_mean,1) ) then
         incsol%paramaterValue(workstruct%eff_general_mean,1)  = &
         incsol%paramaterValue(workstruct%eff_general_mean,1)+xmut(incidenceDesc%ic)
        end if
       end if
    end subroutine set_solution
!!***


!!****f* m_qtlmap_incidence/build_incidence_matrix
!!  NAME
!!    build_incidence_matrix
!!  DESCRIPTION
!!    Build the contingence matrix
!!  INPUTS
!!    hypothesis      : the hypothesis
!!    workstruct      :
!!    curPos          : the initial position
!!    meff            : index of the fixed effect to no add
!!    mcov            : index of the covariate to no add
!!    mint            : index of the fixed effect-qtl interaction to not add
!!    numqtl          : number of qtl to test confusion with the mint interaction
!!  OUTPUTS
!!    incidenceDesc   : description of the incidence
!!    xinc            : the contingence matrix to build
!!  NOTES
!!
!!  SOURCE
    subroutine build_incidence_matrix(workstruct,curPos,xinc,incidenceDesc,meff,mcov,mint,numqtl)
       integer                                 ,intent(in)       :: meff,mcov,mint,numqtl
       type(POSITION_LRT_INCIDENCE)            ,intent(in)       :: curPos
       type(INCIDENCE_GEN_STRUCT)              ,intent(inout)    :: workstruct
       type(INCIDENCE_TYPE)                    ,intent(inout)    :: incidenceDesc
       real (kind=dp) , dimension(nd,incidenceDesc%ntnivmax) ,intent(inout)    :: Xinc

       !local
       integer :: i,ic2,mint2

       SELECT CASE(workstruct%type_incidence)
        CASE (INCIDENCE_TYPE_CLASSIC,INCIDENCE_INTERACTION)
           !add general mean to estim
           if (natureY(incidenceDesc%ic)/='i') then
            call add_general_mean(xinc,incidenceDesc)
            workstruct%eff_general_mean=1
            if (workstruct%nqtl>0) workstruct%listnteff(1)=2
           else
            workstruct%eff_general_mean=-1
            if (workstruct%nqtl>0) workstruct%listnteff(1)=1
           end if
           do i=1,workstruct%nqtl
            mint2=0
            !add qtl effect/interaction at position n to estim
            if ( numqtl == i ) mint2=mint
            call add_qtleffect(workstruct%listnteff(i),i,curPos%listChr(i),curPos%listN(i),&
            xinc,incidenceDesc,mint2,workstruct%linear)
            if (i<workstruct%nqtl) workstruct%listnteff(i+1)=incidenceDesc%nteff+1
           end do
          !add polygenic effect
          call add_polygenic_effect(xinc,incidenceDesc)
          !add fixed effect and covariate
          call add_effcov(xinc,incidenceDesc,meff,mcov)

        CASE(INCIDENCE_TRAIT_COV)
          if (natureY(incidenceDesc%ic)/='r') then
            call stop_application(" ** manage only continue trait ** ")
          end if
          call add_general_mean(xinc,incidenceDesc)
          workstruct%eff_general_mean=1
          if (workstruct%nqtl>0) workstruct%listnteff(1)=2
          do i=1,workstruct%nqtl
            mint2=0
            !add qtl effect/interaction at position n to estim
            if ( numqtl == i ) mint2=mint
            call add_qtleffect(workstruct%listnteff(i),i,curPos%listChr(i),curPos%listN(i),&
            xinc,incidenceDesc,mint2,workstruct%linear)
            if (i<workstruct%nqtl) workstruct%listnteff(i+1)=incidenceDesc%nteff+1
           end do
           !add polygenic effect
           call add_polygenic_effect(xinc,incidenceDesc)
           !add others traits as covariate
           call add_other_continue_traits_as_cov(xinc,incidenceDesc,mcov)
           !add fixed effect and covariate
          call add_effcov(xinc,incidenceDesc,meff,mcov)

        !  call debug_write_incidence(xinc,incidenceDesc)

        CASE (INCIDENCE_TYPE_LDLA)
          !add general mean to estim
           call add_general_mean(xinc,incidenceDesc)
           workstruct%eff_general_mean=1
          !haplotype effect
           do i=1,workstruct%nqtl
            workstruct%listnteffhap(i)=i+1
            call add_haplotype_effect(workstruct%listnteffhap(i),i,curPos%listChr(i),curPos%listN(i),&
              xinc,incidenceDesc,workstruct%hdam,.true.)
           end do
          !qtl effect
           do i=1,workstruct%nqtl
            mint2=0
            workstruct%listnteff(i)=incidenceDesc%nteff+1
            !add qtl effect/interaction at position n to estim
            if ( numqtl == i ) mint2=mint
            call add_qtleffect(workstruct%listnteff(i),i,curPos%listChr(i),curPos%listN(i),&
            xinc,incidenceDesc,mint2,workstruct%linear)
           end do
          !add polygenic effect
          call add_polygenic_effect(xinc,incidenceDesc)
          !add fixed effect and covariate
          call add_effcov(xinc,incidenceDesc,meff,mcov)
        CASE (INCIDENCE_TYPE_LD)
           !add general mean to estim
           if (natureY(incidenceDesc%ic)/='i') then
           call add_general_mean(xinc,incidenceDesc)
            workstruct%eff_general_mean=1
           do i=1,workstruct%nqtl
            workstruct%listnteffhap(i)=i+1
            call add_haplotype_effect(workstruct%listnteffhap(i),i,curPos%listChr(i),curPos%listN(i),&
              xinc,incidenceDesc,workstruct%hdam,.true.)
           end do
           !add polygenic effect
           call add_polygenic_effect(xinc,incidenceDesc)
           !add fixed effect and covariate
           call add_effcov(xinc,incidenceDesc,meff,mcov)
           end if

         CASE DEFAULT
            call stop_application("build_incidence_matrix:Unknown data type description for model ["//&
            trim(str(workstruct%type_incidence))//"]")
        END SELECT
    end subroutine build_incidence_matrix
!!***


!!****f* m_qtlmap_incidence/opti_0qtl
!!  NAME
!!    opti_0qtl
!!  DESCRIPTION
!!    compute the LRT under H0 1 trait
!!  INPUTS
!!    ic              : index of the trait
!!    FUNCT_MODEL     : the function for the resolution (model_lin_h0,model_lin_hn,model_optim_h0,model_optim_hn)
!!  OUTPUTS
!!    incsol          : the solution of the estimation
!!    workstruct      : residual variance and maximum Likelihood under H0
!!  NOTES
!!
!!  SOURCE
      subroutine opti_0qtl(    ic,       &  ! the trait
                               incsol   ,       &  !
                               workstruct, &
                              FUNCT_MODEL)  ! function with a model specific
      integer                                 ,intent(in)       :: ic
      type(INCIDENCE_GEN_STRUCT)              ,intent(inout)    :: workstruct
      type(TYPE_INCIDENCE_SOLUTION)           ,intent(out)      :: incsol
      external :: FUNCT_MODEL   ! function with a model specific

      integer :: nkd,ip,i,listnteff(1)

      real (kind=dp) ,dimension(:,:),pointer      :: xinc,xxx
      real (kind=dp) , dimension(:) ,pointer      :: Bestim
      real (kind=dp) , dimension(np)              :: sigsquareEstime
      type(INCIDENCE_TYPE)                        :: incidenceDesc
      type(POSITION_LRT_INCIDENCE)               :: curPos

      !initialisation of contingence matrix
      call init_incidence(ic,0,incidenceDesc,workstruct%type_incidence)
      workstruct%nqtl=0
      allocate (Bestim(incidenceDesc%ntnivmax))
      allocate (xinc(nd,incidenceDesc%ntnivmax))
      xinc=0.d0
      Bestim=0.d0
      call build_incidence_matrix(workstruct,curPos,xinc,incidenceDesc,0,0,0,0)
      !call debug_write_incidence(xinc,incidenceDesc)
      !call model analysis
      call FUNCT_MODEL(xinc,incidenceDesc,workstruct,sigsquareEstime,Bestim,.true.)
      call set_solution(0,workstruct,sigsquareEstime,Bestim,incidenceDesc,incsol,1,listnteff)
      workstruct%sigsquare(1,:,1)=sigsquareEstime
      workstruct%fnqtl(1)=incidenceDesc%fmax


      call end_incidence(incidenceDesc)
      deallocate (xinc)
      deallocate (Bestim)

      end subroutine opti_0qtl
!!***


!!****f* m_qtlmap_incidence/opti_nqtl
!!  NAME
!!    opti_nqtl
!!  DESCRIPTION
!!    compute the LRT under H(nqtl) 1 trait    nqtl>=1
!!  INPUTS
!!    nqtl            : the hypothesis
!!    ic              : index of the trait
!!    workstruct      : residual variance and maximum Likelihood under H(nqtl-1)
!!    FUNCT_MODEL     : the function for the resolution (model_lin_h0,model_lin_hn,model_optim_h0,model_optim_hn)
!!  OUTPUTS
!!    workstruct      : residual variance and maximum Likelihood under H(nqtl)
!!    incsol          : the solution of the estimation at the maximum finded
!!    lrtsol          : LRT : curves and maximum under H(nqtl)
!!  NOTES
!!    use the iterative scan gen_opti_nqtl function
!!  SOURCE
     subroutine opti_nqtl(      nqtl,      &  ! under hypothesis NQTL
                                        ic,       &  ! the trait
                                workStruct,       &  ! variance / likelihood estimated under NQTL-1 hypothesis
                                 incsol   ,       &  ! incidence solution (estimation of each effect)
                                 lrtsol   ,       &  ! maximum lrt finding at a position
                               FUNCT_MODEL         ) ! function with a model specific

      integer                                 ,intent(in)          :: nqtl,ic
      type(INCIDENCE_GEN_STRUCT)              ,intent(inout)       :: workstruct
      type(TYPE_INCIDENCE_SOLUTION)           ,intent(out)         :: incsol
      type(TYPE_LRT_SOLUTION)                 ,intent(out)         :: lrtsol
      external                                                     :: FUNCT_MODEL


      integer :: i

      real (kind=dp) ,dimension(:,:),pointer      :: xinc ,xxx    ! incidence matrix and temp for testing nuisance effect
      real (kind=dp) , dimension(:) ,pointer      :: Bestim       ! solution of the estimation
      real (kind=dp) , dimension(np)              :: sigsquareEstime
      type(INCIDENCE_TYPE)                        :: incidenceDesc ! description incidence matrix
      type(POSITION_LRT_INCIDENCE)                :: curPos        ! the position with additional info for the model
      integer :: n,j,chr
#ifdef MANAGE_CUBLAS
      integer                                         :: c,d,u,nsim,nGL,nLevelFix,nLevelVar,corrLevel(60),nteffS,nteffE,sizeF(np)
      integer                                         :: ip,kd1,kd2
      real           ,dimension(:,:)    ,pointer      :: xinc2 ! toutes les matrices d incidence
      type(INCIDENCE_TYPE)                        :: incidenceDesc2
      real           ,dimension(:,:)  ,pointer      :: myY,allLrt
      real           ,dimension(:),pointer          :: allmaxlrt
      integer        ,dimension(:),pointer          :: allmaxpos
      logical , dimension(:,:)          ,pointer      :: allvecsol
      real           , dimension(:,:)   ,pointer      :: isigsq
      real         , dimension(:,:,:)   ,pointer      :: allbestim,allosig
    !  real           , dimension(1,np)         ::
#endif

      if ( nqtl <= 0 ) then
        call stop_application("Error dev: can not call opti_nqtl_linear with nqtl:"//trim(str(nqtl)))
      end if

      !Position Allocation
      call init_position (nqtl,curPos)

      !initialisation of incidence matrix
      call init_incidence(ic,nqtl,incidenceDesc,workstruct%type_incidence)

      allocate (xinc(nd,incidenceDesc%ntnivmax))
      xinc=0.d0
      allocate (Bestim(incidenceDesc%ntnivmax))
      allocate (xxx(incidenceDesc%ntnivmax,incidenceDesc%ntnivmax))

      Bestim=0.d0
      do i=1,nqtl
         n=0
         j=i
         do chr=1,nchr
           n=n+get_ilong(chr)
           if (i<=n) exit
           j=1
         end do

         if (chr>nchr) call stop_application("Not enough sampling point to detect QTLs")

         curPos%listChr(i)=1 ! attention si pas assez d echantillonage on doit passer au chromosome suivant....
         curPos%listN(i)=i
         do while (absi(1,curPos%listN(i))<posi(1,1+longhap/2))
          curPos%listN(i)=curPos%listN(i)+1
         end do
      end do

      call build_incidence_matrix(workstruct,curPos,xinc,incidenceDesc,0,0,0,0)

    !  call debug_write_incidence(xinc,incidenceDesc)
      !call model analysis
     ! if ( workstruct%type_incidence  /= )
      call FUNCT_MODEL(xinc,incidenceDesc,curPos,workstruct,sigsquareEstime,Bestim,.false.,.false.,xxx,.false.)

      !initalizing lrtsolution
      call new(nqtl,lrtsol)

      lrtsol%lrtmax=curPos%lrt
      lrtsol%chrmax=curPos%listChr
      lrtsol%nxmax=curPos%listN

!      call gen_loop_opti_nqtl(1,nqtl,curPos,workstruct,xinc,&
 !     incidenceDesc,sigsquareEstime,Bestim,lrtsol,.true.,FUNCT_MODEL)
#ifdef MANAGE_CUBLAS
       nsim=1
       nGL = get_npo(1) ;

       call log_mess(" ******* CUDA  ********",INFO_DEF);
       allocate (myY(nsim,nd))
       allocate (allvecsol(nGL,incidenceDesc%ntnivmax))
       allocate (allbestim(nGL,nsim,incidenceDesc%ntnivmax))
       allocate (allosig(nGL,nsim,np))
       allocate (allLrt(nGL,nsim))
       allocate(allmaxlrt(nsim))
       allocate(allmaxpos(nsim))


       myY(1,:)=y(ic,:)
       !xinc2 = xinc
       nteffS=workstruct%listnteff(1)
       if ( count(estime(incidenceDesc%ic,:))>0) then
          nteffE = workstruct%listnteff(workstruct%nqtl)+1
       else
          nteffE = workstruct%listnteff(workstruct%nqtl)
       end if

       nLevelVar = incidenceDesc%desc(nteffE)%end - incidenceDesc%desc(nteffS)%start + 1
       nLevelFix = incidenceDesc%ntniv - nLevelVar


       !print *,incidenceDesc%ntniv,nLevelFix,nLevelVar

       allocate (xinc2(nd,nLevelFix+nLevelVar*nGL))
       xinc2=0.d0
       xinc2(:,1)=xinc(:,1)

       corrLevel(1)=0
       d=incidenceDesc%ntniv-nLevelVar
       u = 1

       do i=2,incidenceDesc%ntniv
          if ( i <= incidenceDesc%desc(nteffE)%end ) then
            corrLevel(i) = d
           ! print *,i,"-->",d
            xinc2(:,d+1)=xinc(:,i);
            d=d+1
          else
            corrLevel(i) = u
          !  print *,i,"-->",u
            xinc2(:,u+1)=xinc(:,i);
            u = u+1
          end if
       end do

       d=incidenceDesc%ntniv+1
       do c=1,nchr
        do n=2,get_npo(c)
         call change_qtleffect(workstruct%listnteff(1),1,c,n,xinc,incidenceDesc,0,.true.)
         do i=1,nLevelVar
            xinc2(:,d)=xinc(:,i+1);
            d=d+1
         end do
        end do
       end do

      allocate (isigsq(1,np))
      isigsq(1,:) = workstruct%sigsquare(1,:,1)
      do ip=1,np
           kd1=incidenceDesc%dataset%lSires(ip)%half_sib%firstKd
           kd2=incidenceDesc%dataset%lSires(ip)%half_sib%lastkd
           if ( kd2>kd1) then
           sizeF(ip) = kd2-kd1+1
           else
           sizeF(ip) = 0
           end if
      end do
  !     print *,xinc2(1,:100)

    !   print *,xinc2(:,1)
      ! do i=1,nLevelFix+(nLevelVar*3)
!         print *,xinc2(500,:nLevelFix)
!         print *,xinc2(500,nLevelFix+1:nLevelFix+nLevelVar)
!         print *,xinc2(500,nLevelFix+nLevelVar+1:nLevelFix+2*nLevelVar)
!         print *,xinc2(500,nLevelFix+nLevelVar*75+1:nLevelFix+76*nLevelVar)
      ! end do
      ! stop

       !call debug_write_incidence(xinc,incidenceDesc)
       call cuda_homoscedatic_resolv_genome(                       &
                                      nqtl,                        &
                                      isigsq,                      &
                                      nLevelFix,                   &
                                      nLevelVar,                   &
                                      corrLevel,                   &
                                      xinc2,                       &
                                      myY,                         &
                                      nsim,                        &
                                      nd,                          &
                                      incidenceDesc%dataset%nkd,   &
                                      np,                          &
                                      sizeF,                       &
                                      incidenceDesc%ntnivmax,      &
                                      nGL,                         &
                                      SEUIL_CHO,                   &
                                      allvecsol,                   &
                                      allbestim,                   &
                                      allosig,                     &
                                      allLrt,                      &
                                      allmaxlrt,                   &
                                      allmaxpos )
       print *,'Fortran SIG '
       print *,allosig

       print *,'Fortran LRT '
       print *,allLrt

       print *,'Fortran MAXLRT '
       print *,allmaxlrt

       print *,'Fortran MAXPOS '
       print *,allmaxpos

       deallocate (xinc2)
       deallocate (allvecsol)
       deallocate (allbestim)
       deallocate (allosig)
       deallocate (allLrt)
       deallocate (allmaxlrt)
       deallocate (allmaxpos)
       stop
#else
       call gen_opti_nqtl(nqtl,curPos,workstruct,xinc,incidenceDesc,sigsquareEstime,Bestim,lrtsol,FUNCT_MODEL)
#endif
      if (  workstruct%type_incidence == INCIDENCE_TYPE_LD .or.  workstruct%type_incidence == INCIDENCE_TYPE_LDLA ) then
       do i=1,nqtl
         call add_haplotype_effect(workstruct%listnteffhap(i),i,lrtsol%chrmax(i-1),lrtsol%nxmax(i-1),&
              xinc,incidenceDesc,workstruct%hdam,.false.)
       end do
      end if

      if (  workstruct%type_incidence /= INCIDENCE_TYPE_LD ) then
      ! Compute precision at the maximum LRT in position posx
       do i=1,nqtl
        call change_qtleffect(workstruct%listnteff(i),i,lrtsol%chrmax(i-1),lrtsol%nxmax(i-1),&
          xinc,incidenceDesc,0,workstruct%linear)
       end do
      end if

      do i=1,nqtl
       curPos%listChr(i)=lrtsol%chrmax(i-1)
       curPos%listN(i)=lrtsol%nxmax(i-1)
      end do
     ! call debug_write_incidence(xinc,incidenceDesc)
      call FUNCT_MODEL(xinc,incidenceDesc,curPos,workstruct,sigsquareEstime,Bestim,.true.,workstruct%performConfusion,xxx,.false.)
      if ( workstruct%performConfusion) then
         call confusion_type1(incidenceDesc,xxx,workstruct)
       end if

      if ( workstruct%performTestNuis ) then
        call test_nuisances(ic,incidenceDesc,lrtsol,curPos,workstruct,sigsquareEstime,FUNCT_MODEL)
      end if
      call set_solution(nqtl,workstruct,sigsquareEstime,Bestim,incidenceDesc,incsol,nqtl,workstruct%listnteff)

      if (size(workstruct%fnqtl,1)>nqtl) then
          workstruct%fnqtl(nqtl+1)=incidenceDesc%fmax
      end if

      if (size(workstruct%sigsquare,1)>nqtl) then
        workstruct%sigsquare(nqtl+1,:,1)=sigsquareEstime
      end if

      call end_incidence(incidenceDesc)
      deallocate (xinc)
      deallocate (xxx)
      deallocate (Bestim)
      call end_position(curPos)

      end subroutine opti_nqtl
!!***

!!****f* m_qtlmap_incidence/gen_opti_nqtl
!!  NAME
!!    gen_opti_nqtl
!!  DESCRIPTION
!!    scan the genome under the hypothesis nqtl
!!  INPUTS
!!    nqtl            : the hypothesis
!!    workstruct      : residual variance and maximum Likelihood under H(nqtl-1)
!!    FUNCT_MODEL     : the function for the resolution (model_lin_h0,model_lin_hn,model_optim_h0,model_optim_hn)
!!    xinc            : the contingence matrix
!!  OUTPUTS
!!    curPosMax          : maximum position finded
!!    workstruct         : residual variance and maximum Likelihood under H(nqtl)
!!    incidenceDescMax   : the description of the contingence matrix at the maximum position
!!    sigsquareEstimeMax : the residual variance
!!    BestimMax          : the solution
!!    lrtsol             : LRT : curves and maximum under H(nqtl)
!!  NOTES
!!
!!  SOURCE
     subroutine gen_opti_nqtl(      nqtl,      &
                                    curPosMax,    &
                                    workstruct,&
                                    xinc,      &
                                    incidenceDescMax, &
                                    sigsquareEstimeMax, &
                                    BestimMax,     &
                                    lrtsol,        &
                                    FUNCT_MODEL)
          integer                                 , intent(in)    :: nqtl  ! under hypothesis nqtl
          type(POSITION_LRT_INCIDENCE)             ,intent(inout) :: curPosMax
          type(INCIDENCE_GEN_STRUCT)              ,intent(inout)  :: workstruct
          type(INCIDENCE_TYPE) , intent(inout)                    :: incidenceDescMax  ! description of the contingence matrix
          real (kind=dp) , dimension(nd,incidenceDescMax%ntnivmax) , intent(inout) :: xinc  ! contingence matrix
          real (kind=dp) , dimension(np) , intent(inout)          :: sigsquareEstimeMax      ! estimation of parameter on the maximum finded
          real (kind=dp) , dimension(incidenceDescMax%ntnivmax) , intent(inout)    :: BestimMax      ! estimation of parameter on the maximum finded
          type(TYPE_LRT_SOLUTION)               ,intent(inout)    :: lrtsol
          external                                                :: FUNCT_MODEL


          real (kind=dp) , dimension(incidenceDescMax%ntnivmax)        :: Bestim
          real (kind=dp) , dimension(np)              :: sigsquareEstime

          integer :: iip,ifem,ii,chr,ip,iqtl,i,nlinReal
          logical hypotheseOne, hypotheseTwo,ok
          real (kind=dp) ,dimension(:,:),pointer      :: xxx

          integer :: nGL,nGLTotal,nGLFact,InGL,nlinMax,nlin,nmax(nqtl),chmax(nqtl),nlinValide
          integer       , dimension(:,:) ,allocatable :: lchr,lnpos
          real(kind=dp) , dimension(:,:) ,allocatable :: lrt_save,bestim_save,sigsq_save
          type(POSITION_LRT_INCIDENCE)             :: curPos
          type(INCIDENCE_TYPE)                     :: incidenceDesc
          logical                                  :: loopFalse

          hypotheseOne = ( lrtsol%nqtl == 1 )
          hypotheseTwo = ( lrtsol%nqtl == 2 )

          nGL = 0

          !Nombre de point sur le groupe de liaison
          !on cherche le nombre de point dependant de l hyp nqtl du numbre de point sur le groupe de liason

          do chr=1,nchr
              nGL=nGL + get_npo(chr)
          end do

          nGLTotal=1
          do iqtl=1,nqtl
            nGLTotal=nGLTotal*nGL
          end do

          curPosMax%listN(nqtl)=0
          curPosMax%listN(1:nqtl-1)=1
          curPosMax%listChr=1

          allocate (lchr(nGLTotal,nqtl))
          allocate (lnpos(nGLTotal,nqtl))

          nlinValide=0
          do nlin=1,nGLTotal
             i=nqtl
             ok=.false.
             do while ( (.not. ok) .and. (i>=1))
               curPosMax%listN(i) = curPosMax%listN(i)+1
               if ( curPosMax%listN(i) > get_npo(curPosMax%listChr(i)) ) then
                  if ( nchr > curPosMax%listChr(i) ) then
                    curPosMax%listN(i)=1
                    curPosMax%listChr(i)=curPosMax%listChr(i)+1
                    ok=.true.
                  else
                    curPosMax%listN(i)=1
                    curPosMax%listChr(i)=1
                    i=i-1
                  end if
               else
                  ok = .true.
               end if
             end do
             ok=.true.
             do iqtl=2,nqtl
               ok = ok .and. ( curPosMax%listN(iqtl-1)<=curPosMax%listN(iqtl) )
             end do

             if ( ok ) then
              nlinValide=nlinValide+1
              lchr(nlinValide,:)=curPosMax%listChr
              lnpos(nlinValide,:)=curPosMax%listN
             end if
          end do
          nGLTotal=nlinValide

          allocate (bestim_save(nGLTotal,incidenceDescMax%ntnivmax))
          allocate (sigsq_save(nGLTotal,np))
          allocate (lrt_save(nGLTotal,nqtl))
          lrt_save=0.d0
          !$OMP PARALLEL DEFAULT(SHARED)  FIRSTPRIVATE(xinc)  &
          !$OMP PRIVATE(curPos,i,iqtl,ok,sigsquareEstime,Bestim,xxx,ip,iip,ifem,incidenceDesc,loopFalse)
          call init_position (nqtl,curPos)
          allocate (xxx(incidenceDescMax%ntnivmax,incidenceDescMax%ntnivmax))
          !initialisation of contingence matrix
          call copy_incidence_desc (incidenceDescMax,incidenceDesc)

          curPos%lrt(nqtl)=0
          curPos%listN(nqtl)=0
          curPos%listN(1:nqtl-1)=1
          curPos%listChr=1

          !$OMP DO
          do nlin=1,nGLTotal
             call log_mess( trim(str((float(nlin)/float(nGLTotal))*100.d0))//"%", DEBUG_DEF )
             curPos%listN=lnpos(nlin,:)
             curPos%listChr=lchr(nlin,:)

             !traitement specifique pour le LD
             if ( workstruct%type_incidence == INCIDENCE_TYPE_LD .or.  workstruct%type_incidence == INCIDENCE_TYPE_LDLA) then
               i=1
               loopFalse = .false.
               do while (i<=nqtl .and. .not. loopFalse )
               if (absi(curPos%listChr(i),curPos%listN(i))<posi(curPos%listChr(i),1+longhap/2) ) &
                 loopFalse = .true.
               if (absi(curPos%listChr(i),curPos%listN(i))>posi(curPos%listChr(i),nmk(curPos%listChr(i))-longhap/2)) &
                 loopFalse = .true.
                i = i + 1
               end do

               if ( loopFalse ) cycle
               do i=1,nqtl
                 call add_haplotype_effect(workstruct%listnteffhap(i),i,curPos%listChr(i),curPos%listN(i),&
                  xinc,incidenceDesc,workstruct%hdam,.false.)
               end do
             end if

              if ( workstruct%type_incidence /= INCIDENCE_TYPE_LD ) then
               do iqtl=1,nqtl
                call change_qtleffect(workstruct%listnteff(iqtl),iqtl,curPos%listChr(iqtl),curPos%listN(iqtl),&
                                     xinc,incidenceDesc,0,workstruct%linear)
               end do
              end if

             ! call debug_write_incidence(xinc,incidenceDesc)

              call FUNCT_MODEL(xinc,incidenceDesc,curPos,workstruct,sigsquareEstime,Bestim,.false.,.false.,xxx,.false.)

              !save value
              bestim_save(nlin,:incidenceDesc%nbnivest)=Bestim(:incidenceDesc%nbnivest)
              sigsq_save(nlin,:np)=sigsquareEstime(:np)
              lrt_save(nlin,:)=curPos%lrt
             ! print *,curPos%listN,":",curPos%lrt(nqtl),'-->nlin:',nlin!,OMP_GET_THREAD_NUM()

              if ( hypotheseOne ) then
                lrtsol%lrt1(curPos%listChr(1),curPos%listN(1))=curPos%lrt(1)
                iip=1
                do ip=1,np
                  if ( incidenceDesc%vecsol(1+ip)) then
                    iip=iip+1
                    ! ATTENTION VALIDE DANS LE CAS SANS INTERACTION AVEC QTL
                    lrtsol%pater_eff(curPos%listChr(1),ip,curPos%listN(1))=Bestim(iip)
                  end if
                end do
                ifem=0
                do ii=1,nm
                  if ( estime(incidenceDesc%ic,ii)) then
                    ifem=ifem+1
                    if ( incidenceDesc%vecsol(np+1+ifem) ) then
                      iip=iip+1
                      ! ATTENTION VALIDE DANS LE CAS SANS INTERACTION AVEC QTL
                      lrtsol%mater_eff(curPos%listChr(1),ifem,curPos%listN(1))=Bestim(iip)
                    end if
                  end if
                end do
              end if
              if ( hypotheseTwo ) then
                lrtsol%lrt1_2(curPos%listChr(1),curPos%listChr(2),curPos%listN(1),curPos%listN(2))=curPos%lrt(2)
                lrtsol%lrt0_2(curPos%listChr(1),curPos%listChr(2),curPos%listN(1),curPos%listN(2))=curPos%lrt(1)
              end if
        !    end if !ok
         end do ! nlin
         !$OMP END DO
         call end_position (curPos)
         deallocate (xxx)
         call release_copy_incidence_desc(incidenceDesc)
         !$OMP END PARALLEL

         nlinMax=1
         lrtsol%lrtmax(0)=lrt_save(nlinMax,nqtl)
         nmax=1
         chmax=1

         call init_position (nqtl,curPos)

         curPos%listN(nqtl)=0
         curPos%listN(1:nqtl-1)=1
         curPos%listChr=1

         do nlin=1,nGLTotal
           curPos%listN=lnpos(nlin,:)
           curPos%listChr=lchr(nlin,:)

           if(lrtsol%lrtmax(nqtl-1) < lrt_save(nlin,nqtl)) then
             nmax=curPos%listN
             chmax=curPos%listChr
             nlinMax=nlin
             lrtsol%lrtmax=lrt_save(nlin,:)
           end if
         end do

         do iqtl=1,nqtl
            lrtsol%nxmax(iqtl-1)=nmax(iqtl)
            lrtsol%chrmax(iqtl-1)=chmax(iqtl)
            BestimMax = bestim_save(nlinMax,:)
            sigsquareEstimeMax=sigsq_save(nlinMax,:)
            curPosMax%listN=curPos%listN
            curPosMax%listChr=curPos%listChr
         end do

         deallocate (lchr)
         deallocate (lnpos)

         call end_position (curPos)

         deallocate (bestim_save)
         deallocate (sigsq_save)
         deallocate (lrt_save)

     end subroutine gen_opti_nqtl
!!***

!!****f* m_qtlmap_incidence/test_nuisances
!!  NAME
!!    test_nuisances
!!  DESCRIPTION
!!    Test des differents effets de nuisance du modele par une LRT compare a une chi2
!!  INPUTS
!!    ic                     : index of the trait
!!    incidenceDescOrigine   : description of the contingence matrix at the maximum
!!    curPosMax              : maximum position
!!    sigsquareEstime        : the residual variance at the maximum
!!    FUNCT_MODEL            : the function for the resolution (model_lin_h0,model_lin_hn,model_optim_h0,model_optim_hn)
!!  OUTPUTS
!!    workstruct             : residual variance and maximum Likelihood under H(nqtl)
!!  NOTES
!!
!!  SOURCE
      subroutine test_nuisances(ic,incidenceDescOrigine,lrtsol,curPosMax,workstruct,sigsquareEstime,FUNCT_MODEL)
        integer                                 ,intent(in)          :: ic!,nbnivestmax
        type(INCIDENCE_TYPE)                    ,intent(in)          :: incidenceDescOrigine
        type(TYPE_LRT_SOLUTION)                 ,intent(in)          :: lrtsol
        type(POSITION_LRT_INCIDENCE)            ,intent(in)          :: curPosMax
        type(INCIDENCE_GEN_STRUCT)              ,intent(inout)       :: workstruct
        real (kind=dp)                 , dimension(np) ,intent(in)   :: sigsquareEstime
        external                                                     :: FUNCT_MODEL

        real (kind=dp) ,dimension(:,:),pointer      :: xinc,xxx
        real (kind=dp) , dimension(:) ,pointer      :: Bestim,SIGest
        type(INCIDENCE_TYPE)                        :: incidenceDesc
        real (kind=dp) :: prob,sigEstSave(np),fmaxSave
        integer :: itest,i,j,nbreduit,nbef,nbco,nkd,ip,ifail,nqtl,nbnivestmax,nbeffnuis,numqtl

        nbnivestmax = incidenceDescOrigine%nbnivest
        nbef=listModelTrait(ic)%nbfe
        nbco=listModelTrait(ic)%nbco

      !  nbint=listModelTrait(incidenceDesc%ic)%nbint(numqtl)
        nqtl=workstruct%nqtl
        nbeffnuis = nbef+nbco
        do numqtl=1,nqtl
          nbeffnuis = nbeffnuis + listModelTrait(ic)%nbint(numqtl)
        end do

        if (nbeffnuis==0) then
          workstruct%ntest = 0
          return
        end if

        sigEstSave=workstruct%sigsquare(nqtl,:,1)
        fmaxSave=workstruct%fnqtl(nqtl)

        workstruct%sigsquare(workstruct%nqtl,:,1)=sigsquareEstime
        !workstruct%fnqtl(nqtl)=incidenceDesc%fmax

        workstruct%ntest=nbeffnuis
        allocate (workstruct%listtestnuis(workstruct%ntest))
        itest=0

        !initialisation of contingence matrix
        call init_incidence(ic,nqtl,incidenceDesc,workstruct%type_incidence,incidenceDescOrigine%dataset)

        allocate (xinc(nd,incidenceDescOrigine%ntnivmax))
        xinc=0.d0
        allocate (Bestim(incidenceDescOrigine%ntnivmax))
        allocate (SIGest(np))
        allocate (xxx(incidenceDescOrigine%ntnivmax,incidenceDescOrigine%ntnivmax))

        !on laisse seulement les variances comme point de depart
        incidenceDesc%par(np+1:)=0.d0

        Bestim=0.d0
       ! workStruct%

        do i=1,nbef+nbco
          if ( i <= nbef ) then
            call build_incidence_matrix(workstruct,curPosMax,xinc,incidenceDesc,i,0,0,0)
          end if
          if ( i > nbef .and. i <= nbef+nbco ) then
            call build_incidence_matrix(workstruct,curPosMax,xinc,incidenceDesc,0,i-nbef,0,0)
          end if

          !call debug_write_incidence(xinc,incidenceDesc)
          !call model analysis

          call FUNCT_MODEL(xinc,incidenceDesc,curPosMax,workstruct,SIGest,Bestim,.false.,.false.,xxx,.true.)

          ifail=0
          if ( i > nbef .and. i <= nbco ) then
            nbreduit=1
          else
            nbreduit = nbnivestmax -  incidenceDesc%nbnivest
          end if

          prob=MATH_QTLMAP_G01ECF('U',curPosMax%lrt(workstruct%nqtl),dble(nbreduit),ifail)

          itest=itest+1
          workStruct%listtestnuis(itest)%directeffect=.true.
          if ( i <= nbef ) then
           workStruct%listtestnuis(itest)%name=trim(namefix(listModelTrait(ic)%indexFixedEffect(i)))
          else
           workStruct%listtestnuis(itest)%name=trim(namefix(listModelTrait(ic)%indexCovariate(i-nbef)))
          end if
          workStruct%listtestnuis(itest)%df=nbreduit
          workStruct%listtestnuis(itest)%lrt=curPosMax%lrt(workstruct%nqtl)
          workStruct%listtestnuis(itest)%pvalue=prob

          do while(incidenceDesc%nteff>0)
            call remove_last_effect(incidenceDesc,xinc)
          end do
        end do

        do numqtl=1,nqtl
         do i=1,listModelTrait(incidenceDesc%ic)%nbint(numqtl)
            call build_incidence_matrix(workstruct,curPosMax,xinc,incidenceDesc,0,0,i,numqtl)
          !call debug_write_incidence(xinc,incidenceDesc)
          !call model analysis

          call FUNCT_MODEL(xinc,incidenceDesc,curPosMax,workstruct,SIGest,Bestim,.false.,.false.,xxx,.true.)

          ifail=0
          nbreduit = nbnivestmax -  incidenceDesc%nbnivest

          prob=MATH_QTLMAP_G01ECF('U',curPosMax%lrt(workstruct%nqtl),dble(nbreduit),ifail)

          itest=itest+1
          workStruct%listtestnuis(itest)%directeffect=.false.
          workStruct%listtestnuis(itest)%name=trim(namefix(listModelTrait(ic)%indexFixedEffectWithInteraction(numqtl,i)))//&
            " QTL["//trim(str(numqtl))//"]"
          workStruct%listtestnuis(itest)%df=nbreduit
          workStruct%listtestnuis(itest)%lrt=curPosMax%lrt(workstruct%nqtl)
          workStruct%listtestnuis(itest)%pvalue=prob

          do while(incidenceDesc%nteff>0)
            call remove_last_effect(incidenceDesc,xinc)
          end do
         end do
        end do

        workstruct%sigsquare(nqtl,:,1)=sigEstSave
        workstruct%fnqtl(nqtl)=fmaxSave

        call end_incidence(incidenceDesc)

        deallocate (SIGest)
        deallocate (xinc)
        deallocate (Bestim)
        deallocate (xxx)

       end subroutine test_nuisances
!!***

end module m_qtlmap_incidence
