!!****m* ANALYSE/m_qtlmap_incidence_multi
!!  NAME
!!    m_qtlmap_incidence_multi
!!  DESCRIPTION
!!
!!  NOTES
!!
!!  BUGS
!!
!!  HISTORY
!!
!!  SEE ALSO
!!
!!  COPYRIGHT
!!***
!! You can use this space for remarks that should not be included
!! in the documentation.
!!/
module m_qtlmap_incidence_multi
    use m_qtlmap_data
    use qtlmap_types
    use m_qtlmap_haplotype_data
    use m_qtlmap_math
    use m_qtlmap_log
    use m_qtlmap_tools
    use m_qtlmap_incidence
    use m_qtlmap_cd

    implicit none

!!****v* m_qtlmap_incidence_multi/estime_multi
!!  NAME
!!   estime_multi
!!  DESCRIPTION
!!   Estimability per progeny phenotyped for all trait combined
!!  NOTES
!!
!!***
    logical , dimension(:), allocatable  ,private   :: estime_multi
!!****v* m_qtlmap_incidence_multi/estfem_multi
!!  NAME
!!   estfem_multi
!!  DESCRIPTION
!!   Estimability per progeny genotyped for all trait combined
!!  NOTES
!!
!!***
    logical , dimension(:), allocatable  ,private   :: estfem_multi
!!****v* m_qtlmap_incidence_multi/iam_multi
!!  NAME
!!   iam_multi
!!  DESCRIPTION
!!   Indexe of the femal for all trait combined
!!  NOTES
!!
!!***
    integer , dimension(:), allocatable  ,private   :: iam_multi
!!****v* m_qtlmap_incidence_multi/namest_multi
!!  NAME
!!   namest_multi
!!  DESCRIPTION
!!   Number of female with estimability for all trait combined
!!  NOTES
!!
!!***
    integer                              ,private   :: namest_multi
!!****v* m_qtlmap_incidence_multi/dataset
!!  NAME
!!   dataset
!!  DESCRIPTION
!!   The current dataset using bi the likelihood function
!!  NOTES
!!
!!***
    type(DATASET_TYPE)                    ,pointer    ,private         :: dataset     => null()
!!****v* m_qtlmap_incidence_multi/ntnivmaxtotal
!!  NAME
!!   ntnivmaxtotal
!!  DESCRIPTION
!!   maximum level finded from all contingence matrix
!!  NOTES
!!
!!***
    integer                                       ,private             :: ntnivmaxtotal
!!****v* m_qtlmap_incidence_multi/my_listdesc
!!  NAME
!!   my_listdesc
!!  DESCRIPTION
!!   description of each contingence matrix
!!  NOTES
!!
!!***
    type(INCIDENCE_TYPE)   ,dimension(:)  ,pointer    ,private         :: my_listdesc => null()
!!****v* m_qtlmap_incidence_multi/my_xincreduitmul
!!  NAME
!!   my_xincreduitmul
!!  DESCRIPTION
!!   contingence matrix for each trait
!!  NOTES
!!
!!***
    real (kind=dp),dimension(:,:,:),allocatable,private                :: my_xincreduitmul
!!****v* m_qtlmap_incidence_multi/current_chr
!!  NAME
!!   current_chr
!!  DESCRIPTION
!!   current linkage group tested
!!  NOTES
!!
!!***
    integer , dimension(:)   ,pointer    ,private                      :: current_chr

    !$omp threadprivate (dataset,my_listdesc,my_xincreduitmul,current_chr)

    public :: init_incidence_multi
    public :: end_incidence_multi

    contains
!!****f* m_qtlmap_incidence_multi/init_incidence_multi
!! NAME
!!   init_incidence_multi
!! DESCRIPTION
!!
!! NOTE
!!
!! SOURCE
    subroutine init_incidence_multi(nqtl,xinc,listDesc,type_incidence)
      integer                               , intent(in)        :: nqtl,type_incidence
      real (kind=dp) , dimension(:,:,:), pointer, intent(inout) :: xinc
      type(INCIDENCE_TYPE)     ,dimension(ncar), intent(inout)  :: listDesc

      integer :: ip,jm,kd,ifem,eff,nteffmaxtotal,ic,nparmax,k,jc

      ! General array to get index information about femal estimable
      !                      ***
      allocate (estime_multi(nm))
      allocate (estfem_multi(nfem))
      allocate (iam_multi(nfem))
      estime_multi=.false.
      estfem_multi=.false.
      namest_multi=0
      iam_multi=0

      do ip=1,np
        do jm=nmp(ip)+1,nmp(ip+1)
            eff=0
            do kd=ndm(jm)+1,ndm(jm+1)
              if(count(presentc(:,kd))==ncar) then
                 eff=eff+1
              end if
            end do
            ifem=repfem(jm)
            if( eff>=NDMIN ) then
              estime_multi(jm)=.true.
              estfem_multi(ifem)=.true.
            end if
         end do !jm
      end do !ip

      do ifem=1,nfem
          if(estfem_multi(ifem)) then
           namest_multi=namest_multi+1
           iam_multi(ifem)=namest_multi
          end if
      end do

      ntnivmaxtotal=0
      nteffmaxtotal=0
      do ic=1,ncar
        listDesc(ic)%ic=ic
        listDesc(ic)%nqtl=nqtl
        call set_ntnivmax(ic,nqtl,listDesc(ic)%ntnivmax,listDesc(ic)%nteffmax,listDesc(ic)%ntlev,listDesc(ic)%nbniv)
        ntnivmaxtotal=max(ntnivmaxtotal,listDesc(ic)%ntnivmax)
        nteffmaxtotal=max(nteffmaxtotal,listDesc(ic)%nteffmax)

        if ( type_incidence==INCIDENCE_TRAIT_COV ) then
         listDesc(ic)%ntnivmax=listDesc(ic)%ntnivmax+ncar
         listDesc(ic)%nteffmax=listDesc(ic)%nteffmax+1
        end if


        listDesc(ic)%ntniv    = 0
        listDesc(ic)%nbnivest = 0
        listDesc(ic)%nteff    = 0

        call log_mess("********************* INIT CARAC="// trim(str(ic))//" *****************************",DEBUG_DEF)
        call log_mess("NTNIVMAX:"//str(listDesc(ic)%ntnivmax),DEBUG_DEF)
        call log_mess("NTEFFMAX:"//str(listDesc(ic)%nteffmax),DEBUG_DEF)
        call log_mess("NTLEVQTL / Reproductor:"//str(listDesc(ic)%ntlev),DEBUG_DEF)
        call log_mess("NBLEV FIXED EFFECT:"//str(listDesc(ic)%nbniv),DEBUG_DEF)
        call log_mess("*********************************************************",DEBUG_DEF)

      allocate(listDesc(ic)%desc(listDesc(ic)%nteffmax))
      allocate(listDesc(ic)%vecsol(listDesc(ic)%ntnivmax))
      listDesc(ic)%vecsol=.false.
    !  allocate(listDesc(ic)%corniv(ntnivmax))
      allocate(listDesc(ic)%precis(listDesc(ic)%ntnivmax))
      listDesc(ic)%precis=0.d0

      if ( nqtl > 0) then
       allocate (listDesc(ic)%ntniv_qtlsires(nqtl))
       allocate (listDesc(ic)%ntniv_qtldams(nqtl))
      end if
      allocate (listDesc(ic)%corr_niv_nivb(listDesc(ic)%ntnivmax))
     end do

      allocate (xinc(ncar,nd,ntnivmaxtotal))
      xinc=0.d0

      allocate (listDesc(1)%dataset)
      call init_dataset_ncar(nqtl,listDesc(1)%dataset)

      do ic=1,ncar
        allocate (listDesc(ic)%nonull(listDesc(1)%dataset%nkd,ntnivmaxtotal))
        allocate (listDesc(ic)%nbnonull(listDesc(1)%dataset%nkd))
      end do

      nparmax=ncar*np+ncar*(ncar-1)/2+ncar*ntnivmaxtotal

      allocate (listDesc(1)%borni(nparmax))
      allocate (listDesc(1)%borns(nparmax))
      allocate (listDesc(1)%par(nparmax))
      listDesc(1)%borni(1:ncar*np)=SIG_MIN
      listDesc(1)%borns(1:ncar*np)=SIG_MAX

      k=ncar*np
      !bornes des correlations residuelles
      do ic=2,ncar
       do jc=1,ic-1
         k=k+1
         listDesc(1)%borni(k)=-1.d0
         listDesc(1)%borns(k)=1.d0
       end do
      end do

      listDesc(1)%borni(k+1:)=DEFAULT_PARAM_MIN
      listDesc(1)%borns(k+1:)=DEFAULT_PARAM_MAX

      !initialisation des variances
      do ic=1,ncar
        do ip=1,np
          listDesc(1)%par((ic-1)*np+ip)=listDesc(1)%dataset%lSires(ip)%sig0(ic)
        end do
      end do

      listDesc(1)%par(ncar*np+1:)=0.d0

      do ic=2,ncar
         listDesc(ic)%dataset=>listDesc(1)%dataset
         listDesc(ic)%borni=>listDesc(1)%borni
         listDesc(ic)%borns=>listDesc(1)%borns
         listDesc(ic)%par=>listDesc(1)%par
      end do

    end subroutine init_incidence_multi
!!***

!!****f* m_qtlmap_incidence_multi/end_incidence_multi
!! NAME
!!   end_incidence_multi
!! DESCRIPTION
!!
!! NOTE
!!
!! SOURCE
    subroutine end_incidence_multi(listDesc)
      type(INCIDENCE_TYPE)     ,dimension(ncar), intent(inout)  :: listDesc
      integer :: ic

      deallocate (estime_multi)
      deallocate (iam_multi)
      deallocate (estfem_multi)

      call end_dataset(listDesc(1)%dataset)
      deallocate (listDesc(1)%dataset)
      do ic=2,ncar
        listDesc(ic)%dataset=>null()
        listDesc(ic)%borni=>null()
        listDesc(ic)%borns=>null()
        listDesc(ic)%par=>null()
      end do

      do ic=1,ncar
       call end_incidence(listDesc(ic))
      end do

    end subroutine end_incidence_multi
!!***

!!****f* m_qtlmap_incidence_multi/init_dataset_ncar
!! NAME
!!   init_dataset_ncar
!! DESCRIPTION
!!
!! NOTE
!!  constition des famille de plein et demi frere via des indexes
!!  pas de notion d estimable...on comptabilise seulement les effectifs et
!!  on repere les lignes de la matrice de contingence
!! SOURCE
  subroutine init_dataset_ncar(nqtl,dataset)
    type (DATASET_TYPE)                  , intent(inout)    :: dataset
    integer                                , intent(in)     :: nqtl
    logical ,dimension(nd) :: pres
    integer :: ip,jm,kd,ifem,kdd,s1,s2,chr,iq,ic,kkd,effp,efft,jc,eff
    real(kind=dp) :: somyp(ncar),somy(ncar)

    call log_mess("build_family for incidence multitrait analysis...",DEBUG_DEF)

    estime_multi=.false.
    allocate(dataset%lSires(np))
    kdd=0
    do ip=1,np
      allocate(dataset%lSires(ip)%full_sib(nmp(ip+1)-nmp(ip)))
      ifem=0
      do jm=nmp(ip)+1,nmp(ip+1)
        ifem=ifem+1
        kd=ndm(jm)+1
        do while ( kd<=ndm(jm+1) .and. .not. (count(presentc(:,kd))==ncar) )
          kd=kd+1
        end do
        if (kd>ndm(jm+1)) then
          dataset%lSires(ip)%full_sib(ifem)%firstKd=-1
          dataset%lSires(ip)%full_sib(ifem)%lastKd=-2
          cycle
        end if

        kdd=kdd+1
        ! on a trouve le premier kd valide du pere ip dans la matrice d incidence
        dataset%lSires(ip)%full_sib(ifem)%firstKd=kdd
        do kd=kd+1,ndm(jm+1)
          if ( (count(presentc(:,kd))==ncar) ) then
            kdd=kdd + 1
          end if
        end do
        dataset%lSires(ip)%full_sib(ifem)%lastKd=kdd

        !NOUVEAU ESTIME....
        estime_multi(jm)=(dataset%lSires(ip)%full_sib(ifem)%lastKd-dataset%lSires(ip)%full_sib(ifem)%firstKd)>=NDMIN

        call log_mess("Full Sib ["//trim(pere(ip))//"-"//trim(mere(jm))//"] :"//&
        trim(str(dataset%lSires(ip)%full_sib(ifem)%firstKd))//"->"&
        //trim(str(dataset%lSires(ip)%full_sib(ifem)%lastKd)),VERBOSE_DEF)
        s1=0
        do chr=1,nchr
          s1=max(s1,ngenom(chr,jm+1)-ngenom(chr,jm))
        end do
        s2=dataset%lSires(ip)%full_sib(ifem)%lastKd-dataset%lSires(ip)%full_sib(ifem)%firstKd+1
        if ( nqtl >= 1) then
          allocate(dataset%lSires(ip)%full_sib(ifem)%ppt(nqtl,s1,s2))
          allocate(dataset%lSires(ip)%full_sib(ifem)%pmt(nqtl,s1,s2))
        end if
      end do
      ifem=0
      do jm=nmp(ip)+1,nmp(ip+1)
        ifem=ifem+1
        if (dataset%lSires(ip)%full_sib(ifem)%firstKd<0) cycle
        dataset%lSires(ip)%half_sib%firstKd= dataset%lSires(ip)%full_sib(ifem)%firstKd
        exit
      end do
      ifem=nmp(ip+1)-nmp(ip)+1
      do jm=nmp(ip+1),nmp(ip)+1,-1
        ifem=ifem-1
        if (dataset%lSires(ip)%full_sib(ifem)%lastKd<0) cycle
        dataset%lSires(ip)%half_sib%lastKd=dataset%lSires(ip)%full_sib(ifem)%lastKd
        exit
      end do

      call log_mess("Half Sib ["//trim(pere(ip))//"] :"//&
        trim(str(dataset%lSires(ip)%half_sib%firstKd))//"->"//trim(str(dataset%lSires(ip)%half_sib%lastKd)),&
        VERBOSE_DEF)
      !s1=lSires(ip)%half_sib%lastKd-lSires(ip)%half_sib%firstKd+1
      if ( nqtl >= 1) then
        allocate(dataset%lSires(ip)%ppt(nqtl,&
        dataset%lSires(ip)%half_sib%firstKd:dataset%lSires(ip)%half_sib%lastKd))
      end if
    end do ! ip

    do ic=2,ncar
       dataset%lSires = dataset%lSires
    end do

    dataset%nkd=0
    dataset%nkd_max_by_fam=0

    do ip=1,np
        do jm=nmp(ip)+1,nmp(ip+1)
         eff=0
         do kd=ndm(jm)+1,ndm(jm+1)
          if (count(presentc(:,kd))==ncar) then
            dataset%nkd=dataset%nkd+1
            eff=eff+1
          end if
         end do !kd
         dataset%nkd_max_by_fam=max(dataset%nkd_max_by_fam,eff)
        end do ! jm
    end do ! ip

    allocate (dataset%Y(ncar,dataset%nkd))
    allocate (dataset%CD(ncar,dataset%nkd))

     do ic=1,ncar
      kkd=0
      do ip=1,np
        do jm=nmp(ip)+1,nmp(ip+1)
         do kd=ndm(jm)+1,ndm(jm+1)
          if (count(presentc(:,kd))==ncar) then
            kkd=kkd+1
            if ( natureY(ic) /= 'i' ) then
              dataset%Y(ic,kkd)=y(ic,kd)
              dataset%CD(ic,kkd)=cd(ic,kd)
            else
              call stop_application("Type of trait 'i' are not managed in multi-trait analysis!")
              dataset%YDISCRETORD(ic,kkd)=ydiscretord(ic,kd)
            end if
          end if
         end do !kd
        end do ! jm
      end do ! ip
     end do !ic

     somy=0
     efft=0
     do ip=1,np
        allocate(dataset%lSires(ip)%sig0(ncar))
        allocate(dataset%lSires(ip)%xmu0(ncar))
        effp=0
        somyp=0
        do jm=nmp(ip)+1,nmp(ip+1)
          do kd=ndm(jm)+1,ndm(jm+1)
            if (count(presentc(:,kd))==ncar) then
               effp=effp+1
               do ic=1,ncar
                 somyp(ic)=somyp(ic)+y(ic,kd)!*cd(ic,kd)
               end do
            end if
          end do
        end do

        !somme total sur le caractere
        do ic=1,ncar
         somy(ic)=somy(ic)+somyp(ic)
        end do
        !effectif total pour le caractere
        efft=efft+effp

        if (effp == 0.d0) then
           call stop_application('Father ['//trim(pere(ip))//'] has got no child with trait value ['//trim(carac(ic))//"]")
        end if

        !moyenne du caractere dans la famille du pere ip
        do ic=1,ncar
         dataset%lSires(ip)%xmu0(ic)=somyp(ic)/dble(effp)
         call log_mess("Mean for Sire/trait ["//trim(pere(ip))//","//trim(carac(ic))&
         //"]="//str(dataset%lSires(ip)%xmu0(ic)),INFO_DEF)
        end do
        !ecart type du caractere dans la famille du pere ip
        dataset%lSires(ip)%sig0(:)=0.d0
        do jm=nmp(ip)+1,nmp(ip+1)
          do kd=ndm(jm)+1,ndm(jm+1)
            if (count(presentc(:,kd))==ncar) then
               do ic=1,ncar
                dataset%lSires(ip)%sig0(ic)=dataset%lSires(ip)%sig0(ic)+(y(ic,kd)-&
                 dataset%lSires(ip)%xmu0(ic))*(y(ic,kd)-dataset%lSires(ip)%xmu0(ic))
               end do
            end if
          end do !kd
        end do !jm

        do ic=1,ncar
         dataset%lSires(ip)%sig0(ic)=sqrt(dataset%lSires(ip)%sig0(ic)/dble(effp-1))
         call log_mess("Standart deviation for Sire/trait ["//trim(pere(ip))//","//trim(carac(ic))&
         //"]="//str(dataset%lSires(ip)%sig0(ic)),INFO_DEF)
        end do
      end do !ip

      !moyenne du caractere sur l ensemble de la population
      allocate(dataset%xmu(ncar))
      do ic=1,ncar
       dataset%xmu(ic)=somy(ic)/dble(efft)
       call log_mess("Mean for trait ["//trim(carac(ic))//"]="//str(dataset%xmu(ic)),INFO_DEF)
      end do

      allocate(dataset%sig(ncar))
      allocate(dataset%cov(ncar,ncar))
      allocate(dataset%rhoi(ncar,ncar))
      dataset%cov=0.d0
      dataset%sig=0.d0

      do ip=1,np
        do jm=nmp(ip)+1,nmp(ip+1)
          do kd=ndm(jm)+1,ndm(jm+1)
            if (count(presentc(:,kd))==ncar) then
              do ic=1,ncar
               dataset%sig(ic)=dataset%sig(ic)+(y(ic,kd)-dataset%xmu(ic))*(y(ic,kd)-dataset%xmu(ic))
               do jc=1,ic-1
                dataset%cov(jc,ic)=dataset%cov(jc,ic)+(y(jc,kd)-dataset%xmu(jc))*(y(ic,kd)-dataset%xmu(ic))
               end do
              end do
            end if
          end do
        end do
      end do

      !variance sur l ensemble de la population
      do ic=1,ncar
       dataset%sig(ic)=sqrt(dataset%sig(ic)/dble(efft-1))
       call log_mess("Standart deviation for trait ["//trim(carac(ic))//"]="//str(dataset%sig(ic)),INFO_DEF)
      end do
      !calcul des covariances
      do ic=1,ncar
       do jc=1,ic-1
          dataset%cov(jc,ic)=dataset%cov(jc,ic)/dble(efft)
          dataset%cov(ic,jc)=dataset%cov(jc,ic)
          dataset%rhoi(jc,ic)=dataset%cov(jc,ic)/(dataset%sig(ic)*dataset%sig(ic))
          dataset%rhoi(ic,jc)=dataset%rhoi(jc,ic)
       end do
      end do

!   stop
  end subroutine init_dataset_ncar
!!***

!!****f* m_qtlmap_incidence_multi/add_general_mean_multi
!! NAME
!!   add_general_mean_multi
!! DESCRIPTION
!!
!! NOTE
!!
!! SOURCE
    subroutine add_general_mean_multi(xinc,listDesc)
       real (kind=dp) , dimension(ncar,nd,ntnivmaxtotal) , intent(inout) :: xinc
       type(INCIDENCE_TYPE) , dimension(ncar) , intent(inout) :: listDesc
       integer :: ic,ntniv
       call log_mess("multi incidence : ** Add General mean ** ",DEBUG_DEF)

       do ic=1,ncar
         listDesc(ic)%ntniv = listDesc(ic)%ntniv+1
         ntniv = listDesc(ic)%ntniv
         listDesc(ic)%nteff = listDesc(ic)%nteff+1
         listDesc(ic)%desc(listDesc(ic)%nteff)%name="General Mean"
         listDesc(ic)%desc(listDesc(ic)%nteff)%start=ntniv
         listDesc(ic)%desc(listDesc(ic)%nteff)%end=ntniv
         listDesc(ic)%desc(listDesc(ic)%nteff)%haveSubDesc=.true.
         allocate (listDesc(ic)%desc(listDesc(ic)%nteff)%listSubDesc(1))
         listDesc(ic)%desc(listDesc(ic)%nteff)%listSubDesc(1)%name="General Mean"!//"["//trim(carac(ic))//"]"
         listDesc(ic)%desc(listDesc(ic)%nteff)%listSubDesc(1)%start=ntniv
         listDesc(ic)%desc(listDesc(ic)%nteff)%listSubDesc(1)%end=ntniv
         xinc(ic,:,ntniv) = 1
       end do
       !incidenceDesc%nivdir(:,incidenceDesc%nteff)   = 1
    end subroutine add_general_mean_multi
!!***

!!****f* m_qtlmap_incidence_multi/add_polygenic_effect_multi
!! NAME
!!   add_polygenic_effect_multi
!! DESCRIPTION
!!
!! NOTE
!!
!! SOURCE
    subroutine add_polygenic_effect_multi(xinc,listDesc)
       real (kind=dp) , dimension(ncar,nd,ntnivmaxtotal) , intent(inout) :: xinc
       type(INCIDENCE_TYPE) , dimension(ncar) , intent(inout) :: listDesc
       integer :: nkd
       integer :: ip,jm,kd,nt,j,fem,nteff,ic
       call log_mess("incidence : ** Multi : Add Polygenic effect to estimate **",DEBUG_DEF)

       do ic=1,ncar
         nt=listDesc(ic)%ntniv
         nteff = listDesc(ic)%nteff
         listDesc(ic)%desc(listDesc(ic)%nteff+1)%name="Sire polygenic effects"
         listDesc(ic)%desc(listDesc(ic)%nteff+1)%start=nt+1
         listDesc(ic)%desc(listDesc(ic)%nteff+1)%end=nt+np
         listDesc(ic)%desc(listDesc(ic)%nteff+1)%haveSubDesc=.true.
         allocate (listDesc(ic)%desc(listDesc(ic)%nteff+1)%listSubDesc(np))
         do ip=1,np
          listDesc(ic)%desc(listDesc(ic)%nteff+1)%listSubDesc(ip)%name="Sire "//trim(pere(ip))!//"["//trim(carac(ic))//"]"
          listDesc(ic)%desc(listDesc(ic)%nteff+1)%listSubDesc(ip)%start=nt+ip
          listDesc(ic)%desc(listDesc(ic)%nteff+1)%listSubDesc(ip)%end=nt+ip
         end do
         listDesc(ic)%nteff = listDesc(ic)%nteff+1
         listDesc(ic)%ntniv = listDesc(ic)%ntniv+np

       if ( count(estime_multi(:))>0) then
         listDesc(ic)%desc(listDesc(ic)%nteff+1)%name="Dam polygenic effects"
         listDesc(ic)%desc(listDesc(ic)%nteff+1)%start=listDesc(ic)%desc(listDesc(ic)%nteff)%end+1
         listDesc(ic)%desc(listDesc(ic)%nteff+1)%end=nt+np+count(estime_multi(:))!nbfemLoc(ic)
         listDesc(ic)%desc(listDesc(ic)%nteff+1)%haveSubDesc=.true.
         allocate (listDesc(ic)%desc(listDesc(ic)%nteff+1)%listSubDesc(count(estime_multi(:))))
         fem=0
         do ip=1,np
          do jm=nmp(ip)+1,nmp(ip+1)
           if ( estime_multi(jm) ) then
             fem=fem+1
             listDesc(ic)%desc(listDesc(ic)%nteff+1)%listSubDesc(fem)%name="Dam "//trim(mere(jm))//" [Sire "//trim(pere(ip))//"]"!&//"["//trim(carac(ic))//"]"
             listDesc(ic)%desc(listDesc(ic)%nteff+1)%listSubDesc(fem)%start=nt+np+fem
             listDesc(ic)%desc(listDesc(ic)%nteff+1)%listSubDesc(fem)%end=nt+np+fem
           end if
         end do
        end do
         listDesc(ic)%nteff = listDesc(ic)%nteff+1
         listDesc(ic)%ntniv = listDesc(ic)%ntniv+count(estime_multi(:))!nbfemLoc(ic)
       else

       end if
       j=0

       !initialisation
       xinc(ic,:,nt+1:nt+np+count(estime_multi(:nm))) = 0

       nkd=0
       do ip=1,np
         xinc(ic,:,nt+ip) = 0
         do jm=nmp(ip)+1,nmp(ip+1)
           do kd=ndm(jm)+1,ndm(jm+1)
             if ( count(presentc(:,kd))==ncar ) then
               nkd =nkd +1
               xinc(ic,nkd,nt+ip) = 1
          !    listDesc(ic)%nivdir(kd,nteff+1)= nt+ip
               if (estime_multi(jm)) then
                fem=count(estime_multi(:jm)) ! on compte le nombre estimable dans la famille de pere ip
                xinc(ic,nkd,nt+np+fem) = 1
          !    listDesc(ic)%nivdir(kd,nteff+2)= nt+np+fem
               end if
            end if
           end do
         end do
       end do
      end do !ic
    end subroutine add_polygenic_effect_multi
!!***

!!****f* m_qtlmap_incidence_multi/add_qtleffect_multi
!! NAME
!!   add_qtleffect_multi
!! DESCRIPTION
!!
!! NOTE
!!
!! SOURCE
    subroutine add_qtleffect_multi(nteff,numqtl,chr,n,xinc,listDesc,mint,linear)
       integer                                      ,intent(in)     :: nteff,numqtl,chr,n
       real (kind=dp) , dimension(ncar,nd,ntnivmaxtotal) ,intent(inout)  :: xinc
       type(INCIDENCE_TYPE) , dimension(ncar)      , intent(inout)  :: listDesc
       integer                                     , intent(in)     :: mint ! index of interaction to remove from incidence
       logical                                     , intent(in)     :: linear

       integer :: ic,ntniv,nteffstart,ip,fem,jm,l,s,u,nbint,nbt,jef
       logical ,dimension(nfem) :: femInit

       call log_mess("multi incidence : ** Add Qtl effect to estimate **",DEBUG_DEF)

       do ic=1,ncar

         nteffstart = listDesc(ic)%nteff
         listDesc(ic)%nteff = listDesc(ic)%nteff+1
         if ( count(estime_multi(:)) > 0 ) listDesc(ic)%nteff = listDesc(ic)%nteff+1

         nbint=modele(ic,3)
         nbt=3+modele(ic,1)+modele(ic,2)
         listDesc(ic)%ntlev=1
         do jef=1,nbint
          if ( mint == jef) cycle ! this interaction is not adding
          listDesc(ic)%ntlev=listDesc(ic)%ntlev*nlev(modele(ic,nbt+jef))
         end do

         listDesc(ic)%desc(nteffstart+1)%name="Sire QTL effects ["//trim(str(numqtl))//"]"
         listDesc(ic)%desc(nteffstart+1)%start=listDesc(ic)%ntniv+1
         listDesc(ic)%ntniv_qtlsires(numqtl)=listDesc(ic)%desc(nteffstart+1)%start
         listDesc(ic)%desc(nteffstart+1)%end=listDesc(ic)%ntniv+listDesc(ic)%ntlev*np

         listDesc(ic)%desc(nteffstart+1)%haveSubDesc=.true.
         allocate (listDesc(ic)%desc(nteffstart+1)%listSubDesc(np*listDesc(ic)%ntlev))
         u=0
         do ip=1,np
          l=0
          do s=listDesc(ic)%ntniv+(ip-1)*listDesc(ic)%ntlev+1,listDesc(ic)%ntniv+ip*listDesc(ic)%ntlev
           l=l+1
           u=u+1
           listDesc(ic)%desc(nteffstart+1)%listSubDesc(u)%name="Sire "//trim(pere(ip))//" Level "//trim(str(l))!&//"["//trim(carac(ic))//"]"
           listDesc(ic)%desc(nteffstart+1)%listSubDesc(u)%start=s
           listDesc(ic)%desc(nteffstart+1)%listSubDesc(u)%end=s
          end do
         end do

         if ( count(estime_multi(:)) > 0 ) then
          listDesc(ic)%desc(nteffstart+2)%name="Dam QTL effects ["//trim(str(numqtl))//"]"
          listDesc(ic)%desc(nteffstart+2)%start=listDesc(ic)%desc(nteffstart+1)%end+1
          listDesc(ic)%ntniv_qtldams(numqtl)=listDesc(ic)%desc(nteffstart+2)%start
          femInit=.false.
          l=0
          do jm=1,nm
           if (estime_multi(jm) ) then
             fem=iam_multi(repfem(jm))
             if (femInit(fem)) then
                cycle
             end if
             femInit(fem)=.true.
             l=l+1
           end if
         end do

         allocate (listDesc(ic)%desc(nteffstart+2)%listSubDesc(l*listDesc(ic)%ntlev))
         listDesc(ic)%desc(nteffstart+2)%end=listDesc(ic)%ntniv+listDesc(ic)%ntlev*(np+l)
         listDesc(ic)%desc(nteffstart+2)%haveSubDesc=.true.
         !fem=0
         femInit=.false.
         u=0
         do jm=1,nm
           if (estime_multi(jm) ) then
             fem=iam_multi(repfem(jm))
             if (femInit(fem)) then
                cycle
             end if
             femInit(fem)=.true.
             l=0
             do s=listDesc(ic)%desc(nteffstart+2)%start+(fem-1)*listDesc(ic)%ntlev,&
             listDesc(ic)%desc(nteffstart+2)%start-1+fem*listDesc(ic)%ntlev
              l=l+1
              u=u+1
              !fem=fem+1
              listDesc(ic)%desc(nteffstart+2)%listSubDesc(u)%name="Dam "//trim(mere(jm))//" Level "//trim(str(l))!&//"["//trim(carac(ic))//"]"
              listDesc(ic)%desc(nteffstart+2)%listSubDesc(u)%start=s
              listDesc(ic)%desc(nteffstart+2)%listSubDesc(u)%end=s
             end do
           end if
         end do

         listDesc(ic)%ntniv = listDesc(ic)%desc(nteffstart+2)%end
       else
         listDesc(ic)%ntniv = listDesc(ic)%desc(nteffstart+1)%end
       end if
     end do !ic

    call change_qtleffect_multi(nteff,numqtl,chr,n,xinc,listDesc,mint)

    end subroutine add_qtleffect_multi
!!***

!!****f* m_qtlmap_incidence_multi/change_qtleffect_multi
!! NAME
!!   change_qtleffect_multi
!! DESCRIPTION
!!
!! NOTE
!!
!! SOURCE
    subroutine change_qtleffect_multi(nteff,iq,chr,n,xinc,listDesc,mint)
       integer                                 ,intent(in)     :: nteff,iq,chr,n
       real (kind=dp) , dimension(ncar,nd,ntnivmaxtotal) ,intent(inout)  :: xinc
       type(INCIDENCE_TYPE) , dimension(ncar)      , intent(inout)  :: listDesc
       integer                                 ,intent(in)     :: mint ! index of interaction to remove from incidence

       !local
       integer :: nbef,nbco,nbint,nbt,jef,nivint,ntlev,ip,jm,kd,ntt,km,ntniv,nkd,ic
       real (kind=dp) ,dimension(:,:),allocatable:: pp,pm

       allocate (pp(nd,ncar))
       allocate (pm(nd,ncar))

         nkd=0

         do ip=1,np
          do jm=nmp(ip)+1,nmp(ip+1)
           !if ( linear ) then
            !call getprob_lin(chr,n,ic,ip,jm,ndm(jm)+1,ndm(jm+1),pp,pm)
           !else
           do ic=1,ncar
            call getglobalprob(chr,n,ic,ip,jm,ndm(jm)+1,ndm(jm+1),pp(:,ic),pm(:,ic))
           end do
           !end if
           do kd=ndm(jm)+1,ndm(jm+1)
            if ( count(presentc(:,kd)) == ncar ) then
             nkd=nkd+1

           !  si des effets fixes sont en interaction avec l'allele paternel au qtl
           !  il faut estimer les effets qtl pour chacun des niveaux concernes
           !
           !  on commence donc par creer un effet composites regroupant l'ensemble de
           !  ces effets
           !
           do ic=1,ncar
            ntniv = listDesc(ic)%desc(nteff)%start -1
            nivint=1
            ntlev=1
            nbt=3+modele(ic,1)+modele(ic,2)
           !nt=ntniv
            do jef=1,modele(ic,3)
             if ( mint == jef) cycle ! this interaction is not adding
             nivint=nivint+ntlev*(nivx(kd,modele(ic,nbt+jef))-1)
             ntlev=ntlev*nlev(modele(ic,nbt+jef))
           end do
             ntt=ntniv+nivint+ntlev*(ip-1)
             xinc(ic,nkd,ntt)=pp(kd,ic)

          ! incidenceDesc%nivdir(kd,nteff+1)=ntt
          ! print *,'nt:',nivdir(kd,nteff+1),' vp:',pp(kd),' nteff:',nteff,' kd:',kd

             if(estime_multi(jm)) then
              km=iam_multi(repfem(jm))
              ntt= ntniv+ntlev*np+nivint+ntlev*(km-1)
              xinc(ic,nkd,ntt)=pm(kd,ic)
           !   incidenceDesc%nivdir(kd,nteff+2)=ntt
              ! print *,'nt:',nivdir(kd,nteff+2),' vp:',pm(kd)
             end if
           end do !ic
          end if !presentc
         end do
        end do
       end do


       deallocate (pp)
       deallocate (pm)
       ! a modifier
       ! --> on devrait integerer l intialisation des pdds dans la boucle precedente
       ! attention les interaction qtl ne sont pas encore pris en compte
      ! if (.not. linear) then
       call pdd_at_position_multi(listDesc(1)%dataset,iq,chr,n)
     !  end if
    end subroutine change_qtleffect_multi
!!***

!!****f* m_qtlmap_incidence_multi/pdd_at_position_multi
!! NAME
!!   pdd_at_position_multi
!! DESCRIPTION
!!
!! NOTE
!!
!! SOURCE
    subroutine pdd_at_position_multi(dataset,iq,chr,n)
          type(DATASET_TYPE)             , intent(inout)  :: dataset
          integer   ,intent(in)   :: iq,chr,n

          integer :: ip,jm,kd,ig,kkd,igg,ifem,kkkd,kds
          real (kind=dp) :: ppt,pmt

          do ip=1,np
           dataset%lSires(ip)%ppt(iq,:)=0.d0
           ifem=0
           kkkd=0
           do jm=nmp(ip)+1,nmp(ip+1)
             ifem=ifem+1
             if (dataset%lSires(ip)%full_sib(ifem)%firstkd>=0) then
              dataset%lSires(ip)%full_sib(ifem)%ppt(iq,:,:)=0.d0
              dataset%lSires(ip)%full_sib(ifem)%pmt(iq,:,:)=0.d0
              igg=0
              kds=kkkd
            ! print *,"ip,jm,firstkd:",ip,jm,lSires(ip)%half_sib%firstKd
              do ig=ngenom(chr,jm)+1,ngenom(chr,jm+1)
               igg=igg+1
               kkd=0
               kkkd=kds
               do kd=ngend(chr,ig)+1,ngend(chr,ig+1)
                if(count(presentc(:,ndesc(chr,kd)))==ncar) then
	             kkd=kkd+1
	             kkkd=kkkd+1
	             if( estime_multi(jm) )then
                    dataset%lSires(ip)%full_sib(ifem)%ppt(iq,igg,kkd)=&
                     -pdd(chr,kd,1,n)-pdd(chr,kd,2,n)+pdd(chr,kd,3,n)+pdd(chr,kd,4,n)
                    dataset%lSires(ip)%full_sib(ifem)%pmt(iq,igg,kkd)=&
                     -pdd(chr,kd,1,n)+pdd(chr,kd,2,n)-pdd(chr,kd,3,n)+pdd(chr,kd,4,n)
                 else
                   dataset%lSires(ip)%ppt(iq,dataset%lSires(ip)%half_sib%firstKd+kkkd-1)=&
                    -pdd(chr,kd,1,n)+pdd(chr,kd,3,n)
                 end if
               end if
              end do !! kd
             end do !! ig
            end if
           end do ! jm
          end do ! ip
      end subroutine pdd_at_position_multi
!!***

!!****f* m_qtlmap_incidence_multi/add_effcov_multi
!! NAME
!!   add_effcov_multi
!! DESCRIPTION
!!
!! NOTE
!!
!! SOURCE
    subroutine add_effcov_multi(xinc,listDesc,meff,mcov)
       real (kind=dp) , dimension(ncar,nd,ntnivmaxtotal) , intent(inout) :: xinc
       type(INCIDENCE_TYPE) , dimension(ncar)      , intent(inout)  :: listDesc
       integer                                 , intent(in)    :: meff   ! indice on effet fixed list to remove (<=0 otherwise)
       integer                                 , intent(in)    :: mcov   ! indice on covariate list to remove (<=0 otherwise)
       integer :: ic,ntniv,ip,jm,kd,ntnifix,nbt,nbef,nbco,nteff,jef,jcov,ieff,nbniv,nivd,i,j,nbef2,nbco2,nkd

       call log_mess("incidence : ** Add eff and cov ** ",DEBUG_DEF)

       do ic=1,ncar

        nbef=modele(ic,1)
        nbco=modele(ic,2)

        nbef2=nbef
        if ( meff /= 0 ) nbef2=nbef-1
        nbco2=nbco
        if ( mcov /=0 ) nbco2=nbco-1


       if (nbef2+nbco2 <=0)return

       nbniv=0
       ieff=listDesc(ic)%nteff

       if ( nbef2 > 0 ) then
        ieff=ieff+1
        listDesc(ic)%eqtl_print(ieff)=.false.
        do i=1,nbef
         if ( i == meff ) cycle
         nbniv=nlev(modele(ic,3+i))+nbniv
        end do
        listDesc(ic)%desc(ieff)%name="Fixed effects"
        listDesc(ic)%desc(ieff)%start=listDesc(ic)%ntniv+1
        listDesc(ic)%desc(ieff)%end=listDesc(ic)%ntniv+nbniv
        listDesc(ic)%desc(ieff)%haveSubDesc=.true.

        !init of xinc
        xinc(ic,:,listDesc(ic)%desc(ieff)%start:listDesc(ic)%desc(ieff)%end)=0.d0

        allocate ( listDesc(ic)%desc(ieff)%listSubDesc(nbniv) )

        j=0
        do jef=1,nbef
         if ( jef == meff ) cycle
         do i=1,nlev(modele(ic,3+jef))
         j=j+1
         listDesc(ic)%desc(ieff)%listSubDesc(j)%name=trim(namefix(modele(ic,3+jef)))//' level '//trim(str(i))!&//"["//trim(carac(ic))//"]"
         listDesc(ic)%desc(ieff)%listSubDesc(j)%start=listDesc(ic)%ntniv+j
         listDesc(ic)%desc(ieff)%listSubDesc(j)%end=listDesc(ic)%ntniv+j
         end do
        end do
        listDesc(ic)%nteff = listDesc(ic)%nteff+1
       end if

       if ( nbco2 > 0 ) then
        ieff=ieff+1
        listDesc(ic)%eqtl_print(ieff)=.false.
        listDesc(ic)%desc(ieff)%name="Covariates"
        listDesc(ic)%desc(ieff)%start=listDesc(ic)%ntniv+nbniv+1
        listDesc(ic)%desc(ieff)%end=listDesc(ic)%ntniv+nbniv+nbco2
        listDesc(ic)%desc(ieff)%haveSubDesc=.true.

        !init of xinc
        xinc(ic,:,listDesc(ic)%desc(ieff)%start:listDesc(ic)%desc(ieff)%end)=0.d0

        allocate ( listDesc(ic)%desc(ieff)%listSubDesc(nbco2) )

        j=0
        do jcov=1,nbco
         if ( jcov == mcov ) cycle
         j=j+1
         listDesc(ic)%desc(ieff)%listSubDesc(j)%name=trim(namecov(modele(ic,3+modele(ic,1)+jcov)))!//"["//trim(carac(ic))//"]"
         listDesc(ic)%desc(ieff)%listSubDesc(j)%start=listDesc(ic)%ntniv+nbniv+j
         listDesc(ic)%desc(ieff)%listSubDesc(j)%end=listDesc(ic)%ntniv+nbniv+j
        end do
        listDesc(ic)%nteff = listDesc(ic)%nteff+1
       end if

       nkd = 0
       nbniv=0
       do ip=1,np
         do jm=nmp(ip)+1,nmp(ip+1)
           do kd=ndm(jm)+1,ndm(jm+1)

            if ( count(presentc(:,kd))==ncar ) then
            nkd=nkd+1
            ntniv=listDesc(ic)%ntniv

            !
            !  autres effets fixes
            !
            nbniv=0
            do jef=1,nbef
             if ( jef == meff ) cycle ! this effet fixed is not adding
            ! listDesc(ic)%nivdir(kd,nteff+jef)=ntniv+nbniv+nivx(kd,modele(ic,3+jef))
             nivd=ntniv+nbniv+nivx(kd,modele(ic,3+jef))
             xinc(ic,nkd,nivd)=1
             nbniv=nbniv+nlev(modele(ic,3+jef))
            end do

            ntnifix=ntniv+nbniv
            nbt=3+nbef
            !covariate
            j=0
            do jcov=1,nbco
             if ( jcov == mcov ) cycle ! this covariate is not adding
             j=j+1
             ! listDesc(ic)%covdir(kd,jcov)=covar(kd,modele(ic,nbt+jcov))
             xinc(ic,nkd,ntnifix+j)=covar(kd,modele(ic,nbt+jcov))
            end do
            !ntniv=ntnifix+nbco
            !nbt=nbt+nbco
          end if
       end do
      end do
     end do

     listDesc(ic)%ntniv=listDesc(ic)%ntniv+nbniv+nbco2
    end do !ic

    call log_mess("FIN ** Add eff and cov ** ",DEBUG_DEF)

    end subroutine add_effcov_multi
!!***

!!****f* m_qtlmap_incidence_multi/confusion_multi_type1
!! NAME
!!   confusion_multi_type1
!! DESCRIPTION
!!
!! NOTE
!!
!! SOURCE
      subroutine confusion_multi_type1(listDesc,xxx,workstruct)
        type(INCIDENCE_TYPE)   , dimension(ncar)      , intent(inout)  :: listDesc
        real (kind=dp)  ,dimension(ncar,ntnivmaxtotal,ntnivmaxtotal) ,intent(in)      :: xxx
        type(INCIDENCE_GEN_STRUCT)            ,intent(inout)       :: workstruct

        real (kind=dp)  ,dimension(:,:,:),allocatable    :: xcor
        type(CORR_ALERT_TYPE) ,dimension(:),allocatable  :: alerts
        type(CORR_ALERT_TYPE) ,dimension(:),allocatable  :: as
        integer :: ic,ialert,sizealert,eff,i,j,starteff

        allocate (xcor(ncar,ntnivmaxtotal,ntnivmaxtotal))

        sizealert=0
        do ic=1,ncar
         call get_correlations(listDesc(ic),xxx(ic,:,:),xcor(ic,:,:))
          sizealert=sizealert+np*namest_multi*listDesc(ic)%ntlev*listDesc(ic)%nbnivest
        end do

        workstruct%corrmax=0.d0



        allocate (alerts(sizealert))
        allocate (as(sizealert*workstruct%nqtl))

        workstruct%nalert=0
        starteff = workstruct%listnteff(workstruct%nqtl)+1
        if (namest_multi>0) starteff = starteff+1

        do ic=1,ncar
         do eff=starteff,listDesc(ic)%nteff
           call log_mess("computing confusion beetween qtls and:"//listDesc(ic)%desc(eff)%name,VERBOSE_DEF)
           call confusion_between_qtl_and_effect(listDesc(ic),xcor(ic,:,:),&
           eff,sizealert,alerts,ialert,workstruct%corrmax)
           if (ialert>0) then
      !       print *,"ialert:",alerts(1)%corr,alerts(2)%corr,alerts(3)%corr
             as(workstruct%nalert+1:workstruct%nalert+ialert)=alerts(:ialert)
             workstruct%nalert=workstruct%nalert+ialert
           end if
         end do
        end do

        allocate (workstruct%alertQtl(workstruct%nalert))
        workstruct%alertQtl=as(:workstruct%nalert)

        deallocate (alerts)
        deallocate (as)
        deallocate (xcor)


      end subroutine confusion_multi_type1
!!***

!!****f* m_qtlmap_incidence_multi/opti_0qtl_multi
!! NAME
!!   opti_0qtl_multi
!! DESCRIPTION
!!
!! NOTE
!!
!! SOURCE
   subroutine opti_0qtl_multi(  incsol   ,       &  !
                               workstruct, &
                               maxqtl)          ! maximum qtl to find

      integer                                 ,intent(in)       :: maxqtl
      type(INCIDENCE_GEN_STRUCT)              ,intent(inout)    :: workstruct
      type(TYPE_INCIDENCE_SOLUTION)           ,intent(out)      :: incsol
      !external :: FUNCT_MODEL   ! function with a model specific

      integer :: nkd,ip,i,listnteff(1)

      real (kind=dp) ,dimension(:,:,:),pointer      :: xinc
      real (kind=dp) , dimension(:,:) ,pointer      :: Bestim,rhoiestim
      type(INCIDENCE_TYPE)  ,dimension(ncar)        :: listDesc
      type(POSITION_LRT_INCIDENCE)                  :: curPos
      real (kind=dp) :: sigsquareEstime(ncar,np),f
      integer :: k,ic,jc

      !initialisation of incidence matrix
      call init_incidence_multi(0,xinc,listDesc,workstruct%type_incidence)
      dataset=>listDesc(1)%dataset
      allocate (Bestim(ncar,ntnivmaxtotal))
      allocate (rhoiestim(ncar,ncar))
      Bestim=0.d0
      rhoiestim=0.d0
      sigsquareEstime=0.d0
      workstruct%nqtl=0
      Bestim=0.d0
      call add_general_mean_multi(xinc,listDesc)
      workstruct%eff_general_mean=1
      !add polygenic effect
      call add_polygenic_effect_multi(xinc,listDesc)
      !add fixed effect and covariate
      call add_effcov_multi(xinc,listDesc,0,0)

      call model_optim_multi_h0(xinc,listDesc,workstruct,sigsquareEstime,rhoiestim,Bestim,.true.)

      call set_solution_multi(0,workstruct,sigsquareEstime,rhoiestim,Bestim,listDesc,incsol,1,listnteff)

      if (size(workstruct%sigsquare,1)>=1) then
        do ip=1,np
          do ic=1,ncar
           workstruct%sigsquare(1,ip,ic)=sigsquareEstime(ic,ip)
          end do
        end do
      end if

      if (size(workstruct%fnqtl,1)>=1) then
          workstruct%fnqtl(1)=listDesc(1)%fmax
      end if

      if (associated(workstruct%rhoi)) then
        workstruct%rhoi(1,:,:) = rhoiestim
      end if

      !call model analysis
      !call FUNCT_MODEL(xinc,incidenceDesc,workstruct,Bestim,.true.)
     ! call set_solution(ic,0,workstruct,Bestim,incidenceDesc,incsol,1,listnteff)

      call end_incidence_multi(listDesc)
      deallocate (xinc)
      deallocate (Bestim)
      deallocate (rhoiestim)
      end subroutine opti_0qtl_multi
!!***

!!****f* m_qtlmap_incidence_multi/init_startpoint
!! NAME
!!   init_startpoint
!! DESCRIPTION
!!
!! NOTE
!!  initialise le point de depart pour la minimisation
!! SOURCE
      subroutine init_startpoint(workStruct,incDesc)
        type(INCIDENCE_TYPE)              ,intent(inout)       :: incDesc
        type(INCIDENCE_GEN_STRUCT)        ,intent(in)          :: workstruct

        integer :: k,ic,jc,ip

        !initialisation d'un point de depart pour les variances
        k=0
        do ic=1,ncar
         do ip=1,np
          k=k+1
          incDesc%par(k)=sqrt(workstruct%sigsquare(workstruct%nqtl,ip,ic))
         end do
        end do
        !initialisation d'un point de depart pour les correlations residuelles
        do ic=2,ncar
         do jc=1,ic-1
          k=k+1
          incDesc%par(k)=workstruct%rhoi(workstruct%nqtl,jc,ic)
         end do
        end do

        incDesc%par(ncar*np+1+ncar*(ncar-1)/2:)=0.d0

      end subroutine init_startpoint
!!***

!!****f* m_qtlmap_incidence_multi/opti_nqtl_multi
!! NAME
!!   opti_nqtl_multi
!! DESCRIPTION
!!
!! NOTE
!!
!! SOURCE

       subroutine opti_nqtl_multi(      nqtl,      &  ! under hypothesis NQTL
                                workStruct,       &  ! variance estimated under NQTL-1 hypothesis
                                 incsol   ,       &  ! incidence solution (estimation of each effect)
                                 lrtsol   ,       &  ! maximum lrt finding at a position
                               FUNCT_MODEL,       &  ! function with a model specific
                                    maxqtl)

      integer                                 ,intent(in)          :: nqtl,maxqtl
      type(INCIDENCE_GEN_STRUCT)              ,intent(inout)       :: workstruct
      type(TYPE_INCIDENCE_SOLUTION)           ,intent(out)         :: incsol
      type(TYPE_LRT_SOLUTION)                 ,intent(out)         :: lrtsol
      external                                                     :: FUNCT_MODEL


      integer :: i

      real (kind=dp) ,dimension(:,:,:),pointer      :: xxx,xinc   ! incidence matrix and temp for testing nuisance effect
      type(INCIDENCE_TYPE)  ,dimension(ncar)        :: listDesc ! description incidence matrix
      type(POSITION_LRT_INCIDENCE)                  :: curPos        ! the position with additional info for the model
      integer :: n,j,chr,k,ic,jc,ip

       ! solution of the estimation
      real (kind=dp) , dimension(:,:) ,pointer      :: Bestim,rhoiestim
      real(kind=dp)                                 :: sigsquareEstime(ncar,np)

      if ( nqtl <= 0 ) then
        call stop_application("Error dev: can not call opti_nqtl_linear with nqtl:"//trim(str(nqtl)))
      end if

      !Position Allocation
      call init_position(nqtl,curPos)

      !initialisation of incidence matrix
      call init_incidence_multi(nqtl,xinc,listDesc,workstruct%type_incidence)
      dataset=>listDesc(1)%dataset

      allocate (Bestim(ncar,ntnivmaxtotal))
      allocate (rhoiestim(ncar,ncar))
      allocate (xxx(ncar,ntnivmaxtotal,ntnivmaxtotal))

      call init_startpoint(workstruct,listDesc(1))

      Bestim=0.d0
      rhoiestim=0.d0
      sigsquareEstime=0.d0

      do i=1,nqtl
         n=0
         j=i
         do chr=1,nchr
           n=n+get_ilong(chr)
           if (i<=n) exit
           j=1
         end do

         if (chr>nchr) call stop_application("Not enough sampling point to detect QTLs")

         curPos%listChr(i)=1 ! attention si pas assez d echantillonage on doit passer au chromosome suivant....
         curPos%listN(i)=i
      end do

!      call build_incidence_matrix(workstruct,curPos,xinc,incidenceDesc,0,0,0)
      Bestim=0.d0
      call add_general_mean_multi(xinc,listDesc)
      workstruct%eff_general_mean=1

      workstruct%listnteff(1)=2
      do i=1,workstruct%nqtl
          !add qtl effect/interaction at position n to estim
          call add_qtleffect_multi(workstruct%listnteff(i),i,curPos%listChr(i),curPos%listN(i),&
          xinc,listDesc,0,workstruct%linear)
          if (i<workstruct%nqtl) workstruct%listnteff(i+1)=listDesc(1)%nteff+1
      end do
      !add polygenic effect
      call add_polygenic_effect_multi(xinc,listDesc)
      !add fixed effect and covariate
      call add_effcov_multi(xinc,listDesc,0,0)

      !initalizing lrtsolution
      call new(nqtl,lrtsol)

      lrtsol%lrtmax=-INIFINY_REAL_VALUE

    !  call gen_loop_opti_nqtl(1,nqtl,curPos,workstruct,xinc,&
     ! incidenceDesc,Bestim,lrtsol,.true.,FUNCT_MODEL)
     !call gen_loop_opti_nqtl_multi(1,nqtl,curPos,workstruct,&
      !         xinc,listDesc,sigsquareEstime,rhoiestim,Bestim,lrtsol,.false.,FUNCT_MODEL)


      call gen_opti_nqtl_multi(nqtl,curPos,workstruct,xinc,listDesc,sigsquareEstime,rhoiestim,Bestim,lrtsol,FUNCT_MODEL)

      ! Compute precision at the maximum LRT in position posx
      do i=1,nqtl
       call change_qtleffect_multi(workstruct%listnteff(i),i,lrtsol%chrmax(i-1),lrtsol%nxmax(i-1),&
       xinc,listDesc,0)
       curPos%listChr(i)=lrtsol%chrmax(i-1)
       curPos%listN(i)=lrtsol%nxmax(i-1)
      end do

 !    call debug_write_incidence(xinc,incidenceDesc)
      call FUNCT_MODEL(xinc,listDesc,curPos,workstruct,sigsquareEstime,rhoiestim,Bestim,&
      .true.,workstruct%performConfusion,xxx,.false.)

  !    print *,"****************************************************************"
      if ( workstruct%performConfusion) then
       call confusion_multi_type1(listDesc,xxx,workstruct)
      end if
      if ( workstruct%performTestNuis ) then
    !    call test_nuisances(ic,incidenceDesc%nbnivest,lrtsol,curPos,workstruct,FUNCT_MODEL)
      end if
      call set_solution_multi(nqtl,workstruct,sigsquareEstime,rhoiestim,Bestim,listDesc,incsol,nqtl,workstruct%listnteff)

      if (size(workstruct%sigsquare,1)>nqtl) then
        do ip=1,np
          do ic=1,ncar
           workstruct%sigsquare(nqtl+1,ip,ic)=sigsquareEstime(ic,ip)
          end do
        end do
      end if

      if (size(workstruct%fnqtl,1)>nqtl) then
          workstruct%fnqtl(nqtl+1)=listDesc(1)%fmax
      end if

      if (size(workstruct%rhoi,1)>nqtl) then
        workstruct%rhoi(nqtl+1,:,:) = rhoiestim
      end if

      call end_incidence_multi(listDesc)

      deallocate (xxx)
      deallocate (xinc)
      deallocate (Bestim)
      deallocate (rhoiestim)
      call end_position(curPos)
      end subroutine opti_nqtl_multi
!!***

!!****f* m_qtlmap_incidence_multi/model_optim_multi_h0
!! NAME
!!   model_optim_multi_h0
!! DESCRIPTION
!!
!! NOTE
!!
!! SOURCE

     subroutine model_optim_multi_h0(xinc,listDesc,workstruct,sigsquareEstime,rhoiestim,Bestim,performPrecision)
         real (kind=dp) , dimension(ncar,nd,ntnivmaxtotal), intent(in)       :: xinc
         type(INCIDENCE_GEN_STRUCT)                   ,intent(inout)    :: workstruct
         real (kind=dp) , dimension(ncar,ntnivmaxtotal)  ,intent(out)        :: Bestim
         real (kind=dp) , dimension(ncar,ncar)      ,intent(out)        :: rhoiestim
         real (kind=dp) , dimension(ncar,np)        ,intent(out)        :: sigsquareEstime
         type(INCIDENCE_TYPE)   , dimension(ncar)      , intent(inout)  :: listDesc
         logical                                    ,intent(in)         :: performPrecision

         real(kind=dp) , dimension(ntnivmaxtotal,ntnivmaxtotal,ncar) :: xxx


         call model_optim_multi_family(xinc,listDesc,workstruct,sigsquareEstime,rhoiestim,Bestim,&
          performPrecision,likelihood_ncar_h0_family,likelihood_ncar_h0_family_withcd,.true.,xxx)

          !stop
         !workstruct%sigsquareEstime=osig*osig

      end subroutine model_optim_multi_h0
!!***

!!****f* m_qtlmap_incidence_multi/model_optim_multi_hn
!! NAME
!!   model_optim_multi_hn
!! DESCRIPTION
!!
!! NOTE
!!
!! SOURCE
      subroutine model_optim_multi_hn(xinc,listDesc,curPos,workstruct,sigsquareEstime,rhoiestim,Bestim,&
                                      performPrecision,tConf,tempForConfusion,invlrt)
         real (kind=dp) , dimension(ncar,nd,ntnivmaxtotal), intent(in)       :: xinc
         type(INCIDENCE_GEN_STRUCT)                   ,intent(inout)    :: workstruct
         type(POSITION_LRT_INCIDENCE)                 ,intent(inout)    :: curPos
         real (kind=dp) , dimension(ncar,ntnivmaxtotal)  ,intent(out)        :: Bestim
         real (kind=dp) , dimension(ncar,ncar)      ,intent(out)        :: rhoiestim
         real (kind=dp) , dimension(ncar,np)        ,intent(out)        :: sigsquareEstime
         type(INCIDENCE_TYPE)   , dimension(ncar)      , intent(inout)  :: listDesc
         logical                                    ,intent(in)         :: performPrecision,tConf
         real (kind=dp),dimension(ntnivmaxtotal,ntnivmaxtotal,ncar) ,intent(out)  :: tempForConfusion
         logical                                          ,intent(in)   :: invlrt

         integer :: i,hypothesis
         character(len=LEN_L) :: dx

         allocate (current_chr(workstruct%nqtl))
         current_chr=curPos%listChr

         call model_optim_multi_family(xinc,listDesc,workstruct,sigsquareEstime,rhoiestim,Bestim,&
           performPrecision,likelihood_ncar_hn_family,likelihood_ncar_hn_family_withcd,tConf,tempForConfusion)
!         print *,"F:",workstruct%fmax


         if ((listDesc(1)%fmax == INIFINY_REAL_VALUE) ) then
            dx=""
            do i=1,workstruct%nqtl-1
              dx=trim(dx)//trim(str(absi(curPos%listChr(i),curPos%listN(i))))//","
            end do

            dx=trim(dx)//str(absi(curPos%listChr(workstruct%nqtl),curPos%listN(workstruct%nqtl)))
            call log_mess("dx ["//trim(dx)// "]. Can not optimize likelihood....The start point is reinitializing",WARNING_DEF)
            call init_startpoint(workStruct,listDesc(1))
            curPos%lrt=0.d0
            listDesc(1)%fmax=0.d0
            return
         end if

         !compute LRT
         if (invLrt) then
            do hypothesis=1,workstruct%nqtl
            curPos%lrt(hypothesis)=-2.d0*(workstruct%fnqtl(hypothesis)-listDesc(1)%fmax)
           end do
         else
           do hypothesis=1,workstruct%nqtl
            curPos%lrt(hypothesis)=-2.d0*(listDesc(1)%fmax-workstruct%fnqtl(hypothesis))
           end do
         end if

         deallocate (current_chr)

      end subroutine model_optim_multi_hn
!!***

!!****f* m_qtlmap_incidence_multi/model_optim_multi_family
!! NAME
!!   model_optim_multi_family
!! DESCRIPTION
!!
!! NOTE
!!
!! SOURCE
    subroutine model_optim_multi_family(xinc,listDesc,workstruct,sigsquareEstime,rhoiestim,Bestim,&
    performPrecision,FUNCT_PART,FUNCT_PART_CENSURE,tConf,tempForConfusion)
         real (kind=dp) , dimension(ncar,nd,ntnivmaxtotal), intent(in)       :: xinc
         type(INCIDENCE_GEN_STRUCT)              ,intent(inout)         :: workstruct
         real (kind=dp) , dimension(ncar,np)        ,intent(out)        :: sigsquareEstime
         real (kind=dp) , dimension(ncar,ntnivmaxtotal)  ,intent(out)        :: Bestim
         real (kind=dp) , dimension(ncar,ncar)      ,intent(out)        :: rhoiestim
         type(INCIDENCE_TYPE)   , dimension(ncar)   , intent(inout),target  :: listDesc
         logical                                    ,intent(in)         :: performPrecision,tConf
         real (kind=dp),dimension(ntnivmaxtotal,ntnivmaxtotal,ncar) ,intent(out)  :: tempForConfusion
         external                                                       :: FUNCT_PART,FUNCT_PART_CENSURE

         integer :: j,i,ip,ifail,ibound,npar
!         real(kind=dp) ,dimension(np+ntnivmax) :: par
         real (kind=dp) , dimension(ntnivmaxtotal,ntnivmaxtotal)   :: XX
         real (kind=dp) , dimension(ntnivmaxtotal,ntnivmaxtotal)   :: triang
         integer                                    :: iuser(1),ix,kd1,kd2,jm,jjm,ic,s,jc
         real (kind=dp)   ,dimension(1)    :: user
         logical       ,dimension(ncar,ntnivmaxtotal)    :: lastvecsol
         real(kind=dp) :: vci_t(ncar,ncar,np),determ_t(np),fp(np),fm(nm)

         !Filter for the mimimization - dim (np,nm,nd)
         logical              , dimension(np,nm,ncar*np+ncar*(ncar-1)/2+ncar*ntnivmaxtotal)      :: filter_inc
         !Filter for calcul on the vci matrix : dim np,
         logical              , dimension(np,ncar*np+ncar*(ncar-1)/2+ncar*ntnivmaxtotal) ,target :: filter_vci
         logical              , dimension(:,:) , pointer :: p_filter_vci


         iuser=0
         user=0
         my_listDesc=>listDesc
         dataset => listDesc(1)%dataset
         allocate (my_xincreduitmul(nd,ntnivmaxtotal,ncar))

         !filter car initialisation
         filter_vci=.false.
         do ip=1,np
          do ic=1,ncar
           filter_vci(ip,(ic-1)*np+ip)=.true.
          end do
         end do

         filter_vci(:,ncar*np+1:ncar*np+ncar*(ncar-1)/2)=.true.


         my_xincreduitmul=0.d0
         do ic=1,ncar
           lastvecsol(ic,:my_listDesc(ic)%ntniv)=my_listDesc(ic)%vecsol(:my_listDesc(ic)%ntniv)
           ! create X'.X matrix from incidence matrix
           call model_XT_X(xinc(ic,:,:),my_listDesc(ic),XX)
           ! Check all parameters to remove from the estimation
           call estim_cholesky(XX,my_listDesc(ic),ntnivmaxtotal,triang)
           ! compute the precision of each parameter
           if (performPrecision) call get_precision(XX,tempForConfusion(:,:,ic),my_listDesc(ic))
           call set_corrxinc(xinc(ic,:,:),my_listDesc(ic),my_xincreduitmul(:,:,ic))
           !optimisation : on sauvegarde les index des elements non nul de la matrice reduite
           call fill_nonull_elements(my_listDesc(ic),size(my_xincreduitmul,1),size(my_xincreduitmul,2),my_xincreduitmul(:,:,ic))
           !call debug_write_incidence(xinc(ic,:,:),my_listDesc(ic))
         end do

           ! Optimisation de la vraisemblance a la position dx
         ifail=1
         ibound=0
         npar=ncar*np+ncar*(ncar-1)/2

         j=ncar*np+ncar*(ncar-1)/2
         do ic=1,ncar
          npar=npar+my_listDesc(ic)%nbnivest

          if (count(lastvecsol(ic,:))>0) then ! autre que initialisation
           do i=1,my_listDesc(ic)%ntniv
            if(my_listDesc(ic)%vecsol(i))then
             j=j+1
             if(.not. lastvecsol(ic,i)) my_listDesc(ic)%par(j)=0.d0
            end if
           end do
          end if
         end do
       !  allocate (filter_inc(np,npar))
         s=np*ncar+ncar*(ncar-1)/2
         filter_inc=.true.
         filter_inc(:,:,1:np*ncar)=.false.
         filter_inc(:,:,np*ncar+1:s)=.true. !rho
         do ic=1,ncar
          do ip=1,np
           filter_inc(ip,nmp(ip)+1:nmp(ip+1),(ic-1)*np+ip)=.true.
           jjm=0
           do jm=nmp(ip)+1,nmp(ip+1)
             jjm=jjm+1
             if (dataset%lSires(ip)%full_sib(jjm)%lastKd>0) then
             kd1=dataset%lSires(ip)%full_sib(jjm)%firstkd
             kd2=dataset%lSires(ip)%full_sib(jjm)%lastkd
        filter_inc(ip,jm,s+1:s+my_listDesc(ic)%nbnivest)=any(my_xincreduitmul(kd1:kd2,:my_listDesc(ic)%nbnivest,ic)/=0.d0,dim=1)
             else
          !     print *,'pas de desc....'
               filter_inc(ip,jm,s+1:s+my_listDesc(ic)%nbnivest)=.false.
             end if
           end do
          end do
          s=s+my_listDesc(ic)%nbnivest
         end do

         if ( datasetUser%na>0 ) then
          !prise en compte des donnees censures
          call minimizing_funct_family(npar,ibound,FUNCT_PART_CENSURE,filter_inc,fm,fp,&
          my_listDesc(1)%borni,my_listDesc(1)%borns,my_listDesc(1)%par,listDesc(1)%fmax,iuser,user,ifail)
         else
          p_filter_vci=>filter_vci
          !pas de donnee censure=> optimisation pour le calcul de l'inverse de matrice de covariance
          call minimizing_funct_family_multi(npar,ibound,FUNCT_PART,filter_inc,p_filter_vci,vci_t,determ_t,fm,fp,&
          my_listDesc(1)%borni,my_listDesc(1)%borns,my_listDesc(1)%par,listDesc(1)%fmax,iuser,user,ifail)
         end if

         Bestim=0.d0
         s=np*ncar+ncar*(ncar-1)/2
         !getting standart deviation
         do ic=1,ncar
          do ip=1,np
           sigsquareEstime(ic,ip)=my_listDesc(1)%par((ic-1)*np+ip)*my_listDesc(1)%par((ic-1)*np+ip)
          end do
           !The solution
          Bestim(ic,:my_listDesc(ic)%nbnivest)=my_listDesc(1)%par(s+1:s+my_listDesc(ic)%nbnivest)
          s=s+my_listDesc(ic)%nbnivest
         end do

         rhoiestim=0.d0
         s=ncar*np

          do ic=2,ncar
          do jc=1,ic-1
           s=s+1
           rhoiestim(jc,ic)=my_listDesc(1)%par(s)
           rhoiestim(ic,jc)=rhoiestim(jc,ic)
          end do
         end do

         deallocate (my_xincreduitmul)

      end subroutine model_optim_multi_family
!!***

!!****f* m_qtlmap_incidence_multi/get_inv_residual_covariance_matrix
!! NAME
!!   get_inv_residual_covariance_matrix
!! DESCRIPTION
!!
!! NOTE
!!
!! SOURCE
   subroutine get_inv_residual_covariance_matrix(ip,n,x,vci,determ,ok)
      integer         , intent(in)                   :: ip,n
      real (kind=dp)  ,dimension(n)   , intent(in)   :: x
      real(kind=dp),dimension(ncar,ncar),intent(out) :: VCI
      real(kind=dp),                     intent(out) :: determ
      logical                           ,intent(out) :: ok
      real(kind=dp),dimension(ncar,ncar)      :: rhoc
      real(kind=dp),dimension(ncar+1,ncar)    :: VCIInverse

      integer :: ic,jc,k,irank

      ok=.true.
      !RHO(I,J) avec i/=j => x((i-1)*(ncar-i+1))
      k=0
      do ic=2,ncar
        do jc=1,ic-1
          k=k+1
          rhoc(jc,ic)=x(ncar*np+k)
          rhoc(ic,jc)=rhoc(jc,ic)
        end do
      end do

      !VCi : residual covariance matrix from sire I
      !         | SIGi1^2  SIGi1*SIGi2*RHO(1,2) ....  SIGi1*SIGip*RHO(1,p)
      !         | ..       SIGi2^2
      !    VCi  | ..                ...
      !         | ..                       ...
      !         | ..
      !         | SIGi1*SIGip*RHO(1,p)        SIGip^2

      do ic=1,ncar
        do jc=ic,ncar
            VCI(ic,jc)=x((ic-1)*np+ip)*x((jc-1)*np+ip)
            if ( ic /= jc ) then
              VCI(ic,jc)=VCI(ic,jc)*rhoc(ic,jc)
              VCI(jc,ic)=VCI(ic,jc)
            end if
        end do
      end do

      !Calcul Determinant de la matrice de covariance residuelle du pere ip
      !--------------------------------------------------------------------
      irank=0
      determ=0
     ! call MATH_QTLMAP_F03ABF(VCI,ncar,ncar,determ,irank)
!      if (irank /= 0) then
!        print *,"mauvais determinant matrice"
!        ok=.false.
!        return
!      end if
      !Calcul Inverse de la matrice de covariance residuelle du pere ip
      !-----------------------------------------------------------------
      VCIInverse(:ncar,:ncar)=VCI
      CALL MATH_QTLMAP_INVDETMATSYM(ncar,VCIInverse,ncar+1,determ,irank)
    !  call MATH_QTLMAP_F01ADF(ncar,VCIInverse,ncar+1,irank)
       if (irank /= 0) then
        print *,"mauvaise inversion matrice"
        ok=.false.
        return
      end if

      do ic=1,ncar
        do jc=1,ic
            VCI(ic,jc)=VCIInverse(ic+1,jc)
            VCI(jc,ic)=VCI(ic,jc)
        end do
      end do



   end subroutine get_inv_residual_covariance_matrix
!!***

!!****f* m_qtlmap_incidence_multi/likelihood_ncar_h0_family
!! NAME
!!   likelihood_ncar_h0_family
!! DESCRIPTION
!!
!! NOTE
!!
!! SOURCE
  subroutine likelihood_ncar_h0_family(ip,jm,n,x,vci,determ,f,iuser,user)
      integer         , intent(in)                  :: ip,jm,n
      real (kind=dp)      ,dimension(n), intent(in) :: x
      real (kind=dp)  , intent(inout)               :: f
      integer ,       dimension(1), intent(inout)      :: iuser
      real (kind=dp)      ,dimension(1), intent(inout) :: user
      real (kind=dp)                   ,intent(in) :: determ
      real(kind=dp),dimension(ncar,ncar) ,intent(in)    :: VCI
      real (kind=dp) :: vpf
      integer        :: ifem
      real(kind=dp),dimension(dataset%nkd_max_by_fam,ncar) :: V

!
      integer :: ic,jc,s,nbnivest,kd1,kd2,kd,na
      logical :: ok,valide

     ! print *,x
      f = 0.d0
      V = 0.d0
      s=np*ncar+ncar*(ncar-1)/2

      ifem=jm-nmp(ip)
      kd1=dataset%lSires(ip)%full_sib(ifem)%firstkd
      kd2=dataset%lSires(ip)%full_sib(ifem)%lastkd
      na = kd2-kd1+1
      if (kd2<kd1) then
       f=0
       return
      end if

      ! Dans la famille du pere ip, on calcul le Y (performance auquel on soustrait l'ensemble des effets)
      ! pour chaque caractere
      do ic=1,ncar
        nbnivest=my_listdesc(ic)%nbnivest
        call matmul_incidence(kd1,kd2,my_listdesc(ic),nd,ntnivmaxtotal,&
         My_XINCREDUITMUL(:,:,ic),X(s+1:s+nbnivest),dataset%nkd_max_by_fam,1,V(:,ic))
        V(:na,ic) = my_listdesc(ic)%dataset%Y(ic,kd1:kd2)-V(:na,ic)
        s=s+my_listdesc(ic)%nbnivest
      end do


      vpf=0
      !calcul de la vraissemblance de plein frere
      do kd=1,na
        vpf=vpf+dot_product(matmul(V(kd,:),VCI),V(kd,:))
      end do
      !finallement, la vraissemblance de la famille ip/jm:
      f=+0.5*vpf+dble(kd2-kd1+1)*0.5*log(determ)
  end subroutine likelihood_ncar_h0_family
!!***

!!****f* m_qtlmap_incidence_multi/likelihood_ncar_hn_family
!! NAME
!!   likelihood_ncar_hn_family
!! DESCRIPTION
!!
!! NOTE
!!    multivariate analysis
!!     (indicecar-1)*np +ip  : SIG ip,indice_caracetere
!!     ncar*np + (ic-1)*ncar+jc        : RHO (ic,jc) residual correlation between two traits....
!!     ncar*np + ncar*ncar + nbnivest
!! SOURCE
  subroutine likelihood_ncar_hn_family(ip,jm,n,x,VCI,determ,f,iuser,user)
      integer         , intent(in)                                             :: ip,jm,n
      real (kind=dp)      ,dimension(n), intent(in)                            :: x
      real (kind=dp)  , intent(inout)                                          :: f
      integer ,       dimension(1), intent(inout)                              :: iuser
      real (kind=dp) ,dimension(1), intent(inout)                              :: user
      real(kind=dp),dimension(ncar,ncar)                           ,intent(in) :: VCI
      real(kind=dp)                                                ,intent(in) :: determ

      real(kind=dp),dimension(dataset%nkd_max_by_fam,ncar) :: V


!
      integer        :: jjm,ig(my_listdesc(1)%nqtl),ifem,indf,indm,iq,ngg,iig,z,irank,ic,jc
      real(kind=dp)  :: effm,pbr,vmere,vpf
      integer        :: kd1,kd2,jj,nqtl,nbnivest,s,kd,na
      logical        :: ok,valide

      f=0.d0
      nqtl=my_listdesc(1)%nqtl

      jjm=jm-nmp(ip)

      if (dataset%lSires(ip)%full_sib(jjm)%lastKd>0) then
          kd1=dataset%lSires(ip)%full_sib(jjm)%firstKd
          kd2=dataset%lSires(ip)%full_sib(jjm)%lastKd
          na = kd2-kd1+1
          effm=dble(na)
      else
          return
      end if

      vmere=0.d0
      if ( estime_multi(jm) ) ifem=iam_multi(repfem(jm))
          !ngg : le nombre de genotype possible sur tous les qtls...
          ngg=1
          do iq=1,nqtl
             ig(iq)=ngenom(current_chr(iq),jm)+1
             ngg=ngg*(ngenom(current_chr(iq),jm+1)-ngenom(current_chr(iq),jm))
          end do
          !pour toutes les combinaisons possibles des genotypes
          do iig=1,ngg
             pbr=1
             !on modifie la matrice d incidence pour les n qtl
             do iq=1,nqtl
              do ic=1,ncar
               indm=my_listdesc(ic)%ntniv_qtlsires(iq)+ip-1
               if ( estime_multi(jm) ) then
                !Si la mere est estimable, on place dans la matrice les
                !pdds dam et pdds sires (si ces effets sont estimables)
                indf=my_listdesc(ic)%ntniv_qtldams(iq)+ifem-1
                if ( my_listdesc(ic)%vecsol(indf)) then
                 indf=my_listdesc(ic)%corr_niv_nivb(indf) ! vrai correspondance dans le tableau si estimable
                 My_XINCREDUITMUL(kd1:kd2,indf,ic)=dataset%lSires(ip)%full_sib(jjm)%pmt(iq,(ig(iq)-ngenom(current_chr(iq),jm)),:)
                end if
                if ( my_listdesc(ic)%vecsol(indm) ) then
                 indm=my_listdesc(ic)%corr_niv_nivb(indm) ! vrai correspondance dans le tableau si estimable
                 My_XINCREDUITMUL(kd1:kd2,indm,ic)=dataset%lSires(ip)%full_sib(jjm)%ppt(iq,(ig(iq)-ngenom(current_chr(iq),jm)),:)
                end if
               else
               !la mere n est pas estimable, on place seulement les pdds males
                if ( my_listdesc(ic)%vecsol(indm) ) then
                 indm=my_listdesc(ic)%corr_niv_nivb(indm) ! vrai correspondance dans le tableau si estimable
                 My_XINCREDUITMUL(kd1:kd2,indm,ic)=dataset%lSires(ip)%ppt(iq,kd1:kd2)
                end if
               end if
              end do ! ic
              !print *,ig(iq)-ngenom(current_chr(iq),jm),ig(iq)
              pbr=pbr*probg(current_chr(iq),ig(iq))
             end do ! iq
             ! Dans la famille du pere ip, on calcul le Y (performance auquel on soustrait l'ensemble des effets)
             ! pour chaque caractere
             s=np*ncar+ncar*(ncar-1)/2
             do ic=1,ncar
              nbnivest=my_listdesc(ic)%nbnivest
              call matmul_incidence(kd1,kd2,my_listdesc(ic),nd,ntnivmaxtotal,&
                 My_XINCREDUITMUL(:,:,ic),X(s+1:s+nbnivest),dataset%nkd_max_by_fam,1,V(:,ic))
              V(:na,ic) = my_listdesc(ic)%dataset%Y(ic,kd1:kd2)-V(:na,ic)
              s=s+my_listdesc(ic)%nbnivest
             end do
             vpf=0
             !calcul de la vraissemblance de plein frere
             do kd=1,na
              vpf=vpf+dot_product(matmul(V(kd,:),VCI),V(kd,:))
             end do
             vmere=vmere+pbr*dexp(-0.5d0*vpf)

             ! on increment
             ok=.true.
             do iq=1,nqtl
              if (ok) then
                if ((ig(iq) < ngenom(current_chr(iq),jm+1))) then
                 ig(iq)=ig(iq)+1
                 ok=.false.
               end if
             end if
            end do
         end do ! iig


          if (vmere == 0) then
             f=INIFINY_REAL_VALUE
          else
             !finallement, la vraissemblance de la famille ip/jm:
             f=f-dlog(vmere)+dble(kd2-kd1+1)*0.5*log(determ)
          end if
      ! end do
        !  print *,ip,jm,f
        !  stop

  end subroutine likelihood_ncar_hn_family
!!***

!!****f* m_qtlmap_incidence_multi/likelihood_ncar_h0_family_withcd
!! NAME
!!   likelihood_ncar_h0_family_withcd
!! DESCRIPTION
!!
!! NOTE
!!    multivariate analysis
!!     (indicecar-1)*np +ip  : SIG ip,indice_caracetere
!!     ncar*np + (ic-1)*ncar+jc        : RHO (ic,jc) residual correlation between two traits....
!!     ncar*np + ncar*ncar + nbnivest
!! SOURCE
  subroutine likelihood_ncar_h0_family_withcd(ip,jm,n,x,f,iuser,user)
      integer         , intent(in)                  :: ip,jm,n
      real (kind=dp)      ,dimension(n), intent(in) :: x
      real (kind=dp)  , intent(inout)               :: f
      integer ,       dimension(1), intent(inout)      :: iuser
      real (kind=dp)      ,dimension(1), intent(inout) :: user


      real (kind=dp),dimension(dataset%nkd_max_by_fam) :: determ
      real(kind=dp),dimension(ncar,ncar,dataset%nkd_max_by_fam)    :: VCI
      real (kind=dp) :: vpf
      integer        :: ifem,kkk
      real(kind=dp),dimension(dataset%nkd_max_by_fam,ncar) :: V

!
      integer :: ic,jc,s,nbnivest,kd1,kd2,kd,na
      logical :: ok,valide

      determ=0.d0
      kd=1
      do kkk=ndm(jm)+1,ndm(jm+1)
        if ( count(presentc(:,kkk)) == ncar ) then
           call get_inv_residual_covariance_matrix_cd(ip,kkk,n,x,vci(:,:,kd),determ(kd),ok)
           if ( .not. ok ) then
              f=INIFINY_REAL_VALUE
              return
           end if
           kd = kd + 1
        end if
      end do

      f = 0.d0
      V = 0.d0
      s=np*ncar+ncar*(ncar-1)/2

      ifem=jm-nmp(ip)

      kd1=dataset%lSires(ip)%full_sib(ifem)%firstkd
      kd2=dataset%lSires(ip)%full_sib(ifem)%lastkd
      na = kd2-kd1+1

      if (kd2<kd1) then
       f=0
       return
      end if

      ! Dans la famille du pere ip, on calcul le Y (performance auquel on soustrait l'ensemble des effets)
      ! pour chaque caractere
      do ic=1,ncar
        nbnivest=my_listdesc(ic)%nbnivest
        call matmul_incidence(kd1,kd2,my_listdesc(ic),nd,ntnivmaxtotal,&
           My_XINCREDUITMUL(:,:,ic),X(s+1:s+nbnivest),dataset%nkd_max_by_fam,1,V(:,ic))
        V(:na,ic) = my_listdesc(ic)%dataset%Y(ic,kd1:kd2)-V(:na,ic)
        s=s+my_listdesc(ic)%nbnivest
      end do

      vpf=0
      !calcul de la vraissemblance de plein frere
      do kd=1,na
        vpf=vpf+dot_product(matmul(V(kd,:),VCI(:,:,kd)),V(kd,:))
      end do

      !finallement, la vraissemblance de la famille ip/jm:
      f=+0.5*vpf+0.5*sum(log(determ(:na)))
      !print *,f

  end subroutine likelihood_ncar_h0_family_withcd
!!***

!!****f* m_qtlmap_incidence_multi/get_inv_residual_covariance_matrix_cd
!! NAME
!!   get_inv_residual_covariance_matrix_cd
!! DESCRIPTION
!!
!! NOTE
!!
!! SOURCE
   subroutine get_inv_residual_covariance_matrix_cd(ip,kd,n,x,vci,determ,ok)
      integer         , intent(in)                   :: ip,kd,n
      real (kind=dp)  ,dimension(n)   , intent(in)   :: x
      real(kind=dp),dimension(ncar,ncar),intent(out) :: VCI
      real(kind=dp),                     intent(out) :: determ
      logical                           ,intent(out) :: ok
      real(kind=dp),dimension(ncar,ncar)      :: rhoc
      real(kind=dp),dimension(ncar+1,ncar)    :: VCIInverse

      integer :: ic,jc,k,irank

      ok=.true.
      !RHO(I,J) avec i/=j => x((i-1)*(ncar-i+1))
      k=0
      do ic=2,ncar
        do jc=1,ic-1
          k=k+1
          rhoc(jc,ic)=x(ncar*np+k)
          rhoc(ic,jc)=rhoc(jc,ic)
        end do
      end do

      !VCi : residual covariance matrix from sire I
      !         | SIGi1^2  SIGi1*SIGi2*RHO(1,2) ....  SIGi1*SIGip*RHO(1,p)
      !         | ..       SIGi2^2
      !    VCi  | ..                ...
      !         | ..                       ...
      !         | ..
      !         | SIGi1*SIGip*RHO(1,p)        SIGip^2
      !print *,'kd:',kd,cd(1,kd),my_corcd(1,2,kd)
      do ic=1,ncar
        do jc=ic,ncar
            VCI(ic,jc)=x((ic-1)*np+ip)*x((jc-1)*np+ip)
            if ( ic /= jc ) then
              VCI(ic,jc)=VCI(ic,jc)*rhoc(ic,jc)*datasetUser%corcd(ic,jc,kd)
              VCI(jc,ic)=VCI(ic,jc)
            else
              VCI(ic,jc)=VCI(ic,jc)*cd(ic,kd)
            end if
        end do
      end do

      !Calcul Determinant de la matrice de covariance residuelle du pere ip
      !--------------------------------------------------------------------
      irank=0
      determ=0
     ! call MATH_QTLMAP_F03ABF(VCI,ncar,ncar,determ,irank)
!      if (irank /= 0) then
!        print *,"mauvais determinant matrice"
!        ok=.false.
!        return
!      end if
      !Calcul Inverse de la matrice de covariance residuelle du pere ip
      !-----------------------------------------------------------------
      VCIInverse(:ncar,:ncar)=VCI
      CALL MATH_QTLMAP_INVDETMATSYM(ncar,VCIInverse,ncar+1,determ,irank)
    !  call MATH_QTLMAP_F01ADF(ncar,VCIInverse,ncar+1,irank)
       if (irank /= 0) then
        print *,"mauvaise inversion matrice"
        ok=.false.
        return
      end if

      do ic=1,ncar
        do jc=1,ic
            VCI(ic,jc)=VCIInverse(ic+1,jc)
            VCI(jc,ic)=VCI(ic,jc)
        end do
      end do



   end subroutine get_inv_residual_covariance_matrix_cd
!!***

!!****f* m_qtlmap_incidence_multi/likelihood_ncar_hn_family_withcd
!! NAME
!!   likelihood_ncar_hn_family_withcd
!! DESCRIPTION
!!
!! NOTE
!!    multivariate analysis
!!     (indicecar-1)*np +ip  : SIG ip,indice_caracetere
!!     ncar*np + (ic-1)*ncar+jc        : RHO (ic,jc) residual correlation between two traits....
!!     ncar*np + ncar*ncar + nbnivest
!! SOURCE
  subroutine likelihood_ncar_hn_family_withcd(ip,jm,n,x,f,iuser,user)
      integer         , intent(in)                                             :: ip,jm,n
      real (kind=dp)      ,dimension(n), intent(in)                            :: x
      real (kind=dp)  , intent(inout)                                          :: f
      integer ,       dimension(1), intent(inout)                              :: iuser
      real (kind=dp) ,dimension(1), intent(inout)                              :: user


      real(kind=dp),dimension(ncar,ncar,dataset%nkd_max_by_fam)  :: VCI
      real(kind=dp),dimension(dataset%nkd_max_by_fam)            :: determ
      real(kind=dp),dimension(dataset%nkd_max_by_fam,ncar)       :: V
!
      integer        :: jjm,ig(my_listdesc(1)%nqtl),ifem,indf,indm,iq,ngg,iig,z,irank,ic,jc
      real(kind=dp)  :: effm,pbr,vmere,vpf
      integer        :: kd1,kd2,jj,nqtl,nbnivest,s,kd,kkk,na
      logical        :: ok,valide

       determ=0.d0
       kd=1
       do kkk=ndm(jm)+1,ndm(jm+1)
         if ( count(presentc(:,kkk)) == ncar ) then
            call get_inv_residual_covariance_matrix_cd(ip,kkk,n,x,vci(:,:,kd),determ(kd),ok)
            if ( .not. ok ) then
               f=INIFINY_REAL_VALUE
               return
            end if
            kd = kd + 1
         end if
       end do

      f=0.d0
      nqtl=my_listdesc(1)%nqtl

      jjm=jm-nmp(ip)

      if (dataset%lSires(ip)%full_sib(jjm)%lastKd>0) then
          kd1=dataset%lSires(ip)%full_sib(jjm)%firstKd
          kd2=dataset%lSires(ip)%full_sib(jjm)%lastKd
          na = kd2-kd1+1
          effm=dble(na)
      else
          return
      end if

      vmere=0.d0
      if ( estime_multi(jm) ) ifem=iam_multi(repfem(jm))
          !ngg : le nombre de genotype possible sur tous les qtls...
          ngg=1
          do iq=1,nqtl
             ig(iq)=ngenom(current_chr(iq),jm)+1
             ngg=ngg*(ngenom(current_chr(iq),jm+1)-ngenom(current_chr(iq),jm))
          end do
          !pour toutes les combinaisons possibles des genotypes
          do iig=1,ngg
             pbr=1
             !on modifie la matrice d incidence pour les n qtl
             do iq=1,nqtl
              do ic=1,ncar
               indm=my_listdesc(ic)%ntniv_qtlsires(iq)+ip-1
               if ( estime_multi(jm) ) then
                !Si la mere est estimable, on place dans la matrice les
                !pdds dam et pdds sires (si ces effets sont estimables)
                indf=my_listdesc(ic)%ntniv_qtldams(iq)+ifem-1
                if ( my_listdesc(ic)%vecsol(indf)) then
                 indf=my_listdesc(ic)%corr_niv_nivb(indf) ! vrai correspondance dans le tableau si estimable
                 My_XINCREDUITMUL(kd1:kd2,indf,ic)=dataset%lSires(ip)%full_sib(jjm)%pmt(iq,(ig(iq)-ngenom(current_chr(iq),jm)),:)
                end if
                if ( my_listdesc(ic)%vecsol(indm) ) then
                 indm=my_listdesc(ic)%corr_niv_nivb(indm) ! vrai correspondance dans le tableau si estimable
                 My_XINCREDUITMUL(kd1:kd2,indm,ic)=dataset%lSires(ip)%full_sib(jjm)%ppt(iq,(ig(iq)-ngenom(current_chr(iq),jm)),:)
                end if
               else
               !la mere n est pas estimable, on place seulement les pdds males
                if ( my_listdesc(ic)%vecsol(indm) ) then
                 indm=my_listdesc(ic)%corr_niv_nivb(indm) ! vrai correspondance dans le tableau si estimable
                 My_XINCREDUITMUL(kd1:kd2,indm,ic)=dataset%lSires(ip)%ppt(iq,kd1:kd2)
                end if
               end if
              end do ! ic
              !print *,ig(iq)-ngenom(current_chr(iq),jm),ig(iq)
              pbr=pbr*probg(current_chr(iq),ig(iq))
             end do ! iq
             ! Dans la famille du pere ip, on calcul le Y (performance auquel on soustrait l'ensemble des effets)
             ! pour chaque caractere
             s=np*ncar+ncar*(ncar-1)/2
             do ic=1,ncar
              nbnivest=my_listdesc(ic)%nbnivest
              call matmul_incidence(kd1,kd2,my_listdesc(ic),nd,ntnivmaxtotal,&
                My_XINCREDUITMUL(:,:,ic),X(s+1:s+nbnivest),dataset%nkd_max_by_fam,1,V(:,ic))
              V(:na,ic) = my_listdesc(ic)%dataset%Y(ic,kd1:kd2)-V(:na,ic)
              s=s+my_listdesc(ic)%nbnivest
             end do
             vpf=0
             !calcul de la vraissemblance de plein frere
             do kd=1,na
                vpf=vpf+dot_product(matmul(V(kd,:),vci(:,:,kd)),V(kd,:))
             end do
             vmere=vmere+pbr*dexp(-0.5d0*vpf)
             ! on increment
             ok=.true.
             do iq=1,nqtl
              if (ok) then
                if ((ig(iq) < ngenom(current_chr(iq),jm+1))) then
                 ig(iq)=ig(iq)+1
                 ok=.false.
               end if
             end if
            end do
         end do ! iig


          if (vmere == 0) then
             f=INIFINY_REAL_VALUE
          else
             !finallement, la vraissemblance de la famille ip/jm:
             f=f-dlog(vmere)+0.5*sum(log(determ(:na)))
          end if

  end subroutine likelihood_ncar_hn_family_withcd
!!***

!!****f* m_qtlmap_incidence_multi/gen_loop_opti_nqtl_multi
!! NAME
!!   gen_loop_opti_nqtl_multi
!! DESCRIPTION
!!
!! NOTE
!!
!! SOURCE
recursive subroutine gen_loop_opti_nqtl_multi(iqtl,      &
                                    nqtl,      &
                                    curPos,    &
                                    workstruct,&
                                    xinc,      &
                                    listDesc  , &
                                    sigsquareEstimeMax,&
                                    rhoiestimMax,      &
                                    BestimMax,     &
                                    lrtsol,        &
                                    printpercent,  &
                                    FUNCT_MODEL)
          integer                                 , intent(in)    :: nqtl  ! under hypothesis nqtl
          integer                                 , intent(in)    :: iqtl  ! iteration of inside loop
          type(POSITION_LRT_INCIDENCE)             ,intent(inout) :: curPos
          type(INCIDENCE_GEN_STRUCT)              ,intent(inout)  :: workstruct
          real (kind=dp) , dimension(ncar,nd,ntnivmaxtotal) , intent(inout) :: xinc  ! incidence matrix
          type(INCIDENCE_TYPE)   , dimension(ncar)   , intent(inout) :: listDesc ! description of the incidence matrix
          real (kind=dp) , dimension(ncar,ntnivmaxtotal) , intent(inout)  :: BestimMax      ! estimation of parameter on the maximum finded
          real (kind=dp) , dimension(ncar,ncar)      ,intent(out)    :: rhoiestimMax
          real (kind=dp) , dimension(ncar,np) , intent(inout)     :: sigsquareEstimeMax      ! estimation of parameter on the maximum finded
          type(TYPE_LRT_SOLUTION)               ,intent(inout)    :: lrtsol
          logical                              , intent(in)       :: printpercent
          external                                                :: FUNCT_MODEL


          real (kind=dp) , dimension(ncar,ntnivmaxtotal)        :: Bestim
          real (kind=dp) , dimension(ncar,np)              :: sigsquareEstime
          real (kind=dp) , dimension(ncar,ncar)            :: rhoiestim
          integer :: n,hypothesis,chr,chr_start,iip,ifem,ii,lnpo,chr1,n1,ip
          logical hypotheseOne, hypotheseTwo
          real (kind=dp) ,dimension(:,:,:),pointer      :: xxx

          allocate (xxx(ncar,ntnivmaxtotal,ntnivmaxtotal))
          ! print *,"       **************    loop_opti_nqtl ***********************"
       !   incidenceDescSave = incidenceDesc
          if ( iqtl == nqtl ) then
             hypotheseOne = ( lrtsol%nqtl == 1 )
             hypotheseTwo = ( lrtsol%nqtl == 2 )
             if (nqtl >= 2 ) then
               chr_start = curPos%listChr(nqtl-1)
             else
               chr_start = 1
             end if

             do chr=chr_start,nchr
               ! Step above the chromosome
               curPos%listChr(nqtl)=chr
               if ( (nqtl >= 2) .and. (chr == chr_start ) ) then
                 n = curPos%listN(nqtl-1)
               else
                 n=0
               end if

             do while (n < get_npo(chr))
               n=n+1
               curPos%listN(nqtl)=n
              ! if (nqtl == 3)print *,"chr:",chr," n:",n
                !
               !add qtl effect/interaction at position n to estim
               call change_qtleffect_multi(workstruct%listnteff(iqtl),iqtl,chr,n,xinc,listDesc,0)
               !call debug_write_incidence(xinc,incidenceDesc)
!                do ii=1,ncar
!             call debug_write_incidence(xinc(ii,:,:),listDesc(ii))
!           end do
               !call model
               call FUNCT_MODEL(xinc,listDesc,curPos,workstruct,sigsquareEstime,rhoiestim,Bestim,.false.,.false.,xxx,.false.)

               if ( hypotheseOne ) then
                  lrtsol%lrt1(chr,n)=curPos%lrt(1)
                  iip=1
                  do ip=1,np
                    if ( listDesc(1)%vecsol(1+ip)) then
                     iip=iip+1
                     ! ATTENTION VALIDE DANS LE CAS SANS INTERACTION AVEC QTL
                     lrtsol%pater_eff(chr,ip,n)=Bestim(1,iip)
                     end if
                  end do
                  ifem=0
                  do ii=1,nm
                    if ( estime_multi(ii) ) then
                      ifem=ifem+1
                      if ( listDesc(1)%vecsol(np+1+ifem) ) then
                        iip=iip+1
                        ! ATTENTION VALIDE DANS LE CAS SANS INTERACTION AVEC QTL
                        lrtsol%mater_eff(chr,ifem,n)=Bestim(1,iip)
                      end if
                    end if
                  end do
               end if
               if ( hypotheseTwo ) then
               !   print *,n,size(lrtsol%lrt1_2,1),listChr(nqtl-1),listChr(nqtl-2),&
              !    size(lrtsol%lrt1_2,2),size(lrtsol%lrt1_2,3),size(lrtsol%lrt1_2,4)
                  chr1=curPos%listChr(1)
                  n1  = curPos%listN(1)
                  lrtsol%lrt1_2(chr1,chr,n1,n)=curPos%lrt(2)
                  lrtsol%lrt0_2(chr1,chr,n1,n)=curPos%lrt(1)

               end if
               !print *,curPos%listDx,curPos%lrt(nqtl)
               !keep maximum
               if(lrtsol%lrtmax(nqtl-1) < curPos%lrt(nqtl)) then
                 lrtsol%nxmax=curPos%listN
                 lrtsol%chrmax=curPos%listChr
                 BestimMax = Bestim
                 sigsquareEstimeMax=sigsquareEstime
                 rhoiestimMax=rhoiestim
                 lrtsol%lrtmax=curPos%lrt
                 !print *,"max sigsquareestime:",workstruct%sigsquareEstime
               end if
              end do
             end do
            else

             if ( iqtl-2 >= 0  ) then
               chr_start=curPos%listChr(iqtl-1)
             else
               chr_start=1
             end if

             do chr=chr_start,nchr
             ! Step above the chromosome

             if ( iqtl-2 >= 0  .and. chr == chr_start) then
                n=curPos%listN(iqtl-1)
             else
                n=0
             end if

             curPos%listChr(iqtl)=chr
             lnpo=get_npo(chr)
             do while (n < lnpo) !ix=startx,ilong,pas
               if (printpercent) call log_mess( trim(str((float(n)/float(lnpo))*100.d0))//"%", VERBOSE_DEF )
               n=n+1
               curPos%listN(iqtl)=n
               !add qtl effect/interaction at position n to estim
                call change_qtleffect_multi(workstruct%listnteff(iqtl),iqtl,chr,n,xinc,listDesc,0)

               ! ****recursivite ****
               call gen_loop_opti_nqtl_multi(iqtl+1,nqtl,curPos,workstruct,&
               xinc,listDesc,sigsquareEstimeMax,rhoiestimMax,BestimMax,lrtsol,.false.,FUNCT_MODEL)

             end do
            end do
         end if
         deallocate (xxx)


     end subroutine gen_loop_opti_nqtl_multi
!!***

!!****f* m_qtlmap_incidence_multi/gen_opti_nqtl_multi
!! NAME
!!   gen_opti_nqtl_multi
!! DESCRIPTION
!!
!! NOTE
!!
!! SOURCE
     subroutine gen_opti_nqtl_multi( nqtl,      &
                                    curPosMax,    &
                                    workstruct,&
                                    xinc,      &
                                    listDescMax  , &
                                    sigsquareEstimeMax,&
                                    rhoiestimMax,      &
                                    BestimMax,     &
                                    lrtsol,        &
                                    FUNCT_MODEL)
          integer                                 , intent(in)    :: nqtl  ! under hypothesis nqtl
          type(POSITION_LRT_INCIDENCE)             ,intent(inout) :: curPosMax
          type(INCIDENCE_GEN_STRUCT)              ,intent(inout)  :: workstruct
          real (kind=dp) , dimension(ncar,nd,ntnivmaxtotal) , intent(inout) :: xinc  ! incidence matrix
          type(INCIDENCE_TYPE)   , dimension(ncar)   , intent(inout) :: listDescMax ! description of the incidence matrix
          real (kind=dp) , dimension(ncar,ntnivmaxtotal) , intent(inout)  :: BestimMax      ! estimation of parameter on the maximum finded
          real (kind=dp) , dimension(ncar,ncar)      ,intent(out)    :: rhoiestimMax
          real (kind=dp) , dimension(ncar,np) , intent(inout)     :: sigsquareEstimeMax      ! estimation of parameter on the maximum finded
          type(TYPE_LRT_SOLUTION)               ,intent(inout)    :: lrtsol
          external                                                :: FUNCT_MODEL


          real (kind=dp) , dimension(ncar,ntnivmaxtotal)        :: Bestim
          real (kind=dp) , dimension(ncar,np)              :: sigsquareEstime
          real (kind=dp) , dimension(ncar,ncar)            :: rhoiestim

          real (kind=dp) ,dimension(:,:,:),pointer         :: xxx
          real (kind=dp) , dimension(:,:,:),allocatable :: xinc2

          logical :: hypotheseOne,hypotheseTwo,ok
          integer :: i,chr,nGL,nGLTotal,nGLFact,InGL,nlinMax,nlin,nmax(nqtl),chmax(nqtl)
          integer :: nlinValide,iqtl,ic,ii,ifem,iip,ip
          integer , dimension(:,:,:) ,allocatable :: bestim_save,sigsq_save,rhoi_save
          integer , dimension(:,:)   ,allocatable :: lchr,lnpos
          real(kind=dp) , dimension(:,:) ,allocatable :: lrt_save
          type(POSITION_LRT_INCIDENCE)             :: curPos
          type(INCIDENCE_TYPE) , dimension(ncar)   :: listDesc

          hypotheseOne = ( lrtsol%nqtl == 1 )
          hypotheseTwo = ( lrtsol%nqtl == 2 )

          nGL = 0

          !Nombre de point sur le groupe de liaison
          !on cherche le nombre de point dependant de l hyp nqtl du numbre de point sur le groupe de liason

          do chr=1,nchr
              nGL=nGL + get_npo(chr)
          end do

          nGLTotal=1
          do iqtl=1,nqtl
            nGLTotal=nGLTotal*nGL
          end do

          curPosMax%listN(nqtl)=0
          curPosMax%listN(1:nqtl-1)=1
          curPosMax%listChr=1

          allocate (lchr(nGLTotal,nqtl))
          allocate (lnpos(nGLTotal,nqtl))

          nlinValide=0
          do nlin=1,nGLTotal
             i=nqtl
             ok=.false.
             do while ( (.not. ok) .and. (i>=1))
               curPosMax%listN(i) = curPosMax%listN(i)+1
               if ( curPosMax%listN(i) > get_npo(curPosMax%listChr(i)) ) then
                  if ( nchr > curPosMax%listChr(i) ) then
                    curPosMax%listN(i)=1
                    curPosMax%listChr(i)=curPosMax%listChr(i)+1
                    ok=.true.
                  else
                    curPosMax%listN(i)=1
                    curPosMax%listChr(i)=1
                    i=i-1
                  end if
               else
                  ok = .true.
               end if
             end do
             ok=.true.
             do iqtl=2,nqtl
               ok = ok .and. ( curPosMax%listN(iqtl-1)<=curPosMax%listN(iqtl) )
             end do

             if ( ok ) then
              nlinValide=nlinValide+1
              lchr(nlinValide,:)=curPosMax%listChr
              lnpos(nlinValide,:)=curPosMax%listN
             end if
          end do
          nGLTotal=nlinValide

          allocate (bestim_save(nGLTotal,ncar,ntnivmaxtotal))
          allocate (sigsq_save(nGLTotal,ncar,np))
          allocate (rhoi_save(nGLTotal,ncar,ncar))
          allocate (lrt_save(nGLTotal,nqtl))
          lrt_save=0.d0
          !$OMP PARALLEL DEFAULT(SHARED)  &
          !$OMP PRIVATE(curPos,i,ic,iqtl,ok,rhoiestim,sigsquareEstime,Bestim,xxx,ip,iip,ifem,listDesc,xinc2)
          call init_position (nqtl,curPos)
          allocate (xxx(ncar,ntnivmaxtotal,ntnivmaxtotal))
          allocate (xinc2(ncar,nd,ntnivmaxtotal))
          xinc2=xinc
          !initialisation of incidence matrix
          do ic=1,ncar
           call copy_incidence_desc (listDescMax(ic),listDesc(ic))
          end do

          curPos%lrt(nqtl)=0
          curPos%listN(nqtl)=0
          curPos%listN(1:nqtl-1)=1
          curPos%listChr=1

          !$OMP DO
          do nlin=1,nGLTotal
             call log_mess( trim(str((float(nlin)/float(nGLTotal))*100.d0))//"%", INFO_DEF )

             curPos%listN=lnpos(nlin,:)
             curPos%listChr=lchr(nlin,:)

           do iqtl=1,nqtl
             call change_qtleffect_multi(workstruct%listnteff(iqtl),iqtl,curPos%listChr(iqtl),curPos%listN(iqtl),xinc2,listDesc,0)
           end do
!           do ic=1,ncar
!             call debug_write_incidence(xinc2(ic,:,:),listDesc(ic))
!           end do

           call FUNCT_MODEL(xinc2,listDesc,curPos,workstruct,sigsquareEstime,rhoiestim,Bestim,.false.,.false.,xxx,.false.)

              !save value
              bestim_save(nlin,:,:)=Bestim
              sigsq_save(nlin,:,:)=sigsquareEstime
              rhoi_save(nlin,:,:)=rhoiestim
              lrt_save(nlin,:)=curPos%lrt
             ! print *,curPos%listN,":",curPos%lrt(nqtl),'-->nlin:',nlin!,OMP_GET_THREAD_NUM()

              if ( hypotheseOne ) then
                lrtsol%lrt1(curPos%listChr(1),curPos%listN(1))=curPos%lrt(1)
                iip=1
                do ip=1,np
                  if ( listDesc(1)%vecsol(1+ip)) then
                    iip=iip+1
                    ! ATTENTION VALIDE DANS LE CAS SANS INTERACTION AVEC QTL
                    lrtsol%pater_eff(curPos%listChr(1),ip,curPos%listN(1))=Bestim(1,iip)
                  end if
                end do
                ifem=0
                do ii=1,nm
                  if ( estime_multi(ii)) then
                    ifem=ifem+1
                    if ( listDesc(1)%vecsol(np+1+ifem) ) then
                      iip=iip+1
                      ! ATTENTION VALIDE DANS LE CAS SANS INTERACTION AVEC QTL
                      lrtsol%mater_eff(curPos%listChr(1),ifem,curPos%listN(1))=Bestim(1,iip)
                    end if
                  end if
                end do
              end if
              if ( hypotheseTwo ) then
                lrtsol%lrt1_2(curPos%listChr(1),curPos%listChr(2),curPos%listN(1),curPos%listN(2))=curPos%lrt(2)
                lrtsol%lrt0_2(curPos%listChr(1),curPos%listChr(2),curPos%listN(1),curPos%listN(2))=curPos%lrt(1)
              end if
        !    end if !ok
         end do ! nlin
         !$OMP END DO
         call end_position (curPos)
         deallocate (xxx)
         deallocate (xinc2)
         do ic=1,ncar
          call release_copy_incidence_desc(listDesc(ic))
         end do
         !$OMP END PARALLEL

         nlinMax=1
         lrtsol%lrtmax(0)=lrt_save(nlinMax,nqtl)
         nmax=1
         chmax=1

         call init_position (nqtl,curPos)

         curPos%listN(nqtl)=0
         curPos%listN(1:nqtl-1)=1
         curPos%listChr=1

         do nlin=1,nGLTotal
           curPos%listN=lnpos(nlin,:)
           curPos%listChr=lchr(nlin,:)

           if(lrtsol%lrtmax(nqtl-1) < lrt_save(nlin,nqtl)) then
             nmax=curPos%listN
             chmax=curPos%listChr
             nlinMax=nlin
             lrtsol%lrtmax=lrt_save(nlin,:)
           end if
         end do

         do iqtl=1,nqtl
            lrtsol%nxmax(iqtl-1)=nmax(iqtl)
            lrtsol%chrmax(iqtl-1)=chmax(iqtl)
            BestimMax = bestim_save(nlinMax,:,:)
            sigsquareEstimeMax=sigsq_save(nlinMax,:,:)
            rhoiestimMax=rhoi_save(nlinMax,:,:)
            curPosMax%listN=curPos%listN
            curPosMax%listChr=curPos%listChr
         end do

         deallocate (lchr)
         deallocate (lnpos)

         call end_position (curPos)

         deallocate (bestim_save)
         deallocate (sigsq_save)
         deallocate (rhoi_save)
         deallocate (lrt_save)

     end subroutine gen_opti_nqtl_multi
!!***

!!****f*ANALYSE/m_qtlmap_incidence_multi/set_solution_multi
!! NAME
!!    fill incsol the solution from the estimimation give in parameters
!! DESCRIPTION
!!
!! HISTORY
!!    09/09/2010 * modification de l affichage
!! NOTES
!!
!! SOURCE
    subroutine set_solution_multi(hypothesis,workstruct,sigsquareEstime,rhoiestim,Bestim,listDesc,incsol,lennteff,listnteff)
       integer                        , intent(in)          :: hypothesis
       type(INCIDENCE_GEN_STRUCT)            ,intent(in)    :: workstruct
       real (kind=dp) , dimension(ncar,ntnivmaxtotal) , intent(in)    :: Bestim      ! estimation of parameter on the maximum finded
       real (kind=dp) , dimension(ncar,np) , intent(in)     :: sigsquareEstime
       real (kind=dp) , dimension(ncar,ncar),intent(in)     :: rhoiestim
       type(INCIDENCE_TYPE)   ,dimension(ncar),intent(in)   :: listDesc
       type(TYPE_INCIDENCE_SOLUTION)      ,intent(inout)    :: incsol
       integer                               , intent(in)   :: lennteff
       integer        , dimension(lennteff)  , intent(in)   :: listnteff
       integer :: ic,jc,i,j,k,ip,maxNbPar,g,nb,INDEX,ntniv,nt(ncar),nteff,ii
       logical :: addxmut
       real (kind=dp) , dimension(ncar*ncar) :: tempLin !

       incsol%hypothesis=hypothesis
       call log_mess( "" , VERBOSE_DEF)
       call log_mess( "-------------------------------", VERBOSE_DEF)
       call log_mess( "Estimation of parameters under "//trim(str(incsol%hypothesis)), VERBOSE_DEF)
       call log_mess( "-------------------------------", VERBOSE_DEF)
       call log_mess( "rank max to resolve :"//trim(str(ntnivmaxtotal)), VERBOSE_DEF)

       allocate (incsol%sig(ncar,np))

       do ic=1,ncar-1
         if (listDesc(ic)%nteff /= listDesc(ic+1)%nteff) then
           print *,"Dev error: number of effect in the incidence matrix have to be equal..."
           stop
         end if
       end do

       nteff=listDesc(1)%nteff

       allocate (incsol%groupeName(nteff*ncar))
       allocate (incsol%eqtl_print(nteff*ncar))
       allocate (incsol%nbParameterGroup(nteff*ncar))

       if (hypothesis > 0) then
         allocate (incsol%qtl_groupeName(ncar,hypothesis))

         if ( lennteff < hypothesis ) then
           call stop_application("Devel error : bad init of array ** listnteff **")
         end if

         do i=1,hypothesis
           do ic=1,ncar
                incsol%qtl_groupeName(ic,i)=ncar -1 + listnteff(i)+(ic-1)
         !    print *,"NUM EFFET:",incsol%qtl_groupeName(ic,i)
           end do
         end do
       end if

       maxNbPar=1
       do i=1,nteff
           do ic=1,ncar
            nb=0
            if ( listDesc(ic)%desc(i)%haveSubDesc) then
             do j=1,size(listDesc(ic)%desc(i)%listSubDesc)
              nb = nb + listDesc(ic)%desc(i)%listSubDesc(j)%end - listDesc(ic)%desc(i)%listSubDesc(j)%start + 1
             end do
            end if
            maxNbPar = max(maxNbPar,nb)
           end do
       end do

       maxNbPar = max(maxNbPar,ncar) ! pour le general mean

       allocate (incsol%parameterName(nteff*ncar,maxNbPar))
       allocate (incsol%paramaterValue(nteff*ncar,maxNbPar))
       allocate (incsol%parameterVecsol(nteff*ncar,maxNbPar))
       allocate (incsol%parameterPrecis(nteff*ncar,maxNbPar))

       do ic=1,ncar
        do ip=1,np
        incsol%sig(ic,ip)=sqrt(sigsquareEstime(ic,ip))*sigt(ic)
        end do
       end do

       ii=0
       nt=0
       do i=1,nteff
        addxmut = .false.
        if ( workstruct%eff_general_mean == i ) addxmut = .true.
        do ic=1,ncar
          ii = ii + 1
          incsol%eqtl_print(ii) = listDesc(ic)%eqtl_print(i)
          incsol%groupeName(ii) = trim(listDesc(ic)%desc(i)%name)//" ["//trim(carac(ic))//"]"
          if ( listDesc(ic)%desc(i)%haveSubDesc) then
            incsol%nbParameterGroup(ii)=0
            !do ic=1,ncar
              do j=1,size(listDesc(ic)%desc(i)%listSubDesc)
              INDEX=0
               do k=listDesc(ic)%desc(i)%listSubDesc(j)%start,listDesc(ic)%desc(i)%listSubDesc(j)%end
                 INDEX=INDEX+1
                 incsol%nbParameterGroup(ii) = incsol%nbParameterGroup(ii) +1
                 g=incsol%nbParameterGroup(ii)
                 incsol%parameterName(ii,g)=listDesc(ic)%desc(i)%listSubDesc(j)%name//" "//adjustl(STR(INDEX))
                 ntniv=listDesc(ic)%desc(i)%listSubDesc(j)%start+INDEX -1
                 incsol%parameterVecsol(ii,g)=listDesc(ic)%vecsol(ntniv)
                 if ( incsol%parameterVecsol(ii,g) ) then
                    nt(ic) = nt(ic) + 1
                    incsol%paramaterValue(ii,g)= Bestim(ic,nt(ic))*sigt(ic)
                    if ( addxmut ) incsol%paramaterValue(ii,g) = incsol%paramaterValue(ii,g)+xmut(ic)
                    incsol%parameterPrecis(ii,g) = listDesc(ic)%precis(ntniv)
                  !  print *,"----"
                  !  print *,ic,nt(ic),Bestim(ic,nt(ic)),sigt(ic)
                  !  print *,incsol%groupeName(i),incsol%parameterName(i,g),incsol%paramaterValue(i,g),incsol%parameterPrecis(i,g)
                 end if
               end do
              end do
          else
           !do ic=1,ncar
            incsol%nbParameterGroup(ii)=1
            incsol%parameterName(ii,ic)=trim(listDesc(ic)%desc(i)%name)//"*["//trim(carac(ic))//"]*"
            incsol%parameterVecsol(ii,ic)=listDesc(ic)%vecsol(listDesc(ic)%desc(i)%start)
            if ( incsol%parameterVecsol(ii,ic) ) then
              nt(ic) = nt(ic) + 1
              incsol%paramaterValue(ii,ic)  = Bestim(ic,nt(ic))*sigt(ic)
              if ( addxmut ) incsol%paramaterValue(ii,g) = incsol%paramaterValue(ii,g)+xmut(ic)
              incsol%parameterPrecis(ii,ic) = listDesc(ic)%precis(listDesc(ic)%desc(i)%start)
            end if
           !end do
          end if
         end do
       end do

       allocate (incsol%rhoi(ncar,ncar))
       incsol%rhoi=rhoiestim

       i=0
       do ic=2,ncar
         do jc=1,ic-1
           i=i+1
           tempLin(i)=rhoiestim(ic,jc)
         end do
       end do

       i=0
       do jc=1,ncar-1
          do ic=jc+1,ncar
           i = i + 1
           incsol%rhoi(ic,jc) = tempLin(i)
           incsol%rhoi(jc,ic) = incsol%rhoi(ic,jc)
          end do
       end do

    end subroutine set_solution_multi
!!***


end module m_qtlmap_incidence_multi
