!!****m* DATA/m_qtlmap_constant
!!  NAME
!!    m_qtlmap_constant -- Definition of constantes
!!  SYNOPSIS
!!    All definition useb by QTLMap are defined here....
!!  DESCRIPTION
!!
!!  NOTES
!!
!!  BUGS
!!
!!  SEE ALSO
!!
!!  COPYRIGHT
!!***
!! You can use this space for remarks that should not be included
!! in the documentation.
!!/

module m_qtlmap_constant
   implicit none

   !precision a 15 chiffre pour les reel
   !pour une constante  ajoute _dp ex:real, public, parameter :: pi = 3.1415926535897932384_dp
   !pour un real (kind=dp) selected_real_kind(P=15)

   !integer, public, parameter             :: PRECISION_REAL     = 8
!!****d* m_qtlmap_constant/DP
!!  NAME
!!   DP
!!  DESCRIPTION
!!   Precision (kind) of real defined : 15 digits and exponent range greater at least 307.
!!  NOTES
!!
!!***
   integer, public, parameter             :: DP                 = SELECTED_REAL_KIND( 15, 307 )
    !***************************************************************************
   ! CONSTANTE PART
   !***************************************************************************

   integer, public, parameter             :: MAX_ANIMAL                 = 15000

   !-----------------------------------------------------------------------------
   ! dim : nb marker,nmes,2 : description of each allele by animal and by marker
   ! Modif olivier, optimisation -128<pheno<127
   ! Optimisation : codage des allele sur 256 entier
   ! 1) ==> amelioration des tps de calcul pour la comparaison d 'allele
   ! 2) ==> gain memoire
   !


!!****d* m_qtlmap_constant/KIND_PHENO
!!  NAME
!!   KIND_PHENO
!!  DESCRIPTION
!!   Precision (kind) of integer id allele defined
!!  NOTES
!!
!!***
   integer                    ,parameter     ,public :: KIND_PHENO=1
!!****d* m_qtlmap_constant/VAL_MIN_INDEX_PHENO
!!  NAME
!!   VAL_MIN_INDEX_PHENO
!!  DESCRIPTION
!!   Minimum value of an index allele
!!  NOTES
!!
!!***
   integer(kind=KIND_PHENO)   ,parameter     ,public :: VAL_MIN_INDEX_PHENO=1-(2**bit_size(VAL_MIN_INDEX_PHENO))/2
!!****d* m_qtlmap_constant/VAL_MAX_INDEX_PHENO
!!  NAME
!!   VAL_MAX_INDEX_PHENO
!!  DESCRIPTION
!!   Maximum value of an index allele
!!  NOTES
!!
!!***
   integer(kind=KIND_PHENO)   ,parameter     ,public :: VAL_MAX_INDEX_PHENO=(2**bit_size(VAL_MIN_INDEX_PHENO))/2 - 1
!!****d* m_qtlmap_constant/MAX_FILES_INPUT
!!  NAME
!!   MAX_FILES_INPUT
!!  DESCRIPTION
!!   Maximum files to read for input data
!!  NOTES
!!
!!***
   integer, public, parameter             :: MAX_FILES_INPUT    = 6
!!****d* m_qtlmap_constant/LENGTH_MAX_FILE
!!  NAME
!!   LENGTH_MAX_FILE
!!  DESCRIPTION
!!   Maximum length of a name file
!!  NOTES
!!
!!***
   integer, public, parameter             :: LENGTH_MAX_FILE    = 1024
!!****d* m_qtlmap_constant/LEN_S
!!  NAME
!!   LEN_S
!!  DESCRIPTION
!!   Length size of a short word
!!  NOTES
!!
!!***
   integer, public, parameter             :: LEN_S              = 20
!!****d* m_qtlmap_constant/LEN_W
!!  NAME
!!   LEN_W
!!  DESCRIPTION
!!   Length size of a word
!!  NOTES
!!
!!***
   integer, public, parameter             :: LEN_W              = 40
!!****d* m_qtlmap_constant/LEN_L
!!  NAME
!!   LEN_L
!!  DESCRIPTION
!!   Length size of a long word
!!  NOTES
!!
!!***
   integer, public, parameter             :: LEN_L              = 80
!!****d* m_qtlmap_constant/LEN_LINE
!!  NAME
!!   LEN_LINE
!!  DESCRIPTION
!!   Maximum size length of a line readed
!!  NOTES
!!
!!***
   integer, public, parameter             :: LEN_LINE           = 100000
!!****d* m_qtlmap_constant/LEN_DEF
!!  NAME
!!   LEN_DEF
!!  DESCRIPTION
!!   Default size of a string
!!  NOTES
!!
!!***
   integer, public, parameter             :: LEN_DEF            = LEN_W
!!****d* m_qtlmap_constant/LEN_BUFFER_WORD
!!  NAME
!!   LEN_BUFFER_WORD
!!  DESCRIPTION
!!   Default size of a buffer string
!!  NOTES
!!
!!***
   integer, public, parameter             :: LEN_BUFFER_WORD    = LEN_L
!!****d* m_qtlmap_constant/FMT_MAX_BUFFER
!!  NAME
!!   FMT_MAX_BUFFER
!!  DESCRIPTION
!!   Default maximum size of a fmt buffer string
!!  NOTES
!!
!!***
   integer, public, parameter             :: FMT_MAX_BUFFER     = 4048
!!****d* m_qtlmap_constant/LEN_BUFFER_LINE
!!  NAME
!!   LEN_BUFFER_LINE
!!  DESCRIPTION
!!   Default maximum size of a buffer string line
!!  NOTES
!!
!!***
   integer, public, parameter             :: LEN_BUFFER_LINE    = 200
!!****d* m_qtlmap_constant/FMT_REAL
!!  NAME
!!   FMT_REAL
!!  DESCRIPTION
!!   Default Real FMT to print in a console
!!  NOTES
!!
!!***
   character (len=10) , public, parameter :: FMT_REAL           = '(F8.5)'
!!****d* m_qtlmap_constant/FMT_INT
!!  NAME
!!   FMT_INT
!!  DESCRIPTION
!!   Default Integer FMT to print in a console
!!  NOTES
!!
!!***
   character (len=10) , public, parameter :: FMT_INT            = '(I8)'
!!****d* m_qtlmap_constant/FMT_INT_LONG
!!  NAME
!!   FMT_INT_LONG
!!  DESCRIPTION
!!   Default Long Integer FMT to print in a console
!!  NOTES
!!
!!***
   character (len=10) , public, parameter :: FMT_INT_LONG       = '(I9)'
!!****d* m_qtlmap_constant/REAL_NOT_DEFINED
!!  NAME
!!   REAL_NOT_DEFINED
!!  DESCRIPTION
!!   Default initialization of real without correct value
!!  NOTES
!!
!!***
   real (kind=dp),     public,parameter   :: REAL_NOT_DEFINED   = 99999.d0
!!****d* m_qtlmap_constant/INIFINY_REAL_VALUE
!!  NAME
!!   INIFINY_REAL_VALUE
!!  DESCRIPTION
!!   The biggest value using by the optimization package
!!  NOTES
!!
!!***
   real (kind=dp),     public,parameter   :: INIFINY_REAL_VALUE = REAL_NOT_DEFINED
!!****d* m_qtlmap_constant/INT_NOT_DEFINED
!!  NAME
!!   INT_NOT_DEFINED
!!  DESCRIPTION
!!    Default initialization of integer without correct value
!!  NOTES
!!
!!***
   integer, public, parameter             :: INT_NOT_DEFINED    = 9999
!!****d* m_qtlmap_constant/STRING_NOT_DEFINED
!!  NAME
!!   STRING_NOT_DEFINED
!!  DESCRIPTION
!!    Default initialization of string without correct value
!!  NOTES
!!
!!***
   character (len=1) , public, parameter  :: STRING_NOT_DEFINED = '*'
!!****d* m_qtlmap_constant/OPT_SIB_HS
!!  NAME
!!   OPT_SIB_HS
!!  DESCRIPTION
!!    Definition of a constante for half sib analysis
!!  NOTES
!!
!!***
   integer     ,parameter , public        :: OPT_SIB_HS         = 1 ! half sib
!!****d* m_qtlmap_constant/OPT_SIB_FS
!!  NAME
!!   OPT_SIB_FS
!!  DESCRIPTION
!!    Definition of a constante for full sib analysis
!!  NOTES
!!
!!***
   integer     ,parameter , public        :: OPT_SIB_FS         = 2 ! full sib
!!****d* m_qtlmap_constant/DEFAULT_PARAM_MIN
!!  NAME
!!   DEFAULT_PARAM_MIN
!!  DESCRIPTION
!!    Minimum value for a generic parameter for a optimization context
!!  NOTES
!!
!!***
   real (kind=dp)     ,parameter , public        :: DEFAULT_PARAM_MIN  = -1.d4
!!****d* m_qtlmap_constant/DEFAULT_PARAM_MAX
!!  NAME
!!   DEFAULT_PARAM_MAX
!!  DESCRIPTION
!!    Maximum value for a generic parameter for a optimization context
!!  NOTES
!!
!!***
   real (kind=dp)     ,parameter , public        :: DEFAULT_PARAM_MAX  = 1.d4
!!****d* m_qtlmap_constant/SIG_MIN
!!  NAME
!!   SIG_MIN
!!  DESCRIPTION
!!    Minimum value for a standart deviation parameter for a optimization context
!!  NOTES
!!
!!***
   real (kind=dp)     ,parameter , public        :: SIG_MIN            = 1.d-4
!!****d* m_qtlmap_constant/SIG_MAX
!!  NAME
!!   SIG_MAX
!!  DESCRIPTION
!!    Maximum value for a standart deviation parameter for a optimization context
!!  NOTES
!!
!!***
   real (kind=dp)     ,parameter , public        :: SIG_MAX            = DEFAULT_PARAM_MAX
!!****d* m_qtlmap_constant/XMU_MIN
!!  NAME
!!   XMU_MIN
!!  DESCRIPTION
!!    Minimum value for a mean parameter for a optimization context
!!  NOTES
!!
!!***
   real (kind=dp)     ,parameter , public        :: XMU_MIN            = DEFAULT_PARAM_MIN
!!****d* m_qtlmap_constant/XMU_MAX
!!  NAME
!!   XMU_MAX
!!  DESCRIPTION
!!    Maximum value for a mean parameter for a optimization context
!!  NOTES
!!
!!***
   real (kind=dp)     ,parameter , public        :: XMU_MAX            = DEFAULT_PARAM_MAX
!!****d* m_qtlmap_constant/AP_MIN
!!  NAME
!!   AP_MIN
!!  DESCRIPTION
!!    Minimum value for a sire qtl effect parameter for a optimization context
!!  NOTES
!!
!!***
   real (kind=dp)     ,parameter , public        :: AP_MIN             = DEFAULT_PARAM_MIN
!!****d* m_qtlmap_constant/AP_MAX
!!  NAME
!!   AP_MIN
!!  DESCRIPTION
!!    Maximum value for a sire qtl effect parameter for a optimization context
!!  NOTES
!!
!!***
   real (kind=dp)     ,parameter , public        :: AP_MAX             = DEFAULT_PARAM_MAX
!!****d* m_qtlmap_constant/AM_MIN
!!  NAME
!!   AM_MIN
!!  DESCRIPTION
!!    Minimum value for a dam qtl effect parameter for a optimization context
!!  NOTES
!!
!!***
   real (kind=dp)     ,parameter , public        :: AM_MIN             = DEFAULT_PARAM_MIN
!!****d* m_qtlmap_constant/AM_MAX
!!  NAME
!!   AM_MAX
!!  DESCRIPTION
!!    Maximum value for a dam qtl effect parameter for a optimization context
!!  NOTES
!!
!!***
   real (kind=dp)     ,parameter , public        :: AM_MAX             = DEFAULT_PARAM_MAX

!!****d* m_qtlmap_constant/EPS_EM
!!  NAME
!!   EPS_EM
!!  DESCRIPTION
!!    maximum difference between two successive estimation of the haplotype proability
!!    in prob_haplo_complet
!!  NOTES
!!
!!***
   real (kind=dp)     ,parameter , public        :: EPS_EM             = 1.d-4
!!****d* m_qtlmap_constant/ITER_EM_MAX
!!  NAME
!!   ITER_EM_MAX
!!  DESCRIPTION
!!    Maximum number of iteration in the EM in prob_haplo_complet
!!  NOTES
!!
!!***
   real (kind=dp)     ,parameter , public        :: ITER_EM_MAX             = 1000

!!!!*********************************************************** ID ANALYSIS

!!****d* m_qtlmap_constant/TYPE_DATA_CONTINUE
!!  NAME
!!   TYPE_DATA_CONTINUE
!!  DESCRIPTION
!!   Constant for real data
!!  NOTES
!!
!!***
   integer        ,parameter,public              :: TYPE_DATA_CONTINUE                  = 1
!!****d* m_qtlmap_constant/TYPE_DATA_COX
!!  NAME
!!   TYPE_DATA_COX
!!  DESCRIPTION
!!   Constant for model cox data
!!  NOTES
!!
!!***
   integer        ,parameter,public              :: TYPE_DATA_COX                       = 2

!!****d* m_qtlmap_constant/ANALYSE_UNITRAIT
!!  NAME
!!   ANALYSE_UNITRAIT
!!  DESCRIPTION
!!   Constant id LA single trait with pre-corrected data
!!  NOTES
!!
!!***
   integer        ,parameter,public              :: ANALYSE_UNITRAIT                    = 1
!!****d* m_qtlmap_constant/ANALYSE_UNITRAIT_MODLIN
!!  NAME
!!   ANALYSE_UNITRAIT_MODLIN
!!  DESCRIPTION
!!   Constant id LA single trait with model description
!!  NOTES
!!
!!***
   integer        ,parameter,public              :: ANALYSE_UNITRAIT_MODLIN             = 2
!!****d* m_qtlmap_constant/ANALYSE_UNITRAIT_LINEAR_LA_HOMO
!!  NAME
!!   ANALYSE_UNITRAIT_LINEAR_LA_HOMO
!!  DESCRIPTION
!!   Constant id LA for a single data with a model description (likelihood linearised ­ homoscedatic)
!!  NOTES
!!
!!***
   integer        ,parameter,public              :: ANALYSE_UNITRAIT_LINEAR_LA_HOMO     = 3
!!****d* m_qtlmap_constant/ANALYSE_UNITRAIT_LINEAR_LA_HETERO
!!  NAME
!!   ANALYSE_UNITRAIT_LINEAR_LA_HETERO
!!  DESCRIPTION
!!   Constant id LA for a single data with a model description (likelihood linearised ­ heteroscedastic)
!!  NOTES
!!
!!***
   integer        ,parameter,public              :: ANALYSE_UNITRAIT_LINEAR_LA_HETERO   = 4
!!****d* m_qtlmap_constant/ANALYSE_UNITRAIT_LINEAR_LD_HOMO
!!  NAME
!!   ANALYSE_UNITRAIT_LINEAR_LD_HOMO
!!  DESCRIPTION
!!   Constant id LD  for a single data with a model description   (likelihood linearised ­ homoscedatic)
!!  NOTES
!!
!!***
   integer        ,parameter,public              :: ANALYSE_UNITRAIT_LINEAR_LD_HOMO     = 25
!!****d* m_qtlmap_constant/ANALYSE_UNITRAIT_LINEAR_LD_HETERO
!!  NAME
!!   ANALYSE_UNITRAIT_LINEAR_LD_HETERO
!!  DESCRIPTION
!!   Constant id LD for a single data with a model description (likelihood linearised ­ heteroscedastic)
!!  NOTES
!!
!!***
   integer        ,parameter,public              :: ANALYSE_UNITRAIT_LINEAR_LD_HETERO   = 26
!!****d* m_qtlmap_constant/ANALYSE_UNITRAIT_LINEAR_LDLA_HOMO
!!  NAME
!!   ANALYSE_UNITRAIT_LINEAR_LDLA_HOMO
!!  DESCRIPTION
!!   Constant id LDLA  for a single data with a model description (likelihood linearised ­ homoscedatic)
!!  NOTES
!!
!!***
   integer        ,parameter,public              :: ANALYSE_UNITRAIT_LINEAR_LDLA_HOMO   = 27
!!****d* m_qtlmap_constant/ANALYSE_UNITRAIT_LINEAR_LDLA_HETERO
!!  NAME
!!   ANALYSE_UNITRAIT_LINEAR_LDLA_HETERO
!!  DESCRIPTION
!!   Constant id LDLA  for a single data with a model description (likelihood linearised ­ homoscedatic)
!!  NOTES
!!
!!***
   integer        ,parameter,public              :: ANALYSE_UNITRAIT_LINEAR_LDLA_HETERO = 28
!!****d* m_qtlmap_constant/ANALYSE_MULTIRAIT
!!  NAME
!!   ANALYSE_MULTIRAIT
!!  DESCRIPTION
!!   Constant id LA for a set of traits with a multivariate analysis (based on a multi­normal penetrance function)
!!  NOTES
!!
!!***
   integer        ,parameter,public              :: ANALYSE_MULTIRAIT                   = 5
!!****d* m_qtlmap_constant/ANALYSE_MULTITRAIT_DA
!!  NAME
!!   ANALYSE_MULTITRAIT_DA
!!  DESCRIPTION
!!    Constant id  LA for a set of traits (without missing data) with a discriminante analysis
!!  NOTES
!!
!!***
   integer        ,parameter,public              :: ANALYSE_MULTITRAIT_DA               = 6
!!****d* m_qtlmap_constant/ANALYSE_UNITRAIT_MODLIN_COX
!!  NAME
!!   ANALYSE_UNITRAIT_MODLIN_COX
!!  DESCRIPTION
!!    Constant id LA for a single survey trait with the cox model
!!  NOTES
!!
!!***
   integer        ,parameter,public              :: ANALYSE_UNITRAIT_MODLIN_COX    = 7
!!****d* m_qtlmap_constant/ANALYSE_UNITRAIT_LD
!!  NAME
!!   ANALYSE_UNITRAIT_LD
!!  DESCRIPTION
!!    Constant id LD for a single data with a model description
!!  NOTES
!!
!!***
   integer        ,parameter,public              :: ANALYSE_UNITRAIT_LD            = 8
!!****d* m_qtlmap_constant/ANALYSE_UNITRAIT_LDLA
!!  NAME
!!   ANALYSE_UNITRAIT_LDLA
!!  DESCRIPTION
!!    Constant id LDLA  for a single data with a model description
!!  NOTES
!!
!!***
   integer        ,parameter,public              :: ANALYSE_UNITRAIT_LDLA          = 9
!!****d* m_qtlmap_constant/ANALYSE_UNITRAIT_LA
!!  NAME
!!   ANALYSE_UNITRAIT_LA
!!  DESCRIPTION
!!    Constant id LA for a single survey trait with the model description
!!  NOTES
!!
!!***
   integer        ,parameter,public              :: ANALYSE_UNITRAIT_LA            = 10
!!****d* m_qtlmap_constant/ANALYSE_UNITRAIT_LDJH
!!  NAME
!!   ANALYSE_UNITRAIT_LDJH
!!  DESCRIPTION
!!    Constant id LA for a single survey trait with a model description ( DEV model LDJH )
!!  NOTES
!!
!!***
   integer        ,parameter,public              :: ANALYSE_UNITRAIT_LDJH          = 11

!!****d* m_qtlmap_constant/ANALYSE_2QTL_INTERACTION
!!  NAME
!!   ANALYSE_2QTL_INTERACTION
!!  DESCRIPTION
!!    Constant id 2 qtl interaction ( DEV )
!!  NOTES
!!
!!***
   integer        ,parameter,public              :: ANALYSE_2QTL_INTERACTION       = 19

!!****d* m_qtlmap_constant/ANALYSE_UNITRAIT_CONTINGENCE
!!  NAME
!!   ANALYSE_UNITRAIT_CONTINGENCE
!!  DESCRIPTION
!!    Constant id LA for a single survey trait with a model description without interaction fixed effect - qtl (multiqtl version).
!!    this analysis used the structured data of m_qtlmap_incidence.F95
!!  NOTES
!!
!!***
   integer        ,parameter,public              :: ANALYSE_UNITRAIT_CONTINGENCE   = 21
!!****d* m_qtlmap_constant/ANALYSE_TRAIT_COV_CONTINGENCE
!!  NAME
!!   ANALYSE_TRAIT_COV_CONTINGENCE
!!  DESCRIPTION
!!    Constant id LA for a single survey trait with a model description (other performance are treat as covariates (xiaoquiang.wang@rennes.inra.fr))
!!    this analysis used the structured data of m_qtlmap_incidence.F95
!!  NOTES
!!
!!***
   integer        ,parameter,public              :: ANALYSE_TRAIT_COV_CONTINGENCE  = 22
!!****d* m_qtlmap_constant/ANALYSE_MULTITRAIT_INCIDENCE
!!  NAME
!!   ANALYSE_MULTITRAIT_INCIDENCE
!!  DESCRIPTION
!!    Constant id LA for a set of traits with a multivariate analysis with a model description (based on a multi­normal penetrance function) (multi-qtl version)
!!    this analysis used the structured data of m_qtlmap_incidence.F95
!!  NOTES
!!
!!***
   integer        ,parameter,public              :: ANALYSE_MULTITRAIT_INCIDENCE   = 23

!!****d* m_qtlmap_constant/COMMON_ANALYSE
!!  NAME
!!   COMMON_ANALYSE
!!  DESCRIPTION
!!    Constant id for context analysis (analysis case)
!!  NOTES
!!
!!***
   integer, public, parameter                    :: COMMON_ANALYSE         = 0
!!****d* m_qtlmap_constant/SIMULATION
!!  NAME
!!   SIMULATION
!!  DESCRIPTION
!!    Constant id for context analysis (simulation case)
!!  NOTES
!!
!!***
   integer, public, parameter                    :: SIMULATION             = 1
!!****d* m_qtlmap_constant/TRANSCIPTOME_ANALYSE
!!  NAME
!!   TRANSCIPTOME_ANALYSE
!!  DESCRIPTION
!!    Constant id for context analysis (transcriptom analysis case)
!!  NOTES
!!
!!***
   integer, public, parameter                    :: TRANSCIPTOME_ANALYSE   = 2


!!****d* m_qtlmap_constant/F2_KEYWORD
!!  NAME
!!   F2_KEYWORD
!!  DESCRIPTION
!!   "F2" label in the simulation parameter file.
!!***
   character(len=LEN_BUFFER_WORD), public, parameter :: F2_KEYWORD      = 'F2'
!!****d* m_qtlmap_constant/BC_KEYWORD
!!  NAME
!!   BC_KEYWORD
!!  DESCRIPTION
!!   Backcross label in the simulation parameter file.
!!***
   character(len=LEN_BUFFER_WORD), public, parameter :: BC_KEYWORD      = 'BC'
!!****d* m_qtlmap_constant/OUTBRED_KEYWORD
!!  NAME
!!   OUTBRED_KEYWORD
!!  DESCRIPTION
!!   Outbred label in the simulation parameter file.
!!***
   character(len=LEN_BUFFER_WORD), public, parameter :: OUTBRED_KEYWORD = 'OUTBRED'

!!****d* m_qtlmap_constant/LABEL_MARKERS
!!  NAME
!!   LABEL_MARKERS
!!  DESCRIPTION
!!   Label markers description
!!***
  character(len=20)           ,parameter    :: LABEL_MARKERS     = "MARKERS"
!!****d* m_qtlmap_constant/LABEL_GENEALOGY
!!  NAME
!!   LABEL_GENEALOGY
!!  DESCRIPTION
!!   Label genealogy description
!!***
  character(len=20)           ,parameter    :: LABEL_GENEALOGY   = "GENEALOGY"
!!****d* m_qtlmap_constant/LABEL_QTL
!!  NAME
!!   LABEL_QTL
!!  DESCRIPTION
!!   Label QTL description
!!***
  character(len=20)           ,parameter    :: LABEL_QTL         = "QTL"
!!****d* m_qtlmap_constant/LABEL_TRAITS
!!  NAME
!!   LABEL_TRAITS
!!  DESCRIPTION
!!   Label phenotypic data description
!!***
  character(len=20)           ,parameter    :: LABEL_TRAITS      = "TRAITS"
!!****d* m_qtlmap_constant/LABEL_SIMULTRAITS
!!  NAME
!!   LABEL_SIMULTRAITS
!!  DESCRIPTION
!!   Label simulation phenotypic data description
!!***
  character(len=20)           ,parameter    :: LABEL_SIMULTRAITS = "SIMULTRAITS"



end module m_qtlmap_constant
