!!****p* QTLMap/DATA
!!  NAME
!!    DATA
!!  DESCRIPTION
!!  Package data :
!!     contains the module m_qtlmap_data that offers public variables of data user
!!     (map ,genetic marker, traits, model of the traits ) and the module m_qtlmap_haplotype data generated
!!     by the HAPLOTYPE package
!!
!!  CREATION DATE
!!  01/01/2009
!!  COPYRIGHT
!!***

!!****m* DATA/m_qtlmap_data
!!  NAME
!!    m_qtlmap_data -- Storage of data user.
!!  SYNOPSIS
!!    User data are storaged in this module and
!!    Some simples subroutines are included
!!  DESCRIPTION
!!
!!  NOTES
!!
!!  BUGS
!!
!!  SEE ALSO
!!
!!  COPYRIGHT
!!***
!! You can use this space for remarks that should not be included
!! in the documentation.
!!/

module m_qtlmap_data
   use m_qtlmap_constant
   use QTLMAP_TYPES
   implicit none
   save

   !Test sur une nouvelle structuration
   type (GENEALOGY_TYPE)                  , public            :: datasetUser

!!****v* m_qtlmap_data/BASE_STEP
!!  NAME
!!   BASE_STEP
!!  DESCRIPTION
!!   Morgan Base
!!  NOTES
!!   Information from the parameter file
!!***
   real                                   ,        public     :: BASE_STEP  = 1.d2

!!****v* m_qtlmap_data/NDMIN
!!  NAME
!!   NDMIN
!!  DESCRIPTION
!!   Number of progenies to take in care estimable femal in the analysis
!!  NOTES
!!   Information from the parameter file
!!***
   integer                                   ,        public     :: NDMIN  = 0

!**********************************************************************************
   ! OPTION FROM P_ANALYSE_FILE
!!****v* m_qtlmap_data/SEUIL_CHO
!!  NAME
!!   SEUIL_CHO
!!  DESCRIPTION
!!   Cholesky threshold
!!  NOTES
!!   Information from the parameter file
!!***
   real (kind=dp)                       ,            public      :: SEUIL_CHO
!!****v* m_qtlmap_data/THRES_CONFUSION
!!  NAME
!!   THRES_CONFUSION
!!  DESCRIPTION
!!   Threshold to test confusion betwwen level inside a contingence matrix
!!  NOTES
!!   Information from the parameter file
!!***
   real(kind=dp)                          , public              :: THRES_CONFUSION

!!****v* m_qtlmap_data/EPS_LINEAR_HETEROSCEDASTIC
!!  NAME
!!   EPS_LINEAR_HETEROSCEDASTIC
!!  DESCRIPTION
!!   Threshold for convergence in the linear mode heteroscedastic
!!  NOTES
!!   Information from the parameter file
!!***
    real (kind=dp)                                , public      :: EPS_LINEAR_HETEROSCEDASTIC

!!****v* m_qtlmap_data/MAX_LINEAR_ITERATION
!!  NAME
!!   MAX_LINEAR_ITERATION
!!  DESCRIPTION
!!   Maximum iteration in the linear mode heteroscedastic to avoid infinity loop
!!  NOTES
!!   Information from the parameter file
!!***
    integer                                      , public      :: MAX_LINEAR_ITERATION

!!****v* m_qtlmap_data/PAS
!!  NAME
!!   PAS
!!  DESCRIPTION
!!   Step analysis
!!  NOTES
!!   Information from the parameter file
!!***
   integer                              ,            public      :: PAS
!!****v* m_qtlmap_data/PRSEUIL
!!  NAME
!!   PRSEUIL
!!  DESCRIPTION
!!   Minimal maternal phase probability
!!  NOTES
!!   Threshold above which the probable maternal phases will be considered in the analysis
!!   Information from the parameter file
!!***
   real (kind=dp)                       ,            public      :: PRSEUIL
!!****v* m_qtlmap_data/PHPSEUIL
!!  NAME
!!   PHPSEUIL
!!  DESCRIPTION
!!    Minimal paternal phase probability
!!  NOTES
!!   The analysis is interrupted if for a sire, none of its phases reach this threshold
!!   Information from the parameter file
!!***
   real (kind=dp)                       ,            public      :: PHPSEUIL


!!****v* m_qtlmap_data/PROB_SEUIL_RECOMB
!!  NAME
!!   PROB_SEUIL_RECOMB
!!  DESCRIPTION
!!    Minimal paternal phase probability
!!  NOTES
!!   The analysis is interrupted if for a sire, none of its phases reach this threshold
!!   Information from the parameter file
!!***
   real (kind=dp)                       ,            public      :: PROB_SEUIL_RECOMB

!!****v* m_qtlmap_data/PSEUILHWE
!!  NAME
!!   PSEUILHWE
!!  DESCRIPTION
!!    Threshold to check the equilibrium of marker transmission within each family
!!  NOTES
!!
!!***
   real (kind=dp)                       ,            public      :: PSEUILHWE

!!****v* m_qtlmap_data/NMANQUE
!!  NAME
!!   NMANQUE
!!  DESCRIPTION
!!   Genotype code for missing value
!!  NOTES
!!   Information from the parameter file
!!***
   integer(kind=KIND_PHENO)             ,            public      :: NMANQUE


   !***************************************************************************
   ! GENEALOGY PART
   !***************************************************************************

!!****v* m_qtlmap_data/repro
!!  NAME
!!    repro
!!  DESCRIPTION
!!    List of reproductor name ( sire and dam )
!!  NOTES
!!    Information from the genealogy file
!!***
   character(len=LEN_DEF) , dimension (:),allocatable,public :: repro
!!****v* m_qtlmap_data/nr
!!  NAME
!!    nr
!!  DESCRIPTION
!!    Number of reproductor : equivalent to size(repro)
!!***
   integer                 ,public                         :: nr = 0
!!****v* m_qtlmap_data/gpere
!!  NAME
!!    gpere
!!  DESCRIPTION
!!    List of the grand sire name
!!  NOTES
!!    Information from the genealogy file
!!***
   character(len=LEN_DEF) , dimension (:),allocatable,public :: gpere
!!****v* m_qtlmap_data/ngp
!!  NAME
!!    ngp
!!  DESCRIPTION
!!    Number of grand sire : equivalent to size(gpere)
!!***
   integer                 ,public                         :: ngp = 0! gpere size
!!****v* m_qtlmap_data/gmere
!!  NAME
!!    gmere
!!  DESCRIPTION
!!    List of the grand dam name
!!***
   character(len=LEN_DEF) , dimension (:),allocatable,public :: gmere
!!****v* m_qtlmap_data/ngm
!!  NAME
!!    ngm
!!  DESCRIPTION
!!    Number of grand dam : equivalent to size(gmere)
!!  NOTES
!!    Information from the genealogy file
!!***
   integer                 ,public                         :: ngm = 0! gmere size
!!****v* m_qtlmap_data/ngmgp
!!  NAME
!!    ngmgp
!!  DESCRIPTION
!!    Grand dam id by grand sire index
!!  EXAMPLE
!!   do igp=1,ngp
!!     do jgm=ngmgp(igp)+1,ngmgp(igp+1)
!!      ...
!!     end do
!!   end do
!!   NOTES
!!    Information from the genealogy file
!!***
   integer              , dimension (:),allocatable,public :: ngmgp
!!****v* m_qtlmap_data/nrgm
!!  NAME
!!    nrgm
!!  DESCRIPTION
!!    Get reproductor id by grand dam index
!!  EXAMPLE
!!   do igp=1,ngp
!!     do jgm=ngmgp(igp)+1,ngmgp(igp+1)
!!       do kr=nrgm(jgm)+1,nrgm(jgm+1)
!!          ...
!!       end do
!!     end do
!!   end do
!!
!!   NOTES
!!    Information from the genealogy file
!!***
   integer              , dimension (:),allocatable,public :: nrgm
!!****v* m_qtlmap_data/pere
!!  NAME
!!    pere
!!  DESCRIPTION
!!    List of the sire name
!!***
   character(len=LEN_DEF) , dimension (:),allocatable,public :: pere
!!****v* m_qtlmap_data/np
!!  NAME
!!    np
!!  DESCRIPTION
!!    Number of sire : equivalent to size(pere)
!!  NOTES
!!    Information from the genealogy file
!!***
   integer ,                                        public :: np = 0! size(pere)
   !pere --> indice repro : reppere(indpere)=indrepro
!!****v* m_qtlmap_data/reppere
!!  NAME
!!    reppere
!!  DESCRIPTION
!!    Contains the reproductor index equivalent with an index sire
!!  NOTES
!!    reppere(indsire)=indrepro => pere(indsire) == repro(indrepro)
!!***
   integer ,              dimension (:),allocatable,public :: reppere ! size(pere)
!!****v* m_qtlmap_data/mere
!!  NAME
!!    mere
!!  DESCRIPTION
!!    List of the dam name
!!***
   character(len=LEN_DEF) , dimension (:),allocatable,public :: mere
!!****v* m_qtlmap_data/nm
!!  NAME
!!    nm
!!  DESCRIPTION
!!    Number of dam : equivalent to size(mere)
!!  NOTES
!!    Information from the genealogy file
!!***
   integer              ,                           public :: nm = 0! size(mere)
!!****v* m_qtlmap_data/repmere
!!  NAME
!!    repmere
!!  DESCRIPTION
!!    Contains the reproductor index equivalent with an index dam
!!  NOTES
!!    repmere(inddam)=indrepro => mere(inddam) == repro(indrepro)
!!***
   integer              , dimension (:),allocatable,public :: repmere
!!****v* m_qtlmap_data/animal
!!  NAME
!!    animal
!!  DESCRIPTION
!!    List of the progenies
!!  NOTES
!!    Information from the genealogy file
!!***
   character(len=LEN_DEF) , dimension (:),allocatable,public :: animal
!!****v* m_qtlmap_data/nd
!!  NAME
!!    nd
!!  DESCRIPTION
!!    Number of progenies : equivalent to size(animal)
!!***
   integer              ,                           public :: nd = 0! size(animal)
!!****v* m_qtlmap_data/nmp
!!  NAME
!!    nmp
!!  DESCRIPTION
!!    id dam by sire index
!!    family of sire is building : One sire have severals dams who have severals progenies
!!  EXAMPLE
!!    do ip=1,np
!!        nm1=nmp(ip)+1
!!        nm2=nmp(ip+1)
!!        do jm=nm1,nm2
!!         ...
!!        end do
!!    end do
!!***
   integer              , dimension (:),allocatable,public :: nmp
!!****v* m_qtlmap_data/ndm
!!  NAME
!!    ndm
!!  DESCRIPTION
!!    Id progeny by dam index
!!    family of sire is building : One sire have severals dams who have severals progenies
!!  EXAMPLE
!!    do ip=1,np
!!        nm1=nmp(ip)+1
!!        nm2=nmp(ip+1)
!!        do jm=nm1,nm2
!!          nd1=ndm(jm)+1
!!          nd2=ndm(jm+1)
!!          do id=nd1,nd2
!!             ...
!!          end do
!!        end do
!!    end do
!!***
   integer              , dimension (:),allocatable,public :: ndm
!!****v* m_qtlmap_data/femelle
!!  NAME
!!    female
!!  DESCRIPTION
!!    List of female id : a sub set of mere. contains single id
!!  NOTES
!!***
   character(len=LEN_DEF) , dimension (:),allocatable,public :: femelle
!!****v* m_qtlmap_data/nfem
!!  NAME
!!    nfem
!!  DESCRIPTION
!!    Number of female : equivalent to size(female)
!!  NOTES
!!***
   integer                    ,public                      :: nfem = 0! size(female)
!!****v* m_qtlmap_data/repfem
!!  NAME
!!    repfem
!!  DESCRIPTION
!!    Corresponding index between mere array and female array
!!  NOTES
!!   repfem(indmere)=indfemale => mere(indmere) == female(indfemale)
!!***
   integer              , dimension (:),allocatable,public :: repfem


   !***************************************************************************
   ! MAP PART
   !***************************************************************************

!!****v* m_qtlmap_data/nmk
!!  NAME
!!    nmk
!!  DESCRIPTION
!!    Number of the marker by chromosome
!!  NOTES
!!    nmk(1) => number of marker of the first chromosome ch(1)
!!    nmk(2) => number of marker of the second chromosome
!!    ...
!!    Information from the map file
!!***
   integer ,  dimension (:),   allocatable, public            :: nmk
!!****v* m_qtlmap_data/mark
!!  NAME
!!    mark
!!  DESCRIPTION
!!    Name of the markers by chromosome
!!  NOTES
!!    marker(1,1) => name of first marker from the fisrt chromosome ch(1)
!!    ...
!!    Information from the map file (rec=1)
!!***
   character(len=LEN_DEF) , dimension (:,:), allocatable,public :: mark
!!****v* m_qtlmap_data/ch
!!  NAME
!!    ch
!!  DESCRIPTION
!!    Name of the chromosome
!!  NOTES
!!    ch(1) => name of the fisrt chromosome
!!    ...
!!    Information from the map file (rec=2)
!!***
   character(len=LEN_DEF) , dimension (:), allocatable,public   :: ch
!!****v* m_qtlmap_data/posi
!!  NAME
!!    posi
!!  DESCRIPTION
!!    Average position in Morgan by chromosome and marker
!!  NOTES
!!    posi(1,1) => position of the fisrt marker on the fisrt chromosome
!!    ...
!!    Information from the map file (rec=3)
!!***
   real (kind=dp), dimension (:,:), allocatable,public        :: posi
   !Array mal value (rec=4) from map
!!****v* m_qtlmap_data/posim
!!  NAME
!!    posim
!!  DESCRIPTION
!!    Mal position in Morgan by chromosome and marker
!!  NOTES
!!    posim(1,1) => position of the fisrt marker on the fisrt chromosome
!!    ...
!!    Information from the map file (rec=4)
!!***
   real (kind=dp), dimension (:,:), allocatable,public        :: posim
!!****v* m_qtlmap_data/posif
!!  NAME
!!    posif
!!  DESCRIPTION
!!    Female position in Morgan by chromosome and marker
!!  NOTES
!!    posif(1,1) => position of the fisrt marker on the fisrt chromosome
!!    ...
!!    Information from the map file (rec=5)
!!***
   real (kind=dp), dimension (:,:), allocatable,public        :: posif
!!****v* m_qtlmap_data/rm
!!  NAME
!!    rm
!!  DESCRIPTION
!!    Xaldane distance computed with posim position
!!***
   real (kind=dp), dimension (:,:,:), allocatable,public      :: rm
!!****v* m_qtlmap_data/rf
!!  NAME
!!    rf
!!  DESCRIPTION
!!    Xaldane distance computed with posif position
!!***
   real (kind=dp), dimension (:,:,:), allocatable,public      :: rf

!!****v* m_qtlmap_data/absi
!!  NAME
!!    absi
!!  DESCRIPTION
!!    get abscisse in Morgan corresponding to a a position N on the chromosome CHR
!!***
real (kind=dp)   ,dimension(:,:),allocatable   ,public        :: absi

   !***************************************************************************
   ! GENOTYPE PART
   !***************************************************************************
!!****v* m_qtlmap_data/numero
!!  NAME
!!    numero
!!  DESCRIPTION
!!    List of animal with phenotype marker information
!!  NOTES
!!    Information from the genotype file
!!***
   character(len=LEN_DEF) , dimension (:),allocatable      ,public :: numero
!!****v* m_qtlmap_data/nmes
!!  NAME
!!    nmes
!!  DESCRIPTION
!!    Number of animal with phenotype marker
!!***
   integer                                               ,public :: nmes = 0
!!****v* m_qtlmap_data/pheno
!!  NAME
!!    pheno
!!  DESCRIPTION
!!    Phenotype marker (not oriented information) of each animal contains in array numero
!!  DIMENSIONS
!!    1 : chromosome index
!!    2 : marker index
!!    3 : animal index
!!    4 : allele number (1 or 2)
!!
!!  NOTES
!!    Information from the genotype file
!!***
   integer(kind=KIND_PHENO)   ,dimension (:,:,:,:),allocatable  ,public :: pheno
!!****v* m_qtlmap_data/value_pheno
!!  NAME
!!    value_pheno
!!  DESCRIPTION
!!    Contains the original string value of a phenotype marker by chromosome
!!  NOTES
!!    QTLMap manage phenotype marker (allele) with integer id to improve performance
!!    and minimize the memory space during a qtlmap process
!!  SEE ALSO
!!    m_qtlmap_data/get_ind_pheno
!!    m_qtlmap_data/get_pheno
!!***
   character(len=LEN_DEF) , dimension (:,:)    ,allocatable  ,public :: value_pheno

!!****v* m_qtlmap_data/nb_value_pheno
!!  NAME
!!    nb_value_pheno
!!  DESCRIPTION
!!    Contains the number of allele stored in the chromosome c
!!  NOTES
!!    QTLMap manage phenotype marker (allele) with integer id to improve performance
!!    and minimize the memory space during a qtlmap process
!!  SEE ALSO
!!    m_qtlmap_data/value_pheno
!!    m_qtlmap_data/get_ind_pheno
!!    m_qtlmap_data/get_pheno
!!***
   integer, dimension (:)    ,allocatable  ,public :: nb_value_pheno

!!****v* m_qtlmap_data/corregp
!!  NAME
!!    corregp
!!	  DESCRIPTION
!!    Contains the corresponding numro index from a gpere index
!!  NOTES
!!    numeroIndex = corregp(indgpere) => numero(numeroIndex) == gpere(indgpere)
!!    size(corregp) = ngp
!!***
   integer            , dimension (:),allocatable        ,public :: corregp
!!****v* m_qtlmap_data/corregm
!!  NAME
!!    corregm
!!  DESCRIPTION
!!    Contains the corresponding numero index from a gmere index
!!  NOTES
!!    numeroIndex = corregm(indgmere) => numero(numeroIndex) == gmere(indgmere)
!!    size(corregm) = ngm
!!***
   integer            , dimension (:),allocatable        ,public :: corregm
!!****v* m_qtlmap_data/correr
!!  NAME
!!    correr
!!  DESCRIPTION
!!    Contains the corresponding numero index from a reproductor index
!!  NOTES
!!    numeroIndex = correr(indrepro) => numero(numeroIndex) == repro(indrepro)
!!    size(correr) = nr
!!***
   integer            , dimension (:),allocatable        ,public :: correr
!!****v* m_qtlmap_data/correp
!!  NAME
!!    correp
!!  DESCRIPTION
!!    Contains the corresponding numero index from a sire index
!!  NOTES
!!    numeroIndex = correp(indpere) => numero(numeroIndex) == pere(indpere)
!!    size(correp) = np
!!***
   integer            , dimension (:),allocatable        ,public :: correp
!!****v* m_qtlmap_data/correm
!!  NAME
!!    correm
!!  DESCRIPTION
!!    Contains the corresponding numero index from a dam index
!!  NOTES
!!    numeroIndex = correm(indmere) => numero(numeroIndex) == mere(indmere)
!!    size(correm) = nm
!!***
   integer            , dimension (:),allocatable        ,public :: correm
!!****v* m_qtlmap_data/corred
!!  NAME
!!    corred
!!  DESCRIPTION
!!    Contains the corresponding numero index from an animal index
!!  NOTES
!!    numeroIndex = corred(indanaimal) => numero(numeroIndex) == animal(indanimal)
!!    size(corred) = nd
!!***
   integer            , dimension (:),allocatable        ,public :: corred
!!****v* m_qtlmap_data/nall
!!  NAME
!!    nall
!!  DESCRIPTION
!!   Contains the number of allele by chromosome and marker
!!  DIMENSIONS
!!   1 : chromosome index
!!   2 : marker index
!!
!!***
   integer            , dimension (:,:),allocatable        ,public :: nall
   ! dim nd : true if genotype for animal (genealogy) is defined
!!****v* m_qtlmap_data/presentg
!!  NAME
!!    presentg
!!  DESCRIPTION
!!    Logical value for each progeny animal. True if the animal is genotyped on the chromosome CH
!!  NOTES
!!   size (presentg) = size(animal) * nchr
!!   To known if a progeny is genotyped (from animal array) :
!!   animal(kd) => presentg(kd) , true if genotyped ; false otherwise
!!  DIMENSIONS
!!   1 : chromosome index
!!   2 : animal index
!!
!!***
   logical, dimension (:,:), allocatable                   ,public :: presentg
   !============== VERIF ALLELES NECESSAIRE ====================
!!****v* m_qtlmap_data/alleles
!!  NAME
!!    alleles
!!  DESCRIPTION
!!    Get the string value of the allele referenced.
!!  DIMENSIONS
!!   1: chromosome index
!!   2: marker index
!!   3: allele index ( 1 < allele index < nall(ch,m) )
!!***
   character(len=LEN_DEF) , dimension (:,:,:),allocatable    ,public :: alleles
!!****v* m_qtlmap_data/pc_all
!!  NAME
!!    pc_all
!!  DESCRIPTION
!!    Frequencies alleles.
!!  DIMENSIONS
!!   1: chromosome index (ch)
!!   2: marker index     (m)
!!   3: allele index ( 1 < allele index < nall(ch,m) )
!!***
   real (kind=dp), dimension (:,:,:), allocatable          ,public :: pc_all

   !***************************************************************************
   ! VARIABLE PART - PERFORMANCE,COVARIATE,FIXED EFFECT
   !***************************************************************************
!!****v* m_qtlmap_data/bete
!!  NAME
!!    bete
!!  DESCRIPTION
!!    List of animal with traits
!!***
   character(len=LEN_DEF) , dimension (:),allocatable      ,public :: bete
!!****v* m_qtlmap_data/modele
!!  NAME
!!    modele
!!  DESCRIPTION
!!    model for each fixed effects, each covariates and each interactions beetwen the QTL and the fixed effects.
!!  DIMENSIONS
!!    1: trait index
!!    2: see notes
!!  NOTES
!!    modele (x,1) = number of fixed effets for the trait x
!!    modele (x,2) = number of covariate for the trait x
!!    modele (x,3) = number of qtl-fe for the trait x
!!    modele (x,3+if) = value of fixed effect 'if' for the trait x
!!    modele (x,3+nfix+icov) = value of covariate icov for the trait x
!!    modele (x,3+nfix+ncov+ifix) = value of fixed effect with qtl effect for the trait x
!!
!!    size(model,2) = 3+2*nfix+ncov
!!    Information from the trait file
!!***
   integer , dimension (:,:),allocatable                ,public :: modele
!!****v* m_qtlmap_data/ncar
!!  NAME
!!    ncar
!!  DESCRIPTION
!!    Number of traits (continue and discrete)
!!***
   integer                                              ,public :: ncar = 0
!!****v* m_qtlmap_data/ncarcat
!!  NAME
!!    ncarcat
!!  DESCRIPTION
!!    Number of categorials traits
!!***
   integer                                              ,public :: ncarcat = 0
!!****v* m_qtlmap_data/carac
!!  NAME
!!   carac
!!  DESCRIPTION
!!    List of traits name
!!***
   character(len=LEN_DEF) , dimension (:),allocatable     ,public :: carac
!!****v* m_qtlmap_data/natureY
!!  NAME
!!   natureY
!!  DESCRIPTION
!!    Nature of each trait :
!!      * 'r' for real or continue value
!!      * 'i' for integer or discrete value
!!      * 'c' for categorial value
!!    Information from the model file
!!***
   character(len=1)    ,  dimension (:),allocatable     ,public :: natureY
!!****v* m_qtlmap_data/y
!!  NAME
!!   y
!!  DESCRIPTION
!!    Value of discrete and continue value
!!
!!  DIMENSIONS
!!   1: trait index
!!   2: animal index
!!
!!  NOTES
!!    Information from the trait file
!!***
   real (kind=dp), dimension (:,:), allocatable          ,public   :: y
!!****v* m_qtlmap_data/cd
!!  NAME
!!   cd
!!  DESCRIPTION
!!    missing data or cd of index data
!!  DIMENSIONS
!!  1 : Trait index
!!  2 : Phenotyped animal referenced in bete
!!***
   real          , dimension (:,:), allocatable          ,public :: cd


!!****v* m_qtlmap_data/h2
!!  NAME
!!   h2
!!  DESCRIPTION
!!    heritabilities of traits , dim : ncar
!!  DIMENSIONS
!!  1 : Trait index
!!***
  real (kind=dp), dimension (:), allocatable   ,public :: h2
!!****v* m_qtlmap_data/RhoP
!!  NAME
!!   RhoP
!!  DESCRIPTION
!!    Correlation phenotypique
!!  DIMENSIONS
!!  1 : Trait index
!!  2 : Trait index
!!***
  real (kind=dp), dimension (:,:), allocatable   ,public :: RhoP
!!****v* m_qtlmap_data/RhoG
!!  NAME
!!   RhoG
!!  DESCRIPTION
!!    Correlation genetic
!!  DIMENSIONS
!!  1 : Trait index
!!  2 : Trait index
!!***
  real (kind=dp), dimension (:,:), allocatable   ,public :: RhoG

!!****v* m_qtlmap_data/xmut
!!  NAME
!!   xmut
!!  DESCRIPTION
!!    Means of each traits
!!  DIMENSIONS
!!   1: trait index
!!***
   real (kind=dp), dimension (:),allocatable             ,public   :: xmut
!!****v* m_qtlmap_data/sigt
!!  NAME
!!   sigt
!!  DESCRIPTION
!!    Standart deviation of each traits
!!  DIMENSIONS
!!   1: trait index
!!***
   real (kind=dp), dimension (:),allocatable             ,public   :: sigt
!!****v* m_qtlmap_data/ydiscretord
!!  NAME
!!   ydiscretord
!!  DESCRIPTION
!!    recodified class computed from discrete data
!!  DIMENSIONS
!!   1: trait index
!!   2: animal index
!!***
   integer        ,dimension(:,:)    ,allocatable         ,public  :: ydiscretord
!!****v* m_qtlmap_data/ycategorial
!!  NAME
!!   ycategorial
!!  DESCRIPTION
!!    Value of categorial value
!!  DIMENSIONS
!!   1: trait index
!!   2: animal index
!!  NOTES
!!   Information from the trait file
!!***
   character(len=LEN_DEF)  ,dimension(:,:)  ,allocatable  ,public    :: ycategorial
!!****v* m_qtlmap_data/indicemod
!!  NAME
!!   indicemod
!!  DESCRIPTION
!!    Correspond value from a original discrete value and the array ydiscretord
!!  DIMENSIONS
!!   1: trait index , c
!!   2: number of class (nmod(c))
!!***
   integer        ,dimension(:,:)    ,allocatable         ,public  :: indicemod
!!****v* m_qtlmap_data/nmod
!!  NAME
!!   nmod
!!  DESCRIPTION
!!    Number of classes corresponding with discrete value
!!  DIMENSIONS
!!   1: trait index , c
!!***
   integer          ,dimension(:)   ,allocatable          ,public  :: nmod
!!****v* m_qtlmap_data/seuil
!!  NAME
!!   seuil
!!  DESCRIPTION
!!    Thresholds on the underlying scale
!!  DIMENSIONS
!!   1: trait index , c
!!   2: number of class (nmod(c))
!!***
   !
   real(kind=dp)          ,dimension(:,:) ,allocatable    ,public  :: seuil
!!****v* m_qtlmap_data/prop
!!  NAME
!!   prop
!!  DESCRIPTION
!!    Proportion of the discrete classes
!!  DIMENSIONS
!!   1: trait index , c
!!   2: number of class (nmod(c))
!!***
   real(kind=dp)          ,dimension(:,:) ,allocatable    ,public  :: prop

!!****v* m_qtlmap_data/presentc
!!  NAME
!!   presentc
!!  DESCRIPTION
!!    Array indicator to known if a trait is present for a progeny
!!  DIMENSIONS
!!   1: trait index
!!   2: animal id referenced in animal
!!  NOTES
!!    size(2,presentc) == size(animal)
!!***
   logical, dimension (:,:), allocatable                 ,public :: presentc
!!****v* m_qtlmap_data/ndelta
!!  NAME
!!   ndelta
!!  DESCRIPTION
!!   Contains the column "present trait" for each trait
!!  DIMENSIONS
!!   1: trait index
!!   2: animal id referenced in animal
!!  NOTES
!!    size(2,presentc) == size(animal)
!!    Information from the trait file
!!***
   integer , dimension (:,:),allocatable               ,public   :: ndelta
!!****v* m_qtlmap_data/corperf
!!  NAME
!!   corperf
!!  DESCRIPTION
!!   Contains the corresponding index of y from an index animal
!!  DIMENSIONS
!!   1: animal id referenced in animal
!!  NOTES
!!    indBete = corperf(idanimal) => y(1,indBete) is the first trait of the animal animal(indanimal)
!!***
   integer , dimension (:),allocatable                   ,public :: corperf
  ! qtl effect ,dim  ncar,nqtl
  real (kind=dp), dimension (:,:), allocatable ,public :: ue
  ! dim : 1er trait, 2nd modality
  real (kind=dp),dimension(:,:)  , allocatable ,public :: soglia ! user threhold
   ! dim : 1er trait,2nd modality
  real (kind=dp),dimension(:,:)  , allocatable ,public :: freqmodsimultrait
  ! dim : trait
  integer,dimension(:)         , allocatable ,public   :: nbmodsimultrait
  ! Simulation H1 - QTL description
  !-------------------
  !integer                      ,public                  :: nqtl
  real (kind=dp), dimension (:), allocatable,   public  :: posiqtl
  integer       , dimension (:), allocatable,   public  :: chrqtl
  real (kind=dp), dimension (:), allocatable   ,public  :: xlim



!!****v* m_qtlmap_data/nfix
!!  NAME
!!   nfix
!!  DESCRIPTION
!!   Number of fixed effects
!!  NOTES
!!    Information from the model file
!!***
   integer                                              ,public  :: nfix = 0
!!****v* m_qtlmap_data/namefix
!!  NAME
!!   namefix
!!  DESCRIPTION
!!   List of fixed effects
!!  NOTES
!!    Information from the model file
!!***
   character(len=LEN_DEF) , dimension (:),allocatable     ,public  :: namefix
!!****v* m_qtlmap_data/nlev
!!  NAME
!!   nlev
!!  DESCRIPTION
!!   Number of levels per fixed effect
!!  DIMENSIONS:
!!  1: fixed effect index
!!***
   integer , dimension (:),  allocatable                 ,public :: nlev
!!****v* m_qtlmap_data/listelev
!!  NAME
!!   listelev
!!  DESCRIPTION
!!   String value of a fixed effect
!!  DIMENSIONS:
!!    1: fixed effect index
!!  NOTES
!!    valueFixedEffect=listelev(ifix,valueIndex),  0 <= valueIndex <= nlev(ifix)
!!***
   character(len=LEN_DEF) , dimension (:,:)  ,allocatable  ,public :: listelev
!!****v* m_qtlmap_data/nivx
!!  NAME
!!   nivx
!!  DESCRIPTION
!!   Index value of fixed effect for each animal
!!  DIMENSIONS:
!!    1: animal index
!!    2: fixed effect index
!!  NOTES
!!    nivx(idAnimal,ifix)=valueIndex => value=listelev(ifix,valueIndex)
!!***
   integer , dimension (:,:),allocatable                ,public  :: nivx
!!****v* m_qtlmap_data/ncov
!!  NAME
!!   ncov
!!  DESCRIPTION
!!   Number of covariate
!!  NOTES
!!    Information from the model file
!!***
   integer                                              ,public  :: ncov = 0
!!****v* m_qtlmap_data/namecov
!!  NAME
!!   namecov
!!  DESCRIPTION
!!   List of Covariate name
!!  NOTES
!!    Information from the model file
!!***
   character(len=LEN_DEF) , dimension (:),allocatable     ,public  :: namecov
!!****v* m_qtlmap_data/covar
!!  NAME
!!   covar
!!  DESCRIPTION
!!   Index value of the covariate for each animal
!!  DIMENSIONS:
!!    1: animal index
!!    2: covariate index
!!***
   real (kind=dp), dimension (:,:), allocatable         ,public  :: covar


!!****v* m_qtlmap_data/nchr
!!  NAME
!!   nchr
!!  DESCRIPTION
!!   Number of chromosome to analyse
!!  NOTES
!!   Information from the parameter file
!!***
   integer                              ,            public      :: nchr
!!****v* m_qtlmap_data/chromo
!!  NAME
!!   chromo
!!  DESCRIPTION
!!   List of chromosome name
!!  NOTES
!!   Information from the map file
!!***
   character(len=LEN_S) ,dimension(:) , allocatable,         public      :: chromo

!****************************************************************************
! OPTION PART

!!****v* m_qtlmap_data/opt_sib
!!  NAME
!!   opt_sib
!!  DESCRIPTION
!!   Treat family
!!  NOTES
!!   Information from argument of the execution
!!***
 ! option de traitement des familles en demi germains ou m�lange plein/ demi germains
   integer                              ,            public      :: opt_sib

   !****************************************************************************
   !
   !
!!****v* m_qtlmap_data/estime
!!  NAME
!!   estime
!!  DESCRIPTION
!!   Estimability per progeny phenotyped
!!  DIMENSIONS
!!   1: trait index
!!   2: dam index
!!  NOTES
!!   size(2,estime) == size(mere)
!!
!!***
   ! Estimabilité / nb de descendant phenotype : (1er dim indicie caractere/ 2 eme dim : indice mere)
   logical , dimension (:,:),allocatable,public :: estime

!!****v* m_qtlmap_data/nmumest
!!  NAME
!!   nmumest
!!  DESCRIPTION
!!   Number of dam with estimability
!!  DIMENSIONS
!!   1: trait index
!!
!!  NOTES
!!
!!
!!***
   integer  ,    dimension (:),allocatable         ,public          :: nmumest
!!****v* m_qtlmap_data/namest
!!  NAME
!!   namest
!!  DESCRIPTION
!!   Number of female with estimability
!!  DIMENSIONS
!!   1: trait index
!!
!!  NOTES
!!  if none dams are coupled with several sire, nmumest == namest
!!
!!***
   integer  ,    dimension (:),allocatable         ,public          :: namest

!!****v* m_qtlmap_data/iam
!!  NAME
!!   iam
!!  DESCRIPTION
!!   Indexe of the female
!!  DIMENSIONS
!!   1: trait index
!!   2: female index
!!  NOTES
!!  iam(ifem) = indam : 0<= indam <= namest
!!
!!***
   integer              , dimension (:,:)      ,allocatable,public :: iam

!!****v* m_qtlmap_data/estfem
!!  NAME
!!   estfem
!!  DESCRIPTION
!!   Estimability per progeny genotyped
!!  DIMENSIONS
!!   1: trait index
!!   2: female index
!!  NOTES
!!   size(2,estime) == size(mere)
!!
!!***
   ! Estimabilité / nb de descendant genotypé    ( 1ere dim : indice female)
   logical , dimension (:),allocatable,public   :: estfem


!!****v* m_qtlmap_data/listModelTrait
!!  NAME
!!   listModelTrait
!!  DESCRIPTION
!!   Description of model for each trait
!!  DIMENSIONS
!!   1: trait index
!!  NOTES
!!***
   type(model_trait) , dimension(:),allocatable :: listModelTrait

!!****v* m_qtlmap_data/optim_maxeval
!! NAME
!!   optim_maxeval
!! DESCRIPTION
!!   number maximum of evaluation
!! SOURCE
      integer                       , public        :: optim_maxeval        = 0
!!***

!!****v* m_qtlmap_data/optim_maxtime
!! NAME
!!   optim_maxtime
!! DESCRIPTION
!!   maxtime for each optimization
!! SOURCE
      real(kind=dp)                 , public        :: optim_maxtime        = 0
!!***

!!****v* m_qtlmap_data/optim_tolx
!! NAME
!!   optim_tolx
!! DESCRIPTION
!!   tolerance for the variable x
!! SOURCE
      real(kind=dp)                 , public        :: optim_tolx           = 0
!!***

!!****v* m_qtlmap_data/optim_tolf
!! NAME
!!   optim_tolf
!! DESCRIPTION
!!   tolerance for the function evaluation result f
!! SOURCE
      real(kind=dp)                 , public        :: optim_tolf           = 0
!!***

!!****v* m_qtlmap_data/optim_tolg
!! NAME
!!   optim_tolg
!! DESCRIPTION
!!  tolerance for the gradient
!! SOURCE
      real(kind=dp)                 , public        :: optim_tolg           = 0
!!***

!!****v* m_qtlmap_data/optim_H_PRECISION
!! NAME
!!   optim_H_PRECISION
!! DESCRIPTION
!!  Precision gradiant df/dx = f(x+h)-f(x-h)/2*h
!! SOURCE
      real(kind=dp)                 , public        :: optim_H_PRECISION    = 5.d-5
!!***

!!****v* m_qtlmap_data/simTyp
!!  NAME
!!   simTyp
!!  DESCRIPTION
!!   flag to activate the simulation genotype
!!***
  logical                      ,public      :: simTyp                 = .false.
!!****v* m_qtlmap_data/genotypeFileDefined
!!  NAME
!!   genotypeFileDefined
!!  DESCRIPTION
!!   flag to test if a user genotype file existe
!!***
  logical                      ,public      :: genotypeFileDefined    = .false.
!!****v* m_qtlmap_data/simulMap
!!  NAME
!!   simulMap
!!  DESCRIPTION
!!   flag to activate a simulation map
!!***
  logical                      ,public      :: simulMap               = .false.
!!****v* m_qtlmap_data/mapFileDefined
!!  NAME
!!   mapFileDefined
!!  DESCRIPTION
!!   flag to test if a user map file existe
!!***
  logical                      ,public      :: mapFileDefined         = .false.
!!****v* m_qtlmap_data/simulTraits
!!  NAME
!!   simulTraits
!!  DESCRIPTION
!!   flag to activate a simulation trait
!!***
  logical                      ,public      :: simulTraits            = .false.
!!****v* m_qtlmap_data/traitsFileDefined
!!  NAME
!!   traitsFileDefined
!!  DESCRIPTION
!!   flag to test if a user phenotypic file existe
!!***
  logical                      ,public      :: traitsFileDefined      = .false.
!!****v* m_qtlmap_data/simulGenea
!!  NAME
!!   simulGenea
!!  DESCRIPTION
!!   flag to activate a simulation genealogy
!!***
  logical                      ,public      :: simulGenea             = .false.

!!****v* m_qtlmap_data/geneaFileDefined
!!  NAME
!!   geneaFileDefined
!!  DESCRIPTION
!!   flag to test if a user genealogy file existe
!!***
  logical                      ,public      :: geneaFileDefined       = .false.

!!! FILE NAME  INFORMATION

!!****v* m_qtlmap_data/in_genea
!!  NAME
!!   in_genea
!!  DESCRIPTION
!!   Genealogy file name.
!!***
   character(len=LENGTH_MAX_FILE)                            , public    :: in_genea=''


!!****v* m_qtlmap_data/in_perfs
!!  NAME
!!   perfs
!!  DESCRIPTION
!!   Phenotypic records user files / traits data
!!  DIMENSIONS
!!    MAX_FILES_INPUT
!!  NOTES
!!
!!***
  character(len=LENGTH_MAX_FILE), dimension(MAX_FILES_INPUT) , public    :: in_perfs=''     ! traits files
!!****v* m_qtlmap_data/in_param_ef
!!  NAME
!!   param_ef
!!  DESCRIPTION
!!   Model files describing fixed effect, covariate, interaction fixed effect and qtl for each traits
!!  NOTES
!!
!!***
  character(len=LENGTH_MAX_FILE)                             , public    :: in_param_ef=''  ! model description
!!****v* m_qtlmap_data/in_typage
!!  NAME
!!   typage
!!  DESCRIPTION
!!   Genotype records user files
!!***
  character(len=LENGTH_MAX_FILE), public                                 :: in_typage=''   ! genotype file
!!****v* m_qtlmap_data/in_carte
!!  NAME
!!   carte
!!  DESCRIPTION
!!   File map name
!!***
      character(len=LENGTH_MAX_FILE), public                             :: in_carte =''  ! fichier carte
!!****v* m_qtlmap_simulation/in_parsim
!!  NAME
!!   parsim
!!  DESCRIPTION
!!   simulation user file
!!***
  character(len=LENGTH_MAX_FILE)                             , public    :: in_parsim = ''  ! fichier de simulation


   !fichier analyse : 1er line
   character(len=LENGTH_MAX_FILE), public    :: resul=''    ! main result file
   character(len=LENGTH_MAX_FILE), public    :: resp=''     ! sire result file
   character(len=LENGTH_MAX_FILE), public    :: resm=''     ! dam result file
   character(len=LENGTH_MAX_FILE), public    :: pateff=''   ! files with Sire QTL effect estimations
   character(len=LENGTH_MAX_FILE), public    :: mateff=''   ! files with Dam QTL effect estimations
   character(len=LENGTH_MAX_FILE), public    :: pdedf=''     ! Generate file with grand parental segment transmission marginal probabilities
   character(len=LENGTH_MAX_FILE), public    :: pdecouple='' ! Generate file with grand parental segment transmission joint probabilities
   character(len=LENGTH_MAX_FILE), public    :: coeffda=''   ! linear combinaison of discriminant analysis output
   character(len=LENGTH_MAX_FILE), public    :: grid2qtl=''  ! grid LRT for 2 qtl

   character(len=LENGTH_MAX_FILE), public    :: summary=''       ! summary result file
   character(len=LENGTH_MAX_FILE), public    :: sum_2qtl=''      ! summary result file 2 qtl
   character(len=LENGTH_MAX_FILE), public    :: simula=''        ! fichier de resultat de simulation
   character(len=LENGTH_MAX_FILE), public    :: out_phases=''    ! fichier avec les phases completes
   character(len=LENGTH_MAX_FILE), public    :: out_haplotypes='' ! fichier avec les haplotypes
   character(len=LENGTH_MAX_FILE), public    :: out_freqall=''    ! allele frequency


end module m_qtlmap_data
