!!****m* DATA/m_qtlmap_haplotype_data
!!  NAME
!!    m_qtlmap_haplotype_data
!!  SYNOPSIS
!!   Storage of generating data by the haplotype package HAPLOTYPE
!!  DESCRIPTION
!!  Repository of haplotype data:
!!     - These data are instancied and initialized by the haplotype package
!!     - haplotype data are used by the analyse.
!!  NOTES
!!
!!  BUGS
!!
!!  SEE ALSO
!!    DATA/m_qtlmap_data
!!
!!  CREATION DATE
!!  28/08/2009
!!
!!  COPYRIGHT
!!***
!!/

module m_qtlmap_haplotype_data
   use m_qtlmap_constant
   implicit none
   save

!!****v* m_qtlmap_haplotype_data/prob_seuil_gam
!!  NAME
!!   prob_seuil_gam
!!  DESCRIPTION
!!    Minimal gamete probability (LDA Haplotype option)
!!
!!***
   real(kind=dp)                      ,public       :: prob_seuil_gam = 0.25d0

!!****v* m_qtlmap_haplotype_data/NB_HAPLO_PRIOR
!!  NAME
!!   NB_HAPLO_PRIOR
!!  DESCRIPTION
!!   Number maximum of haplotype allowed
!!  NOTES
!!   Initialized by m_qtlmap_parameter_file.f95
!!
!!***
   integer                       , public           :: NB_HAPLO_PRIOR     = 0

!!****v* m_qtlmap_haplotype_data/PROB_HAPLO_MIN
!!  NAME
!!   PROB_HAPLO_MIN
!!  DESCRIPTION
!!   miminum probability of a haplotype in the LD analysis
!!  NOTES
!!   Initialized by m_qtlmap_parameter_file.f95
!!
!!***
   real (kind=dp)                , public           :: PROB_HAPLO_MIN     = 0.1d0
!!****v* m_qtlmap_haplotype_data/LONG_MIN_IBS
!!  NAME
!!   LONG_MIN_IBS
!!  DESCRIPTION
!!
!!  NOTES
!!   Initialized by m_qtlmap_parameter_file.f95
!!
!!***
   integer                ,            public       :: LONG_MIN_IBS        = 0


!!****v* m_qtlmap_haplotype_data/longhap
!!  NAME
!!   longhap
!!  DESCRIPTION
!!   Length of the haplotypes to be followed in LD analyses
!!  NOTES
!!   Information from the parameter file
!!***
   integer ,                  public      :: LONGHAP                      = 0



!!****v* m_qtlmap_haplotype_data/genotyp
!!  NAME
!!   genotyp
!!  DESCRIPTION
!!   Genotype (phenotype oriented) from individu referenced in the pheno array
!!  DIMENSIONS
!!    1 : chromosome index
!!    2 : marker index
!!    3 : animal index
!!    4 : allele number (1 or 2)
!!
!!  NOTES
!!    Generated by the HAPLOTYPE package (Allocation and fill)
!!
!!***
   ! Genotype of individu
   ! dim : chr,marker,animal,orientation(1|2)
   integer(kind=KIND_PHENO)    ,dimension(:,:,:,:),allocatable   ,public :: genotyp
   
!!
!!****v* m_qtlmap_haplotype_data/genotypm
!!  NAME
!!   genotypm
!!  DESCRIPTION
!!   Within phase Genotype (phenotype oriented) of the dams referenced in the pheno array
!!  DIMENSIONS
!!    1 : chromosome index
!!    2 : marker index
!!    3 : animal x phase index
!!    4 : allele number (1 or 2)
!!
!!  NOTES
!!    Generated by the HAPLOTYPE package (Allocation and fill)
!!
!!***
   ! Within phase Genotype of dam
   ! dim : chr,marker,animal x phase,orientation(1|2)
   integer(kind=KIND_PHENO)    ,dimension(:,:,:,:),allocatable   ,public :: genotypm
!!****v* m_qtlmap_haplotype_data/tab_IBS
!!  NAME
!!   tab_IBS
!!  DESCRIPTION
!!   list of the gametes received by all progeny, assuming no double recombination between
!!   flanking informative markers
!!  DIMENSIONS
!!    1 : animal  index
!!    2 : marker index
!!    3 : gamete number (1 or 2)
!!
!!  NOTES
!!    Generated by the HAPLOTYPE package (Allocation and fill)
!!
!!***
   ! haplotye of progenies
   ! dim : animal,marker,orientation(1|2)
   integer(kind=KIND_PHENO)    ,dimension(:,:,:),allocatable   ,public :: tab_IBS

   !$omp threadprivate (tab_IBS)

!
!!****v* m_qtlmap_haplotype_data/shared_haplo
!!  NAME
!!   shared_haplo
!!  DESCRIPTION
!!   list of the maternally tranmitted segment wchich are IBS to one a the 2 sire haplotypes around a given position
!!  DIMENSIONS
!!    1 : animal  index
!!
!!  NOTES
!!    Generated by the HAPLOTYPE package (Allocation and fill)
!!
!!***
   ! haplotye of progenies
   ! dim : animal
   integer        ,dimension(:),allocatable   ,public :: shared_haplo

   !$omp threadprivate (shared_haplo)
!
!!****v* m_qtlmap_haplotype_data/tab_shared_haplo
!!  NAME
!!   tab_shared_haplo
!!  DESCRIPTION
!!   bilan of the maternally tranmitted segment which are IBS to one a the 2 sire haplotypes around a given position
!!  for each sire  : nb of time the first ans second sire haplotypes are found,
!! and frequency of discovery per length
!!  DIMENSIONS
!!    1 : sire ID
!!    2 : sire haplotype (1 or 2)
!!    3 : count ( total, then, per length
!!
!!  NOTES
!!    Generated by the HAPLOTYPE package (Allocation and fill)
!!
!!***
   ! haplotye of progenies
   ! dim : nb sires, 2, nb length
   integer        ,dimension(:,:,:),allocatable   ,public :: tab_shared_haplo

   !$omp threadprivate (tab_shared_haplo)
!!
!!****v* m_qtlmap_haplotype_data/num_haplo_pere
!!  NAME
!!   num_haplo_pere
!!  DESCRIPTION
!!   Index of both halotype of sires, defined by the central position and the length of this haplotype
!!  DIMENSIONS
!!    1 : sire index
!!    2 : allele number (1 or 2)
!!    3 : fully known haplotype
!!
!!  NOTES
!!    Generated by the HAPLOTYPE package (Allocation and fill)
!!
!!***
   ! Sire haplotypes
   ! dim : nb sire ,orientation(1|2), maxi nb of fully known haplotypes
     integer                   , dimension (:,:,:)    , allocatable    ,  public :: num_haplo_pere

   !$omp threadprivate (num_haplo_pere)

!
!!****v* m_qtlmap_haplotype_data/num_haplo_mere
!!  NAME
!!   num_haplo_mere
!!  DESCRIPTION
!!   Index of both halotype of dams, defined by the central position and the length of this haplotype
!!  DIMENSIONS
!!    1 : dam genotype index
!!    2 : allele number (1 or 2)
!!    3 : fully known haplotype
!!
!!  NOTES
!!    Generated by the HAPLOTYPE package (Allocation and fill)
!!
!!***
   ! Dam haplotypes
   ! dim : nb dam genotype ,orientation(1|2), maxi nb of fully known haplotypes
     integer                   , dimension (:,:,:)    , allocatable    ,  public :: num_haplo_mere

     !$omp threadprivate (num_haplo_mere)
!
!!****v* m_qtlmap_haplotype_data/num_haplo_desc
!!  NAME
!!   num_haplo_desc
!!  DESCRIPTION
!!   Index of haplotype received by each progeny from its dam, defined by the central position and the length of this haplotype
!!  DIMENSIONS
!!    1 : progeny index
!!    2 : allele number (1 to nb_gam)
!!    3 : fully known haplotype
!!
!!  NOTES
!!    Generated by the HAPLOTYPE package (Allocation and fill)
!!
!!***
   ! dam haplotypes transmitted to a progeny
   ! dim : nb progeny ,orientation(1|2), maxi nb of fully known haplotypes
     integer                   , dimension (:,:,:)    , allocatable    ,  public :: num_haplo_desc
!
    !$omp threadprivate (num_haplo_desc)

!!****v* m_qtlmap_haplotype_data/nb_gam_pere
!!  NAME
!!   nb_gam_pere
!!  DESCRIPTION
!!   Number of possible different haplotypes carried by a sire on a chromosome
!!  DIMENSIONS
!!    1 : sire index
!!    2 : allele number (1 /2)
!!
!!  NOTES
!!    Generated by the HAPLOTYPE package (Allocation and fill)
!!
!!***
   ! dim : nb sire ,orientation(1|2)
     integer                   , dimension (:,:)    , allocatable    ,  public :: nb_gam_pere


     !$omp threadprivate (nb_gam_pere)
!
!!****v* m_qtlmap_haplotype_data/nb_gam_mere
!!  NAME
!!   nb_gam_mere
!!  DESCRIPTION
!!   Number of possible different haplotypes carried by a dam x phase on a chromosome
!!  DIMENSIONS
!!    1 : dam genotype index
!!    2 : allele number (1 /2)
!!
!!  NOTES
!!    Generated by the HAPLOTYPE package (Allocation and fill)
!!
!!***
   ! dim : nb dam x genotypes  ,orientation(1|2)
     integer                   , dimension (:,:)    , allocatable    ,  public :: nb_gam_mere

     !$omp threadprivate (nb_gam_mere)
!
!!****v* m_qtlmap_haplotype_data/nb_gam_desc
!!  NAME
!!   nb_gam_desc
!!  DESCRIPTION
!!   Number of possible different haplotypes carried by a progeny on a possibe maternal chromosome
!!  DIMENSIONS
!!    1 : progeny index
!!    2 : possible haplotype id transmitted from the dam
!!
!!  NOTES
!!    Generated by the HAPLOTYPE package (Allocation and fill)
!!
!!***
   ! dim : nb progeny  ,3**longhap
     integer                   , dimension (:,:)    , allocatable    ,  public :: nb_gam_desc

     !$omp threadprivate (nb_gam_desc)
!
!!****v* m_qtlmap_haplotype_data/prob_gam_pere
!!  NAME
!!   prob_gam_pere
!!  DESCRIPTION
!!   probability of possible different haplotypes carried by a sire on a chromosome
!!  DIMENSIONS
!!    1 : sire index
!!    2 : allele number (1 /2)
!!    3 : fully known haplotype
!!
!!  NOTES
!!    Generated by the HAPLOTYPE package (Allocation and fill)
!!
!!***
   ! dim : nb sire ,orientation(1|2), nb of fully known haplotype (2**longhap)
     double precision           , dimension (:,:,:)    , allocatable    ,  public :: prob_gam_pere

     !$omp threadprivate (prob_gam_pere)
!
!!****v* m_qtlmap_haplotype_data/prob_gam_mere
!!  NAME
!!   prob_gam_mere
!!  DESCRIPTION
!!   Probability of possible different haplotypes carried by a dam x phase on a chromosome
!!  DIMENSIONS
!!    1 : dam genotype index
!!    2 : allele number (1 /2)
!!    3 : fully known haplotype
!!
!!  NOTES
!!    Generated by the HAPLOTYPE package (Allocation and fill)
!!
!!***
   ! dim : nb dam x genotype  ,orientation(1|2), nb of fully known haplotype (2**longhap)
     double precision           , dimension (:,:,:)    , allocatable    ,  public :: prob_gam_mere

     !$omp threadprivate (prob_gam_mere)
!
!!!!!****v* m_qtlmap_haplotype_data/prob_gam_desc
!!  NAME
!!   prob_gam_desc
!!  DESCRIPTION
!!   Probability of possible different haplotypes carried by a progeny on a possibe maternal chromosome
!!  DIMENSIONS
!!    1 : progenyindex
!!    2 : allele number (1 /2)
!!    3 : fully known haplotype
!!
!!  NOTES
!!    Generated by the HAPLOTYPE package (Allocation and fill)
!!
!!!!***
   ! dim : nb progeny ,nb possibe maternal chromosome, nb of fully known haplotype (2**longhap)
   !  double precision           , dimension (:,:,:)    , allocatable    ,  public :: prob_gam_desc

     !!!$omp threadprivate (prob_gam_desc)

!!****v* m_qtlmap_haplotype_data/nb_max_haplo
!!  NAME
!!   nb_max_haplo
!!  DESCRIPTION
!!   number of haplotypes found around tested location
!!   and having a length of longhap markers
!!
!!
!!***
   ! dim : nb haplotypes
     integer                                                        ,  public :: nb_max_haplo

     !$omp threadprivate (nb_max_haplo)

!!****v* m_qtlmap_haplotype_data/nb_haplo
!!  NAME
!!   nb_haplo
!!  DESCRIPTION
!!
!!  NOTES
!!
!!***
   integer ,                                              public      :: nb_haplo

   !$omp threadprivate (nb_haplo)
!!****v* m_qtlmap_haplotype_data/nb_haplo_complet
!!  NAME
!!   nb_haplo_complet
!!  DESCRIPTION
!!
!!  NOTES
!!
!!***
   integer ,                                              public      :: nb_haplo_complet

   !$omp threadprivate (nb_haplo_complet)

!****v* m_qtlmap_haplotype_data/nb_haplo_reduit
!!  NAME
!!   nb_haplo_reduit
!!  DESCRIPTION
!!
!!  NOTES
!!
!!***
   integer ,                                              public      :: nb_haplo_reduit

   !$omp threadprivate (nb_haplo_reduit)

!!****v* m_qtlmap_haplotype_data/liste_haplo_reduit
!!  NAME
!!   liste_haplo_reduit
!!  DESCRIPTION
!!
!!  NOTES
!!
!!***
    integer                 , dimension (:)      , allocatable   :: liste_haplo_reduit

    !$omp threadprivate (liste_haplo_reduit)

!!****v* m_qtlmap_haplotype_data/pb_haplo_reduit
!!  NAME
!!   pb_haplo_reduit
!!  DESCRIPTION
!!
!!  NOTES
!!
!!***
    real(kind=dp)                 , dimension (:)      , allocatable   :: pb_haplo_reduit

    !$omp threadprivate (pb_haplo_reduit)

!!****v* m_qtlmap_haplotype_data/comp_haplo_reduit
!!  NAME
!!   comp_haplo_reduit
!!  DESCRIPTION
!!
!!  NOTES
!!
!!***
   logical                 , dimension (:,:)      , allocatable   :: comp_haplo_reduit

   !$omp threadprivate (comp_haplo_reduit)

!!****v* m_qtlmap_haplotype_data/name_haplo
!!  NAME
!!   name_haplo
!!  DESCRIPTION
!!   character name of the haplotype found around a given locationn
!!   and having a length of longhap markers
!!  DIMENSIONS
!!    1 : haplotype index (as defined in num_haplo_pere, num_haplo_mere and num_haplo_desc)
!!
!!  NOTES
!!    Generated by the HAPLOTYPE_LDLA package (Allocation and fill)
!!
!!***
   ! dim : nb haplotypes
     character(LEN=400)                  , dimension (:)    , allocatable    ,  public :: name_haplo

     !$omp threadprivate (name_haplo)

!!****v* m_qtlmap_haplotype_data/name_haplo_complet
!!  NAME
!!   name_haplo_complet
!!  DESCRIPTION
!!   character name of possible fully known haplotype found around a given locationn
!!   and having a length of longhap markers
!!  DIMENSIONS
!!    1 : haplotype index (as defined in num_haplo_pere, num_haplo_mere and num_haplo_desc)
!!
!!  NOTES
!!    Generated by the HAPLOTYPE_LDLA package (Allocation and fill)
!!
!!***
   ! dim : nb haplotypes
     character(LEN=400)                  , dimension (:)    , allocatable    ,  public :: name_haplo_complet

     !$omp threadprivate (name_haplo_complet)

!!****v* m_qtlmap_haplotype_data/name_haplo_reduit
!!  NAME
!!   name_haplo_reduit
!!  DESCRIPTION
!!   character name of the haplotype found around a given locationn
!!   and having a length of longhap markers
!!  DIMENSIONS
!!    1 : haplotype index (as defined in num_haplo_pere, num_haplo_mere and num_haplo_desc)
!!
!!  NOTES
!!    Generated by the HAPLOTYPE_LDLA package (Allocation and fill)
!!
!!***
   ! dim : nb haplotypes
     character(LEN=400)                  , dimension (:)    , allocatable    ,  public :: name_haplo_reduit

     !$omp threadprivate (name_haplo_reduit)

!!****v* m_qtlmap_haplotype_data/count_haplo
!!  NAME
!!   count_haplo
!!  DESCRIPTION
!!   frequencies of the haplotypes found around a given location
!!   and having a length of longhap markers
!!  DIMENSIONS
!!    1 : number of positions tested on the chromosome
!!    2 : haplotype index (as defined in num_haplo_pere, num_haplo_mere and num_haplo_desc)
!!
!!  NOTES
!!    Generated by the HAPLOTYPE_LDLA package (Allocation and fill)
!!
!!***
   ! dim : nb haplotypes retenus
     real(kind=dp)                  , dimension (:)    , allocatable    ,  public :: count_haplo


     !$omp threadprivate (count_haplo)


!!****v* m_qtlmap_haplotype_data/nb_gam
!!  NAME
!!   nb_gam
!!  DESCRIPTION
!!
!!  NOTES
!!
!!***
    integer                 , dimension (:)      , allocatable   :: nb_gam

    !$omp threadprivate (nb_gam)

!!****v* m_qtlmap_haplotype_data/prob_gam
!!  NAME
!!   prob_gam
!!  DESCRIPTION
!!
!!  NOTES
!!
!!***
    real   (kind=dp)        , dimension (:,:)      , allocatable   :: prob_gam

     !$omp threadprivate (prob_gam)

!!****v* m_qtlmap_haplotype_data/pdd
!!  NAME
!!   pdd
!!  DESCRIPTION
!!
!!  DIMENSIONS
!!    1 : chromosome index (chr)
!!    2 : index from ngend array
!!    3 : Case of allele transmission 1 or 2 or 3 or 4 see above
!!    4 : position index above the chromosome chr
!!  NOTES
!!    Generated by the HAPLOTYPE package (Allocation and fill)
!!
!!   Cases of allele transmission
!!   ------------------------------------|
!!   |       Dam |  All1    |    All2    |
!!   | Sire      |          |            |
!!   |-----------------------------------|
!!   | All1      |   1      |       2    |
!!   |-----------|----------|------------|
!!   | All2      |   3      |       4    |
!!   |-------------------------------- --|
!!
!!   pdd(ChIndex,progenyInd,1,POSITION) = probability of transmision All1 from Sire and All1 from Dam at the position POSITION
!!   pdd(ChIndex,progenyInd,2,POSITION) = probability of transmision All1 from Sire and All2 from Dam at the position POSITION
!!   pdd(ChIndex,progenyInd,3,POSITION) = probability of transmision All2 from Sire and All1 from Dam at the position POSITION
!!   pdd(ChIndex,progenyInd,4,POSITION) = probability of transmision All2 from Sire and All2 from Dam at the position POSITION
!!
!!   1 <= POSTION <= get_npo
!!   SEE ALSO
!!   m_qtlmap_data/get_ilong
!!   m_qtlmap_data/get_npo
!!***
   ! dim : chr,ngend(size(ngend)),(1..4),npox
   real (kind=dp)             ,dimension(:,:,:,:),allocatable   ,public  :: pdd
!!****v* m_qtlmap_haplotype_data/ngenom
!!  NAME
!!   ngenom
!!  DESCRIPTION
!!
!!  DIMENSIONS
!!    1 : chromosome index
!!    2 : index dam
!!
!!  NOTES
!!    Generated by the HAPLOTYPE package (Allocation and fill)
!!
!!  EXAMPLE
!!      do ip=1,np
!!        nm1=nmp(ip)+1
!!        nm2=nmp(ip+1)
!!        do jm=nm1,nm2
!!         ngeno1=ngenom(c,jm)+1   ! first genotype of the dam jm
!!         ngeno2=ngenom(c,jm+1)   ! last genotype of the dam jm
!!         do geno=ngeno1,ngeno2
!!            ...
!!         end do
!!        end do
!!      end do
!!***
   ! dim nchr,nm+1
   integer     ,dimension(:,:),allocatable                       ,public :: ngenom
!!****v* m_qtlmap_haplotype_data/ngend
!!  NAME
!!   ngend
!!  DESCRIPTION
!!   Progeny associated with a genotype dam in a sire family
!!  DIMENSIONS
!!    1 : chromosome index
!!    2 : index genotype referenced in the ngenom array
!!
!!  NOTES
!!    Generated by the HAPLOTYPE package (Allocation and fill)
!!
!!  EXAMPLE
!!       do ip=1,np                      ! for each sire
!!          nm1=nmp(ip)+1
!!          nm2=nmp(ip+1)
!!          do jm=nm1,nm2                ! for each dam in the sire family
!!            ngeno1=ngenom(chr,jm)+1
!!            ngeno2=ngenom(chr,jm+1)
!!            do ig=ngeno1,ngeno2        ! for each genotype possibly of the current dam
!!              nd1=ngend(chr,ig)+1
!!              nd2=ngend(chr,ig+1)
!!              do kd=nd1,nd2            ! for each progeny associated with this genotype
!!                ...
!!              end do
!!            end do
!!          end do
!!       end do
!!***
   integer            ,dimension(:,:) ,allocatable            ,public    :: ngend
!!****v* m_qtlmap_haplotype_data/ndesc
!!  NAME
!!   ndesc
!!  DESCRIPTION
!!   Get the progeny index from the index ngend
!!  DIMENSIONS
!!    1 : chromosome index
!!    2 : index ngend
!!
!!  NOTES
!!    Generated by the HAPLOTYPE package (Allocation and fill)
!!
!!  EXAMPLE
!!       do ip=1,np                      ! for each sire
!!          nm1=nmp(ip)+1
!!          nm2=nmp(ip+1)
!!          do jm=nm1,nm2                ! for each dam in the sire family
!!            ngeno1=ngenom(chr,jm)+1
!!            ngeno2=ngenom(chr,jm+1)
!!            do ig=ngeno1,ngeno2        ! for each genotype possibly of the current dam
!!              nd1=ngend(chr,ig)+1
!!              nd2=ngend(chr,ig+1)
!!              do kd=nd1,nd2            ! for each progeny associated with this genotype
!!                   kkd=ndesc(chr,kd)   ! Get the kkd index corresponding to the animal array
!!                   if(presentc(ic,kkd)) then
!!                     pp=-pdd(chr,kd,1,n)-pdd(chr,kd,2,n)+pdd(chr,kd,3,n)+pdd(chr,kd,4,n)
!!                     pm=-pdd(chr,kd,1,n)+pdd(chr,kd,2,n)-pdd(chr,kd,3,n)+pdd(chr,kd,4,n)
!!                   end if
!!                   ...
!!              end do
!!            end do
!!          end do
!!       end do
!!***
   integer            ,dimension(:,:) ,allocatable            ,public    :: ndesc
!!****v* m_qtlmap_haplotype_data/probg
!!  NAME
!!   probg
!!  DESCRIPTION
!!   Probability of a dam genotype referenced in the ngend array
!!  DIMENSIONS
!!    1 : chromosome index
!!    2 : index ngend
!!
!!  NOTES
!!    Generated by the HAPLOTYPE package (Allocation and fill)
!!
!!***
   ! dim : nchr,size(ngend)
   real (kind=dp)         ,dimension(:,:),allocatable         ,public    :: probg
!!****v* m_qtlmap_haplotype_data/phasp
!!  NAME
!!   phasp
!!  DESCRIPTION
!!
!!  DIMENSIONS
!!    1 : chromosome index
!!    2 : index sire
!!
!!  NOTES
!!    Generated by the HAPLOTYPE package (Allocation and fill)
!!
!!***
   logical          ,dimension(:,:),allocatable                ,public   :: phasp
!!****v* m_qtlmap_haplotype_data/phasm
!!  NAME
!!   phasp
!!  DESCRIPTION
!!
!!  DIMENSIONS
!!    1 : chromosome index
!!    2 : index dam
!!
!!  NOTES
!!    Generated by the HAPLOTYPE package (Allocation and fill)
!!
!!***
   logical          ,dimension(:,:),allocatable                ,public   :: phasm


   public :: release_haplotype_data

   contains


    subroutine release_haplotype_data

      deallocate (genotyp)
      deallocate (genotypm)
      if (allocated(num_haplo_pere)) deallocate (num_haplo_pere)
      if (allocated(num_haplo_mere)) deallocate (num_haplo_mere)
      if (allocated(num_haplo_desc)) deallocate (num_haplo_desc)
      if (allocated(name_haplo)) deallocate (name_haplo)
      if (allocated(nb_gam)) deallocate (nb_gam)
      if (allocated(prob_gam)) deallocate (prob_gam)
      deallocate (pdd)
      deallocate (ngenom)
      deallocate (ngend)
      deallocate (ndesc)
      deallocate (probg)
      deallocate (phasp)
      deallocate (phasm)

    end subroutine release_haplotype_data

end module m_qtlmap_haplotype_data
