!!****m* DATA/QTLMAP_TYPES
!!  NAME
!!    QTLMAP_TYPES -- Definition of qtlmap types.
!!  SYNOPSIS
!!
!!  DESCRIPTION
!!
!!  NOTES
!!
!!  BUGS
!!
!!  SEE ALSO
!!
!!  COPYRIGHT
!!***
!! You can use this space for remarks that should not be included
!! in the documentation.
!!/
module QTLMAP_TYPES
  use m_qtlmap_constant

  implicit none
!!****t* QTLMAP_TYPES/model_trait
!! NAME
!!   DESC_EFFECT_TYPE
!! DESCRIPTION
!!
!! NOTES
!!    This type is used by all analysis to make homogene the print of a estimation parameter solution
!! SOURCE
  type model_trait
     ! number of fixed effect
     integer                                      :: nbfe
     ! number of covariate
     integer                                      :: nbco
     ! number of qtl with interaction
     integer                                      :: nbqtlinter
     ! number of interaction with the qtl for each qtl with interaction defined
     !size : nbqtlinter
     integer     ,dimension(:) ,pointer           :: nbint
     ! size : nbfe : reference
     integer     ,dimension(:) ,pointer           :: indexFixedEffect
     ! size : nbco : reference
     integer     ,dimension(:) ,pointer           :: indexCovariate
     !size : nbqtlinter,nbint
     integer     ,dimension(:,:) ,pointer         :: indexFixedEffectWithInteraction

  end type model_trait
!!***

!!****t* QTLMAP_TYPES/DESC_EFFECT_TYPE
!!  NAME
!!   DESC_EFFECT_TYPE
!!  DESCRIPTION
!!    Description of an effect enumered in the INCIDENCE_TYPE
!!
!!    attributes members :
!!    --------------------
!!    name        : name of the effect
!!    start       : level start of the effect in the incidence matrix
!!    end         : level end of the effect in the incidence matrix
!!    haveSubDesc : true if the effect have contains sub effect :
!!                  sample :'Fixed effect A' is a effect that contains several level (value 1, value 2,...)
!!    listSubDesc : recursive structure that contains the sub effect
!!  NOTES
!!    This type is used by all analysis to make homogene the print of a estimation parameter solution
!! SOURCE
    type DESC_EFFECT_TYPE
        ! Name of effect
        character(len=LEN_W)                                   :: name
        ! start ntniv
        integer                                                :: start   = -1
        ! end ntniv
        integer                                                :: end     = -1
        !If sub description :
        logical                                                :: haveSubDesc = .false.
        ! sub effect name
        type(DESC_EFFECT_TYPE),dimension(:),pointer            :: listSubDesc => NULL()

    end type DESC_EFFECT_TYPE
!!***
!!****t* QTLMAP_TYPES/TYPE_INCIDENCE_SOLUTION
!!  NAME
!!   TYPE_INCIDENCE_SOLUTION
!!  DESCRIPTION
!!    Description of a solution (estimation) of a analysis. All analysis fill this structure and give return the results.
!!
!!    attributes members :
!!    --------------------
!!    hypothesis       : Test hypothesis of the analysis
!!    sig              : solution of the standart deviation of each sire
!!    eqtl_print       : logical array that referenced the effect to print in the context of a eqtl printing
!!                       eqtl solution are printing in a array, we give only the following information :
!!                       st.dev, mean, qtl effect. All estimation of fixed effect and covariate are skipped
!!    groupeName       : All effect are categorized : General Mean, Polygenic effect, Fixed effect, Covariate, Qtl Effect
!!    nbParameterGroup : Number of parameter inside a group. sample, inside Polygenic effect : Sire A, Sire B,...
!!    parameterName    : Name of parameters by groupname
!!    paramaterValue   : Value of parameters by groupname
!!    parameterVecsol  : Estimability of parameters by groupname
!!    parameterPrecis  : Precision value of parameters by groupname
!!    qtl_groupeName   : Referenced the group of the QTL effect estimation
!!    rhoi             : residual correlation of traits
!! HISTORY
!!  09/09/2010    * add a dimension to qtl_groupeName (for detected the qtl effect with a specific trait on multi-trait analysis)
!! SOURCE
    type TYPE_INCIDENCE_SOLUTION
      ! Test based on the number of Qtls
      integer                                      :: hypothesis
      ! formated to print
      real (kind=dp)     ,dimension(:,:),pointer   :: sig => NULL()
      ! Standart deviation for unknown haplotype dam
      real (kind=dp)     ,dimension(:,:),pointer   :: unknown_dam_sig => NULL()
      !
      logical            ,dimension(:) ,pointer    :: eqtl_print => NULL()
      !
      character(len=LEN_W) ,dimension(:),pointer   :: groupeName => NULL()
      !
      integer              ,dimension(:),pointer   :: nbParameterGroup => NULL()
      ! List of parameter name
      character(len=LEN_W) ,dimension(:,:),pointer :: parameterName => NULL()
      ! List of value
      real (kind=dp)     ,dimension(:,:),pointer   :: paramaterValue => NULL()

      logical        , dimension(:,:) , pointer    :: parameterVecsol => NULL()

      real (kind=dp) ,dimension(:,:),pointer       :: parameterPrecis => NULL()

      ! Index postion in groupeName of qtl effect : ncar,nqtl
      integer       ,dimension(:,:),pointer        :: qtl_groupeName => NULL()

      real (kind=dp)     ,dimension(:,:),pointer   :: rhoi => NULL()
    end type TYPE_INCIDENCE_SOLUTION
!!***

!!****t* QTLMAP_TYPES/TYPE_LRT_SOLUTION
!!  NAME
!!   TYPE_LRT_SOLUTION
!!  DESCRIPTION
!!    Description of the likelihood ratio test
!!
!!    attributes members :
!!    --------------------
!!    nqtl       : hypothesis to test
!!    lrtmax     : maximum reached
!!    dxmax      : position where are the maximum
!!    nxmax      : position in the data structure where the maximum are reached
!!    chrmax     : chromosome where the maximum are reached
!!
!!    lrt1       : LRT curve under Hypothesis One (Only this hypothesis case)
!!    pater_eff  : paternal effect curve under Hypothesis One
!!    mater_eff  : maternal effect curve under Hypothesis One
!!    xlrp       : LRT sires surve under Hypothesis One
!!    xlrm       : LRT dams surve under Hypothesis One
!!
!!    lrt0_2     : LRT curve under Hypothesis Two against Zero (Only this hypothesis case)
!!    lrt1_2     : LRT curve under Hypothesis Two against One (Only this hypothesis case)
!!    pater_eff2 : paternal effect curve under Hypothesis Two
!!    mater_eff2 : maternal effect curve under Hypothesis Two
!!    xlrp2      : LRT sires surve under Hypothesis Two against One
!!    xlrm2      : LRT dams surve under Hypothesis Two against One
!!
!!  NOTES
!! SOURCE

    type TYPE_LRT_SOLUTION
      ! Test based on the number of Qtls
      integer                                      :: nqtl = -1
      ! LRT Maximum reached
      real (kind=dp)   ,dimension(:),pointer       :: lrtmax => NULL()
      ! Centimorgan positions of Qtls
      !real (kind=dp)   ,dimension(:),pointer       :: dxmax => NULL()
      ! Position of Qtl in data structure
      integer          ,dimension(:),pointer       :: nxmax => NULL()
      ! Chromosome where the position is localised
      integer          ,dimension(:),pointer       :: chrmax => NULL()
      !--------------------- QTL = 1 -----------------------------------
      ! chr,LRT curves if nqtl = 1
      real (kind=dp)   ,dimension(:,:),pointer     :: lrt1 => NULL()
      real (kind=dp)  ,dimension(:,:,:),pointer    :: pater_eff => NULL()
      real (kind=dp)   ,dimension(:,:,:),pointer   :: mater_eff => NULL()
      real (kind=dp)   ,dimension(:,:,:),pointer   :: xlrp      => NULL()
      real (kind=dp)   ,dimension(:,:,:),pointer   :: xlrm      => NULL()

      !--------------------- QTL = 2 ---------------------------------
      ! chr,LRT curves if nqtl = 2
      real (kind=dp) ,dimension(:,:,:,:),pointer   :: lrt0_2 => NULL()
      ! chr,LRT curves if nqtl = 2
      real (kind=dp) ,dimension(:,:,:,:),pointer   :: lrt1_2 => NULL()
      ! chr1,chr2,np,npo1,npo2,2 (effet 1, effet2)
      real (kind=dp)  ,dimension(:,:,:,:,:,:),pointer    :: pater_eff2 => NULL()
      real (kind=dp)  ,dimension(:,:,:,:,:,:),pointer    :: mater_eff2 => NULL()
      ! chr1,chr2,,np,npo1,npo2
      real (kind=dp)   ,dimension(:,:,:,:,:),pointer   :: xlrp2 => NULL()
      real (kind=dp)   ,dimension(:,:,:,:,:),pointer   :: xlrm2 => NULL()
    end type TYPE_LRT_SOLUTION
!!***

!!****t* QTLMAP_TYPES/CORR_ALERT_TYPE
!! NAME
!!   CORR_ALERT_TYPE
!! DESCRIPTION
!!    Description of a alert of a correlation to high between classical effect and a qtl effect
!!
!!    attributes members :
!!    --------------------
!!    qtl    : number of qtl to test
!!    ip     : sire index where the correlation are finding
!!    jm     : dams index where the correlation are finding
!!    corr   : value of the correlation
!!    ntlev  : level of the qtl
!!    name_effect  : name of the classical effect
!!    name_level   : name of qtl level
!! NOTES
!! SOURCE
    ! Type pour l impression des alerts de correlation trop eleve entre les effets polygenic, de nuisance et les effets qtls
    type CORR_ALERT_TYPE
         integer              :: qtl=0
         integer              :: ip = -1         ! indice du pere sinon -1
         integer              :: jm = -1         ! indice de la mere concerne sinon -1
         real(kind=dp)        :: corr            ! la correlation
         integer              :: ntlev =-1       ! le niveau du qtl concerne
         character(len=LEN_W) :: name_effect     ! le nom de l effet
         character(len=LEN_W) :: name_level      ! le nom du niveau
    end type CORR_ALERT_TYPE
!!***
!!****t* QTLMAP_TYPES/TEST_NUISANCES_TYPE
!!  NAME
!!   TEST_NUISANCES_TYPE
!!  DESCRIPTION
!!    Description of a nuisance test
!!
!!    attributes members :
!!    --------------------
!!    directeffect  : false if the qtl are in interaction with the effect
!!    name          : name without the nuisance
!!    df            : freedom degree
!!    lrt           : likelihood ratio test
!!    pvalue        :
!!  NOTES
!! SOURCE
    !type pour l impression de test lin
    type TEST_NUISANCES_TYPE
         logical              :: directeffect    ! intra qtl (fixed effect) or direct effect
         character(len=LEN_W) :: name            ! name without the nuisances
         integer              :: df
         real(kind=dp)        :: lrt
         real(kind=dp)        :: pvalue
    end type TEST_NUISANCES_TYPE
!!***

!!****d* QTLMAP_TYPES/MAXCAR
!!  NAME
!!   MAXCAR
!!  DESCRIPTION
!!   max trait for TRIO_TYPE type description
!!  NOTES
!!***
     integer                , parameter                         :: MAXCAR = 20

!!****t* QTLMAP_TYPES/TRIO_TYPE
!!  NAME
!!   TRIO_TYPE
!!  DESCRIPTION
!!
!!  NOTES
!!  SOURCE
     type TRIO_TYPE
          integer                                               :: idSire    = -1   ! sire
          integer                                               :: idDam     = -1   ! dam
          integer                                               :: idKd      = -1   ! Kd
          real(kind=dp)    ,dimension(MAXCAR)                   :: perfKd    = 0.d0
          logical          ,dimension(MAXCAR)                   :: presentc  =.false.

     end type TRIO_TYPE
!!***

!!****t* QTLMAP_TYPES/GENEALOGY_TYPE
!!  NAME
!!     GENEALOGY_TYPE
!!  DESCRIPTION
!!
!!  NOTES
!!
!! SOURCE
    type GENEALOGY_TYPE
         integer                                                 :: na        = 0       ! number of animal (the population)
         character(LEN=LEN_DEF) ,dimension(:) ,pointer           :: listNA    => null() ! the key ID => Name for each animal of the population
         type(TRIO_TYPE)  ,dimension(:) ,pointer                 :: trioKd    => null() ! List of all trio parent link
         integer           ,dimension(:) ,pointer                :: corranim  => null()

         integer          ,dimension(:,:),pointer                :: NCK                 ! nombre de descendant ayant une perf par KD pour le Caracteres i
         integer          ,dimension(:,:,:),pointer              :: NCiCjK              ! nombre de descendant ayant une perf par KD pour le Caracteres i et j
         real(kind=dp)    ,dimension(:,:,:),pointer              :: corcd    => null()
    end type GENEALOGY_TYPE
!!***



end module QTLMAP_TYPES
