!!****m* HAPLOTYPE/m_qtlmap_haplotype
!!  NAME
!!    m_qtlmap_haplotype
!!  DESCRIPTION
!!
!!  NOTES
!!
!!  BUGS
!!
!!  HISTORY
!!
!!  SEE ALSO
!!
!!  COPYRIGHT
!!***
!! You can use this space for remarks that should not be included
!! in the documentation.
!!/
module m_qtlmap_haplotype

   use m_qtlmap_tools
   use m_qtlmap_log
   use m_qtlmap_output_handler
   use m_qtlmap_haplotype_V1
   use m_qtlmap_haplotype_V2

   implicit none
!!****d* m_qtlmap_haplotype/VERSION_HAPLOTYPE_V1
!!  NAME
!!   VERSION_HAPLOTYPE_V1
!!  DESCRIPTION
!!   Constant for the first version of the haplotypes routines
!!  NOTES
!!
!!  SOURCE
   integer, public, parameter             :: VERSION_HAPLOTYPE_V1         = 1  ! Original version
!!***

!!****d* m_qtlmap_haplotype/VERSION_HAPLOTYPE_V2
!!  NAME
!!   VERSION_HAPLOTYPE_V2
!!  DESCRIPTION
!!   Constant for the second version of the haplotypes routines
!!  NOTES
!!
!!  SOURCE
   integer, public, parameter             :: VERSION_HAPLOTYPE_V2         = 2  !
!!***

!!****d* m_qtlmap_haplotype/VERSION_HAPLOTYPE_V3
!!  NAME
!!   VERSION_HAPLOTYPE_V3
!!  DESCRIPTION
!!   Constant for the third version of the haplotypes routines
!!  NOTES
!!
!!  SOURCE
   integer, public, parameter             :: VERSION_HAPLOTYPE_V3         = 3  !
!!***

!!****d* m_qtlmap_haplotype/VERSION_HAPLOTYPE_SNP
!!  NAME
!!   VERSION_HAPLOTYPE_SNP
!!  DESCRIPTION
!!   Constant for the 4th version of the haplotypes routines
!!  NOTES
!!   Elsen JM. A fast algorithm for estimating transmission probabilities in QTL detection designs with dense maps.
!!  SOURCE
   integer, public, parameter             :: VERSION_HAPLOTYPE_SNP        = 4  !
!!***

   public :: haplotype


   contains
!!****f* m_qtlmap_haplotype/haplotype
!!  NAME
!!    m_qtlmap_haplotype
!!  DESCRIPTION
!!   Apply the algorithm identify by opt_version
!!    - Initialisation of the following structures : pdd, genotyp, genotypm, ngenom, ngend, ndesc, probg, phasp, phasm
!!  INPUTS
!!    opt_version : the version of the haplotype to used
!!  SOURCE
      subroutine haplotype(opt_version)
        integer                         , intent(in)                           :: opt_version

        !free structure if allocated
        if ( allocated(pdd) ) deallocate( pdd )
        if ( allocated(ngend) ) deallocate( ngend )
        if ( allocated(ndesc) ) deallocate( ndesc )
        if ( allocated(probg) ) deallocate( probg )
        if ( allocated(genotyp) ) deallocate( genotyp )
        if ( allocated(genotypm) ) deallocate( genotypm )
        if ( allocated(ngenom) ) deallocate( ngenom )
        if ( allocated(phasp) ) deallocate( phasp )
        if ( allocated(phasm) ) deallocate( phasm )

        select case (opt_version)

        case (VERSION_HAPLOTYPE_V1)
         call log_mess('Computation of transmission probabilities V1',INFO_DEF)
         call haplotype_V1()

        case (VERSION_HAPLOTYPE_V2)
          call log_mess('Computation of transmission probabilities V2',INFO_DEF)
          call haplotype_V2()

        case (VERSION_HAPLOTYPE_V3)
            call log_mess('Computation of transmission probabilities V3',INFO_DEF)
            call haplotype_V3()

        case (VERSION_HAPLOTYPE_SNP)
          call log_mess('Computation of transmission probabilities SNP',INFO_DEF)
          call haplotype_SNP()
        case default
         call stop_application('bad value of opt_version['//trim(str(opt_version))//'].')
       end select

      end subroutine
!!***

!!****f* m_qtlmap_haplotype/get_pdd_at_mark
!!  NAME
!!    get_pdd_at_mark
!!  DESCRIPTION
!!    Get the index position of the array pdd (prob of transmission) at the position : marker lk on the chromosome ch.
!!  INPUTS
!!    ch : index chromosome
!!    lk : index marker
!!  OUTPUTS
!!    pos : index position
!!  SOURCE
      subroutine get_pdd_at_mark(ch,lk,pos)
          integer , intent(in)  :: ch
          integer , intent(in)  :: lk
          integer , intent(out) :: pos

          real(kind=dp) ,parameter :: epsil = 0.001

          integer :: istart,iend



      end subroutine get_pdd_at_mark
!!***

!!****f* m_qtlmap_haplotype/test_print_haplotype
!!  NAME
!!    test_print_haplotype
!!  DESCRIPTION
!!
!!  SOURCE
     subroutine test_print_haplotype

!      real (kind=dp)   ,dimension(:,:,:,:),allocatable :: prot
!      allocate (prot(nchr,maxval(nmk),nd,4),STAT=stat)
!
!      call gammapf(prot)
!
!      do ip=1,np
!	   do jm=nmp(ip)+1,nmp(ip+1)
!        do geno=ngenom(jm)+1,ngenom(jm+1)
!          do kd=ngend(geno)+1,ngend(geno+1)
!             kkd=ndesc(kd)
!             !! animal s ALL_M1 ALL_M2 ....  origine : 1/2
!          end do
!        end do
!       end do
!      end do
!
!      deallocate (prot)

     end subroutine test_print_haplotype
!!***

end module m_qtlmap_haplotype
