!!****m* INPUT/m_qtlmap_genealogy
!!  NAME
!!    m_qtlmap_genealogy -- Genealogy routines.
!!  SYNOPSIS

!!  DESCRIPTION
!!
!!  NOTES
!!
!!  BUGS
!!
!!  SEE ALSO
!!
!!  COPYRIGHT
!!***
!! You can use this space for remarks that should not be included
!! in the documentation.
!!/
module m_qtlmap_genealogy
  !Internal parameter
  use m_qtlmap_tools
  use m_qtlmap_log
  use m_qtlmap_data
  use m_qtlmap_output_handler
  use m_qtlmap_cd

  implicit none
  save

!!****v* m_qtlmap_genealogy/genealogy_outbred_gen
!!  NAME
!!   genealogy_outbred_gen
!!  DESCRIPTION
!!   Indicates the outbred generation in simulation case.
!!***
   logical                             ,private      :: genealogy_outbred_gen = .false.

!!****v* m_qtlmap_genealogy/calculCd
!!  NAME
!!   calculCd
!!  DESCRIPTION
!!   Indicates the calculus of censured data (asked by the user).
!!***
   logical                             ,private      :: calculCd = .false.

!!****v* m_qtlmap_genealogy/genea_list
!!  NAME
!!   genea_list
!!  DESCRIPTION
!!   Get information from genealogy file
!!  DIMENSIONS
!!    number of animal with the genealogy information
!!    3 : 1 -> ID ANIMAL, 2 -> SIRE, 3 -> DAM
!!***
   character(len=LEN_DEF)        , dimension(:,:),allocatable  ,private  :: genea_list
!!****v* m_qtlmap_genealogy/genea_niv
!!  NAME
!!   genea_niv
!!  DESCRIPTION
!!   Get the genealogy (0 grand-parents, 1, parents, 2 progeny)
!!  DIMENSIONS
!!    number of animal with the genealogy information
!!***
   integer                         , dimension(:),allocatable  ,private   :: genea_niv

  public :: read_genealogy
  public :: sim_genea
  public :: write_genea

  CONTAINS

!!****f* m_qtlmap_genealogy/read_genealogy
!!  NAME
!!    read_traits
!!  DESCRIPTION
!!    Read the genealogy user file. In a computation of censure data case, the genealogy number (4th record) can be greater than 3 :
!!    new traits values (and a censured data) are computated for each progeny which have progenies.
!!  INPUTS
!!   optcalculCd   : indicates the computation of censured data
!!
!!  NOTES
!!  SOURCE
       SUBROUTINE read_genealogy(optcalculCd)
        logical     ,optional  ,intent(in) :: optcalculCd
        integer                            :: nb_max_indiv = 0
        integer                            :: ios,eof,i,alloc_stat,j,niv
        integer , parameter                :: GENERATION_MAX=10

        call log_mess('SUBROUTINE : read_genealogy',DEBUG_DEF)
        call log_mess('reading genealogy file...',INFO_DEF)



        if (present(optcalculCd)) calculCd = optcalculCd

        allocate (genea_list(MAX_ANIMAL,3))
        allocate (genea_niv(GENERATION_MAX))
        genea_niv=0
        ios = 57
        ! compte max indiv and check line
        open(ios,file=in_genea)
        eof = 0
        i=1
        nb_max_indiv=0
        do while ( eof == 0 )
         read(ios,*,iostat=eof) (genea_list(i,j),j=1,3),niv
         if ( trim(genea_list(i,1)) /= '' .and. eof == 0 ) then
             nb_max_indiv = nb_max_indiv+1
             if (calculCd) then
               if ( niv == 2 ) then
                call add_animal_genea(datasetUser,genea_list(i,2),genea_list(i,3),genea_list(i,1),nb_max_indiv-sum(genea_niv(:1)))
               else
                call add_animal_genea(datasetUser,genea_list(i,2),genea_list(i,3),genea_list(i,1))
               end if
             end if
             i=i+1
             if (niv < 1 .or. niv> GENERATION_MAX ) then
               call stop_application("Bad definition of generation :"//trim(str(i)))
             end if
             genea_niv(niv) = genea_niv(niv)+1
         end if
        end do

        close(ios)

        !$OMP PARALLEL DEFAULT(SHARED) NUM_THREADS(2)
        !$OMP SECTIONS
        !$OMP SECTION
        CALL CREATE_STRUCT_GRAND_PARENT(nb_max_indiv)
        !$OMP SECTION
        CALL CREATE_STRUCT_PARENT(nb_max_indiv)
        !$OMP END SECTIONS NOWAIT
        !$OMP END PARALLEL

        ! create repere
        ALLOCATE (reppere(size(pere)), stat = alloc_stat)
        CALL check_allocate(alloc_stat,'reppere')
        ALLOCATE (repmere(size(mere)), stat = alloc_stat)
        CALL check_allocate(alloc_stat,'repmere')
        ALLOCATE (femelle(size(mere)), stat = alloc_stat)
        CALL check_allocate(alloc_stat,'femelle')
        ALLOCATE (repfem(size(mere)), stat = alloc_stat)
        CALL check_allocate(alloc_stat,'repfem')

        CALL CREATE_STRUCT_DERIVED_GENEALOGY()
        call log_mess('NP='//trim(str(np)),VERBOSE_DEF)
        call log_mess('NM='//trim(str(nm)),VERBOSE_DEF)
        call log_mess('ND='//trim(str(nd)),VERBOSE_DEF)
        call log_mess('END SUBROUTINE : read_genealogy',DEBUG_DEF)

        deallocate (genea_list)
        deallocate (genea_niv)

       END SUBROUTINE read_genealogy
!!***

!!****f* m_qtlmap_genealogy/CREATE_STRUCT_GRAND_PARENT
!!  NAME
!!    CREATE_STRUCT_GRAND_PARENT
!!  DESCRIPTION
!!    Fill ngp,ngm,ngmgp,nrgm,repro,gpere,gmere arrays from the information readed (genea_list,genea_niv).
!!  INPUTS
!!   nb_max_indiv   : get the number of animal defined in the genalogy file
!!
!!  NOTES
!!  SOURCE
       SUBROUTINE CREATE_STRUCT_GRAND_PARENT(nb_max_indiv)
       integer,intent(in)              ::nb_max_indiv

       character(len=LEN_DEF)            ::ind
       character(len=LEN_DEF)            ::father
       character(len=LEN_DEF)            ::mother
       integer                         ::gen,alloc_stat,l,i
       character(len=LEN_DEF)            ::word_token
       character(len=LEN_LINE)            ::line_read

       ! nombre de gd meme par gd pere : dim nbre gp
       integer , dimension (:),allocatable :: ngmgp_t
       ! nombre de parent par gd mere
       integer , dimension (:),allocatable :: nrgm_t

       character(len=LEN_DEF) , dimension (:),allocatable ::gmere_t
       character(len=LEN_DEF) , dimension (:),allocatable ::gpere_t
       character(len=LEN_DEF) , dimension (:),allocatable ::repro_t
       logical                                          :: is_ok

       call log_mess('SUBROUTINE : CREATE_STRUCT_GRAND_PARENT',DEBUG_DEF)
       ! Initialize Buffer with the indiv max
       ALLOCATE (ngmgp_t(nb_max_indiv), stat = alloc_stat)
       CALL check_allocate(alloc_stat)

       ALLOCATE (nrgm_t(nb_max_indiv), stat = alloc_stat)
       CALL check_allocate(alloc_stat)

       ALLOCATE (gmere_t(nb_max_indiv), stat = alloc_stat)
       CALL check_allocate(alloc_stat)

       ALLOCATE (gpere_t(nb_max_indiv), stat = alloc_stat)
       CALL check_allocate(alloc_stat)

       ALLOCATE (repro_t(nb_max_indiv), stat = alloc_stat)
       CALL check_allocate(alloc_stat)

       nr = 0 ; ngp = 0 ; ngm = 0
       ngmgp_t(1)=0 ; ngmgp_t(2)=0 ; nrgm_t(1)=0 ; nrgm_t(2)=0
        !********------- GEN=1 ------*********
       !lire old..
       !si gen==1
       ind=""

       if (genea_niv(1)>0) then
          ind = trim(genea_list(1,1))
          father = trim(genea_list(1,2))
          mother = trim(genea_list(1,3))
          nr = 1
          gpere_t(nr) = trim(genea_list(1,2))
          gmere_t(nr) = trim(genea_list(1,3))
          repro_t(nr) = trim(genea_list(1,1))
          ngp = 1 ; ngm = 1
          nrgm_t(ngm+1)=nrgm_t(ngm+1)+1
          ngmgp_t(ngp+1)=ngmgp_t(ngp+1)+1
       end if

       do l=2,genea_niv(1)
         ind = trim(genea_list(l,1))
         father = trim(genea_list(l,2))
         mother = trim(genea_list(l,3))
         nr=nr+1
         repro_t(nr) = ind

      !  print *,'last gp:',trim(gpere_t(ngp)),' current:',trim(father)
      !   print *,'last gm:',trim(gmere_t(ngm)),' current:',trim(mother)
        !New grandfather and grandmother
        IF ( gpere_t(ngp) /= father) THEN
          !print *,'nvx gp:',trim(father)
          ngm=ngm+1
          nrgm_t(ngm+1)=nrgm_t(ngm)+1
          gmere_t(ngm)=mother
          ngp=ngp+1
          gpere_t(ngp)=father
         ! print *,'ngmgp_t[',ngp,']:',ngmgp_t(ngp)
          ngmgp_t(ngp+1)=ngmgp_t(ngp)+1

        !New grandmother
        ELSE IF ( gmere_t(ngm) /= mother) THEN
           !print *,'nvx gm:',trim(mother)
          ngm=ngm+1
          nrgm_t(ngm+1)=nrgm_t(ngm)+1
          gmere_t(ngm)=mother
          ngmgp_t(ngp+1)=ngmgp_t(ngp+1)+1
        ELSE
           nrgm_t(ngm+1)=nrgm_t(ngm+1)+1
        ENDIF
       END DO

       !ALLOCATES TABLE AND DESALLOCATE BUFFER TAB
       !-------------------------------------------
       ALLOCATE (ngmgp(ngp+1), stat = alloc_stat)
       CALL check_allocate(alloc_stat)
       DO i=1,ngp+1
         ngmgp(i) = ngmgp_t(i)
       END DO
       DEALLOCATE(ngmgp_t)

       ALLOCATE (nrgm(ngm+1), stat = alloc_stat)
       CALL check_allocate(alloc_stat)
       DO i=1,ngm+1
         nrgm(i) = nrgm_t(i)
       END DO
       DEALLOCATE(nrgm_t)

       ALLOCATE (gmere(ngm), stat = alloc_stat)
       CALL check_allocate(alloc_stat)
       DO i=1,ngm
         gmere(i) = gmere_t(i)
       END DO
       DEALLOCATE(gmere_t)

       ALLOCATE (gpere(ngp), stat = alloc_stat)
       CALL check_allocate(alloc_stat)
       DO i=1,ngp
         gpere(i) = gpere_t(i)
       END DO
       DEALLOCATE(gpere_t)

       ALLOCATE (repro(nr), stat = alloc_stat)
       CALL check_allocate(alloc_stat)
        DO i=1,nr
         repro(i) = repro_t(i)
       END DO
       DEALLOCATE(repro_t)
       call log_mess('END SUBROUTINE : CREATE_STRUCT_GRAND_PARENT',DEBUG_DEF)
       END SUBROUTINE CREATE_STRUCT_GRAND_PARENT
!!***

!!****f* m_qtlmap_genealogy/CREATE_STRUCT_PARENT
!!  NAME
!!    CREATE_STRUCT_PARENT
!!  DESCRIPTION
!!    Fill ndm,nmp,pere,mere,animal,nd,nm,np arrays from the information readed (genea_list,genea_niv).
!!  INPUTS
!!   nb_max_indiv   : get the number of animal defined in the genalogy file
!!
!!  NOTES
!!  SOURCE
       SUBROUTINE CREATE_STRUCT_PARENT(nb_max_indiv)
        integer,intent(in)              ::nb_max_indiv


        character(len=LEN_DEF)            ::ind
        character(len=LEN_DEF)            ::father
        character(len=LEN_DEF)            ::mother
        integer                         ::gen,alloc_stat,err,eof,l,i,start
        character(len=LEN_DEF)            ::word_token
        character(len=LEN_LINE)            ::line_read
        integer , dimension (:),allocatable :: ndm_t
        integer , dimension (:),allocatable :: nmp_t

        character(len=LEN_DEF) , dimension (:),allocatable ::mere_t
        character(len=LEN_DEF) , dimension (:),allocatable ::pere_t
        character(len=LEN_DEF) , dimension (:),allocatable ::animal_t
        logical                                          ::is_ok

        call log_mess('SUBROUTINE : CREATE_STRUCT_PARENT',DEBUG_DEF)

        if ( genea_niv(2) <= 0 ) then
          call stop_application("none animals with generation 2 is detected");
        end if

         !***
        ! Initialize Buffer with the indiv max
        ALLOCATE (ndm_t(nb_max_indiv+1), stat = alloc_stat)
        CALL check_allocate(alloc_stat)

        ALLOCATE (nmp_t(nb_max_indiv+1), stat = alloc_stat)
        CALL check_allocate(alloc_stat)

        ALLOCATE (mere_t(nb_max_indiv), stat = alloc_stat)
        CALL check_allocate(alloc_stat)

        ALLOCATE (pere_t(nb_max_indiv), stat = alloc_stat)
        CALL check_allocate(alloc_stat)

        ALLOCATE (animal_t(nb_max_indiv), stat = alloc_stat)
        CALL check_allocate(alloc_stat)

        nd=1
        nm=1
        np=1

        ndm_t(1)=0
        ndm_t(2)=1

        nmp_t(1)=0
        nmp_t(2)=1

        start = sum(genea_niv(:1))
        ind = trim(genea_list(start+1,1))
        father = trim(genea_list(start+1,2))
        mother = trim(genea_list(start+1,3))

        animal_t(nd) = ind
        pere_t(np) = father
        mere_t(nm) = mother

        DO l=start+2,start+genea_niv(2)
            ind = trim(genea_list(l,1))
            father = trim(genea_list(l,2))
            mother = trim(genea_list(l,3))
            nd = nd+1
            animal_t(nd) = ind

         IF ( pere_t(np) /= father ) THEN
             nm = nm + 1
             mere_t(nm)= mother
             ndm_t(nm+1)=ndm_t(nm)+1
             np=np+1
             pere_t(np)=father
             nmp_t(np+1)=nmp_t(np)+1
         ELSE IF ( mere_t(nm) /= mother ) THEN
             nm = nm + 1
             mere_t(nm)= mother
             ndm_t(nm+1)= ndm_t(nm)+1
             nmp_t(np+1)= nmp_t(np+1)+1
         ELSE
             ndm_t(nm+1)=ndm_t(nm+1)+1
         END IF

        END DO

       !ALLOCATES TABLE AND DESALLOCATE BUFFER TAB
       !-------------------------------------------
       ALLOCATE (ndm(nm+1), stat = alloc_stat)
       CALL check_allocate(alloc_stat)
       DO i=1,nm+1
         ndm(i) = ndm_t(i)
       END DO
       DEALLOCATE(ndm_t)

       ALLOCATE (nmp(np+1), stat = alloc_stat)
       CALL check_allocate(alloc_stat)
       DO i=1,np+1
         nmp(i) = nmp_t(i)
       END DO
       DEALLOCATE(nmp_t)

       ALLOCATE (mere(nm), stat = alloc_stat)
       CALL check_allocate(alloc_stat)
       DO i=1,nm
         mere(i) = mere_t(i)
       END DO
       DEALLOCATE(mere_t)

       ALLOCATE (pere(np), stat = alloc_stat)
       CALL check_allocate(alloc_stat)
       DO i=1,np
         pere(i) = pere_t(i)
       END DO
       DEALLOCATE(pere_t)
       ALLOCATE (animal(nd), stat = alloc_stat)
       CALL check_allocate(alloc_stat)
        DO i=1,nd
         animal(i) = animal_t(i)
       END DO
       DEALLOCATE(animal_t)


       call log_mess('END SUBROUTINE : CREATE_STRUCT_PARENT',DEBUG_DEF)
       END SUBROUTINE CREATE_STRUCT_PARENT

       !**********************************************
       ! SUBROUTINE : CREATE_STRUCT_DERIVED_GENEALOGY
       !**********************************************
      SUBROUTINE CREATE_STRUCT_DERIVED_GENEALOGY ()

          integer                 :: alloc_stat
          integer                 :: ip,im,ir,ifem,i
          call log_mess('SUBROUTINE CREATE_STRUCT_DERIVED_GENEALOGY',DEBUG_DEF)

          do  ip=1,size(pere)
            reppere(ip)=INT_NOT_DEFINED
           do ir=1,size(repro)
              if ( pere(ip) == repro(ir) ) then
                 reppere(ip)= ir
                 exit
              endif
            end do ! irlabel
          end do ! iplabel

          nfem=1
          femelle(nfem)=mere(nfem)
          do  im=1,size(mere)
              repmere(im)=INT_NOT_DEFINED
           do ir=1,size(repro)
              if ( mere(im) == repro(ir) ) then
                 repmere(im)= ir
                 exit
              endif
            end do ! irlabel
            do ifem=1,nfem
              if(mere(im).eq.femelle(ifem)) then
                 repfem(im)=ifem
                 exit
              end if
            end do
            if (ifem > nfem) then
               nfem=nfem+1
               femelle(nfem)=mere(im)
               repfem(im)=nfem
            end if
          end do ! iplabel

          call log_mess('END SUBROUTINE CREATE_STRUCT_DERIVED_GENEALOGY',DEBUG_DEF)
      END SUBROUTINE CREATE_STRUCT_DERIVED_GENEALOGY
!!***

!!****f* m_qtlmap_genealogy/sim_genea
!!  NAME
!!    sim_genea
!!  DESCRIPTION
!!    Simulates a population according :
!!      * number of dam by sire
!!      * number of progenies by dam
!!      * croisement type
!!  INPUTS
!!   inmp         : number of dam by sire
!!   indm         : number of progenies by dam
!!   croisement   : OUTBRED_KEYWORD, F2_KEYWORD, BC_KEYWORD
!!
!!  NOTES
!!  SOURCE
     subroutine sim_genea(inmp,indm,croisement)
      integer          ,intent(in)             :: inmp,indm
      character(len=LEN_BUFFER_WORD),intent(in):: croisement

      if ( croisement == OUTBRED_KEYWORD ) then
          !ne necessite pas de regneration si une genealogie a deja ete creer
          if ( .not. genealogy_outbred_gen ) then
            call sim_genea_outbread(inmp,indm)
          end if
      else
          call sim_genea_F2_BC(inmp,indm)
      end if

     end subroutine sim_genea
!!***

!!****f* m_qtlmap_genealogy/sim_genea_outbread
!!  NAME
!!    sim_genea_outbread
!!  DESCRIPTION
!!
!!  INPUTS
!!   inmp         : number of dam by sire
!!   indm         : number of progenies by dam
!!
!!  NOTES
!!  SOURCE
     subroutine sim_genea_outbread(inmp,indm)
        integer          ,intent(in)         :: inmp,indm

        integer   :: ind,i,j,kd

        ! creation of sires and parent o them
        ngmgp(1)=0
        nrgm(1)=0
        ind = 1
        do i=1,np
           pere(i)=str(ind)
           repro(i)=pere(i)
           ind = ind + 1
           gpere(i)=str(ind)
           ind = ind +1
           gmere(i)=str(ind)
           ind = ind + 1
           nrgm(i+1)=nrgm(i)+1
           ngmgp(i+1)=ngmgp(i)+1
        end do
        ! dams
        do i=1,nm
           mere(i)=str(ind)
           repro(np+i)=mere(i)
           ind = ind + 1
           gpere(np+i)=str(ind)
           ind = ind +1
           gmere(np+i)=str(ind)
           ind = ind + 1
           nrgm(np+i+1)=nrgm(np+i)+1
           ngmgp(np+i+1)=ngmgp(np+i)+1
        end do
        !progeny
        nmp(1)=0
        ndm(1)=0
        kd=1

        do i=1,np
          nmp(i+1)=nmp(i)+inmp
          do j=nmp(i)+1,nmp(i+1)
            ndm(j+1)=ndm(j)+indm
            do kd=ndm(j)+1,ndm(j+1)
              animal(kd) = str(ind)
              ind = ind+1
            end do
          end do
        end do

       call CREATE_STRUCT_DERIVED_GENEALOGY

       genealogy_outbred_gen = .true.
      ! call log_debug_genea()
      ! stop

     end subroutine sim_genea_outbread
!!***

!!****f* m_qtlmap_genealogy/sim_genea_F2_BC
!!  NAME
!!    sim_genea_F2_BC
!!  DESCRIPTION
!!
!!  INPUTS
!!   inmp         : number of dam by sire
!!   indm         : number of progenies by dam
!!
!!  NOTES
!!    Sous programme de simulation de la genealogie de la population: familles de tailles equilibrees np peres, nmp meres par pere et ndm descendants par mere
!!
!!  SOURCE
      subroutine sim_genea_F2_BC(inmp,indm)
      integer          ,intent(in)         :: inmp,indm
!Divers
      integer ind,igp,jgm,ir,ip,jm,kr,ijm,nm1,nm2,kd,i,j
      integer , dimension(:,:),allocatable :: ncr
      integer , dimension(:),allocatable   :: meres
      real                                 :: xcr
      real,external                        :: ranf
!
!******************************************************************************
!******************************************************************************
!    Dispositif F2 �quilibre
!         ngp=ngm=np, 1 male et inmp femelles par famille
!                      nm*indm descendants
!******************************************************************************
!******************************************************************************
!
!
!**************************************************************************
!        Construction des numeros d'animaux : ind
!
!  GRAND-PARENTS
!
!    - 1 � ngp => gd peres
!    - ngp+1 a ngp+ngm => gd meres
!**************************************************************************
!
      ind=1
! Initialisation des numeros des gd peres
!
      gpere(1)=str(ind)
      ngmgp(1)=0
      do igp=2,ngp
        ind=ind+1
        gpere(igp)=str(ind)
!       sexe(ind)=1
!       gener(ind)=0
        ngmgp(igp)=ngmgp(igp-1)+1             !! 1 par defaut
      end do
      ngmgp(ngp+1)=ngmgp(ngp)+1
!
! Initialisation des numeros des gd peres
!
      nrgm(1)=0
      do jgm=1,ngm
        ind=ind+1
        gmere(jgm)=str(ind)
!       sexe(ind)=2
!       gener(ind)=0
        if(jgm.gt.1)nrgm(jgm)=nrgm(jgm-1)+1+inmp    !! chaq couple F0 => 1pere+nmp meres
      end do
      nrgm(ngm+1)=nrgm(ngm)+1+inmp
!
!**************************************************************************
!  REPRODUCTEURS
!
!    -  ngp+ngm +1 a ngp+ngm+nr => repro
!**************************************************************************
! Initialisation
!
      allocate(meres(size(mere)))
      ir=0
      ip=0
      jm=0
      nmp(1)=0
!
      do igp=1,ngp
        jgm=igp                      !! 1 couple F0,
        ir=ir+1
        ip=ip+1
        ind=ind+1
!
! Creation des males F1 et tables de correspondaces peres
        repro(ir)=str(ind)
        pere(ip)=repro(ir)
!        reppere(ip)=ir
!        gener(ind)=1
!        sexe(ind)=1
        if(ip.gt.1) nmp(ip)=nmp(ip-1)+inmp
      ! write (1,1000) trim(repro(ir)),trim(gpere(igp)),trim(gmere(jgm)),' 1'
!
! Creation des femelles F1
        do kr=1,inmp
          ind=ind+1
          jm=jm+1
          ir=ir+1
          repro(ir)=str(ind)
          meres(jm)=ind
!          gener(ind)=1
!          sexe(ind)=2
         ! write (1,1000) trim(repro(ir)),trim(gpere(igp)),trim(gmere(jgm)),' 1'
        end do
      end do
!
! Affectations des croisements F1 (BOURRIN)
      allocate (ncr(np,inmp))
      do ijm=1,inmp
       ip=1
!XXX       xcr=g05caf(xcr)
       xcr=ranf()
       xcr=xcr*(np+1)+np*(ijm-1)
       ncr(ip,ijm)=xcr
       ! OFI: modif, sinon ncr(ip,ijm) peut valoir 0,vu que c est un index de tableaux.....
       if (ncr(ip,ijm)==0) ncr(ip,ijm) = 1
       do ip=2,np
        if(ncr(ip-1,ijm).eq.np*ijm)then
          ncr(ip,ijm)=1+np*(ijm-1)
        else
          ncr(ip,ijm)=ncr(ip-1,ijm)+1
        end if
       end do
      end do

!
! Tables de correspondances meres
      nmp(1)=0
      do ip=1,np
       if(ip.gt.1)nmp(ip)=nmp(ip-1)+inmp
       do ijm=1,inmp
         jm=nmp(ip)+ijm
         mere(jm)=str(meres(ncr(ip,ijm)))
         repfem(jm)=jm
!         do ir=1,nr
!           if(mere(jm).eq.repro(ir)) repmere(jm)=ir
!         end do
       end do
      end do



      nfem=nm
      nmp(np+1)=nmp(np)+inmp
!
! Descendants: genealogie et numeros
       ndm(1)=0
       do ip=1,np
         nm1=nmp(ip)+1
         nm2=nmp(ip+1)
         do jm=nm1,nm2
           if(jm.gt.1)ndm(jm)=ndm(jm-1)+indm
           do kd=ndm(jm)+1,ndm(jm)+indm
            ind=ind+1
            animal(kd)=str(ind)
!           gener(ind)=2
!           sexe(ind)=2
!            xcr=g05caf(xcr)
!           if(xcr.gt.0.5)sexe(ind)=1
          !  write (1,1000) trim(animal(kd)),trim(pere(ip)),trim(mere(jm)),' 2'
           end do
         end do
       end do
       ndm(nm+1)=ndm(nm)+indm
 !1000  format(1x,a,1x,a,1x,a,1x,a3)
    !   close(1)

       deallocate (ncr)
       deallocate(meres)
       call CREATE_STRUCT_DERIVED_GENEALOGY

      ! call log_debug_genea()
      ! stop

       end subroutine sim_genea_F2_BC
!!***

!!****f* m_qtlmap_genealogy/write_genea
!!  NAME
!!    write_genea
!!  DESCRIPTION
!!
!!  INPUTS
!!   file_name         : path name of the output file
!!
!!  NOTES
!!
!!  SOURCE
     subroutine write_genea(file_name)
        character(len=*),intent(in)     :: file_name
        integer :: ip,jm,kr,jgm,igp,id

        open(1,file=file_name)

        call log_mess('TODO:write generation file for genealogy...')
        do igp=1,ngp
          do jgm=ngmgp(igp)+1,ngmgp(igp+1)
            do kr=nrgm(jgm)+1,nrgm(jgm+1)
              write (1,*) trim(repro(kr)),' ',trim(gpere(igp)),' ',trim(gmere(jgm)),' 1'
            end do
          end do
         end do
         do ip=1,np
        do jm=nmp(ip)+1,nmp(ip+1)
          do id=ndm(jm)+1,ndm(jm+1)
             write (1,*) trim(animal(id)),' ',trim(pere(ip)),' ',trim(mere(jm)),' 2'
          end do
        end do
       end do
       close(1)

     end subroutine write_genea
!!***

!!****f* m_qtlmap_genealogy/log_debug_genea
!!  NAME
!!    log_debug_genea
!!  DESCRIPTION
!!
!!  NOTES
!!
!!  SOURCE
     subroutine log_debug_genea()
       integer  :: i,j,k

       do i=1,ngp
         print *,'------------------------------------------------'
         print *,'index gp:',i,' ngmgp(',i,')=',ngmgp(i),' ngmgp(',(i+1),')=',ngmgp(i+1)
         do j=ngmgp(i)+1,ngmgp(i+1)

            do k=nrgm(j)+1,nrgm(j+1)
              print *,k
              print *,trim(repro(k)),' ',trim(gpere(i)),' ',trim(gmere(j)),' 1'
            end do
         end do
       end do
     end subroutine log_debug_genea
!!***

END MODULE m_qtlmap_genealogy




