!!****m* INPUT/m_qtlmap_parameter_file
!!  NAME
!!    m_qtlmap_parameter_file -- This modules manages the parameter user file (p_analyse).
!!  SYNOPSIS

!!  DESCRIPTION
!!
!!  NOTES
!!
!!  BUGS
!!
!!  SEE ALSO
!!
!!  COPYRIGHT
!!***
!! You can use this space for remarks that should not be included
!! in the documentation.
!!/
module m_qtlmap_parameter_file
      use m_qtlmap_constant
      use m_qtlmap_data
      use m_qtlmap_haplotype_data
      use m_qtlmap_tools
      use m_qtlmap_log

      implicit none
      save


      logical                     , public        :: simul    ! the context is a simulation or not
      character(len=LENGTH_MAX_FILE), public      :: analyse_file    ! analyse file

      !OPTIONAL KEYS
      !-------------


!!****d* m_qtlmap_parameter_file/K_MAP
!!  NAME
!!   K_MAP
!!  DESCRIPTION
!!   map file
!!  SOURCE
      character(len=LEN_L)   , parameter , private :: K_MAP                ='in_map'
!!***

!!****d* m_qtlmap_parameter_file/K_GENEA
!!  NAME
!!   K_GENEA
!!  DESCRIPTION
!!   genealogy file
!! SOURCE
      character(len=LEN_L)   , parameter , private :: K_GENEA              ='in_genealogy'
!!***

!!****d* m_qtlmap_parameter_file/K_TRAITS
!!  NAME
!!   K_TRAITS
!!  DESCRIPTION
!!   phenotypics user file
!! SOURCE
      character(len=LEN_L)   , parameter , private :: K_TRAITS             ='in_traits'
!!***

!!****d* m_qtlmap_parameter_file/K_GENOTYPE
!!  NAME
!!   K_GENOTYPE
!!  DESCRIPTION
!!   genotypics user file
!! SOURCE
      character(len=LEN_L)   , parameter , private :: K_GENOTYPE           ='in_genotype'
!!***

!!****d* m_qtlmap_parameter_file/K_MODEL
!!  NAME
!!   K_MODEL
!!  DESCRIPTION
!!   model phenotypic user file
!! SOURCE
      character(len=LEN_L)   , parameter , private :: K_MODEL              ='in_model'
!!***

!!****d* m_qtlmap_parameter_file/K_PARAMSIM
!!  NAME
!!   K_PARAMSIM
!!  DESCRIPTION
!!   parameter simulation user file
!! SOURCE
      character(len=LEN_L)   , parameter , private :: K_PARAMSIM           ='in_paramsimul'
!!***

!!****d* m_qtlmap_parameter_file/K_OUTSIM
!!  NAME
!!   K_OUTSIM
!!  DESCRIPTION
!!   output result file
!! SOURCE
      character(len=LEN_L)   , parameter , private :: K_OUTSIM             ='out_maxlrt'
!!***

!!****d* m_qtlmap_parameter_file/K_PDED
!!  NAME
!!   K_PDED
!!  DESCRIPTION
!!   output probabilities of transmission file
!! SOURCE
      character(len=LEN_L)   , parameter , private :: K_PDED               ='out_pded'
!!***

!!****d* m_qtlmap_parameter_file/K_PDECPLE
!!  NAME
!!   K_PDECPLE
!!  DESCRIPTION
!!   output probabilities of join transmission file
!! SOURCE
      character(len=LEN_L)   , parameter , private :: K_PDECPLE            ='out_pdedjoin'
!!***

!!****d* m_qtlmap_parameter_file/K_PATEFF
!!  NAME
!!   K_PATEFF
!!  DESCRIPTION
!!   output paternal effect file
!! SOURCE
      character(len=LEN_L)   , parameter , private :: K_PATEFF             ='out_pateff'
!!***

!!****d* m_qtlmap_parameter_file/K_MATEFF
!!  NAME
!!   K_MATEFF
!!  DESCRIPTION
!!   output maternal effect file
!! SOURCE
      character(len=LEN_L)   , parameter , private :: K_MATEFF             ='out_mateff'
!!***

!!****d* m_qtlmap_parameter_file/K_LRTSIRE
!!  NAME
!!   K_LRTSIRE
!!  DESCRIPTION
!!   LRT curve output file (General LRT and by half-sib family)
!! SOURCE
      character(len=LEN_L)   , parameter , private :: K_LRTSIRE            ='out_lrtsires'
!!***

!!****d* m_qtlmap_parameter_file/K_LRTDAM
!!  NAME
!!   K_LRTDAM
!!  DESCRIPTION
!!    LRT curve output file
!! SOURCE
      character(len=LEN_L)   , parameter , private :: K_LRTDAM             ='out_lrtdams'
!!***


!!****d* m_qtlmap_parameter_file/K_FREQALL
!!  NAME
!!   K_FREQALL
!!  DESCRIPTION
!!   Allele frequency information
!! SOURCE
      character(len=LEN_L)   , parameter , private :: K_FREQALL            ='out_freqall'
!!***


!!****d* m_qtlmap_parameter_file/K_COEFFDA
!!  NAME
!!   K_COEFFDA
!!  DESCRIPTION
!!   output coefficent discriminante analysis result file
!! SOURCE
      character(len=LEN_L)   , parameter , private :: K_COEFFDA            ='out_coeffda'
!!***

!!****d* m_qtlmap_parameter_file/K_GRID2QTL
!!  NAME
!!   K_GRID2QTL
!!  DESCRIPTION
!!   LRT curves describe in a grid (hypothesis : 2 qtl)
!! SOURCE
      character(len=LEN_L)   , parameter , private :: K_GRID2QTL           ='out_grid2qtl'
!!***

!!****d* m_qtlmap_parameter_file/K_PHASES
!!  NAME
!!   K_PHASES
!!  DESCRIPTION
!!   phases probabilities of reproductor file
!! SOURCE
      character(len=LEN_L)   , parameter , private :: K_PHASES             ='out_phases'
!!***

!!****d* m_qtlmap_parameter_file/K_HAPLOTYPES
!!  NAME
!!    K_HAPLOTYPES
!!  DESCRIPTION
!!    haplotypes files
!! SOURCE
      character(len=LEN_L)   , parameter , private :: K_HAPLOTYPES         ='out_haplotypes'
!!***

      !AUTOMATIC KEY TO DEFINED
      !------------------------


!!****d* m_qtlmap_parameter_file/K_OUTPUT
!!  NAME
!!   K_OUTPUT
!!  DESCRIPTION
!!   main output result file
!! SOURCE
      character(len=LEN_L)   , parameter , private :: K_OUTPUT             ='out_output'
!!***

!!****d* m_qtlmap_parameter_file/K_SUMM
!!  NAME
!!   K_SUMM
!!  DESCRIPTION
!!   summary result file
!! SOURCE
      character(len=LEN_L)   , parameter , private :: K_SUMM               ='out_summary'
!!***

!!****d* m_qtlmap_parameter_file/K_CHROM
!!  NAME
!!   K_CHROM
!!  DESCRIPTION
!!   chromosom list to explore
!! SOURCE
      character(len=LEN_L)   , parameter , private :: K_CHROM              ='opt_chromosome'
!!***

!!****d* m_qtlmap_parameter_file/K_NDMIN
!!  NAME
!!   K_NDMIN
!!  DESCRIPTION
!!   Minimal number of progeny by dam to considere full sib family in the analysis
!! SOURCE
      character(len=LEN_L)   , parameter , private :: K_NDMIN              ='opt_ndmin'
!!***

!!****d* m_qtlmap_parameter_file/K_STEP
!!  NAME
!!   K_STEP
!!  DESCRIPTION
!!   step definition for LA analysis (in Morgan)
!! SOURCE
      character(len=LEN_L)   , parameter , private :: K_STEP               ='opt_step'
!!***

!!****d* m_qtlmap_parameter_file/K_MINDAMPHASEPROB
!!  NAME
!!   K_MINDAMPHASEPROB
!!  DESCRIPTION
!!   Minimal dam phase probability
!! SOURCE
      character(len=LEN_L)   , parameter , private :: K_MINDAMPHASEPROB    ='opt_mindamphaseproba'
!!***

!!****d* m_qtlmap_parameter_file/K_MINSIREPHASEPROB
!!  NAME
!!   K_MINSIREPHASEPROB
!!  DESCRIPTION
!!   Minimal sire phase probability
!! SOURCE
      character(len=LEN_L)   , parameter , private :: K_MINSIREPHASEPROB   ='opt_minsirephaseproba'
!!***

!!****d* m_qtlmap_parameter_file/K_UNKNOWN_GENO
!!  NAME
!!   K_UNKNOWN_GENO
!!  DESCRIPTION
!!   Unknown genotype value
!! SOURCE
      character(len=LEN_L)   , parameter , private :: K_UNKNOWN_GENO       ='opt_unknown_char'
!!***

!!****d* m_qtlmap_parameter_file/K_CHOLESKY
!!  NAME
!!   K_CHOLESKY
!!  DESCRIPTION
!!   coefficent used in the cholesky decomposition to establish the parameters estimabilities
!! SOURCE
      character(len=LEN_L)   , parameter , private :: K_CHOLESKY           ='opt_eps_cholesky'
!!***

!!****d* m_qtlmap_parameter_file/K_THRES_CONFUSION
!!  NAME
!!   K_THRES_CONFUSION
!!  DESCRIPTION
!!   Threshold to test confusion betwwen level inside a contingence matrix
!! SOURCE
      character(len=LEN_L)   , parameter , private :: K_THRES_CONFUSION    ='opt_eps_confusion'
!!***

!!****d* m_qtlmap_parameter_file/K_PSEUILHWE
!!  NAME
!!   K_PSEUILHWE
!!  DESCRIPTION
!!   Threshold to check the equilibrium of marker transmission within each family
!! SOURCE
      character(len=LEN_L)   , parameter , private :: K_PSEUILHWE          ='opt_eps_hwe'
!!***

!!****d* m_qtlmap_parameter_file/K_LINEAR_CONV
!!  NAME
!!   K_LINEAR_CONV
!!  DESCRIPTION
!!   Threshold for convergence in the linear mode heteroscedastic
!! SOURCE
      character(len=LEN_L)   , parameter , private :: K_LINEAR_CONV        ='opt_eps_linear_heteroscedastic'
!!***

!!****d* m_qtlmap_parameter_file/K_MAX_LINEAR_ITERATION
!!  NAME
!!   K_MAX_LINEAR_ITERATION
!!  DESCRIPTION
!!   number maximum of iteration (heteroscedastic linear analysis)
!! SOURCE
      character(len=LEN_L)   , parameter , private :: K_MAX_LINEAR_ITERATION = 'opt_max_iteration_linear_heteroscedastic'
!!***

!!****d* m_qtlmap_parameter_file/K_PROB_SEUIL_RECOMB
!!  NAME
!!   K_PROB_SEUIL_RECOMB
!!  DESCRIPTION
!!   Minimal paternal phase probability
!! SOURCE
      character(len=LEN_L)   , parameter , private :: K_PROB_SEUIL_RECOMB  ='opt_eps_recomb'
!!***

!!****d* m_qtlmap_parameter_file/K_LONGHAP
!!  NAME
!!   K_LONGHAP
!!  DESCRIPTION
!!   Length of the haplotypes to be followed in LD analyses
!! SOURCE
      character(len=LEN_L)   , parameter , private :: K_LONGHAP            ='opt_longhap'
!!***

!!****d* m_qtlmap_parameter_file/K_NB_HAPLO_PRIOR
!!  NAME
!!   K_NB_HAPLO_PRIOR
!!  DESCRIPTION
!!   Number maximum of haplotype allowed
!! SOURCE
      character(len=LEN_L)   , parameter , private :: K_NB_HAPLO_PRIOR     ='opt_nb_haplo_prior'
!!***

!!****d* m_qtlmap_parameter_file/K_PROB_HAPLO_MIN
!!  NAME
!!   K_PROB_HAPLO_MIN
!!  DESCRIPTION
!!   miminum probability of a haplotype in the LD analysis
!! SOURCE
      character(len=LEN_L)   , parameter , private :: K_PROB_HAPLO_MIN     ='opt_prob_haplo_min'
!!***

!!****d* m_qtlmap_parameter_file/K_LONG_MIN_IBS
!!  NAME
!!   K_LONG_MIN_IBS
!!  DESCRIPTION
!!
!! SOURCE
      character(len=LEN_L)   , parameter , private :: K_LONG_MIN_IBS       ='opt_long_min_ibs'
!!***

!!****d* m_qtlmap_parameter_file/K_OPTIM_MAXEVAL
!!  NAME
!!   K_OPTIM_MAXEVAL
!!  DESCRIPTION
!!  number mmaximum of objective function evaluation
!! SOURCE
      character(len=LEN_L)   , parameter , private :: K_OPTIM_MAXEVAL      ='opt_optim_maxeval'
!!***

!!****d* m_qtlmap_parameter_file/K_OPTIM_MAXTIME
!!  NAME
!!   K_OPTIM_MAXTIME
!!  DESCRIPTION
!!   maximum elapsed time in a optimisation of a objective function
!!  SOURCE
      character(len=LEN_L)   , parameter , private :: K_OPTIM_MAXTIME      ='opt_optim_maxtime'
!!***

!!****d* m_qtlmap_parameter_file/K_OPTIM_TOLX
!!  NAME
!!   K_OPTIM_TOLX
!!  DESCRIPTION
!!   tolerance precison of the paramter X (m_qtlmap_optimization)
!!  SOURCE
      character(len=LEN_L)   , parameter , private :: K_OPTIM_TOLX         ='opt_optim_tolx'
!!***

!!****d* m_qtlmap_parameter_file/K_OPTIM_TOLF
!!  NAME
!!   K_OPTIM_TOLF
!!  DESCRIPTION
!!   tolerance precison of the objective function (m_qtlmap_optimization)
!!  SOURCE
      character(len=LEN_L)   , parameter , private :: K_OPTIM_TOLF         ='opt_optim_tolf'
!!***

!!****d* m_qtlmap_parameter_file/K_OPTIM_TOLG
!!  NAME
!!   K_OPTIM_TOLG
!!  DESCRIPTION
!!   tolerance precison of gradient (m_qtlmap_optimization)
!!  SOURCE
      character(len=LEN_L)   , parameter , private :: K_OPTIM_TOLG         ='opt_optim_tolg'
!!***

!!****d* m_qtlmap_parameter_file/K_OPTIM_H_PREC
!!  NAME
!!   K_OPTIM_H_PREC
!!  DESCRIPTION
!!   precision of numerical computation of the derivate F'(x) = (F(x-h)+F(x+h))/2*h (m_qtlmap_optimization)
!!  SOURCE
      character(len=LEN_L)   , parameter , private :: K_OPTIM_H_PREC       ='opt_optim_h_precision'
!!***



!!****d* m_qtlmap_parameter_file/NUMBER_OPT_KEYS
!!  NAME
!!   NUMBER_OPT_KEYS
!!  DESCRIPTION
!!   optional keys
!! SOURCE
      integer    , parameter, private              :: NUMBER_OPT_KEYS=18
!!***

      ! NUMBER OF AUTOMATIC KEYS DEFINED.......
!!****d* m_qtlmap_parameter_file/NUMBER_AUTO_KEYS
!!  NAME
!!   NUMBER_AUTO_KEYS
!!  DESCRIPTION
!!   number of essential keys to defined by the user in the paramter file
!!  SOURCE
      integer    , parameter, private              :: NUMBER_AUTO_KEYS=24
!!***
      ! ALL KEYS
      ! --------
!!****d* m_qtlmap_parameter_file/NUMBER_ALL_KEYS
!!  NAME
!!   NUMBER_ALL_KEYS
!!  DESCRIPTION
!!   Number of qtlmap keys
!!  SOURCE
      integer    , parameter, public              :: NUMBER_ALL_KEYS=NUMBER_AUTO_KEYS+NUMBER_OPT_KEYS
!!***
      !DEFAULT VALUE
      !-------------
!!****v* m_qtlmap_parameter_file/index_key
!!  NAME
!!   index_key
!!  DESCRIPTION
!!   Index keys with a default value
!!  NOTES
!!    see the array default_values for correspondence
!!  SOURCE
      character(len=LEN_L)  , dimension(NUMBER_AUTO_KEYS) , private :: index_key     = (/ &
                                                                         K_OUTPUT,  &
                                                                         K_SUMM,    &
                                                                         K_CHROM,   &
                                                                         K_NDMIN,   &
                                                                         K_STEP,    &
                                                                         K_UNKNOWN_GENO,    &
                                                                         K_MINDAMPHASEPROB,    &
                                                                         K_MINSIREPHASEPROB,  &
                                                                         K_CHOLESKY,          &
                                                                         K_THRES_CONFUSION,   &
                                                                         K_PSEUILHWE,         &
                                                                         K_LINEAR_CONV,       &
                                                                         K_MAX_LINEAR_ITERATION,&
                                                                         K_PROB_SEUIL_RECOMB, &
                                                                         K_NB_HAPLO_PRIOR,&
                                                                         K_PROB_HAPLO_MIN,&
                                                                         K_LONG_MIN_IBS,&
                                                                         K_LONGHAP,&
                                                                         K_OPTIM_MAXEVAL,&
                                                                         K_OPTIM_MAXTIME,&
                                                                         K_OPTIM_TOLX,&
                                                                         K_OPTIM_TOLF,&
                                                                         K_OPTIM_TOLG,&
                                                                         K_OPTIM_H_PREC/)
!!***

!!****v* m_qtlmap_parameter_file/default_values
!!  NAME
!!   default_values
!!  DESCRIPTION
!!   Default values for key predefined by qtlmap.
!!  NOTES
!!   see the array index_key for correspondence
!!  SOURCE
      character(len=LEN_L)  , dimension(NUMBER_AUTO_KEYS) , private :: default_values = (/ &
                                                                         "       ",  &
                                                                         "       ",  &
                                                                         "       ",  &
                                                                         "       ",  &
                                                                         "0.05   ",  &
                                                                         "0      ",  &
                                                                         "0.10   ",  &
                                                                         "0.90   ",  &
                                                                         "0.01   ",  &
                                                                         "0.70   ",  &
                                                                         "0.01   ",  &
                                                                         "0.5    ",  &
                                                                         "30     ",  &
                                                                         "0.05   ",  &
                                                                         "200    ",  &
                                                                         "0.05   ",  &
                                                                         "4      ",  &
                                                                         "4      ",  &
                                                                         "10000  ",  &
                                                                         "100    ",  &
                                                                         "0.00005",  &
                                                                         "0.00005",  &
                                                                         "0.00005",  &
                                                                         "0.00005"/)
!!***

!!****v* m_qtlmap_parameter_file/all_key
!!  NAME
!!   all_key
!!  DESCRIPTION
!!   The entirely list of possibility key used by qtlmap
!!  SOURCE
      character(len=LEN_L)  , dimension(NUMBER_ALL_KEYS) , private :: all_key = (/ &
                                                                         K_OUTPUT,  &
                                                                         K_LRTSIRE, &
                                                                         K_LRTDAM,  &
                                                                         K_PDED,    &
                                                                         K_PDECPLE, &
                                                                         K_PATEFF,  &
                                                                         K_MATEFF,  &
                                                                         K_FREQALL, &
                                                                         K_COEFFDA, &
                                                                         K_GRID2QTL,&
                                                                         K_SUMM,    &
                                                                         K_CHROM,   &
                                                                         K_NDMIN,   &
                                                                         K_STEP,    &
                                                                         K_UNKNOWN_GENO,    &
                                                                         K_MINDAMPHASEPROB,    &
                                                                         K_MINSIREPHASEPROB,  &
                                                                         K_CHOLESKY, &
                                                                         K_THRES_CONFUSION,   &
                                                                         K_PSEUILHWE,         &
                                                                         K_LINEAR_CONV,&
                                                                         K_MAP,     &
                                                                         K_GENEA,   &
                                                                         K_TRAITS,  &
                                                                         K_GENOTYPE, &
                                                                         K_MODEL,    &
                                                                         K_PARAMSIM, &
                                                                         K_OUTSIM,  &
                                                                         K_MAX_LINEAR_ITERATION, &
                                                                         K_PROB_SEUIL_RECOMB,    &
                                                                         K_PHASES, &
                                                                         K_HAPLOTYPES,&
                                                                         K_NB_HAPLO_PRIOR,&
                                                                         K_PROB_HAPLO_MIN,&
                                                                         K_LONG_MIN_IBS,&
                                                                         K_LONGHAP,&
                                                                         K_OPTIM_MAXEVAL,&
                                                                         K_OPTIM_MAXTIME,&
                                                                         K_OPTIM_TOLX,&
                                                                         K_OPTIM_TOLF,&
                                                                         K_OPTIM_TOLG,&
                                                                         K_OPTIM_H_PREC/)
!!***

!!****v* m_qtlmap_parameter_file/values
!!  NAME
!!   values
!!  DESCRIPTION
!!   The values of the p_analyses file (referenced by the array keys)
!!***
      character(len=LENGTH_MAX_FILE)   , dimension(:), private ,allocatable :: values
!!****v* m_qtlmap_parameter_file/keys
!!  NAME
!!   keys
!!  DESCRIPTION
!!   The keys defined of the p_analyses file
!!***
      character(len=LEN_L)             , dimension(:), private ,allocatable :: keys

      !********************************************
      !METHOD
      !********************************************
      public :: read_analyse_file
      public :: initialize_qtlmap_context
      public :: initialize_tocartha_context
      public :: end_analyse_parameterfile
      public :: get_key_value
      public :: key_exist
      public :: help_panalyse

      contains
!!****f* m_qtlmap_parameter_file/read_analyse_file
!!  NAME
!!    read_analyse_file
!!  DESCRIPTION
!!    read the parameter analyse user file. fill the internal buffer keys and values.
!!    keys and values are used like a stack (only the last value for the one specific key is considered)
!!  INPUTS
!!   param      : name of the parameter file
!!  NOTES
!!  SOURCE
      subroutine read_analyse_file(param)
        character(len=LENGTH_MAX_FILE),intent(inout) :: param

        integer  :: ios_id = 20
        integer  :: n ,ios,cln,i,j,chr
        character(len=LEN_W)   :: buf
        character(len=LEN_LINE)   :: buf2
        logical :: ok,find
        real(kind=dp) :: pas_temp

         analyse_file = trim(param)
         call file_exist(analyse_file)
         open(unit=ios_id,file=trim(analyse_file),action="read",form="formatted")

         n = 0
         ios = 0
         ! count the number of line to record
         do while ( ios == 0)
            read(ios_id,*,iostat=ios) buf2
            i = index (buf2,'#')
            if ( i == 1 ) cycle
            if ( ios == 0 .and. ( trim(buf2)/='') ) then
              n = n + 1
            end if
         end do

         !allocate structure
         allocate (values(n))
         allocate(keys(n))
         keys=''

         ! init structures
         rewind(ios_id)
         cln = 0! it s the current line
         n = 0
         ios=-2
         do while ( ios /= -1 )
            call GET (ios_id,buf2,iostat=ios)
            !remove comments
            i = index (buf2,'#')

            if ( i /= 0) then
              if ( i == 1 ) buf2=''
              if ( i > 1) buf2=trim(buf2(:i-1))
            end if
            !probleme case
            if (trim(buf2)=='' .or. ios /= 0 ) cycle

            cln = cln+1
            i = index (buf2,'=')! where is =
            if ( i == 0) then
                call stop_on_error(ios,analyse_file,cln,'bad expression of "key=value"');
            end if

              ! search if the key if exist...
              do j=1,n
                 if ( keys(j) == trim(buf2(1:i-1)) ) exit
              end do

              if ( j <= n ) then
                keys(j) = trim(buf2(1:i-1))
                values(j) = trim(buf2(i+1:))
              else
                n = n+1
                keys(n) = trim(buf2(1:i-1))
                values(n) = trim(buf2(i+1:))
              end if

              call log_mess("KEY ["//trim(keys(n))//"] = ["//trim(values(n))//"]",DEBUG_DEF)

         end do

        close(ios_id)

      end subroutine read_analyse_file
!!***

!!****f* m_qtlmap_parameter_file/initialize_qtlmap_context
!!  NAME
!!    initialize_qtlmap_context
!!  DESCRIPTION
!!    Initialize all public variables option come from the module m_qtlmap_data and file name to read
!!
!!  INPUTS
!!   sim         : if the context is in the simulation case
!!   is_permute  : if dynamic option permutation mode is activate
!!
!!  OUTPUTS
!!   parsim      : the name of the simulation file
!!  NOTES
!!  SOURCE
      subroutine initialize_qtlmap_context(sim,is_permute)
        logical                       ,intent(in)    :: sim
        logical                      ,intent(in)     :: is_permute

        integer  :: ios_id = 20
        integer  :: n ,ios,cln,i,j,chr
        character(len=LEN_W)   :: buf
        character(len=LEN_S),dimension(100)   :: chromo_t
        character(len=LEN_LINE)   :: buf2
        logical :: ok,find
        real(kind=dp) :: pas_temp

         ! check present keys
        do i=1,size(index_key)
          if (.not. key_exist(index_key(i))) then
            call stop_application("Please defined key:"//trim(index_key(i)))
          end if
        end do

        !check if key are present but not interpreted by qtlmap
        do i=1,size(keys)
          if ( trim(keys(i))=='' ) cycle
          find = .false.
          do j=1,size(all_key)
            if (keys(i) == all_key(j)) then
               find = .true.
               exit
            end if
          end do
          if ( .not. find ) then
            call log_mess(" ---------------------------------- ",WARNING_DEF)
            call log_mess(" UNKNOWN KEYS ["//trim(keys(i))//"] ",WARNING_DEF)
            call log_mess(" ---------------------------------- ",WARNING_DEF)
          end if
        end do

        call get_key_value(K_OUTPUT,resul)
        call get_key_value(K_SUMM,summary)

        if ( .not. sim ) then
          call get_key_value(K_LRTSIRE,resp)
          call get_key_value(K_LRTDAM,resm)
          if ( key_exist(K_PDED) ) then
            call get_key_value(K_PDED,pdedf)
          end if
          if ( key_exist(K_PDECPLE) ) then
            call get_key_value(K_PDECPLE,pdecouple)
          end if
          if ( key_exist(K_PATEFF) ) then
            call get_key_value(K_PATEFF,pateff)
          end if
          if ( key_exist(K_FREQALL) ) then
            call get_key_value(K_FREQALL,out_freqall)
          end if
          if ( key_exist(K_MATEFF) ) then
            call get_key_value(K_MATEFF,mateff)
          end if
          if ( key_exist(K_COEFFDA) ) then
            call get_key_value(K_COEFFDA,coeffda)
          end if
          if ( key_exist(K_GRID2QTL) ) then
            call get_key_value(K_GRID2QTL,grid2qtl)
          end if
          if ( key_exist(K_PHASES) ) then
            call get_key_value(K_PHASES,out_phases)
          end if
          if ( key_exist(K_HAPLOTYPES) ) then
            call get_key_value(K_HAPLOTYPES,out_haplotypes)
          end if
        end if

        call get_key_value(K_MINDAMPHASEPROB,buf)
        PRSEUIL  = get_real(buf,ok)
        if ( .not. ok ) call stop_application("bad definition of key ["//trim(K_MINDAMPHASEPROB)//"] expecting real value.")

        call get_key_value(K_NDMIN,buf)
        ndmin    =  get_int(buf,ok)
        if ( .not. ok ) call stop_application("bad definition of key ["//trim(K_NDMIN)//"] expecting int value.")

        call get_key_value(K_MINSIREPHASEPROB,buf)
        PHPSEUIL = get_real(buf,ok)
        if ( .not. ok ) call stop_application("bad definition of key ["//trim(K_MINSIREPHASEPROB)//"] expecting real value.")

        call get_key_value(K_CHOLESKY,buf)
        SEUIL_CHO = get_real(buf,ok)
        if ( .not. ok ) call stop_application("bad definition of key ["//trim(K_CHOLESKY)//"] expecting real value.")

        call get_key_value(K_THRES_CONFUSION,buf)
        THRES_CONFUSION = get_real(buf,ok)
        if ( .not. ok ) call stop_application("bad definition of key ["//trim(K_THRES_CONFUSION)//"] expecting real value.")

        call get_key_value(K_LINEAR_CONV,buf)
        EPS_LINEAR_HETEROSCEDASTIC = get_real(buf,ok)
        if ( .not. ok ) call stop_application("bad definition of key ["//trim(K_LINEAR_CONV)//"] expecting real value.")

        call get_key_value(K_MAX_LINEAR_ITERATION,buf)
        MAX_LINEAR_ITERATION    =  get_int(buf,ok)
        if ( .not. ok ) call stop_application("bad definition of key ["//trim(K_MAX_LINEAR_ITERATION)//"] expecting int value.")

        call get_key_value(K_PSEUILHWE,buf)
        PSEUILHWE = get_real(buf,ok)
        if ( .not. ok ) call stop_application("bad definition of key ["//trim(K_PSEUILHWE)//"] expecting real value.")

        call get_key_value(K_PROB_SEUIL_RECOMB,buf)
        PROB_SEUIL_RECOMB = get_real(buf,ok)
        if ( .not. ok ) call stop_application("bad definition of key ["//trim(K_PROB_SEUIL_RECOMB)//"] expecting real value.")

        call get_key_value(K_NB_HAPLO_PRIOR,buf)
        NB_HAPLO_PRIOR = get_int(buf,ok)
        if ( .not. ok ) call stop_application("bad definition of key ["//trim(K_NB_HAPLO_PRIOR)//"] expecting integer value.")

        call get_key_value(K_PROB_HAPLO_MIN,buf)
        PROB_HAPLO_MIN = get_real(buf,ok)
        if ( .not. ok ) call stop_application("bad definition of key ["//trim(K_PROB_HAPLO_MIN)//"] expecting real value.")

        call get_key_value(K_LONG_MIN_IBS,buf)
        LONG_MIN_IBS = get_int(buf,ok)
        if ( .not. ok ) call stop_application("bad definition of key ["//trim(K_LONG_MIN_IBS)//"] expecting integer value.")

        call get_key_value(K_LONGHAP,buf)
        LONGHAP = get_int(buf,ok)
        if ( .not. ok ) call stop_application("bad definition of key ["//trim(K_LONGHAP)//"] expecting integer value.")

        call get_key_value(K_STEP,buf)
        pas_temp    =  get_real(buf,ok)
        if ( .not. ok ) call stop_application("bad definition of key ["//trim(K_STEP)//"] expecting int value.")
        call set_base_and_step(buf)

        !! Optimization initialisation
        !! -----------------------

        call get_key_value(K_OPTIM_MAXEVAL,buf)
        optim_maxeval = get_int(buf,ok)
        if ( .not. ok ) call stop_application("bad definition of key ["//trim(K_OPTIM_MAXEVAL)//"] expecting integer value.")

        call get_key_value(K_OPTIM_MAXTIME,buf)
        optim_maxtime    =  get_real(buf,ok)
        if ( .not. ok ) call stop_application("bad definition of key ["//trim(K_OPTIM_MAXTIME)//"] expecting int value.")

        call get_key_value(K_OPTIM_TOLX,buf)
        optim_tolx    =  get_real(buf,ok)
        if ( .not. ok ) call stop_application("bad definition of key ["//trim(K_OPTIM_TOLX)//"] expecting int value.")

        call get_key_value(K_OPTIM_TOLF,buf)
        optim_tolf    =  get_real(buf,ok)
        if ( .not. ok ) call stop_application("bad definition of key ["//trim(K_OPTIM_TOLF)//"] expecting int value.")

        call get_key_value(K_OPTIM_TOLG,buf)
        optim_tolg    =  get_real(buf,ok)
        if ( .not. ok ) call stop_application("bad definition of key ["//trim(K_OPTIM_TOLG)//"] expecting int value.")

        call get_key_value(K_OPTIM_H_PREC,buf)
        optim_H_PRECISION    =  get_real(buf,ok)
        if ( .not. ok ) call stop_application("bad definition of key ["//trim(K_OPTIM_H_PREC)//"] expecting int value.")


        !! Chromome initialisation
        !! -----------------------

        call get_key_value(K_CHROM,buf2)
        i=1;
        nchr=0

        do while ( i /= 0)
          i=index(buf2,",")
          if ( i /= 0 ) then
            nchr = nchr+1
            chromo_t(nchr) = trim(buf2(:i-1))
            buf2 = buf2(i+1:)
          else
            nchr = nchr+1
            chromo_t(nchr) = trim(buf2)
          end if
        end do

        allocate (chromo(nchr))
        chromo(:nchr)=chromo_t(:nchr)
        ALLOCATE (nmk(nchr))

        call get_key_value(K_UNKNOWN_GENO,buf)
        buf=trim(buf)
        call init_pheno(nchr)
        do chr=1,nchr
         nmanque =  set_pheno(chr,buf)
        end do

        ! INPUT FILES
        !------------
        if ( key_exist(K_MAP) ) then
             call get_key_value(K_MAP,in_carte)
             call file_exist(in_carte)
             mapFileDefined = .true.
         else
              mapFileDefined = .false.
         end if

        if ( key_exist(K_GENEA) ) then
            call get_key_value(K_GENEA,in_genea)
            call file_exist(in_genea)
            geneaFileDefined = .true.
        else
            geneaFileDefined = .false.
        end if
        in_perfs = ''
        if ( key_exist(K_TRAITS) ) then
            call get_key_value(K_TRAITS,in_perfs(1))
            call file_exist(in_perfs(1))
            traitsFileDefined = .true.
            !The model have to be declared...
            if ( key_exist(K_MODEL) ) then
              call get_key_value(K_MODEL,in_param_ef)
              call file_exist(in_param_ef)
            else
              call stop_application("key ["//trim(K_TRAITS)//"] is defined. The key ["//trim(K_MODEL)//"] have to be set.")
            end if
        else
            traitsFileDefined = .false.
        end if

        if ( key_exist(K_GENOTYPE) ) then
            call get_key_value(K_GENOTYPE,in_typage)
            call file_exist(in_typage)
            genotypeFileDefined = .true.
        else
            genotypeFileDefined = .false.
        end if

       if (sim ) then
      !  t_imp = .true.
        if ( key_exist(K_PARAMSIM) ) then
            call get_key_value(K_PARAMSIM,in_parsim)
            call file_exist(in_parsim)
        else if ( .not. is_permute) then
            call stop_application("key ["//trim(K_PARAMSIM)//"] is not defined.")
        end if

      ! output lrt max file is now optional

        if ( key_exist(K_OUTSIM) ) then
            call get_key_value(K_OUTSIM,simula)
      !  else
      !      call stop_application("key ["//trim(K_PARAMSIM)//"] is defined. The key ["//trim(K_OUTSIM)//"] have to be set.")
        end if

        if ( is_permute .and. .not. key_exist(K_MODEL) ) then
           call stop_application("key ["//trim(K_MODEL)//&
           "] is not defined. To perform a permutation, you have to defined a model.")
        end if

       else
        !    t_imp = .false.
       end if

        ! check base....

        if ( .not.  sim ) then
           if ( .not. mapFileDefined ) &
             call stop_application("Analysis case : You have to defined the key ["//trim(K_MAP)//"]")
           if ( .not. geneaFileDefined ) &
             call stop_application("Analysis case : You have to defined the key ["//trim(K_GENEA)//"]")
           if ( .not. traitsFileDefined ) &
             call stop_application("Analysis case : You have to defined the key ["//trim(K_TRAITS)//"]")
           if ( .not. genotypeFileDefined ) &
             call stop_application("Analysis case : You have to defined the key ["//trim(K_GENOTYPE)//"]")
        end if
      end subroutine initialize_qtlmap_context
!!***

!!****f* m_qtlmap_parameter_file/initialize_tocartha_context
!!  NAME
!!    initialize_tocartha_context
!!  DESCRIPTION
!!    Initialize all public variables option come from the module m_qtlmap_data and file name to read for
!!   the binary qtlmap2cartha.
!!
!!  OUTPUTS
!!   type_out      :  intercross 1, hybrid 2
!!  NOTES
!!  SOURCE
      subroutine initialize_tocartha_context(type_out)
        integer             ,intent(out) :: type_out ! format of file : intercross 1, hybrid 2
        character(len=LEN_W)   :: buf
        logical :: ok

        if ( key_exist(K_GENEA) ) then
            call get_key_value(K_GENEA,in_genea)
            call file_exist(in_genea)
        else
            call stop_application("Analysis case : You have to defined the key ["//trim(K_GENEA)//"]")
        end if

        if ( key_exist(K_GENOTYPE) ) then
            call get_key_value(K_GENOTYPE,in_typage)
            call file_exist(in_typage)
        else
            call stop_application("Analysis case : You have to defined the key ["//trim(K_GENOTYPE)//"]")
        end if

        if ( .not. key_exist_file(K_UNKNOWN_GENO) ) then
          call stop_application("Analysis case : You have to defined the key ["//trim(K_UNKNOWN_GENO)//"]")
        end if

        nchr=1
        allocate (chromo(nchr))
        chromo='chromosome'
        allocate(nmk(nchr))
        call get_key_value(K_UNKNOWN_GENO,buf)
        call init_pheno(nchr)
        buf=trim(buf)
        nmanque =  set_pheno(1,buf)

        if ( .not. key_exist("in_format") ) then
         call stop_application("Analysis case : You have to defined the key [in_format] 1 : intercross, 2 : hybrid")
        end if
        call get_key_value("in_format",buf)
        type_out = get_int(buf,ok)
        if ( .not. ok ) call stop_application("bad definition of key [in_format] expecting int value.")


      end subroutine  initialize_tocartha_context
!!***

!!****f* m_qtlmap_parameter_file/end_analyse_parameterfile
!!  NAME
!!    end_analyse_parameterfile
!!  DESCRIPTION
!!    release internal module data (array keys and values)
!!  NOTES
!!  SOURCE
      subroutine end_analyse_parameterfile
         deallocate (values)
         deallocate (keys)
      end subroutine
!!***

!!****f* m_qtlmap_parameter_file/get_key_value
!!  NAME
!!    get_key_value
!!  DESCRIPTION
!!    Get value according to the key give in parameter
!!  NOTES
!!  SOURCE
      subroutine get_key_value(keysearch,value)
         character(len=*) ,intent(in)     :: keysearch
         character(len=*) ,intent(out)    :: value

         integer                :: i

         value=''

         if (allocated(keys)) then
          do i=1,size(keys)
           if ( trim(keys(i)) == trim(keysearch)) then
             value = values(i)
             return
           end if
          end do
         end if

         do i=1,size(index_key)
           if ( trim(index_key(i)) == trim(keysearch)) then
             if ( trim(default_values(i)) /= '') then
                 value = default_values(i)
                 return
             end if
           end if
         end do

      end subroutine get_key_value
!!***

!!****f* m_qtlmap_parameter_file/key_exist
!!  NAME
!!    key_exist
!!  DESCRIPTION
!!    True if the key exist in the parameter user file or if the key have a default value
!!  NOTES
!!  SOURCE
      function key_exist(keysearch) result(vexist)
         character(len=*) ,intent(in)   :: keysearch

         logical                :: vexist
         integer                :: i

         if (allocated(keys)) then
          do i=1,size(keys)
           if ( trim(keys(i)) == trim(keysearch) ) then
               if ( trim(values(i)) /= '' ) then
                 vexist=.true.
                 return
               end if

           end if
          end do
         end if

         !if a default value exist...
          do i=1,size(index_key)
           if ( trim(index_key(i)) == trim(keysearch)) then
               if ( trim(default_values(i)) /= '') then
                  vexist=.true.
               else
                  vexist=.false.
               end if
               return
           end if
         end do

         vexist=.false.
         return

      end function key_exist
!!***

!!****f* m_qtlmap_parameter_file/key_exist_file
!!  NAME
!!    key_exist_file
!!  DESCRIPTION
!!    True if the key exist in the parameter user file
!!  NOTES
!!  SOURCE
      function key_exist_file(keysearch) result(vexist)
         character(len=*) ,intent(in)   :: keysearch

         logical                :: vexist
         integer                :: i

         if (allocated(keys)) then
          do i=1,size(keys)
           if ( trim(keys(i)) == trim(keysearch) ) then
               if ( trim(values(i)) /= '' ) then
                 vexist=.true.
                 return
               end if

           end if
          end do
         end if

         vexist=.false.
         return

      end function key_exist_file
!!***

!!****f* m_qtlmap_parameter_file/help_panalyse
!!  NAME
!!    help_panalyse
!!  DESCRIPTION
!!    print a help (all keys availables).
!!  NOTES
!!  SOURCE
      subroutine help_panalyse

              character(len=LEN_L) :: headkey ,headvalues,form,buf
              character(len=LEN_S) :: opt

              integer :: i

              headkey = "KEY"
              headvalues="                     DEFINITION"
              opt = "DEFAULT"
              form='(a60,a50,4x,a10)'

              print form,headkey,headvalues,opt
              print form,"------------------------------","--------------------------------","--------------"
              print form,K_OUTPUT,"Output main file"
              print form,K_LRTSIRE,"Output file paternal effects"
              print form,K_LRTDAM,"Output file maternal effects"

              print form,K_PDED,"Grand parental segment transmission marginal probabilities"
              print form,K_PDECPLE,"Grand parental segment transmission joint probabilities"
              print form,K_PATEFF,"Sire QTL effect estimations"
              print form,K_MATEFF,"Dam QTL effect estimations"

              print form,K_SUMM,"Output summary file"
              print form,K_CHROM,"Chromosomes used in the analysis : chr1,chr2,.."
              print form,K_NDMIN,'Minimal number of progeny by dam'

              call get_key_value(K_STEP,buf)
              print form,K_STEP,"Chromosomic segment exploration steps in Morgan",buf

              call get_key_value(K_MINDAMPHASEPROB,buf)
              print form,K_MINDAMPHASEPROB, 'Minimal dam phase probability',buf

              call get_key_value(K_MINSIREPHASEPROB,buf)
              print form,K_MINSIREPHASEPROB,'Minimal sire phase probability',buf

              call get_key_value(K_UNKNOWN_GENO,buf)
              print form,K_UNKNOWN_GENO,'Unknown genotype value',buf

              call get_key_value(K_CHOLESKY,buf)
              print form,K_CHOLESKY,"coeff cholesky decomposition",buf
              print form,K_MAP,"Input map file"
              print form,K_GENEA,"Input genealogy file"
              print form,K_TRAITS,"Input traits file"
              print form,K_GENOTYPE,"Input genotype file"
              print form,K_MODEL,"Input model description of traits"
              print form,K_PARAMSIM,"Input simulation parameters"
              print form,K_OUTSIM,"Output for each simulation (Position and maxLRT for each traits)"
              print form,K_PHASES,"Output phases file"
              print form,K_HAPLOTYPES,"Output haplotype file"

              call get_key_value(K_THRES_CONFUSION,buf)
              print form,K_THRES_CONFUSION,"Threshold to test confusion betwwen level inside a contingence matrix",buf
              call get_key_value(K_PSEUILHWE,buf)
              print form,K_PSEUILHWE,"Threshold to check the equilibrium of marker transmission within each family",buf
              call get_key_value(K_LINEAR_CONV,buf)
              print form,K_LINEAR_CONV,"Threshold for convergence in the linear mode heteroscedastic",buf
              call get_key_value(K_MAX_LINEAR_ITERATION,buf)
              print form,K_MAX_LINEAR_ITERATION,"Maximum iteration in the linear mode heteroscedastic to avoid infinity loop",buf

              call get_key_value(K_PROB_SEUIL_RECOMB,buf)
              print form,K_PROB_SEUIL_RECOMB,"",buf

           end subroutine help_panalyse
!!***

!!****f* m_qtlmap_parameter_file/get_summary_panalyse
!!  NAME
!!    get_summary_panalyse
!!  DESCRIPTION
!!    get the both vector of key and values defined by the user
!!  NOTES
!!  SOURCE
          subroutine get_summary_panalyse(my_index_key,values,n)
             character(len=LEN_L) ,dimension(NUMBER_ALL_KEYS),intent(out) :: my_index_key
             character(len=LEN_L) ,dimension(NUMBER_ALL_KEYS),intent(out) :: values
             integer                                         ,intent(out) :: n

             character(len=LEN_L) :: buf
             integer :: i

             n=0
             do i=1,size(all_key)
                 if ( key_exist(all_key(i)) ) then
                   n=n+1
                   call get_key_value(all_key(i),buf)
                   my_index_key(n)=trim(all_key(i))
                   values(n)=trim(buf)
                 end if
             end do

          end subroutine get_summary_panalyse
!!***

end module m_qtlmap_parameter_file
