!!****m* INPUT/m_qtlmap_simulation
!!  NAME
!!    m_qtlmap_simulation -- Simulation routines.
!!  SYNOPSIS

!!  DESCRIPTION
!!
!!  NOTES
!!
!!  BUGS
!!
!!***
!! You can use this space for remarks that should not be included
!! in the documentation.
!!/
module m_qtlmap_simulation

  use m_qtlmap_tools
  use m_qtlmap_output_handler
  use m_qtlmap_data
  use m_qtlmap_haplotype_data
  use m_qtlmap_log
  use m_qtlmap_map
  use m_qtlmap_genealogy
  use m_qtlmap_genotype
  use m_qtlmap_traits

  implicit none
  save

  !PARAMETERS

!!****d* m_qtlmap_simulation/ios_simul_file
!!  NAME
!!   ios_simul_file
!!  DESCRIPTION
!!   Unit fortran record to read the simulation file
!!***
  integer                     ,parameter    :: ios_simul_file  = 40
!!****v* m_qtlmap_simulation/current_line
!!  NAME
!!   ios_simul_file
!!  DESCRIPTION
!!   number of the current line while the parse
!!***
  integer                                   :: current_line    = 0

  integer  , dimension(:),allocatable ,private      :: filter_car

  !
  integer                      ,private      :: nqtl

  ! marker description
  !-------------------
  integer                      ,public      :: nalle
  real (kind=dp)               ,public      :: dens,taille

  ! Genealogy description
  !-------------------
  integer                      ,public      :: nbpere, inmp, indm


  public :: read_simulation_file
  public :: write_perf_qtl
  public :: create_simulation_file
  public :: init_simul_genealogy
  public :: init_simul_marker
  public :: init_genotype_simul
  public :: init_traits_simul
  public :: help_paramsim
  public :: init_permutation

  contains

!!****f* m_qtlmap_simulation/read_simulation_file
!!  NAME
!!    read_simulation_file
!!  DESCRIPTION
!!   read simulation user file and initialize internal array
!!  INPUTS
!!   parsim_file      : the simulation user file
!!   is_transcriptome : data are transcriptomic ?
!!   ndmin            : minimum of progenies for consider full sib family
!!  OUTPUTS
!!    croisement    : OUTBRED_KEYWORD, F2_KEYWORD, BC_KEYWORD
!!    nqtlsimul     : number of qtl to simulate
!!    simulQtl      : at least one qtl is simulated
!!    simMap        : simulation map case ?
!!  NOTES
!!  SOURCE
      subroutine read_simulation_file(is_transcriptome,simulQtl,&
                                      nqtlsimul,simMap,ndmin,croisement)
         logical           , intent(in)            :: is_transcriptome
         logical,intent(inout)                     :: simulQtl
         logical     , intent(inout)               :: simMap
         integer     , intent(in)                  :: ndmin
         integer     , intent(out)                 :: nqtlsimul
	     character(len=LEN_BUFFER_WORD),intent(out):: croisement
         character(len=LEN_BUFFER_WORD)            :: token
         integer                                   :: ios
         logical                                   :: label_traits_present,label_simultraits_present

         call log_mess('SUBROUTINE : read_simulation_file',DEBUG_DEF)
         call log_mess('reading simulation file...',INFO_DEF)
         !in_parsim = parsim_file
         simulQtl = .false.
         simMap   = .false.
         label_traits_present = .false.
         label_simultraits_present = .false.
         nqtlsimul=0

         call file_exist(in_parsim)


         open(unit=ios_simul_file,file=in_parsim,action="read",form="formatted")
         ios = 0
         do while ( ios == 0 )
            current_line = current_line + 1
            read(ios_simul_file,*,iostat=ios) token
            if ( ios == 0 ) then
               select case (trim(token))
                 case (LABEL_MARKERS)
                    call log_mess('setting simulation of markers...',INFO_DEF)
                    call set_simulation_marker(simMap)
                 case (LABEL_GENEALOGY)
                    call log_mess('setting simulation of genealogy...',INFO_DEF)
                    call set_simulation_genealogy(croisement)
                    call init_simul_genealogy(ngp,ngm,np,nm,nd,nr)
                    call sim_genea(inmp,indm,croisement)
                 case (LABEL_QTL)
                    call log_mess('setting simulation of qtl...',INFO_DEF)
                    call set_simulation_qtl(simulQtl)
                    nqtlsimul=nqtl
                 case (LABEL_TRAITS) ! trait are described in the model and unknown data are taken
                    if ( .not. traitsFileDefined ) then
                       call stop_application("None traits file are defined but a LABEL "//trim(LABEL_TRAITS)&
                       //" is detected. Please use a LABEL "//trim(LABEL_SIMULTRAITS)//" to not use real traits.")
                    end if
                    if ( label_simultraits_present ) then
                         call stop_application(trim(LABEL_SIMULTRAITS)//" have been defined. You cannot define "//&
                         trim(LABEL_TRAITS)//"." )
                    end if
                    call log_mess('setting simulation of traits...',INFO_DEF)
                    if (simulGenea) then
                      call stop_application("You cannot use real data traits with a simulated genealogy.")
                    end if
                    call log_mess('setting simul with real traits...',INFO_DEF)
                    croisement=F2_KEYWORD

                    if ( .not. geneaFileDefined) then
                      call stop_application("Genealogy file is not defined and none genealogy are simulated.")
                    end if

                    call read_genealogy
                    call read_model(in_param_ef)
                    call set_simulation_traits() !! initilialize filter_car private integer array
                    call fixe_structure_model(filter_car)
                    call read_traits(is_transcriptome,filter_car)
                    label_traits_present=.true.

                  case (LABEL_SIMULTRAITS) ! traits are not real (not defined in the model file)
                       if ( label_traits_present ) then
                         call stop_application(LABEL_TRAITS//" have been defined. You cannot define "//LABEL_SIMULTRAITS//"." )
                       end if
                       if (.not. simulGenea) then
                          if ( .not. geneaFileDefined) then
                             call stop_application("Genealogy file is not defined and none genealogy are simulated.")
                          end if
                          croisement=F2_KEYWORD
                          call read_genealogy
                       end if
                       call log_mess('setting simul traits...',INFO_DEF)
                       call set_simulation_simultraits()
                       label_simultraits_present=.true.
                 case default
                    call stop_on_error(-1,in_parsim, current_line,"label unknown:"//trim(token))
               end select
            end if
         end do

         ! checking at least simulation traits
         if ( .not. simulTraits ) then
             call stop_application('Check simulation file. '//trim(LABEL_SIMULTRAITS)//' Label is not defined.')
         end if

         if ( .not. simulMap .and. .not. genotypeFileDefined) then
           call stop_application("Genotype file is not defined and none genotype are simulated.")
         end if
         if ( .not. simulMap .and. .not. mapFileDefined) then
           call stop_application("Map file is not defined and none map are simulated.")
         end if
        !
         close(ios_simul_file)

         call log_mess('END SUBROUTINE : read_simulation_file',DEBUG_DEF)
      end subroutine read_simulation_file
!!***

!!****f* m_qtlmap_simulation/init_permutation
!!  NAME
!!    init_permutation
!!  DESCRIPTION
!!   initialise a permutation (none simulation user file is readed)
!!  INPUTS
!!   is_transcriptome : data are transcriptomic ?
!!  NOTES
!!  SOURCE
      subroutine init_permutation(is_transcriptome)
         logical           , intent(in)            :: is_transcriptome

          simulMap=.false.
          simulGenea=.false.
          call read_genealogy
          call read_model(in_param_ef)
          call set_simulation_traits() !! initilialize filter_car private integer array
          call fixe_structure_model(filter_car)
          call read_traits(is_transcriptome,filter_car)

      end subroutine init_permutation
!!***

!!****f* m_qtlmap_simulation/set_simulation_marker
!!  NAME
!!    set_simulation_marker
!!  DESCRIPTION
!!   read the MARKER part
!!  OUTPUTS
!!   simMap : simulation map ?
!!  NOTES
!!  SOURCE
      subroutine set_simulation_marker(simMap)
       logical     , intent(inout) :: simMap
       character(len=LEN_DEF)        :: message
       integer                     :: ios,nb_marker_used,alloc_stat,iq,ic
       real                        :: temp
       simMap = .true.
       simulMap = .true.
       current_line = current_line + 1
       read(ios_simul_file,*,iostat=ios) dens,nalle,taille

       if ( dens <= 0.d0 ) then
          call stop_application("Density of marker can not be equal or less than 0.")
       end if

       if ( taille <= 0.d0 ) then
          call stop_application("Size of map can not be equal or less than 0.")
       end if

       if ( nalle <= 0.d0 ) then
          call stop_application("Number allele per marker can not be equal or less than 0.")
       end if

       if ( dens > taille ) then
           call stop_application("Density can not be greater than the size map.")
       end if


       message = "Problem detected at the markers description:" // &
       " Allowed: density(float) allele number(int) chromosome size(float)" // &
       " unity(int) unknwon char(char)"

       call stop_on_error(ios,in_parsim, current_line,trim(message))
       ! size marker...
       !allocate (nmk(1),stat=ios)
       !call check_allocate(ios,'nmk (m_qtlmap_simulation)')
        !print *,'HOLA'
       temp = taille
       nmk(1) = 0
       do while ( temp >= 0 )
          nmk(1) = nmk(1)+1
          temp = temp - dens
       end do

       nb_marker_used = nmk(1)

       do iq=1,nqtl
         do ic=1,ncar
            call log_mess('Trait :'//trim(str(ic))//' Qtl:'//trim(str(iq))//' val:'//trim(str(ue(ic,iq)))//' M')
         end do                              !! fin ic
       end do                              !! fin iq

      end subroutine set_simulation_marker
!!***

!!****f* m_qtlmap_simulation/init_simul_marker
!!  NAME
!!    init_simul_marker
!!  DESCRIPTION
!!   initialize the array of marker coming from m_qtlmap_data
!!  NOTES
!!  SOURCE
      subroutine init_simul_marker()
        integer               :: alloc_stat,nballelepossible,i

        allocate (mark(size(nmk,1),maxval(nmk)), stat = alloc_stat)
	    call check_allocate(alloc_stat,'mark')

        do i=1,nmk(1)
          mark(1,i)='gen_mark_'//str(i)
        end do

	    allocate (posi(size(nmk,1),maxval(nmk)), stat = alloc_stat)
	    call check_allocate(alloc_stat,'posi')

	    allocate (posif(size(nmk,1),maxval(nmk)), stat = alloc_stat)
	    call check_allocate(alloc_stat,'posif')

	    allocate (posim(size(nmk,1),maxval(nmk)), stat = alloc_stat)
	    call check_allocate(alloc_stat,'posim')

        allocate (rm(size(nmk,1),maxval(nmk),maxval(nmk)), stat = alloc_stat)
        call check_allocate(alloc_stat,'rm')

        allocate (rf(size(nmk,1),maxval(nmk),maxval(nmk)), stat = alloc_stat)
        call check_allocate(alloc_stat,'rf')

        allocate ( nall(size(nmk,1),maxval(nmk)) , stat = alloc_stat)
        call check_allocate(alloc_stat,'nall')

        nballelepossible = int(VAL_MAX_INDEX_PHENO) - int(VAL_MIN_INDEX_PHENO) + 1

        allocate ( alleles(size(nmk,1),maxval(nmk),nballelepossible) , stat = alloc_stat)
        call check_allocate(alloc_stat,'alleles')

        allocate ( pc_all(size(nmk,1),maxval(nmk),nballelepossible) , stat = alloc_stat)
        call check_allocate(alloc_stat,'pc_all')

      end subroutine init_simul_marker
!!***

!!****f* m_qtlmap_simulation/set_simulation_genealogy
!!  NAME
!!    set_simulation_genealogy
!!  DESCRIPTION
!!   read the GENEALOGY part
!!
!!  OUTPUTS
!!   croisement : OUTBRED_KEYWORD, F2_KEYWORD, BC_KEYWORD
!!  NOTES
!!  SOURCE
      subroutine set_simulation_genealogy(croisement)
        character(len=LEN_DEF)      :: message
        integer                   :: ios
	    character(len=LEN_BUFFER_WORD),intent(out):: croisement

        simulGenea = .true.
        current_line = current_line + 1
        read(ios_simul_file,*,iostat=ios) croisement
        message = "None genealogy description is finding."
        call stop_on_error(ios,in_parsim, current_line,trim(message))

        if ( croisement /= F2_KEYWORD .and. &
             croisement /= BC_KEYWORD .and. &
             croisement /= OUTBRED_KEYWORD ) then
           message = in_parsim // "-Problem detected at the genealogy description:"//&
       ". SIMULATION FOR NEW PEDIGREE IS ONLY AVAILABLE FOR F2 TYPE CROSSES : value ["//trim(croisement)//"]"
           call stop_application(trim(message))
        endif


        current_line = current_line + 1
        read(ios_simul_file,*,iostat=ios) nbpere, inmp, indm
        call stop_on_error(ios,in_parsim, current_line,"number of sires , number of dams per sire" // &
        ", number of progeny per dam")

        if (croisement /= OUTBRED_KEYWORD ) then
           np = nbpere
           ngp= np
           ngm= np
           nm=np*inmp
           nd=nm*indm
           nr=np+nm
        else
           np = nbpere
           ngp = nbpere*inmp + nbpere
           ngm = nbpere*inmp + nbpere
           nm = np*inmp
           nd=  nm*indm
           nr=np+nm
        end if

      end subroutine set_simulation_genealogy
!!***

!!****f* m_qtlmap_simulation/init_simul_genealogy
!!  NAME
!!    init_simul_genealogy
!!  DESCRIPTION
!!    initialize the array of marker coming from m_qtlmap_data
!!  NOTES
!!  SOURCE
      subroutine init_simul_genealogy(ngp,ngm,np,nm,nd,nr)
           integer , intent(in)    :: ngp,ngm,np,nm,nd,nr
           integer                 :: stat

           allocate (gmere(ngm), STAT = stat)
           call check_allocate(stat,'gmere')
           allocate (gpere(ngp), STAT = stat)
            call check_allocate(stat,'gpere')
           allocate (repro(nr), STAT = stat)
            call check_allocate(stat,'repro')
           allocate (animal(nd), STAT = stat)
            call check_allocate(stat,'animal')
           allocate (pere(np), STAT = stat)
            call check_allocate(stat,'pere')
           allocate (mere(nm), STAT = stat)
            call check_allocate(stat,'mere')
           allocate (femelle(nm), STAT = stat)
            call check_allocate(stat,'femelle')
           nfem = 0

           allocate (ngmgp(ngp+1), STAT = stat)
            call check_allocate(stat,'ngmgp')
           allocate (nrgm(ngm+1), STAT = stat)
            call check_allocate(stat,'nrgm')
           allocate (nmp(np+1), STAT = stat)
            call check_allocate(stat,'nmp')
           allocate (ndm(nm+1), STAT = stat)
            call check_allocate(stat,'ndm')
           allocate (reppere(np), STAT = stat)
            call check_allocate(stat,'reppere')
           allocate (repmere(nm), STAT = stat)
            call check_allocate(stat,'repmere')
           allocate (repfem(nm), STAT = stat)
            call check_allocate(stat,'repfem')
      end subroutine init_simul_genealogy
!!***

!!****f* m_qtlmap_simulation/init_genotype_simul
!!  NAME
!!    init_genotype_simul
!!  DESCRIPTION
!!    initialize the array of marker coming from m_qtlmap_data
!!  NOTES
!!  SOURCE
    subroutine  init_genotype_simul
         integer                    :: ios

         nmes = ngp+ngm+np+nm+nd
         allocate ( numero(nmes) , stat=ios)
         call check_allocate(ios,'numero')

         allocate ( pheno(size(nmk),maxval(nmk),nmes,2) , stat=ios)
         call check_allocate(ios,'pheno')

         allocate (corregp(size(gpere)), stat=ios)
         call check_allocate(ios,'corregp')

         allocate (corregm(size(gmere)), stat=ios)
         call check_allocate(ios,'corregm')

         allocate (correr(size(repro)), stat=ios)
         call check_allocate(ios,'correr')

         allocate (correp(size(pere)), stat=ios)
         call check_allocate(ios,'correp')

         allocate (correm(size(mere)), stat=ios)
         call check_allocate(ios,'correm')

         allocate (corred(size(animal)), stat=ios)
         call check_allocate(ios,'corred')

         allocate (presentg(nchr,size(animal)), stat=ios)
         call check_allocate(ios,'presentg')

    end subroutine init_genotype_simul
!!***

!!****f* m_qtlmap_simulation/init_traits_simul
!!  NAME
!!    init_traits_simul
!!  DESCRIPTION
!!    initialize the array of marker coming from m_qtlmap_data
!!  NOTES
!!  SOURCE
    subroutine  init_traits_simul
         integer                    :: alloc_stat,ic

         allocate (carac(ncar) , stat=alloc_stat)
         call check_allocate(alloc_stat,'carac')

         allocate (corperf(size(animal)), stat = alloc_stat)
         call check_allocate(alloc_stat,'corperf')

         allocate (y(ncar,size(animal)), stat = alloc_stat)
         call check_allocate(alloc_stat,'y')

         allocate (presentc(ncar,size(animal)), stat = alloc_stat)
         call check_allocate(alloc_stat,'presentc')

         allocate(natureY(ncar), stat = alloc_stat)
         call check_allocate(alloc_stat,'natureY')

         allocate (cd(ncar,size(animal)), stat = alloc_stat)
         call check_allocate(alloc_stat,'cd')
         cd=1.d0
         presentc = .true.

         allocate (ndelta(ncar,size(animal)), stat = alloc_stat)
         call check_allocate(alloc_stat,'ndelta')

         allocate (covar(nd,ncov), stat = alloc_stat)
         call check_allocate(alloc_stat,'covar')

         allocate (namefix(nfix), stat = alloc_stat)
         call check_allocate(alloc_stat,'namefix')

         allocate (namecov(ncov), stat = alloc_stat)
         call check_allocate(alloc_stat,'namecov')

         allocate (listelev(nfix,nd), stat = alloc_stat)
         call check_allocate(alloc_stat,'listelev')

         allocate (modele(ncar,3+(2*nfix)+ncov), stat = alloc_stat)
         call check_allocate(alloc_stat,'modele')
         modele=0

         allocate (listModelTrait(ncar))
         do ic=1,ncar
            allocate ( listModelTrait(ic)%indexFixedEffect(0))
            listModelTrait(ic)%nbfe=0
            allocate ( listModelTrait(ic)%indexCovariate(0))
            listModelTrait(ic)%nbco=0
            allocate (listModelTrait(ic)%nbint(MAX_QTL))
            listModelTrait(ic)%nbint=0
         end do

         allocate (nlev(nfix), stat = alloc_stat)
         call check_allocate(alloc_stat,'nlev')

         allocate (nivx(nd,nfix), stat = alloc_stat)
         call check_allocate(alloc_stat,'nivx')

    end subroutine init_traits_simul
!!***

!!****f* m_qtlmap_simulation/set_simulation_qtl
!!  NAME
!!    set_simulation_qtl
!!  DESCRIPTION
!!
!!  NOTES
!!  SOURCE
      subroutine set_simulation_qtl(simQtl)
          logical,intent(inout)     :: simQtl
          integer                   :: ios,i,iq,jq,chr
          real :: l
          character(len=LEN_DEF)                 :: word
          character(len=LEN_LINE)                :: line_read
          logical                                :: is_ok

          simtyp   = .true.

           current_line = current_line + 1
           read(ios_simul_file,*,iostat=ios) nqtl

           call stop_on_error(ios,in_parsim, current_line,"QTL number")

           if (nqtl.gt.0) then
               simQtl=.true. ! More than one qtl are simulated
           end if

           allocate(posiqtl(nqtl))
           allocate(chrqtl(nqtl))
           allocate (xlim(nqtl))

           !*** POSITION  ***
           !*****************
           current_line = current_line + 1
           call GET(ios_simul_file,line_read,IOSTAT=ios)
           call stop_on_error(ios,in_parsim, current_line,"QTL : waiting for keyword 'position' and values' ")

           word = trim(next_word(line_read,is_ok))
           if (word /= 'position') then
             call stop_application("'position' keyword expected! **"//trim(word)//"**")
           end if

           do i=1,nqtl
             word = trim(next_word(line_read,is_ok))
             if (.not. is_ok ) then
               call stop_application("qtl "// trim(str(i))//" position  expected!")
             end if
             posiqtl(i)=get_real(word,is_ok)
             if ( .not. is_ok ) then
                call stop_application("qtl "// trim(str(i))//" position  expected! **"//trim(word)//"**")
             end if
           end do

           !*** CHROMOSOME  ***
           !*******************
           current_line = current_line + 1
           call GET(ios_simul_file,line_read,IOSTAT=ios)
           call stop_on_error(ios,in_parsim, current_line,"QTL : waiting for keyword 'chromosome' and values' ")

           word = trim(next_word(line_read,is_ok))
           if (word /= 'chromosome') then
             call stop_application("'chromosome' keyword expected! **"//trim(word)//"**")
           end if

           do i=1,nqtl
             word = trim(next_word(line_read,is_ok))
             if (.not. is_ok ) then
               call stop_application("qtl "// trim(str(i))//" chromosome  expected!")
             end if
             chrqtl(i)=get_int(word,is_ok)
             if ( .not. is_ok ) then
                call stop_application("qtl "// trim(str(i))//" chromosome  expected! **"//trim(word)//"**")
             end if
           end do

           !*** FREQUENCIES  ***
           !********************
           current_line = current_line + 1
           call GET(ios_simul_file,line_read,IOSTAT=ios)
           call stop_on_error(ios,in_parsim, current_line,"QTL : waiting for keyword 'frequency' and values' ")

           word = trim(next_word(line_read,is_ok))
           if (word /= 'frequency') then
             call stop_application("'frequency' keyword expected! **"//trim(word)//"**")
           end if

           do i=1,nqtl
             word = trim(next_word(line_read,is_ok))
             if (.not. is_ok ) then
               call stop_application("qtl "// trim(str(i))//" frequency  expected!")
             end if
             xlim(i)=get_real(word,is_ok)
             if ( .not. is_ok ) then
                call stop_application("qtl "// trim(str(i))//" frequency  expected! **"//trim(word)//"**")
             end if
           end do


           ! check : * position have to be ordered and not at the same position
           do iq=1,nqtl-1
              do jq=iq+1,nqtl
               if (posiqtl(iq) == posiqtl(jq)) then
                call stop_application("QTLs"//trim(str(iq))//" and "//trim(str(jq))//"are simulated on the same position")
               end if

               if (posiqtl(iq) > posiqtl(jq)) then
                call stop_application("QTLs have to be ordered : qtl "//trim(str(iq))//"> qtl "//trim(str(jq)))
               end if
             end do
           end do

           call stop_on_error(ios,in_parsim, current_line,"QTL frequencies")

      end subroutine set_simulation_qtl
!!***

!!****f* m_qtlmap_simulation/set_simulation_traits
!!  NAME
!!    set_simulation_traits
!!  DESCRIPTION
!!    set the array filter_car. This filter contains index trait referenced in the model.
!!  NOTES
!!  SOURCE
      subroutine set_simulation_traits()
        integer                                :: ic,jc,iq,j
        integer                                :: ios
        character(len=LEN_DEF)                 :: word
        character(len=LEN_LINE)                :: line_read
        logical                                :: is_ok
        integer                                :: nc

        simulTraits = .true.
        current_line = current_line + 1
        read (ios_simul_file,*,iostat=ios) nc

         if ( nc > ncar ) then
             call log_mess("               Number of trait in the model file:"//trim(str(ncar)),ERROR_DEF)
             call log_mess("Number of trait in the simulation parameter file:"//trim(str(nc)),ERROR_DEF)
             call stop_application("Your model file do not match with the simulation parameter file."//&
             " Too many traits defined in the parameter file.")
         end if

        call stop_on_error(ios,in_parsim, current_line,"Number of traits")

        ! allocation of filter
        allocate(filter_car(nc))

        if ( (.not. allocated(carac_t)) .or. size(carac_t) <= 0 ) then
           call stop_application("DEVEL ERROR- set_simulation_traits :: carac_t is empty!")
        end if

!       FORMAT: TRAIT_NAME H2
        do ic=1,nc
          current_line = current_line + 1
          call GET(ios_simul_file,line_read,IOSTAT=ios)
          if (trim(line_read)=='') cycle ! empty line

          if ( (ios /=0) ) then
             call stop_on_error (1,in_parsim,current_line,'traits ['//trim(str(ic))//'] is not defined.')
          end if

          word = trim(next_word(line_read,is_ok))
          if ( .not. is_ok ) call stop_on_error (1,in_parsim,current_line,'traits ['//trim(str(ic))//'] is not defined.')

          ! carac_t is a buffer array read from the model file
          do j=1,size(carac_t)
            if ( carac_t(j) == word ) exit
          end do
          if ( j>size(carac_t) ) then
             call stop_application("Trait["//trim(word)//"] is not defined in the model!")
          end if
          filter_car(ic)=j

          if (natureY_t(ic) == 'i') call read_attributes_discrete_trait(nc,ic,line_read)

        end do

        call set_soglia(nc,natureY_t)

        if (nc == 1) call log_mess('Only one trait --> no correlation matrix to define',VERBOSE_DEF)

        !call read_correlation_matrix(nc)

        call read_qtl_effect_on_trait(nc)

      end subroutine set_simulation_traits
!!***

!!****f* m_qtlmap_simulation/read_correlation_matrix
!!  NAME
!!    read_correlation_matrix
!!  DESCRIPTION
!!
!!  NOTES
!!  SOURCE
     subroutine read_correlation_matrix(nc)
          integer ,intent(in)                    :: nc
          character(len=LEN_DEF)                 :: word
          character(len=LEN_LINE)                :: line_read
          character(len=1000)                    :: buf_line
          logical                                :: is_ok
          integer                                :: j,ic,ios,jc


          if ( nc <= 1 ) return
          !correlation matrix definition
          line_read=''

          do while ( trim(line_read)=='')
            current_line = current_line + 1
            call GET(ios_simul_file,line_read,IOSTAT=ios)
            if ( (ios /=0) ) then
              !read(ios_simul_file,*,iostat=ios) (rho(ic,jc),jc=1,ic-1)
              call stop_on_error(ios,in_parsim, current_line,"The correlation matrix is not defined")
            end if
          end do

          !FORMAT [ [corr(1)] [ corr(2,1) corr(2,2)] ... ]

          !!keyword correlation
          word = trim(next_word(line_read,is_ok))
          if (word /= 'correlation') then
             call stop_application("'correlation' keyword expected! **"//trim(word)//"**")
          end if
          !! first "["
          buf_line = trim(line_read)

          j=index(buf_line,"[")
          if ( j == 0 ) then
              call stop_on_error(ios,in_parsim, current_line,"Expecting '[' :**"//trim(line_read)//"**")
          end if
          buf_line=buf_line(j+1:)

          do ic=2,nc
            call parse_real_array(buf_line,ic-1,RhoP(ic,:),is_ok)
            if ( .not. is_ok ) call stop_on_error (1,in_parsim,current_line,'Correlation matrix definition for ['//&
             trim(str(ic))//'] unknown.')
            do jc=1,ic-1

             !Add test : a correlation is between -1 and 1
             if ( RhoP(ic,jc) > 1 .or. RhoP(ic,jc) < -1 ) then
                call stop_application("Correlation have to be defined betwwen -1 and 1 bad value :["//trim(str(RhoP(ic,jc)))//"]")
             end if
             call log_mess ('Correlation matrix definition ['//trim(str(ic))//']:'//trim(str(RhoP(ic,jc))),VERBOSE_DEF)
            end do
          end do

          j=index(buf_line,"]")
          if ( j == 0 ) then
              call stop_on_error(1,in_parsim, current_line,"Expecting ']' to close the matrix definition")
          end if
         buf_line=buf_line(:j-1)
         if ( trim(buf_line) /= '' ) then
           call stop_on_error(1,in_parsim, current_line,"Too many information in correlation matrix :"//trim(buf_line)//']')
         end if

          !fill the matrix
          do ic=2,nc
           do jc=1,ic-1
            RhoP(jc,ic)=RhoP(ic,jc)
           end do
          end do

     end subroutine read_correlation_matrix
!!***

!!****f* m_qtlmap_simulation/read_qtl_effect_on_trait
!!  NAME
!!    read_qtl_effect_on_trait
!!  DESCRIPTION
!!
!!  NOTES
!!  SOURCE
     subroutine read_qtl_effect_on_trait(nc)
        integer, intent(in)    :: nc
        character(len=LEN_DEF)                 :: word
        character(len=LEN_LINE)                :: line_read
        logical                                :: is_ok
        integer                                :: ic,iq,ios

        allocate (ue(nc,nqtl))
        ue=0.d0
        if (nqtl > 0) then
           !qtl effect

           current_line = current_line + 1
           line_read=''
           do while ( trim(line_read) == '' )
             call GET(ios_simul_file,line_read,IOSTAT=ios)
             if ( (ios /= 0) ) then
              call stop_on_error(1,in_parsim, current_line,"QTL effects on traits")
             end if
           end do
           word = trim(next_word(line_read,is_ok))
           if (word /= 'qtleffect') then
             call stop_application("'qtleffect' keyword expected! **"//trim(word)//"**")
           end if


           do ic=1,ncar
             do iq=1,nqtl
               word = next_word(line_read,is_ok)
               if ( .not. is_ok ) call stop_on_error (1,in_parsim,current_line,&
               'QTL ['//trim(str(iq))//'] effects on traits ['//trim(str(ic))//'] unknown.')
               ue(ic,iq) = get_real(word,is_ok)
               if ( .not. is_ok ) call stop_on_error (1,in_parsim,current_line,&
                   'QTL ['//trim(str(iq))//'] effects on traits ['//trim(str(ic))//'] have to be a real.')
               call log_mess ( 'QTL ['//trim(str(iq))//'] effects on traits ['//trim(str(ic))//']:'&
               //trim(str(ue(ic,iq))),VERBOSE_DEF)
           !read(ios_simul_file,*,iostat=ios) ((ue(ic,iq),iq=1,nqtl),ic=1,ncar)
           !call stop_on_error(ios,in_parsim, current_line,"QTL effects on traits")
             end do
           end do

        else
        call log_mess('NQTL=0 --> No qtl effect to define',VERBOSE_DEF)
       end if
     end subroutine read_qtl_effect_on_trait
!!***

!!****f* m_qtlmap_simulation/set_soglia
!!  NAME
!!    set_soglia
!!  DESCRIPTION
!!
!!  NOTES
!!  SOURCE
     subroutine set_soglia(nc,nature)
       integer       ,intent(in):: nc
       character(len=*),dimension(nc),intent(in) :: nature
       double precision         :: ppf,freqtot
       integer   :: ic,j
    
       if ( .not. allocated(freqmodsimultrait) ) return

       allocate (soglia(nc,size(freqmodsimultrait,2)))

       do ic=1,nc

        if ( nature(ic) /='i') cycle

        freqtot=0.d0
        do j=1,nbmodsimultrait(ic)-1
          freqtot=freqtot+freqmodsimultrait(ic,j)
          call NORPPF(freqtot,PPF)
          soglia(ic,j)=PPF
        end do
       end do

     end  subroutine set_soglia
!!***
!**********************************************************************

           SUBROUTINE NORPPF(P,PPF)
!
!     PURPOSE--THIS SUBROUTINE COMPUTES THE PERCENT POINT
!              FUNCTION VALUE FOR THE NORMAL (GAUSSIAN)
!              DISTRIBUTION WITH MEAN = 0 AND STANDARD DEVIATION = 1.
!              THIS DISTRIBUTION IS DEFINED FOR ALL X AND HAS
!              THE PROBABILITY DENSITY FUNCTION
!              F(X) = (1/SQRT(2*PI))*EXP(-X*X/2).
!              NOTE THAT THE PERCENT POINT FUNCTION OF A DISTRIBUTION
!              IS IDENTICALLY THE SAME AS THE INVERSE CUMULATIVE
!              DISTRIBUTION FUNCTION OF THE DISTRIBUTION.
!     INPUT  ARGUMENTS--P      = THE SINGLE PRECISION VALUE
!                                (BETWEEN 0.0 AND 1.0)
!                                AT WHICH THE PERCENT POINT
!                                FUNCTION IS TO BE EVALUATED.
!     OUTPUT ARGUMENTS--PPF    = THE SINGLE PRECISION PERCENT
!                                POINT FUNCTION VALUE.
!     OUTPUT--THE SINGLE PRECISION PERCENT POINT
!             FUNCTION VALUE PPF.
!     PRINTING--NONE UNLESS AN INPUT ARGUMENT ERROR CONDITION EXISTS.
!     RESTRICTIONS--P SHOULD BE BETWEEN 0.0 AND 1.0, EXCLUSIVELY.
!     OTHER DATAPAC   SUBROUTINES NEEDED--NONE.
!     FORTRAN LIBRARY SUBROUTINES NEEDED--SQRT, ALOG.
!     MODE OF INTERNAL OPERATIONS--SINGLE PRECISION.
!     LANGUAGE--ANSI FORTRAN.
!     REFERENCES--ODEH AND EVANS, THE PERCENTAGE POINTS
!                 OF THE NORMAL DISTRIBUTION, ALGORTIHM 70,
!                 APPLIED STATISTICS, 1974, PAGES 96-97.
!               --EVANS, ALGORITHMS FOR MINIMAL DEGREE
!                 POLYNOMIAL AND RATIONAL APPROXIMATION,
!                 M. SC. THESIS, 1972, UNIVERSITY
!                 OF VICTORIA, B. C., CANADA.
!               --HASTINGS, APPROXIMATIONS FOR DIGITAL
!                 COMPUTERS, 1955, PAGES 113, 191, 192.
!               --NATIONAL BUREAU OF STANDARDS APPLIED MATHEMATICS
!                 SERIES 55, 1964, PAGE 933, FORMULA 26.2.23.
!               --FILLIBEN, SIMPLE AND ROBUST LINEAR ESTIMATION
!                 OF THE LOCATION PARAMETER OF A SYMMETRIC
!                 DISTRIBUTION (UNPUBLISHED PH.D. DISSERTATION,
!                 PRINCETON UNIVERSITY), 1969, PAGES 21-44, 229-231.
!               --FILLIBEN, 'THE PERCENT POINT FUNCTION',
!                 (UNPUBLISHED MANUSCRIPT), 1970, PAGES 28-31.
!               --JOHNSON AND KOTZ, CONTINUOUS UNIVARIATE
!                 DISTRIBUTIONS--1, 1970, PAGES 40-111.
!               --THE KELLEY STATISTICAL TABLES, 1948.
!               --OWEN, HANDBOOK OF STATISTICAL TABLES,
!                 1962, PAGES 3-16.
!               --PEARSON AND HARTLEY, BIOMETRIKA TABLES
!                 FOR STATISTICIANS, VOLUME 1, 1954,
!                 PAGES 104-113.
!     COMMENTS--THE CODING AS PRESENTED BELOW
!               IS ESSENTIALLY IDENTICAL TO THAT
!               PRESENTED BY ODEH AND EVANS
!               AS ALGORTIHM 70 OF APPLIED STATISTICS.
!               THE PRESENT AUTHOR HAS MODIFIED THE
!               ORIGINAL ODEH AND EVANS CODE WITH ONLY
!               MINOR STYLISTIC CHANGES.
!             --AS POINTED OUT BY ODEH AND EVANS
!               IN APPLIED STATISTICS,
!               THEIR ALGORITHM REPRESENTES A
!               SUBSTANTIAL IMPROVEMENT OVER THE
!               PREVIOUSLY EMPLOYED
!               HASTINGS APPROXIMATION FOR THE
!               NORMAL PERCENT POINT FUNCTION--
!               THE ACCURACY OF APPROXIMATION
!               BEING IMPROVED FROM 4.5*(10**-4)
!               TO 1.5*(10**-8).
!     WRITTEN BY--JAMES J. FILLIBEN
!                 STATISTICAL ENGINEERING LABORATORY (205.03)
!                 NATIONAL BUREAU OF STANDARDS
!                 WASHINGTON, D. C. 20234
!                 PHONE:  301-921-2315
!     ORIGINAL VERSION--JUNE      1972.
!     UPDATED         --SEPTEMBER 1975.
!     UPDATED         --NOVEMBER  1975.
!     UPDATED         --OCTOBER   1976.
!
!---------------------------------------------------------------------
!
     double precision  :: P,PPF,P0,P1,P2,P3,P4,Q0,Q1,Q2,Q3,Q4,ADEN,T,ANUM,R
     integer           :: IPR

      DATA P0,P1,P2,P3,P4   &
     /-.322232431088,-1.0,  &
      -.342242088547,-.204231210245E-1, &
      -.453642210148E-4/
      DATA Q0,Q1,Q2,Q3,Q4       &
     /.993484626060E-1,.588581570495, &
      .531103462366,.103537752850,  &
      .38560700634E-2/
!
      IPR=6
!
!     CHECK THE INPUT ARGUMENTS FOR ERRORS
!
      IF(P.LE.0.0.OR.P.GE.1.0) GOTO 50
      GOTO 90
   50 WRITE(IPR,1)
 !     WRITE(IPR,46)P
      RETURN
   90 CONTINUE
    1 FORMAT(1H ,'115H***** FATAL ERROR--THE FIRST  INPUT ARGUMENT TO THE'//  &
      'NORPPF SUBROUTINE IS OUTSIDE THE ALLOWABLE (0,1) INTERVAL *****')
 !  46 FORMAT(1H ,'35H***** THE VALUE OF THE ARGUMENT IS ,F15.8,6H *****')
!
!-----START POINT-----------------------------------------------------
!
      IF(P.NE.0.5)GOTO 150
      PPF=0.0
      RETURN
!
  150 R=P
      IF(P.GT.0.5)R=1.0-R
      T=SQRT(-2.0*ALOG(real(R)))
      ANUM=((((T*P4+P3)*T+P2)*T+P1)*T+P0)
      ADEN=((((T*Q4+Q3)*T+Q2)*T+Q1)*T+Q0)
      PPF=T+(ANUM/ADEN)
      IF(P.LT.0.5)PPF=-PPF
      RETURN
!
      END  SUBROUTINE

!!****f* m_qtlmap_simulation/read_attributes_discrete_trait
!!  NAME
!!    read_attributes_discrete_trait
!!  DESCRIPTION
!!
!!  NOTES
!!  SOURCE
      subroutine read_attributes_discrete_trait(nc,ic,line_read)
        integer              , intent(in)      :: nc,ic
        character(len=LEN_DEF) ,intent(inout)    :: line_read
        character(len=LEN_DEF)                   :: word
        integer                                :: nbmod,i
        logical                                :: is_ok
        real                                   :: summod
        real (kind=dp),dimension(:,:)  , allocatable :: freqmodsimultrait_t

        if ( .not. allocated(nbmodsimultrait)) then
           allocate(nbmodsimultrait(nc))
           nbmodsimultrait=0
        end if
        !number of modality
        word = trim(next_word(line_read,is_ok))

        if ( .not. is_ok ) call stop_on_error (1,in_parsim,current_line,'number of modality ['//trim(str(ic))//'] is not defined.')
        nbmod=get_int(word,is_ok)

        if ( .not. is_ok ) call stop_on_error (1,in_parsim,current_line,&
          'modality of traits ['//trim(str(ic))//'] have to be an integer.')
        nbmodsimultrait(ic)=nbmod

        if ( .not. allocated(freqmodsimultrait)) then
            allocate(freqmodsimultrait(nc,nbmod))
            freqmodsimultrait=0.d0
        else
          if (size(freqmodsimultrait,2)<nbmod) then
            allocate(freqmodsimultrait_t(size(freqmodsimultrait,1),size(freqmodsimultrait,2)))
            freqmodsimultrait_t=freqmodsimultrait
            deallocate(freqmodsimultrait)
            allocate(freqmodsimultrait(size(freqmodsimultrait_t,1),nbmod))
            freqmodsimultrait=0.0
            freqmodsimultrait(:,1:size(freqmodsimultrait_t,2))=freqmodsimultrait_t
            deallocate(freqmodsimultrait_t)
          end if
        end if

        summod=0.d0
        do i=1,nbmod
          word = trim(next_word(line_read,is_ok))
          if ( .not. is_ok ) call stop_on_error (1,in_parsim,current_line,'freq [trait:'//&
          trim(str(ic))//' mod:"//trim(str(i))//"] is not defined.')
          freqmodsimultrait(ic,i)=get_real(word,is_ok)
          if ( .not. is_ok ) call stop_on_error (1,in_parsim,current_line,&
          'frequency of modality :'//trim(str(i))//'for trait ['//trim(str(ic))//'] have to be a real.')
          if ( (freqmodsimultrait(ic,i) > 1.d0) .or. (freqmodsimultrait(ic,i) < 0.d0) ) then
            call stop_on_error (1,in_parsim,current_line,&
          'frequency of modality :'//trim(str(i))//'for trait ['//trim(str(ic))//'] have to be 0<=mod<=1.')
          end if
          summod = summod + freqmodsimultrait(ic,i)
        end do

        if ( summod /= 1.d0 ) then
             call stop_on_error (1,in_parsim,current_line,&
             'sum of frequency have to be equal to 1. Trait ['//trim(str(ic))//' : sum:'//trim(str(summod))//']')
        end if

      end subroutine read_attributes_discrete_trait
!!***

!!****f* m_qtlmap_simulation/set_simulation_simultraits
!!  NAME
!!    set_simulation_simultraits
!!  DESCRIPTION
!!
!!  NOTES
!!  SOURCE
      subroutine set_simulation_simultraits
        integer                                :: ic,jc,iq,j
        integer                                :: ios
        character(len=LEN_DEF)                 :: word
        character(len=LEN_LINE)                :: line_read
        logical                                :: is_ok

        simulTraits = .true.
        current_line = current_line + 1
        ios=0
        word=""

        do while (ios == 0 .and. word == "" )
           read (ios_simul_file,*,iostat=ios) word
           ncar = get_int(word,is_ok)
           if ( .not. is_ok ) then
             call stop_on_error (1,in_parsim,current_line,'bad definition of number trait: ['//trim(word)//'].')
           end if
        end do

        call stop_on_error(ios,in_parsim, current_line,"Number of traits")

        call init_traits_simul
        allocate(h2(ncar))
        h2=0.d0
        allocate(RhoP(ncar,ncar))
        RhoP=0.d0
        allocate(RhoG(ncar,ncar))
        RhoG=0.d0

        ! FORMAT TRAITNAME NATURE HERITABILITY N=NUMBER_OF_MOD FREQ1 FREQ2..FREQN
        ! Constraint : FREQ1+FREQ2+..+FREQN=1
        do ic=1,ncar
          current_line = current_line + 1
          call GET(ios_simul_file,line_read,IOSTAT=ios)
          if (trim(line_read)=='') cycle ! empty line

          if ( (ios /=0) ) then
             call stop_on_error (1,in_parsim,current_line,'simul traits ['//trim(str(ic))//'] is not defined.')
          end if

          word = trim(next_word(line_read,is_ok))
          if ( .not. is_ok ) call stop_on_error (1,in_parsim,current_line,'traits ['//trim(str(ic))//'] is not defined.')
          carac(ic)=word

          word = trim(next_word(line_read,is_ok))
          if ( .not. is_ok ) call stop_on_error (1,in_parsim,current_line,'nature of traits ['//trim(str(ic))//'] is not defined.')

          if ( word /= 'i' .and. word /= 'r') then
             call stop_on_error (1,in_parsim,current_line,&
             "Availbale value for nature of traits is 'i' or 'r' **"//trim(word)//'** .')
          end if

          natureY(ic)=word

          word = trim(next_word(line_read,is_ok))
          if ( .not. is_ok ) call stop_on_error (1,in_parsim,current_line,'Heritability of traits ['//&
          trim(carac(j))//'] is not defined.')
          h2(ic) = get_real(word,is_ok)

          if ( .not. is_ok ) call stop_on_error (1,in_parsim,current_line,&
          'Heritability of traits ['//trim(carac(j))//'] have to be a real.')
          call log_mess ('Heritability of traits ['//trim(str(ic))//']:'//trim(str(h2(ic))),VERBOSE_DEF)

          if (natureY(ic) == 'i') call read_attributes_discrete_trait(ncar,ic,line_read)

        end do
        call set_soglia(ncar,natureY)
         !checking heritability
        do ic=1,ncar
           if (h2(ic)>1) then
              call stop_application("Heritability for the "//trim(str(ic))//" is greater than 1")
           end if
           if (h2(ic)<0) then
              call stop_application("Heritability for the "//trim(str(ic))//" is less than 0")
           end if
        end do


        call read_correlation_matrix(ncar)
        call read_qtl_effect_on_trait(ncar)

      end subroutine set_simulation_simultraits
!!***

      subroutine write_perf_qtl(file_name,qtl)
        character(len=*),intent(in)     :: file_name
        integer          ,intent(in)    :: qtl(np+nfem+nd,nqtl,2)
        integer ,dimension(ncar)        :: ipresent
        integer                         :: kd,ic,iph,iq

        call log_mess("writing performance qtl in file :["//file_name//']',INFO_DEF)
        open(1,file=file_name)
        do kd=1,nd
         do ic=1,ncar
           if(presentc(ic,kd)) then
              ipresent(ic)=1
           else
              ipresent(ic)=0
           end if
        end do

        write(1,*) trim(animal(kd)),          &
       (y(ic,kd),ipresent(ic),'1',ic=1,ncar), &
       ((qtl(np+nfem+kd,iq,iph),iph=1,2),iq=1,2)
       end do
       close(1)

      end subroutine write_perf_qtl


      subroutine create_simulation_file(parsim_file,irho,ih2)
         character(len=*),intent(in)     :: parsim_file
         real (kind=dp), dimension (:,:) :: irho
         real (kind=dp), dimension (:)   :: ih2

         integer :: i,j

         call log_mess("creating simulation file:["//trim(parsim_file)//']',INFO_DEF)

         open(unit=ios_simul_file,file=parsim_file,action="write",form="formatted")

         write (ios_simul_file ,* ) LABEL_TRAITS
         write (ios_simul_file ,fmt="(i5)" ) ncar
         write (ios_simul_file ,fmt="("//trim(str(ncar))//"(f7.3,1x))" ) (ih2(i),i=1,ncar)

         if ( ncar > 1 ) then
           do i=2,ncar
             write (ios_simul_file ,fmt="(30(f6.3,1x))" ) (irho(i,j),j=1,i-1)
           end do
         end if

         !write (ios_simul_file ,* ) ('0.0 ',i=1,ncar)
         close (ios_simul_file)

!         do i=1,nqtl
!            do j=1,ncar
!                write (ios_simul_file ,* )
!            end do
!         end do

      end subroutine create_simulation_file

      subroutine help_paramsim

      end subroutine help_paramsim

end module m_qtlmap_simulation
