!!****p* QTLMap/INPUT
!!  NAME
!!    INPUT
!!  DESCRIPTION
!!  Package input :
!!     - Initilize Data user routines
!!
!!  CREATION DATE
!!  01/01/2009
!!  COPYRIGHT
!!***

!!****m* INPUT/m_qtlmap_traits
!!  NAME
!!    m_qtlmap_traits -- Performances routines.
!!  SYNOPSIS

!!  DESCRIPTION
!!
!!  NOTES
!!
!!  BUGS
!!
!!  SEE ALSO
!!
!!  COPYRIGHT
!!***
!! You can use this space for remarks that should not be included
!! in the documentation.
!!/
module m_qtlmap_traits
  use m_qtlmap_tools
  use m_qtlmap_data
  use m_qtlmap_log
  use m_qtlmap_output_handler
  use m_qtlmap_genealogy
  use m_qtlmap_cd

  implicit none
  save

!!****d* m_qtlmap_traits/unit_perf
!!  NAME
!!   unit_perf
!!  DESCRIPTION
!!   the unit fortran assigned to the phenotypic file
!!  NOTES
!!
!!***
  integer                                            ,parameter,private  :: unit_perf = 777
!!****d* m_qtlmap_traits/unit_mod
!!  NAME
!!   unit_mod
!!  DESCRIPTION
!!   the unit fortran assigned to the model file
!!  NOTES
!!
!!***
  integer                                            ,parameter,private  :: unit_mod  = 3
!!****d* m_qtlmap_traits/ALL_LABEL_MODEL
!!  NAME
!!   ALL_LABEL_MODEL
!!  DESCRIPTION
!!   Keyword to apply a generic model for all files
!!  NOTES
!!
!!***
  character(len=10)                                  ,parameter,private  :: ALL_LABEL_MODEL = 'all'

!!****d* m_qtlmap_traits/NB_DES_MIN
!!  NAME
!!   NB_DES_MIN
!!  DESCRIPTION
!!   The minimum number of progenies used by the permutation method
!!  NOTES
!!   nbre minimum de descendant a considerer dans une famille (pere, pere-mere) pour faire des permutations
!!***
  integer                                            ,parameter,public   :: NB_DES_MIN=10

  ! INTERNAL VAR
  ! ----------------------------------------------------------------------
!!****v* m_qtlmap_traits/nuis
!!  NAME
!!   nuis
!!  DESCRIPTION
!!   indicate if fixed effect / covariate are take in care (1 otherwise 0) for a specific trait ic
!!  DIMENSIONS
!!   nc,nfix+ncov
!!  NOTES
!!***
  integer           , dimension (:,:),allocatable                ,private :: nuis
!!****v* m_qtlmap_traits/nuis_t
!!  NAME
!!   nuis_t
!!  DESCRIPTION
!!   buffer array for nuis
!!  DIMENSIONS
!!   nc,nfix+ncov
!!  NOTES
!!***
  integer           , dimension (:,:),allocatable                ,private :: nuis_t
!!****v* m_qtlmap_traits/int_qtl
!!  NAME
!!   int_qtl
!!  DESCRIPTION
!!   indicate if a qtl is in interaction with a fixed effect in the model trait (1 otherwise 0)
!!  DIMENSIONS
!!   nc,nqtl,nfix
!!  NOTES
!!***
  integer         , dimension (:,:,:),allocatable                ,private :: int_qtl

!!****v* m_qtlmap_traits/int_qtl_t
!!  NAME
!!   int_qtl_t
!!  DESCRIPTION
!!   buffer array for int_qtl
!!  DIMENSIONS
!!   nc,nqtl,nfix
!!  NOTES
!!
!!***
  integer         , dimension (:,:,:),allocatable                ,private :: int_qtl_t

!!****v* m_qtlmap_traits/nb_qtl_def
!!  NAME
!!   nb_qtl_def
!!  DESCRIPTION
!!   number of qtl - fixed effect interaction defined for each trait
!!  DIMENSIONS
!!   ncar
!!  NOTES
!!   nombre de qtl en interaction definit pour le caractere
!!***
  integer          , dimension (:),allocatable                   ,private :: nb_qtl_def
!!****v* m_qtlmap_traits/carac_t
!!  NAME
!!   carac_t
!!  DESCRIPTION
!!   buffer array for carac
!!  DIMENSIONS
!!   ncar
!!  NOTES
!!   nombre de qtl en interaction definit pour le caractere
!!***
  character(len=LEN_DEF)          , dimension (:),allocatable     ,public :: carac_t
!!****v* m_qtlmap_traits/natureY_t
!!  NAME
!!   natureY_t
!!  DESCRIPTION
!!   buffer array for natureY
!!  DIMENSIONS
!!   ncar
!!  NOTES
!!***
  character(len=1)               ,  dimension (:),allocatable     ,public :: natureY_t
!!****d* m_qtlmap_traits/MAX_QTL
!!  NAME
!!   MAX_QTL
!!  DESCRIPTION
!!   Maximum number of interaction
!!  NOTES
!!   nombre maximum possible de qtl en interaction definit dans le fichier model
!!***
  integer                    , parameter, public                          :: MAX_QTL=10
!!****v* m_qtlmap_traits/h2_t
!!  NAME
!!   h2_t
!!  DESCRIPTION
!!   buffer array for h2
!!  DIMENSIONS
!!   ncar
!!  NOTES
!!***
  real (kind=dp)                   , dimension (:), allocatable   ,public :: h2_t
!!****v* m_qtlmap_traits/RhoP_t
!!  NAME
!!   RhoP_t
!!  DESCRIPTION
!!   buffer array for RhoP
!!  DIMENSIONS
!!   ncar,ncar
!!  NOTES
!!***
  real (kind=dp)                 , dimension (:,:), allocatable   ,public :: RhoP_t
!!****v* m_qtlmap_traits/RhoG_t
!!  NAME
!!   RhoG_t
!!  DESCRIPTION
!!   buffer array for RhoG
!!  DIMENSIONS
!!   ncar,ncar
!!  NOTES
!!***
  real (kind=dp)                 , dimension (:,:), allocatable   ,public :: RhoG_t

  !-----------
  !DATA PERF :
  !-----------
!!****v* m_qtlmap_traits/niv
!!  NAME
!!   niv
!!  DESCRIPTION
!!   level of the fixed effect fe for a progeny kd
!!  DIMENSIONS
!!   nd,nfix
!!  NOTES
!!***
  character(len=LEN_DEF)         , dimension (:,:),allocatable   ,private :: niv
!!****v* m_qtlmap_traits/cov
!!  NAME
!!   cov
!!  DESCRIPTION
!!   value of the covariate cov for a progeny kd
!!  DIMENSIONS
!!   nd,ncov
!!  NOTES
!!***
  real (kind=dp)          , dimension (:,:), allocatable         ,private :: cov
!!****v* m_qtlmap_traits/cdt
!!  NAME
!!   cdt
!!  DESCRIPTION
!!   value of censured data for a trait ic/progeny kd
!!  DIMENSIONS
!!   nc,nbete
!!  NOTES
!!***
  real (kind=dp)          , dimension (:,:), allocatable         ,private :: cdt
!!****v* m_qtlmap_traits/valeur
!!  NAME
!!   valeur
!!  DESCRIPTION
!!   phenotypic value for a progeny kd
!!  DIMENSIONS
!!   nc,nbete
!!  NOTES
!!***
  real (kind=dp)          , dimension (:,:), allocatable         ,private :: valeur
!!****v* m_qtlmap_traits/ndelt
!!  NAME
!!   ndelt
!!  DESCRIPTION
!!   validity of a phenotypic value for a progeny kd
!!  DIMENSIONS
!!   nc,nbete
!!  NOTES
!!***
  integer           , dimension (:,:),allocatable                ,private :: ndelt


  !END DATA PERF
  !-------------

  public :: read_traits
  public :: read_model
  public :: fixe_structure_model
  public :: manage_simulator_traits
  public :: sim_QTL
  public :: write_perf
  public :: set_estime
  public :: normalize_data
  public :: set_count_discrete
  public :: set_proportion_discrete
  public :: manage_data



 ! public :: normal

contains

!!****f* m_qtlmap_traits/read_traits
!!  NAME
!!    read_traits
!!  DESCRIPTION
!!    Read the user phenotypic files and the model file
!!  INPUTS
!!   type_data_transcr   : true => read the transcriptome format (animal defined by column), otherwise read as classical file (animal defined by line)
!!   filter              : vector of index to select a subset of trait defined in the model file
!!   calculCd            : infer the censored data
!!  NOTES
!!  SOURCE
      subroutine read_traits(type_data_transcr,filter,calculCd)
       logical,intent(in)          :: type_data_transcr
       integer, dimension(:),intent(in) :: filter
       logical   ,optional  ,intent(in) :: calculCd

       character(len=LENGTH_MAX_FILE), dimension(size(in_perfs)) :: traits_files
       character(len=LENGTH_MAX_FILE)                            :: model_file
       integer :: i,j

       character(len=LEN_DEF) :: temp
       traits_files = in_perfs
       model_file   = in_param_ef
       call log_mess('SUBROUTINE read_traits',DEBUG_DEF)

       if (type_data_transcr) then ! transcriptome information ...
         call read_perf_by_line(traits_files,filter)
       else
         call read_perf_by_column(traits_files,filter)
       end if
       !************* NCAR ACCORDING TO FILTER *********
       if ( size(filter)/=0 ) ncar = size(filter)

       if ( present(calculCd) ) then
         if ( calculCd ) then
          do i=1,size(bete)
           call init_perf_animal(datasetUser,bete(i),valeur(:,i),cdt(:,i),ncar)
          end do
        end if
       end if

       call init_model_struct()
       call initialise_struct_internal()
       call check_traits_and_fathers()
       call set_estime()

       deallocate (ndelt)
       deallocate (cdt)
       deallocate (valeur)
       deallocate (niv)
       deallocate (cov)
       deallocate (nuis)
       deallocate (int_qtl)

       call log_mess('END SUBROUTINE read_traits',DEBUG_DEF)
      end subroutine read_traits
!!***

!!****f* m_qtlmap_traits/read_model
!!  NAME
!!    read_model
!!  DESCRIPTION
!!    Read the model file
!!  INPUTS
!!   file            : model file name
!!   filter_car      : vector of index to select a subset of trait defined in the model file
!!  NOTES
!!  SOURCE
      subroutine read_model(file,filter_car)
       character(len=LENGTH_MAX_FILE),intent(in) :: file
       integer , dimension(:),   pointer ,optional     :: filter_car
       integer                     :: ios,alloc_stat
       character(len=LEN_LINE)            ::line_read,line_orig,saveLineRead
       character(len=LEN_DEF)        :: word,word2
       integer                     :: i,j,l,k,ii,ncar2,iqtl,ic
       logical                     :: is_ok,all_mode,interQtl
       character(len=LEN_DEF),dimension(:),allocatable :: lcar,carac_t2,natureY_t2
       integer,dimension(:,:),allocatable :: int_qtl_t2,nuis_t2

       call log_mess('SUBROUTINE read_model',DEBUG_DEF)
       call log_mess('reading model file...',INFO_DEF)
       open(unit_mod,FILE=file,IOSTAT=ios)

       if ( ios /= 0 ) then
          call stop_application('Cannot open model file ['//trim(file)//']')
       endif

       call log_mess('Model description....',VERBOSE_DEF)

       read(unit_mod,*,IOSTAT=ios) ncar
       if (ios /= 0) then
          call stop_application('Cannot read number of trait ['//trim(file)//']' &
          //' line:1')
       end if

       call log_mess('Number of traits            :'//trim(str(ncar)),VERBOSE_DEF)

       read(unit_mod,*,IOSTAT=ios) nfix,ncov
        if (ios /= 0) then
          call stop_application('Cannot read number of fixed effect and number of covariate ['//trim(file)//']' &
          //' line:2')
       end if
       call log_mess('Number of fixed effects     :'//trim(str(nfix)),VERBOSE_DEF)
       call log_mess('Number of covariates        :'//trim(str(ncov)),VERBOSE_DEF)

       !Lecture des cov et effets fixe
       CALL GET(unit_mod,line_read,IOSTAT=ios)
       if ( ios /= 0) then
        call stop_application('give name of fixed effect and covariate in the file ['//trim(file)//']' &
          //' line:3'//' by default write "none"')
       endif

       allocate (namefix(nfix))
       allocate (namecov(ncov))

       l = 3
       do i=1,nfix
          namefix(i) = trim(next_word(line_read,is_ok))
          if ( .not. is_ok ) call stop_on_error (1,file,l,'fixed effect name ['//trim(str(i))//']')
          call log_mess('Fixed effect   :'//trim(trim(namefix(i))),VERBOSE_DEF)
       end do
       do i=1,ncov
          namecov(i) = next_word(line_read,is_ok)
          if ( .not. is_ok ) call stop_on_error (1,file,l,'covariate name ['//trim(str(i))//']')
          call log_mess('Covariate      :'//trim(trim(namecov(i))),VERBOSE_DEF)
       end do

       !lecture des caracteres
       !----------------------
       ALLOCATE (carac_t(ncar), stat = alloc_stat)
       CALL check_allocate(alloc_stat,'carac')
       ALLOCATE (nuis_t(ncar,nfix+ncov), stat = alloc_stat)
       CALL check_allocate(alloc_stat,'nuis')
       ALLOCATE (int_qtl_t(ncar,MAX_QTL,nfix), stat = alloc_stat)
       CALL check_allocate(alloc_stat,'int_qtl')
       ALLOCATE (nb_qtl_def(ncar))
       nb_qtl_def=0
       allocate(natureY_t(ncar), stat = alloc_stat)
       call check_allocate(alloc_stat,'natureY')

       all_mode = .false.
       ncarcat = 0
       do i=1,ncar
          !for each trait we read the model description
          l = l+1
          call GET(unit_mod,line_read,IOSTAT=ios)

          if ( ios /= 0) then
              call stop_application('none description trait find for the'// trim(str(i))//' trait ' &
          //' line:'//trim(str(l))//'model file:['//trim(file)//']')
          end if
          word = next_word(line_read,is_ok)
          if ( .not. is_ok ) call stop_on_error (1,file,l,'trait name')

          !! ALL is a generic name for apply model on all traits
          if (word == ALL_LABEL_MODEL) then
             call log_mess('Keywords ['//ALL_LABEL_MODEL//'] detected. apply the description for each trait',INFO_DEF)

             do j=i,ncar
               word2='Trait_'//trim(str(i))
               carac_t(i) = trim(word2)
             end do
             all_mode = .true.
          end if
          if (.not. all_mode) then
             carac_t(i) = trim(word)
          else
            do j=i,ncar
             carac_t(j) = 'TRAIT_'//trim(str(j))
            end do
          end if

          word = trim(next_word(line_read,is_ok))

          if ( .not. is_ok ) call stop_on_error (1,file,l,'Trait nature')

          if ( word /= 'r' .and. word /= 'i' .and. word /= 'c' .and.word /= 'a') then
              call stop_application('Trait nature have to be r(real), i(integer), c(categorial) or a(average)'//&
              ' token:'//trim(word))
           end if

           if ( .not. all_mode ) then
             natureY_t(i)=trim(word)
           else
             do j=i,ncar
               natureY_t(j)=trim(word)
             end do
           end if

           if ( .not. all_mode ) then
             if ( natureY_t(i) == 'c') ncarcat = ncarcat +1
           else

             do j=i,ncar
               if ( natureY_t(j) == 'c') ncarcat = ncarcat +1
             end do
           end if

          do j=1,(nfix+ncov)
             word = trim(next_word(line_read,is_ok))
             if ( .not. is_ok ) call stop_on_error (1,file,l,'model for trait ['//trim(carac_t(i))//']')

             nuis_t (i,j) = get_int(word,is_ok)

             if ( .not. is_ok ) call stop_on_error (1,file,l,'possible value for model : 0,1 [val:'//trim(word)//']')
             !ALL mode
             if (all_mode) then
                do k=i+1,ncar
                  nuis_t (k,j) = nuis_t(i,j)
                end do
             end if
          end do

          iqtl=0
          interQtl=(nfix/=0)
          int_qtl_t=0

          do while (interQtl)
           saveLineRead=line_read
           word = trim(next_word(saveLineRead,is_ok))
           if ( is_ok ) then
            iqtl=iqtl+1
            if ( MAX_QTL < iqtl ) call stop_on_error(1,file,l,&
                  "You can not define more than ["//trim(str(MAX_QTL))//"] qtl interaction.")
            do j=1,nfix
               word = trim(next_word(line_read,is_ok))
               if ( .not. is_ok ) call stop_on_error (1,file,l,'possible value for model : 0,1 [val:'//trim(word)//'] numQTL '//&
               '['// trim(str(iqtl))//']')

               int_qtl_t (i,iqtl,j) = get_int(word,is_ok)
               if ( .not. is_ok ) call stop_on_error (1,file,l,'possible value for model : 0,1 [val:'//trim(word)//'] numQTL '//&
               '['// trim(str(iqtl))//']')
               !ALL mode

               if (all_mode) then
                do k=i+1,ncar
                  int_qtl_t (k,iqtl,j) = int_qtl_t(i,iqtl,j)
                end do
               end if
           end do
           else
            interQtl = .false.
           end if
          end do

          if (all_mode) then
           nb_qtl_def = iqtl
          else
           nb_qtl_def(i) = iqtl
          end if

          call log_mess('Description of ['//trim(carac_t(i))//']... ok',VERBOSE_DEF)

          if (all_mode) then
            exit ! go out
          end if
       end do

      !Lecture des heritabilite et des matrices de correlations genetique et phenotypique
      ! on lit cette matrice si la ligne "matrice_correlation" existe
      allocate(h2_t(ncar))
      allocate(RhoP_t(ncar,ncar))
      allocate(RhoG_t(ncar,ncar))

      h2_t=0.5d0
      RhoP_t=0.5d0
      RhoG_t=0.5d0

      ios=0
      line_read=''
      do while( ios == 0 .and. (trim(line_read)=='') )
         call GET(unit_mod,line_read,IOSTAT=ios)
      end do
      saveLineRead=line_read
      word = trim(next_word(line_read,is_ok))

      if ( trim(word) == "correlation_matrix" ) then
         ios=0
         ic=0
         do while( ios == 0 .and. ic < ncar )
            call GET(unit_mod,line_read,IOSTAT=ios)
            if (ios /= 0) cycle
            if (trim(line_read) =='' ) cycle
            saveLineRead=line_read
            ic=ic+1
            do i=1,ncar
              !Heritability
              if ( ic == i ) then
                 word = trim(next_word(line_read,is_ok))
                 if ( .not. is_ok ) then
                   call stop_application("Bad definition of heritability :"//trim(saveLineRead))
                 end if
                 h2_t(ic) = get_real(word,is_ok)
                 if ( .not. is_ok ) then
                   call stop_application("Bad definition of heritability :"//trim(saveLineRead))
                 end if

                  !checking heritability
                 if (h2_t(ic)>1) then
                   call stop_application("Heritability for the "//trim(str(ic))//" is greater than 1")
                 end if
                 if (h2_t(ic)<0) then
                   call stop_application("Heritability for the "//trim(str(ic))//" is less than 0")
                 end if
              end if
              !Phenotype correlation
              if ( i < ic ) then
                 word = trim(next_word(line_read,is_ok))
                 if ( .not. is_ok ) then
                   call stop_application("Bad definition of Phenotypique correlation :"//trim(saveLineRead))
                 end if
                 RhoP_t(ic,i) = get_real(word,is_ok)
                 if ( .not. is_ok ) then
                   call stop_application("Bad definition of Phenotypique correlation :"//trim(saveLineRead))
                 end if
                 RhoP_t(i,ic) = RhoP_t(ic,i)
              end if

              !Genetic correlation
              if ( i > ic ) then
                 word = trim(next_word(line_read,is_ok))
                 if ( .not. is_ok ) then
                   call stop_application("Bad definition of Phenotypique correlation :"//trim(saveLineRead))
                 end if
                 RhoG_t(ic,i) = get_real(word,is_ok)
                 if ( .not. is_ok ) then
                   call stop_application("Bad definition of Phenotypique correlation :"//trim(saveLineRead))
                 end if
                 RhoG_t(i,ic) = RhoG_t(ic,i)
              end if
            end do
         end do

         if ( ic /= ncar ) then
            print *,' Format : '
            print *,'correlation_matrix'
            print *,'h2[C1]             RhoG[C1,C2]          RhoG[C1,C3] ....'
            print *,'RhoP[C1,C2]        h2[C2]               RhoG[C2,C3] ....'
            print *,'RhoP[C1,C3]        RhoP[C2,C3]          h2[C3] ....'
            print *,' ** '
            call stop_application(" ** Bad definition of Correlation_matrix **")
         end if

         saveLineRead=''
      else
        call log_mess("correlation_matrix keyword are not found. Default value is 0.5 for "//&
                      "heritability and phenotypic,genotypic correlations",WARNING_DEF);

      end if

      ! pour l instant on fait rien si pas de filtre entree
      if (.not. present(filter_car) ) return

      !AJOUT OFI
      ! SI L UTILISATEUR MET UNE LISTE, ON CONSIDERE CETTE LISTE COMME ETANT LA LISTE DES CARACTERES A PRENDRE EN COMPTE DANS L ANALYSE
      ! PAR DEFAUT, donc,ON A ANALYSE TOUS LES CARACTERES DEFINIS
      allocate (lcar(ncar))
      lcar=''
      ncar2=0
      line_read=saveLineRead
      ! Selection of particular trait
      ios=0
!      print *,1,trim(saveLineRead)
!      print *,2,trim(line_read)
!      stop
      do while ( trim(line_read)=='' .and. ios == 0 )
         call GET(unit_mod,line_read,IOSTAT=ios)
      end do

      do while( ios == 0)
         ! liste des caracteres a prendre en consideration lors de l analyse
         line_orig=trim(line_read)
         do while ( trim(line_read)/='' )
           ncar2 = ncar2 + 1
           if (ncar2 > ncar ) then
             call log_mess("Filter trait:"//trim(line_orig),WARNING_DEF)
             ios=-1
             exit
           end if
           lcar(ncar2)=trim(next_word(line_read,is_ok))
         end do
         line_read=''
         do while ( trim(line_read)=='' .and. ios == 0 )
           call GET(unit_mod,line_read,IOSTAT=ios)
         end do
      end do

       if (associated(filter_car)) deallocate(filter_car)
       allocate (filter_car(ncar2))

       if (ncar2 == 0 ) return

       if (ncar2 > ncar) then
         call stop_application ("Too many traits defines in the list trait filter [Model file:"//trim(file)//"]")
       end if

       close(unit_mod)

       do ii=1,ncar2
        do i=1,ncar
          if ( trim(carac_t(i)) == trim(lcar(ii))) exit
        end do
        if ( i > ncar ) then
          !call stop_application("Trait ["//trim(lcar(ii))//"] is not defined in the model file [Model file:"//trim(file)//"]")
          i=get_int(lcar(ii),is_ok)
          if (is_ok) then
           if ( i > ncar .or. i<=0 ) then
              call stop_application("Bad definition of the index trait ["//trim(lcar(ii))//&
              "] in the model file [Model file:"//trim(file)//"]")
           end if

            call log_mess(" ***> get index trait :"//trim(str(i)),INFO_DEF)
            filter_car(ii)=i
          else
           call stop_application("Trait ["//trim(lcar(ii))//"] is not defined in the model file [Model file:"//trim(file)//"]")
          end if

        else
          filter_car(ii)=i
        end if

       end do

       deallocate (lcar)

       call log_mess('END SUBROUTINE read_model',DEBUG_DEF)
      end subroutine  read_model
!!***


!!****f* m_qtlmap_traits/fixe_structure_model
!!  NAME
!!    fixe_structure_model
!!  DESCRIPTION
!!    Initialize the general array : carac,nuis,int_qtl,natureY,h2,RhoP,RhoG and release buffer array associated
!!  INPUTS
!!   filter      : vector of index to select a subset of trait defined in the model file
!!  NOTES
!!  SOURCE
      subroutine fixe_structure_model(filter)
        integer, dimension(:),intent(in) :: filter ! index filter to initialize model structure
        integer :: i,nc,j
        nc = size(filter)
        if (  nc /= 0 ) then
          allocate (carac(nc))
          allocate (nuis(nc,nfix+ncov))
          allocate (int_qtl(nc,MAX_QTL,nfix))
          allocate (natureY(nc))

          allocate (h2(nc))
          allocate (RhoP(nc,nc))
          allocate (RhoG(nc,nc))

          do i=1,nc
            carac(i)=carac_t(filter(i))
            nuis(i,:)=nuis_t(filter(i),:)
            int_qtl(i,:,:)=int_qtl_t(filter(i),:,:)
            natureY(i)=natureY_t(filter(i))
            h2(i) = h2_t(filter(i))

            do j=i+1,nc
              RhoP(i,j) = RhoP_t(filter(i),filter(j))
              RhoP(j,i) = RhoP(i,j)
              RhoG(i,j) = RhoG_t(filter(i),filter(j))
              RhoG(j,i) = RhoG(i,j)
            end do
          end do

        else
          allocate (carac(ncar))
          allocate (nuis(ncar,nfix+ncov))
          allocate (int_qtl(ncar,MAX_QTL,nfix))
          allocate (natureY(ncar))
          allocate (h2(ncar))
          allocate (RhoP(ncar,ncar))
          allocate (RhoG(ncar,ncar))

          carac=carac_t
          nuis=nuis_t
          int_qtl=int_qtl_t
          natureY=natureY_t
          h2=h2_t
          RhoP=RhoP_t
          RhoG=RhoG_t
        end if

       deallocate(carac_t)
       deallocate(nuis_t)
       deallocate(int_qtl_t)
       deallocate(natureY_t)
       deallocate (h2_t)
       deallocate (RhoP_t)
       deallocate (RhoG_t)

      end subroutine fixe_structure_model
!!***


!!****f* m_qtlmap_traits/init_model_struct
!!  NAME
!!    init_model_struct
!!  DESCRIPTION
!!    fill the array model and listModelTrait variable
!!  NOTES
!!   * the array model is used by the m_qtlmap_modlin routines and derived
!!   * the listModelTrait variable is used by the m_qtlmap_incidence routines and derived
!!  SOURCE
    subroutine init_model_struct()
       integer                     :: alloc_stat
       integer                     :: i,j,ic,k,nf,nc,ifix,ico,nqf,nl,il,kc,iqtl

       call log_mess('init_model_struct',INFO_DEF)

       allocate (modele(ncar,3+(3*nfix)+ncov), stat = alloc_stat)
       call check_allocate(alloc_stat,'model')
       allocate (listModelTrait(ncar))

       do ic=1,ncar
          allocate ( listModelTrait(ic)%indexFixedEffect(nfix))
          nf=0
          ! Pour les effets fixes et covariables
          do ifix=1,nfix
            if(nuis(ic,ifix) == 1) then
              nf=nf+1
              modele(ic,3+nf)=ifix
              listModelTrait(ic)%indexFixedEffect(nf)=ifix
            end if
          end do
          listModelTrait(ic)%nbfe = nf
          modele(ic,1)=nf

          allocate ( listModelTrait(ic)%indexCovariate(ncov))
          nc=0
          do ico=1,ncov
            if(nuis(ic,nfix+ico) == 1) then
              nc=nc+1
              modele(ic,3+nf+nc)=ico
              listModelTrait(ic)%indexCovariate(nc)=ico
            end if
          end do
          modele(ic,2)=nc
          listModelTrait(ic)%nbco = nc
          ! Pour les interactions effets fixes, QTL
          nqf=0

          if ( nb_qtl_def(ic) >= 1 ) then
           do ifix=1,nfix
            if(int_qtl(ic,1,ifix) == 1) then
              nqf=nqf+1
              modele(ic,3+nf+nc+nqf)=ifix
            end if
           end do
          end if

          modele(ic,3)=nqf

          allocate (listModelTrait(ic)%nbint(MAX_QTL))
          listModelTrait(ic)%nbint=0
          allocate ( listModelTrait(ic)%indexFixedEffectWithInteraction(nb_qtl_def(ic),nfix))
          do iqtl=1,nb_qtl_def(ic)
             nqf = 0
             do ifix=1,nfix
                  if(int_qtl(ic,iqtl,ifix) == 1) then
                    nqf=nqf+1
                    listModelTrait(ic)%indexFixedEffectWithInteraction(iqtl,nqf)=ifix
                  end if
              end do
              listModelTrait(ic)%nbint(iqtl)=nqf
          end do

       end do

       deallocate (nb_qtl_def)

      end subroutine init_model_struct
!!***


!!****f* m_qtlmap_traits/initialise_struct_internal
!!  NAME
!!    initialise_struct_internal
!!  DESCRIPTION
!!    allocation of main array of dataset user : corperf,niveau,covar,y,ycategorial,presentc,ndelta,cd
!!  NOTES
!!
!!  SOURCE
    subroutine initialise_struct_internal()
       integer                     :: alloc_stat
       integer                     :: i,j,ic,k,nf,nc,ifix,ico,nqf,nl,il,icCategorial,kc,iqtl
       !dim : nd
       integer , dimension (:)  ,allocatable    :: nbvx
       !dim: ndx,nfix
       character(len=LEN_DEF) , dimension (:,:),allocatable   :: niveau

       call log_mess('initialise_struct_internal model and performance',INFO_DEF)

       allocate (corperf(nd), stat = alloc_stat)
       call check_allocate(alloc_stat,'corperf')

       allocate (niveau(nd,nfix), stat = alloc_stat)
       call check_allocate(alloc_stat,'niveau')

       allocate (covar(nd,ncov), stat = alloc_stat)
       call check_allocate(alloc_stat,'covar')

       allocate (y(ncar,nd), stat = alloc_stat)
       call check_allocate(alloc_stat,'y')

       if ( ncarcat /=0 ) then
         allocate (ycategorial(ncarcat,nd), stat = alloc_stat)
         call check_allocate(alloc_stat,'ycategorial')
       end if

       allocate (presentc(ncar,nd), stat = alloc_stat)
       call check_allocate(alloc_stat,'presentc')

       allocate (ndelta(ncar,nd), stat = alloc_stat)
       call check_allocate(alloc_stat,'ndelta')

       allocate (cd(ncar,nd), stat = alloc_stat)
       call check_allocate(alloc_stat,'cdt')

       y=REAL_NOT_DEFINED
       presentc=.false.
       covar=REAL_NOT_DEFINED
       niveau=STRING_NOT_DEFINED
       corperf=INT_NOT_DEFINED
       cd=0.d0
!---------------------- corperf et presentc
        !for each descendant
        do i=1,nd
           !find the animal associated
           do j=1,size(bete)
             if ( animal(i) == bete(j) ) then
               exit ! exit from this do
             endif
           end do

           if ( j >= size(bete)+1 ) then  !we does not find the animal
              cycle ! next descendant
           else ! animal find....
               corperf(i)=j
               icCategorial = 0
               do ic=1,ncar
                 if (natureY(ic) == 'c') icCategorial = icCategorial + 1
                 cd(ic,i)=cdt(ic,j)
                 ndelta(ic,i)=ndelt(ic,j)
                 if(cdt(ic,j) /= 0.d0) then
                   if (natureY(ic) == 'c') then
                      ycategorial(icCategorial,i)=str(valeur(ic,j))
                   else
                      y(ic,i)=valeur(ic,j)
                   end if
                   presentc(ic,i)=.true.
                 endif
               end do

               do ifix=1,nfix
                 niveau(i,ifix)=niv(j,ifix)

               ! un phenotype manquant est considere lorsque l'effet fixe,
               ! considere dans le modele, est non renseigne

                if (niveau(i,ifix) == STRING_NOT_DEFINED) then
                  do ic=1, ncar
                    do k=1, modele(ic,1)
                      if(modele(ic,3+k) == ifix) then
                        presentc(ic,i)=.false.
                      end if
                    enddo
                    do k=1, modele(ic,3)
                      if(modele(ic,3+modele(ic,1)+modele(ic,2)+k)==ifix) then
                        presentc(ic,i)=.false.
                      end if
                    enddo
                  enddo
                endif
               end do

                do ico=1,ncov
                  covar(i,ico)=cov(j,ico)

              !  un phenotype manquant est considere lorsque la covariable
              !  est non renseignee

                 if (covar(i,ico) == REAL_NOT_DEFINED) then
                    do ic=1, ncar
                      do k=1, modele(ic,2)
                        if (modele(ic,3+modele(ic,1)+k) == ico) then
                         presentc(ic,i)=.false.
                      endif
                   enddo
                 enddo
               endif
           end do
           end if
        end do
         !-----------------------------------------------------------------------------
         ! DET DU NOMBRE DE NIVEAUX DES EFFETS FIXES
         !-----------------------------------------------------------------------------
       allocate (nlev(nfix), stat = alloc_stat)
       call check_allocate(alloc_stat,'nlev')

       allocate (nbvx(nd), stat = alloc_stat)
       call check_allocate(alloc_stat,'nbvx')

       allocate (nivx(nd,nfix), stat = alloc_stat)
       call check_allocate(alloc_stat,'nivx')

       !call log_mess("DEVEL TODO : listelev can be optimized",WARNING_DEF)
       allocate (listelev(nfix,nd), stat = alloc_stat)
       call check_allocate(alloc_stat,'listelev')

        do ifix=1,nfix
            nl=0
            nlev(ifix)=0
            do i=1,nd
              nbvx(i)=0
            enddo
     !Comptage du nombre de fois qu'un niveau est rencontre pour le premier i et mise a
     ! 0 pour les autres
             do i=1,nd
               if (niveau(i,ifix) /= STRING_NOT_DEFINED) then
                 if (nbvx(i) == 2) then
                   nbvx(i)=0
                   cycle
                 else
                   nbvx(i)=1
                   do j=i+1,nd
                     if (niveau(i,ifix) == niveau(j,ifix)) then
                       nbvx(i)=nbvx(i)+1
                       nbvx(j)=2
                     end if
                    end do
                 end if
               end if
              end do

             do i=1,nd
               if ( nbvx(i) /= 0 ) then
                 nl=nl+1
                 listelev(ifix,nl)= niveau(i,ifix)
               endif
             enddo

            nlev(ifix)=nl

! affectation des differents niveaux d'effets fixes renumerotes de 1 a nl
             do i=1,nd
               if (niveau(i,ifix) == STRING_NOT_DEFINED) nivx(i,ifix)=0
               do il=1,nlev(ifix)
                 if (listelev(ifix,il) == niveau(i,ifix)) nivx(i,ifix)=il
               enddo
             enddo
        enddo

       deallocate(niveau)
       deallocate(nbvx)
       call log_mess('END SUBROUTINE initialise_struct_internal',DEBUG_DEF)
     end subroutine  initialise_struct_internal
!!***


!!****f* m_qtlmap_traits/read_perf_by_line
!!  NAME
!!    read_perf_by_line
!!  DESCRIPTION
!!    read the phenotypic file with the expression quantitative trait value format :
!!    The header line is the list of animals phenotyped. The following line are the fixed effects, covariates and finally the phenotype.
!!    The format of the nuisances effects and phenotype line is  :
!!    <IDANIMAL> <FIXED_EFFECT1> <FIXED_EFFECT2>...<COV1> <COV2>...<VALUE1> <VALUE2>...
!!    For missing data, insert a character string which is not interpretable as a numeric(e.g. n/a).
!!    The following array are filling : bete, niv, cov, valeur, cdt, ndelt
!!
!!  INPUTS
!!   files  : list of phenotypic files
!!   filter : vector of index to select a subset of trait defined in the model file
!!  NOTES
!!
!!  SOURCE
    subroutine read_perf_by_line(files,filter)
       character(len=LENGTH_MAX_FILE), dimension(MAX_FILES_INPUT), intent(in) :: files
       integer, dimension(:),intent(in) :: filter
       integer                     :: ios,alloc_stat
       integer                     :: nbete,current_number,l,i,j,futureNcar,k,ii
       character(len=LEN_DEF)      :: token
       character(len=LEN_LINE)     :: line_read
       logical                     :: is_ok,finloop
       character(len=1)            :: buff_read
       character(len=LENGTH_MAX_FILE) :: file
       character(len=LEN_BUFFER_WORD) :: name
       character(len=LEN_BUFFER_WORD),dimension(:),allocatable :: bete_char
       character(len=LEN_DEF) ,dimension(:),allocatable :: niv_t
       character(len=LEN_DEF) ,dimension(:,:),allocatable ::locvaleur
       real(kind=dp) ,dimension(:),allocatable :: cov_t
       real(kind=dp)                  :: v

       call log_mess('SUBROUTINE read_perf_by_line',DEBUG_DEF)
       call log_mess('reading transcriptom data file...',INFO_DEF)
       nbete = 0
       futureNcar = ncar
       if (size(filter) /=0 ) futureNcar= size(filter)

       call log_mess(' ** transcriptomic file : do not write number of animal at the first line ** ',WARNING_DEF)

       file = files(1)
       if  (ncar <= 0 .or. nfix < 0 .or. ncov < 0 ) then
            call stop_application('Devel error: call read_model before read_traits')
       end if

       open(unit_perf,FILE=file,IOSTAT=ios,FORM="formatted",recl=2**20)

       if ( ios /= 0 ) then
          call stop_application('The application can not open the traits file ['//trim(file)//']')
       endif

       line_read = ''
       l = 1
       ios = 0

       ! blank line....
       do while ( (ios == 0) .and. trim(line_read)=='')
          l = l + 1
          call GET(unit_perf,line_read,IOSTAT=ios)
       end do

       if ( ios /= 0 ) then
          call stop_application('The application can not read the first line ['//trim(file)//'].'// &
              ' Split the file to resolve this error')
       endif

       is_ok = .true.

       do while ( is_ok )
           token = next_word(line_read,is_ok)
           nbete = nbete + 1
       end do

       nbete = nbete -1

       rewind(unit_perf)

       call log_mess('Number of animals detected:'//trim(str(nbete)),VERBOSE_DEF)

       allocate (bete(nbete), stat = alloc_stat)
       call check_allocate(alloc_stat,'bete')
       allocate (bete_char(nbete), stat = alloc_stat)
       call check_allocate(alloc_stat,'bete_char')
       allocate (niv(nbete,nfix), stat = alloc_stat)
       call check_allocate(alloc_stat,'niv')

       allocate(niv_t(nbete))

       allocate (cov(nbete,ncov), stat = alloc_stat)
       call check_allocate(alloc_stat,'cov')

       allocate (cov_t(nbete), stat = alloc_stat)

       allocate (locvaleur(futureNcar,nbete), stat = alloc_stat)
       allocate (valeur(futureNcar,nbete), stat = alloc_stat)
       call check_allocate(alloc_stat,'locvaleur')
       allocate (cdt(futureNcar,nbete), stat = alloc_stat)
       call check_allocate(alloc_stat,'cdt')
       allocate (ndelt(futureNcar,nbete), stat = alloc_stat)
       call check_allocate(alloc_stat,'ndelt')

       l = 1
       ! animal name header
       read(unit_perf,*,iostat=ios) (bete_char(i),i=1,nbete)
       bete = bete_char
       if ( ios /= 0 ) then
           call log_mess('Problem detected at the line:'//trim(str(l)),ERROR_DEF)
           call stop_application('No header (animals id) are finding in traits file:'//file)
       end if

       i = 1
       do while ( i <= nfix )
        l = l + 1
        read(unit_perf,*,iostat=ios) name,(niv_t(j),j=1,nbete)
        if ( trim(name) == '') cycle
        if ( ios /= 0 ) then
           call log_mess('Problem detected at the line:'//trim(str(l)),ERROR_DEF)
           call stop_application('The application can not initialize the fixed effect ['//&
           trim(str(i))//']  file:['//trim(file)//'].')
        end if
        !on cherche l effet fixe correspondant
        do j=1,nfix
          if ( trim(namefix(j))==trim(name)) then
            exit
          end if
        end do

        if ( j<=nfix) then
          niv(:,j)=niv_t(:)
        else
          call log_mess("QTLMap do not use fixed effect :"//trim(name),WARNING_DEF)
        end if

        i=i+1
       end do

       i = 1
       do while ( i <= ncov )
        l = l + 1
        read(unit_perf,*,iostat=ios) name,(cov_t(j),j=1,nbete)
        if ( trim(name) == '') cycle
        if ( ios /= 0 ) then
           call log_mess('Problem detected at the line:'//trim(str(l)),ERROR_DEF)
           call stop_application('The application can not initialize the covariate ['//&
           trim(str(i))//']  file:['//trim(file)//'].')
        end if

         !on cherche l effet fixe correspondant
        do j=1,ncov
          if ( trim(namecov(j))==trim(name)) then
            exit
          end if
        end do

        if ( j<=ncov) then
          cov(:,j)=cov_t(:)
        else
          call log_mess("QTLMap do not use Covariate :"//trim(name),WARNING_DEF)
        end if

        i=i+1
       end do

       i = 1
       ii=0
       ios=0
       finloop=.false.
       do while ( i <= ncar .and. (.not. finloop) )
         !count line
         l = l + 1
         call log_mess("setting value with line:"//trim(str(l)),DEBUG_DEF)

         if ( futureNcar /= ncar ) then
            ii=ii+1
            do k=1,size(filter)
                if ( filter(k) == ii ) exit
            end do
            if ( k > size(filter )) then
               name=""
               do while ( ios == 0 .and. trim(name)=="")
                 read(unit_perf,*,iostat=ios) name
               end do
               cycle
            else
              if ( k == size(filter)) finloop=.true.
            end if
         else
            k = i
         end if
         name=""
         ios=0
         do while ( ios == 0 .and. trim(name)=="")
          !reading line
          read(unit_perf,*,iostat=ios) name,(locvaleur(k,j),j=1,nbete)
         end do
         !manage blank line
         if ( trim(name) == '') cycle
         !No enought line defined in the file
         if ( ios /= 0 ) then
           call log_mess('Problem detected at the line:'//trim(str(l)),ERROR_DEF)
           call stop_application('The application can not initialize the trait ['//&
           trim(str(i))//']  file:['//trim(file)//'].')
         end if


         !first colomn : name
         carac(k) = trim(name)

         do j=1,nbete
            valeur(k,j) = get_real(locvaleur(k,j),is_ok)
            if ( .not. is_ok ) then
               call log_mess('** value unknown for trait ['//trim(carac(i))//&
                       '] of animal ['//trim(bete(j))//']',WARNING_DEF)
               cdt(k,j)       = 0
            else
               cdt(k,j)       = 1
            end if
         end do

         ndelt(k,:) = 1

         i = i + 1
       end do

       close(unit_perf)
       deallocate (locvaleur)
       deallocate(bete_char)
       deallocate (niv_t)
       deallocate (cov_t)

       call log_mess('END SUBROUTINE read_perf_by_line',DEBUG_DEF)
    end subroutine
!!***


!!****f* m_qtlmap_traits/read_perf_by_column
!!  NAME
!!    read_perf_by_column
!!  DESCRIPTION
!!    read the phenotypic file with the format :
!!    For each animal, its ID (identical to the ID given in the pedigree file) is followed by information
!!    about nuisance effects (fixed effect levels, covariable value) and then by three information for each trait :
!!      * the performance
!!      * an 0/1 variable IP which indicates if (IP=1) or not (IP=0) the trait was measured for this animal and must be included in the analysis
!!      *  and 0/1 variable (IC) which indicates if (IC=0) it was censored or not (IC=1), this IC information being needed for survival analysis (by default IC=1).
!!
!!    Grammar : <animal_id> <fixed_effect1> <fixed_effect2>... <cov1>..<cov2> <IP> <IC> <VALUE>
!!
!!    The following array are filling : bete, niv, cov, valeur, cdt, ndelt
!!
!!  INPUTS
!!   files  : list of phenotypic files
!!   filter : vector of index to select a subset of trait defined in the model file
!!  NOTES
!!
!!  SOURCE
    subroutine read_perf_by_column(files,filter)
       character(len=LENGTH_MAX_FILE),dimension(MAX_FILES_INPUT),intent(in) :: files
       integer, dimension(:),intent(in) :: filter
       integer                        :: ios,alloc_stat
       integer                        :: nbete
       character(len=LEN_DEF)         :: vs,temp_vs
       character(len=LEN_LINE)        ::line_read
       character(len=LEN_BUFFER_WORD) :: word_token
       integer                     :: l,i,j,k
       logical                     :: is_ok,filterActive
       character(len=LEN_BUFFER_WORD) :: file
       integer :: futureNcar
       real (kind=dp), dimension (:,:), allocatable         :: cdt1,valeur1
       !dim : nc,nbete
       integer , dimension (:,:),allocatable                :: ndelt1
       character(len=LEN_DEF), dimension (:), allocatable   :: temp
       character(len=LEN_DEF), dimension (:,:), allocatable :: temp2
       real(kind=dp), dimension (:,:), allocatable :: temp3

       call log_mess('SUBROUTINE read_perf_by_column',DEBUG_DEF)
       file = files(1)
       open(unit_perf,FILE=file,IOSTAT=ios)
       call log_mess('reading traits file...',INFO_DEF)
       if ( ios /= 0 ) then
          call stop_application('Cannot open the traits file ['//trim(file)//']')
       endif

       futureNcar = ncar
       filterActive=.false.
       if (size(filter) /=0 ) then
         filterActive=.true.
         futureNcar= size(filter)
       end if

       nbete=MAX_ANIMAL
       allocate (bete(nbete), stat = alloc_stat)
       allocate (niv(nbete,nfix), stat = alloc_stat)
       allocate (cov(nbete,ncov), stat = alloc_stat)
       allocate (valeur1(ncar,nbete), stat = alloc_stat)
       allocate (cdt1(ncar,nbete), stat = alloc_stat)
       allocate (ndelt1(ncar,nbete), stat = alloc_stat)

       ios = 0
       nbete = 0
       i=1
       rewind(unit_perf)
       do while ( ios == 0)
        read (unit_perf,*,iostat=ios) bete(i),(niv(i,j),j=1,nfix),(cov(i,j),j=1,ncov),(valeur1(j,i),cdt1(j,i),ndelt1(j,i),j=1,ncar)
           if ( trim(bete(i))/='' .and. ( ios == 0) ) then
                nbete = nbete+1
!                print *,trim(bete(i)),(niv(i,j),j=1,nfix),(cov(i,j),j=1,ncov),(valeur1(j,i),cdt1(j,i),ndelt1(j,i),j=1,ncar)
                i = i + 1
           endif
       end do
       close(unit_perf)

       allocate (valeur(futureNcar,nbete))
       allocate (cdt(futureNcar,nbete))
       allocate (ndelt(futureNcar,nbete))

       do i=1,nbete

!           do j=1,(i-1)
!              if ( bete(i) == bete(j)) then
!                call stop_on_error (1,file,l,'animal ['//trim(bete(i))//'] have two lines definitions of traits !!')
!              end if
!           end do
           do j=1,ncar
!             if ( futureNcar /= ncar ) then
            if ( filterActive ) then
               do k=1,size(filter)
                 if ( filter(k) == j ) exit
               end do
               ! the index car is not used
               if ( k > size(filter)) cycle
             else
                k = j
             end if
             cdt(k,i) = cdt1(j,i)
             valeur(k,i) = valeur1(j,i)
             ndelt(k,i) = ndelt1(j,i)
           end do
       end do
       deallocate (valeur1)
       deallocate (cdt1)
       deallocate (ndelt1)

       !on retasse les tableau bete,niv,cov
       allocate (temp(nbete))
       temp = bete(:nbete)
       deallocate (bete)
       allocate (bete(nbete))
       bete = temp

       allocate (temp2(nbete,nfix))
       temp2 = niv(:nbete,:)
       deallocate (niv)
       allocate (niv(nbete,nfix))
       niv=temp2

       allocate (temp3(nbete,ncov))
       temp3 = cov(:nbete,:)
       deallocate (cov)
       allocate (cov(nbete,ncov))
       cov=temp3

       call log_mess('END SUBROUTINE read_perf_by_column',DEBUG_DEF)
    end subroutine read_perf_by_column
!!***


!!****f* m_qtlmap_traits/check_traits_and_fathers
!!  NAME
!!    check_traits_and_fathers
!!  DESCRIPTION
!!    check if number of progenies is greater than 1 otheriwse (stop the program with an error message)
!!
!!  NOTES
!!
!!  SOURCE
    subroutine check_traits_and_fathers()
        integer :: i,ic,nm1,nm2,nd1,nd2,jm,kd,effp
        call log_mess("check traits of sire family...",INFO_DEF)

        if ( .not. allocated(nmp) ) then
           call stop_application("Dev error : try to read traits file before genealogy initialization...")
        end if

        do ic=1,size(carac)
          do i=1,size(pere)
            nm1=nmp(i)+1
            nm2=nmp(i+1)
            effp=0
            do jm=nm1,nm2
             nd1=ndm(jm)+1
             nd2=ndm(jm+1)
             effp = effp+count(presentc(ic,nd1:nd2))
            end do
            if (effp == 0) then
             call stop_application('Father ['//trim(pere(i))// &
            '] have no child with the trait ['//trim(carac(ic))//']')
          end if
           if (effp == 1) then
            call stop_application('Father ['//trim(pere(i))// &
            '] have only one child with the trait ['//trim(carac(ic))//']')
          end if
          end do
        end do
      call log_mess("end check traits...",INFO_DEF)
    end subroutine check_traits_and_fathers
!!***

!!****f* m_qtlmap_traits/sim_QTL
!!  NAME
!!    sim_QTL
!!  DESCRIPTION
!!
!!
!!  NOTES
!!   Simulation des typages sur trois generations , reperage des alleles au qtl des descendants.
!!  SOURCE
      subroutine sim_QTL(icar,nqtl,pas,qtl,croisement)

      integer                            ,intent(in)    :: icar,nqtl,pas
      !
! Tableaux dimensionnes selon n, le nombre d'individus dans le pedigree
      integer                           ,intent(out)    :: qtl(np+nm+nd,nqtl,2)
      character(len=LEN_BUFFER_WORD) , intent(in) :: croisement
!
!C Divers
      real              :: x_rand
      integer :: geno,iq,ip,ifem,ipos,nm1,nm2,jm,ngeno1,ngeno2,nd1,nd2,chr
      integer :: kd,kkd,iph,n
      real (kind=dp) :: pp,pm
      real,external                        :: ranf

!
!
!***********************************************************************
!                Simulation des alleles au QTL des parents
!
!***********************************************************************
      do iq=1,nqtl
       do ip=1,np
! xlim(iq)=frequence de allele Q2 chez les GP et de Q1 chez les GM
! si xlim(iq)=1, tous GP Q1Q1 et toutes GM Q2Q2
        qtl(ip,iq,1)=2
        qtl(ip,iq,2)=1
        x_rand=ranf()
        if (dble(x_rand).lt.xlim(iq))  qtl(ip,iq,1)=1
        x_rand=ranf()
        if (dble(x_rand).lt.xlim(iq))  qtl(ip,iq,2)=2
       end do
!
       do ifem=1,nfem
! xlim(iq)=frequence de allele Q2 chez les GP et de Q1 chez les GM
! si xlim(iq)=1, tous GP Q1Q1 et toutes GM Q2Q2
        qtl(np+ifem,iq,1)=2
        qtl(np+ifem,iq,2)=1
        if (croisement /= BC_KEYWORD) then
          x_rand=ranf()
          if (dble(x_rand).gt.xlim(iq))  qtl(np+ifem,iq,1)=1
          x_rand=ranf()
          if (dble(x_rand).gt.xlim(iq))  qtl(np+ifem,iq,2)=2
        else
          qtl(np+ifem,iq,2)=2
        end if

       end do
      end do


!
!
!**********************************************************************
!           Simulation de la transmission des alleles QTL des parents
! aux descendants : les estimations de phase des meres sont correctes
! la phase la plus probable est systematiquement transmise
!  ==> simulation de la deviation a la moyenne nulle
!**********************************************************************
!
      do iq=1,nqtl
       chr=chrqtl(iq)
       n=get_pos((posiqtl(iq)-posi(chr,1)))
       if ( n > get_ilong(chr)) n = get_ilong(chr)
       if ( n <= 0 ) n = 1
       do ip=1,np
        nm1=nmp(ip)+1
        nm2=nmp(ip+1)
        do jm=nm1,nm2
         ifem=repfem(jm)
         ngeno1=ngenom(chr,jm)+1
         ngeno2=ngenom(chr,jm+1)
         geno=ngeno1
         nd1=ngend(chr,geno)+1
         nd2=ngend(chr,geno+1)
         do kd=nd1,nd2
            kkd=ndesc(chr,kd)
! Transmission de l'allele paternel: a priori re�oit allele 1 (grand p�re) du p�re
! si pt� transmission pp non d�pass�e, re�oit all�le 2
          qtl(np+nfem+kkd,iq,1)=qtl(ip,iq,1)
        !  pp=(-pdd(chr,kd,1,n)-pdd(chr,kd,2,n)+pdd(chr,kd,3,n)+pdd(chr,kd,4,n))/2.d0+0.5d0
          ! MODIF OFI :  proba qu'on est recu le 2eme allele du pere
          pp=pdd(chr,kd,3,n)+pdd(chr,kd,4,n)
          x_rand=ranf()
          if(dble(x_rand).lt.pp)qtl(np+nfem+kkd,iq,1)=qtl(ip,iq,2)
!
! Transmission de l'allele maternel: a priori re�oit allele 1 (grand p�re) de la m�re
          qtl(np+nfem+kkd,iq,2)=qtl(np+ifem,iq,1)
           ! pm=(-pdd(chr,kd,1,n)+pdd(chr,kd,2,n)-pdd(chr,kd,3,n)+pdd(chr,kd,4,n))/2.d0+0.5d0
          ! MODIF OFI :  proba qu'on est recu le 2eme allele de la mere
          pm=pdd(chr,kd,2,n)+pdd(chr,kd,4,n)
          x_rand=ranf()
          if(dble(x_rand).lt.pm)qtl(np+nfem+kkd,iq,2)=qtl(np+ifem,iq,2)
!
! Mise � jour des perf simul�es sous H0
          
           if (presentc(icar,kkd)) then
            if (qtl(np+nfem+kkd,iq,1).eq.qtl(np+nfem+kkd,iq,2)) then
              if (qtl(np+nfem+kkd,iq,1).eq.1) then
                y(icar,kkd)=y(icar,kkd)+ue(icar,iq)
              else
                y(icar,kkd)=y(icar,kkd)-ue(icar,iq)
              end if
            end if
           end if
         end do
        end do
       end do
      end do
    !  call log_mess('DEPLACER LA GENERATION DU FICHIER PERFOSIM1 SIMULER...',WARNING_DEF)

      return
      end subroutine sim_QTL
!!***


!!****f* m_qtlmap_traits/manage_simulator_traits
!!  NAME
!!    manage_simulator_traits
!!  DESCRIPTION
!!      Manage the type of simulator (depends opt_cal/nature of traits)
!!
!!      4 cases :
!!       1) permutation are switch on : -> permuted data according to the type analysis
!!                          a) MULTITRAIT ANALYSIS -> PERMUTATION BY LINE
!!                          b) UNITRAIT ANALYSIS   -> PERMUTATION on each trait
!!       2) permutation are switch off and analysis is a multitrait
!!                         --> simulation of traits with heritability and correlation matrix (according to the nature)
!!       3) permutation are switch off and analysis is unitrait
!!                         --> simulation of traits with heritability according to the nature of the trait
!!
!!  NOTES
!!   Simulation des typages sur trois generations , reperage des alleles au qtl des descendants.
!!  SOURCE
    subroutine manage_simulator_traits(analyse_is_multi_traits,permute_mode,step,simqtl,nqtl,qtl,croisement)
        logical ,intent(in)                          :: analyse_is_multi_traits ! generaly the simulation
        logical ,intent(in)                          :: permute_mode
        integer,intent(in)                           :: nqtl,step

        integer      ,dimension(np+nfem+nd,nqtl,2)  ,intent(inout)         :: qtl
        character(len=LEN_BUFFER_WORD)  , intent(in)           :: croisement
        logical                         ,intent(in)       :: simqtl

        real (kind=dp),dimension (1)     :: h2_t

        real (kind=dp),dimension (1,size(animal)) :: ys

        real (kind=dp),dimension (1,1)          :: bidonrho
        integer :: i,j,ic
        character(len=1) :: nat

        if (ncar <= 0 ) then
           call stop_application("DEVEL ERROR : NCAR is not initialized")
        end if

        if ( .not. allocated(natureY) ) then
            call stop_application("DEVEL ERROR : NATUREY is not initialized")
        endif
        if ( size(natureY) <= 0  ) then
            call stop_application("DEVEL ERROR : NATUREY is empty")
        end if

        !the permutation mode is active, none nimulator are called
        if ( permute_mode ) then
              call set_estime()
              if ( .not. analyse_is_multi_traits ) then
                  call log_mess("** Permutation for unitrait analysis **",INFO_DEF)
                  call sim_perf_shuffling(.false.)
              else !traits line are permuted between animals
                  call log_mess("** Permutation for multitraits analysis **",INFO_DEF)
                  call sim_perf_shuffling(.true.)
              end if
              return
        end if


       if ( analyse_is_multi_traits .or. all(natureY=='r')) then
          !check the nature of all traits (have to bo the same)
          nat=natureY(1)
          do i=1,size(natureY)-1
                if (natureY(i) /= natureY(i+1)) then
                  call stop_application("You have to defined a model value with the same nature of trait"//&
                  " for a multitrait analysis - [trait "//trim(str(i))//" nature:"//natureY(i)//"] [trait "//&
                  trim(str(i+1))//" nature:"//natureY(i+1)//"]")
                end if
          end do

          if ( nat == 'r' ) then
             call log_mess("** Simulation for real multitrait analysis ** ",INFO_DEF)
             call sim_perf_tirage(ncar,RhoP,h2,y)
             if (simqtl) then
              do ic=1,ncar
               call sim_QTL(ic,nqtl,step,qtl,croisement)
              end do
             end if
          end if

          if ( nat == 'i' ) then
              call log_mess(" ** Simulation for discrete multitrait analysis ** ",INFO_DEF)
              call sim_perf_tirage(ncar,RhoP,h2,y)
              if (simqtl) then
               do ic=1,ncar
                 call sim_QTL(ic,nqtl,step,qtl,croisement)
               end do
              end if
             do i=1,ncar
               call sim_transform_discret(i)
             end do
          end if

          if ( nat == 'c' ) then
              call stop_application("None simulator are defined for categorial data")
          end if

          if ( nat == 'a' ) then
             call log_mess("** Simulation for real multitrait analysis ** ",INFO_DEF)
             call sim_perf_tirage(ncar,RhoP,h2,y)
             do i=1,ncar
              do j=1,size(y,2)
                 if (cd(i,j)/=0) y(i,j)=ys(1,j)/sqrt(cd(i,j))
              end do
             end do

             if (simqtl) then
              do ic=1,ncar
               call sim_QTL(ic,nqtl,step,qtl,croisement)
              end do 
            end if
          end if

       else !otherwise we call for each trait the simulator corresponding to the nature of the traits
          bidonrho=0.d0
          do i=1,ncar
            h2_t(1) = h2(i)
            call log_mess("** Simulator for TRAIT ["//trim(str(i))//"]->Nature:"//trim(natureY(i)//"] **"),INFO_DEF)
            select case (natureY(i))
            case ('r')  !! real / continue value
             call sim_perf_tirage(1,bidonrho,h2_t,ys)
             y(i,:)=ys(1,:)
             if (simqtl) then
              call sim_QTL(i,nqtl,step,qtl,croisement)
             end if
            case ('i')  !! discrete value
             call sim_perf_tirage(1,bidonrho,h2_t,ys)
             y(i,:)=ys(1,:)
             if (simqtl) then
                call sim_QTL(i,nqtl,step,qtl,croisement)
             end if
             call sim_transform_discret(i)
            case ('c')
              call stop_application("None simulator are defined for categorial data")
            case ('a')
              call sim_perf_tirage(1,bidonrho,h2_t,ys)

               do j=1,size(y,2)
                 if (cd(i,j)/=0) y(i,j)=ys(1,j)/sqrt(cd(i,j))
               end do

               if (simqtl) then
                call sim_QTL(i,nqtl,step,qtl,croisement)
               end if

               !call stop_application("None simulator are defined for average data")
             case default
              call stop_application("Unknown nature of data ["//natureY(i)//"]")
            end select
          end do
        end if

!
        call set_estime()

    end subroutine manage_simulator_traits
!!***


!!****f* m_qtlmap_traits/sim_perf_tirage
!!  NAME
!!    sim_perf_tirage
!!  DESCRIPTION
!!
!!  NOTES
!!   Tirage des performances pour deux caracteres correles , avec ou sans qtl
!!  SOURCE
      subroutine sim_perf_tirage(ncar,rho,h2,ys)
      integer , intent(in)                                     :: ncar
      !heritabilities of traits , dim : ncar
      real (kind=dp),dimension (ncar), intent(in)  :: h2
      ! correlation matrix , dim : ncar,ncar
      real (kind=dp),dimension (ncar,ncar) ,intent(in):: rho
      !dim : ncar
      real           ,dimension(ncar)           :: u0
      real (kind=dp) ,dimension(:),allocatable  :: upv,umv,udv,ugpv,ugmv

      !dim ncar,ncar
      real (kind=dp) ,dimension(:,:),allocatable :: varcovg,varcovgd
      real (kind=dp) ,dimension(:,:),allocatable :: varcove
      !dim ncar*ncar
      real           ,dimension(ncar,ncar)       :: ccovvar
      !dim : ncar,nr
      real (kind=dp) ,dimension(:,:),allocatable :: urv
      !dim : nd
      !integer ,dimension(:),allocatable  :: itest

      !dim : ncar,nd
      real (kind=dp) ,dimension(ncar,size(animal)),intent(out) :: ys
!
      ! dim : nrsx=((ncar+1)*(ncar+2))/2)
      real  ,dimension(:),allocatable :: zr,zd,zh,zg

      real  ,dimension(ncar) :: WORK

      ! dim : ncar*(ncar+3)/2 + 1
      real  ,dimension(:),allocatable :: refgg,refg,refe

      integer       :: i,j,ngm1,ngm2,igm,nr1,nr2,ir,nm2
      integer       :: jm,nd1,nd2,kd,ifail,igp,ic,ip,nm1
      real (kind=dp) :: vare,vargd,varg,varej
      character(len=LEN_DEF) :: tempvs

      call log_mess('simulation of traits...',VERBOSE_DEF)

       if (size(rho) <= 0 ) then
         call stop_application("DEVEL ERROR: sim_perf_tirage RHO is not initialized")
      end if

      if (size(h2) <= 0 ) then
         call stop_application("DEVEL ERROR: sim_perf_tirage H2 is not initialized")
      end if

      allocate (varcove(ncar,ncar))
      allocate (varcovg(ncar,ncar))
      allocate (varcovgd(ncar,ncar))
      allocate (upv(ncar))
      allocate (umv(ncar))
      allocate (udv(ncar))
      allocate (ugpv(ncar))
      allocate (ugmv(ncar))
      allocate (urv(ncar,nr))


      allocate (zr( ((ncar+1)*(ncar+2))/2) )
      allocate (zd( ((ncar+1)*(ncar+2))/2) )
      allocate (zh( ((ncar+1)*(ncar+2))/2) )
      allocate (zg( ((ncar+1)*(ncar+2))/2) )

      allocate (refgg(ncar*(ncar+3)/2 + 1))
      allocate (refg(ncar*(ncar+3)/2 + 1))
      allocate (refe(ncar*(ncar+3)/2 + 1))



     ! call log_mess("Simulation trait only for real value",WARNING_DEF);
     ! natureY='r'
!
!*****************************************************************************
!
! Initialisation des matrices de variance covariance residuelles et polygen
!
      do i=1,ncar
        u0(i)=0.d0
        varg = sqrt(h2(i))
        vargd =sqrt(0.5d0*h2(i))
        vare=sqrt(1.d0-h2(i))
        varcovg(i,i)=varg*varg
        varcovgd(i,i)=vargd*vargd
        varcove(i,i)=vare*vare
        do j=i+1,ncar
            varej = sqrt(1.d0-h2(j))
!           varcovg(i,j)=varg(i)*varg(j)
            varcovg(i,j)=0.0d0
            varcovg(j,i)=varcovg(i,j)
!           varcovgd(i,j)=vargd(i)*vargd(j)
            varcovgd(i,j)=0.0d0
            varcovgd(j,i)=varcovgd(i,j)
            varcove(i,j)=rho(i,j)*vare*varej
            varcove(j,i)=varcove(i,j)
        end do
      end do
! 12   format(1x,f10.5,3x,10(f10.5,1x))
!
!
!*****************************************************************************
! Initialisation des vecteurs de tirages des performances dans une binormale
!
      do i=1,ncar
       do j=1,ncar
         ccovvar(i,j) = varcovg(i,j)
       end do
      end do

      call setgmn(u0,ccovvar,ncar,refgg)
!
      do i=1,ncar
       do j=1,ncar
         ccovvar(i,j) = varcovgd(i,j)
       end do
      end do

      call setgmn(u0,ccovvar,ncar,refg)
!
      do i=1,ncar
       do j=1,ncar
         ccovvar(i,j) = varcove(i,j)
       end do
      end do

      call setgmn(u0,ccovvar,ncar,refe)
!
!
!*****************************************************************************
! Tirage des valeurs g�n�tiques des gparents et des reproducteurs
!
      ifail=0
! Grd peres
      do igp=1,ngp
         call genmn(refgg,zg,WORK)
          do ic=1,ncar
            ugpv(ic)=0.5d0*dble(zg(ic))
          end do
!
! Grd meres
          ngm1=ngmgp(igp)+1
          ngm2=ngmgp(igp+1)
          do igm=ngm1,ngm2
             call genmn(refgg,zg,WORK)
             do ic=1,ncar
                ugmv(ic)=0.5d0*dble(zg(ic))
             end do
!
! Reproducteurs
             nr1=nrgm(igm)+1
             nr2=nrgm(igm+1)

             do ir=nr1,nr2
                call genmn(refg,zr,WORK)
                do ic=1,ncar
                   urv(ic,ir)=dble(zr(ic))+ugpv(ic)+ugmv(ic)
                end do
             end do
          end do
       end do
!*****************************************************************************
! Tirage des performances des descendants
!
        do ip=1,np
         if (reppere(ip).eq.9999) call genmn(refgg,zg,WORK)
         do ic=1,ncar
           if (reppere(ip).eq.9999) then
             upv(ic)=0.5d0*dble(zg(ic))
           else
             upv(ic)=0.5d0*urv(ic,reppere(ip))
           end if
         end do
         nm1=nmp(ip)+1
         nm2=nmp(ip+1)
         do jm=nm1,nm2
          if (repmere(jm).eq.9999) call genmn(refgg,zg,WORK)
          do ic=1,ncar
           if (repmere(jm).eq.9999) then
             umv(ic)=0.5d0*dble(zg(ic))
           else
             umv(ic)=0.5d0*urv(ic,repmere(jm))
           end if
          end do
          nd1=ndm(jm)+1
          nd2=ndm(jm+1)
          do kd=nd1,nd2
             call genmn(refg,zd,WORK)
           do ic=1,ncar
             udv(ic)=upv(ic)+umv(ic)+dble(zd(ic))
           end do
!
! Residuelle
           call genmn(refe,zh,WORK)

           do ic=1,ncar
            ys(ic,kd)=udv(ic)+dble(zh(ic))
           end do
        end do
      end do
      end do

      deallocate (urv)
      deallocate (varcove)
      deallocate (varcovg)
      deallocate (varcovgd)
      deallocate (upv)
      deallocate (umv)
      deallocate (udv)
      deallocate (ugpv)
      deallocate (ugmv)

      deallocate (zr)
      deallocate (zd)
      deallocate (zh)
      deallocate (zg)

      deallocate (refgg)
      deallocate (refg)
      deallocate (refe)

      end subroutine sim_perf_tirage
!!***


!!****f* m_qtlmap_traits/sim_transform_discret
!!  NAME
!!    sim_transform_discret
!!  DESCRIPTION
!!
!!  NOTES
!!   dans cette subroutine, on genere les donnees discretes (a mettre dans le tableau y) a partir des valeurs de la sous jacente (trouvees dans le tableau y)
!!  SOURCE
  subroutine sim_transform_discret(ic)
      integer , intent(in)                                     :: ic
      integer     :: kd,m

       do kd=1,size(y,2)
         if(presentc(ic,kd))then
            m=1
            do while (m < nbmodsimultrait(ic) .and. y(ic,kd) > soglia(ic,m))
             m=m+1
            enddo
            y(ic,kd)=m
         end if
      end do

  end subroutine sim_transform_discret
!!***


!!****f* m_qtlmap_traits/sim_perf_shuffling
!!  NAME
!!    sim_perf_shuffling
!!  DESCRIPTION
!!
!!  NOTES
!!     realise un suffling intrafamille:
!!      - de pere et mere lorsque le couple a plus de ndmin descendants
!!      !-de pere, pour tous les descendants des meres ayant moins de ndmin
!!      !descendants avec ce pere (l'ensemble etant > a ndmin)
!!  SOURCE
    subroutine sim_perf_shuffling(analyse_is_multi_traits)

      logical                                     :: analyse_is_multi_traits
      real (kind=dp) ,dimension(:,:),allocatable   :: zy,zcd, zcovar
      integer ,dimension(:,:),allocatable          ::  zndelta,znivx
      integer ,dimension(:),allocatable          ::  effpr,iv
      logical ,dimension(:),allocatable          ::  permut_pere
      logical ,dimension(:,:),allocatable          ::  permut_pere_mere, zpresentc,zpresentg
      integer ,dimension(:,:),allocatable        ::   effpm
      integer                                    :: ifail,ip,jm,kd,nm1,nm2,nd1,nd2,i,j,id,nc, npresp
      integer                                   :: alloc_stat

       ALLOCATE (zy(size(carac),nd), stat = alloc_stat)
       CALL check_allocate(alloc_stat,'zy')
        ALLOCATE (znivx(nd,nfix), stat = alloc_stat)
       CALL check_allocate(alloc_stat,'znivx')
        ALLOCATE (zcovar(nd,ncov), stat = alloc_stat)
       CALL check_allocate(alloc_stat,'zcovar')
      ALLOCATE (zndelta(size(carac),nd), stat = alloc_stat)
       CALL check_allocate(alloc_stat,'zndelta')
       ALLOCATE (zcd(size(carac),nd), stat = alloc_stat)
       CALL check_allocate(alloc_stat,'zcd')
       ALLOCATE (zpresentc(ncar,nd), stat = alloc_stat)
       CALL check_allocate(alloc_stat,'zpresentc')
       ALLOCATE (permut_pere(np), stat = alloc_stat)
       CALL check_allocate(alloc_stat,'permut_pere')
       ALLOCATE (zpresentg(nchr,nd), stat = alloc_stat)
       CALL check_allocate(alloc_stat,'zpresentg')
       ALLOCATE (permut_pere_mere(np,nm), stat = alloc_stat)
       CALL check_allocate(alloc_stat,'permut_pere_mere')
       ALLOCATE (effpr(np), stat = alloc_stat)
       CALL check_allocate(alloc_stat,'effpr')
       ALLOCATE (iv(nd), stat = alloc_stat)
       CALL check_allocate(alloc_stat,'iv')

       ALLOCATE (effpm(np,nm), stat = alloc_stat)
       CALL check_allocate(alloc_stat,'effpm')

       call log_mess('simulation of traits by permutation (multitrait mode)...',VERBOSE_DEF)
!*****************************************************************************
!1�boucle sur les peres mere descendants
!   --> comptage du nombre de descendants par couple pere-mere: effpm(ip,jm)
!   --> comptage du nombre minimum de descendants d'un pere parmi
!   les couple ayant plus de nb_des_min descendants: si ce nbre est inf�rieur � ndim
!   alors ont donne un message d'erreur � l'execution


        zy=0.d0
        zndelta=0
        do ip=1,np
          permut_pere(ip)=.false.
          effpr(ip)=0
          nm1=nmp(ip)+1
          nm2=nmp(ip+1)
          do jm=nm1,nm2
           effpm(ip,jm)=0
            nd1=ndm(jm)+1
            nd2=ndm(jm+1)
	    permut_pere_mere(ip,jm)=.true.
            do kd=nd1,nd2
              zy(:,kd)=y(:,kd)
              zndelta(:,kd)= ndelta(:,kd)
              zcd(:,kd)=cd(:,kd)
	      zpresentc(:,kd)=presentc(:,kd)
              zpresentg(:,kd)=presentg(:,kd)
	      znivx(kd,:)=nivx(kd,:)
	      zcovar(kd,:)=covar(kd,:)
              if ((analyse_is_multi_traits.and.(count(zpresentg(:,kd))>0).and.(count(zpresentc(:,kd))==size(carac))) &
               .or.(.not.analyse_is_multi_traits.and.(count(zpresentg(:,kd))>0).and.(count(zpresentc(:,kd)).ne.0))) then
                effpm(ip,jm)=effpm(ip,jm)+1
                effpr(ip)=effpr(ip)+1
	      endif
             end do
! si une famille de pere-mere est trop petite les permutations se font en utilisant les performances de toute la famille de ce pere
             if (effpm(ip,jm)<ndmin.or.effpm(ip,jm)<nb_des_min) then
	       permut_pere(ip)=.true.
	       permut_pere_mere(ip,jm)=.false.
! impression d'un warning lorsque l'utilisateur a permut� dans la famille de p�re au lieu de la famille pere mere
               call log_mess ('WARNING: permutation will be performed within sire family for sire ['//trim(pere(ip))//&
               '], and dam ['//trim(mere(jm))//']',VERBOSE_DEF)
	     endif
          end do

! impression d'un warning lorsque des famille de pere sont trop petites pour �tre permut�es
          if (effpr(ip).LT.nb_des_min) then
            call log_mess ('family  size of sire ['//trim(pere(ip))//'] is too small (['//&
            str(effpr(ip))//'] animals) to use CONFIDENTLY permutation method, try the simulation method ',WARNING_DEF)
          endif
          !print *, 'EFFECTIF PERMUTE',effpr(ip)
        end do
!

!*****************************************************************************
               !PERMUTATION DES PERFORMANCES
!*****************************************************************************
    iv=0
    do ip=1, np
          npresp=0
          nm1=nmp(ip)+1
          nm2=nmp(ip+1)
!*****************************************************************************
!   1--> r�attribution des performances par permutation au hasard intrafamille de pere( issus
!   de MERE AVEC - de nb_des_min DESCENDANTS CHACUNE )
          IF (permut_pere(ip)) then
            do jm=nm1,nm2
              nd1=ndm(jm)+1
              nd2=ndm(jm+1)
                 do kd=nd1,nd2
                   if ((analyse_is_multi_traits.and.(count(zpresentg(:,kd))>0).and.(count(zpresentc(:,kd))==size(carac))) &
		   .or.(.not.analyse_is_multi_traits.and.(count(zpresentg(:,kd))>0).and.(count(zpresentc(:,kd)).ne.0))) then
                       npresp=npresp+1
                       iv(npresp)=kd
                     endif
                  end do
            end do
            ifail=1
!permutation au hasard dans iv
            call MATH_QTLMAP_G05EHF(iv,npresp,ifail)
 !permutation du vecteur de performances complet (sans donn�es manquantes) lorsqu'on fait une analyse multicaract�re
!permutation du vecteur de performances incluant les donn�es manquantes lorsqu'on fait une analyse unicaract�re
!(seuls les animaux avec aucune performance sont �cart�s
            id=0
            do jm=nm1,nm2
              nd1=ndm(jm)+1
              nd2=ndm(jm+1)
                 do kd=nd1,nd2
                   if (((analyse_is_multi_traits.and.(count(zpresentg(:,kd))>0).and.(count(zpresentc(:,kd))==size(carac))) &
		   .or.(.not.analyse_is_multi_traits.and.(count(zpresentg(:,kd))>0).and.(count(zpresentc(:,kd)).ne.0))) &
		   .and..not.permut_pere_mere(ip,jm)) then
                        id=id+1
                     !  print *, 'kd', kd, 'Y', y(:,kd)
                        y(:,kd)=zy(:,iv(id))
                      ! print *, 'kd', kd, 'Y', y(:,kd)
                        ndelta(:,kd)=zndelta(:,iv(id))
                        cd(:,kd)=zcd(:,iv(id))
          	        nivx(kd,:)=znivx(iv(id),:)
	                covar(kd,:)=zcovar(iv(id),:)
			presentc(:,kd)=zpresentc(:,iv(id))
                        presentg(:,kd)=zpresentg(:,iv(id))
                   endif
                 end do
            end do
   !   print*, 'EFFECTIF FINAL', id
	  ENDIF
!*****************************************************************************
!   2--> permutation par famille de pere-mere
             do jm=nm1,nm2
               npresp=0
               id=0
               nd1=ndm(jm)+1
               nd2=ndm(jm+1)
               IF (permut_pere_mere(ip,jm)) THEN
                 do kd=nd1,nd2
                   if ((analyse_is_multi_traits.and.(count(zpresentg(:,kd))>0).and.(count(zpresentc(:,kd))==size(carac))) &
		   .or.(.not.analyse_is_multi_traits.and.(count(zpresentg(:,kd))>0).and.(count(zpresentc(:,kd)).ne.0))) then
                        npresp=npresp+1
                        iv(npresp)=kd
		   endif
                 end do
                 ifail=1
 !permutation au hasard dans iv
                 call MATH_QTLMAP_G05EHF(iv,npresp,ifail)
 !permutation du vecteur de performances complet (sans donn�es manquantes) lorsqu'on fait une analyse multicaract�re
!permutation du vecteur de performances incluant les donn�es manquantes lorsqu'on fait une analyse unicaract�re
!(seuls les animaux avec aucune performance sont �cart�s
                 do kd=nd1,nd2
                   if ((analyse_is_multi_traits.and.(count(zpresentg(:,kd))>0).and.(count(zpresentc(:,kd))==size(carac))) &
		   .or.(.not.analyse_is_multi_traits.and.(count(zpresentg(:,kd))>0).and.(count(zpresentc(:,kd)).ne.0))) then
                        id=id+1
                         y(:,kd)=zy(:,iv(id))
                        ndelta(:,kd)=zndelta(:,iv(id))
                        cd(:,kd)=zcd(:,iv(id))
          	        nivx(kd,:)=znivx(iv(id),:)
	                covar(kd,:)=zcovar(iv(id),:)
			presentc(:,kd)=zpresentc(:,iv(id))
                        presentg(:,kd)=zpresentg(:,iv(id))
                   endif
                 end do
               ENDIF
            end do
!*****************************************************************************
       end do
!*****************************************************************************
! IMPRESSION DES DONNEES PERMUTEES
!*****************************************************************************
      ! do ic=1,ncar
      !      call log_mess('Trait ['//CHAR(carac(ic))//'] qtlmap normalize data with means:'//&
      !       str(xmut(ic))//'+-'//str(sigt(ic)),INFO_DEF)
      ! enddo
  !    do ip=1,np
!	nm1=nmp(ip)+1
	!nm2=nmp(ip+1)
	!do jm=nm1,nm2
	!   nd1=ndm(jm)+1
	!   nd2=ndm(jm+1)
!	   print *, 'permute pere',permut_pere(ip)
!	   print *, 'permute pere-mere',permut_pere_mere(ip,jm), effpm(ip,jm), ndmin,nb_des_min
	 !    do kd=nd1,nd2
 	!        print *, trim(animal(kd)), ' ', trim(pere(ip)),' ',  trim(mere(jm)),&
	!	' data ',(trim(carac(ic)),' ',zy(ic,kd), zpresentc(ic,kd),ic=1,ncar),(znivx(kd,i), i=1,nfix), (zcovar(kd, j), j=1, ncov), &
	!        ' sim1 ', (trim(carac(ic)),' ',y(ic,kd), presentc(ic,kd),ic=1,ncar), (nivx(kd,i), i=1,nfix), (covar(kd, j), j=1, ncov)
	!     enddo
	!enddo
    !  enddo
       DEALLOCATE (zy)
       DEALLOCATE (znivx)
       DEALLOCATE (zcovar)
       DEALLOCATE (zndelta)
       DEALLOCATE (zpresentc)
       DEALLOCATE (effpr)
       DEALLOCATE (iv)
       DEALLOCATE (permut_pere)
       DEALLOCATE (permut_pere_mere)
       DEALLOCATE (zcd)
       DEALLOCATE (effpm)
      return

      end subroutine sim_perf_shuffling
!!***


!!****f* m_qtlmap_traits/write_perf
!!  NAME
!!    write_perf
!!  DESCRIPTION
!!
!!  NOTES
!!
!!  SOURCE
      subroutine write_perf(file_name)
        character(len=*),intent(in)     :: file_name
        integer :: kd,ic
        integer :: myunit=99999
        integer ,dimension(ncar)    :: cd
        open (myunit,file=file_name)
        do kd=1,nd
          cd = 1
          do ic=1,ncar
            if(.not. presentc(ic,kd)) then
              y(ic,kd)=-99.d0
              cd(ic) = 0
            end if
          end do

          write(myunit,FMT='(a12,'// trim(str(ncar)) //'(1x,f9.5,1x,i1,1x,"1"))')&
          trim(animal(kd)),(y(ic,kd),cd(ic),ic=1,ncar)
        end do
        close(myunit)

      end subroutine write_perf
!!***


!!****f* m_qtlmap_traits/set_estime
!!  NAME
!!    set_estime
!!  DESCRIPTION
!!    Initialize the following data :
!!      * estime  (dim : ncar,nm)   : indicate the estimability of a full sib family (condition ndmin involve full sib family are used in the qtl analysis)
!!      * nmumest (dim ncar)        : number of full sib family with enough progenies (ndmin>=)
!!      * namest  (dim ncar)        : number of unique female (several full sib family are building with the same dam)
!!      * iam     (dim : ncar,nfem) : get the index of female (estimable)
!!
!!  NOTES
!!     anciennement dans optinit, on initialisse estime,nmumest,namest,iam dans cette routine
!!  SOURCE

     subroutine set_estime()
       logical         :: estfem(nfem)
       integer :: ic,ip,jm,kd,eff,ifem

       call log_mess('SUBROUTINE set_estime',DEBUG_DEF)

       if ( allocated (estime) ) then
         deallocate( estime )
         deallocate( namest )
         deallocate( nmumest )
       end if

       if ( allocated (iam) )  deallocate( iam )

       allocate (estime(size(carac),size(mere)) )
       allocate (namest(size(carac)) )
       allocate (nmumest(size(carac)))
       allocate (iam(size(carac),size(femelle)) )

       estime=.false.
       nmumest=0
       namest=0
       iam=0.d0

       do ic=1,ncar
         estfem=.false.
         do ip=1,np
          do jm=nmp(ip)+1,nmp(ip+1)
            eff=0
            do kd=ndm(jm)+1,ndm(jm+1)
              if(presentc(ic,kd)) eff=eff+1
            end do
            ifem=repfem(jm)
            if(eff.ge.ndmin) then
              estime(ic,jm)=.true.
              estfem(ifem)=.true.
              nmumest(ic)=nmumest(ic)+1
            end if
           end do
         end do
         do ifem=1,nfem
          iam(ic,ifem)=0
          if(estfem(ifem)) then
           namest(ic)=namest(ic)+1
           iam(ic,ifem)=namest(ic)
          end if
         end do
      end do

      call log_mess('END SUBROUTINE set_estime',DEBUG_DEF)

     end subroutine set_estime
!!***


!!****f* m_qtlmap_traits/check_cd
!!  NAME
!!    check_cd
!!  DESCRIPTION
!!    set cd to 1.0 if the trait is not 'average' and cd /= 0 for each animal
!!
!!  NOTES
!!
!!  SOURCE
     subroutine check_cd
        integer :: i,j

        call log_mess("check cd...",INFO_DEF)
        do i=1,ncar
           if ( natureY(i) /= 'a' ) then
              do j=1,size(cd,2)
               if (cd(i,j)/=0.d0) cd(i,j)=1.d0
              end do
           end if
        end do ! i

     end subroutine check_cd
!!***

!!****f*  m_qtlmap_traits/normalize_data
!!  NAME
!!    normalize_data
!!  SYNOPSIS
!!    Normalizing data for continue traits
!!  FUNCTION
!!    Compute xmut (Mean) and sigt (Standart deviation) and normalize Y array.
!!  INPUTS
!!    * is_transcriptom    -- boolean to set for no printing information whith transcriptomic data
!!    * is_simul           -- boolean to set for not printing information in simulation case
!!  RESULT
!!    none.
!!  EXAMPLE
!!
!!  NOTES
!!  If none traits are found found for one founder, the analyse stop
!!  BUGS
!!
!!  SEE ALSO
!!  xmut
!!  sigt
!!  natureY
!!  MATH_QTLMAP_G01FAF
!!
!!  SOURCE
    subroutine normalize_data(is_transcriptom,is_simul)

       logical  , intent(in)                          :: is_transcriptom
       logical  , intent(in)                          :: is_simul
       integer                                        :: kd,i
       real (kind=dp)                                 :: sy,sy2,eff

       call log_mess('SUBROUTINE normalize_data',DEBUG_DEF)

       xmut = 0.d0
       sigt = 1.d0

       do i=1,ncar
          if (.not.(natureY(i) == 'r' .or. natureY(i) == 'a')) cycle
          sy=0.d0
          sy2=0.d0
          eff=0.d0
          xmut(i)=0.d0
          do kd=1,nd
             if(presentc(i,kd)) then
                sy2=sy2+y(i,kd)*y(i,kd)
                sy=sy+y(i,kd)
                eff=eff+1.d0
             endif
          enddo
          if (eff == 0.d0) then
              call stop_application("Can not find animal with performance for trait :"//trim(carac(i)))
          end if

          xmut(i)=sy/eff
          sigt(i)=dble(sqrt((sy2/eff)-xmut(i)*xmut(i)))

          !  normalisation des donn�es
          if ( .not. is_transcriptom .and. .not. is_simul ) then
            call log_mess('Trait ['//trim(carac(i))//'] qtlmap normalize data with means:'//&
            str(xmut(i))//'+-'//str(sigt(i)),INFO_DEF)
          end if

          do kd=1,nd
           if(presentc(i,kd)) then
             y(i,kd)=(y(i,kd)-xmut(i))/sigt(i)
           end if
         end do
        end do

      call log_mess('END SUBROUTINE normalize_data',DEBUG_DEF)
    end subroutine normalize_data
!!***

!!****f*  m_qtlmap_traits/set_count_discrete
!!  NAME
!!    set_count_discrete
!!  SYNOPSIS
!!    Manage discrete data to initialized associated structure
!!  FUNCTION
!!    Initialize structure ydiscretord , nmod and indicemod for discrete data
!!  INPUTS
!!
!!  RESULT
!!    none.
!!  EXAMPLE
!!
!!  NOTES
!!
!!  BUGS
!!
!!  SEE ALSO
!!  ydiscretord
!!  nmod
!!  indicemod
!!
!!  SOURCE
       subroutine set_count_discrete()

         logical  :: found
         integer  :: i,ic,ideb,m,m1,j,temp
         integer , dimension(ncar,nd) :: indicemod_t

         if (.not. allocated(ydiscretord)) allocate (ydiscretord(ncar,nd))
         if (.not. allocated(nmod)) allocate (nmod(ncar))
         ydiscretord = 0

!******************************************************************************
!******************************************************************************
!
! transformation des donn�es discret � des donn�e utilisable par QTLMAP
!
!
!*****************************************************************************
!******************************************************************************
!


!  Parametres de maximisation

!  nmod est le nombre des modalit�s du caract�re �tudi�
!  on reserve donc la partie du vecteur param�tre qui contiendra le lambda � estimer
   nmod=0

   do ic=1,ncar
    if (natureY(ic) /= 'i') cycle
    nmod(ic)=1
	i=1
	found=.false.
	do while(i.le.nd.and..not.found)
	  if(presentc(ic,i)) then
	    found=.true.
	    ideb=i+1
	    indicemod_t(ic,1) = int(y(ic,i))
	  end if
	end do

      do i=ideb,nd
          if(presentc(ic,i))then
            m=1
            found=.false.
            do while (m<=nmod(ic) .and. .not. found)
	      if (int(y(ic,i))==indicemod_t(ic,m)) then
		    found=.true.
	      else
		    m=m+1
	      end if
            enddo

            if (.not. found) then
	          nmod(ic)=nmod(ic)+1
              indicemod_t(ic,nmod(ic))=y(ic,i)
            end if
	  end if
	enddo


! maintenant on tri le tableau des modalit�s du plus petit au plus grand
!
!*************************************************************************
!**************************tri a bulle************************************
!*************************************************************************
	do j=1, nmod(ic)
          do m1=1, j-1
             if (indicemod_t(ic,j)<indicemod_t(ic,m1)) then
               temp=indicemod_t(ic,m1)
               indicemod_t(ic,m1)=indicemod_t(ic,j)
               indicemod_t(ic,j)=temp
             end if
          end do
       end do
!**************************************************************************
!
! nmod est le nombre de modalit� possible pour le caract�re �tudi�
!
! maintenent il faut donc cr�er le nouveau tableau de performances qui sera
!utilis� dans QTLMap, ydiscretord
!
      do i=1,nd
        do j=1, nmod(ic)
          if (presentc(ic,i)) then
            if ( int(y(ic,i))==indicemod_t(ic,j) )ydiscretord(ic,i)=j
          end if
        end do
      end do

    end do ! ic



    if (.not.allocated(indicemod)) allocate (indicemod(ncar,maxval(nmod)))
    indicemod = 0
    do ic=1,ncar
      do j=1,nmod(ic)
        indicemod(ic,j) = indicemod_t(ic,j)
      end do
    end do

   end subroutine set_count_discrete
!!***

!!****f*  m_qtlmap_traits/set_proportion_discrete
!!  NAME
!!    set_proportion_discrete
!!  SYNOPSIS
!!    Mange process according to the type of the data (continue, discrete or categorial)
!!  FUNCTION
!!    Initialize structure Prop (proportion) and Seuil (Threshold) for discrete data
!!  INPUTS
!!
!!  RESULT
!!    none.
!!  EXAMPLE
!!
!!  NOTES
!!
!!  BUGS
!!
!!  SEE ALSO
!!    prop
!!    seuil
!!    MATH_QTLMAP_G01FAF
!!
!!  SOURCE
   subroutine set_proportion_discrete

       integer       :: ic,i,ifail,ii
       real(kind=dp) :: cumul,eff

       ifail = 0

!
!  comptage pour la cr�ation du point de d�part
!
      if(.not. allocated(seuil)) allocate (seuil(ncar,maxval(nmod)))
      if (.not. allocated(prop)) allocate (prop(ncar,maxval(nmod)))

      prop=0.d0
      seuil = 0.d0

      do ic=1,ncar
          if (natureY(ic) /= 'i') cycle
         eff=0.d0
         do ii=1,nd
          if(presentc(ic,ii)) then
	        prop(ic,ydiscretord(ic,ii))=prop(ic,ydiscretord(ic,ii))+1.d0
	        eff=eff+1.d0
           end if
         end do
         do ii=1,nmod(ic)
            prop(ic,ii) = prop(ic,ii)/eff
         end do

         cumul=0.d0
      do i=1,nmod(ic)-1
        cumul=cumul+prop(ic,i)
        seuil(ic,i)=MATH_QTLMAP_G01FAF('L',cumul,ifail)
      end do
!
!  fin du comptage
       end do ! ic
    end subroutine set_proportion_discrete
!!***

!!****f*  m_qtlmap_traits/manage_data
!!  NAME
!!    manage_data
!!  SYNOPSIS
!!    Mange process according to the type of the data (continue, discrete or categorial)
!!  FUNCTION
!!    Normalize data for continue data and initilialized data structure sigt, xmut
!!    Count discrete data for discrete data and compute Proportions (prop structure) and Threshold (seuil structure)
!!  INPUTS
!!    is_transcriptom    -- boolean to set for no printing information whith transcriptomic data
!!    is_simul           -- boolean to set for not printing information in simulation case
!!  RESULT
!!    none.
!!  EXAMPLE
!!
!!  NOTES
!!
!!  BUGS
!!
!!  SEE ALSO
!!    is_transcriptom,
!!    set_count_discrete,
!!    set_proportion_discrete
!!
!!  SOURCE
    subroutine manage_data(is_transcriptom,is_simul,normalize)

      logical  , intent(in)                          :: is_transcriptom
      logical  , intent(in)                          :: is_simul
      logical  , intent(in)                          :: normalize

      if ( .not. allocated(xmut) )allocate(xmut(ncar))
      if ( .not. allocated(sigt) )allocate(sigt(ncar))

      if ( normalize ) then
         call normalize_data(is_transcriptom,is_simul)
      else
         xmut=0.d0
         sigt=1.d0
      end if

      call set_count_discrete()
      call set_proportion_discrete()

    end subroutine manage_data
!!***


!!****f*  m_qtlmap_traits/log_infoqtldefinedbyuser
!!  NAME
!!    log_infoqtldefinedbyuser
!!  DESCRIPTION
!!   print in the console information about localisation of qtl simulated.
!!
!!  NOTES
!!
!!  SOURCE
     subroutine log_infoqtldefinedbyuser(nqtl)
      integer , intent(in) :: nqtl
      integer :: i,chr
      real(kind=dp) :: l
 
      if ( nqtl <=0 ) return
      ! LOG*****
      call log_mess("--------------- QTL DEFINED --------------------",INFO_DEF)
   
      do i=1,nqtl
          chr=chrqtl(i)
          call log_mess(trim(str(i))//":Position defined by the user :"//trim(str(posiqtl(i))),INFO_DEF)
          l=(get_pos((posiqtl(i)-posi(chr,1)))*((posi(chr,nmk(chr))-posi(chr,1))/get_npo(chr)))+posi(chr,1)
          call log_mess(trim(str(i))//":Position according the sampling :"//trim(str(l)),INFO_DEF)
       end do

       call log_mess("--------------------------------------------------------",INFO_DEF)
     end subroutine log_infoqtldefinedbyuser
!!***


!!****f*  m_qtlmap_traits/init_random
!!  NAME
!!    init_random
!!  DESCRIPTION
!!
!!
!!  NOTES
!!   faire un module random
!!  SOURCE
      subroutine init_random()
        integer :: n,clock,i
        integer, dimension(:), allocatable :: seed

        !initialisation pour les routines intrasec de fortran
        call random_seed(size=n)
        allocate(seed(n))
        call system_clock(count=clock)
        seed = clock + 37 * (/ (i - 1, i = 1, n) /)
        call random_seed(put = seed)
        deallocate(seed)

    end subroutine init_random
!!***


!!****f*  m_qtlmap_traits/normal
!!  NAME
!!    normal
!!  DESCRIPTION
!!
!!
!!  NOTES
!!   faire un module random
!!  SOURCE
    function normal(mean,sigma) !returns a normal distribution
        real(kind=dp)  , intent(in) :: mean,sigma
        real(kind=dp) normal,tmp
        integer flag
        real(kind=dp) fac,gsave,rsq,r1,r2,u
        save flag,gsave
        data flag /0/
        !$omp threadprivate (flag)
        if (flag.eq.0) then
        rsq=2.0
            do while(rsq.ge.1.0.or.rsq.eq.0.0) ! new from for do
                call random_number(u)
                r1=2.0*u-1.0
                call random_number(u)
                r2=2.0*u-1.0
                rsq=r1*r1+r2*r2
            enddo
            fac=sqrt(-2.0*log(rsq)/rsq)
            gsave=r1*fac
            tmp=r2*fac
            flag=1
        else
            tmp=gsave
            flag=0
        endif
        normal=tmp*sigma+mean
        return
    end function normal
!!***

     !! Fait par jm en australie.....

   subroutine create_trait(genin,perfin,perfout,nqtl,hdeux,position_qtl,effet_qtl)
      character(len=LENGTH_MAX_FILE)                         :: genin,perfin,perfout
      integer      ,            intent(in)                   :: nqtl
      real(kind=dp)       , intent(in)                       :: hdeux
      integer      ,    dimension(nqtl),        intent(in)   :: position_qtl ! chaque qtl est position sur un indice de marqueur
      real (kind=dp) ,dimension(nqtl)   ,        intent(in)  :: effet_qtl ! effet par qtl

      real   :: vh2(2,2),u0(2),refgg(6),z(6),work(2)
      real   :: gennor


      real (kind=dp)   :: up,um,ud,varE,varP
      real (kind=dp) , dimension(8000)   :: perf_d
      integer code1(8000),code2(8000),a1(8000)
      integer nkd,ip,jm,kd,ld,i,nh2,ii
      integer(kind=KIND_PHENO)   ,dimension (:,:),allocatable  ::   gene_o_qtl

      character*4  num(8000)
      character a2(8000)
      real val
      logical found

      character*7 snp(5111)
      integer m1(5111),m2(5111),i_ind

      call init_random()

      open(111,file=genin)
      read(111,*)(snp(i),i=1,100)
      print*,'position_qtl',position_qtl
      i_ind=1

      allocate (gene_o_qtl(nqtl,8000))

   13 read(111,*,end=14)num(i_ind),(m1(i),m2(i), i=1,100)
         do ii=1,nqtl
         i=position_qtl(ii)
         gene_o_qtl(ii,i_ind)= m1(i) + m2(i) -1
         i_ind=i_ind+1
         end do
      go to 13
   14 continue
      close(111)

!      vh2=0.d0
!      vh2(1,1)=0.15d0
!      vh2(2,2)=0.85d0
!      u0=0.d0
!      nh2=2
!      call setgmn(u0,vh2,nh2,refgg)
!      do ip=1,iseed
!        call genmn(refgg,z,work)
!      end do

      varP = 0.25d0*hdeux
      varE = 1 - varP


      open(111,file=perfin)
      open(222,file=perfout, form="formatted",recl=26)

      nkd=1
    1 read(111,*,end =2)num(nkd),a1(nkd),a2(nkd),val,code1(nkd),code2(nkd)
      nkd=nkd+1
      go to 1
    2 continue

      do ip=1,np
        up=normal(0.d0,sqrt(varP))
        do jm=nmp(ip)+1,nmp(ip+1)
          um=normal(0.d0,sqrt(varP))
          do kd=ndm(jm)+1,ndm(jm+1)
            ud=normal(0.d0,sqrt(varE))
     !       print*,kd,corred(kd),gene_o_qtl(corred(kd))
            perf_d(kd)=up+um+ud
            do ii=1,nqtl
              print *,( ( real(gene_o_qtl(ii,corred(kd)) - 1.d0 ) / 2.d0 ) )
              perf_d(kd)= perf_d(kd) + effet_qtl(ii)*( ( real(gene_o_qtl(ii,corred(kd)) - 1.d0 ) / 2.d0 ) )
            end do
          end do
        end do
      end do

      do kd=1,nkd-1
        found=.false.
        do ld=1,nd
          if(animal(ld) ==num(kd)) then
            found=.true.
            exit
          end if
        end do
 !       print*,kd,ld
        write(222,2222)num(kd),a1(kd),a2(kd),perf_d(ld),code1(kd),code2(kd)
 2222 format(a4,1x,i4,1x,a1,1x,F10.5,1x,i1,1x,i1)
     end do
     close(111)
     rewind(222)
     close(222)
     deallocate (gene_o_qtl)
  !    stop
      end subroutine create_trait



end module m_qtlmap_traits
