!!****p* QTLMap/LOG
!!  NAME
!!    LOG
!!  DESCRIPTION
!!  Package log
!!  CREATION DATE
!!  01/01/2009
!!  COPYRIGHT
!!***

!!****m* LOG/m_qtlmap_log
!!  NAME
!!    m_qtlmap_log
!!  SYNOPSIS
!!  This module manage the log console output.
!!  DESCRIPTION
!!  NOTES
!!
!!  BUGS
!!
!!  SEE ALSO
!!    m_qtlmap_data
!!
!!  CREATION DATE
!!  01/01/2009
!!
!!  COPYRIGHT
!!***
!!/
MODULE m_qtlmap_log

#ifdef MPI_ACTIVE
       use mpi
#endif

#ifdef MANAGE_OMP
       use omp_lib
#endif
IMPLICIT NONE
SAVE

    integer, public, parameter             :: DEBUG_DEF     = 0
    integer, public, parameter             :: VERBOSE_DEF   = 1
    integer, public, parameter             :: INFO_DEF      = 2
    integer, public, parameter             :: WARNING_DEF   = 3
    integer, public, parameter             :: ERROR_DEF     = 4

    ! internal level to handle messages
    integer             ,private                   :: internal_level = INFO_DEF

    !identifiant du processus mpi courant
    integer                   ,private     :: thread_rang = 0

    public init_log
	public log_mess
    public end_log

CONTAINS

    function strint(value) result(res)
       integer ,intent(in)  :: value
       character(len=20)                    :: res
             integer                              :: io

             res=''
             write (res,fmt='(i11)',iostat=io) value
             if ( io /= 0 ) then
               res='<NO-DEF>'
             else
               res=adjustl(res)
             end if
             return


    end function strint

    function strfloat(value) result(res)
             double precision ,intent(in)           :: value
             character(len=20)                    :: res
             integer                              :: io

             res=''
             write (res,fmt='(f11.4)',iostat=io) value
             if ( io /= 0 ) then
               res='<NO-DEF>'
             else
               res=adjustl(res)
             end if
             return

     end function strfloat

      function strbool(value) result(res)
             logical ,intent(in)           :: value
             character(len=20)                    :: res

            if ( value ) then
               res = 'True'
            else
               res = 'False'
            end if

     end function strbool

!-----------------------------
! subroutine : init_log
!
   SUBROUTINE init_log(level_info)
     integer,optional,intent(in)                :: level_info
     integer  :: code

     thread_rang = 0
     IF ( .NOT. present(level_info) ) THEN
		internal_level = WARNING_DEF
     ELSE
        internal_level = level_info
     ENDIF


#ifdef MPI_ACTIVE
     call MPI_COMM_RANK (MPI_COMM_WORLD, thread_rang , code)
     IF ( thread_rang /= 0 ) return
#endif

   END SUBROUTINE init_log

!----------------------------------
!subroutine end_log
   SUBROUTINE end_log()

   END SUBROUTINE end_log

!
   SUBROUTINE log_mess(message,level)
     character (len = *) , intent(in)   :: message
     integer,optional,intent(in)        :: level

     IF ( present(level) ) THEN
         IF ( level >= internal_level ) THEN
              CALL WRITE_LOG(message,level)
              RETURN
         ENDIF
     ELSE
         IF ( INFO_DEF >= internal_level ) THEN
              CALL WRITE_LOG(message,INFO_DEF)
              RETURN
         ENDIF
     ENDIF
   END SUBROUTINE log_mess

!----------------------------------

   SUBROUTINE WRITE_LOG(message,level)
      character (len = *) , intent(in)   :: message
      integer,         intent(in)        :: level
      character(len=8)               :: date
      character(len=10)              :: time
      character(len=50)              :: addinfo

     ! integer, dimension(8)          :: value
     addinfo=''

#ifdef MPI_ACTIVE
       addinfo='[mpi_proc '//trim(strint(thread_rang))//']'
#endif

#ifdef MANAGE_OMP
      if ( omp_in_parallel() ) then
        addinfo=trim(addinfo)//'[omp_thread '//trim(strint(omp_get_thread_num()+1))//']'
      end if
#endif



      call DATE_AND_TIME(date, time)

      print *,'qtlmap:[', &
        date(7:8),'/',date(5:6),'/',date(1:4),' ',&
        time(1:2),':',time(3:4),':',time(5:6) &
        ,'][',trim(TOSTRING(level)),']',trim(addinfo),':',trim(message)

      if ( level == WARNING_DEF ) then
        write (0,*) 'WARNING:',message
      else if ( level == ERROR_DEF ) then
        write (0,*) 'ERROR:',message
      endif

   END SUBROUTINE WRITE_LOG


!----------------------------------

    FUNCTION TOSTRING(level) result(slevel)
       integer,         intent(in)        :: level
       character(len=10)                  :: slevel
       SELECT CASE(level)
          CASE(DEBUG_DEF)
             slevel = "debug  "
          CASE(VERBOSE_DEF)
             slevel = "verbose"
          CASE(INFO_DEF)
             slevel = "info   "
          CASE(WARNING_DEF)
             slevel = "warning"
          CASE(ERROR_DEF)
             slevel = "error  "
          CASE DEFAULT
             slevel = "*******"
        END SELECT

       RETURN
    END FUNCTION TOSTRING


END MODULE m_qtlmap_log
