!!****m* QTLMap/m_qtlmap_cli
!!  NAME
!!    m_qtlmap_cli
!!  SYNOPSIS
!!   A command-line interface (CLI) is a mechanism for interacting with th QTLMap software by typing commands to perform specific tasks.
!!
!!  DESCRIPTION
!!   This module manage the options give as argument to qtlmap.
!!
!!  NOTES
!!
!!  BUGS
!!
!!  SEE ALSO
!!
!!  COPYRIGHT
!!***
!! You can use this space for remarks that should not be included
!! in the documentation.
!!/
module m_qtlmap_cli
   use m_qtlmap_constant
   use m_qtlmap_haplotype
   use m_qtlmap_optimization

   implicit none
!!****d* m_qtlmap_cli/LEN_OPT
!!  NAME
!!   LEN_OPT
!!  DESCRIPTION
!!   Option maximum length
!!  SOURCE
   integer                            ,parameter,private   :: LEN_OPT         = 44
!!***

!!****d* m_qtlmap_cli/MARG_HELP
!!  NAME
!!   MARG_HELP
!!  DESCRIPTION
!!   blank to print before the help output
!!  SOURCE
   integer                            ,parameter,private   :: MARG_HELP       = 30
!!***

!!****d* m_qtlmap_cli/NSIM_DEFAULT
!!  NAME
!!   NSIM_DEFAULT
!!  DESCRIPTION
!!   default simulation number if is not specified by the user
!!  SOURCE
   integer                            ,parameter,private   :: NSIM_DEFAULT    = 1000
!!***

!!****d* m_qtlmap_cli/NQTL_DEFAULT
!!  NAME
!!   NQTL_DEFAULT
!!  DESCRIPTION
!!   default hypothesis number (qtl) to test if is not specified by the user
!!  SOURCE
   integer                            ,parameter,private   :: NQTL_DEFAULT    = 1
!!***

!!****d* m_qtlmap_cli/SIB_DEFAULT
!!  NAME
!!   SIB_DEFAULT
!!  DESCRIPTION
!!   default value of the family option (full sib family is the default).
!!  SOURCE
   integer                            ,parameter,private   :: SIB_DEFAULT     = 2
!!***

!!****d* m_qtlmap_cli/OPTIM_DEFAULT
!!  NAME
!!   OPTIM_DEFAULT
!!  DESCRIPTION
!!   default optimization method using by qtlmap ( LA optimized analysis )
!!  SOURCE
   integer                            ,parameter,private   :: OPTIM_DEFAULT   = 1 ! NAG
!!***

! *****************************************************************************************************

!!****d* m_qtlmap_cli/OPT_CALCUL
!!  NAME
!!   OPT_CALCUL
!!  DESCRIPTION
!!   calcul option name
!!  NOTES
!!   see m_qtlmap_analyse module to have the available analysis
!!  SOURCE
   character(len=LEN_OPT)             , parameter, private :: OPT_CALCUL       ='--calcul'
!!***
!!****d* m_qtlmap_cli/OPT_QTL
!!  NAME
!!   OPT_QTL
!!  DESCRIPTION
!!   hypothesis option name
!!  NOTES
!!   Some analysis have limitation with this value.
!!  SOURCE
   character(len=LEN_OPT)             , parameter, private :: OPT_QTL          ='--qtl'
!!***
!!****d* m_qtlmap_cli/OPT_HAPLOTYPE
!!  NAME
!!   OPT_HAPLOTYPE
!!  DESCRIPTION
!!   haplotype option name
!!  NOTES
!!   m_qtlmap_haplotype/haplotype to see the availables haplotypes versions
!!  SOURCE
   character(len=LEN_OPT)             , parameter, private :: OPT_HAPLOTYPE    ='--haplotype'
!!***
!!****d* m_qtlmap_cli/OPT_SIBF
!!  NAME
!!   OPT_SIBF
!!  DESCRIPTION
!!   family option name
!!  NOTES
!!
!!  SOURCE
   character(len=LEN_OPT)             , parameter, private :: OPT_SIBF         ='--family'
!!***
!!****d* m_qtlmap_cli/OPT_OPTIM
!!  NAME
!!   OPT_OPTIM
!!  DESCRIPTION
!!   optimization option name
!!  NOTES
!!   see m_qtlmap_optimization
!!  SOURCE
   character(len=LEN_OPT)             , parameter, private :: OPT_OPTIM        ='--optim'
!!***
!!****d* m_qtlmap_cli/OPT_SIM
!!  NAME
!!   OPT_SIM
!!  DESCRIPTION
!!   random simulation option name
!!  SOURCE
   character(len=LEN_OPT)             , parameter, private :: OPT_SIM          ='--simulation'
!!***
!!****d* m_qtlmap_cli/OPT_NSIM
!!  NAME
!!   OPT_NSIM
!!  DESCRIPTION
!!   number of simulation option name
!!  SOURCE
   character(len=LEN_OPT)             , parameter, private :: OPT_NSIM         ='--nsim'
!!***
!!****d* m_qtlmap_cli/OPT_EQTL
!!  NAME
!!   OPT_EQTL
!!  DESCRIPTION
!!   expression phenotypics values option name
!!  SOURCE
   character(len=LEN_OPT)             , parameter, private :: OPT_EQTL         ='--data-transcriptomic'
!!***
!!****d* m_qtlmap_cli/OPT_PERMUT
!!  NAME
!!   OPT_PERMUT
!!  DESCRIPTION
!!   permutation simulation option name
!!  SOURCE
   character(len=LEN_OPT)             , parameter, private :: OPT_PERMUT       ='--permute'
!!***
!!****d* m_qtlmap_cli/OPT_SNP
!!  NAME
!!   OPT_SNP
!!  DESCRIPTION
!!   snp option name
!!  SOURCE
   character(len=LEN_OPT)             , parameter, private :: OPT_SNP          ='--snp'
!!***
!!****d* m_qtlmap_cli/OPT_HDAM
!!  NAME
!!   OPT_HDAM
!!  DESCRIPTION
!!   haplotype dam option name
!!  NOTES
!!   LD analysis
!!  SOURCE
   character(len=LEN_OPT)             , parameter, private :: OPT_HDAM         ='--hdam'
!!***
!!****d* m_qtlmap_cli/OPT_CENSORED_CALCUL
!!  NAME
!!   OPT_CENSORED_CALCUL
!!  DESCRIPTION
!!   censored data option name
!!  NOTES
!!   Used to take care about a third generation with trait
!!  SOURCE
   character(len=LEN_OPT)             , parameter, private :: OPT_CENSORED_CALCUL  ='--calcul-cd'
!!***
!!****d* m_qtlmap_cli/OPT_ESTIME_THRESHOLDS
!!  NAME
!!   OPT_ESTIME_THRESHOLDS
!!  DESCRIPTION
!!   threshold estimation option name
!!  SOURCE
   character(len=LEN_OPT)             , parameter, private :: OPT_ESTIME_THRESHOLDS = '--estimate-thresholds'
!!***
!!****d* m_qtlmap_cli/OPT_ESTIME_THRESHOLDS_SHORT
!!  NAME
!!   OPT_ESTIME_THRESHOLDS_SHORT
!!  DESCRIPTION
!!   threshold estimation short option name
!!  SOURCE
   character(len=LEN_OPT)             , parameter, private :: OPT_ESTIME_THRESHOLDS_SHORT = '-s'
!!***
!!****d* m_qtlmap_cli/OPT_QUIET
!!  NAME
!!   OPT_QUIET
!!  DESCRIPTION
!!   quiet output console option name
!!  NOTES
!!   m_qtlmap_log
!!  SOURCE
   character(len=LEN_OPT)             , parameter, private :: OPT_QUIET        ='--quiet'
!!***
!!****d* m_qtlmap_cli/OPT_VERBOSE
!!  NAME
!!   OPT_VERBOSE
!!  DESCRIPTION
!!   verbose output console option name
!!  NOTES
!!   m_qtlmap_log
!!  SOURCE
   character(len=LEN_OPT)             , parameter, private :: OPT_VERBOSE      ='--verbose'
!!***
!!****d* m_qtlmap_cli/OPT_DEBUG
!!  NAME
!!   OPT_DEBUG
!!  DESCRIPTION
!!   debug output console option name
!!  NOTES
!!   m_qtlmap_log
!!  SOURCE
   character(len=LEN_OPT)             , parameter, private :: OPT_DEBUG        ='--debug'
!!***
!!****d* m_qtlmap_cli/OPT_QUIET_SHORT
!!  NAME
!!   OPT_QUIET_SHORT
!!  DESCRIPTION
!!   quiet output console short option name
!!  NOTES
!!   m_qtlmap_log
!!  SOURCE
   character(len=LEN_OPT)             , parameter, private :: OPT_QUIET_SHORT  ='-q'
!!***
!!****d* m_qtlmap_cli/OPT_VERBOSE_SHORT
!!  NAME
!!   OPT_VERBOSE_SHORT
!!  DESCRIPTION
!!   verbose output console short option name
!!  NOTES
!!   m_qtlmap_log
!!  SOURCE
   character(len=LEN_OPT)             , parameter, private :: OPT_VERBOSE_SHORT='-v'
!!***
!!****d* m_qtlmap_cli/OPT_DEBUG_SHORT
!!  NAME
!!   OPT_DEBUG_SHORT
!!  DESCRIPTION
!!   debug output console short option name
!!  NOTES
!!   m_qtlmap_log
!!  SOURCE
   character(len=LEN_OPT)             , parameter, private :: OPT_DEBUG_SHORT  ='-d'
!!***
!!****d* m_qtlmap_cli/OPT_PRINT_CLASSIC
!!  NAME
!!   OPT_PRINT_CLASSIC
!!  DESCRIPTION
!!   classic output result option name
!!  NOTES
!!   used to print a classical output analysis in a simulation case or expressions phenotypics values analysis.
!!  SOURCE
   character(len=LEN_OPT)             , parameter, private :: OPT_PRINT_CLASSIC='--print-all'
!!***
!!****d* m_qtlmap_cli/OPT_HELP_PANALYSE
!!  NAME
!!   OPT_HELP_PANALYSE
!!  DESCRIPTION
!!   help parameter file option name
!!  SOURCE
   character(len=LEN_OPT)             , parameter, private :: OPT_HELP_PANALYSE = "--help-panalyse"
!!***
!!****d* m_qtlmap_cli/OPT_HELP_PARAMSIM
!!  NAME
!!   OPT_HELP_PARAMSIM
!!  DESCRIPTION
!!   help simulation parameter file option name
!!  SOURCE
   character(len=LEN_OPT)             , parameter, private :: OPT_HELP_PARAMSIM = "--help-paramsim"
!!***
!!****d* m_qtlmap_cli/OPT_HELP
!!  NAME
!!   OPT_HELP
!!  DESCRIPTION
!!   help option name
!!  SOURCE
   character(len=LEN_OPT)             , parameter, private :: OPT_HELP         ='--help'
!!***
!!****d* m_qtlmap_cli/OPT_XML
!!  NAME
!!   OPT_XML
!!  DESCRIPTION
!!   xml output result file option name
!!  SOURCE
   character(len=LEN_OPT)             , parameter, private :: OPT_XML             ='--output-xml'
!!***

   !------------------------------------------
   ! Print QtlMAP Version
   !------------------------------------------
   public :: cli_print_qtlmap_version

   !------------------------------------------
   ! Print help
   !------------------------------------------
   public :: cli_print_help
   !------------------------------------------
   ! True if the user ask help message
   !------------------------------------------
   public :: cli_is_help
   !------------------------------------------
   ! Get the analyse parameters file
   !------------------------------------------
   public :: cli_get_parameters_file
   !------------------------------------------
   ! Get the analyse type
   !------------------------------------------
   public :: cli_get_analyse
   !------------------------------------------
   ! Get the Haplotype version
   !------------------------------------------
   public :: cli_get_haplotype
   !------------------------------------------
   ! Get the Family version
   !------------------------------------------
   public :: cli_get_family
   !------------------------------------------
   ! Get the optimization
   !------------------------------------------
   public :: cli_get_optimization
   !------------------------------------------
   ! True if the user want estimate rejection threholds of the current analysis
   !------------------------------------------
   public :: cli_is_estimate_thresholds
   !------------------------------------------
   ! True if the user ask a simulation
   !------------------------------------------
   public :: cli_is_simulation
   !------------------------------------------
   ! True if the user ask a transcriptomic analysis/simulation
   !------------------------------------------
   public :: cli_is_transcriptomic_data
   !------------------------------------------
   ! True if the user ask a permutation method for the simulation
   !------------------------------------------
   public :: cli_is_permute
   !------------------------------------------
   ! True if the user ask to estimat haplotype dam in the model LD and derivated LD analysis
   !------------------------------------------
   public :: cli_is_hdam
   !------------------------------------------
   ! True if the user ask an iteration and so a simulation
   !------------------------------------------
   public :: cli_is_nsim
   !------------------------------------------
   ! Get the number of the simulation
   !------------------------------------------
   public :: cli_get_nsim
   !------------------------------------------
   ! Get the number of the qtl to detect
   !------------------------------------------
   public :: cli_get_nqtl
   !------------------------------------------
   ! True if the user ask none information
   !------------------------------------------
   public :: cli_is_quiet
   !------------------------------------------
   ! True if the user ask verbose information
   !------------------------------------------
   public :: cli_is_verbose
   !------------------------------------------
   ! True if the user ask to print all information
   !------------------------------------------
   public :: cli_is_print_all
   !------------------------------------------
   ! True if the user ask debug information
   !------------------------------------------
   public :: cli_is_debug
   !------------------------------------------
   ! True if the user ask help p_analyse information
   !------------------------------------------
   public :: cli_is_help_panalyse
   !------------------------------------------
   ! True if the user ask help paramsim information
   !------------------------------------------
   public :: cli_is_help_paramsim

   public :: cli_is_calcul_cd


   contains

!!****f* m_qtlmap_cli/cli_print_qtlmap_version
!!  NAME
!!    cli_print_qtlmap_version
!!  DESCRIPTION
!!    Print the version of qtlmap and the date of the binary build.
!!  NOTES
!!  SOURCE
       subroutine cli_print_qtlmap_version
#ifdef QTLMAP_VERSION
#ifdef DATE_BUILD
            print *,'Release-Build:',QTLMAP_VERSION,'-',DATE_BUILD
#endif
#endif
       end subroutine cli_print_qtlmap_version
!!***

!!****f* m_qtlmap_cli/cli_print_help
!!  NAME
!!    cli_print_help
!!  DESCRIPTION
!!    Print command line interface help
!!  NOTES
!!  SOURCE
        subroutine cli_print_help
        integer        :: i,nlopt,iopt
        character(len=200) , dimension(40)  :: info
        external :: get_nb_opti_nlopt_qtlmap,info_nlopt_qtlmap

        print *,'Usage: ',trim(get_binary_name()),' [FILE_ANALYSE] [OPTIONS]...'
        print *,'QTLmap is a linkage analysis software dedicated to the analysis of outbred population in an experimental design.'
        print *,''

        !----------- ANALYSIS ----------
        print *,'Analysis options'
        print *,'----------------'
        print *,trim(OPT_CALCUL)   ,'='
        print *                    ,(' ',i=1,MARG_HELP),trim(str(ANALYSE_UNITRAIT)),&
                                       ' : Analyse Trait by trait (precorrected data) - qtl={1,2} '
        print *                    ,(' ',i=1,MARG_HELP),trim(str(ANALYSE_UNITRAIT_MODLIN)) &
                             ,' : Analyse Trait by trait with fixed effects, covariates and qtl interaction with fixed effect',&
                              ' - qtl={1} '
        print *                    ,(' ',i=1,MARG_HELP),trim(str(ANALYSE_UNITRAIT_LINEAR_LA_HOMO)),&
        ' : Linear homoscedastic LA with fixed effects and covariates (using the most probable genotype dam)  - qtl={1,..,n}'
        print *                    ,(' ',i=1,MARG_HELP),trim(str(ANALYSE_UNITRAIT_LINEAR_LA_HETERO)),&
        ' : Linear heteroscedastic LA with fixed effects and covariates (using the most probable genotype dam) - qtl={1,..,n}'
        print *                    ,(' ',i=1,MARG_HELP),trim(str(ANALYSE_MULTIRAIT)), &
                                       ' : Join traits analysis (precorrected data) - qtl={1} '
        print *                    ,(' ',i=1,MARG_HELP),trim(str(ANALYSE_MULTITRAIT_DA)),' : Joint analysis of the traits',&
        ' using a discriminant function (precorrected data) - qtl={1} - Only avalaible in NAG environment.'
        print *                    ,(' ',i=1,MARG_HELP),trim(str(ANALYSE_UNITRAIT_MODLIN_COX)),' : survival analysis with fixed ',&
                                   'effects and covariates (Cox model) - qtl={1}'
        print *                    ,(' ',i=1,MARG_HELP),trim(str(ANALYSE_UNITRAIT_LD)),' : LD analysis with fixed effects',&
                                 ' , covariates and qtl interaction with fixed effect - qtl={1} '
        print *                    ,(' ',i=1,MARG_HELP),trim(str(ANALYSE_UNITRAIT_LDLA)),' : LDLA analysis with fixed effects',&
                                 ' , covariates and qtl interaction with fixed effect - qtl={1} '
        print *                    ,(' ',i=1,MARG_HELP),trim(str(ANALYSE_UNITRAIT_LDJH)),' : LDJH analysis(dev jm) ',&
                                  'with fixed effects , covariates and qtl interaction with fixed effect - qtl={1} '
        print *                    ,(' ',i=1,MARG_HELP),trim(str(ANALYSE_UNITRAIT_CONTINGENCE)) &
                             ,' : Analyse Trait by trait with fixed effects and covariates  - qtl={1,..,n} '
        print *                    ,(' ',i=1,MARG_HELP),trim(str(ANALYSE_MULTITRAIT_INCIDENCE)), &
                                       ' : Join traits analysis with fixed effects, covariates - qtl={1,..,n} '
        print *                    ,(' ',i=1,MARG_HELP),trim(str(ANALYSE_UNITRAIT_LINEAR_LD_HOMO)),&
        ' : Linear homoscedastic LD with fixed effects and covariates (using the most probable genotype dam)  - qtl={1,..,n}'
        print *                    ,(' ',i=1,MARG_HELP),trim(str(ANALYSE_UNITRAIT_LINEAR_LD_HETERO)),&
        ' : Linear heteroscedastic LD with fixed effects and covariates (using the most probable genotype dam) - qtl={1,..,n}'
        print *                    ,(' ',i=1,MARG_HELP),trim(str(ANALYSE_UNITRAIT_LINEAR_LDLA_HOMO)),&
        ' : Linear homoscedastic LDLA with fixed effects and covariates (using the most probable genotype dam)  - qtl={1,..,n}'
        print *                    ,(' ',i=1,MARG_HELP),trim(str(ANALYSE_UNITRAIT_LINEAR_LDLA_HETERO)),&
        ' : Linear heteroscedastic LDLA with fixed effects and covariates (using the most probable genotype dam) - qtl={1,..,n}'

        print *,trim(OPT_QTL)      ,'='
        print *                    ,(' ',i=1,MARG_HELP),'1 : single QTL detection: test 0 versus 1 QTL (default)'
        print *                    ,(' ',i=1,MARG_HELP),'N : N-QTL detection: test 0 versus N QTL and 1 versus N QTL... '
        print *,trim(OPT_HDAM)
        print *                    ,(' ',i=1,MARG_HELP),'Add haplotype dam in the LD/LDLA model analysis.'
        print *,trim(OPT_CENSORED_CALCUL)
        print *                    ,(' ',i=1,MARG_HELP),'Calcul the cd of each kd from the 3th generation.'
        print *,trim(OPT_EQTL)
        print *                    ,(' ',i=1,MARG_HELP),'Perform an analysis with transcriptomic datas.'
        print *                    ,(' ',i=1,MARG_HELP),'The Traits file is reading by line and the result',&
        ' log is significantly different than a classic analysis'
        print *,trim(OPT_SNP)
        print *                    ,(' ',i=1,MARG_HELP),'Perform an analysis with SNP markers.'
        print *,trim(OPT_ESTIME_THRESHOLDS),',',trim(OPT_ESTIME_THRESHOLDS_SHORT)
        print *                    ,(' ',i=1,MARG_HELP),'Estimate the rejection thresholds.'
        !----------- SIMUL ----------
        print *,''
        print *,'Simulation options'
        print *,'------------------'
        print *,trim(OPT_SIM)
!        print *                    ,(' ',i=1,MARG_HELP),'To perform a simulation for a threshold estimation or',&
!        ' looking for a design optimization.'
        print *,trim(OPT_NSIM)     ,'='
        print *              ,(' ',i=1,MARG_HELP),'Number of simulation ','[ default=',trim(str(NSIM_DEFAULT)),'].'

        print *,trim(OPT_PERMUT)
        print *                    ,(' ',i=1,MARG_HELP),'To permut performances and associated effects between genotyped',&
        ' animals within half or full-sib families..'

        !-----------OUTPUT ----------
        print *,''
        print *,'Output options'
        print *,'------------------'
        print *,trim(OPT_XML)
        print *                    ,(' ',i=1,MARG_HELP),'XML format output file.'
        print *,trim(OPT_QUIET),',',trim(OPT_QUIET_SHORT)

        print *                    ,(' ',i=1,MARG_HELP),'Display only warning and error messages.'
        print *,trim(OPT_VERBOSE),',',trim(OPT_VERBOSE_SHORT)
        print *                    ,(' ',i=1,MARG_HELP),'Display verbose message.'

        print *,trim(OPT_DEBUG) ,',',trim(OPT_DEBUG_SHORT)
        print *                    ,(' ',i=1,MARG_HELP),'Display debug and warning message.'
        print *,trim(OPT_PRINT_CLASSIC)
        print *                    ,(' ',i=1,MARG_HELP),'Print all information in the result file (useful for eqtl analysis).'

        !----------- ADNVANCED OPTION ----------
        print *,''
        print *,'Advanced options'
        print *,'------------------'
        print *,trim(OPT_SIBF)      ,'='
        print *                    ,(' ',i=1,MARG_HELP),'1 : Half Sib family'
        print *                    ,(' ',i=1,MARG_HELP),'2 : Mixture of full and half sib family (default)'
        print *,trim(OPT_OPTIM)      ,'='
        print *                    ,(' ',i=1,MARG_HELP),'1 : * NAG * E04JYF '
        print *                    ,(' ',i=1,MARG_HELP),'2 : * L-BFGS-B * Broyden–Fletcher–Goldfarb–Shanno update',&
                               ' to approximate the Hessian matrix (L-BFGS stands for (limited memory BFGS))'
#ifdef HAVE_NLOPT
        call get_nb_opti_nlopt_qtlmap(nlopt)
        info=''
        call info_nlopt_qtlmap(info)

        do iopt=1,nlopt
           print *                    ,(' ',i=1,MARG_HELP),trim(str(OPTI_LAST+iopt))//' : * NLopt * ',trim(info(iopt))
        end do
#endif
        print *,''
        !----------- DEVEL ----------
        print *,'Devel options'
        print *,'-------------'
        print *,trim(OPT_HAPLOTYPE),'='
        print *                    ,(' ',i=1,MARG_HELP),'1 : base'
        print *                    ,(' ',i=1,MARG_HELP),'2 : PDEGM_V4,PDEGP_V4,PDED_V4'
        print *                    ,(' ',i=1,MARG_HELP),'3 : PDEGM_V4,PDEGP_V4,PDED_V5'
        print *                    ,(' ',i=1,MARG_HELP),'4 : PDEGM_FAST,PDEGP_FAST,PDED_V5'
        !-----------  HELP ---------
        print *,'Help'
        print *,'----'
        print *,trim(OPT_HELP_PANALYSE)
        print *                    ,(' ',i=1,MARG_HELP),'Print help to build p_analyse file.'
        print *,trim(OPT_HELP_PARAMSIM)
        print *                    ,(' ',i=1,MARG_HELP),'Print help to build simulation parameter file.'
        print *,trim(OPT_HELP)
        print *                    ,(' ',i=1,MARG_HELP),'Display this message.'

        print *,''
        print *,'The QTLMAP software is developed at INRA (french National Institute for Agronomical Research) to map QTL'
        print *,'following linkage analyses approaches in various experimental populations. Specific models were implemented to'
        print *,'take maximum advantage of large data sets now available for QTL detection and get fine locus descriptions'
        print *,'(indications of pleiotropy, of linked loci…) to facilitate further explorations. In QTLMAP, '
        print *,'the basis of the QTL detection technique is an interval mapping applied to half-sib families or a mixture '
        print *,'of full- and half- sib families where the exact computation of phase probability is undertaken from a 2 or 3'
        print *,'generation pedigree. The test statistics are computed either using exact calculation of likelihood ratios '
        print *,'or using partial linear approximations of the likelihood or complete regression on marker genotypes. '
        print *,''
        print *,'   ***  support : send email to qtlmap-users@listes.inra.fr. *** '
        print *,'                              *                                  '

      end subroutine cli_print_help
!!***

!!****f* m_qtlmap_cli/cli_is_help
!!  NAME
!!    cli_is_help
!!  DESCRIPTION
!!    True if the user ask help message
!!  NOTES
!!  SOURCE
      function cli_is_help() result(val)
         logical :: val
         val = key_exist(OPT_HELP)
      end function cli_is_help
!!***

!!****f* m_qtlmap_cli/cli_get_parameters_file
!!  NAME
!!    cli_get_parameters_file
!!  DESCRIPTION
!!    Get the analyse parameters file
!!  NOTES
!!  SOURCE
      function cli_get_parameters_file() result(f)
              character(len=LENGTH_MAX_FILE) :: f
              if (get_nb_arg() <=0) then
                 print *,trim(get_binary_name()),' can not find parameters file analyse (first argument).'
                 call cli_print_help
                 stop
              end if

              call get_arg(1,f)
      end function cli_get_parameters_file
!!***

!!****f* m_qtlmap_cli/cli_get_analyse
!!  NAME
!!    cli_get_analyse
!!  DESCRIPTION
!!   Get the analyse type
!!  NOTES
!!  SOURCE
      function cli_get_analyse() result(type)
         integer                :: type
         character(len=LEN_DEF)   :: value

         type = 0
         call get_key_value(OPT_CALCUL,value)
         type = to_integer(value)
         if (type == 0) type = 1  ! Default first option analyse

      end function cli_get_analyse
!!***

!!****f* m_qtlmap_cli/cli_get_haplotype
!!  NAME
!!    cli_get_haplotype
!!  DESCRIPTION
!!   Get the Haplotype version
!!  NOTES
!!  SOURCE
      function cli_get_haplotype() result(type)
        integer                :: type
         character(len=LEN_DEF)   :: value

         type = 0
         call get_key_value(OPT_HAPLOTYPE,value)
         type = to_integer(value)
         if (type /= huge(type)) then
            if (type == 0) then
               if (key_exist(OPT_SNP)) then
                 type = VERSION_HAPLOTYPE_SNP
               else
                 type = VERSION_HAPLOTYPE_V3  ! Default first option haplotype
               end if
            end if
            return
         end if

         print *,trim(get_binary_name()),' can not find ['//trim(OPT_HAPLOTYPE)//'] argument.'
         call cli_print_help
         stop

      end function cli_get_haplotype
!!***

!!****f* m_qtlmap_cli/cli_is_simulation
!!  NAME
!!    cli_is_simulation
!!  DESCRIPTION
!!   True if the user ask a simulation
!!  NOTES
!!  SOURCE
      function cli_is_simulation() result(val)
       logical :: val
       val = key_exist(OPT_SIM)
      end function cli_is_simulation
!!***

!!****f* m_qtlmap_cli/cli_is_estimate_thresholds
!!  NAME
!!    cli_is_estimate_thresholds
!!  DESCRIPTION
!!   True if the user want estimate rejection threholds of the current analysis
!!
!!  OUTPUT
!!   nsim : the number of simulation
!!  NOTES
!!  SOURCE
     function cli_is_estimate_thresholds(nsim) result(val)
        logical :: val
        integer, intent(out)   :: nsim
        character(len=LEN_DEF)   :: value

        val = key_exist(OPT_ESTIME_THRESHOLDS)
        if ( val ) then
            call get_key_value(OPT_ESTIME_THRESHOLDS,value)
        else
          val = key_exist(OPT_ESTIME_THRESHOLDS_SHORT)
          if ( val ) then
            call get_key_value(OPT_ESTIME_THRESHOLDS_SHORT,value)
          end if
        end if

        nsim = 0
        if ( .not. val ) return

        nsim = NSIM_DEFAULT ! default value
        if (trim(value)/='') then
	        nsim = to_integer(value)
	        if (nsim == huge(nsim)) then
	           print *,'bad value defined :'// trim(value) //'['//trim(OPT_NSIM)//']'
	           call cli_print_help
	           stop
	        end if
        end if

     end function cli_is_estimate_thresholds
!!***

!!****f* m_qtlmap_cli/cli_is_transcriptomic_data
!!  NAME
!!    cli_is_transcriptomic_data
!!  DESCRIPTION
!!   True if the user ask a transcriptomic analysis/simulation
!!  NOTES
!!  SOURCE
      function cli_is_transcriptomic_data() result(val)
        logical :: val
        val = key_exist(OPT_EQTL)
      end function cli_is_transcriptomic_data
!!***

!!****f* m_qtlmap_cli/cli_is_permute
!!  NAME
!!    cli_is_permute
!!  DESCRIPTION
!!   True if the user ask permutation for the current simulation
!!  NOTES
!!  SOURCE
      function cli_is_permute() result(val)
        logical :: val
        val = key_exist(OPT_PERMUT)
      end function cli_is_permute
!!***

!!****f* m_qtlmap_cli/cli_is_calcul_cd
!!  NAME
!!    cli_is_calcul_cd
!!  DESCRIPTION
!!    True if the user ask to compute a CD from a third generation with trait value
!!  NOTES
!!  SOURCE
      function cli_is_calcul_cd() result(val)
        logical :: val
        val = key_exist(OPT_CENSORED_CALCUL)
      end function cli_is_calcul_cd
!!***

!!****f* m_qtlmap_cli/cli_is_hdam
!!  NAME
!!    cli_is_hdam
!!  DESCRIPTION
!!    True if the user ask to take care about haplotype dam in the model LD and derivated LD analysis
!!  NOTES
!!  SOURCE
      function cli_is_hdam() result(val)
        logical :: val
        val = key_exist(OPT_HDAM)
      end function cli_is_hdam
!!***

!!****f* m_qtlmap_cli/cli_is_nsim
!!  NAME
!!    cli_is_nsim
!!  DESCRIPTION
!!    True if the user ask an iteration and so a simulation
!!  NOTES
!!  SOURCE
      function cli_is_nsim() result(val)
        logical :: val
        val = key_exist(OPT_NSIM)
      end function cli_is_nsim
!!***

!!****f* m_qtlmap_cli/cli_get_nsim
!!  NAME
!!    cli_get_nsim
!!  DESCRIPTION
!!    Get the number of the simulation
!!  NOTES
!!  SOURCE
      function cli_get_nsim() result(nsim)
        integer     :: nsim
        character(len=LEN_DEF)   :: value
        nsim = NSIM_DEFAULT ! default value
        call get_key_value(OPT_NSIM,value)
        if (trim(value)/='') then
	        nsim = to_integer(value)
	        if (nsim == huge(nsim)) then
	           print *,'bad value defined :'// trim(value) //'['//trim(OPT_NSIM)//']'
	           call cli_print_help
	           stop
	        end if
        end if
      end function cli_get_nsim
!!***

!!****f* m_qtlmap_cli/cli_get_nqtl
!!  NAME
!!    cli_get_nqtl
!!  DESCRIPTION
!!    Get the number of the qtl hypothesis to test
!!  NOTES
!!  SOURCE
      function cli_get_nqtl() result(nqtl)
        integer     :: nqtl
        character(len=LEN_DEF)   :: value

        nqtl = NQTL_DEFAULT ! default value
        call get_key_value(OPT_QTL,value)
        if (trim(value)/='') then
            nqtl = to_integer(value)
	        if (nqtl == huge(nqtl)) then
	           print *,'bad value defined :'// trim(value) //'['//trim(OPT_QTL)//']'
	           call cli_print_help
	           stop
	        end if
        end if
      end function cli_get_nqtl
!!***

!!****f* m_qtlmap_cli/cli_get_family
!!  NAME
!!    cli_get_family
!!  DESCRIPTION
!!    Get the type of family to analyse
!!  NOTES
!!  SOURCE
      function cli_get_family() result(o_sib)
        integer     :: o_sib
        character(len=LEN_DEF)   :: value

        o_sib = SIB_DEFAULT ! default value
        call get_key_value(OPT_SIBF,value)
        if (trim(value)/='') then
            o_sib = to_integer(value)
	        if (o_sib /= 1 .and. o_sib /= 2) then
	           print *,'bad value defined :'// trim(value) //'['//trim(OPT_SIBF)//']'
	           call cli_print_help
	           stop
	        end if
        end if
      end function cli_get_family
!!***

!!****f* m_qtlmap_cli/cli_get_optimization
!!  NAME
!!    cli_get_optimization
!!  DESCRIPTION
!!    Get the optimization number id for LA analysis with optimized method
!!  NOTES
!!    see m_qtlmap_optimization
!!  SOURCE
      function cli_get_optimization() result(o_optim)
        integer     :: o_optim
        character(len=LEN_DEF)   :: value

        o_optim = OPTIM_DEFAULT ! default value
        call get_key_value(OPT_OPTIM,value)
        if (trim(value)/='') then
            o_optim = to_integer(value)
        end if
      end function cli_get_optimization
!!***

!!****f* m_qtlmap_cli/cli_is_xmloutput
!!  NAME
!!    cli_is_xmloutput
!!  DESCRIPTION
!!    True if the user ask xml output format
!!  SOURCE
      function cli_is_xmloutput() result(val)
          logical :: val
          val = key_exist(OPT_XML)
      end function cli_is_xmloutput
!!***

!!****f* m_qtlmap_cli/cli_is_quiet
!!  NAME
!!    cli_is_quiet
!!  DESCRIPTION
!!    True if the user ask none information
!!  SOURCE
      function cli_is_quiet() result(val)
          logical :: val
          val = key_exist(OPT_QUIET)
          if (.not. val) val = key_exist(OPT_QUIET_SHORT)
      end function cli_is_quiet
!!***

!!****f* m_qtlmap_cli/cli_is_verbose
!!  NAME
!!    cli_is_verbose
!!  DESCRIPTION
!!    True if the user ask verbose information
!!  SOURCE
      function cli_is_verbose() result(val)
          logical :: val
          val = key_exist(OPT_VERBOSE)
          if (.not. val) then
             val = key_exist(OPT_VERBOSE_SHORT)
          end if
      end function cli_is_verbose
!!***

!!****f* m_qtlmap_cli/cli_is_print_all
!!  NAME
!!    cli_is_print_all
!!  DESCRIPTION
!!    True if the user ask to print all information
!!  SOURCE
      function cli_is_print_all() result(val)
          logical :: val
          val = key_exist(OPT_PRINT_CLASSIC)
      end function cli_is_print_all
!!***

!!****f* m_qtlmap_cli/cli_is_debug
!!  NAME
!!    cli_is_debug
!!  DESCRIPTION
!!    True if the user ask debug information
!!  SOURCE
      function cli_is_debug() result(val)
          logical :: val
          val = key_exist(OPT_DEBUG)
          if (.not. val) val = key_exist(OPT_DEBUG_SHORT)
      end function cli_is_debug
!!***

!!****f* m_qtlmap_cli/cli_is_help_panalyse
!!  NAME
!!    cli_is_help_panalyse
!!  DESCRIPTION
!!    True if the user ask help p_analyse information
!!  SOURCE
      function  cli_is_help_panalyse() result(val)
          logical :: val
          val = key_exist(OPT_HELP_PANALYSE)

      end function  cli_is_help_panalyse
!!***

!!****f* m_qtlmap_cli/cli_is_help_paramsim
!!  NAME
!!    cli_is_help_paramsim
!!  DESCRIPTION
!!    True if the user ask help paramsim information
!!  SOURCE
     function cli_is_help_paramsim() result(val)
          logical :: val
          val = key_exist(OPT_HELP_PARAMSIM)
     end function  cli_is_help_paramsim
!!***

!*******************************************************************************************************************************
!*
!*                               INTERNAL PROCEDURES


!!****f* m_qtlmap_cli/get_nb_arg
!!  NAME
!!    get_nb_arg
!!  DESCRIPTION
!!    get the number of option give in the command line
!!  SOURCE
      function get_nb_arg() result(nb)
         integer           :: nb

#ifdef AIX_SYS
         character(len=LEN_OPT) :: v
         v=''
         nb = 0
         call GETARG(nb+1,v)
         v = trim(v)
         do while (v/='')
           nb = nb+1
           call getarg(nb+1,v)
           v = trim(v)
         end do
#else
         nb = COMMAND_ARGUMENT_COUNT()
#endif
         return

      end function get_nb_arg
!!***

!!****f* m_qtlmap_cli/get_arg
!!  NAME
!!    get_arg
!!  DESCRIPTION
!!    get the argument value of the ith argument
!!  SOURCE
      subroutine get_arg(it,value)
       integer , intent(in)                    :: it
       character(len=*),intent(out)      :: value

#ifdef AIX_SYS
        call GETARG(it,value)
#else
        call GET_COMMAND_ARGUMENT(it,value)
#endif
      end subroutine get_arg
!!***

!!****f* m_qtlmap_cli/get_binary_name
!!  NAME
!!    get_binary_name
!!  DESCRIPTION
!!    Get the name of the binary
!!  SOURCE
      function get_binary_name() result(name)
        character(len=LEN_OPT) :: v
        character(len=LEN_DEF)   :: name
        integer                :: id

        call get_arg(0,v)
        name=trim(v)
        id=SCAN(name,'/')
        do while(id>0 .and. (id+1)<=LEN(name))
          name = trim(EXTRACT(name,id+1,LEN(name)))
          id=SCAN(name,'/')
        end do
        return
      end function get_binary_name
!!***

!!****f* m_qtlmap_cli/get_key_value
!!  NAME
!!    get_key_value
!!  DESCRIPTION
!!    Get value according to the key give in parameter
!!  SOURCE
      subroutine get_key_value(keysearch,value)
         character(len=LEN_OPT) ,intent(in)   :: keysearch
         character(len=LEN_DEF) ,intent(out)    :: value

         integer                :: i,id
         character(len=LEN_OPT) :: v
         character(len=LEN_DEF)   :: option,key

         value=''

         do i=1,get_nb_arg()
            call get_arg(i,v)
            if (trim(v) == '') cycle
            option = v
            id=SCAN(option,'=')
            if (id<=0) cycle
            key = trim(EXTRACT(option,1,id-1))

            if (key == keysearch) then
               value = trim(EXTRACT(option,id+1,LEN(option)))
               return
            end if
         end do

      end subroutine get_key_value
!!***

!!****f* m_qtlmap_cli/key_exist
!!  NAME
!!    key_exist
!!  DESCRIPTION
!!    True if the key exist in the argument command line
!!  SOURCE
      function key_exist(keysearch) result(vexist)
         character(len=LEN_OPT) ,intent(in)   :: keysearch

         logical                :: vexist
         integer                :: i,id
         character(len=LEN_OPT) :: v
         character(len=LEN_DEF)   :: option,key

         vexist=.false.
         do i=1,get_nb_arg()
            call get_arg(i,v)
            if (trim(v) == '') cycle
            if (trim(v) == keysearch) then
               vexist = .true.
               return
            end if
            option = v
            id=SCAN(option,'=')
            if (id<=0) cycle
            key = trim(EXTRACT(option,1,id-1))
            if (key == keysearch) then
               vexist = .true.
               return
            end if
         end do
      end function key_exist
!!***

!!****f* m_qtlmap_cli/to_integer
!!  NAME
!!    to_integer
!!  DESCRIPTION
!!     Get the integer conversion of value (string) if value is not a integer, get HUGE(integer)....
!!  SOURCE
      function to_integer(value) result(res_int)
             character(len=*),intent(in)      :: value
             integer                              :: res_int
             integer                              :: ios
             res_int=0
             read (value,FMT_INT,IOSTAT=ios) res_int
             if (  ios /= 0 ) res_int = 0

      end function
!!***

end module m_qtlmap_cli
