!!****p* QTLMap/OUTPUT
!!  NAME
!!    INPUT
!!  DESCRIPTION
!!  Package output :
!!
!!  CREATION DATE
!!  01/01/2009
!!  COPYRIGHT
!!***

!!****m* OUTPUT/m_qtlmap_output_handler
!!  NAME
!!    m_qtlmap_output_handler -- Print analysisresult information on a ASCII format
!!  SYNOPSIS
!!
!!  DESCRIPTION
!!
!!  NOTES
!!
!!  BUGS
!!
!!  SEE ALSO
!!
!!  COPYRIGHT
!!***
!! You can use this space for remarks that should not be included
!! in the documentation.
!!/
module m_qtlmap_output_handler
   use m_qtlmap_constant
   use qtlmap_types
   use m_qtlmap_log
   use m_qtlmap_tools
   use m_qtlmap_data
   use m_qtlmap_haplotype_data
   use m_qtlmap_math

   implicit none
   save

   integer ,private , parameter              :: BUF_ALLOC_FILE=2**14
   ! doit disparaitre....
  ! logical                      ,public      :: t_imp

   ! METTRE CES CONSTANTES EN PRIVE POUR INTERDIRE L UTILISATION EN DEHORS DE CE MODULE
   integer, public, parameter             :: nficout                = 16
   integer, public, parameter             :: nficerr                = 7
   integer, public, parameter             :: nficopti               = 18
   integer, public, parameter             :: nficpere               = 19
   integer, public, parameter             :: nficmere               = 20
   integer, public, parameter             :: unit_sire_res          = 7
   integer, public, parameter             :: unit_dam_res           = 8
   integer, public, parameter             :: unit_paternal_effects  = 61
   integer, public, parameter             :: unit_maternal_effects  = 62
   integer, public, parameter             :: unit_pded              = 1222
   integer, public, parameter             :: unit_pdedjoin          = 1223
   integer, public, parameter             :: unit_summary           = 9
   integer, public, parameter             :: unit_summary_2qtl      = 90
   integer, public, parameter             :: unit_coeff             = 80
   integer, public, parameter             :: unit_haplotypes        = 81
   integer, public, parameter             :: unit_phases            = 82
   integer, public, parameter             :: unit_freqall           = 83

   integer, public, parameter             :: MAX_GENOTYP_PRINT  = 15

   logical , private        :: xml       = .false.
   logical , private        :: file      = .true.


   public :: init_output_handler
   public :: end_output_handler
   public :: set_xml_output
   public :: set_file_output

   !General information
   !-------------------
   public :: log_descriptif_genealogy
   public :: log_marker_description
   public :: log_descriptif_traits
   public :: log_simulation_message

   !haplotype print
   !---------------
   public :: print_pded
   public :: print_dam_phase
   public :: print_sire_phase

   public :: print_freqall

   !Analyse print
   !-------------
   public :: print_start_multitraits
   public :: print_start_multitrait_DA
   public :: print_start_unitrait
   public :: print_end_multitraits
   public :: print_end_unitrait
   public :: print_courbe_LRT
   public :: print_maximum_LRT

   public :: print_transcriptome
   public :: print_transcriptome_Struct_famille


   public :: print_paternal_maternal_effect
   public :: print_pat_mat_effect_2QTL
   public :: print_LRT
   public :: print_LRT_2QTL

   public :: print_allelic_origin
   public :: print_incidence_solution
   public :: print_incidence_solution_risk_factor
   public :: print_lrt_solution

   !Simulation
   !-----------
   public :: print_resume_simulation
   public :: print_resume_simulation_2



   contains

!!****f* m_qtlmap_output_handler/init_output_handler
!! NAME
!!   init_output_handler
!! DESCRIPTION
!!
!! SOURCE
    subroutine init_output_handler(opt_qtl,opt_calcul,name_funct,nb_thread)
       integer, intent(in)                           :: opt_qtl,opt_calcul,nb_thread
       character(len=300)         , intent(in)       :: name_funct
       integer :: ios
       integer :: val_recl = BUF_ALLOC_FILE ! anciennement 2048
       character(8)  :: date
       character(10) :: time


       ios=0
       open(UNIT=nficout,file=resul, form="formatted",recl=val_recl,iostat=ios)
       if (ios/=0) then
         call stop_application("Can not open the file :"//trim(resul))
       end if

       call date_and_time(DATE=date,TIME=time)

       write (nficout,*) '    ***    '
       write (nficout,*) '       DATE      = ',date(1:4),'/',date(5:6),'/',date(7:8),'-',time(1:2),':',time(3:4),':',time(5:6)
       write (nficout,*) '       --QTL     = ',opt_qtl
       write (nficout,*) '       --CALCUL  = ',opt_calcul,' (',trim(name_funct),')'
       write (nficout,*) 'OMP_NUM_THREADS  = ',nb_thread

       open(UNIT=unit_summary,file=summary, form="formatted",recl=val_recl,iostat=ios)
       if (ios/=0) then
         call stop_application("Can not open the file :"//trim(summary))
       end if

       if ( trim(resp)/='' ) then
          open(UNIT=unit_sire_res,file=resp, form="formatted",recl=val_recl,iostat=ios)
          if (ios/=0) then
            call stop_application("Can not open the file :"//trim(resp))
          end if
       end if

       if ( trim(resm)/='' ) then
         open(UNIT=unit_dam_res,file=resm, form="formatted",recl=val_recl,iostat=ios)
         if (ios/=0) then
            call stop_application("Can not open the file :"//trim(resm))
          end if
       end if

       if ( trim(pateff)/='' ) then
         open(UNIT=unit_paternal_effects,file=pateff,form="formatted",recl=val_recl,iostat=ios)
         if (ios/=0) then
            call stop_application("Can not open the file :"//trim(pateff))
         end if

         write(unit_paternal_effects,*) ' ********************************************* '
         write(unit_paternal_effects,*) ' This file is unvalide if interaction qtl case '
         write(unit_paternal_effects,*) ' ********************************************* '

       end if

       if ( trim(mateff)/='' ) then
          open(UNIT=unit_maternal_effects,file=mateff,form="formatted",recl=val_recl,iostat=ios)
          if (ios/=0) then
            call stop_application("Can not open the file :"//trim(mateff))
          end if

          write(unit_maternal_effects,*) ' ********************************************* '
          write(unit_maternal_effects,*) ' This file is unvalide if interaction qtl case '
          write(unit_maternal_effects,*) ' ********************************************* '
       end if

       if ( trim(out_phases)=='' ) then
           out_phases = trim(resul)//'_phases'
       end if

       open(UNIT=unit_phases,file=out_phases,form="formatted",recl=val_recl,iostat=ios)
       if (ios/=0) then
           call stop_application("Can not open the file :"//trim(out_phases))
       end if

       if ( trim(out_freqall)=='' ) then
         out_freqall = trim(resul)//'_freqall'
       end if

       open(UNIT=unit_freqall,file=out_freqall,form="formatted",recl=val_recl,iostat=ios)
       if (ios/=0) then
           call stop_application("Can not open the file :"//trim(out_freqall))
       end if

       if ( trim(out_haplotypes)/='' ) then
            open(UNIT=unit_haplotypes,file=out_haplotypes,form="formatted",recl=val_recl,iostat=ios)
            if (ios/=0) then
              call stop_application("Can not open the file :"//trim(out_haplotypes))
            end if
       end if

       if (opt_qtl == 2) then
         sum_2qtl=trim(summary)//'2qtl'
         open(UNIT=unit_summary_2qtl,file=sum_2qtl, form="formatted",recl=val_recl,iostat=ios)
         if (ios/=0) then
            call stop_application("Can not open the file :"//trim(sum_2qtl))
          end if

       end if
    end subroutine init_output_handler
!!***

!!****f* m_qtlmap_output_handler/end_output_handler
!! NAME
!!   end_output_handler
!! DESCRIPTION
!!
!! SOURCE
    subroutine end_output_handler(opt_qtl)
       integer, intent(in)     :: opt_qtl
       character(8)  :: date
       character(10) :: time

       if ( trim(resp)/='' ) close(unit_sire_res)
       if ( trim(resm)/='' ) close(unit_dam_res)
       close(unit_summary)
        if (opt_qtl == 2) then
         close(unit_summary_2qtl)
       end if

       if ( trim(pateff)/='' ) close(unit_paternal_effects)
       if ( trim(mateff)/='' ) close(unit_maternal_effects)

       if ( trim(out_phases)/='' ) close(unit_phases)
       if ( trim(out_haplotypes)/='' ) close(unit_haplotypes)
       if ( trim(out_freqall)/='' ) close(unit_freqall)

       call date_and_time(DATE=date,TIME=time)

       write (nficout,*) '    ***    '
       write (nficout,*) '    DATE      = ',date(1:4),'/',date(5:6),'/',date(7:8),'-',time(1:2),':',time(3:4),':',time(5:6)
       write (nficout,*) '    ***    '
       close(nficout)

    end subroutine end_output_handler
!!***

!!****f* m_qtlmap_output_handler/set_file_output
!! NAME
!!   set_file_output
!! DESCRIPTION
!!
!! SOURCE
    subroutine set_file_output(active)
       logical, intent(in)     :: active
       file = active
    end subroutine set_file_output
!!***

!!****f* m_qtlmap_output_handler/set_xml_output
!! NAME
!!   set_xml_output
!! DESCRIPTION
!!
!! SOURCE
    subroutine set_xml_output(active)
       logical, intent(in)     :: active
       xml = active
    end subroutine set_xml_output
!!***

!!****f* m_qtlmap_output_handler/print_summary_panalyse
!! NAME
!!   print_summary_panalyse
!! DESCRIPTION
!!
!! SOURCE
    subroutine print_summary_panalyse(n,index_key,values)
             integer                           ,intent(in) :: n
             character(len=LEN_L) ,dimension(:),intent(in) :: index_key
             character(len=LEN_L) ,dimension(:),intent(in) :: values
             integer :: i
             call log_mess("",VERBOSE_DEF)
             call log_mess("-------------------------------",VERBOSE_DEF)
             call log_mess("********* P_ANALYSE KEYS ******",VERBOSE_DEF)
             call log_mess("-------------------------------",VERBOSE_DEF)

              write(nficout,FMT='(/,/,5x,'                 // &
        '"*****************  PARAMETERS ANALYSE FILE SUMMARY *****************",/)')
             do i=1,n
                   write (nficout,FMT="(a40,'=',a)") index_key(i),values(i)
                   call log_mess("["//trim(index_key(i))//"]-->"//trim(values(i)),VERBOSE_DEF)
             end do

          end subroutine print_summary_panalyse
!!***

!!****f* m_qtlmap_output_handler/log_descriptif_genealogy
!! NAME
!!   log_descriptif_genealogy
!! DESCRIPTION
!!
!! SOURCE
      subroutine log_descriptif_genealogy(file_gena)
          character(len=LENGTH_MAX_FILE)   , intent(in)  :: file_gena
          integer                                        :: ip,ir,im

          character(len=LEN_DEF)                           :: unknown_r

           write(nficout,FMT='(/,/,5x,'                 // &
        '"*****************  GENEALOGY DESCRIPTION *****************",/)')


          write (nficout,FMT='(1x,"The pedigree file includes ",i3," parents",'    //  &
                  '" born from",i3," grand sires and ",i3," grand dams")') &
                  size(repro),size(gpere),size(gmere)

          write(nficout,FMT='(1x,"and ",i4," progeny born from",'           // &
                  'i3," sires and ",i3," dams")')size(animal),size(pere),size(mere)

          unknown_r=''
          do ip=1,size(pere)
             do ir=1,size(repro)
              if(pere(ip) == repro(ir)) then
                exit
              endif
             end do
            if (ir > size(repro) ) then
              unknown_r = unknown_r//' '//trim(pere(ip))
            end if
          end do
          if (unknown_r /= '' ) then
             write(nficout,FMT='(1x,"Sires",a10," have no known ancestor")') trim(unknown_r)
          end if

          unknown_r=''
          do im=1,size(mere)
             do ir=1,size(repro)
              if(mere(im) == repro(ir)) then
                exit
              endif
             end do
            if (ir > size(repro) ) then
               unknown_r = unknown_r//' '//trim(mere(im))
            end if
          end do

          if (unknown_r /= '' ) then
             write(nficout,FMT='(1x,"Dams ",a10," have no known ancestor")') trim(unknown_r)
          end if

      end subroutine log_descriptif_genealogy
!!***

!!****f* m_qtlmap_output_handler/log_marker_description
!! NAME
!!   log_marker_description
!! DESCRIPTION
!!
!! SOURCE
    subroutine  log_marker_description()
       integer                                 :: i,j,k,jm,ip,igp,jgm
       integer                                 :: nmes0,l
       character(len=LEN_DEF) , dimension(:),allocatable    :: num_t

       allocate( num_t (size(numero)) )
       num_t=''
       !! compute number of animal which are not informative
       nmes0 = 0
       do i=1,size(animal)
          if ( count(presentg(:,i)) == 0 ) nmes0 = nmes0 + 1
       end do
       l = 0
       do i=1,nmes
          do j=1,size(animal)
              if (numero(i) == animal(j)) then
                exit
              endif
          end do

          if ( j <= size(animal) ) then
             cycle
          end if

          do jm=1,size(mere)
             if (mere(jm) == numero(i)) exit
          enddo
          if ( jm <= size(mere) ) cycle
          do ip=1,size(pere)
             if (pere(ip) == numero(i)) exit
          enddo
          if ( ip <= size(pere) ) cycle
          do  jgm=1,size(gmere)
             if(gmere(jgm) == numero(i)) exit
          enddo
          if ( jgm <= size(gmere) ) cycle
          do igp=1,size(gpere)
             if(gpere(igp) == numero(i)) exit
          enddo
          if ( igp <= size(gpere) ) cycle
          l = l + 1
          num_t(l)=numero(i)
       end do

      write(nficout,FMT='(/,/,5x,'                 // &
        '"***************** MARKER DESCRIPTION *****************",/)')

      write (nficout,FMT='(i5, " animals are present in the genotype file ")') size(animal)

     if (l > 0) then
       if ( l<50) then
       write(nficout,*) 'animal',(trim(num_t(i))//' ',i=1,l),'of genotype file ', &
                'are not in the pedigree file'
       else
           write(nficout,*) l,' animals of genotype file are not in the pedigree file'
       end if

     end if

     deallocate( num_t )



      if (nmes0.eq.0) then
            write(nficout,*) 'where all animals are genotyped for at least ', &
              'one marker.'
      else
            write(nficout,*) 'where',nmes0,'animals have no genotyped marker .'
      endif

      write(nficout,FMT='("markers were selected among ",i4," markers")') size(mark)
      write(nficout,FMT='("There are ",i4," genotyped animals")') (size(animal)-nmes0)

      if (trim(out_freqall)/='' ) then
        write (nficout,*) " ** Check allele frequency in the file :",trim(out_freqall),' **'
      end if

    end subroutine log_marker_description
!!***

!!****f* m_qtlmap_output_handler/print_freqall
!! NAME
!!   print_freqall
!! DESCRIPTION
!!
!! SOURCE
    subroutine print_freqall

      integer :: i,j,k

      do i=1,nchr ! chr
        do j=1,nmk(i) ! mark/chr
         write(unit_freqall, &
         FMT='(a," (Chr ",a,")"," has",i3," alleles:",90(" ",a,"(",f5.1,"%) ",a,"(",f5.1,"%)"))') &
         trim(mark(i,j)),trim(chromo(i)),nall(i,j), ( trim(alleles(i,j,k)),pc_all(i,j,k), k=1,nall(i,j))
        end do
      end do

    end subroutine print_freqall
!!***

!!****f* m_qtlmap_output_handler/log_descriptif_traits
!! NAME
!!   log_descriptif_traits
!! DESCRIPTION
!!
!! SOURCE
    subroutine log_descriptif_traits()
       integer                                        :: ngeno,kd,ic,ifx,ilev,ico,alloc_stat
       integer                                        :: iperf,i,nnn,jfx,l
       real (kind=dp)                                 :: sy,sy2
       real (kind=dp), dimension (:), allocatable     :: covmu,covsig,covmin,covmax,ymax,ymin
       integer, dimension (:), allocatable            :: ncens,nmanq
       character(len=LEN_DEF) , dimension(size(bete))   :: manq

       write(nficout,FMT='(/,/,'    // &
         '"    ***************** TRAITS DESCRIPTION *****************",/)')!  // &
!         '"The performance(s) file(s) is ",15(a10),/)') perfs

       l = 0

       do iperf=1,size(bete)
        do kd=1,size(animal)
          if(animal(kd) == bete(iperf)) exit
        enddo
        if ( kd <= size(animal)) cycle
        l = l + 1
        manq(l)=bete(iperf)
       end do
       if (l > 0) then
       write(nficout,*) 'animal ',(trim(manq(i))//' ',i=1,l),' of performance file are not in the pedigree file'
      end if
      !! ---------  BUFFER ARRAY FOR LOG ----------------------------------------------------------------
      allocate ( covmu(ncov), STAT = alloc_stat )
      call check_allocate(alloc_stat,'covmu')
      allocate ( covsig(ncov), STAT = alloc_stat )
      call check_allocate(alloc_stat,'covsig')
      allocate ( covmin(ncov), STAT = alloc_stat )
      call check_allocate(alloc_stat,'covmin')
      allocate ( covmax(ncov), STAT = alloc_stat )
      call check_allocate(alloc_stat,'covmax')
      allocate ( ncens(ncar), STAT = alloc_stat )
      call check_allocate(alloc_stat,'ncens')
      allocate ( nmanq(ncar), STAT = alloc_stat )
      call check_allocate(alloc_stat,'nmanq')
      allocate ( ymax(ncar), STAT = alloc_stat )
      call check_allocate(alloc_stat,'ymax')
      allocate ( ymin(ncar), STAT = alloc_stat )
      call check_allocate(alloc_stat,'ymin')

      do i=1, ncar
          ncens(i)=0
          nmanq(i)=0
          ymax(i)=0.d0
          ymin(i)=INIFINY_REAL_VALUE
          do kd=1,nd
             if((presentc(i,kd)) .and. ndelta(i,kd) == 0) ncens(i)=ncens(i)+1
             if(.not. presentc(i,kd)) nmanq(i)=nmanq(i)+1
             if(presentc(i,kd)) then
                if(y(i,kd).GT.ymax(i)) ymax(i)=y(i,kd)
                if(y(i,kd).LT.ymin(i)) ymin(i)=y(i,kd)
             endif
          enddo
        enddo

        do ico=1,ncov
          covmax(ico)=0.d0
          covmin(ico)=INIFINY_REAL_VALUE
          sy=0.d0
          nnn=0
          sy2=0.d0
          covmu(ico)=0.d0
          covsig(ico)=0.d0
          do kd=1,nd
              if(covar(kd,ico) .ne. INIFINY_REAL_VALUE) then
                 if(covar(kd,ico).GT.covmax(ico)) covmax(ico)=covar(kd,ico)
                 if(covar(kd,ico).LT.covmin(ico)) covmin(ico)=covar(kd,ico)
                 sy=sy+covar(kd,ico)
                 nnn=nnn+1
              endif
           enddo
           covmu(ico)=sy/(dble(nnn))
           do kd=1,nd
             sy2=sy2+((covar(kd,ico)-covmu(ico))*(covar(kd,ico)-covmu(ico)))
          enddo
          covsig(ico)=sqrt(sy2/(dble(nnn-1.d0)))
       enddo

      !!--------------------------------------------------------------------------

     ! IMPRESSION DU DESCRIPTIF DES DONNEES

         ngeno=0
         do kd=1,nd
           do ic=1, size(presentc,1)
             !si il existe au moin un chromosome genotype pour l animal kd
             if ((count(presentg(:,kd)) >= 1) .and. presentc(ic,kd))then
               ngeno=ngeno+1
               exit
             end if
          end do
        end do


         write(nficout,FMT='(/,5x,"NUMBER OF PHENOTYPED ANIMALS   : ",i5/,'               // &
         '5x,"NUMBER OF PHENOTYPED AND GENOTYPED ANIMALS : ",i5/,'              // &
         '5x,"NUMBER OF TRAITS               : ",i5/,'                          // &
         '5x,"NUMBER OF FIXED EFFECTS        : ",i5/,'                          // &
         '5x,"NUMBER OF COVARIABLES          : ",i5)') size(bete),ngeno, ncar, nfix, ncov


      !POUR LES COVARIABLES

      if( nfix /= 0 ) then
         do ifx=1,nfix
             write(nficout,*) 'FIXED EFFECT Num:', ifx,trim(namefix(ifx)),         &
                             'HAS', nlev(ifx),'LEVELS: ',                         &
                             (trim(listelev(ifx,ilev))//" ",ilev=1,nlev(ifx))
         end do
      end if

      if(ncov /= 0) then
        do ico=1,ncov
           write(nficout,FMT='(" COVARIABLE Num",i2,", ", a15,'                             // &
                '" MEAN = ",f8.3,"+-",f8.3," (MIN=",f8.3,",MAX=",f8.3,")")')&
                 ico,trim(trim(namecov(ico))),covmu(ico),covsig(ico), covmin(ico),covmax(ico)
        end do
       end if

     !
     ! POUR CHAQUE CARACTERE
     !
      do ic=1, ncar
         if ((natureY(ic) == 'r' ).or.( natureY(ic) == 'a')) then
           write (nficout,FMT= '(/,1x,"TRAIT :",a6/, 5x                                    '  // &
                 ',"NUMBER OF PHENOTYPED PROGENY               : ",i5/, 5x   '  // &
                 ',"MEANS                                      : ",f8.3, "+-"'  // &
                 ',f8.3/, 5x,"MINIMUM                                    : " '  // &
                 ',f8.3/, 5x,"MAXIMUM                                    :",'  // &
                 'f8.3/ , 5x,"NUMBER OF MISSING PHENOTYPES               : ",'  // &
                 'i5)')&
            trim(trim(carac(ic))),(nd-nmanq(ic)),xmut(ic),sigt(ic),(ymin(ic)*sigt(ic)+xmut(ic)),&
                                                                (ymax(ic)*sigt(ic)+xmut(ic)),nmanq(ic)
         endif
         if ( natureY(ic) == 'i' ) then
           write (nficout,FMT= '(/,1x,"TRAIT :",a6/, 5x                                    '  // &
                 ',"NUMBER OF PHENOTYPED PROGENY               : ",i5/   '  // &
                 ',5x,"MINIMUM                                    : " '  // &
                 ',f8.3/, 5x,"MAXIMUM                                    :",'  // &
                 'f8.3/ , 5x,"NUMBER OF MISSING PHENOTYPES               : ",'  // &
                 'i5)')&
            trim(trim(carac(ic))),(nd-nmanq(ic)),ymin(ic),ymax(ic),nmanq(ic)

             write (nficout,FMT= '(/,1x,"  Class  ",3x," Frequency ")')

             do i=1,nmod(ic)
                write (nficout,FMT= '(1x,i9,3x,f5.3)') indicemod(ic,i),prop(ic,i)
             end do

          end if

!    if (opt_calcul == ANALYSE_UNITRAIT_MODLIN_COX) then
            write(nficout,FMT='(  5x,"NUMBER OF CENSORED PHENOTYPES              :",I5)') ncens(ic)
!         end if

             !POUR LES COVARIABLES
            if  (modele(ic,1).ne.0.AND.modele(ic,2).eq.0.AND.modele(ic,3).eq.0) then
                 write(nficout,*) '     MODEL = mu',(' + ',trim(namefix(modele(ic,3+ifx))),ifx=1,modele(ic,1))

            else if  (modele(ic,1).eq.0.AND.modele(ic,2).ne.0.AND.modele(ic,3).eq.0) then
                 write(nficout,*) '     MODEL = mu',(' + ',trim(namecov(modele(ic,3+ico))),ico=1,modele(ic,2))

            else if  (modele(ic,1).ne.0.AND.modele(ic,2).ne.0.AND.modele(ic,3).eq.0) then
                 write(nficout,*) '     MODEL = mu',(' + ',trim(namefix(modele(ic,3+ifx))),ifx=1,modele(ic,1)),      &
               (' + ',trim(namecov(modele(ic,3+modele(ic,1)+ico))),ico=1,modele(ic,2))

            else if  (modele(ic,1).ne.0.AND.modele(ic,2).eq.0.AND.modele(ic,3).ne.0) then
                  write(nficout,*) '     MODEL = mu',(' + ',trim(namefix(modele(ic,3+ifx))),ifx=1,modele(ic,1)),     &
                 (' + QTL*',trim(namefix(modele(ic,3+modele(ic,1)+modele(ic,2)+ifx))),ifx=1,modele(ic,3))

            else if  (modele(ic,1).ne.0.AND.modele(ic,2).ne.0.AND.modele(ic,3).ne.0) then
                  write(nficout,*) '     MODEL = mu',(' + ',trim(namefix(modele(ic,3+ifx))),ifx=1,modele(ic,1)),     &
                (' + ',trim(namecov(modele(ic,3+modele(ic,1)+ico))),ico=1,modele(ic,2)),(' + QTL*',                   &
                 trim(namefix(modele(ic,3+modele(ic,1)+modele(ic,2)+jfx))),jfx=1,modele(ic,3))

            else if  (modele(ic,1).eq.0.AND.modele(ic,2).eq.0.AND.modele(ic,3).ne.0) then
                 write(nficout,*) '     MODEL = mu',(' + QTL*',trim(namefix(modele(ic,3+modele(ic,1)+modele(ic,2)+jfx))), &
                 jfx=1,modele(ic,3))

            else if  (modele(ic,1).eq.0.AND.modele(ic,2).ne.0.AND.modele(ic,3).ne.0) then
                  write(nficout,*) '     MODEL = mu',(' + ',trim(namecov(modele(ic,3+modele(ic,1)+ico))),              &
                 ico=1,modele(ic,2)),(' + QTL*',trim(namefix(modele(ic,3+modele(ic,1)+modele(ic,2)+ifx))),             &
                ifx=1,modele(ic,3))
            else
               write(nficout,*)'WITHOUT MODEL for fixed effects and covariables'
            end if
        end do

      deallocate ( covmu )
      deallocate ( covsig )
      deallocate ( covmin )
      deallocate ( covmax )
      deallocate ( ncens )
      deallocate ( nmanq )
      deallocate ( ymax )
      deallocate ( ymin )

    end subroutine log_descriptif_traits
!!***

!!****f* m_qtlmap_output_handler/log_simulation_message
!! NAME
!!   log_simulation_message
!! DESCRIPTION
!!
!! SOURCE
      subroutine log_simulation_message(simulMap,h2,ue,dens,nalle,taille)
         logical , intent(in)          :: simulMap
         integer         , intent(in)  :: nalle
         real (kind=dp)  ,intent(in)   :: dens,taille
         real (kind=dp)  ,dimension(:),intent(in)   :: h2
         real (kind=dp)  ,dimension(:,:),intent(in)   :: ue

         integer                              :: ic,iq

         write(nficout,*)'SIMULATION FOR NEW MOLECULAR DATA IS ONLY ',&
         'AVAILABLE FOR REGULARLY SPACED MARKERS AND EQUAL',          &
         ' ALLELE FREQUENCIES'

        if ( simulMap ) then
           write(nficout,FMT="(1x,a8,'|',1x,f4.3,' M')") 'Densite ',dens
           write(nficout,FMT="(1x,a8,'|',2x,i3,' alleles')") 'Nb all  ',nalle
           write(nficout,FMT="(1x,a8,'|',2x,f3.1,' M')") 'LongChr ',taille
           write(nficout,FMT="(1x,a8,'|',2x,i3,' marq')") 'NbMarq  ',nmk(1)
        end if

        if ( size(h2) <= 0 ) return

        if ( size(h2) == size(carac)) then
          write(nficout,*)
          write(nficout,*) 'Trait    heritabil effetsQTL'
          write(nficout,*)
          do ic=1,size(carac)
            write(nficout,'(1x,a5,1x,i2,1x,40(f8.3,2x))')'trait  ',ic,h2(ic), &
             (ue(ic,iq), iq=1,size(ue,2))
          end do
        else
          write(nficout,*)
          write(nficout,*) 'Trait  effetsQTL'
          write(nficout,*)
          do ic=1,size(carac)
            write(nficout,'(1x,a5,1x,i2,1x,40(f8.3,2x))')'trait  ',ic, &
            (ue(ic,iq), iq=1,size(ue,2))
          end do
        end if

      end subroutine log_simulation_message
!!***

!!****f* m_qtlmap_output_handler/print_start_multitraits
!! NAME
!!   print_start_multitraits
!! DESCRIPTION
!!
!! SOURCE
    subroutine print_start_multitraits()

        write(nficout,1100)
1100      format(//9x,36('*')/9x,'*',34x,'*'/9x,            &
               '*  Joint analysis of the traits    *'/9x,    &
               '*',34x,'*'/9x,36('*')//)

    end subroutine print_start_multitraits
!!***

!!****f* m_qtlmap_output_handler/print_start_multitrait_DA
!! NAME
!!   print_start_multitrait_DA
!! DESCRIPTION
!!
!! SOURCE
    subroutine print_start_multitrait_DA

        write(nficout,1100)
1100      format(//9x,36('*')/9x,'*',34x,'*'/9x,            &
               '*  Joint analysis of the traits    *'/9x,    &
               '*  using a discriminant  function  *'/9x,    &
               '*',34x,'*'/9x,36('*')//)

    end subroutine print_start_multitrait_DA
!!***

!!****f* m_qtlmap_output_handler/print_start_unitrait
!! NAME
!!   print_start_unitrait
!! DESCRIPTION
!!
!! SOURCE
    subroutine print_start_unitrait(name_trait)
          character(len=LEN_DEF) ,intent(in)   :: name_trait

          write(nficout,1000) trim(name_trait)

1000      format(//9x,36('*')/9x,'*',34x,'*'/9x,'*  Analysis of trait      ',a8,' *'/9x,'*',&
                                     34x,'*'/9x,'*',34x,'*'/9x,36('*')//)
                  write(nficout,1001)np,nfem
1001      format(1x,'LRT profile on the linkage group :'/1x,' position, test statistic  , '/3x,i3,&
                ' sire QTL effects , '/3x,i3,' dam QTL effects')

    end subroutine print_start_unitrait
!!***

!!****f* m_qtlmap_output_handler/print_courbe_LRT
!! NAME
!!   print_courbe_LRT
!! DESCRIPTION
!!
!! SOURCE
    subroutine print_courbe_LRT(lrtsol)
         type(TYPE_LRT_SOLUTION)       ,intent(in)    :: lrtsol

         integer                       :: ipos,chr,nlong,s
         real       ,dimension(size(lrtsol%lrt1,2))    :: x_p,y_p

         do chr=1,nchr
          nlong=get_ilong(chr)
          s=0
          do ipos=1,nlong,pas
           s=s+1
           x_p(s) = absi(chr,s)
           y_p(s) = lrtsol%lrt1(chr,s)
          end do

          write (nficout,*) "  ** chromosome  "//trim(chromo(chr))//" ** "
          call plott(y_p,x_p,s,nficout)
         end do
    end subroutine print_courbe_LRT
!!***

!!****f* m_qtlmap_output_handler/print_transcriptome_H0
!! NAME
!!   print_transcriptome_H0
!! DESCRIPTION
!!
!! SOURCE
    subroutine print_transcriptome_H0(listincsol)
       type(TYPE_INCIDENCE_SOLUTION) , intent(in) ,dimension(size(carac))    :: listincsol

       type(TYPE_INCIDENCE_SOLUTION)      :: incsol
       character(len=LEN_LINE)                                 :: fmt,title,likelyhood
       integer                                            :: ip,j,i,nbvalue,g,icar,ind
       real (kind=dp)       , dimension(:)  ,allocatable         :: values

       title=""

       ! on compte le nombre de valeur que doit contenir le tableau de transcrit
       ! + les position et les LRT selon l hypothese
       nbvalue = 0

       !construction du titre
       ! on prend comme reference le premier caractere pour l'estimabilite des effets
       do i=1,size(listincsol(1)%groupeName)
           if ( .not. listincsol(1)%eqtl_print(i) ) cycle
           if ( listincsol(1)%nbParameterGroup(i) == 1 ) then
                title=trim(title)//"["//trim(listincsol(1)%groupeName(i))//"] "
                nbvalue = nbvalue +1
           else
              title=trim(title)//"[ *"//trim(listincsol(1)%groupeName(i))//"* "
              do g=1,listincsol(1)%nbParameterGroup(i)
                  title=trim(title)//trim(listincsol(1)%parameterName(i,g))
                  if ( g <  listincsol(1)%nbParameterGroup(i) )title=trim(title)//","
                  nbvalue = nbvalue +1
              end do
              title=trim(title)//"] "
           end if
       end do

       fmt="(1x,a20,"// trim(str(nbvalue+np))//"f7.3)"
       likelyhood=""
       likelyhood=trim(likelyhood)//trim('[ *std dev *')

       do i=1,np
        likelyhood=trim(likelyhood)//trim(pere(i))
        if ( i< np) likelyhood=trim(likelyhood)//','
       end do
       likelyhood=trim(likelyhood)//']'

       write(nficout,*) 'Hypothesis :0'
       write(nficout,*) 'Given parameters are respectively :'
       write(nficout,*) 'Gene position on the array, '//trim(likelyhood)//trim(title)
       write(nficout,*)
       write(nficout,*) 'note : 0.0 means not estimable '
       write(nficout,*)

       allocate(values(np+nbvalue))
       values=0.0

       do icar=1,size(listincsol)
          incsol=listincsol(icar)
          ind=0
          do i=1,np
            ind=ind+1
            values(ind)=incsol%sig(1,i)
          end do

          do i=1,size(listincsol(1)%groupeName)
           if ( .not. listincsol(1)%eqtl_print(i) ) cycle
           if ( listincsol(1)%nbParameterGroup(i) == 1 ) then
              ind=ind+1
              if ( incsol%parameterVecsol(i,1) ) then
                values(ind)=incsol%paramaterValue(i,1)
              else
                values(ind)=0.d0
              end if
           else
              do g=1,listincsol(1)%nbParameterGroup(i)
                ind=ind+1
                if ( incsol%parameterVecsol(i,g) ) then
                  values(ind)=incsol%paramaterValue(i,g)
                else
                  values(ind)=0.d0
                end if
              end do
           end if
          end do
          write(nficout,FMT=fmt)trim(carac(icar)),(values(i),i=1,ind)
       end do


       deallocate(values)
    end subroutine print_transcriptome_H0
!!***

!!****f* m_qtlmap_output_handler/print_transcriptome
!! NAME
!!   print_transcriptome
!! DESCRIPTION
!!
!! SOURCE
    subroutine print_transcriptome(listlrtsol,listincsol)
       type(TYPE_LRT_SOLUTION)       , intent(in) ,dimension(size(carac))    :: listlrtsol
       type(TYPE_INCIDENCE_SOLUTION) , intent(in) ,dimension(size(carac))    :: listincsol

       type(TYPE_LRT_SOLUTION)            :: lrtsol
       type(TYPE_INCIDENCE_SOLUTION)      :: incsol
       character(len=LEN_LINE)                             :: fmt,title,likelyhood
       integer                                             :: ip,j,i,nbvalue,g,icar,ind,nbpos
       real (kind=dp)       , dimension(:)  ,allocatable   :: values

       title=""

       ! on compte le nombre de valeur que doit contenir le tableau de transcrit
       ! + les position et les LRT selon l hypothese
       nbvalue = 0

       !construction du titre
       ! on prend comme reference le premier caractere pour l'estimabilite des effets
       do i=1,size(listincsol(1)%groupeName)
           if ( .not. listincsol(1)%eqtl_print(i) ) cycle
           if ( listincsol(1)%nbParameterGroup(i) == 1 ) then
                title=trim(title)//"["//trim(listincsol(1)%groupeName(i))//"] "
                nbvalue = nbvalue +1
           else
              title=trim(title)//"[ *"//trim(listincsol(1)%groupeName(i))//"* "
              do g=1,listincsol(1)%nbParameterGroup(i)
                  title=trim(title)//trim(listincsol(1)%parameterName(i,g))
                  if ( g <  listincsol(1)%nbParameterGroup(i) )title=trim(title)//","
                  nbvalue = nbvalue +1
              end do
              title=trim(title)//"] "
           end if
       end do

       fmt="(1x,a20,"//trim(str(listlrtsol(1)%nqtl))//"(1x,a5,1x,f7.3)"//trim(str(listlrtsol(1)%nqtl+nbvalue+np))//"(f7.3,1x))"

       likelyhood=""

       do i=1,listlrtsol(1)%nqtl
         likelyhood=trim(likelyhood)//&
         'Chromosome '//trim(str(i))//', QTL Position '//trim(str(i))//','
       end do
       do i=1,listlrtsol(1)%nqtl
         likelyhood=trim(likelyhood)//&
         'H'//trim(str(i-1))//"/H"//trim(str(listlrtsol(1)%nqtl))//","
       end do


       likelyhood=trim(likelyhood)//'[ *std dev *'

       do i=1,np
        likelyhood=trim(likelyhood)//pere(i)
        if ( i< np) likelyhood=trim(likelyhood)//','
       end do
       likelyhood=trim(likelyhood)//']'

       write(nficout,*) 'Hypothesis :'//trim(str(listlrtsol(1)%nqtl))
       write(nficout,*) 'Given parameters are respectively :'
       write(nficout,*) 'Gene position on the array, '//trim(likelyhood)//trim(title)
       write(nficout,*)
       write(nficout,*) 'note : 0.0 means not estimable '
       write(nficout,*)

       allocate(values(np+nbvalue+3*size(absi,2)))
       values=0.0

       do icar=1,size(listincsol)
          incsol=listincsol(icar)
          lrtsol=listlrtsol(icar)
          ind=0

          do i=1,np
            ind=ind+1
            values(ind)=incsol%sig(1,i)
          end do

          do i=1,size(listincsol(1)%groupeName)
           if ( .not. listincsol(1)%eqtl_print(i) ) cycle
           if ( listincsol(1)%nbParameterGroup(i) == 1 ) then
              ind=ind+1
              if ( incsol%parameterVecsol(i,1) ) then
                values(ind)=incsol%paramaterValue(i,1)
              else
                values(ind)=0.d0
              end if
           else
              do g=1,listincsol(1)%nbParameterGroup(i)
                ind=ind+1
                if ( incsol%parameterVecsol(i,g) ) then
                  values(ind)=incsol%paramaterValue(i,g)
                else
                  values(ind)=0.d0
                end if
              end do
           end if
          end do

        write(nficout,FMT=fmt)trim(carac(icar)),(chromo(lrtsol%chrmax(i)),absi(lrtsol%chrmax(i),lrtsol%nxmax(i)),i=0,&
        lrtsol%nqtl-1),(lrtsol%lrtmax(i),i=0,lrtsol%nqtl-1),(values(i),i=1,ind)
       end do


       deallocate(values)
    end subroutine print_transcriptome
!!***

!!****f* m_qtlmap_output_handler/print_transcriptome_Struct_famille
!! NAME
!!   print_transcriptome_Struct_famille
!! DESCRIPTION
!!
!! SOURCE
   subroutine print_transcriptome_Struct_famille
      integer  , dimension(:,:) , allocatable :: lestypes      ! correspondance type avec lmes caracteres associe a ce type
      integer  , dimension(:)   , allocatable :: nbcarbytype   ! nombre de carac dans le type
      logical  , dimension(:,:) , allocatable :: struct_fam    ! pour chaque type on stocke le profil c.a.d : presence du phenotype pour l individu kd
      integer                                 :: ntype = 0
      character(len=LEN_L) :: listUnknown
      integer :: kd,ic,typ

      allocate (lestypes (ncar,ncar))
      allocate (nbcarbytype(ncar))
      allocate (struct_fam (ncar,nd))
      nbcarbytype=0

      ntype=1

      struct_fam(ntype,:)=presentc(1,:)
      nbcarbytype(ntype)=1
      lestypes(ntype,nbcarbytype(ntype))=1


      do ic=2,ncar
         do typ=1,ntype
           if ( all(presentc(ic,:) .EQV. struct_fam(typ,:))) then
               !on a trouve le meme profil
               nbcarbytype(typ)=nbcarbytype(typ)+1
               lestypes(typ,nbcarbytype(ntype))=ic
               exit
           end if
         end do

         !new kind....
         if (typ>ntype) then
            ntype=ntype+1
            struct_fam(ntype,:)=presentc(ic,:)
            nbcarbytype(ntype)=1
            lestypes(ntype,nbcarbytype(ntype))=ic
         end if
      end do

      write (nficout,fmt="(2x,'TYPE',2x,'Transcript',2x,'Unknown',2x)")

      do typ=1,ntype
         listUnknown=''
         do kd=1,nd
             if (.not. struct_fam(typ,kd)) then
               if ( trim(listUnknown) == '') then
                 listUnknown=adjustl(animal(kd))
               else
                 listUnknown=trim(listUnknown)//','//adjustl(animal(kd))
               end if
             end if
         end do
         write (nficout,*) typ,lestypes(typ,:nbcarbytype(ntype)),trim(listUnknown)
      end do


      deallocate (nbcarbytype)
      deallocate (struct_fam)
      deallocate (lestypes)

   end subroutine print_transcriptome_Struct_famille
!!***

!!****f* m_qtlmap_output_handler/print_maximum_LRT
!! NAME
!!   print_maximum_LRT
!! DESCRIPTION
!!
!! SOURCE
    subroutine print_maximum_LRT(dxmax,xlrmax)
        real (kind=dp)  ,intent(in)        :: dxmax,xlrmax

        write(nficout,3003)
 3003   format(//1x,'Maximum likelihood ratio test :'/)
        write(nficout,3000)dxmax,xlrmax
 3000   format(1x,'The maximum is reached at position ',f8.4,' M, with value ',f8.3/)

    end subroutine print_maximum_LRT
!!***

!!****f* m_qtlmap_output_handler/print_residual_correlation
!! NAME
!!   print_residual_correlation
!! DESCRIPTION
!!
!! SOURCE
    subroutine print_residual_correlation(rhoi)
      real (kind=dp)  ,intent(in)  ,dimension(:,:) :: rhoi
      integer   :: ic,jc

      if (size(rhoi,1) /= size(rhoi,2)) then
        call stop_application("Devel error: print_residual_correlation")
      end if

      write(nficout,*)
      write(nficout,*) 'Residual Correlations '
      write(nficout,3015) (ic,ic=1,size(rhoi,1)-1)

      do ic=2,size(rhoi,1)
       write(nficout,3014) ic,(rhoi(ic,jc),jc= 1,ic-1)
      end do
      3014 format(7x,i3,1x,30(f6.3,4x))
      3015 format(10x,30(1x,i3,9x))

    end subroutine print_residual_correlation
!!***

!!****f* m_qtlmap_output_handler/print_coeff_linear_combination_max
!! NAME
!!   print_coeff_linear_combination_max
!! DESCRIPTION
!!
!! SOURCE
    subroutine print_coeff_linear_combination_max(coeff_max)
       real (kind=dp) ,intent(in) , dimension(size(carac))     :: coeff_max
       integer   :: ic

       write(nficout,*)
       write(nficout,*) 'Coefficients of the linear  combination'
       write(nficout,3016) 'Trait ', (trim(carac(ic)),ic=1,size(carac))
       write(nficout,3017) (coeff_max(ic),ic=1,size(carac))

       3016 format(1x,a6,1x,30(a10,4x))
       3017 format(7x,30(4x,f8.4,2x))

    end subroutine print_coeff_linear_combination_max
!!***

!!****f* m_qtlmap_output_handler/print_coeff_linear_combination
!! NAME
!!   print_coeff_linear_combination
!! DESCRIPTION
!!
!! SOURCE
    subroutine print_coeff_linear_combination(chr,ilong,npo,coeff)
       integer, intent(in)         :: chr,npo,ilong
       real (kind=dp) ,intent(in) , dimension(size(carac),npo)     :: coeff
       integer   :: ic,ix,n,ios

       if (trim(coeffda) == '' ) then
         return
       end if

       open(UNIT=unit_coeff, file=trim(coeffda)//trim(str(chr)), form="formatted",recl=BUF_ALLOC_FILE,iostat=ios)
       if (ios/=0) then
            call stop_application("Can not open the file :"//trim(coeffda))
       end if

       write(unit_coeff,*) '# Coefficients of the linear  combination'
       write(unit_coeff,3018) '# Position ', (trim(carac(ic)),ic=1,size(carac))
       n=0
       do ix=0,ilong,pas
         n=n+1
     write(unit_coeff,3019) ix,(coeff(ic,n),ic=1,size(carac))
       end do

       3018 format(1x,a9,1x,30(a10,4x))
       3019 format(2x,i4,3x,30(4x,f8.4,2x))
       close(unit_coeff)

    end subroutine print_coeff_linear_combination
!!***

!!****f* m_qtlmap_output_handler/print_confusion
!! NAME
!!   print_confusion
!! DESCRIPTION
!!
!! SOURCE
    subroutine print_confusion(nalert,alertCorrQtl,corrmax)
       integer                          ,intent(in)            :: nalert
       type(CORR_ALERT_TYPE) ,dimension(nalert)  ,intent(in)   :: alertCorrQtl
       real (kind=dp)                   ,intent(in)            :: corrmax
       character(len=LEN_L)  :: fmts,fmtd
       character(LEN_W)      :: last
       integer :: i

      write (nficout,fmt="(//,80('*')/'Test of confusion between QTL ','and other effects in the final constained model',/,"//&
      "'(test based on the correlation between columns of the ','incidence matrix)',//)")

      if (nalert == 0) then
          write(nficout,fmt="(/,' No confusion detected',/,' the highest correlation is : ', F7.3,/,80('*')/)") corrmax
          return
       end if

      fmts="('Risk for sire ',a,' of confusion between the QTL ',i3,' level',i3,' and :')"
      fmtd="('Risk for dam ',a,' of confusion between the QTL ',i3,' level',i3,' and :')"
      last=""
      do i=1,nalert
         if (alertCorrQtl(i)%ip > 0 ) then
           if (trim(last)/=trim(pere(alertCorrQtl(i)%ip))) then
               write(nficout,*)
               write(nficout,fmt=fmts) trim(pere(alertCorrQtl(i)%ip)),alertCorrQtl(i)%qtl,alertCorrQtl(i)%ntlev
               last=trim(pere(alertCorrQtl(i)%ip))
           end if
         end if
         if (alertCorrQtl(i)%jm > 0 ) then
           if (trim(last)/=trim(mere(alertCorrQtl(i)%jm))) then
               write(nficout,fmt=fmtd) trim(mere(alertCorrQtl(i)%jm)),alertCorrQtl(i)%qtl,alertCorrQtl(i)%ntlev
               last=trim(mere(alertCorrQtl(i)%jm))
           end if
         end if
         write(nficout,*) alertCorrQtl(i)%name_effect,alertCorrQtl(i)%name_level,alertCorrQtl(i)%corr
      end do
    end subroutine print_confusion
!!***

!!****f* m_qtlmap_output_handler/print_test_nuisances
!! NAME
!!   print_test_nuisances
!! DESCRIPTION
!!
!! SOURCE
    subroutine print_test_nuisances(ntest,listtestnuis)
       integer                                    ,intent(in)  :: ntest
       type(TEST_NUISANCES_TYPE) ,dimension(ntest),intent(in)  :: listtestnuis
       character(len=LEN_LINE)  :: fmt1,fmt2,myfmt
       integer :: i
       if ( ntest == 0 ) return

       myfmt="(//,80('*')/'test of the effets of the model',//)"
       write(nficout,fmt=myfmt)
       myfmt="('Tested effect     df.    Likelihood     p-value'/'                         ratio                 '/)"
       !header
       write(nficout,fmt=myfmt)

       fmt1="(a15,'(direct effect)',1x,i5,3x,f8.3,3x,f5.3)"
       fmt2="(a15,'  (intra qtl)  ',1x,i5,3x,f8.3,3x,f5.3)"

       do i=1,ntest
        if ( listtestnuis(i)%directeffect ) then
           write(nficout,fmt=fmt1) listtestnuis(i)%name,listtestnuis(i)%df,listtestnuis(i)%lrt,listtestnuis(i)%pvalue
        else
           write(nficout,fmt=fmt2) listtestnuis(i)%name,listtestnuis(i)%df,listtestnuis(i)%lrt,listtestnuis(i)%pvalue
        end if
       end do

    end subroutine print_test_nuisances
!!***

!!****f* m_qtlmap_output_handler/print_summary_analyse
!! NAME
!!   print_summary_analyse
!! DESCRIPTION
!!
!! NOTE
!!  A MODIFIER print_summary_analyse pour prendre en compte les effets d interaction QTL
!! SOURCE
     subroutine print_summary_analyse(listlrtsol,listincsol,nqtl)
        integer              , intent(in)                                   :: nqtl
        type(TYPE_LRT_SOLUTION)  , intent(in) ,dimension(size(carac))       :: listlrtsol
        type(TYPE_INCIDENCE_SOLUTION) , intent(in) ,dimension(size(carac))  :: listincsol

        integer   :: ip,jm,ic,ifail,iq,i,ieff,ntlev,jef,nbtp,c,lp,effp(np),efft
        real (kind=dp)    :: deffp,tst,prob
        character(len=4) :: ctest(size(pere)*20,nqtl),tail,nqtlc
        character(len=LEN_LINE) :: fmt1,summary,sum2,sum3

        summary=""
        sum2=""
        sum3=""
        nqtlc=trim(str(nqtl))
        do iq=1,nqtl
          summary=trim(summary)//trim(str(iq-1))//" QTL versus "//trim(nqtlc)//" QTL"
          if (iq < nqtl) summary=trim(summary)//","
          sum2=trim(sum2)//" "//trim(str(iq-1))//"/"//trim(str(nqtl))//"QTL"
          sum3=trim(sum3)//"   Chr "//trim(str(iq))//"    Pos"//trim(str(iq))
        end do

        write(unit_summary,*)
        write(unit_summary,*) '*************************************************************************************'
        write(unit_summary,*) 'Summary '//trim(summary)

        write(unit_summary,3017)(trim(pere(ip)),ip=1,size(pere))
        write(unit_summary,3018) trim(sum2)//" "//trim(sum3),             &
      ( (' eff'//trim(str(iq)),iq=1,nqtl),' SD ', (' sig'//trim(str(iq)),iq=1,nqtl),ip=1,size(pere))
 3017   format('Variable  N        Max Lik        Pos (M)    Sire',        &
           '          ',             50(1x,a12,25x))
 3018   format(14x,26a,9x,50(28a,2x))

       fmt1="(a8,1x,i3,"//trim(nqtlc)//"(2x,f5.1),"//trim(nqtlc)//"(2x,a5),"//trim(nqtlc)//"(2x,f5.3),7x,"//&
       trim(str(np))//"( "//trim(nqtlc)//"(1x,f6.3),2x,f5.3,1x,"//trim(nqtlc)//"(1x,a4)))"

       do ic=1,size(carac)
         if (natureY(ic) /= 'r') cycle

         !OFI 02/09/2010 - le calcul des effectifs est dependant du caracteres, on ajoute dans la boucle interne de ic, le calcul
         !et on le retire des parametres de la procedure
         efft = 0
         do ip=1,np
          effp(ip) = 0
          do jm=nmp(ip)+1,nmp(ip+1)

           if ( ndm(jm)+1 > ndm(jm+1) ) cycle
           if ( ndm(jm)+1 <= 0 ) cycle

           effp(ip) = effp(ip) + count(presentc(1,ndm(jm)+1:ndm(jm+1)))
          end do
          efft = efft + effp(ip)
         end do

         ntlev=1
         nbtp = 3 + modele(ic,1)+modele(ic,2)!+modele(ic,3)
         do jef=1,modele(ic,3)
	       ntlev=ntlev*nlev(modele(ic,nbtp+jef))
	     end do
         if ( ntlev > 1 ) then
            write(unit_summary,*) " .....Summary do not describe interaction*qtl effect..... : trait "//trim(carac(ic))
            cycle
         end if

         ctest=' ns '
         c=0
         do ip=1,size(pere)
          do lp=1,ntlev
           c=c+1
           ifail=0
           deffp=dble(effp(ip)/2 -1)
           do iq=1,nqtl
            if ( .not. associated(listincsol(ic)%qtl_groupeName) ) then
              call stop_application("Devel error : ** qtl_groupeName in TYPE_INCIDENCE_SOLUTION is not initialized **")
            end if
            ieff=listincsol(ic)%qtl_groupeName(1,iq)
            tst=sqrt(deffp)*dabs(listincsol(ic)%paramaterValue(ieff,c))*2.d0/listincsol(ic)%sig(1,ip)
            tail='U'
            if (deffp.ge.1.d0) then
              prob=MATH_QTLMAP_G01EBF(tail,tst,deffp, ifail)
              if(0.05.gt.prob) ctest(c,iq)='sign'
            else
              ctest(ip,iq)='na'
            end if
           end do
          end do
         end do
        write(unit_summary,fmt=fmt1)trim(carac(ic)),efft,(listlrtsol(ic)%lrtmax(i),i=0,nqtl-1), &
                                                         (chromo(listlrtsol(ic)%chrmax(i)),i=0,nqtl-1), &
                                                     (absi(listlrtsol(ic)%chrmax(i),listlrtsol(ic)%nxmax(i)),i=0,nqtl-1),&
             (((listincsol(ic)%paramaterValue(listincsol(ic)%qtl_groupeName(1,iq),lp),lp=(ntlev*(ip-1))+1,ip*ntlev),iq=1,nqtl),&
                                                   listincsol(ic)%sig(1,ip),           &
                                             ((ctest(lp,iq),iq=1,nqtl),lp=(ntlev*(ip-1))+1,ip*ntlev),ip=1,size(pere))

      end do
     !  close(unit_summary)

    end subroutine print_summary_analyse
!!***

!!****f* m_qtlmap_output_handler/create_grid_file_2QTL
!! NAME
!!   create_grid_file_2QTL
!! DESCRIPTION
!!
!! NOTE
!!  A MODIFIER print_summary_analyse pour prendre en compte les effets d interaction QTL
!! SOURCE
    subroutine create_grid_file_2QTL(lrtsol)
         type(TYPE_LRT_SOLUTION)            , intent(in)    :: lrtsol

         integer    :: n,n1,nout2,chr,chr2,ios
         character(len=100) :: FMT1,FMT2

         if ( trim(grid2qtl) == '' ) then
           return
         end if

         nout2=17
         open(UNIT=nout2,file=grid2qtl, form="formatted",recl=BUF_ALLOC_FILE,iostat=ios)
         if (ios/=0) then
            call stop_application("Can not open the file :"//trim(grid2qtl))
         end if


         FMT1="(6x,"//trim(str(get_maxnpo()))//"(2x,f4.2,2x))"
         FMT2="(1x,f4.2,1x,"//trim(str(get_maxnpo()))//"(f7.2,1x))"

!
! Cr�ation d'un fichier grid_qtl pour ue representation graphique
!

         write(nout2,*)
         write(nout2,*) '++++++++++++++++ TEST 1QTL / 2QTL ++++++++++++++++++++'
         write(nout2,*)
         write(nout2,FMT=FMT1) ((absi(chr,n),n=1,get_npo(chr)),chr=1,nchr)
         do chr=1,nchr
          do n=1,get_npo(chr)
            write(nout2,FMT=FMT2) absi(chr,n),((lrtsol%lrt1_2(chr,chr2,n,n1),n1=1,get_npo(chr2)),chr2=1,nchr)
          end do
         end do

         write(nout2,*)
         write(nout2,*) ' +++++++++++++++++ TEST 0QTL / 2QTL ++++++++++++++++++++'
         write(nout2,*)

         write(nout2,FMT=FMT1)((absi(chr,n),n=1,get_npo(chr)),chr=1,nchr)
         do chr=1,nchr
          do n=1,get_npo(chr)
            write(nout2,FMT=FMT2) absi(chr,n),((lrtsol%lrt0_2(chr,chr2,n,n1),n1=1,get_npo(chr2)),chr2=1,nchr)
          end do
         end do

         close(nout2)
    end subroutine
!!***

!!****f* m_qtlmap_output_handler/print_end_multitraits
!! NAME
!!   print_end_multitraits
!! DESCRIPTION
!!
!! NOTE
!!
!! SOURCE
    subroutine print_end_multitraits

    end subroutine print_end_multitraits
!!***

!!****f* m_qtlmap_output_handler/print_end_unitrait
!! NAME
!!   print_end_unitrait
!! DESCRIPTION
!!
!! NOTE
!!
!! SOURCE
    subroutine print_end_unitrait

    end subroutine print_end_unitrait
!!***

!!****f* m_qtlmap_output_handler/print_paternal_maternal_effect
!! NAME
!!   print_paternal_maternal_effect
!! DESCRIPTION
!!
!! NOTE
!!
!! SOURCE
    subroutine print_paternal_maternal_effect(ic,lrtsol)
       integer                    ,intent(in)    :: ic
       type(TYPE_LRT_SOLUTION)    ,intent(in)    :: lrtsol

       integer    :: i,ii,npo,chr
       character(len=100) :: FMT1,FMT2

       if (  ( trim(mateff)=='' )  .and. ( trim(pateff)=='' )) then
          return
       end if

       FMT1="(1x,i3,f8.3,"// trim(str(np)) //"(1x,f7.2))"
       if ( namest(ic) /= 0 ) then
         FMT2="(1x,i3,f8.3,"// trim(str(namest(ic))) //"(1x,f7.2))"
       else
         FMT2="(1x,i3,f8.3,1x,f7.2)"
       end if

       if ( trim(pateff)/='' ) write(unit_paternal_effects,*)"  Chr  Pos  ",("   "// trim(pere(ii)),ii=1,np )
       if ( trim(mateff)/='' ) write(unit_maternal_effects,*)"  Chr  Pos  ",("   "// trim(mere(ii)),ii=1,namest(ic) )

       if (file) then
         do chr=1,nchr
           npo = get_npo(chr)
           do i=1,npo
              if ( trim(pateff)/='' ) write(unit_paternal_effects,FMT=FMT1)&
              chr, absi(chr,i),(lrtsol%pater_eff(chr,ii,i),ii=1,np)
              if ( trim(mateff)/='' ) write(unit_maternal_effects,FMT=FMT2)&
              chr,absi(chr,i),(lrtsol%mater_eff(chr,ii,i),ii=1,namest(ic))
           end do
         end do
       end if
    end subroutine print_paternal_maternal_effect
!!***

!!****f* m_qtlmap_output_handler/print_LRT
!! NAME
!!   print_LRT
!! DESCRIPTION
!!
!! NOTE
!!
!! SOURCE
    subroutine print_LRT(lrtsol)
       type(TYPE_LRT_SOLUTION)                           :: lrtsol

       integer    :: i,ii,npo,chr
       character(len=100) :: FMT1,FMT2

       if ( (trim(resp)=='' ) .and. (trim(resm)=='' )) then
         return
       end if

       FMT1="(1x,i3,f8.3,1x,f7.2,"// trim(str(np)) //"(1x,f7.2))"
       FMT2="(1x,i3,f8.3,"// trim(str(nm)) //"(1x,f7.2))"

       if ( trim(resp)/='' ) write(unit_sire_res,*)"   Chr Pos    GlobalLRT  ",("   "// trim(pere(ii)),ii=1,np )
       if ( trim(resm)/='' ) write(unit_dam_res,*)"   Chr Pos   ",("   "// trim(mere(ii)),ii=1,nm )

       if (file) then
          do chr=1,nchr
           npo = get_npo(chr)
           do i=1,npo
             if ( trim(resp)/='' ) write(unit_sire_res,FMT=FMT1)&
             chr,absi(chr,i),lrtsol%lrt1(chr,i),(lrtsol%xlrp(chr,ii,i),ii=1,np)
             if ( trim(resm)/='' ) write(unit_dam_res,FMT=FMT2)&
             chr,absi(chr,i),(lrtsol%xlrm(chr,ii,i),ii=1,nm)
           end do
          end do
       end if

    end subroutine print_LRT
!!***

!!****f* m_qtlmap_output_handler/print_pat_mat_effect_2QTL
!! NAME
!!   print_pat_mat_effect_2QTL
!! DESCRIPTION
!!
!! NOTE
!!
!! SOURCE
    subroutine print_pat_mat_effect_2QTL(ic,lrtsol)
       integer , intent(in)                              :: ic
       type(TYPE_LRT_SOLUTION)         ,intent(in)       :: lrtsol

       integer    :: i,j,ip,jm,iq,init,chr,chr2
       character(len=100) :: FMT1,FMT2

       if (  ( trim(mateff)=='' )  .and. ( trim(pateff)=='' )) then
          return
       end if

       FMT1="(1x,i5,1x,i5,1x,f8.3,1x,f8.3,"// trim(str(np*2)) //"(1x,f7.2))"
       if ( namest(ic) /= 0 ) then
         FMT2="(1x,i5,1x,i5,1x,f8.3,1x,f8.3,"// trim(str(namest(ic)*2)) //"(1x,f7.2))"
       else
         FMT2="(1x,i5,1x,i5,1x,f8.3,1x,f8.3,1x,f7.2)"
       end if

       if ( trim(pateff)/='' ) write(unit_paternal_effects,*)&
       "   Chr1    Chr2   Pos1  ","   Pos2  ",(( "   "// trim(pere(ip))//"/Qtl["//&
        trim(str(iq))//"] ",iq=1,2),ip=1,np)

     !il faut afficher que les mere estimable.....
     !  if ( trim(mateff)/='' ) write(unit_maternal_effects,*)&
     !  "   Chr1    Chr2   Pos1  ","   Pos2  ",(( "   "// trim(mere(jm))//"/Qtl["//&
      !  trim(str(iq))//"] ",iq=1,2),jm=1,namest(ic))

       if (file) then
         do chr=1,nchr
           do i=1,get_npo(chr)-1
            do chr2=chr,nchr
             init=1
             if ( chr2 == chr ) init=i+1
             do j=init,get_npo(chr2)
              if ( trim(pateff)/='' ) &
                write(unit_paternal_effects,FMT=FMT1)&
                chr,chr2,absi(chr,i),absi(chr2,j),((lrtsol%pater_eff2(chr,chr2,ip,i,j,iq),iq=1,2),ip=1,np)
              if ( trim(mateff)/='' ) &
                write(unit_maternal_effects,FMT=FMT2)&
                chr,chr2,absi(chr,i),absi(chr2,j),((lrtsol%mater_eff2(chr,chr2,jm,i,j,iq),iq=1,2),jm=1,namest(ic))
             end do
           end do
         end do
        end do
       end if
    end subroutine print_pat_mat_effect_2QTL
!!***

!!****f* m_qtlmap_output_handler/print_LRT_2QTL
!! NAME
!!   print_LRT_2QTL
!! DESCRIPTION
!!
!! NOTE
!!
!! SOURCE
    subroutine print_LRT_2QTL(lrtsol)
       type(TYPE_LRT_SOLUTION)         ,intent(in)       :: lrtsol

       integer    :: i,j,ii,chr,chr2,init
       character(len=100) :: FMT1,FMT2

       if ( (trim(resp)=='' ) .and. (trim(resm)=='' )) then
         return
       end if

       FMT1="(1x,i5,1x,i5,1x,f7.2,1x,f7.2,1x,f7.2,"// trim(str(np)) //"(1x,f7.2))"
       FMT2="(1x,i5,1x,i5,1x,f7.2,1x,f7.2,"// trim(str(nm)) //"(1x,f7.2))"

       if ( trim(resp)/='' ) write(unit_sire_res,*)&
       " Chr1  Chr2   Pos1      Pos2      GlobalLRT  ",("   "// trim(pere(ii)),ii=1,np )
       if ( trim(resm)/='' ) write(unit_dam_res,*)&
       "  Chr1   Chr2   Pos1       Pos2  ",("   "// trim(mere(ii)),ii=1,nm )

       if (file) then
        do chr=1,nchr
           do i=1,get_npo(chr)-1
             do chr2=chr,nchr
              init=1
              if ( chr2 == chr ) init=i+1
              do j=init,get_npo(chr2)
                if ( trim(resp)/='' ) write(unit_sire_res,FMT=FMT1)&
                 chr,chr2,absi(chr,i),absi(chr2,j),lrtsol%lrt1_2(chr,chr2,i,j),(lrtsol%xlrp2(chr,chr2,ii,i,j),ii=1,np)
                if ( trim(resm)/='' ) write(unit_dam_res,FMT=FMT2)&
                 chr,chr2,absi(chr,i),absi(chr2,j),(lrtsol%xlrm2(chr,chr2,ii,i,j),ii=1,nm)
             end do
           end do
          end do
         end do
       end if

    end subroutine print_LRT_2QTL
!!***

!!****f* m_qtlmap_output_handler/print_sire_phase
!! NAME
!!   print_sire_phase
!! DESCRIPTION
!!
!! NOTE
!!
!! SOURCE
    subroutine print_sire_phase()
        integer :: ip,i,maxp,chr
        character(len=3) :: endline
        character(len=LEN_DEF)  :: sep,sep2

       do chr=1,nchr
        sep=' '
        sep2=' / '
        if ( MAX_GENOTYP_PRINT < nmk(chr) ) then
           maxp = MAX_GENOTYP_PRINT
           endline="..."
        else
           maxp = nmk(chr)
           endline=""
        end if


        write(nficout,*)
        write(nficout,*) '    ***************** PARENTAL PHASES *****************'
        write(nficout,*) '                      FILE :',trim(out_phases)
        write(nficout,*) '    ***************************************************'

        if ( trim(out_phases)/='' ) then
             write(unit_phases,*)
             write(unit_phases,*) '    ***************** SIRE PARENTAL PHASES *****************'
             write(unit_phases,*) '                      CHROMOSOME :',chromo(chr)
             write(unit_phases,*)
        end if

        do ip=1,size(pere)
          if(phasp(chr,ip)) then
            if ( trim(out_phases)/='' ) write(unit_phases,1000)trim(pere(ip))
          else
            if ( trim(out_phases)/='' ) write(unit_phases,1001)trim(pere(ip))
          end if
          if ( trim(out_phases)/='' ) then
          write(unit_phases,*)(trim(get_pheno(chr,genotyp(chr,i,correp(ip),1)))//' ',i=1,nmk(chr)),trim(endline),' / ',&
                          (trim(get_pheno(chr,genotyp(chr,i,correp(ip),2)))//' ',i=1,nmk(chr)),trim(endline)
          end if


 1000     format(/1x,'Sire ',a,' genotype (paternal / maternal phases): ')
 1001     format(/1x,'Sire ',a,' genotype (unknown phase origins): ')

        end do
      end do
    end subroutine  print_sire_phase
!!***

!!****f* m_qtlmap_output_handler/print_dam_phase
!! NAME
!!   print_dam_phase
!! DESCRIPTION
!!
!! NOTE
!!
!! SOURCE
    subroutine print_dam_phase()
      real      :: pb
      integer   :: chr,jm,nd1,nd2,kd,ifem,nombfem,j,ii,ngeno1,ngeno2,maxp
      character(len=3) :: endline

      nombfem=0
      do chr=1,nchr

       if ( MAX_GENOTYP_PRINT < nmk(chr) ) then
           maxp = MAX_GENOTYP_PRINT
           endline="..."
       else
           maxp = nmk(chr)
           endline=""
       end if

        endline=""

        if ( trim(out_phases)/='' ) then
         write(unit_phases,*)
         write(unit_phases,*) '    ***************** DAM PARENTAL PHASES *****************'
         write(unit_phases,*) '                      CHROMOSOME :',chromo(chr)
         write(unit_phases,*)
        end if

      do jm=1,nm
        nd1=ndm(jm)+1
        nd2=ndm(jm+1)
        ngeno1=ngenom(chr,jm)+1
        ngeno2=ngenom(chr,jm+1)
        ifem=repfem(jm)

        if ( estfem(ifem) ) then
          nombfem=nombfem+1

        if (phasm(chr,jm)) then
           if ( trim(out_phases)/='' ) write(unit_phases,1000)trim(femelle(ifem)),(ngeno2-ngeno1+1)
        else
           if ( trim(out_phases)/='' ) write(unit_phases,1010)trim(femelle(ifem)),(ngeno2-ngeno1+1)
        end if

 1000   format(/1x,'Dam ',a,' has ',i3,' likely genotypes',' (paternal / maternal phases):')
 1010   format(/1x,'Dam ',a,' has ',i3,' likely genotypes',' (unknown phase origins) :')

        do j=ngeno1,ngeno2
           pb = probg(chr,j)
          if ( trim(out_phases)/='' ) then
            write(unit_phases,*)(trim(get_pheno(chr,genotypm(chr,ii,j,1)))//' ',ii=1,nmk(chr)),&
                trim(endline),' / ',       &
            (trim(get_pheno(chr,genotypm(chr,ii,j,2)))//' ',ii=1,nmk(chr)),trim(endline),&
            ' proba : ',pb
          end if
        end do
        end if

      end do
     end do
     if ( trim(out_phases)/='' ) then
       if(nombfem.eq.0) write(unit_phases,1020)
     end if

 1020 format(/,'None of the females had more than the minimum number of progeny needed to estimate its possible phases')


    end subroutine print_dam_phase
!!***

!!****f* m_qtlmap_output_handler/print_pded
!! NAME
!!   print_pded
!! DESCRIPTION
!!
!! NOTE
!!
!! SOURCE
    subroutine print_pded(chr)
      integer, intent(in)                               :: chr

     real (kind=dp)    :: app,apm,pp,pm,dx
     integer           :: ip,nm1,nm2,ngeno1,ngeno2,jm,geno,igeno,kd,kkd,n,nd1,nd2,ios

     if ( (trim(pdedf) == '' ) .and. ( trim(pdecouple) =='') ) then
        return
     end if

     if ( trim(pdedf) /= '' ) then
        open(UNIT=unit_pded, file=trim(pdedf)//'_'//trim(str(chr)), form="formatted",recl=BUF_ALLOC_FILE,iostat=ios)
        if (ios/=0) then
            call stop_application("Can not open the file :"//trim(pdedf))
        end if
     end if

     if ( trim(pdecouple) /= '' ) then
        open(UNIT=unit_pdedjoin, file=trim(pdecouple)//'_'//trim(str(chr)), form="formatted",recl=BUF_ALLOC_FILE,iostat=ios)
        if (ios/=0) then
            call stop_application("Can not open the file :"//trim(pdecouple))
        end if
     end if

     if ( trim(pdedf) /= '' ) &
       write(unit_pded,*) 'Position   Sire    Dam Dam_Phase Animal   p(2nd sire allele)   p(2nd dam allele) '

     if ( trim(pdecouple) /= '' ) &
       write(unit_pdedjoin,*) 'Position   Sire    Dam  Dam_Phase Animal   p(Hs1/Hd1 )  p(Hs1/Hd2 )  p(Hs2/Hd1 )  p(Hs2/Hd2 ) '

     do ip=1,np
        nm1=nmp(ip)+1
        nm2=nmp(ip+1)
        do jm=nm1,nm2
          ngeno1=ngenom(chr,jm)+1
          ngeno2=ngenom(chr,jm+1)
          do geno=ngeno1,ngeno2
           igeno=ngeno2-geno+1
           nd1=ngend(chr,geno)+1
           nd2=ngend(chr,geno+1)
           do kd=nd1,nd2
             kkd=ndesc(chr,kd)
             do n=1,(get_npo(chr)-1)
              dx=absi(chr,n)

              if(.not. estfem(repfem(jm)).or.opt_sib.eq.OPT_SIB_HS) then
                pp=-pdd(chr,kd,1,n)+pdd(chr,kd,3,n)
                app=(pp+1.d0)/2.d0
                if ( trim(pdedf) /= '' ) then
                  write(unit_pded,333) dx,trim(pere(ip)),trim(mere(jm)),igeno,trim(animal(kkd)),app,0.5d0
                end if
!
! Impression des probabilites des couples d'haplo parentaux
                 if ( trim(pdecouple) /= '' ) then
                  write(unit_pdedjoin,334) dx,trim(pere(ip)),trim(mere(jm)),igeno,trim(animal(kkd)),&
                     0.5d0*pdd(chr,kd,1,n),0.5d0*pdd(chr,kd,1,n),&
                     0.5d0*pdd(chr,kd,3,n),0.5d0*pdd(chr,kd,3,n)
  !332           format(2x,f4.0,1x,a12,1x,a12,1x,2(5x,f8.3))
                 end if

              else    !OPT_SIB_FS

                pp=-pdd(chr,kd,1,n)-pdd(chr,kd,2,n)+pdd(chr,kd,3,n)+pdd(chr,kd,4,n)
                pm=-pdd(chr,kd,1,n)+pdd(chr,kd,2,n)-pdd(chr,kd,3,n)+pdd(chr,kd,4,n)
                app=(pp+1.d0)/2.d0
                apm=(pm+1.d0)/2.d0


!Impression des probabilites des haplo parentaux
                if ( trim(pdedf) /= '' ) then
                    write(unit_pded,333) dx,trim(pere(ip)),trim(mere(jm)),igeno,trim(animal(kkd)),app,apm
                end if

  333           format(2x,f4.0,1x,2(a12,1x),i5,1x,a12,1x,2(12x,f8.3))
!Impression des probabilites des couples d'haplo parentaux
                if ( trim(pdecouple) /= '' ) then
                  write(unit_pdedjoin,334) dx,trim(pere(ip)),trim(mere(jm)),igeno,&
                       trim(animal(kkd)),pdd(chr,kd,1,n),pdd(chr,kd,2,n),pdd(chr,kd,3,n),pdd(chr,kd,4,n)
                end if

  334           format(2x,f4.0,1x,2(a12,1x),i5,1x,a12,1x,4(5x,f8.3))

              end if

             end do ! N
           end do ! KD
        end do ! GENO
      end do  ! JM
     end do ! IP

     if ( trim(pdedf) /= '' )  close(unit_pded)
     if ( trim(pdecouple) /= '' ) close(unit_pdedjoin)

    end subroutine print_pded
!!***

!!****f* m_qtlmap_output_handler/print_resume_simulation
!! NAME
!!   print_resume_simulation
!! DESCRIPTION
!!
!! NOTE
!!
!! SOURCE
    subroutine print_resume_simulation(trait,test,ns,YMU1,SIG1,S21,S31,XMIN1,XMAX1,prob,z,minP,maxP)
         character(len=*),intent(in)              :: trait,test
         integer         ,intent(in)              :: ns
         real(kind=dp)   ,intent(in)              :: YMU1,SIG1,S21,S31,XMIN1,XMAX1
         real(kind=dp) ,dimension(16),intent(in)  :: prob,z
         integer         ,intent(in)              :: minP,maxP
         integer        :: ii
      write(nficout,*)
      write(nficout,4004) trim(trait)
 4004 format(                                             &
     1X,'*---------------------------------------*'        &
     ,/,                                                  &
     1x,'           Variable ',a)

      write(nficout,4015) test
 4015 format(                                      &
     1X,'*---------------------------------------*' &
     ,/,                                           &
     1x,'          Test ',a   &
     ,/,                                              &
     &1X,'*---------------------------------------*',/)

      WRITE(nficout,6103)ns,YMU1,SIG1,S21,S31,XMIN1,XMAX1
 6103 FORMAT(                                      &
     1X,' Test statistic distribution  :',/,       &
     1X,'     Number of simulations : ',i6,/,      &
     1X,'     Mean                  : ',F12.5,/,   &
     1X,'     Standard deviation    : ',F12.5,/,   &
     1X,'     Skewness              : ',F12.5,/,   &
     1X,'     Kurtosis              : ',F12.5,/,   &
     1X,'     Minimum               : ',F12.5,/,   &
     1X,'     Maximum               : ',F12.5,/)

      write(nficout,4005)
 4005 format(                                              &
     1X,'*--------------------------------------*'         &
     ,/,                                                   &
     1x,'| chromosome | genome     |  Threshold |'         &
     ,/,                                                   &
     1x,'|          level          |            |'         &
     ,/,                                                   &
     1X,'|--------------------------------------|')
        do ii=minP,maxP
         if (ii.eq.11)write(nficout,4007) 1-prob(ii),'chrom_level',z(ii)
         if (ii.eq.12)write(nficout,4007) 1-prob(ii),'     *     ',z(ii)
         if (ii.eq.13)write(nficout,4007) 1-prob(ii),'  nb_chrom ',z(ii)
         if (ii.lt.11.or.ii.gt.13)write(nficout,4007) 1-prob(ii),'           ',z(ii)
 4007    format(1x,'|',2x,F7.4,3x,'|',a12,'|',2x,F8.2,2x,'|')
        end do
        write(nficout,*)'*--------------------------------------*'

    end subroutine print_resume_simulation
!!***

!!****f* m_qtlmap_output_handler/print_resume_simulation_2
!! NAME
!!   print_resume_simulation_2
!! DESCRIPTION
!!
!! NOTE
!!
!! SOURCE
    subroutine print_resume_simulation_2(test,ncar,carac,z)
      character(len=*),intent(in)                   :: test
      integer         ,intent(in)                   :: ncar
      character(len=*),dimension(ncar),intent(in)   :: carac
      real(kind=dp) ,dimension(ncar,16),intent(in)  :: z

      integer    :: i


      write(nficout,4009) test
 4009   format(/,                                                  &
     '    ',a8,'               p_value at                 '        &
     ,/,                                                           &
     'Trait                 chromosome level                ',     &
       '   genome level   ',/,                                     &
     '              5%        1%        0.1%          5%        1% ', &
     '       0.1%',/)

      do i=1,ncar
        write(nficout,4010)carac(i),z(i,10),z(i,11),z(i,14),z(i,13),z(i,15),z(i,16)
      end do
 4010 format(a8,4x,F6.2,4x,F6.2,4x,F6.2,8x,F6.2,4x,F6.2,4x,F6.2)

    end subroutine print_resume_simulation_2
!!***

!!****f* m_qtlmap_output_handler/print_allelic_origin
!! NAME
!!   print_allelic_origin
!! DESCRIPTION
!!
!! NOTE
!!
!! SOURCE
    subroutine print_allelic_origin

         integer :: i,g,ip,chr

         write (nficout,*)
         do ip=1,np
           write (nficout,fmt="(' Allelic origin for ',a16)") pere(ip)
          do chr=1,nchr
            if ( phasp(chr,ip)) then
              write (nficout,fmt="('Chromosome ',a4,' : known')") chromo(chr)
            else
              write (nficout,fmt="('Chromosome ',a4,' : unknown')") chromo(chr)
            end if
          end do
         end do
         write (nficout,*)
         write (nficout,*) 'NOTE: known allelic origin means QTL effect =  maternal - paternal allele effects'


    end subroutine print_allelic_origin
!!***

!!****f* m_qtlmap_analyse_unitrait/print_incidence_solution
!! NAME
!!    print_incidence_solution
!! DESCRIPTION
!!
!! HISTORY
!!  09/09/2010 . add information :
!!               - mean of absolute value of substitution effect (within sires family)
!!               - mean of part of s.d (within sire family)
!! HISTORY
!!  14/09/2010 : correction bug : pas de calcul wq si pas d effet qtl (LD analyse)
!! SOURCE
    subroutine print_incidence_solution(incsol)
         type(TYPE_INCIDENCE_SOLUTION)     , intent(in)   :: incsol
         integer :: i,g,ip,chr,ic,l,iq
         real(kind=dp) :: wq

         write(nficout,*) "---------------------------------------------------------------"
         write(nficout,*) 'Estimation of parameters under H'//trim(str(incsol%hypothesis))
         write(nficout,*) "---------------------------------------------------------------"
         write(nficout,*)
         write(nficout,*) 'Within sire standard deviation'
         do ic=1,size(incsol%sig,1)
           if ( size(incsol%sig,1) > 1 ) then
             write(nficout,*) " ** Trait ",trim(carac(ic))," **"
           end if
          do ip = 1,np
           write(nficout,fmt="(' sire ',a, '  s.d. :',f10.3)") trim(pere(ip)),incsol%sig(ic,ip)
          end do
         end do

         write(nficout,*)

         write(nficout,"(//,'  parameter    ','        estimable ?    value     ','precision'/)")

         do i=1,size(incsol%groupeName)
           write (nficout,*) incsol%groupeName(i)
           write (nficout,*)

           if ( incsol%nbParameterGroup(i) == 1 ) then
             if ( incsol%parameterVecsol(i,1) ) then
               write (nficout,fmt="(a50,'  yes ',2f10.3,1x)") " " , incsol%paramaterValue(i,1),&
                        incsol%parameterPrecis(i,1)
             else
               write (nficout,fmt="(a50,'  no ')") incsol%groupeName(i)
             end if
           else

              do g=1,incsol%nbParameterGroup(i)
                if ( incsol%parameterVecsol(i,g) ) then
                  write (nficout,fmt="(a50,'  yes ',2f10.3,1x)") incsol%parameterName(i,g)&
                   , incsol%paramaterValue(i,g) ,  incsol%parameterPrecis(i,g)
                else
                  write (nficout,fmt="(a50,'  no ')") incsol%parameterName(i,g)
                end if
              end do
           end if
           write (nficout,*)
         end do
         write(nficout,*)' NOTE: known allelic origin means QTL effect =  maternal - paternal allele effects'

         if (associated(incsol%rhoi)) call print_residual_correlation(incsol%rhoi)

         write (nficout,*) '                        ***                          '
         write (nficout,*) ' The mean of absolute value of subtitution effect WQ ='
         write (nficout,*) ' -------------------------- '
         do iq=1,incsol%hypothesis
          if ( .not. associated(incsol%qtl_groupeName) ) then
            call stop_application("Devel error*** incsol%qtl_groupeName is not allocated"//&
             " [m_qtlmap_output_handler:print_incidence_solution] ")
          end if

          ! Moyenne des valeurs absolues des effets de substitution
          do ic=1,size(incsol%sig,1)
!          print *,iq,'QTLEFFECT:',incsol%qtl_groupeName(ic,iq)
           g = incsol%qtl_groupeName(ic,iq) ! get the index of qtl position effect
           if (g <= 0 .or. g > size(incsol%parameterVecsol,1)) cycle
           wq = 0
           l = 0
           do ip=1,np
          !  print *,incsol%parameterName(g,ip),incsol%groupeName(g)
            if (incsol%parameterVecsol(g,ip)) then
             wq = wq + abs( incsol%paramaterValue(g,ip) / incsol%sig(ic,ip))
             l = l + 1
            end if
           end do
           wq = wq / l
           if ( size(incsol%sig,1) > 1 ) then
            write (nficout,fmt="(' | qtl ',i5, ' | wq :',f10.3,' |',a)") iq,wq,trim(carac(ic))
           else
             write (nficout,fmt="(' | qtl ',i5, ' | wq :',f10.3,' |')") iq,wq
           end if
          end do
         end do
         write (nficout,*) ' -------------------------- '

    end subroutine print_incidence_solution
!!***

!!****f* m_qtlmap_output_handler/print_incidence_solution_risk_factor
!! NAME
!!   print_incidence_solution_risk_factor
!! DESCRIPTION
!!
!! NOTE
!!
!! SOURCE
    subroutine print_incidence_solution_risk_factor(incsol)
         type(TYPE_INCIDENCE_SOLUTION)     , intent(in)   :: incsol

         integer :: i,g,ip,chr,ic

         write(nficout,*) "---------------------------------------------------------------"
         write(nficout,*) 'Estimation of parameters under H'//trim(str(incsol%hypothesis))
         write(nficout,*) "---------------------------------------------------------------"
         write(nficout,*)
         write(nficout,*)

         write(nficout,"(//,'  parameter    ','        estimable ?     risk factor '/)")

         do i=1,size(incsol%groupeName)
           write (nficout,*) incsol%groupeName(i)
           write (nficout,*)

           if ( incsol%nbParameterGroup(i) == 1 ) then
             if ( incsol%parameterVecsol(i,1) ) then
               write (nficout,fmt="(a25,'  yes ',2f10.3,1x)") " " , incsol%paramaterValue(i,1)
             else
               write (nficout,fmt="(a25,'  no ')") incsol%groupeName(i)
             end if
           else

              do g=1,incsol%nbParameterGroup(i)
                if ( incsol%parameterVecsol(i,g) ) then
                  write (nficout,fmt="(a25,'  yes ',f10.3,1x)") incsol%parameterName(i,g), incsol%paramaterValue(i,g)
                else
                  write (nficout,fmt="(a25,'  no ')") incsol%parameterName(i,g)
                end if
              end do
           end if
           write (nficout,*)
         end do

         write(nficout,*)' NOTE: known allelic origin means QTL effect =  maternal - paternal allele effects'

    end subroutine print_incidence_solution_risk_factor
!!***

!!****f* m_qtlmap_output_handler/print_lrt_solution
!! NAME
!!   print_lrt_solution
!! DESCRIPTION
!!
!! NOTE
!!
!! SOURCE
    subroutine print_lrt_solution(lrtsol)
       type(TYPE_LRT_SOLUTION)       ,intent(in)    :: lrtsol
       integer :: iq
       character(len=1) :: l

       if ( lrtsol%nqtl <= 0 ) return
        write(nficout,*) "                      *****                    "
       if ( lrtsol%nqtl == 1 ) then
         call print_courbe_LRT(lrtsol)
       end if

       write(nficout,fmt="(//1x,'Maximum likelihood ratio test :'/)")
       l='1'
       if (lrtsol%nqtl >= 10 ) l='2'

       do iq=1,lrtsol%nqtl
         write(nficout,fmt="('Test H',i"//l//",' / H',i"//l//",' : ',f9.5,/)") iq-1, lrtsol%nqtl, lrtsol%lrtmax(iq-1)
       end do

        write(nficout,fmt="(1x,'The maximum is reached at position(s) ',"//trim(str(lrtsol%nqtl))//"(f9.4,'(Chr :',a4,') '))") &
             (absi(lrtsol%chrmax(iq),lrtsol%nxmax(iq)),chromo(lrtsol%chrmax(iq)),iq=0,lrtsol%nqtl-1)

    end subroutine print_lrt_solution
!!***

!!****f* m_qtlmap_output_handler/write_simulation_file
!! NAME
!!   write_simulation_file
!! DESCRIPTION
!!
!! NOTE
!!
!! SOURCE
    subroutine write_simulation_file(opt_qtl,is_multi,nsim,lrtsol)
      integer       , intent(in)     :: opt_qtl,nsim
      logical       , intent(in)     :: is_multi

      type(TYPE_LRT_SOLUTION)  , dimension(nsim,ncar,opt_qtl),intent(in)      :: lrtsol
      integer, parameter             :: unit_simula        = 14
      integer :: isim,i,ic,iiq,iq,ios
      character(len=LEN_LINE)                                 :: myfmt

      if ( trim(simula)=='' ) then
        return
      end if

      open(unit_simula,file=simula,form="formatted",recl=BUF_ALLOC_FILE,iostat=ios)
      if (ios/=0) then
            call stop_application("Can not open the file :"//trim(simula))
      end if


      if (.not. is_multi ) then
      myfmt="("
      do iq=1,opt_qtl
       myfmt=trim(myfmt)//"(f10.4"
       do iiq=1,iq
         myfmt=trim(myfmt)//",a5,f10.4"
       end do
       myfmt=trim(myfmt)//")"
      end do
      myfmt=trim(myfmt)//")"

     do ic=1,ncar
        if (opt_qtl==1) then
        write(unit_simula,*) "# Trait ["//trim(carac(ic))//"] LRTMAX H0/H1 , Position CHR, Position DX "
        else
        write(unit_simula,*) "# Trait ["//trim(carac(ic))//"] LRTMAX H0/H1 , Position CHR, Position DX LRTMAX H1/H2 ,"//&
        "  Position1 CHR, Position1 DX Position2 CHR, Position1 DX2"
        end if
        do isim=1,nsim
            write(unit_simula,fmt=trim(myfmt)) &
            (lrtsol(isim,ic,iq)%lrtmax(iq-1),(chromo(lrtsol(isim,ic,iq)%chrmax(iiq)),&
            absi(lrtsol(isim,ic,iq)%chrmax(iiq),lrtsol(isim,ic,iq)%nxmax(iiq)),iiq=0,iq-1)&
            ,iq=1,opt_qtl)
        end do
     end do
    else
        write(unit_simula,*)"# LRTMAX , Position CHR, Position DX "
        do isim=1,nsim
            write(unit_simula,fmt='(f10.4,a5,f10.4)') &
            lrtsol(isim,1,1)%lrtmax(0),chromo(lrtsol(isim,1,1)%chrmax(0)),&
              absi(lrtsol(isim,1,1)%chrmax(0),lrtsol(isim,1,1)%nxmax(0))
        end do

    end if

    close(unit_simula)

    end subroutine write_simulation_file
!!***


end module m_qtlmap_output_handler
