!!****m* OUTPUT/m_qtlmap_output_xml
!!  NAME
!!    m_qtlmap_output_xml -- Export result information on a XML format
!!  SYNOPSIS
!!
!!  DESCRIPTION
!!
!!  NOTES
!!
!!  BUGS
!!
!!  SEE ALSO
!!
!!  COPYRIGHT
!!***
!! You can use this space for remarks that should not be included
!! in the documentation.
!!/
module m_qtlmap_output_xml
    use m_qtlmap_data
    use m_qtlmap_haplotype_data
    use m_qtlmap_tools
    use qtlmap_types

    implicit none

    integer ,private , parameter              :: BUF_ALLOC_FILE=2**14

    public :: print_pded_xml
    public :: print_solution_xml


    contains

!!****f* OUTPUT/m_qtlmap_output_xml/print_pded_xml
!!  NAME
!!    print_pded_xml
!!  SYNOPSIS
!!
!!  FUNCTION
!!
!!  INPUTS
!!
!!  RESULT
!!
!!  EXAMPLE
!!
!!  NOTES
!!
!!  BUGS
!!
!!  SEE ALSO
!!
!!***
  subroutine print_pded_xml
     integer :: ip,jm,igeno,geno,kkd,kd,n,chr,ios,myunit,ii

     myunit=44
     open(UNIT=myunit, file="pded.xml", form="formatted",recl=BUF_ALLOC_FILE,iostat=ios)

     write (myunit,'(a)')'<?xml version="1.0" encoding="UTF-8"?>'

     !-------------------------
     ! By Chromosome
     write (myunit,fmt="(a)") '<Chromosomes>'
     do chr=1,nchr
     write (myunit,fmt="(a,i5,a)") '<Chromosome id="',chr,'">'

      !-------------------------
      ! By Sire
      write (myunit,fmt="(a)") '<Sires>'
      do ip=1,np
       write (myunit,fmt="(a,i5,a)") '<Sire id="',ip,'">'
       write (myunit,*) "<Name>",trim(pere(ip)),"</Name>"
        !Phase
        !-----
        if (phasp(chr,ip)) then
             write (myunit,*) "<Phase>true</Phase>"
          else
             write (myunit,*) "<Phase>false</Phase>"
          end if
         write (myunit,*) "<Phenotype>"
            do ii=1,nmk(chr)
              write (myunit,*) "<m1>",trim(get_pheno(chr,genotyp(chr,ii,correp(ip),1))),&
              "</m1><m2>",trim(get_pheno(chr,genotyp(chr,ii,correp(ip),2))),"</m2>"
            end do
         write (myunit,*) "</Phenotype>"


        !--------------------------
        ! By Dam
        write (myunit,fmt="(a)") '<Dams>'
        do jm=nmp(ip)+1,nmp(ip+1)
           write (myunit,fmt="(a,i5,a)") '<Dam id="',jm,'">'
           write (myunit,*) "<Name>",trim(mere(jm)),"</Name>"
          if (estfem(repfem(jm))) then
            write (myunit,*) "<estfem>true</estfem>"
          else
            write (myunit,*) "<estfem>false</estfem>"
          end if

          !Phase
          !-----
          if (phasm(chr,jm)) then
             write (myunit,*) "<Phase>true</Phase>"
          else
             write (myunit,*) "<Phase>false</Phase>"
          end if

          write (myunit,fmt="(a)") '<Animals>'
          do kd=ndm(jm)+1,ndm(jm+1)
             write (myunit,fmt="(a,i5,a)") '<Animal id="',kd,'">'
             write (myunit,*) "<Name>",trim(animal(kd)),"</Name>"
             write (myunit,fmt="(a,i5,a)") '</Animal>'
          end do
           write (myunit,fmt="(a)") '</Animals>'

           do geno=ngenom(chr,jm)+1,ngenom(chr,jm+1)
            igeno=geno-ngenom(chr,jm)
            write (myunit,fmt="(a,i5,a,f8.3,a)") '<Genotype id="',igeno,'"  prob="',probg(chr,geno),'">'

            write (myunit,*) "<Phenotype>"
            do ii=1,nmk(chr)
              write (myunit,*) "<m1>",trim(get_pheno(chr,genotyp(chr,ii,geno,1))),&
              "</m1><m2>",trim(get_pheno(chr,genotyp(chr,ii,geno,2))),"</m2>"
            end do
             write (myunit,*) "</Phenotype>"

            do n=1,(get_npo(chr)-1)
              write(myunit,fmt="(a,i5,a,f8.3,a)") '<Position id="',n,'" dx="',absi(chr,n),'">'
              do kd=ngend(chr,geno)+1,ngend(chr,geno+1)
                kkd=ndesc(chr,kd)


              if(.not. estfem(repfem(jm)).or.opt_sib.eq.OPT_SIB_HS) then
! Impression des probabilites des couples d'haplo parentaux

                 write(myunit,FMT="(a,i5,a,f8.3,a,f8.3,a,f8.3,a,f8.3,a)") '<pdd idRef="',kkd,'" p1="',0.5d0*pdd(chr,kd,1,n),&
                 '" p2="',0.5d0*pdd(chr,kd,1,n),'" p3="',0.5d0*pdd(chr,kd,3,n),'" p4="',0.5d0*pdd(chr,kd,3,n),'"/>'

              else    !OPT_SIB_FS
!Impression des probabilites des couples d'haplo parentaux
                write(myunit,FMT="(a,i5,a,f8.3,a,f8.3,a,f8.3,a,f8.3,a)") '<pdd idRef="',kkd,'" p1="',pdd(chr,kd,1,n),&
                '" p2="',pdd(chr,kd,2,n),'" p3="',pdd(chr,kd,3,n),'" p4="',pdd(chr,kd,4,n),'"/>'

              end if
            end do ! KD
              write (myunit,'("</Position>")')
            end do ! N
            write (myunit,'("</Genotype>")')
           end do ! GENO
         write (myunit,'("</Dam>")')
      end do  ! JM
      write (myunit,fmt="(a)") '</Dams>'
      write (myunit,'("</Sire>")')
     end do ! IP
     write (myunit,fmt="(a)") '</Sires>'
    write (myunit,'("</Chromosome>")')
   end do ! chr
   write (myunit,fmt="(a)") '</Chromosomes>'

   close (myunit)

  end subroutine print_pded_xml


  subroutine print_solution_xml(nqtl,lrtsol,listincsol,multi)
    integer                                         ,intent(in)      :: nqtl
    type(TYPE_LRT_SOLUTION)  , dimension(ncar,nqtl) ,intent(in)      :: lrtsol
    type(TYPE_INCIDENCE_SOLUTION),dimension(ncar,nqtl+1) ,intent(in) :: listincsol
    logical ,                                          intent(in)    :: multi

    integer :: ip,jm,igeno,geno,kkd,kd,n,chr,ios,myunit,ii,ic,hyp,i,j,h,ifem
    integer :: chr1,chr2,n1,n2
    character(len=LEN_W) :: buf

    myunit=44
    open(UNIT=myunit, file="solution.xml", form="formatted",recl=BUF_ALLOC_FILE,iostat=ios)

    write (myunit,'(a)')'<?xml version="1.0" encoding="UTF-8"?>'


    write (myunit,fmt="(a)") '<Solution>'
    write (myunit,fmt="(a)") '<Sires>'
    do ip=1,np
       write (myunit,fmt="(a,i5,a)") '<Sire id="',ip,'">'
       write (myunit,*) "<Name>",trim(pere(ip)),"</Name>"

       write (myunit,fmt="(a)") '<Dams>'
       do jm=nmp(ip)+1,nmp(ip+1)
           write (myunit,fmt="(a,i5,a)") '<Dam id="',jm,'">'
           write (myunit,*) "<Name>",trim(mere(jm)),"</Name>"

          write (myunit,fmt="(a)") '<Animals>'
          do kd=ndm(jm)+1,ndm(jm+1)
             write (myunit,fmt="(a,i5,a)") '<Animal id="',kd,'">'
             write (myunit,*) "<Name>",trim(animal(kd)),"</Name>"
             write (myunit,fmt="(a,i5,a)") '</Animal>'
          end do
          write (myunit,fmt="(a)") '</Animals>'
          write (myunit,fmt="(a)") '</Dam>'
       end do
       write (myunit,fmt="(a)") '</Dams>'

       write (myunit,'("</Sire>")')
     end do
     write (myunit,fmt="(a)") '</Sires>'

    write (myunit,fmt="(a)") '<Traits>'
    if (.not. multi) then
     do ic=1,ncar
            write (myunit,fmt="(a,i5,a)") '<Trait id="',ic,'">'
            do hyp=1,nqtl+1
              write (myunit,fmt="(a,i5,a)") '<Hypothesis id="',listincsol(ic,hyp)%hypothesis,'">'

              !Maximum Reached
              !---------------

              if (hyp>1) then
                 do h=0,hyp-2
                    write (myunit,fmt='(a,i5,a,i5,a,f8.3,a,f8.3,a)') '<Maximum chromosome="',&
                    lrtsol(ic,hyp-1)%chrmax(h),&
                    '" idpos="',lrtsol(ic,hyp-1)%nxmax(h),&
                    '" position="',absi(lrtsol(ic,hyp-1)%chrmax(h),lrtsol(ic,hyp-1)%nxmax(h)),&
                    '" lrt="',lrtsol(ic,hyp-1)%lrtmax(h),'" />'
                 end do


!******************************************************************************************************************************
!                        QTL = 1
                 if ( lrtsol(ic,hyp-1)%nqtl == 1) then
                    write (myunit,fmt="(a)") '<LRTs>'

                    write (myunit,fmt="(a)") '<GlobalLRT>'
                    do chr=1,nchr
                      write (myunit,fmt="(a,i5,a)") '<LRT chromosome="',chr,'">'
                      do n=1,get_npo(chr)
                          write(myunit,fmt="(a,i6,a,f8.3,a,f8.3,a)") &
                '<Position id="',n,'" dx="',absi(chr,n),'">',lrtsol(ic,hyp-1)%lrt1(chr,n),'</Position>'
                      end do
                      write (myunit,fmt="(a)") '</LRT>'
                    end do
                     write (myunit,fmt="(a)") '</GlobalLRT>'

                    do ip=1,np
                    write (myunit,fmt="(a,i5,a)") '<LRTSire idSire="',ip,'">'
                    do chr=1,nchr
                      write (myunit,fmt="(a,i5,a)") '<LRT chromosome="',chr,'">'
                      do n=1,get_npo(chr)
                          write(myunit,fmt="(a,i6,a,f8.3,a,f8.3,a)") &
                '<Position id="',n,'" dx="',absi(chr,n),'">',lrtsol(ic,hyp-1)%xlrp(chr,ip,n),'</Position>'
                      end do
                      write (myunit,fmt="(a)") '</LRT>'
                    end do
                     write (myunit,fmt="(a)") '</LRTSire>'
                    end do

                    do jm=1,nm
                    if ( estime(ic,jm) ) then
                    write (myunit,fmt="(a,i5,a)") '<LRTDam idDam="',jm,'">'
                    do chr=1,nchr
                      write (myunit,fmt="(a,i5,a)") '<LRT chromosome="',chr,'">'
                      do n=1,get_npo(chr)
                          write(myunit,fmt="(a,i6,a,f8.3,a,f8.3,a)") &
                '<Position id="',n,'" dx="',absi(chr,n),'">',lrtsol(ic,hyp-1)%xlrm(chr,jm,n),'</Position>'
                      end do
                      write (myunit,fmt="(a)") '</LRT>'
                    end do
                     write (myunit,fmt="(a)") '</LRTDam>'
                    end if
                    end do
                    write (myunit,fmt="(a)") '</LRTs>'

                   !QTL Effect

                   write (myunit,fmt="(a)") '<QTLEffects>'

                   write (myunit,fmt="(a)") '<PaternalEffects>'
                   do ip=1,np
                    write (myunit,fmt="(a,i5,a)") '<SireEffects refid="',ip,'">'
                    do chr=1,nchr
                      write (myunit,fmt="(a,i5,a)") '<Effects chromosome="',chr,'">'
                      do n=1,get_npo(chr)
                          write(myunit,fmt="(a,i6,a,f8.3,a,f8.3,a)") &
                '<Position id="',n,'" dx="',absi(chr,n),'">',lrtsol(ic,hyp-1)%pater_eff(chr,ip,n),'</Position>'
                      end do
                      write (myunit,fmt="(a)") '</Effects>'
                    end do
                     write (myunit,fmt="(a)") '</SireEffects>'
                   end do

                   write (myunit,fmt="(a)") '</PaternalEffects>'

                   write (myunit,fmt="(a)") '<MaternalEffects>'
                   do ifem=1,namest(ic)
                     do jm=1,nm
                       if (iam(ic,repfem(jm)) == ifem ) exit
                     end do

                     if (jm > nm ) then
                       print *,"internal error"
                       print *,"impossible de trouver la femme estimable.......xml output"
                       stop
                     end if

                    !if (estime(ic,jm)) then
                       write (myunit,fmt="(a,i5,a)") '<DamEffects refid="',jm,'">'
                       do chr=1,nchr
                        write (myunit,fmt="(a,i5,a)") '<Effects chromosome="',chr,'">'
                        do n=1,get_npo(chr)
                          write(myunit,fmt="(a,i6,a,f8.3,a,f8.3,a)") &
                '<Position id="',n,'" dx="',absi(chr,n),'">',lrtsol(ic,hyp-1)%mater_eff(chr,ifem,n),'</Position>'
                        end do
                        write (myunit,fmt="(a)") '</Effects>'
                       end do
                       write (myunit,fmt="(a)") '</DamEffects>'
                     !end if
                   end do

                   write (myunit,fmt="(a)") '</MaternalEffects>'

                  write (myunit,fmt="(a)") '</QTLEffects>'
                 end if

!******************************************************************************************************************************
!                        QTL = 2
                  if ( lrtsol(ic,hyp-1)%nqtl == 2) then

                    write (myunit,fmt="(a)") '<LRTs>'

                    write (myunit,fmt="(a)") '<GlobalLRT against="0">'

                    do chr1=1,nchr
                     do chr2=chr1,nchr
                      do n1=1,get_npo(chr1)
                       do n2=1,get_npo(chr2)
                          write(myunit,fmt="(a,i6,a,i6,a,i6,a,i6,a,f8.3,a,f8.3,a,f8.3,a)") &
                '<Position chr1="',chr1,'" chr2="',chr2,'" id1="',n1,'" id2="',n2,&
                '" dx1="',absi(chr1,n1),'" dx2="',absi(chr2,n2),'">',&
                lrtsol(ic,hyp-1)%lrt0_2(chr1,chr2,n1,n2),'</Position>'
                       end do
                      end do
                     end do
                    end do
                     write (myunit,fmt="(a)") '</GlobalLRT>'

                     write (myunit,fmt="(a)") '<GlobalLRT against="1">'
                     do chr1=1,nchr
                     do chr2=chr1,nchr
                      do n1=1,get_npo(chr1)
                       do n2=1,get_npo(chr2)
                          write(myunit,fmt="(a,i6,a,i6,a,i6,a,i6,a,f8.3,a,f8.3,a,f8.3,a)") &
                '<Position chr1="',chr1,'" chr2="',chr2,'"  id1="',n1,'" id2="',n2,&
                '" dx1="',absi(chr1,n1),'" dx2="',absi(chr2,n2),'">',&
                lrtsol(ic,hyp-1)%lrt1_2(chr1,chr2,n1,n2),'</Position>'
                       end do
                      end do
                     end do
                    end do

                     write (myunit,fmt="(a)") '</GlobalLRT>'


                   do ip=1,np
                    write (myunit,fmt="(a,i5,a)") '<LRTSire idSire="',ip,'"  against="1">'
                    do chr1=1,nchr
                     do chr2=chr1,nchr
                       do n1=1,get_npo(chr1)
                        do n2=1,get_npo(chr2)
                          write(myunit,fmt="(a,i6,a,i6,a,i6,a,i6,a,f8.3,a,f8.3,a,f8.3,a)") &
                '<Position chr1="',chr1,'" chr2="',chr2,'" id1="',n1,'" id2="',n2,&
                '" dx1="',absi(chr1,n1),'" dx2="',absi(chr2,n2),'">',&
                lrtsol(ic,hyp-1)%xlrp2(chr1,chr2,ip,n1,n2),'</Position>'
                        end do
                       end do
                    end do
                   end do
                     write (myunit,fmt="(a)") '</LRTSire>'
                    end do

                    do jm=1,nm
                    if ( estime(ic,jm) ) then
                    write (myunit,fmt="(a,i5,a)") '<LRTDam idDam="',jm,'" against="1">'
                   do chr1=1,nchr
                     do chr2=chr1,nchr
                       do n1=1,get_npo(chr1)
                        do n2=1,get_npo(chr2)
                          write(myunit,fmt="(a,i6,a,i6,a,i6,a,i6,a,f8.3,a,f8.3,a,f8.3,a)") &
                '<Position chr1="',chr1,'" chr2="',chr2,'" id1="',n1,'" id2="',n2,&
                '" dx1="',absi(chr1,n1),'" dx2="',absi(chr2,n2),'">',&
                lrtsol(ic,hyp-1)%xlrm2(chr1,chr2,jm,n1,n2),'</Position>'
                        end do
                       end do
                    end do
                   end do
                     write (myunit,fmt="(a)") '</LRTDam>'
                    end if
                    end do
                      write (myunit,fmt="(a)") '</LRTs>'

                   !!QTL Effect

                  write (myunit,fmt="(a)") '<QTLEffects>'

                   write (myunit,fmt="(a)") '<PaternalEffects>'
                   do ip=1,np
                    write (myunit,fmt="(a,i5,a,i5,a)") '<SireEffects refid="',ip,'"  numqtl="1">'
                     do chr1=1,nchr
                     do chr2=chr1,nchr
                      do n1=1,get_npo(chr1)
                        do n2=1,get_npo(chr2)
                           write(myunit,fmt="(a,i6,a,i6,a,i6,a,i6,a,f8.3,a,f8.3,a,f8.3,a)") &
                '<Position chr1="',chr1,'" chr2="',chr2,'" id1="',n1,'" id2="',n2,&
                '" dx1="',absi(chr1,n1),'" dx2="',absi(chr2,n2),'">',&
                lrtsol(ic,hyp-1)%pater_eff2(chr1,chr2,ip,n1,n2,1),'</Position>'

                        end do
                       end do
                    end do
                   end do
                   write (myunit,fmt="(a)") '</SireEffects>'
                   write (myunit,fmt="(a,i5,a,i5,a)") '<SireEffects refid="',ip,'"  numqtl="2">'
                     do chr1=1,nchr
                     do chr2=chr1,nchr
                      do n1=1,get_npo(chr1)
                        do n2=1,get_npo(chr2)
                           write(myunit,fmt="(a,i6,a,i6,a,i6,a,i6,a,f8.3,a,f8.3,a,f8.3,a)") &
                '<Position chr1="',chr1,'" chr2="',chr2,'" id1="',n1,'" id2="',n2,&
                '" dx1="',absi(chr1,n1),'" dx2="',absi(chr2,n2),'">',&
                lrtsol(ic,hyp-1)%pater_eff2(chr1,chr2,ip,n1,n2,2),'</Position>'
                        end do
                       end do
                    end do
                   end do
                    write (myunit,fmt="(a)") '</SireEffects>'

                   end do

                   write (myunit,fmt="(a)") '</PaternalEffects>'

                   write (myunit,fmt="(a)") '<MaternalEffects>'
                   do ifem=1,namest(ic)
                     do jm=1,nm
                       if (iam(ic,repfem(jm)) == ifem ) exit
                     end do

                     if (jm > nm ) then
                       print *,"internal error"
                       print *,"impossible de trouver la femme estimable.......xml output"
                       stop
                     end if

                   write (myunit,fmt="(a,i5,a,i5,a)") '<DamEffects refid="',jm,'"  numqtl="1">'
                    do chr1=1,nchr
                     do chr2=chr1,nchr
                      do n1=1,get_npo(chr1)
                        do n2=1,get_npo(chr2)
                           write(myunit,fmt="(a,i6,a,i6,a,i6,a,i6,a,f8.3,a,f8.3,a,f8.3,a)") &
                '<Position chr1="',chr1,'" chr2="',chr2,'" id1="',n1,'" id2="',n2,&
                '" dx1="',absi(chr1,n1),'" dx2="',absi(chr2,n2),'">',&
                lrtsol(ic,hyp-1)%mater_eff2(chr1,chr2,ifem,n1,n2,1),'</Position>'
                        end do
                       end do
                    end do
                   end do
                       write (myunit,fmt="(a)") '</DamEffects>'
                     write (myunit,fmt="(a,i5,a,i5,a)") '<DamEffects refid="',jm,'"  numqtl="2">'
                    do chr1=1,nchr
                     do chr2=chr1,nchr
                      do n1=1,get_npo(chr1)
                        do n2=1,get_npo(chr2)
                           write(myunit,fmt="(a,i6,a,i6,a,i6,a,i6,a,f8.3,a,f8.3,a,f8.3,a)") &
                '<Position chr1="',chr1,'" chr2="',chr2,'" id1="',n1,'" id2="',n2,&
                '" dx1="',absi(chr1,n1),'" dx2="',absi(chr2,n2),'">',&
                lrtsol(ic,hyp-1)%mater_eff2(chr1,chr2,ifem,n1,n2,2),'</Position>'
                        end do
                       end do
                    end do
                   end do
                       write (myunit,fmt="(a)") '</DamEffects>'

                   end do

                   write (myunit,fmt="(a)") '</MaternalEffects>'

                  write (myunit,fmt="(a)") '</QTLEffects>'

                  end if


              end if

              !Resultat de l estimation sous l hypothese HYP-1
              !-----------------------------------------------
              do ip=1,np
                write (myunit,fmt="(a,i5,a,f8.3,a)") &
                '<StandartDeviation sireRef="',ip,'">',listincsol(ic,hyp)%sig(1,ip),'</StandartDeviation>'
              end do

              do i=1,size(listincsol(ic,hyp)%groupeName)
                  if (listincsol(ic,hyp)%eqtl_print(i)) then
                     buf='true'
                   else
                     buf='false'
                   end if
                 write (myunit,fmt='(5(a))') &
                 '<GroupParam name="',trim(listincsol(ic,hyp)%groupeName(i)),'" EqtlPrint="',trim(buf),'">'
                 do j=1,listincsol(ic,hyp)%nbParameterGroup(i)
                   if (listincsol(ic,hyp)%parameterVecsol(i,j)) then
                     buf='true'
                   else
                     buf='false'
                   end if
                   write (myunit,fmt='(a,a,a,f8.3,a,f8.3,a)')'<param name="',trim(listincsol(ic,hyp)%parameterName(i,j)),&
                   '" value="',listincsol(ic,hyp)%paramaterValue(i,j),'" precision="',&
                   listincsol(ic,hyp)%parameterPrecis(i,j),'" estim="'//trim(buf)//'"/>'
                 end do
                 write (myunit,*)'</GroupParam>'
              end do

             write (myunit,fmt="(a)") '</Hypothesis>'
            end do

             write (myunit,fmt="(a)") '</Trait>'
     end do
    end if

    write (myunit,fmt="(a)") '</Traits>'
    write (myunit,fmt="(a)") '</Solution>'
    close (myunit)

  end subroutine print_solution_xml

end module m_qtlmap_output_xml
